@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2018 - pancake, nibble */
+/* radare - LGPL - Copyright 2009-2019 - pancake, nibble */
 
 #include <r_anal.h>
 #include <r_util.h>
@@ -25,63 +25,137 @@ R_API void r_anal_unset_limits(RAnal *anal) {
 	R_FREE (anal->limit);
 }
 
-static void meta_unset_for(void *user, int idx) {
-	RSpaces *s = (RSpaces*)user;
-	RAnal *anal = (RAnal*)s->user;
-	r_meta_space_unset_for (anal, idx);
+static void meta_unset_for(REvent *ev, int type, void *user, void *data) {
+	RSpaces *s = (RSpaces *)ev->user;
+	RAnal *anal = container_of (s, RAnal, meta_spaces);
+	RSpaceEvent *se = (RSpaceEvent *)data;
+	r_meta_space_unset_for (anal, se->data.unset.space);
 }
 
-static int meta_count_for(void *user, int idx) {
-	RSpaces *s = (RSpaces*)user;
-	RAnal *anal = (RAnal*)s->user;
-	return r_meta_space_count_for (anal, idx);
+static void meta_count_for(REvent *ev, int type, void *user, void *data) {
+	RSpaces *s = (RSpaces *)ev->user;
+	RAnal *anal = container_of (s, RAnal, meta_spaces);
+	RSpaceEvent *se = (RSpaceEvent *)data;
+	se->res = r_meta_space_count_for (anal, se->data.count.space);
 }
 
-static void zign_unset_for(void *user, int idx) {
-	RSpaces *s = (RSpaces*)user;
-	RAnal *anal = (RAnal*)s->user;
-	r_sign_space_unset_for (anal, idx);
+static void zign_unset_for(REvent *ev, int type, void *user, void *data) {
+	RSpaces *s = (RSpaces *)ev->user;
+	RAnal *anal = container_of (s, RAnal, zign_spaces);
+	RSpaceEvent *se = (RSpaceEvent *)data;
+	r_sign_space_unset_for (anal, se->data.unset.space);
 }
 
-static int zign_count_for(void *user, int idx) {
-	RSpaces *s = (RSpaces*)user;
-	RAnal *anal = (RAnal*)s->user;
-	return r_sign_space_count_for (anal, idx);
+static void zign_count_for(REvent *ev, int type, void *user, void *data) {
+	RSpaces *s = (RSpaces *)ev->user;
+	RAnal *anal = container_of (s, RAnal, zign_spaces);
+	RSpaceEvent *se = (RSpaceEvent *)data;
+	se->res = r_sign_space_count_for (anal, se->data.count.space);
 }
 
-static void zign_rename_for(void *user, int idx, const char *oname, const char *nname) {
-	RSpaces *s = (RSpaces*)user;
-	RAnal *anal = (RAnal*)s->user;
-	r_sign_space_rename_for (anal, idx, oname, nname);
+static void zign_rename_for(REvent *ev, int type, void *user, void *data) {
+	RSpaces *s = (RSpaces *)ev->user;
+	RAnal *anal = container_of (s, RAnal, zign_spaces);
+	RSpaceEvent *se = (RSpaceEvent *)data;
+	r_sign_space_rename_for (anal, se->data.rename.space,
+		se->data.rename.oldname, se->data.rename.newname);
 }
 
-R_API RAnal *r_anal_new() {
+static int __anal_hint_range_tree_cmp(const void *a_, const RBNode *b_, void *user) {
+	const RAnalRange *a = a_;
+	const RAnalRange *b = container_of (b_, const RAnalRange, rb);
+	if (a && b) {
+		ut64 from0 = a->from, from1 = b->from;
+		return from0 < from1 ? -1 : 1;
+	}
+	return 0;
+}
+
+static void __anal_hint_range_tree_free(RBNode *node, void *user) {
+	free (container_of (node, RAnalRange, rb));
+}
+
+
+static RAnalRange *__anal_range_hint_tree_find_at(RBNode *node, ut64 addr) {
+	while (node) {
+		RAnalRange *range = container_of (node, RAnalRange, rb);
+		if (range->from == addr) {
+			return range;
+		}
+		node = node->child[range->from < addr];
+	}
+	return NULL;
+}
+
+static void __anal_range_hint_tree_insert(RBNode **root, RAnalRange *range) {
+	r_rbtree_aug_insert (root, range, &(range->rb),
+			     __anal_hint_range_tree_cmp,
+			     NULL, NULL);
+}
+
+static void __anal_add_range_on_hints(RAnal *a, ut64 addr, int bits) {
+	r_return_if_fail (a);
+	//do we have already a node with that addr? if yes then update its bits
+	RAnalRange *range = __anal_range_hint_tree_find_at (a->rb_hints_ranges, addr);
+	if (range) {
+		range->bits = bits;
+	} else {
+		//otherwise insert new range into the tree
+		range = R_NEW0 (RAnalRange);
+		if (range) {
+			range->bits = bits;
+			range->from = addr;
+			__anal_range_hint_tree_insert (&a->rb_hints_ranges, range);
+		}
+	}
+}
+
+static void __anal_hint_on_bits(RAnal *a, ut64 addr, int bits, bool set) {
+	if (set) {
+		__anal_add_range_on_hints (a, addr, bits);
+	}
+}
+
+R_API RAnal *r_anal_new(void) {
 	int i;
 	RAnal *anal = R_NEW0 (RAnal);
 	if (!anal) {
 		return NULL;
 	}
+	if (!r_str_constpool_init (&anal->constpool)) {
+		free (anal);
+		return NULL;
+	}
+	anal->bb_tree = NULL;
+	anal->ht_addr_fun = ht_up_new0 ();
+	anal->ht_name_fun = ht_pp_new0 ();
 	anal->os = strdup (R_SYS_OS);
-	anal->reflines = anal->reflines2 = NULL;
 	anal->esil_goto_limit = R_ANAL_ESIL_GOTO_LIMIT;
-	anal->limit = NULL;
 	anal->opt.nopskip = true; // skip nops in code analysis
 	anal->opt.hpskip = false; // skip `mov reg,reg` and `lea reg,[reg]`
-	anal->decode = true; // slow slow if not used
 	anal->gp = 0LL;
 	anal->sdb = sdb_new0 ();
 	anal->cpp_abi = R_ANAL_CPP_ABI_ITANIUM;
 	anal->opt.depth = 32;
 	anal->opt.noncode = false; // do not analyze data by default
-	r_space_new (&anal->meta_spaces, "CS", meta_unset_for, meta_count_for, NULL, anal);
-	r_space_new (&anal->zign_spaces, "zs", zign_unset_for, zign_count_for, zign_rename_for, anal);
+	r_spaces_init (&anal->meta_spaces, "CS");
+	r_event_hook (anal->meta_spaces.event, R_SPACE_EVENT_UNSET, meta_unset_for, NULL);
+	r_event_hook (anal->meta_spaces.event, R_SPACE_EVENT_COUNT, meta_count_for, NULL);
+
+	r_spaces_init (&anal->zign_spaces, "zs");
+	r_event_hook (anal->zign_spaces.event, R_SPACE_EVENT_UNSET, zign_unset_for, NULL);
+	r_event_hook (anal->zign_spaces.event, R_SPACE_EVENT_COUNT, zign_count_for, NULL);
+	r_event_hook (anal->zign_spaces.event, R_SPACE_EVENT_RENAME, zign_rename_for, NULL);
 	anal->sdb_fcns = sdb_ns (anal->sdb, "fcns", 1);
 	anal->sdb_meta = sdb_ns (anal->sdb, "meta", 1);
 	anal->sdb_hints = sdb_ns (anal->sdb, "hints", 1);
+	anal->hint_cbs.on_bits = __anal_hint_on_bits;
 	anal->sdb_types = sdb_ns (anal->sdb, "types", 1);
 	anal->sdb_fmts = sdb_ns (anal->sdb, "spec", 1);
 	anal->sdb_cc = sdb_ns (anal->sdb, "cc", 1);
 	anal->sdb_zigns = sdb_ns (anal->sdb, "zigns", 1);
+	anal->sdb_classes = sdb_ns (anal->sdb, "classes", 1);
+	anal->sdb_classes_attrs = sdb_ns (anal->sdb_classes, "attrs", 1);
 	anal->zign_path = strdup ("");
 	anal->cb_printf = (PrintfCallback) printf;
 	(void)r_anal_pin_init (anal);
@@ -94,11 +168,11 @@ R_API RAnal *r_anal_new() {
 	anal->reg = r_reg_new ();
 	anal->last_disasm_reg = NULL;
 	anal->stackptr = 0;
-	anal->bits_ranges = r_list_newf (free);
+	anal->rb_hints_ranges = NULL;
 	anal->lineswidth = 0;
-	anal->fcns = r_anal_fcn_list_new ();
-	anal->fcn_tree = NULL;
+	anal->fcns = r_list_newf (r_anal_function_free);
 	anal->refs = r_anal_ref_list_new ();
+	anal->leaddrs = NULL;
 	r_anal_set_bits (anal, 32);
 	anal->plugins = r_list_newf ((RListFree) r_anal_plugin_free);
 	if (anal->plugins) {
@@ -106,44 +180,52 @@ R_API RAnal *r_anal_new() {
 			r_anal_add (anal, anal_static_plugins[i]);
 		}
 	}
+	anal->cmdtail = r_strbuf_new (NULL);
 	return anal;
 }
 
+
 R_API void r_anal_plugin_free (RAnalPlugin *p) {
 	if (p && p->fini) {
 		p->fini (NULL);
 	}
 }
 
+void __block_free_rb(RBNode *node, void *user);
+
 R_API RAnal *r_anal_free(RAnal *a) {
 	if (!a) {
 		return NULL;
 	}
 	/* TODO: Free anals here */
-	R_FREE (a->cpu);
-	R_FREE (a->os);
-	R_FREE (a->zign_path);
-	r_list_free (a->plugins);
-	a->fcns->free = r_anal_fcn_free;
 	r_list_free (a->fcns);
-	r_space_free (&a->meta_spaces);
-	r_space_free (&a->zign_spaces);
+	ht_up_free (a->ht_addr_fun);
+	ht_pp_free (a->ht_name_fun);
+	set_u_free (a->visited);
+	free (a->cpu);
+	free (a->os);
+	free (a->zign_path);
+	r_list_free (a->plugins);
+	r_rbtree_free (a->bb_tree, __block_free_rb, NULL);
+	r_spaces_fini (&a->meta_spaces);
+	r_spaces_fini (&a->zign_spaces);
 	r_anal_pin_fini (a);
 	r_list_free (a->refs);
 	r_syscall_free (a->syscall);
 	r_reg_free (a->reg);
 	r_anal_op_free (a->queued);
-	r_list_free (a->bits_ranges);
-	ht_free (a->dict_refs);
-	ht_free (a->dict_xrefs);
-	a->sdb = NULL;
-	sdb_ns_free (a->sdb);
+	r_rbtree_free (a->rb_hints_ranges, __anal_hint_range_tree_free, NULL);
+	ht_up_free (a->dict_refs);
+	ht_up_free (a->dict_xrefs);
+	r_list_free (a->leaddrs);
+	sdb_free (a->sdb);
 	if (a->esil) {
 		r_anal_esil_free (a->esil);
 		a->esil = NULL;
 	}
 	free (a->last_disasm_reg);
-	memset (a, 0, sizeof (RAnal));
+	r_strbuf_free (a->cmdtail);
+	r_str_constpool_fini (&a->constpool);
 	free (a);
 	return NULL;
 }
@@ -160,15 +242,6 @@ R_API int r_anal_add(RAnal *anal, RAnalPlugin *foo) {
 	return true;
 }
 
-// TODO: Must be deprecated
-R_API void r_anal_list(RAnal *anal) {
-	RAnalPlugin *h;
-	RListIter *it;
-	r_list_foreach (anal->plugins, it, h) {
-		anal->cb_printf ("anal %-10s %s\n", h->name, h->desc);
-	}
-}
-
 R_API bool r_anal_use(RAnal *anal, const char *name) {
 	RListIter *it;
 	RAnalPlugin *h;
@@ -176,27 +249,21 @@ R_API bool r_anal_use(RAnal *anal, const char *name) {
 	if (anal) {
 		bool change = anal->cur && strcmp (anal->cur->name, name);
 		r_list_foreach (anal->plugins, it, h) {
-			if (!strcmp (h->name, name)) {
-	#if 0
-				// regression happening here for asm.emu
-				if (anal->cur && anal->cur == h) {
-					return true;
-				}
-	#endif
-				anal->cur = h;
-				r_anal_set_reg_profile (anal);
-				if (change) {
-					r_anal_set_fcnsign (anal, NULL);
-				}
-	#if 1
-				/* invalidate esil state? really ? */
-				if (anal->esil) {
-					r_anal_esil_free (anal->esil);
-					anal->esil = NULL;
-				}
-	#endif
+			if (!h->name || strcmp (h->name, name)) {
+				continue;
+			}
+#if 0
+			// regression happening here for asm.emu
+			if (anal->cur && anal->cur == h) {
 				return true;
 			}
+#endif
+			anal->cur = h;
+			r_anal_set_reg_profile (anal);
+			if (change) {
+				r_anal_set_fcnsign (anal, NULL);
+			}
+			return true;
 		}
 	}
 	return false;
@@ -225,14 +292,11 @@ R_API bool r_anal_set_reg_profile(RAnal *anal) {
 
 R_API bool r_anal_set_fcnsign(RAnal *anal, const char *name) {
 	const char *dirPrefix = r_sys_prefix (NULL);
-	char *file = NULL;
 	const char *arch = (anal->cur && anal->cur->arch) ? anal->cur->arch : R_SYS_ARCH;
-	if (name && *name) {
-		file = sdb_fmt (R_JOIN_3_PATHS ("%s", R2_SDB_FCNSIGN, "%s.sdb"), dirPrefix, name);
-	} else {
-		file = sdb_fmt (R_JOIN_3_PATHS ("%s", R2_SDB_FCNSIGN, "%s-%s-%d.sdb"), dirPrefix,
+	const char *file = (name && *name)
+		? sdb_fmt (R_JOIN_3_PATHS ("%s", R2_SDB_FCNSIGN, "%s.sdb"), dirPrefix, name)
+		: sdb_fmt (R_JOIN_3_PATHS ("%s", R2_SDB_FCNSIGN, "%s-%s-%d.sdb"), dirPrefix,
 			anal->os, arch, anal->bits);
-	}
 	if (r_file_exists (file)) {
 		sdb_close (anal->sdb_fcnsign);
 		sdb_free (anal->sdb_fcnsign);
@@ -247,7 +311,8 @@ R_API const char *r_anal_get_fcnsign(RAnal *anal, const char *sym) {
 	return sdb_const_get (anal->sdb_fcnsign, sym, 0);
 }
 
-R_API int r_anal_set_triplet(RAnal *anal, const char *os, const char *arch, int bits) {
+R_API bool r_anal_set_triplet(RAnal *anal, const char *os, const char *arch, int bits) {
+	r_return_val_if_fail (anal, false);
 	if (!os || !*os) {
 		os = R_SYS_OS;
 	}
@@ -263,7 +328,22 @@ R_API int r_anal_set_triplet(RAnal *anal, const char *os, const char *arch, int
 	return r_anal_use (anal, arch);
 }
 
+// copypasta from core/cbin.c
+static void sdb_concat_by_path(Sdb *s, const char *path) {
+	Sdb *db = sdb_new (0, path, 0);
+	sdb_merge (s, db);
+	sdb_close (db);
+	sdb_free (db);
+}
+
 R_API bool r_anal_set_os(RAnal *anal, const char *os) {
+	Sdb *types = anal->sdb_types;
+	const char *dir_prefix = r_sys_prefix (NULL);
+	const char *dbpath = sdb_fmt (R_JOIN_3_PATHS ("%s", R2_SDB_FCNSIGN, "types-%s.sdb"),
+		dir_prefix, os);
+	if (r_file_exists (dbpath)) {
+		sdb_concat_by_path (types, dbpath);
+	}
 	return r_anal_set_triplet (anal, os, NULL, -1);
 }
 
@@ -331,6 +411,7 @@ R_API ut8 *r_anal_mask(RAnal *anal, int size, const ut8 *data, ut64 at) {
 			memset (ret + idx + op->nopcode, 0, oplen - op->nopcode);
 		}
 		idx += oplen;
+		at += oplen;
 	}
 
 	r_anal_op_free (op);
@@ -353,6 +434,14 @@ R_API void r_anal_trace_bb(RAnal *anal, ut64 addr) {
 	}
 }
 
+R_API void r_anal_colorize_bb(RAnal *anal, ut64 addr, ut32 color) {
+	RAnalBlock *bbi;
+	bbi = r_anal_bb_from_offset (anal, addr);
+	if (bbi) {
+		bbi->colorize = color;
+	}
+}
+
 R_API RList* r_anal_get_fcns (RAnal *anal) {
 	// avoid received to free this thing
 	anal->fcns->free = NULL;
@@ -360,24 +449,22 @@ R_API RList* r_anal_get_fcns (RAnal *anal) {
 }
 
 R_API RAnalOp *r_anal_op_hexstr(RAnal *anal, ut64 addr, const char *str) {
-	int len;
-	ut8 *buf;
 	RAnalOp *op = R_NEW0 (RAnalOp);
 	if (!op) {
 		return NULL;
 	}
-	buf = calloc (1, strlen (str) + 1);
+	ut8 *buf = calloc (1, strlen (str) + 1);
 	if (!buf) {
 		free (op);
 		return NULL;
 	}
-	len = r_hex_str2bin (str, buf);
+	int len = r_hex_str2bin (str, buf);
 	r_anal_op (anal, op, addr, buf, len, R_ANAL_OP_MASK_BASIC);
 	free (buf);
 	return op;
 }
 
-R_API bool r_anal_op_is_eob (RAnalOp *op) {
+R_API bool r_anal_op_is_eob(RAnalOp *op) {
 	if (op->eob) {
 		return true;
 	}
@@ -402,20 +489,22 @@ R_API int r_anal_purge (RAnal *anal) {
 	sdb_reset (anal->sdb_hints);
 	sdb_reset (anal->sdb_types);
 	sdb_reset (anal->sdb_zigns);
+	sdb_reset (anal->sdb_classes);
+	sdb_reset (anal->sdb_classes_attrs);
 	r_list_free (anal->fcns);
-	anal->fcns = r_anal_fcn_list_new ();
-	anal->fcn_tree = NULL;
+	anal->fcns = r_list_newf (r_anal_function_free);
 	r_list_free (anal->refs);
 	anal->refs = r_anal_ref_list_new ();
 	return 0;
 }
 
 R_API int r_anal_archinfo(RAnal *anal, int query) {
+	r_return_val_if_fail (anal, -1);
 	switch (query) {
 	case R_ANAL_ARCHINFO_MIN_OP_SIZE:
 	case R_ANAL_ARCHINFO_MAX_OP_SIZE:
 	case R_ANAL_ARCHINFO_ALIGN:
-		if (anal && anal->cur && anal->cur->archinfo) {
+		if (anal->cur && anal->cur->archinfo) {
 			return anal->cur->archinfo (anal, query);
 		}
 		break;
@@ -423,7 +512,7 @@ R_API int r_anal_archinfo(RAnal *anal, int query) {
 	return -1;
 }
 
-static int nonreturn_print_commands(void *p, const char *k, const char *v) {
+static int __nonreturn_print_commands(void *p, const char *k, const char *v) {
 	RAnal *anal = (RAnal *)p;
 	if (!strncmp (v, "func", strlen ("func") + 1)) {
 		char *query = sdb_fmt ("func.%s.noreturn", k);
@@ -437,20 +526,23 @@ static int nonreturn_print_commands(void *p, const char *k, const char *v) {
 	return 1;
 }
 
-static int nonreturn_print(void *p, const char *k, const char *v) {
+static int __nonreturn_print(void *p, const char *k, const char *v) {
 	RAnal *anal = (RAnal *)p;
-	if (!strncmp (v, "func", strlen ("func") + 1)) {
-		const char *query = sdb_fmt ("func.%s.noreturn", k);
-		if (sdb_bool_get (anal->sdb_types, query, NULL)) {
-			anal->cb_printf ("%s\n", k);
+	if (!strncmp (k, "func.", 5) && strstr (k, ".noreturn")) {
+		char *s = strdup (k + 5);
+		char *d = strchr (s, '.');
+		if (d) {
+			*d = 0;
 		}
+		anal->cb_printf ("%s\n", s);
+		free (s);
 	}
 	if (!strncmp (k, "addr.", 5)) {
 		char *off;
 		if (!(off = strdup (k + 5))) {
 			return 1;
 		}
-		char *ptr = strstr (off, ".noret");
+		char *ptr = strstr (off, ".noreturn");
 		if (ptr) {
 			*ptr = 0;
 			anal->cb_printf ("0x%s\n", off);
@@ -465,10 +557,10 @@ R_API void r_anal_noreturn_list(RAnal *anal, int mode) {
 	case 1:
 	case '*':
 	case 'r':
-		sdb_foreach (anal->sdb_types, nonreturn_print_commands, anal);
+		sdb_foreach (anal->sdb_types, __nonreturn_print_commands, anal);
 		break;
 	default:
-		sdb_foreach (anal->sdb_types, nonreturn_print, anal);
+		sdb_foreach (anal->sdb_types, __nonreturn_print, anal);
 		break;
 	}
 }
@@ -480,8 +572,10 @@ R_API bool r_anal_noreturn_add(RAnal *anal, const char *name, ut64 addr) {
 	const char *tmp_name = NULL;
 	Sdb *TDB = anal->sdb_types;
 	char *fnl_name = NULL;
-	if (sdb_bool_set (TDB, K_NORET_ADDR (addr), true, 0)) {
-		return true;
+	if (addr != UT64_MAX) {
+		if (sdb_bool_set (TDB, K_NORET_ADDR (addr), true, 0)) {
+			return true;
+		}
 	}
 	if (name && *name) {
 		tmp_name = name;
@@ -489,7 +583,7 @@ R_API bool r_anal_noreturn_add(RAnal *anal, const char *name, ut64 addr) {
 		RAnalFunction *fcn = r_anal_get_fcn_in (anal, addr, -1);
 		RFlagItem *fi = anal->flb.get_at (anal->flb.f, addr, false);
 		if (!fcn && !fi) {
-			eprintf ("Cant find Function at given address\n");
+			eprintf ("Can't find Function at given address\n");
 			return false;
 		}
 		tmp_name = fcn ? fcn->name: fi->name;
@@ -497,53 +591,55 @@ R_API bool r_anal_noreturn_add(RAnal *anal, const char *name, ut64 addr) {
 	if (r_type_func_exist (TDB, tmp_name)) {
 		fnl_name = strdup (tmp_name);
 	} else if (!(fnl_name = r_type_func_guess (TDB, (char *)tmp_name))) {
-		eprintf ("Cant find prototype for %s in types databse\n", tmp_name);
-		return false;
+		if (addr == UT64_MAX) {
+			if (name) {
+				sdb_bool_set (TDB, K_NORET_FUNC (name), true, 0);
+			} else {
+				eprintf ("Can't find prototype for: %s\n", tmp_name);
+			}
+		} else {
+			eprintf ("Can't find prototype for: %s\n", tmp_name);
+		}
+		//return false;
+	}
+	if (fnl_name) {
+		sdb_bool_set (TDB, K_NORET_FUNC (fnl_name), true, 0);
+		free (fnl_name);
 	}
-	sdb_bool_set (TDB, K_NORET_FUNC(fnl_name), true, 0);
-	free (fnl_name);
 	return true;
 }
 
-static int noreturn_dropall(void *p, const char *k, const char *v) {
-	RAnal *anal = (RAnal *)p;
-	if (!strcmp (v, "func")) {
-		sdb_unset (anal->sdb_types, K_NORET_FUNC(k), 0);
-	}
-	return 1;
-}
-
-R_API int r_anal_noreturn_drop(RAnal *anal, const char *expr) {
+R_API bool r_anal_noreturn_drop(RAnal *anal, const char *expr) {
 	Sdb *TDB = anal->sdb_types;
-	if (!strcmp (expr, "*")) {
-		sdb_foreach (TDB, noreturn_dropall, anal);
-		return true;
-	} else {
-		const char *fcnname = NULL;
-		char *tmp;
-		if (!strncmp (expr, "0x", 2)) {
-			ut64 n = r_num_math (NULL, expr);
-			RAnalFunction *fcn = r_anal_get_fcn_in (anal, n, -1);
-			if (!fcn) {
-				eprintf ("can't find function at 0x%"PFMT64x"\n", n);
-				return false;
-			}
-			fcnname = fcn->name;
-		} else {
-			fcnname = expr;
-		}
-		if (r_type_func_exist (TDB, fcnname)) {
-			sdb_unset (TDB, K_NORET_FUNC (fcnname), 0);
-			return true;
-		} else if ((tmp = r_type_func_guess (TDB, (char *)fcnname))) {
-			sdb_unset (TDB, K_NORET_FUNC (fcnname), 0);
-			free (tmp);
-			return true;
-		} else {
-			eprintf ("Cant find prototype for %s in types databse", fcnname);
+	expr = r_str_trim_ro (expr);
+	const char *fcnname = NULL;
+	if (!strncmp (expr, "0x", 2)) {
+		ut64 n = r_num_math (NULL, expr);
+		sdb_unset (TDB, K_NORET_ADDR (n), 0);
+		RAnalFunction *fcn = r_anal_get_fcn_in (anal, n, -1);
+		if (!fcn) {
+			// eprintf ("can't find function at 0x%"PFMT64x"\n", n);
 			return false;
 		}
+		fcnname = fcn->name;
+	} else {
+		fcnname = expr;
 	}
+	sdb_unset (TDB, K_NORET_FUNC (fcnname), 0);
+#if 0
+	char *tmp;
+	// unnsecessary checks, imho the noreturn db should be pretty simple to allow forward and custom declarations without having to define the function prototype before
+	if (r_type_func_exist (TDB, fcnname)) {
+		sdb_unset (TDB, K_NORET_FUNC (fcnname), 0);
+		return true;
+	} else if ((tmp = r_type_func_guess (TDB, (char *)fcnname))) {
+		sdb_unset (TDB, K_NORET_FUNC (fcnname), 0);
+		free (tmp);
+		return true;
+	}
+	eprintf ("Can't find prototype for %s in types database", fcnname);
+#endif
+	return false;
 }
 
 static bool r_anal_noreturn_at_name(RAnal *anal, const char *name) {
@@ -558,6 +654,9 @@ static bool r_anal_noreturn_at_name(RAnal *anal, const char *name) {
 		}
 		free (tmp);
 	}
+	if (r_str_startswith (name, "reloc.")) {
+		return r_anal_noreturn_at_name (anal, name + 6);
+	}
 	return false;
 }
 
@@ -565,7 +664,7 @@ R_API bool r_anal_noreturn_at_addr(RAnal *anal, ut64 addr) {
 	return sdb_bool_get (anal->sdb_types, K_NORET_ADDR (addr), NULL);
 }
 
-bool noreturn_recurse(RAnal *anal, ut64 addr) {
+static bool noreturn_recurse(RAnal *anal, ut64 addr) {
 	RAnalOp op = {0};
 	ut8 bbuf[0x10] = {0};
 	ut64 recurse_addr = UT64_MAX;
@@ -573,8 +672,9 @@ bool noreturn_recurse(RAnal *anal, ut64 addr) {
 		eprintf ("Couldn't read buffer\n");
 		return false;
 	}
-	// TODO: check return value
-	(void)r_anal_op (anal, &op, addr, bbuf, sizeof (bbuf), R_ANAL_OP_MASK_BASIC);
+	if (r_anal_op (anal, &op, addr, bbuf, sizeof (bbuf), R_ANAL_OP_MASK_BASIC | R_ANAL_OP_MASK_VAL) < 1) {
+		return false;
+	}
 	switch (op.type & R_ANAL_OP_TYPE_MASK) {
 	case R_ANAL_OP_TYPE_JMP:
 		if (op.jump == UT64_MAX) {
@@ -601,29 +701,22 @@ bool noreturn_recurse(RAnal *anal, ut64 addr) {
 }
 
 R_API bool r_anal_noreturn_at(RAnal *anal, ut64 addr) {
+	if (!addr || addr == UT64_MAX) {
+		return false;
+	}
 	if (r_anal_noreturn_at_addr (anal, addr)) {
 		return true;
 	}
 	/* XXX this is very slow */
-	RAnalFunction *f = r_anal_get_fcn_at (anal, addr, 0);
+	RAnalFunction *f = r_anal_get_function_at (anal, addr);
 	if (f) {
 		if (r_anal_noreturn_at_name (anal, f->name)) {
 			return true;
 		}
 	}
-	int oss = anal->flb.f->space_strict;
-	int ofs = anal->flb.f->space_idx;
-	anal->flb.set_fs (anal->flb.f, "imports");
-	anal->flb.f->space_strict = true;
-	RFlagItem *fi = anal->flb.get_at (anal->flb.f, addr, false);
-	if (!fi) {
-		anal->flb.set_fs (anal->flb.f, "symbols");
-		fi = anal->flb.get_at (anal->flb.f, addr, false);
-	}
-	anal->flb.f->space_idx = ofs;
-	anal->flb.f->space_strict = oss;
+	RFlagItem *fi = anal->flag_get (anal->flb.f, addr);
 	if (fi) {
-		if (r_anal_noreturn_at_name (anal, fi->name)) {
+		if (r_anal_noreturn_at_name (anal, fi->realname ? fi->realname : fi->name)) {
 			return true;
 		}
 	}
@@ -633,45 +726,71 @@ R_API bool r_anal_noreturn_at(RAnal *anal, ut64 addr) {
 	return false;
 }
 
-// based on anal hint we construct a list of RAnalRange to handle
-// better arm/thumb though maybe handy in other contexts
-R_API void r_anal_build_range_on_hints(RAnal *a) {
-	if (a->bits_hints_changed) {
-		SdbListIter *iter;
-		RListIter *it;
-		SdbKv *kv;
-		RAnalRange *range;
-		int range_bits = 0;
-		// construct again the range from hint to handle properly arm/thumb
-		r_list_free (a->bits_ranges);
-		a->bits_ranges = r_list_newf ((RListFree)free);
-		SdbList *sdb_range = sdb_foreach_list (a->sdb_hints, true);
-		//just grab when hint->bit changes with the previous one
-		ls_foreach (sdb_range, iter, kv) {
-			RAnalHint *hint = r_anal_hint_from_string (a, sdb_atoi (kv->key + 5), kv->value);
-			if (hint->bits && range_bits != hint->bits) {
-				RAnalRange *range = R_NEW0 (RAnalRange);
-				if (range) {
-					range->bits = hint->bits;
-					range->from = hint->addr;
-					range->to = UT64_MAX;
-					r_list_append (a->bits_ranges, range);
-				}
-			} else {
-				//remove this hint is not needed
-				r_anal_hint_unset_bits (a, hint->addr);
-			}
-			range_bits = hint->bits;
-			r_anal_hint_free (hint);
+R_API int r_anal_range_tree_find_bits_at(RBNode *root, ut64 addr) {
+	RAnalRange *tmp = NULL;
+	RBNode *ny;
+	RAnalRange *path[R_RBTREE_MAX_HEIGHT + 1];
+	int i, bits = 0, len = 0;
+	ut64 min_diff = UT64_MAX;
+	if (!root) {
+		return 0;
+	}
+	path[len++] = container_of (root, RAnalRange, rb);
+	ny = root->child[path[0]->from < addr];
+	if (!ny) {
+		return path[0]->bits;
+	}
+	tmp = container_of (ny, RAnalRange, rb);
+	path[len++] = tmp;
+	//build path of RAnalRange
+	while (len < R_RBTREE_MAX_HEIGHT) {
+		ny = ny->child[tmp->from < addr];
+		if (!ny) {
+			break;
 		}
-		//close ranges addr
-		r_list_foreach (a->bits_ranges, it, range) {
-			if (it->n && it->n->data) {
-				range->to = ((RAnalRange *)(it->n->data))->from;
+		tmp = container_of (ny, RAnalRange, rb);
+		path[len++] = tmp;
+	}
+	i = len - 1;
+	//find the nearest RAnalRange
+	while (i >= 0) {
+		ut64 diff = addr - path[i]->from;
+		if ((st64)diff < 0) {
+			i--;
+			continue;
+		}
+		if (diff < min_diff) {
+			bits = path[i]->bits;
+			min_diff = diff;
+		}
+		i--;
+	}
+	return bits;
+}
+
+R_API void r_anal_merge_hint_ranges(RAnal *a) {
+	if (a->merge_hints) {
+		SdbListIter *iter;
+		SdbKv *kv;
+		SdbList *sdb_range = sdb_foreach_list (a->sdb_hints, true);
+		int range_bits = 0;
+		r_rbtree_free (a->rb_hints_ranges, __anal_hint_range_tree_free, NULL);
+		a->rb_hints_ranges = NULL;
+		ls_foreach (sdb_range, iter, kv) {
+			ut64 addr = sdb_atoi (sdbkv_key (kv) + 5);
+			int bits = r_anal_hint_get_bits_at (a, addr,  sdbkv_value (kv));
+			if (bits && range_bits == bits) {
+				r_anal_hint_unset_bits (a, addr);
+			} else {
+				RAnalRange *range = R_NEW0 (RAnalRange);
+				range->bits = bits;
+				range->from = addr;
+				__anal_range_hint_tree_insert (&a->rb_hints_ranges, range);
 			}
+			range_bits = bits;
 		}
 		ls_free (sdb_range);
-		a->bits_hints_changed = false;
+		a->merge_hints = false;
 	}
 }
 
@@ -679,5 +798,30 @@ R_API void r_anal_bind(RAnal *anal, RAnalBind *b) {
 	if (b) {
 		b->anal = anal;
 		b->get_fcn_in = r_anal_get_fcn_in;
+		b->get_hint = r_anal_hint_get;
 	}
 }
+
+R_API RList *r_anal_preludes(RAnal *anal) {
+	if (anal->cur && anal->cur->preludes ) {
+		return anal->cur->preludes (anal);
+	}
+	return NULL;
+}
+
+R_API bool r_anal_is_prelude(RAnal *anal, const ut8 *data, int len) {
+	RList *l = r_anal_preludes (anal);
+	if (l) {
+		RSearchKeyword *kw;
+		RListIter *iter;
+		r_list_foreach (l, iter, kw) {
+			int ks = kw->keyword_length;
+			if (len >= ks && !memcmp (data, kw->bin_keyword, ks)) {
+				r_list_free (l);
+				return true;
+			}
+		}
+		r_list_free (l);
+	}
+	return false;
+}
