@@ -8,15 +8,16 @@ R_API RAnalCycleFrame *r_anal_cycle_frame_new() {
 	RAnalCycleFrame *cf = R_NEW0 (RAnalCycleFrame);
 	if (cf) {
 		if (!(cf->hooks = r_list_new ())) {
-			free (cf);
-			cf = NULL;
+			R_FREE (cf);
 		}
 	}
 	return cf;
 }
 
 R_API void r_anal_cycle_frame_free(RAnalCycleFrame *cf) {
-	if (!cf) return;
+	if (!cf) {
+		return;
+	}
 	r_list_free (cf->hooks);
 	free (cf);
 }
