@@ -31,8 +31,6 @@ RAnalReilArgType reil_get_arg_type(RAnalEsil *esil, char *s) {
 		return ARG_REG;
 	case R_ANAL_ESIL_PARM_NUM:
 		return ARG_CONST;
-	case R_ANAL_ESIL_PARM_INTERNAL:
-		return ARG_ESIL_INTERNAL;
 	default:
 		return ARG_NONE;
 	}
@@ -189,7 +187,7 @@ void reil_cast_size(RAnalEsil *esil, RAnalReilArg *src, RAnalReilArg *dst) {
 }
 
 // Here start translation functions!
-static int reil_eq(RAnalEsil *esil) {
+static bool reil_eq(RAnalEsil *esil) {
 	RAnalReilInst *ins;
 	char tmp_buf[REGBUFSZ];
 	RAnalReilArgType src_type, dst_type;
@@ -215,6 +213,8 @@ static int reil_eq(RAnalEsil *esil) {
 		// No direct register to register transfer.
 		ins = R_NEW0 (RAnalReilInst);
 		if (!ins) {
+			free (src);
+			free (dst);
 			return false;
 		}
 		ins->opcode = REIL_STR;
@@ -379,27 +379,27 @@ static int reil_bineqop(RAnalEsil *esil, RAnalReilOpcode opcode) {
 	return ret;
 }
 
-static int reil_add(RAnalEsil *esil)     { return reil_binop (esil, REIL_ADD);   }
-static int reil_addeq(RAnalEsil *esil)   { return reil_bineqop (esil, REIL_ADD); }
-static int reil_mul(RAnalEsil *esil)     { return reil_binop (esil, REIL_MUL);   }
-static int reil_muleq(RAnalEsil *esil)   { return reil_bineqop (esil, REIL_MUL); }
-static int reil_sub(RAnalEsil *esil)     { return reil_binop (esil, REIL_SUB);   }
-static int reil_subeq(RAnalEsil *esil)   { return reil_bineqop (esil, REIL_SUB); }
-static int reil_div(RAnalEsil *esil)     { return reil_binop (esil, REIL_DIV);   }
-static int reil_diveq(RAnalEsil *esil)   { return reil_bineqop (esil, REIL_DIV); }
-static int reil_xor(RAnalEsil *esil)     { return reil_binop (esil, REIL_XOR);   }
-static int reil_xoreq(RAnalEsil *esil)   { return reil_bineqop (esil, REIL_XOR); }
-static int reil_and(RAnalEsil *esil)     { return reil_binop (esil, REIL_AND);   }
-static int reil_andeq(RAnalEsil *esil)   { return reil_bineqop (esil, REIL_AND); }
-static int reil_or(RAnalEsil *esil)      { return reil_binop (esil, REIL_OR);    }
-static int reil_oreq(RAnalEsil *esil)    { return reil_bineqop (esil, REIL_OR);  }
-static int reil_lsl(RAnalEsil *esil)     { return reil_binop (esil, REIL_SHL);   }
-static int reil_lsleq(RAnalEsil *esil)   { return reil_bineqop (esil, REIL_SHL); }
-static int reil_lsr(RAnalEsil *esil)     { return reil_binop (esil, REIL_SHR);   }
-static int reil_lsreq(RAnalEsil *esil)   { return reil_bineqop (esil, REIL_SHR); }
-static int reil_smaller(RAnalEsil *esil) { return reil_binop (esil, REIL_LT);    }
+static bool reil_add(RAnalEsil *esil)     { return reil_binop (esil, REIL_ADD);   }
+static bool reil_addeq(RAnalEsil *esil)   { return reil_bineqop (esil, REIL_ADD); }
+static bool reil_mul(RAnalEsil *esil)     { return reil_binop (esil, REIL_MUL);   }
+static bool reil_muleq(RAnalEsil *esil)   { return reil_bineqop (esil, REIL_MUL); }
+static bool reil_sub(RAnalEsil *esil)     { return reil_binop (esil, REIL_SUB);   }
+static bool reil_subeq(RAnalEsil *esil)   { return reil_bineqop (esil, REIL_SUB); }
+static bool reil_div(RAnalEsil *esil)     { return reil_binop (esil, REIL_DIV);   }
+static bool reil_diveq(RAnalEsil *esil)   { return reil_bineqop (esil, REIL_DIV); }
+static bool reil_xor(RAnalEsil *esil)     { return reil_binop (esil, REIL_XOR);   }
+static bool reil_xoreq(RAnalEsil *esil)   { return reil_bineqop (esil, REIL_XOR); }
+static bool reil_and(RAnalEsil *esil)     { return reil_binop (esil, REIL_AND);   }
+static bool reil_andeq(RAnalEsil *esil)   { return reil_bineqop (esil, REIL_AND); }
+static bool reil_or(RAnalEsil *esil)      { return reil_binop (esil, REIL_OR);    }
+static bool reil_oreq(RAnalEsil *esil)    { return reil_bineqop (esil, REIL_OR);  }
+static bool reil_lsl(RAnalEsil *esil)     { return reil_binop (esil, REIL_SHL);   }
+static bool reil_lsleq(RAnalEsil *esil)   { return reil_bineqop (esil, REIL_SHL); }
+static bool reil_lsr(RAnalEsil *esil)     { return reil_binop (esil, REIL_SHR);   }
+static bool reil_lsreq(RAnalEsil *esil)   { return reil_bineqop (esil, REIL_SHR); }
+static bool reil_smaller(RAnalEsil *esil) { return reil_binop (esil, REIL_LT);    }
 
-static int reil_cmp(RAnalEsil *esil) {
+static bool reil_cmp(RAnalEsil *esil) {
 	RAnalReilInst *ins;
 	char tmp_buf[REGBUFSZ];
 	RAnalReilArg *op2, *op1;
@@ -447,7 +447,7 @@ static int reil_cmp(RAnalEsil *esil) {
 	return true;
 }
 
-static int reil_smaller_equal(RAnalEsil *esil) {
+static bool reil_smaller_equal(RAnalEsil *esil) {
 	RAnalReilArg *op2, *op1;
 
 	op2 = reil_pop_arg(esil);
@@ -473,7 +473,7 @@ static int reil_smaller_equal(RAnalEsil *esil) {
 	return true;
 }
 
-static int reil_larger(RAnalEsil *esil) {
+static bool reil_larger(RAnalEsil *esil) {
 	RAnalReilArg *op2 = reil_pop_arg(esil);
 	if (!op2) {
 		return false;
@@ -491,7 +491,7 @@ static int reil_larger(RAnalEsil *esil) {
 	return true;
 }
 
-static int reil_larger_equal(RAnalEsil *esil) {
+static bool reil_larger_equal(RAnalEsil *esil) {
 	RAnalReilArg *op2, *op1;
 
 	op2 = reil_pop_arg(esil);
@@ -512,7 +512,7 @@ static int reil_larger_equal(RAnalEsil *esil) {
 	return true;
 }
 
-static int reil_dec(RAnalEsil *esil) {
+static bool reil_dec(RAnalEsil *esil) {
 	RAnalReilArg *op = reil_pop_arg(esil);
 	if (!op) {
 		return false;
@@ -524,7 +524,7 @@ static int reil_dec(RAnalEsil *esil) {
 	return true;
 }
 
-static int reil_deceq(RAnalEsil *esil) {
+static bool reil_deceq(RAnalEsil *esil) {
 	RAnalReilArg *op1 = reil_pop_arg(esil);
 	if (!op1) {
 		return false;
@@ -537,7 +537,7 @@ static int reil_deceq(RAnalEsil *esil) {
 	return true;
 }
 
-static int reil_inc(RAnalEsil *esil) {
+static bool reil_inc(RAnalEsil *esil) {
 	RAnalReilArg *op = reil_pop_arg(esil);
 	if (!op) {
 		return false;
@@ -550,7 +550,7 @@ static int reil_inc(RAnalEsil *esil) {
 	return true;
 }
 
-static int reil_inceq(RAnalEsil *esil) {
+static bool reil_inceq(RAnalEsil *esil) {
 	RAnalReilArg *op = reil_pop_arg(esil);
 	if (!op) {
 		return false;
@@ -563,7 +563,7 @@ static int reil_inceq(RAnalEsil *esil) {
 	return true;
 }
 
-static int reil_neg(RAnalEsil *esil) {
+static bool reil_neg(RAnalEsil *esil) {
 	char tmp_buf[REGBUFSZ];
 	RAnalReilInst *ins;
 	RAnalReilArg *op = reil_pop_arg (esil);
@@ -601,7 +601,7 @@ static int reil_neg(RAnalEsil *esil) {
 	return true;
 }
 
-static int reil_negeq(RAnalEsil *esil) {
+static bool reil_negeq(RAnalEsil *esil) {
 	RAnalReilArg *op = reil_pop_arg(esil);
 	if (!op) {
 		return false;
@@ -614,7 +614,7 @@ static int reil_negeq(RAnalEsil *esil) {
 	return true;
 }
 
-static int reil_not(RAnalEsil *esil) {
+static bool reil_not(RAnalEsil *esil) {
 	char tmp_buf[REGBUFSZ];
 	RAnalReilInst *ins;
 	RAnalReilArg *op = reil_pop_arg (esil);
@@ -649,7 +649,7 @@ static int reil_not(RAnalEsil *esil) {
 	return true;
 }
 
-static int reil_if(RAnalEsil *esil) {
+static bool reil_if(RAnalEsil *esil) {
 	RAnalReilInst *ins;
 	RAnalReilArg *op2, *op1;
 
@@ -683,9 +683,9 @@ static int reil_if(RAnalEsil *esil) {
 	return true;
 }
 
-static int reil_if_end(RAnalEsil *esil) { return true; }
+static bool reil_if_end(RAnalEsil *esil) { return true; }
 
-static int reil_peek(RAnalEsil *esil) {
+static bool reil_peek(RAnalEsil *esil) {
 	RAnalReilInst *ins;
 	char tmp_buf[REGBUFSZ];
 	RAnalReilArg *op1 = reil_pop_arg(esil);
@@ -721,7 +721,7 @@ static int reil_peek(RAnalEsil *esil) {
 }
 
 // n = 8, 4, 2, 1
-static int reil_peekn(RAnalEsil *esil, ut8 n) {
+static bool reil_peekn(RAnalEsil *esil, ut8 n) {
 	RAnalReilArg *op2;
 	RAnalReilArg *op1 = reil_pop_arg (esil);
 	if (!op1) {
@@ -757,13 +757,13 @@ static int reil_peekn(RAnalEsil *esil, ut8 n) {
 	return true;
 }
 
-static int reil_peek1(RAnalEsil *esil) { return reil_peekn(esil, 1); }
-static int reil_peek2(RAnalEsil *esil) { return reil_peekn(esil, 2); }
-static int reil_peek4(RAnalEsil *esil) { return reil_peekn(esil, 4); }
-static int reil_peek8(RAnalEsil *esil) { return reil_peekn(esil, 8); }
+static bool reil_peek1(RAnalEsil *esil) { return reil_peekn(esil, 1); }
+static bool reil_peek2(RAnalEsil *esil) { return reil_peekn(esil, 2); }
+static bool reil_peek4(RAnalEsil *esil) { return reil_peekn(esil, 4); }
+static bool reil_peek8(RAnalEsil *esil) { return reil_peekn(esil, 8); }
 
 // n = 8, 4, 2, 1
-static int reil_poken(RAnalEsil *esil, ut8 n) {
+static bool reil_poken(RAnalEsil *esil, ut8 n) {
 	char tmp_buf[REGBUFSZ];
 	RAnalReilInst *ins;
 	RAnalReilArg *op2, *op1;
@@ -841,17 +841,17 @@ static int reil_poken(RAnalEsil *esil, ut8 n) {
 	return true;
 }
 
-static int reil_poke(RAnalEsil *esil) {
+static bool reil_poke(RAnalEsil *esil) {
 	return reil_poken (esil, esil->anal->bits / 8);
 }
 
-static int reil_poke1(RAnalEsil *esil) { return reil_poken(esil, 1); }
-static int reil_poke2(RAnalEsil *esil) { return reil_poken(esil, 2); }
-static int reil_poke4(RAnalEsil *esil) { return reil_poken(esil, 4); }
-static int reil_poke8(RAnalEsil *esil) { return reil_poken(esil, 8); }
+static bool reil_poke1(RAnalEsil *esil) { return reil_poken(esil, 1); }
+static bool reil_poke2(RAnalEsil *esil) { return reil_poken(esil, 2); }
+static bool reil_poke4(RAnalEsil *esil) { return reil_poken(esil, 4); }
+static bool reil_poke8(RAnalEsil *esil) { return reil_poken(esil, 8); }
 
 // Generic function to handle all mem_*eq_n functions. Example, mem_oreq_n
-static int reil_mem_bineq_n(RAnalEsil *esil, RAnalReilOpcode opcode, ut8 size) {
+static bool reil_mem_bineq_n(RAnalEsil *esil, RAnalReilOpcode opcode, ut8 size) {
 	int ret = 1;
 	RAnalReilArg *op2, *op1;
 
@@ -877,43 +877,43 @@ static int reil_mem_bineq_n(RAnalEsil *esil, RAnalReilOpcode opcode, ut8 size) {
 	return ret;
 }
 
-static int reil_mem_oreq(RAnalEsil *esil)  { return reil_mem_bineq_n(esil, REIL_OR, esil->anal->bits / 8); }
-static int reil_mem_oreq1(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_OR, 1); }
-static int reil_mem_oreq2(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_OR, 2); }
-static int reil_mem_oreq4(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_OR, 4); }
-static int reil_mem_oreq8(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_OR, 8); }
+static bool reil_mem_oreq(RAnalEsil *esil)  { return reil_mem_bineq_n(esil, REIL_OR, esil->anal->bits / 8); }
+static bool reil_mem_oreq1(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_OR, 1); }
+static bool reil_mem_oreq2(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_OR, 2); }
+static bool reil_mem_oreq4(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_OR, 4); }
+static bool reil_mem_oreq8(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_OR, 8); }
 
-static int reil_mem_andeq(RAnalEsil *esil)  { return reil_mem_bineq_n(esil, REIL_AND, esil->anal->bits / 8); }
-static int reil_mem_andeq1(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_AND, 1); }
-static int reil_mem_andeq2(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_AND, 2); }
-static int reil_mem_andeq4(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_AND, 4); }
-static int reil_mem_andeq8(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_AND, 8); }
+static bool reil_mem_andeq(RAnalEsil *esil)  { return reil_mem_bineq_n(esil, REIL_AND, esil->anal->bits / 8); }
+static bool reil_mem_andeq1(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_AND, 1); }
+static bool reil_mem_andeq2(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_AND, 2); }
+static bool reil_mem_andeq4(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_AND, 4); }
+static bool reil_mem_andeq8(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_AND, 8); }
 
-static int reil_mem_xoreq(RAnalEsil *esil)  { return reil_mem_bineq_n(esil, REIL_XOR, esil->anal->bits / 8); }
-static int reil_mem_xoreq1(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_XOR, 1); }
-static int reil_mem_xoreq2(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_XOR, 2); }
-static int reil_mem_xoreq4(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_XOR, 4); }
-static int reil_mem_xoreq8(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_XOR, 8); }
+static bool reil_mem_xoreq(RAnalEsil *esil)  { return reil_mem_bineq_n(esil, REIL_XOR, esil->anal->bits / 8); }
+static bool reil_mem_xoreq1(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_XOR, 1); }
+static bool reil_mem_xoreq2(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_XOR, 2); }
+static bool reil_mem_xoreq4(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_XOR, 4); }
+static bool reil_mem_xoreq8(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_XOR, 8); }
 
-static int reil_mem_addeq(RAnalEsil *esil)  { return reil_mem_bineq_n(esil, REIL_ADD, esil->anal->bits / 8); }
-static int reil_mem_addeq1(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_ADD, 1); }
-static int reil_mem_addeq2(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_ADD, 2); }
-static int reil_mem_addeq4(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_ADD, 4); }
-static int reil_mem_addeq8(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_ADD, 8); }
+static bool reil_mem_addeq(RAnalEsil *esil)  { return reil_mem_bineq_n(esil, REIL_ADD, esil->anal->bits / 8); }
+static bool reil_mem_addeq1(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_ADD, 1); }
+static bool reil_mem_addeq2(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_ADD, 2); }
+static bool reil_mem_addeq4(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_ADD, 4); }
+static bool reil_mem_addeq8(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_ADD, 8); }
 
-static int reil_mem_subeq(RAnalEsil *esil)  { return reil_mem_bineq_n(esil, REIL_SUB, esil->anal->bits / 8); }
-static int reil_mem_subeq1(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_SUB, 1); }
-static int reil_mem_subeq2(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_SUB, 2); }
-static int reil_mem_subeq4(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_SUB, 4); }
-static int reil_mem_subeq8(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_SUB, 8); }
+static bool reil_mem_subeq(RAnalEsil *esil)  { return reil_mem_bineq_n(esil, REIL_SUB, esil->anal->bits / 8); }
+static bool reil_mem_subeq1(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_SUB, 1); }
+static bool reil_mem_subeq2(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_SUB, 2); }
+static bool reil_mem_subeq4(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_SUB, 4); }
+static bool reil_mem_subeq8(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_SUB, 8); }
 
-static int reil_mem_muleq(RAnalEsil *esil)  { return reil_mem_bineq_n(esil, REIL_MUL, esil->anal->bits / 8); }
-static int reil_mem_muleq1(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_MUL, 1); }
-static int reil_mem_muleq2(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_MUL, 2); }
-static int reil_mem_muleq4(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_MUL, 4); }
-static int reil_mem_muleq8(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_MUL, 8); }
+static bool reil_mem_muleq(RAnalEsil *esil)  { return reil_mem_bineq_n(esil, REIL_MUL, esil->anal->bits / 8); }
+static bool reil_mem_muleq1(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_MUL, 1); }
+static bool reil_mem_muleq2(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_MUL, 2); }
+static bool reil_mem_muleq4(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_MUL, 4); }
+static bool reil_mem_muleq8(RAnalEsil *esil) { return reil_mem_bineq_n(esil, REIL_MUL, 8); }
 
-static int reil_mem_inceq_n(RAnalEsil *esil, ut8 size) {
+static bool reil_mem_inceq_n(RAnalEsil *esil, ut8 size) {
 	int ret = 1;
 	RAnalReilArg *op1 = reil_pop_arg(esil);
 	if (!op1) {
@@ -928,13 +928,13 @@ static int reil_mem_inceq_n(RAnalEsil *esil, ut8 size) {
 	return ret;
 }
 
-static int reil_mem_inceq(RAnalEsil *esil) {
+static bool reil_mem_inceq(RAnalEsil *esil) {
 	return reil_mem_inceq_n(esil, esil->anal->bits / 8);
 }
-static int reil_mem_inceq1(RAnalEsil *esil) { return reil_mem_inceq_n(esil, 1); }
-static int reil_mem_inceq2(RAnalEsil *esil) { return reil_mem_inceq_n(esil, 2); }
-static int reil_mem_inceq4(RAnalEsil *esil) { return reil_mem_inceq_n(esil, 4); }
-static int reil_mem_inceq8(RAnalEsil *esil) { return reil_mem_inceq_n(esil, 8); }
+static bool reil_mem_inceq1(RAnalEsil *esil) { return reil_mem_inceq_n(esil, 1); }
+static bool reil_mem_inceq2(RAnalEsil *esil) { return reil_mem_inceq_n(esil, 2); }
+static bool reil_mem_inceq4(RAnalEsil *esil) { return reil_mem_inceq_n(esil, 4); }
+static bool reil_mem_inceq8(RAnalEsil *esil) { return reil_mem_inceq_n(esil, 8); }
 
 static int reil_mem_deceq_n(RAnalEsil *esil, ut8 size) {
 	int ret = 1;
@@ -951,13 +951,13 @@ static int reil_mem_deceq_n(RAnalEsil *esil, ut8 size) {
 	return ret;
 }
 
-static int reil_mem_deceq(RAnalEsil *esil) {
+static bool reil_mem_deceq(RAnalEsil *esil) {
 	return reil_mem_deceq_n(esil, esil->anal->bits / 8);
 }
-static int reil_mem_deceq1(RAnalEsil *esil) { return reil_mem_deceq_n(esil, 1); }
-static int reil_mem_deceq2(RAnalEsil *esil) { return reil_mem_deceq_n(esil, 2); }
-static int reil_mem_deceq4(RAnalEsil *esil) { return reil_mem_deceq_n(esil, 4); }
-static int reil_mem_deceq8(RAnalEsil *esil) { return reil_mem_deceq_n(esil, 8); }
+static bool reil_mem_deceq1(RAnalEsil *esil) { return reil_mem_deceq_n(esil, 1); }
+static bool reil_mem_deceq2(RAnalEsil *esil) { return reil_mem_deceq_n(esil, 2); }
+static bool reil_mem_deceq4(RAnalEsil *esil) { return reil_mem_deceq_n(esil, 4); }
+static bool reil_mem_deceq8(RAnalEsil *esil) { return reil_mem_deceq_n(esil, 8); }
 
 // Functions to resolve internal vars.
 // performs (2 << op) - 1
@@ -1168,88 +1168,95 @@ R_API int r_anal_esil_to_reil_setup(RAnalEsil *esil, RAnal *anal, int romem,
 
 	r_anal_esil_mem_ro(esil, romem);
 
-	r_anal_esil_set_op(esil, "=", reil_eq);
-	r_anal_esil_set_op(esil, "+", reil_add);
-	r_anal_esil_set_op(esil, "+=", reil_addeq);
-	r_anal_esil_set_op(esil, "-", reil_sub);
-	r_anal_esil_set_op(esil, "-=", reil_subeq);
-	r_anal_esil_set_op(esil, "*", reil_mul);
-	r_anal_esil_set_op(esil, "*=", reil_muleq);
-	r_anal_esil_set_op(esil, "/", reil_div);
-	r_anal_esil_set_op(esil, "/=", reil_diveq);
-	r_anal_esil_set_op(esil, "^", reil_xor);
-	r_anal_esil_set_op(esil, "^=", reil_xoreq);
-	r_anal_esil_set_op(esil, "|", reil_or);
-	r_anal_esil_set_op(esil, "|=", reil_oreq);
-	r_anal_esil_set_op(esil, "&", reil_and);
-	r_anal_esil_set_op(esil, "&=", reil_andeq);
-	r_anal_esil_set_op(esil, "<<", reil_lsl);
-	r_anal_esil_set_op(esil, "<<=", reil_lsleq);
-	r_anal_esil_set_op(esil, ">>", reil_lsr);
-	r_anal_esil_set_op(esil, ">>=", reil_lsreq);
-	r_anal_esil_set_op(esil, "++=", reil_inceq);
-	r_anal_esil_set_op(esil, "++", reil_inc);
-	r_anal_esil_set_op(esil, "--=", reil_deceq);
-	r_anal_esil_set_op(esil, "--", reil_dec);
-	r_anal_esil_set_op(esil, "!", reil_neg);
-	r_anal_esil_set_op(esil, "!=", reil_negeq);
-	r_anal_esil_set_op(esil, "==", reil_cmp);
-	r_anal_esil_set_op(esil, "<", reil_smaller);
-	r_anal_esil_set_op(esil, ">", reil_larger);
-	r_anal_esil_set_op(esil, "<=", reil_smaller_equal);
-	r_anal_esil_set_op(esil, ">=", reil_larger_equal);
-	r_anal_esil_set_op(esil, "[]", reil_peek);
-	r_anal_esil_set_op(esil, "=[]", reil_poke);
-	r_anal_esil_set_op(esil, "|=[]", reil_mem_oreq);
-	r_anal_esil_set_op(esil, "^=[]", reil_mem_xoreq);
-	r_anal_esil_set_op(esil, "&=[]", reil_mem_andeq);
-	r_anal_esil_set_op(esil, "+=[]", reil_mem_addeq);
-	r_anal_esil_set_op(esil, "-=[]", reil_mem_subeq);
-	r_anal_esil_set_op(esil, "*=[]", reil_mem_muleq);
-	r_anal_esil_set_op(esil, "++=[]", reil_mem_inceq);
-	r_anal_esil_set_op(esil, "--=[]", reil_mem_deceq);
-	r_anal_esil_set_op(esil, "=[1]", reil_poke1);
-	r_anal_esil_set_op(esil, "=[2]", reil_poke2);
-	r_anal_esil_set_op(esil, "=[4]", reil_poke4);
-	r_anal_esil_set_op(esil, "=[8]", reil_poke8);
-	r_anal_esil_set_op(esil, "[1]", reil_peek1);
-	r_anal_esil_set_op(esil, "[2]", reil_peek2);
-	r_anal_esil_set_op(esil, "[4]", reil_peek4);
-	r_anal_esil_set_op(esil, "[8]", reil_peek8);
-	r_anal_esil_set_op(esil, "|=[1]", reil_mem_oreq1);
-	r_anal_esil_set_op(esil, "|=[2]", reil_mem_oreq2);
-	r_anal_esil_set_op(esil, "|=[4]", reil_mem_oreq4);
-	r_anal_esil_set_op(esil, "|=[8]", reil_mem_oreq8);
-	r_anal_esil_set_op(esil, "^=[1]", reil_mem_xoreq1);
-	r_anal_esil_set_op(esil, "^=[2]", reil_mem_xoreq2);
-	r_anal_esil_set_op(esil, "^=[4]", reil_mem_xoreq4);
-	r_anal_esil_set_op(esil, "^=[8]", reil_mem_xoreq8);
-	r_anal_esil_set_op(esil, "&=[1]", reil_mem_andeq1);
-	r_anal_esil_set_op(esil, "&=[2]", reil_mem_andeq2);
-	r_anal_esil_set_op(esil, "&=[4]", reil_mem_andeq4);
-	r_anal_esil_set_op(esil, "&=[8]", reil_mem_andeq8);
-	r_anal_esil_set_op(esil, "+=[1]", reil_mem_addeq1);
-	r_anal_esil_set_op(esil, "+=[2]", reil_mem_addeq2);
-	r_anal_esil_set_op(esil, "+=[4]", reil_mem_addeq4);
-	r_anal_esil_set_op(esil, "+=[8]", reil_mem_addeq8);
-	r_anal_esil_set_op(esil, "-=[1]", reil_mem_subeq1);
-	r_anal_esil_set_op(esil, "-=[2]", reil_mem_subeq2);
-	r_anal_esil_set_op(esil, "-=[4]", reil_mem_subeq4);
-	r_anal_esil_set_op(esil, "-=[8]", reil_mem_subeq8);
-	r_anal_esil_set_op(esil, "*=[1]", reil_mem_muleq1);
-	r_anal_esil_set_op(esil, "*=[2]", reil_mem_muleq2);
-	r_anal_esil_set_op(esil, "*=[4]", reil_mem_muleq4);
-	r_anal_esil_set_op(esil, "*=[8]", reil_mem_muleq8);
-	r_anal_esil_set_op(esil, "++=[1]", reil_mem_inceq1);
-	r_anal_esil_set_op(esil, "++=[2]", reil_mem_inceq2);
-	r_anal_esil_set_op(esil, "++=[4]", reil_mem_inceq4);
-	r_anal_esil_set_op(esil, "++=[8]", reil_mem_inceq8);
-	r_anal_esil_set_op(esil, "--=[1]", reil_mem_deceq1);
-	r_anal_esil_set_op(esil, "--=[2]", reil_mem_deceq2);
-	r_anal_esil_set_op(esil, "--=[4]", reil_mem_deceq4);
-	r_anal_esil_set_op(esil, "--=[8]", reil_mem_deceq8);
-	r_anal_esil_set_op(esil, "?{", reil_if);
-	r_anal_esil_set_op(esil, "}", reil_if_end);
+#define	OT_UNK	R_ANAL_ESIL_OP_TYPE_UNKNOWN
+#define	OT_CTR	R_ANAL_ESIL_OP_TYPE_CONTROL_FLOW
+#define	OT_MATH	R_ANAL_ESIL_OP_TYPE_MATH
+#define	OT_REGW	R_ANAL_ESIL_OP_TYPE_REG_WRITE
+#define	OT_MEMW	R_ANAL_ESIL_OP_TYPE_MEM_WRITE
+#define	OT_MEMR	R_ANAL_ESIL_OP_TYPE_MEM_READ
+
+	r_anal_esil_set_op(esil, "=", reil_eq, 0, 2, OT_REGW);
+	r_anal_esil_set_op(esil, "+", reil_add, 1, 2, OT_MATH);
+	r_anal_esil_set_op(esil, "+=", reil_addeq, 0, 2, OT_MATH | OT_REGW);
+	r_anal_esil_set_op(esil, "-", reil_sub, 1, 2, OT_MATH);
+	r_anal_esil_set_op(esil, "-=", reil_subeq, 0, 2, OT_MATH | OT_REGW);
+	r_anal_esil_set_op(esil, "*", reil_mul, 1, 2, OT_MATH);
+	r_anal_esil_set_op(esil, "*=", reil_muleq, 0, 2, OT_MATH | OT_REGW);
+	r_anal_esil_set_op(esil, "/", reil_div, 1, 2, OT_MATH);
+	r_anal_esil_set_op(esil, "/=", reil_diveq, 0, 2, OT_MATH | OT_REGW);
+	r_anal_esil_set_op(esil, "^", reil_xor, 1, 2, OT_MATH);
+	r_anal_esil_set_op(esil, "^=", reil_xoreq, 0, 2, OT_MATH | OT_REGW);
+	r_anal_esil_set_op(esil, "|", reil_or, 1, 2, OT_MATH);
+	r_anal_esil_set_op(esil, "|=", reil_oreq, 0, 2, OT_MATH | OT_REGW);
+	r_anal_esil_set_op(esil, "&", reil_and, 1, 2, OT_MATH);
+	r_anal_esil_set_op(esil, "&=", reil_andeq, 0, 2, OT_MATH | OT_REGW);
+	r_anal_esil_set_op(esil, "<<", reil_lsl, 1, 2, OT_MATH);
+	r_anal_esil_set_op(esil, "<<=", reil_lsleq, 0, 2, OT_MATH | OT_REGW);
+	r_anal_esil_set_op(esil, ">>", reil_lsr, 1, 2, OT_MATH);
+	r_anal_esil_set_op(esil, ">>=", reil_lsreq, 0, 2, OT_MATH | OT_REGW);
+	r_anal_esil_set_op(esil, "++=", reil_inceq, 0, 1, OT_MATH | OT_REGW);
+	r_anal_esil_set_op(esil, "++", reil_inc, 1, 1, OT_MATH);
+	r_anal_esil_set_op(esil, "--=", reil_deceq, 0, 1, OT_MATH | OT_REGW);
+	r_anal_esil_set_op(esil, "--", reil_dec, 1, 1, OT_MATH);
+	r_anal_esil_set_op(esil, "!", reil_neg, 1, 1, OT_MATH);
+	r_anal_esil_set_op(esil, "!=", reil_negeq, 0, 1, OT_MATH);
+	r_anal_esil_set_op(esil, "==", reil_cmp, 0, 2, OT_MATH);
+	r_anal_esil_set_op(esil, "<", reil_smaller, 1, 2, OT_MATH);
+	r_anal_esil_set_op(esil, ">", reil_larger, 1, 2, OT_MATH);
+	r_anal_esil_set_op(esil, "<=", reil_smaller_equal, 1, 2, OT_MATH);
+	r_anal_esil_set_op(esil, ">=", reil_larger_equal, 1, 2, OT_MATH);
+	r_anal_esil_set_op(esil, "[]", reil_peek, 1, 1, OT_MEMR);
+	r_anal_esil_set_op(esil, "=[]", reil_poke, 0, 2, OT_MEMW);
+	r_anal_esil_set_op(esil, "|=[]", reil_mem_oreq, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "^=[]", reil_mem_xoreq, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "&=[]", reil_mem_andeq, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "+=[]", reil_mem_addeq, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "-=[]", reil_mem_subeq, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "*=[]", reil_mem_muleq, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "++=[]", reil_mem_inceq, 0, 1, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "--=[]", reil_mem_deceq, 0, 1, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "=[1]", reil_poke1, 0, 2, OT_MEMW);
+	r_anal_esil_set_op(esil, "=[2]", reil_poke2, 0, 2, OT_MEMW);
+	r_anal_esil_set_op(esil, "=[4]", reil_poke4, 0, 2, OT_MEMW);
+	r_anal_esil_set_op(esil, "=[8]", reil_poke8, 0, 2, OT_MEMW);
+	r_anal_esil_set_op(esil, "[1]", reil_peek1, 1, 1, OT_MEMR);
+	r_anal_esil_set_op(esil, "[2]", reil_peek2, 1, 1, OT_MEMR);
+	r_anal_esil_set_op(esil, "[4]", reil_peek4, 1, 1, OT_MEMR);
+	r_anal_esil_set_op(esil, "[8]", reil_peek8, 1, 1, OT_MEMR);
+	r_anal_esil_set_op(esil, "|=[1]", reil_mem_oreq1, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "|=[2]", reil_mem_oreq2, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "|=[4]", reil_mem_oreq4, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "|=[8]", reil_mem_oreq8, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "^=[1]", reil_mem_xoreq1, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "^=[2]", reil_mem_xoreq2, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "^=[4]", reil_mem_xoreq4, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "^=[8]", reil_mem_xoreq8, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "&=[1]", reil_mem_andeq1, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "&=[2]", reil_mem_andeq2, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "&=[4]", reil_mem_andeq4, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "&=[8]", reil_mem_andeq8, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "+=[1]", reil_mem_addeq1, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "+=[2]", reil_mem_addeq2, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "+=[4]", reil_mem_addeq4, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "+=[8]", reil_mem_addeq8, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "-=[1]", reil_mem_subeq1, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "-=[2]", reil_mem_subeq2, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "-=[4]", reil_mem_subeq4, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "-=[8]", reil_mem_subeq8, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "*=[1]", reil_mem_muleq1, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "*=[2]", reil_mem_muleq2, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "*=[4]", reil_mem_muleq4, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "*=[8]", reil_mem_muleq8, 0, 2, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "++=[1]", reil_mem_inceq1, 0, 1, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "++=[2]", reil_mem_inceq2, 0, 1, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "++=[4]", reil_mem_inceq4, 0, 1, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "++=[8]", reil_mem_inceq8, 0, 1, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "--=[1]", reil_mem_deceq1, 0, 1, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "--=[2]", reil_mem_deceq2, 0, 1, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "--=[4]", reil_mem_deceq4, 0, 1, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "--=[8]", reil_mem_deceq8, 0, 1, OT_MATH | OT_MEMR | OT_MEMW);
+	r_anal_esil_set_op(esil, "?{", reil_if, 0, 1, OT_CTR);
+	r_anal_esil_set_op(esil, "}", reil_if_end, 0, 0, OT_CTR);
 
 	return true;
 }
