@@ -239,7 +239,7 @@ static int arcompact_genops(RAnalOp *op, ut64 addr, ut32 words[2]) {
 		if ((fields.format == 1 || fields.format == 2) && fields.b == ARC_REG_PCL) {
 			/* dst = PCL + src */
 			op->ptr = (addr & ~3) + fields.imm;
-			op->refptr = 1; /* HACK! we dont actually know what size it is */
+			op->refptr = 1; /* HACK! we don't actually know what size it is */
 		}
 	case 0x01: /* add with carry */
 	case 0x14: /* add with left shift by 1 */
@@ -454,7 +454,7 @@ static int arcompact_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, in
 
 	/* no unaligned code */
 	if (addr % 2 != 0) {
-		/* this fixes some of the reverse dissassembly issues */
+		/* this fixes some of the reverse disassembly issues */
 		op->type = R_ANAL_OP_TYPE_ILL;
 		return 0;
 	}
@@ -503,8 +503,8 @@ static int arcompact_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, in
 		} else {
 			/* Branch Unconditional Far 0x00 [0x1] */
 			fields.limm |= (fields.c & 0x0f) << 21;
-			/* the  & 0xf clearly shows we dont overflow */
-			/* TODO: dont generate code to show this */
+			/* the  & 0xf clearly shows we don't overflow */
+			/* TODO: don't generate code to show this */
 			fields.limm = SEX_S25 (fields.limm);
 			op->type = R_ANAL_OP_TYPE_JMP;
 		}
@@ -550,8 +550,8 @@ static int arcompact_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, in
 			} else {
 				/* Branch and Link Unconditional Far, 0x01, [0x0, 0x1] */
 				fields.imm |= (fields.c & 0x0f) << 21;
-				/* the  & 0xf clearly shows we dont overflow */
-				/* TODO: dont generate code to show this */
+				/* the  & 0xf clearly shows we don't overflow */
+				/* TODO: don't generate code to show this */
 				fields.imm = SEX_S25 (fields.imm);
 				op->type = R_ANAL_OP_TYPE_CALL;
 			}
@@ -839,7 +839,7 @@ static int arcompact_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, in
 		case 0x1e:
 			op->type = R_ANAL_OP_TYPE_TRAP;
 			/* TODO: the description sounds more like a */
-			/* R_ANAL_OP_TYPE_SWI, but I dont know what */
+			/* R_ANAL_OP_TYPE_SWI, but I don't know what */
 			/* difference that would make to radare */
 			break;
 		case 0x1f:
@@ -1013,9 +1013,8 @@ static int arcompact_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, in
 	return op->size;
 }
 
-static int arc_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int len) {
+static int arc_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int len, RAnalOpMask mask) {
 	const ut8 *b = (ut8 *)data;
-	memset (op, '\0', sizeof (RAnalOp));
 
 	if (anal->bits == 16) {
 		return arcompact_op (anal, op, addr, data, len);
@@ -1024,7 +1023,7 @@ static int arc_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int len)
 	/* ARCtangent A4 */
 	op->size = 4;
 	op->fail = addr + 4;
-	ut8 basecode = (b[3] & 0xf8) >> 3;
+	ut8 basecode = (len > 3)? ((b[3] & 0xf8) >> 3): 0;
 	switch (basecode) {
 	case 0x04: /* Branch */
 	case 0x05: /* Branch with Link */
@@ -1149,7 +1148,7 @@ RAnalPlugin r_anal_plugin_arc = {
 	.set_reg_profile = set_reg_profile,
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ANAL,
 	.data = &r_anal_plugin_arc,
