@@ -12,17 +12,17 @@
 #define emit(frag) r_strbuf_appendf(&op->esil, frag)
 #define emitf(...) r_strbuf_appendf(&op->esil, __VA_ARGS__)
 //setting the appropriate flags, NOTE: semicolon included
-#define setZ r_strbuf_appendf(&op->esil, ",$z,Z,=") //zero flag
-#define setN r_strbuf_appendf(&op->esil, ",$s,N,=") //negative(sign) flag
+#define setZ r_strbuf_appendf(&op->esil, ",$z,Z,:=") //zero flag
+#define setN r_strbuf_appendf(&op->esil, ",15,$s,N,=") //negative(sign) flag
 #define setV(val) r_strbuf_appendf(&op->esil, ",%s,V,=", val) //overflow flag
-#define setC_B r_strbuf_appendf(&op->esil, ",$c7,C,=") //carry flag for byte op
-#define setC_W r_strbuf_appendf(&op->esil, ",$c15,C,=") //carryflag for word op
-#define setCb_B r_strbuf_appendf(&op->esil, ",$b7,C,=") //borrow flag for byte
-#define setCb_W r_strbuf_appendf(&op->esil, ",$b15,C,=") //borrow flag for word
-#define setH_B r_strbuf_appendf(&op->esil, ",$c3,H,=") //half carry(byte)-bcd
-#define setH_W r_strbuf_appendf(&op->esil, ",$c11,H,=") //half carry(word)-bcd
-#define setHb_B r_strbuf_appendf(&op->esil, ",$b3,H,=") //half borrow(byte)-bcd
-#define setHb_W r_strbuf_appendf(&op->esil, ",$b11,H,=") //halfborrow(word)-bcd
+#define setC_B r_strbuf_appendf(&op->esil, ",7,$c,C,:=") //carry flag for byte op
+#define setC_W r_strbuf_appendf(&op->esil, ",15,$c,C,:=") //carryflag for word op
+#define setCb_B r_strbuf_appendf(&op->esil, ",7,$b,C,:=") //borrow flag for byte
+#define setCb_W r_strbuf_appendf(&op->esil, ",15,$b,C,:=") //borrow flag for word
+#define setH_B r_strbuf_appendf(&op->esil, ",3,$c,H,:=") //half carry(byte)-bcd
+#define setH_W r_strbuf_appendf(&op->esil, ",11,$c,H,:=") //half carry(word)-bcd
+#define setHb_B r_strbuf_appendf(&op->esil, ",3,$b,H,:=") //half borrow(byte)-bcd
+#define setHb_W r_strbuf_appendf(&op->esil, ",11,$b,H,:=") //halfborrow(word)-bcd
 
 //get reg. from opcodes
 #define rs() (buf[1]&0x70)>>4 //upper nibble used as source generally
@@ -538,7 +538,7 @@ static int analop_esil(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf) {
 }
 
 static int h8300_op(RAnal *anal, RAnalOp *op, ut64 addr,
-		const ut8 *buf, int len)
+		const ut8 *buf, int len, RAnalOpMask mask)
 {
 	int ret;
 	ut8 opcode = buf[0];
@@ -678,7 +678,7 @@ static int h8300_op(RAnal *anal, RAnalOp *op, ut64 addr,
 		op->type = R_ANAL_OP_TYPE_UNK;
 		break;
 	};
-	if (anal->decode) {
+	if (mask & R_ANAL_OP_MASK_ESIL) {
 		analop_esil(anal, op, addr, buf);
 	}
 	return ret;
@@ -736,8 +736,8 @@ RAnalPlugin r_anal_plugin_h8300 = {
 	.set_reg_profile = set_reg_profile,
 };
 
-#ifndef CORELIB
-struct r_lib_sturct_t radare_plugin = {
+#ifndef R2_PLUGIN_INCORE
+struct r_lib_struct_t radare_plugin = {
 	.type = R_LIB_TYPE_ANAL,
 	.data = &r_anal_plugin_h8300,
 	.version = R2_VERSION
