@@ -21,7 +21,7 @@ static const char* mips_reg_decode(unsigned reg_num) {
 	return NULL;
 }
 
-static int mips_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *b, int len) {
+static int mips_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *b, int len, RAnalOpMask mask) {
 	ut32 opcode;
 	// WIP char buf[10]; int reg; int family;
 	int optype, oplen = (anal->bits==16)?2:4;
@@ -275,7 +275,7 @@ static int mips_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *b, int len) {
 		// but the macro needs the opcode values as input
 		//
 		// TODO: this is a stop-gap. Really we need some smarts in here to tie this into the
-		// flags directly, as suggested here: https://github.com/radare/radare2/issues/949#issuecomment-43654922
+		// flags directly, as suggested here: https://github.com/radareorg/radare2/issues/949#issuecomment-43654922
 		case 15: // lui
 			op->dst = r_anal_value_new ();
 			op->dst->reg = r_reg_get (anal->reg, mips_reg_decode(rt), R_REG_TYPE_GPR);
@@ -540,7 +540,7 @@ RAnalPlugin r_anal_plugin_mips_gnu = {
 	.set_reg_profile = mips_set_reg_profile,
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
         .type = R_LIB_TYPE_ANAL,
         .data = &r_anal_plugin_mips_gnu
