@@ -51,8 +51,6 @@ static void opex(RStrBuf *buf, csh handle, cs_insn *insn) {
 	r_strbuf_append (buf, "}");
 }
 
-#define ZERO_FILL(x) memset (&x, 0, sizeof (x))
-
 static int parse_reg_name(RRegItem *reg, csh handle, cs_insn *insn, int reg_num) {
 	if (!reg) {
 		return -1;
@@ -96,7 +94,7 @@ static void op_fillval(RAnalOp *op, csh handle, cs_insn *insn) {
 	}
 }
 
-static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
+static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len, RAnalOpMask mask) {
 	static csh handle = 0;
 	static int omode;
 	cs_insn *insn;
@@ -107,8 +105,9 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 	}
 
 	mode = CS_MODE_LITTLE_ENDIAN;
-	if (!strcmp (a->cpu, "v9"))
+	if (!strcmp (a->cpu, "v9")) {
 		mode |= CS_MODE_V9;
+	}
 	if (mode != omode) {
 		cs_close (&handle);
 		handle = 0;
@@ -134,7 +133,9 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 	if (n < 1) {
 		op->type = R_ANAL_OP_TYPE_ILL;
 	} else {
-		opex (&op->opex, handle, insn);
+		if (mask & R_ANAL_OP_MASK_OPEX) {
+			opex (&op->opex, handle, insn);
+		}
 		op->size = insn->size;
 		op->id = insn->id;
 		switch (insn->id) {
@@ -145,7 +146,10 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 			op->type = R_ANAL_OP_TYPE_MOV;
 			break;
 		case SPARC_INS_RETT:
+		case SPARC_INS_RET:
+		case SPARC_INS_RETL:
 			op->type = R_ANAL_OP_TYPE_RET;
+			op->delay = 1;
 			break;
 		case SPARC_INS_UNIMP:
 			op->type = R_ANAL_OP_TYPE_UNK;
@@ -157,9 +161,11 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 				break;
 			case SPARC_OP_REG:
 				op->type = R_ANAL_OP_TYPE_UCALL;
+				op->delay = 1;
 				break;
 			default:
 				op->type = R_ANAL_OP_TYPE_CALL;
+				op->delay = 1;
 				op->jump = INSOP(0).imm;
 				break;
 			}
@@ -173,6 +179,7 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 		case SPARC_INS_JMP:
 		case SPARC_INS_JMPL:
 			op->type = R_ANAL_OP_TYPE_JMP;
+			op->delay = 1;
 			op->jump = INSOP(0).imm;
 			break;
 		case SPARC_INS_LDD:
@@ -213,17 +220,23 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 			switch (INSOP(0).type) {
 			case SPARC_OP_REG:
 				op->type = R_ANAL_OP_TYPE_CJMP;
-				if (INSCC != SPARC_CC_ICC_N) // never
-					op->jump = INSOP(1).imm;
-				if (INSCC != SPARC_CC_ICC_A) // always
-					op->fail = addr+4;
+				op->delay = 1;
+				if (INSCC != SPARC_CC_ICC_N) { // never
+					op->jump = INSOP (1).imm;
+				}
+				if (INSCC != SPARC_CC_ICC_A) { // always
+					op->fail = addr + 8;
+				}
 				break;
 			case SPARC_OP_IMM:
 				op->type = R_ANAL_OP_TYPE_CJMP;
-				if (INSCC != SPARC_CC_ICC_N) // never
-					op->jump = INSOP(0).imm;
-				if (INSCC != SPARC_CC_ICC_A) // always
-					op->fail = addr+4;
+				op->delay = 1;
+				if (INSCC != SPARC_CC_ICC_N) { // never
+					op->jump = INSOP (0).imm;
+				}
+				if (INSCC != SPARC_CC_ICC_A) { // always
+					op->fail = addr + 8;
+				}
 				break;
 			default:
 				// MEM?
@@ -305,7 +318,7 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 			op->type = R_ANAL_OP_TYPE_DIV;
 			break;
 		}
-		if (a->fillval) {
+		if (mask & R_ANAL_OP_MASK_VAL) {
 			op_fillval (op, handle, insn);
 		}
 		cs_free (insn, n);
@@ -381,8 +394,8 @@ RAnalPlugin r_anal_plugin_sparc_cs = {
 	.set_reg_profile = &set_reg_profile,
 };
 
-#ifndef CORELIB
-RLibStruct radare_plugin = {
+#ifndef R2_PLUGIN_INCORE
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ANAL,
 	.data = &r_anal_plugin_sparc_cs,
 	.version = R2_VERSION
