@@ -84,7 +84,7 @@ enum {
 	FCC_UL = 0x3,
 	FCC_ULE = 0xe,
 };
-/* Define some additional conditions that are nor mapable to
+/* Define some additional conditions that are nor mappable to
    the existing R_ANAL_COND* ones and need to be handled in a
    special way. */
 enum {
@@ -369,7 +369,7 @@ static void anal_branch(RAnalOp *op, const ut32 insn, const ut64 addr) {
 }
 
 // TODO: this implementation is just a fast hack. needs to be rewritten and completed
-static int sparc_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int len) {
+static int sparc_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int len, RAnalOpMask mask) {
 	int sz = 4;
 	ut32 insn;
 
@@ -619,7 +619,7 @@ RAnalPlugin r_anal_plugin_sparc_gnu = {
 	.set_reg_profile = set_reg_profile,
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ANAL,
 	.data = &r_anal_plugin_sparc_gnu,
