@@ -1,8 +1,8 @@
-/* radare - LGPL - Copyright 2014-2018 - pancake, dso */
+/* radare - LGPL - Copyright 2014-2019 - pancake, dso */
 
 #include <r_anal.h>
 
-RAnalSwitchOp *switch_op_new() {
+static RAnalSwitchOp *__switch_op_new() {
 	RAnalSwitchOp * swop = R_NEW0 (RAnalSwitchOp);
 	if (swop) {
 		swop->cases = r_list_new ();
@@ -17,7 +17,7 @@ RAnalSwitchOp *switch_op_new() {
 }
 
 R_API RAnalSwitchOp * r_anal_switch_op_new(ut64 addr, ut64 min_val, ut64 def_val) {
-	RAnalSwitchOp *swop = switch_op_new ();
+	RAnalSwitchOp *swop = __switch_op_new ();
 	if (swop) {
 		swop->addr = addr;
 		swop->min_val = min_val;
@@ -27,6 +27,16 @@ R_API RAnalSwitchOp * r_anal_switch_op_new(ut64 addr, ut64 min_val, ut64 def_val
 	return swop;
 }
 
+R_API RAnalCaseOp * r_anal_case_op_new(ut64 addr, ut64 val, ut64 jump) {
+	RAnalCaseOp *c = R_NEW0 (RAnalCaseOp);
+	if (c) {
+		c->addr = addr;
+		c->value = val;
+		c->jump = jump;
+	}
+	return c;
+}
+
 R_API void r_anal_switch_op_free(RAnalSwitchOp * swop) {
 	if (swop) {
 		r_list_free (swop->cases);
@@ -35,18 +45,10 @@ R_API void r_anal_switch_op_free(RAnalSwitchOp * swop) {
 }
 
 R_API RAnalCaseOp* r_anal_switch_op_add_case(RAnalSwitchOp * swop, ut64 addr, ut64 value, ut64 jump) {
-	if (!swop) {
-		return NULL;
+	r_return_val_if_fail (swop && addr != UT64_MAX, NULL);
+	RAnalCaseOp * caseop = r_anal_case_op_new (addr, value, jump);
+	if (caseop) {
+		r_list_append (swop->cases, caseop);
 	}
-	RAnalCaseOp * caseop = R_NEW0 (RAnalCaseOp);
-	if (!caseop) {
-		return NULL;
-	}
-	caseop->addr = addr;
-	caseop->value = value;
-	caseop->jump = jump;
-	r_list_append (swop->cases, caseop);
 	return caseop;
 }
-
-
