@@ -138,39 +138,41 @@ static int _6502Disass (ut64 pc, RAsmOp *op, const ut8 *buf, ut64 len) {
 	int i;
 	for (i=0; ops[i].name != NULL; i++) {
 		if (ops[i].op == buf[0]) {
+			const char *buf_asm = "invalid";
+			int len = ops[i].len;
 			switch (ops[i].len) {
 			case 1:
-				sprintf (op->buf_asm, "%s", ops[i].name);
+				buf_asm = sdb_fmt ("%s", ops[i].name);
 				break;
 			case 2:
-				if (len>1) {
-					sprintf (op->buf_asm, ops[i].name, buf[1]);
+				if (len > 1) {
+					buf_asm = sdb_fmt (ops[i].name, buf[1]);
 				} else {
-					strcpy (op->buf_asm, "truncated");
-					return -1;
+					buf_asm = "truncated";
+					len = -1;
 				}
 				break;
 			case 3:
-				if (len>2) {
-					snprintf (op->buf_asm, sizeof (op->buf_asm), ops[i].name, buf[1]+0x100*buf[2]);
+				if (len > 2) {
+					buf_asm = sdb_fmt (ops[i].name, buf[1] + 0x100 * buf[2]);
 				} else {
-					strcpy (op->buf_asm, "truncated");
-					return -1;
+					buf_asm = "truncated";
+					len = -1;
 				}
 				break;
 			case 4:
-				if (len>3) {
-					sprintf (op->buf_asm, ops[i].name,
-						buf[1]+0x100*buf[2]+0x10000*buf[3]);
+				if (len > 3) {
+					buf_asm = sdb_fmt (ops[i].name, buf[1]+0x100*buf[2]+0x10000*buf[3]);
 				} else {
-					strcpy (op->buf_asm, "truncated");
-					return -1;
+					buf_asm = "truncated";
+					len = -1;
 				}
 				break;
 			default:
 				goto beach;
 			}
-			return ops[i].len;
+			r_strbuf_set (&op->buf_asm, buf_asm);
+			return len;
 		}
 	}
 beach:
