@@ -196,20 +196,32 @@ static ut32 mov(ArmOp *op) {
 		}
 	} else if (!strncmp (op->mnemonic, "mov", 3)) {
 		//printf ("%d - %d [%d]\n", op->operands[0].type, op->operands[1].type, ARM_GPR);
+		if (op->operands[0].reg_type & ARM_REG64) {
+            if (op->operands[1].reg_type & ARM_REG64) {
+                k = 0xe00300aa;
+            } else if (op->operands[1].type & ARM_CONSTANT) {
+                k = 0x80d2;
+            } else {
+                return data;
+            }
+        } else if (op->operands[0].reg_type & ARM_REG32) {
+            if (op->operands[1].reg_type & ARM_REG32) {
+                k = 0xe003002a;
+            } else if (op->operands[1].type & ARM_CONSTANT) {
+                k = 0x8052;
+            } else {
+                return data;
+            }
+        }
 		if (op->operands[0].type & ARM_GPR) {
-			if (op->operands[1].type & ARM_GPR) {
-				if (op->operands[1].reg_type & ARM_REG64) {
-					k = 0xe00300aa;
-				} else {
-					k = 0xe003002a;
-				}
-				data = k | op->operands[1].reg << 8;
-			} else if (op->operands[1].type & ARM_CONSTANT) {
-				k = 0x80d2;
-				data = k | op->operands[1].immediate << 29;
-			}
+            if (op->operands[1].type & ARM_GPR) {
+                data = k | op->operands[1].reg << 8;
+            } else if (op->operands[1].type & ARM_CONSTANT) {
+                ut32 imm = op->operands[1].immediate << 1;
+                data = k | ((imm & 0xf) << 28) | ((imm & 0x1f0) << 12) ;
+            }
 			data |=  op->operands[0].reg << 24;
-		}
+        }
 		return data;
 	}
 
@@ -221,6 +233,37 @@ static ut32 mov(ArmOp *op) {
 	return data;
 }
 
+static ut32 cb(ArmOp *op) {
+	ut32 data = UT32_MAX;
+	int k = 0;
+    if (!strncmp (op->mnemonic, "cbnz", 4)) {
+        if (op->operands[0].reg_type & ARM_REG64) {
+            k =  0x000000b5;
+        } else if (op->operands[0].reg_type & ARM_REG32) {
+            k =  0x00000035;
+        } else {
+            return UT32_MAX;
+        }
+    } else if (!strncmp (op->mnemonic, "cbz", 3)) {
+        if (op->operands[0].reg_type & ARM_REG64) {
+            k =  0x000000b4;
+        } else if (op->operands[0].reg_type & ARM_REG32) {
+            k =  0x00000034;
+        } else {
+            return UT32_MAX;
+        }
+    } else {
+        return UT32_MAX;
+    }
+    //printf ("%s %d, %llu\n", op->mnemonic, op->operands[0].reg, op->operands[1].immediate);
+    ut32 imm = op->operands[1].immediate;
+	data = k | (op->operands[0].reg << 24) | ((imm & 0x1c) << 27) | ((imm & 0x1fe0) << 11);
+    data = data | ((imm & 0x1fe000) >> 5);
+
+	return data;
+}
+
+
 static ut32 cmp(ArmOp *op) {
 	ut32 data = UT32_MAX;
 	int k = 0;
@@ -418,7 +461,7 @@ static ut32 branch(ArmOp *op, ut64 addr, int k) {
 	if (op->operands[0].type & ARM_CONSTANT) {
 		n = op->operands[0].immediate;
 		if (!(n & 0x3 || n > 0x7ffffff)) {
-			if (n > addr) {
+			if (n >= addr) {
 				n -= addr;
 			} else {
 				n -= addr;
@@ -501,17 +544,10 @@ static ut32 mem_barrier (ArmOp *op, ut64 addr, int k) {
 
 #include "armass64_const.h"
 
-static ut32 msrk(ut16 v) {
-	ut32 r = 0;
-	ut32 a = ((v >> 12) & 0xf) << 1;
-	ut32 b = ((v & 0xfff) >> 3) & 0xff;
-	r |= a << 8;
-	r |= b << 16;
-	return r;
-}
-
 static ut32 msr(ArmOp *op, int w) {
 	ut32 data = UT32_MAX;
+	ut32 seq_data = UT32_MAX;
+	int is_immediate = 0;
 	int i;
 	ut32 r, b;
 	/* handle swapped args */
@@ -530,7 +566,7 @@ static ut32 msr(ArmOp *op, int w) {
 			}
 		}
 		r = op->operands[0].reg;
-		b = msrk (op->operands[0].sp_val);
+		b = op->operands[1].sp_val;
 	} else {
 		if (op->operands[0].reg_type != (ARM_REG64 | ARM_SP)) {
 			if (op->operands[0].type == ARM_CONSTANT) {
@@ -545,19 +581,47 @@ static ut32 msr(ArmOp *op, int w) {
 				return data;
 			}
 		}
-		r = op->operands[0].reg;
-		b = msrk (op->operands[0].sp_val);
+		r = op->operands[1].reg;
+		if ( op->operands[1].sp_val == 0xfffe ) {
+			is_immediate = 1;
+			r = op->operands[1].immediate;
+		}
+		b = op->operands[0].sp_val;
 	}
-	data = (r << 24) | b | 0xd5;
-	if (w) {
-		/* mrs */
-		data |= 0x413000;
+	data = 0x00000000;
+
+	if (is_immediate) {
+		//only msr has immediate mode
+		data = 0xd500401f;
+		if (b == 0xc210) { //op0 is SPSel
+			b = 0x05; //set to immediate mode encoding
+		}
+
+		data |= (b & 0xf0) << 12; //op1
+		data |= (b & 0x0f) << 5; //op2
+		data |= (r & 0xf) << 8; //CRm(#imm)
+
+	} else {
+		if (w) {
+			/* mrs */
+			data |= 0xd5200000;
+		} else {
+			data |= 0xd5000000;
+		}
+		data |= b << 5;
+		data |= r;
 	}
-	if (op->operands[1].reg_type == ARM_REG64) {
+	seq_data = 0x00000000;
+	seq_data |= (data & 0xff) << 8*3;
+	seq_data |= (data & 0xff00) << 8*1;
+	seq_data |= (data & 0xff0000) >> 8*1;
+	seq_data |= (data & 0xff000000) >> 8*3;
+/*
+if (op->operands[1].reg_type == ARM_REG64) {
 		data |= op->operands[1].reg << 24;
 	}
-
-	return data;
+*/
+	return seq_data;
 }
 
 static ut32 orr(ArmOp *op, int addr) {
@@ -715,7 +779,7 @@ static ut32 arithmetic (ArmOp *op, int k) {
 	data += op->operands[0].reg << 24;
 	data += (op->operands[1].reg & 7) << (24 + 5);
 	data += (op->operands[1].reg >> 3) << 16;
-	if (op->operands[2].reg_type & ARM_REG64) {
+	if (op->operands[2].type & ARM_GPR) {
 		data += op->operands[2].reg << 8;
 	} else {
 		data += (op->operands[2].reg & 0x3f) << 18;
@@ -724,6 +788,17 @@ static ut32 arithmetic (ArmOp *op, int k) {
 	return data;
 }
 
+static ut32 neg(ArmOp *op) {
+	if (op->operands_count < 2) {
+		return -1;
+	}
+	op->operands_count++;
+	op->operands[2] = op->operands[1];
+	op->operands[1].reg = 31; // xzr
+
+	return arithmetic (op, 0xd1); // sub reg0, xzr, reg1
+}
+
 static bool parseOperands(char* str, ArmOp *op) {
 	char *t = strdup (str);
 	int operand = 0;
@@ -731,6 +806,7 @@ static bool parseOperands(char* str, ArmOp *op) {
 	char *x;
 	int imm_count = 0;
 	int mem_opt = 0;
+	int msr_op_index = 0;
 	if (!token) {
 		return false;
 	}
@@ -738,7 +814,9 @@ static bool parseOperands(char* str, ArmOp *op) {
 	while (token) {
 		char *next = strchr (token, ',');
 		if (next) {
-			*next++ = 0;
+			// Change the ',' in token to null byte
+			// essentialy split the token by commas
+			*next++ = '\0';
 		}
 		while (token[0] == ' ') {
 			token++;
@@ -751,6 +829,38 @@ static bool parseOperands(char* str, ArmOp *op) {
 		op->operands[operand].reg_type = ARM_UNDEFINED;
 		op->operands[operand].shift = ARM_NO_SHIFT;
 
+		//parse MSR (immediate) operand 1
+		if (strcmp (op->mnemonic, "msr") == 0 && operand == 1) {
+
+			//operand 1 must be a immediate
+			if ( token[0] == '#' || (token[0] >= '0' && token[0] <= '9')) {
+				//immediate operand found.
+				op->operands[operand].sp_val = 0xfffe; //not regiter, but a immediate
+				op->operands[operand].immediate = r_num_math (NULL, token[0]=='#'?token+1:token);
+				operand ++;
+				token = next;
+				continue;
+			}
+		}
+
+		//parse system registers
+		if ((strcmp (op->mnemonic, "mrs") == 0 && operand == 1) || (strcmp (op->mnemonic, "msr") == 0 && operand == 0)) {
+			for(msr_op_index = 0; msr_const[msr_op_index].name; msr_op_index++) {
+				if (strcasecmp (token, msr_const[msr_op_index].name) == 0) {
+					op->operands_count ++;
+					op->operands[operand].type = ARM_CONSTANT;
+					op->operands[operand].immediate = msr_const[msr_op_index].val;
+					imm_count++;
+					break;
+				}
+			}
+			if (msr_const[msr_op_index].name) {
+				operand ++;
+				token = next;
+				continue;
+			}
+		}
+
 		while (token[0] == ' ' || token[0] == '[' || token[0] == ']') {
 			token ++;
 		}
@@ -783,7 +893,14 @@ static bool parseOperands(char* str, ArmOp *op) {
 			op->operands_count ++;
 			op->operands[operand].type = ARM_GPR;
 			op->operands[operand].reg_type = ARM_REG64;
-			op->operands[operand].reg = r_num_math (NULL, token + 1);
+
+			if (!strncmp(token + 1, "zr", 2)) {
+				// XZR
+				op->operands[operand].reg = 31;
+			} else {
+				op->operands[operand].reg = r_num_math (NULL, token + 1);
+			}
+
 			if (op->operands[operand].reg > 31) {
 				free (t);
 				return false;
@@ -793,7 +910,17 @@ static bool parseOperands(char* str, ArmOp *op) {
 			op->operands_count ++;
 			op->operands[operand].type = ARM_GPR;
 			op->operands[operand].reg_type = ARM_REG32;
-			op->operands[operand].reg = r_num_math (NULL, token + 1);
+
+			if (!strncmp(token + 1, "zr", 2)) {
+				// WZR
+				op->operands[operand].reg = 31;
+			} else if (!strncmp(token + 1, "sp", 2)) {
+				// WSP
+				op->operands[operand].reg = 31;
+			} else {
+				op->operands[operand].reg = r_num_math (NULL, token + 1);
+			}
+
 			if (op->operands[operand].reg > 31) {
 				free (t);
 				return false;
@@ -844,6 +971,15 @@ static bool parseOperands(char* str, ArmOp *op) {
 				op->operands[operand].mem_option = mem_opt;
 			}
 			break;
+		case '#':
+			if (token[1] == '-') {
+				op->operands[operand].sign = -1;
+			}
+			op->operands_count ++;
+			op->operands[operand].type = ARM_CONSTANT;
+			op->operands[operand].immediate = r_num_math (NULL, token + 1);
+			imm_count++;
+			break;
 		case '-':
 			op->operands[operand].sign = -1;
 			// falthru
@@ -880,6 +1016,46 @@ static bool parseOpcode(const char *str, ArmOp *op) {
 	return parseOperands (space, op);
 }
 
+static bool handlePAC(ut32 *op, const char *str) {
+	if (!strcmp (str, "autiasp")) {
+		*op = 0xbf2303d5;
+		return true;
+	}
+	if (!strcmp (str, "autiaz")) {
+		*op = 0x9f2303d5;
+		return true;
+	}
+	if (!strcmp (str, "autibsp")) {
+		*op = 0xff2303d5;
+		return true;
+	}
+	if (!strcmp (str, "autibz")) {
+		*op = 0xdf2303d5;
+		return true;
+	}
+	if (!strcmp (str, "paciaz")) {
+		*op = 0x1f2303d5;
+		return true;
+	}
+	if (!strcmp (str, "pacibz")) {
+		*op = 0x5f2303d5;
+		return true;
+	}
+	if (!strcmp (str, "paciasp")) {
+		*op = 0x3f2303d5;
+		return true;
+	}
+	if (!strcmp (str, "pacibsp")) {
+		*op = 0x7f2303d5;
+		return true;
+	}
+	if (!strcmp (str, "retab")) {
+		*op = 0xff0f5fd6;
+		return true;
+	}
+	return false;
+}
+
 bool arm64ass(const char *str, ut64 addr, ut32 *op) {
 	ArmOp ops = {0};
 	if (!parseOpcode (str, &ops)) {
@@ -890,6 +1066,10 @@ bool arm64ass(const char *str, ut64 addr, ut32 *op) {
 		*op = mov (&ops);
 		return *op != -1;
 	}
+	if (!strncmp (str, "cb", 2)) {
+		*op = cb (&ops);
+		return *op != -1;
+	}
 	if (!strncmp (str, "cmp", 3)) {
 		*op = cmp (&ops);
 		return *op != -1;
@@ -950,10 +1130,14 @@ bool arm64ass(const char *str, ut64 addr, ut32 *op) {
 		*op = arithmetic (&ops, 0xd1);
 		return *op != -1;
 	}
-	if (!strncmp (str, "add", 3)) { // w
+	if (!strncmp (str, "add x", 5)) {
 		*op = arithmetic (&ops, 0x91);
 		return *op != -1;
 	}
+	if (!strncmp (str, "add w", 5)) {
+		*op = arithmetic (&ops, 0x11);
+		return *op != -1;
+	}
 	if (!strncmp (str, "adr x", 5)) { // w
 		*op = adr (&ops, addr);
 		return *op != -1;
@@ -962,10 +1146,18 @@ bool arm64ass(const char *str, ut64 addr, ut32 *op) {
 		*op = adrp (&ops, addr, 0x00000090);
 		return *op != -1;
 	}
+	if (!strncmp (str, "neg", 3)) {
+		*op = neg (&ops);
+		return *op != -1;
+	}
 	if (!strcmp (str, "isb")) {
 		*op = 0xdf3f03d5;
 		return *op != -1;
 	}
+	// PAC
+	if (handlePAC (op, str)) {
+		return true;
+	}
 	if (!strcmp (str, "nop")) {
 		*op = 0x1f2003d5;
 		return *op != -1;
