@@ -18,7 +18,8 @@
 
    You should have received a copy of the GNU General Public License
    along with GAS or GDB; see the file COPYING.	If not, write to
-   the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
+*/
 
 #ifndef OPCODES_HEXAGON_H
 #define OPCODES_HEXAGON_H 1
