@@ -3,6 +3,25 @@
 
 #include "msp430_disas.h"
 
+static const char *msp430_register_names[] = {
+	"pc",
+	"sp",
+	"sr",
+	"cg",
+	"r4",
+	"r5",
+	"r6",
+	"r7",
+	"r8",
+	"r9",
+	"r10",
+	"r11",
+	"r12",
+	"r13",
+	"r14",
+	"r15",
+};
+
 static const char *two_op_instrs[] = {
 	[MSP430_MOV]	= "mov",
 	[MSP430_ADD]	= "add",
@@ -77,12 +96,15 @@ static void remove_first_operand(struct msp430_cmd *cmd)
 
 static void remove_second_operand(struct msp430_cmd *cmd)
 {
-	if (strchr(cmd->operands, ','))
-		*strchr(cmd->operands, ',') = '\0';
+	if (strchr (cmd->operands, ',')) {
+		{
+			*strchr (cmd->operands, ',') = '\0';
+		}
+	}
 }
 
 /* TODO: This is ugly as hell */
-static int decode_emulation(ut16 instr, ut16 op1, struct msp430_cmd *cmd)
+static int decode_emulation(ut16 instr, struct msp430_cmd *cmd)
 {
 	int ret = -1;
 	ut8 as, ad, src, dst, bw, opcode;
@@ -95,79 +117,79 @@ static int decode_emulation(ut16 instr, ut16 op1, struct msp430_cmd *cmd)
 	opcode = get_twoop_opcode(instr);
 
 	if (opcode == MSP430_ADDC && as == 0 && src == MSP430_R3) {
-		snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "%s", bw ? "adc.b" : "adc");
-		snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1, "r%d", dst);
+		snprintf(cmd->instr, sizeof (cmd->instr), "%s", bw ? "adc.b" : "adc");
+		snprintf(cmd->operands, sizeof (cmd->operands), "%s", msp430_register_names[dst]);
 	} else if (opcode == MSP430_MOV && as == 0 && src == MSP430_R3) {
 		if (ad == 0 && dst == MSP430_R3) {
-			snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "nop");
+			snprintf(cmd->instr, sizeof (cmd->instr), "nop");
 			cmd->operands[0] = '\0';
 		} else {
-			snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "%s", bw ? "clr.b" : "clr");
+			snprintf(cmd->instr, sizeof (cmd->instr), "%s", bw ? "clr.b" : "clr");
 			remove_first_operand(cmd);
 		}
 	} else if (opcode == MSP430_MOV && as == 3 && src == MSP430_SP) {
 		if (dst == MSP430_PC) {
-			snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "ret");
+			snprintf(cmd->instr, sizeof (cmd->instr), "ret");
        			cmd->type = MSP430_ONEOP;
 			cmd->opcode = MSP430_RETI;
 			cmd->operands[0] = '\0';
 		} else {
-			snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "%s", bw ? "pop.b" : "pop");
+			snprintf(cmd->instr, sizeof (cmd->instr), "%s", bw ? "pop.b" : "pop");
 			remove_first_operand(cmd);
 		}
 	} else if (opcode == MSP430_MOV && ad == 0 && dst == MSP430_PC) {
-		snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "%s", "br");
+		snprintf(cmd->instr, sizeof (cmd->instr), "%s", "br");
 		remove_second_operand(cmd);
 	} else if (opcode == MSP430_BIC && as == 2 && src == MSP430_SR && dst == MSP430_SR && ad == 0) {
-		snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "%s", "clrn");
+		snprintf(cmd->instr, sizeof (cmd->instr), "%s", "clrn");
 		cmd->operands[0] = '\0';
 	} else if (opcode == MSP430_BIC && as == 2 && src == MSP430_R3 && dst == MSP430_SR && ad == 0) {
-		snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "%s", "clrz");
+		snprintf(cmd->instr, sizeof (cmd->instr), "%s", "clrz");
 		cmd->operands[0] = '\0';
 	} else if (opcode == MSP430_BIC && as == 3 && src == MSP430_SR && dst == MSP430_SR && ad == 0) {
-		snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "%s", "dint");
+		snprintf(cmd->instr, sizeof (cmd->instr), "%s", "dint");
 		cmd->operands[0] = '\0';
 	} else if (opcode == MSP430_BIS && as == 3 && src == MSP430_SR && dst == MSP430_SR && ad == 0) {
-		snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "%s", "eint");
+		snprintf(cmd->instr, sizeof (cmd->instr), "%s", "eint");
 		cmd->operands[0] = '\0';
 	} else if (opcode == MSP430_DADD && as == 0 && src == MSP430_R3) {
-		snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "%s", bw ? "dadc.b" : "dadc");
+		snprintf(cmd->instr, sizeof (cmd->instr), "%s", bw ? "dadc.b" : "dadc");
 		remove_first_operand(cmd);
 	} else if (opcode == MSP430_SUB && as == 1 && src == MSP430_R3) {
-		snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "%s", bw ? "dec.b" : "dec");
+		snprintf(cmd->instr, sizeof (cmd->instr), "%s", bw ? "dec.b" : "dec");
 		remove_first_operand(cmd);
 	} else if (opcode == MSP430_SUB && as == 2 && src == MSP430_R3) {
-		snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "%s", bw ? "decd.b" : "decd");
+		snprintf(cmd->instr, sizeof (cmd->instr), "%s", bw ? "decd.b" : "decd");
 		remove_first_operand(cmd);
 	} else if (opcode == MSP430_ADD && as == 1 && src == MSP430_R3) {
-		snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "%s", bw ? "inc.b" : "inc");
+		snprintf(cmd->instr, sizeof (cmd->instr), "%s", bw ? "inc.b" : "inc");
 		remove_first_operand(cmd);
 	} else if (opcode == MSP430_ADD && as == 2 && src == MSP430_R3) {
-		snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "%s", bw ? "incd.b" : "incd");
+		snprintf(cmd->instr, sizeof (cmd->instr), "%s", bw ? "incd.b" : "incd");
 		remove_first_operand(cmd);
 	} else if (opcode == MSP430_XOR && as == 3 && src == MSP430_R3) { 
-		snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "%s", bw ? "inv.b" : "inv");
+		snprintf(cmd->instr, sizeof (cmd->instr), "%s", bw ? "inv.b" : "inv");
 		remove_first_operand(cmd);
 	} else if (opcode == MSP430_ADD && src == dst) {
-		snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "%s", bw ? "rla.b" : "rla");
+		snprintf(cmd->instr, sizeof (cmd->instr), "%s", bw ? "rla.b" : "rla");
 		remove_second_operand(cmd);
 	} else if (opcode == MSP430_ADDC && src == dst) {
-		snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "%s", bw ? "rlc.b" : "rlc");
+		snprintf(cmd->instr, sizeof (cmd->instr), "%s", bw ? "rlc.b" : "rlc");
 		remove_second_operand(cmd);
 	} else if (opcode == MSP430_SUBC && as == 0 && src == MSP430_R3) {
-		snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "%s", bw ? "sbc.b" : "sbc");
+		snprintf(cmd->instr, sizeof (cmd->instr), "%s", bw ? "sbc.b" : "sbc");
 		remove_first_operand(cmd);
 	} else if (opcode == MSP430_BIS && as == 1 && src == MSP430_R3 && dst == MSP430_SR && ad == 0) {
-		snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "setc");
+		snprintf(cmd->instr, sizeof (cmd->instr), "setc");
 		cmd->operands[0] = '\0';
 	} else if (opcode == MSP430_BIS && as == 2 && src == MSP430_SR && dst == MSP430_SR && ad == 0) {
-		snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "setn");
+		snprintf(cmd->instr, sizeof (cmd->instr), "setn");
 		cmd->operands[0] = '\0';
 	} else if (opcode == MSP430_BIS && as == 2 && src == MSP430_R3 && dst == MSP430_SR && ad == 0) {
-		snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "setz");
+		snprintf(cmd->instr, sizeof (cmd->instr), "setz");
 		cmd->operands[0] = '\0';
 	} else if (opcode == MSP430_CMP && as == 0 && src == MSP430_R3) {
-		snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "%s", bw ? "tst.b" : "tst");
+		snprintf(cmd->instr, sizeof (cmd->instr), "%s", bw ? "tst.b" : "tst");
 		remove_first_operand(cmd);
 	}
 
@@ -177,7 +199,7 @@ static int decode_emulation(ut16 instr, ut16 op1, struct msp430_cmd *cmd)
 /* return #byte of instruction */
 static int decode_addressing_mode(ut16 instr, ut16 op1, ut16 op2, struct msp430_cmd *cmd)
 {
-	int ret, cg;
+	int ret = 0, srcOperInCodeWord = 0;
 	ut8 as, ad, src, dst;
 	ut16 op;
 	char dstbuf[16];
@@ -188,18 +210,16 @@ static int decode_addressing_mode(ut16 instr, ut16 op1, ut16 op2, struct msp430_
 	ad = get_ad(instr);
 	src = get_src(instr);
 	dst = get_dst(instr);
-	cg = 0;
 
 	/* addressing mode of source operand */
 	switch (as) {
 	case 0:
 		switch (src) {
 		case MSP430_R3: /* CG2 */
-			snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1, "#0");
-			cg = 1;
+			snprintf(cmd->operands, sizeof (cmd->operands), "#0");
 			break;
 		default: /* register mode */
-			snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1, "r%d", src);
+			snprintf(cmd->operands, sizeof (cmd->operands), "%s", msp430_register_names[src]);
 		}
 		ret = 2;
 		break;
@@ -207,32 +227,32 @@ static int decode_addressing_mode(ut16 instr, ut16 op1, ut16 op2, struct msp430_
 		ret = 4;
 		switch (src) {
 		case MSP430_PC: /* symbolic mode */
-			snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1, "0x%04x", op1);
+			snprintf(cmd->operands, sizeof (cmd->operands), "0x%04x", op1);
+			srcOperInCodeWord = 1;
 			break;
 		case MSP430_R3: /* CG2 */
-			snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1, "%s", "#1");
-			cg = 1;
+			snprintf(cmd->operands, sizeof (cmd->operands), "%s", "#1");
 			ret = 2;
 			break;
 		case MSP430_SR: /* absolute mode */
-			snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1, "&0x%04x", op1);
+			snprintf(cmd->operands, sizeof (cmd->operands), "&0x%04x", op1);
+			srcOperInCodeWord = 1;
 			break;
 		default: /* indexed mode */
-			snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1, "0x%x(r%d)", op1, src);
+			snprintf(cmd->operands, sizeof (cmd->operands), "0x%x(%s)", op1, msp430_register_names[src]);
+			srcOperInCodeWord = 1;
 		}
 		break;
 	case 2:
 		switch (src) {
 		case MSP430_SR: /* CG1 */
-			snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1, "#4");
-			cg = 1;
+			snprintf(cmd->operands, sizeof (cmd->operands), "#4");
 			break;
 		case MSP430_R3: /* CG2 */
-			snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1, "#2");
-			cg = 1;
+			snprintf(cmd->operands, sizeof (cmd->operands), "#2");
 			break;
 		default: /* indirect register mode */
-			snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1, "@r%d", src);
+			snprintf(cmd->operands, sizeof (cmd->operands), "@%s", msp430_register_names[src]);
 		}
 		ret = 2;
 		break;
@@ -240,36 +260,30 @@ static int decode_addressing_mode(ut16 instr, ut16 op1, ut16 op2, struct msp430_
 		ret = 2;
 		switch (src) {
 		case MSP430_SR: /* CG1 */
-			snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1, "#8");
-			cg = 1;
+			snprintf(cmd->operands, sizeof (cmd->operands), "#8");
 			break;
 		case MSP430_R3: /* CG2 */
-			snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1, "#-1");
-			cg = 1;
+			snprintf(cmd->operands, sizeof (cmd->operands), "#-1");
 			break;
 		case MSP430_PC: /* immediate mode */
-			snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1, "#0x%04x", op1);
+			snprintf(cmd->operands, sizeof (cmd->operands), "#0x%04x", op1);
+			srcOperInCodeWord = 1;
 			ret = 4;
 			break;
 		default: /* indirect autoincrement mode */
-			snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1, "@r%d+", src);
+			snprintf(cmd->operands, sizeof (cmd->operands), "@%s+", msp430_register_names[src]);
 		}
 		break;
-	default:
-		ret = -1;
 	}
 
-	if (ret < 0)
-		return ret;
-
 	/* addressing mode of destination operand */
 	switch (ad) {
 	case 0: /* register mode */
-		snprintf(dstbuf, 15, ", r%d", dst); 
+		snprintf(dstbuf, sizeof (dstbuf), ", %s", msp430_register_names[dst]); 
 		break;
 	case 1:
 		/* check addr. mode of source operand */
-		if ((as == 1 || as == 3) && cg == 0) {
+		if (srcOperInCodeWord != 0) {
 		    op = op2;
 		    ret = 6;
 		} else {
@@ -278,21 +292,19 @@ static int decode_addressing_mode(ut16 instr, ut16 op1, ut16 op2, struct msp430_
 		}
 		switch (get_dst(instr)) {
 		case MSP430_PC: /* symbolic mode */
-			snprintf(dstbuf, 15, ", 0x%04x", op);
+			snprintf(dstbuf, sizeof (dstbuf), ", 0x%04x", op);
 			break;
 		case MSP430_SR: /* absolute mode */
-		    	snprintf(dstbuf, 15, ", &0x%04x", op);
+		    	snprintf(dstbuf, sizeof (dstbuf), ", &0x%04x", op);
 			break;
 		default: /* indexed mode */
-			snprintf(dstbuf, 15, ", 0x%x(r%d)", op, dst);
+			snprintf(dstbuf, sizeof (dstbuf), ", 0x%x(%s)", op, msp430_register_names[dst]);
 		}
 		break;
-	default:
-		ret = -1;
 	}
 
-	strncat(cmd->operands, dstbuf, MSP430_INSTR_MAXLEN - 1 - strlen(cmd->operands));
-	decode_emulation(instr, op1, cmd);
+	strncat(cmd->operands, dstbuf, sizeof (cmd->operands) - 1 - strlen(cmd->operands));
+	decode_emulation(instr, cmd);
 	return ret;
 }
 
@@ -300,12 +312,12 @@ static int decode_twoop_opcode(ut16 instr, ut16 op1, ut16 op2, struct msp430_cmd
 {
 	ut8 opcode = get_twoop_opcode(instr);
 
-	snprintf (cmd->instr, MSP430_INSTR_MAXLEN - 1, "%s", two_op_instrs[opcode]);
+	snprintf (cmd->instr, sizeof (cmd->instr), "%s", two_op_instrs[opcode]);
 	if (get_bw (instr)) {
-		strncat (cmd->instr, ".b", MSP430_INSTR_MAXLEN - 1 - strlen(cmd->instr));
+		strncat (cmd->instr, ".b", sizeof (cmd->instr) - 1 - strlen(cmd->instr));
 	}
 
-	cmd->opcode = get_twoop_opcode (instr);
+	cmd->opcode = opcode;
 	return decode_addressing_mode (instr, op1, op2, cmd);
 }
 
@@ -318,26 +330,21 @@ static ut8 get_jmp_cond(ut16 instr)
 	return (instr >> 10 ) & 7;
 }
 
-static int decode_jmp(ut16 instr, struct msp430_cmd *cmd)
+static void decode_jmp(ut16 instr, struct msp430_cmd *cmd)
 {
 	ut16 addr;
-	if (get_jmp_opcode(instr) != MSP430_JMP_OPC)
-		return -1;
 
-	snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "%s", jmp_instrs[get_jmp_cond(instr)]);
+	snprintf(cmd->instr, sizeof (cmd->instr), "%s", jmp_instrs[get_jmp_cond(instr)]);
 
 	addr = instr & 0x3FF;
 
 	cmd->jmp_addr = addr >= 0x300 ? (st16)((0xFE00 | addr) * 2 + 2) : (addr & 0x1FF) * 2 + 2;
-	snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1,
-			"$%c0x%04x", addr >= 0x300 ? '-' : '+',
+	snprintf(cmd->operands, sizeof (cmd->operands), "$%c0x%04x", addr >= 0x300 ? '-' : '+',
 			addr >= 0x300 ? 0x400 - ((addr & 0x1FF) * 2 + 2) : (addr & 0x1FF) * 2 + 2);
 
 	cmd->jmp_cond = get_jmp_cond(instr);
 	cmd->opcode = get_jmp_opcode(instr);
 	cmd->type = MSP430_JUMP;
-
-	return 2;
 }
 
 
@@ -351,15 +358,11 @@ static int decode_oneop_opcode(ut16 instr, ut16 op, struct msp430_cmd *cmd)
 	int ret = 2;
 	ut8 as, opcode;
 
-	if ((instr >> 10) != 4)
-		return -1;
-
 	opcode = get_oneop_opcode(instr);
 
 	as = get_as(instr);
 
-	snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "%s",
-			one_op_instrs[opcode]);
+	snprintf(cmd->instr, sizeof (cmd->instr), "%s", one_op_instrs[opcode]);
 
 	cmd->opcode = get_oneop_opcode(instr);
 
@@ -374,11 +377,11 @@ static int decode_oneop_opcode(ut16 instr, ut16 op, struct msp430_cmd *cmd)
 		case 0:
 			switch (get_dst(instr)) {
 			case MSP430_R3:
-				snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1, "#0");
+				snprintf(cmd->operands, sizeof (cmd->operands), "#0");
 				break;
 			default:
-				snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1,
-						"r%d", get_dst(instr));
+				snprintf(cmd->operands, sizeof (cmd->operands),
+						"%s", msp430_register_names[get_dst(instr)]);
 			}
 			ret = 2;
 			break;
@@ -387,42 +390,39 @@ static int decode_oneop_opcode(ut16 instr, ut16 op, struct msp430_cmd *cmd)
 			ret = 4;
 			switch (get_dst(instr)) {
 			case MSP430_R3:
-				snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1, "#1");
+				snprintf(cmd->operands, sizeof (cmd->operands), "#1");
 				/* this is an unusual encoding in that there's no index word */
 				ret = 2;
 				break;
 			case MSP430_PC:
-				snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1,
-						"0x%04x", op);
+				snprintf(cmd->operands, sizeof (cmd->operands), "0x%04x", op);
 				break;
 			case MSP430_SR:
-				snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1,
-						"&0x%04x", op);
+				snprintf(cmd->operands, sizeof (cmd->operands), "&0x%04x", op);
 				break;
 			default:
-				snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1,
-						"0x%x(r%d)", op, get_dst(instr));
+				snprintf(cmd->operands, sizeof (cmd->operands),
+						"0x%x(%s)", op, msp430_register_names[get_dst(instr)]);
 			}
 
 			break;
 		case 2:
 			switch (get_dst(instr)) {
 			case MSP430_SR:
-				snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1, "#4");
+				snprintf(cmd->operands, sizeof (cmd->operands), "#4");
 				break;
 			case MSP430_R3:
-				snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1, "#2");
+				snprintf(cmd->operands, sizeof (cmd->operands), "#2");
 				break;
 			default:
-				snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1,
-						"@r%d", get_dst(instr));
+				snprintf(cmd->operands, sizeof (cmd->operands),
+						"@%s", msp430_register_names[get_dst(instr)]);
 			}
 
 			ret = 2;
 			break;
 		case 3:
-			snprintf(cmd->operands, MSP430_INSTR_MAXLEN - 1,
-					"#0x%04x", op);
+			snprintf(cmd->operands, sizeof (cmd->operands), "#0x%04x", op);
 			ret = 4;
 			break;
 		default:
@@ -439,9 +439,16 @@ static int decode_oneop_opcode(ut16 instr, ut16 op, struct msp430_cmd *cmd)
 	return ret;
 }
 
+enum {
+	MSP430_TWOOP_OPCODE_INVALID = 0,
+	MSP430_TWOOP_OPCODE_SINGLEOP,
+	MSP430_TWOOP_OPCODE_JUMP2,
+	MSP430_TWOOP_OPCODE_JUMP3,
+};
+
 int msp430_decode_command(const ut8 *in, int len, struct msp430_cmd *cmd) {
 	int ret = -1;
-	ut16 operand1, operand2;
+	ut16 operand1 = 0, operand2 = 0;
 	if (len < 2) {
 		return -1;
 	}
@@ -449,49 +456,44 @@ int msp430_decode_command(const ut8 *in, int len, struct msp430_cmd *cmd) {
 	ut8 opcode = get_twoop_opcode (instr);
 
 	switch (opcode) {
-	case MSP430_MOV:
-	case MSP430_ADD:
-	case MSP430_ADDC:
-	case MSP430_SUBC:
-	case MSP430_SUB:
-	case MSP430_CMP:
-	case MSP430_DADD:
-	case MSP430_BIT:
-	case MSP430_BIC:
-	case MSP430_BIS:
-	case MSP430_XOR:
-	case MSP430_AND:
-		// XXX this conditional is wrong, but seems to be safe
+	case MSP430_TWOOP_OPCODE_INVALID:
+		// Invalid opcode.
+		break;
+	case MSP430_TWOOP_OPCODE_SINGLEOP:
+		// Single operand instructions or invalid opcode.
+		if ((instr & 0x0f80) <= 0x0300) {
+			// Single operand instructions.
+			if (len >= 4) {
+				operand1 = r_read_at_le16 (in, 2);
+			}
+			ret = decode_oneop_opcode(instr, operand1, cmd);
+		}
+		break;
+	case MSP430_TWOOP_OPCODE_JUMP2:
+	case MSP430_TWOOP_OPCODE_JUMP3:
+		// Jumps.
+		decode_jmp (instr, cmd);
+		ret = 2;
+		break;
+	default:
+		// Double operand instructions.
+		cmd->type = MSP430_TWOOP;
 		if (len >= 4) {
-			cmd->type = MSP430_TWOOP;
 			operand1 = r_read_at_le16 (in, 2);
-			operand2 = r_read_at_le16 (in, 4);
-		} else {
-			operand1 = 0;
-			operand2 = 0;
+			if (len >= 6) {
+				operand2 = r_read_at_le16 (in, 4);
+			}
 		}
 		ret = decode_twoop_opcode (instr, operand1, operand2, cmd);
 		break;
 	}
 
-	if (ret > 0) {
-		return ret;
-	}
-
-	ret = decode_jmp (instr, cmd);
-
-	if (ret > 0) {
-		return ret;
-	}
-
-	operand1 = r_read_at_le16 (in, 2);
-	ret = decode_oneop_opcode(instr, operand1, cmd);
-
 	/* if ret < 0, it's an invalid opcode.Say so and return 2 since
 	 * all MSP430 opcodes are of 16 bits,valid or invalid */
 	if (ret < 0) {
 		cmd->type = MSP430_INV;
-		snprintf(cmd->instr, MSP430_INSTR_MAXLEN - 1, "invalid opcode");
+		snprintf(cmd->instr, sizeof (cmd->instr), "invalid");
+		cmd->operands[0] = '\0';
 		ret = 2;
 	}
 
