@@ -50,8 +50,9 @@ ut32 get_ins_len(ut8 opcode) {
 ut32 get_ins_part(ut32 pos, ut32 len) {
 	ut32 ret = 0;
 	has_failed = 0;
-	if (C55PLUS_DEBUG)
-        	printf("pos => 0x%x len => %d ins_buff_len => %d\n", pos, len, ins_buff_len);
+	if (C55PLUS_DEBUG) {
+		printf ("pos => 0x%x len => %d ins_buff_len => %d\n", pos, len, ins_buff_len);
+	}
 
 	if ((st32)pos < 0 || pos >= ins_buff_len) {
 		has_failed = 1;
@@ -60,9 +61,11 @@ ut32 get_ins_part(ut32 pos, ut32 len) {
 
 	for (; len > 0; --len) {
 		ret <<= 8;
-		if (pos >= ins_buff_len) 
+		if (pos >= ins_buff_len) {
 			has_failed = 1;
-		else ret |= ins_buff[pos++];
+		} else {
+			ret |= ins_buff[pos++];
+		}
 	}
 
 	return ret;
