@@ -48,7 +48,7 @@ rd_number (const char **p, const char **endp, int base)
 	     "(string=%s).\n", stack[sp].line, addr, base, *p);
   num[base] = '\0';
   *p = delspc (*p);
-  while (**p && (c = strchr (num, tolower (**p))))
+  while (**p && (c = strchr (num, tolower ((const unsigned char)**p))))
     {
       i = c - num;
       if (verbose >= 7)
@@ -83,7 +83,7 @@ rd_otherbasenumber (const char **p, int *valid, int print_errors)
 	printerr (1, "unexpected end of line after `@'\n");
       return 0;
     }
-  if (**p == '0' || !isalnum (**p))
+  if (**p == '0' || !isalnum ((const unsigned char)**p))
     {
       if (valid)
 	*valid = 0;
@@ -93,8 +93,8 @@ rd_otherbasenumber (const char **p, int *valid, int print_errors)
     }
   c = **p;
   (*p)++;
-  if (isalpha (**p))
-    return rd_number (p, NULL, tolower (c) - 'a' + 1);
+  if (isalpha ((const unsigned char)**p))
+    return rd_number (p, NULL, tolower ((unsigned char)c) - 'a' + 1);
   return rd_number (p, NULL, c - '0' + 1);
 }
 
@@ -189,7 +189,7 @@ check_label (struct label *labels, const char **p, struct label **ret,
   const char *c;
   unsigned s2;
   *p = delspc (*p);
-  for (c = *p; isalnum (*c) || *c == '_' || *c == '.'; ++c)
+  for (c = *p; isalnum ((const unsigned char)*c) || *c == '_' || *c == '.'; ++c)
     {
     }
   s2 = c - *p;
@@ -262,7 +262,7 @@ rd_label (const char **p, int *exists, struct label **previous, int level,
       /* not yet found */
       const char *old_p = *p;
 	  /* label does not exist, or is invalid.  This is an error if there
-	   * is no existance check.  */
+	   * is no existence check.  */
 	  if (!exists && print_errors)
 	    printerr (1, "using undefined label %.*s\n", *p - old_p, old_p);
 	  /* Return a value to discriminate between non-existing and invalid */
@@ -461,7 +461,12 @@ rd_factor (const char **p, int *valid, int level, int *check, int print_errors)
       else if (**p == '/')
 	{
 	  (*p)++;
-	  result /= rd_value (p, valid, level, check, print_errors);
+      int value = rd_value (p, valid, level, check, print_errors);
+      if (value == 0){ 
+        printerr (1, "division by zero\n");
+        return -1;
+      }
+      result /= value;
 	}
       *p = delspc (*p);
     }
