@@ -1,16 +1,17 @@
-/* radare - LGPL - Copyright 2009-2018 - pancake, nibble */
+/* radare - LGPL - Copyright 2009-2019 - pancake, nibble */
 
 #include <stdio.h>
 #include <r_core.h>
 #include <r_types.h>
 #include <r_util.h>
 #include <r_asm.h>
+#define USE_R2 1
 #include <spp/spp.h>
 #include <config.h>
 
 R_LIB_VERSION (r_asm);
 
-char *directives[] = {
+static char *directives[] = {
 	".include", ".error", ".warning",
 	".echo", ".if", ".ifeq", ".endif",
 	".else", ".set", ".get", NULL
@@ -18,6 +19,20 @@ char *directives[] = {
 
 static RAsmPlugin *asm_static_plugins[] = { R_ASM_STATIC_PLUGINS };
 
+static void parseHeap(RParse *p, RStrBuf *s) {
+	char *op_buf_asm = r_strbuf_get (s);
+	size_t len = r_strbuf_length (s);
+	char *out = malloc (64 + (len * 2));
+	if (out) {
+		*out = 0;
+		strcpy (out , op_buf_asm);
+	// XXX we shouldn't pad here because we have t orefactor the RParse API to handle boundaries and chunks properly
+		r_parse_parse (p, op_buf_asm, out);
+		r_strbuf_set (s, out);
+		free (out);
+	}
+}
+
 /* pseudo.c - private api */
 static int r_asm_pseudo_align(RAsmCode *acode, RAsmOp *op, char *input) {
 	acode->code_align = r_num_math (NULL, input);
@@ -37,7 +52,6 @@ static int r_asm_pseudo_string(RAsmOp *op, char *input, int zero) {
 		input++;
 	}
 	len = r_str_unescape (input) + zero;
-	r_hex_bin2str ((ut8*)input, len, r_strbuf_get (&op->buf_hex));
 	r_strbuf_set (&op->buf, input); // uh?
 	return len;
 }
@@ -52,7 +66,7 @@ static inline int r_asm_pseudo_arch(RAsm *a, char *input) {
 
 static inline int r_asm_pseudo_bits(RAsm *a, char *input) {
 	if (!(r_asm_set_bits (a, r_num_math (NULL, input)))) {
-		eprintf ("Error: Unsupported bits value\n");
+		eprintf ("Error: Unsupported value for .bits.\n");
 		return -1;
 	}
 	return 0;
@@ -63,19 +77,6 @@ static inline int r_asm_pseudo_org(RAsm *a, char *input) {
 	return 0;
 }
 
-// wtf isnt this the same as r_asm_op_set_hex() ??
-static inline int r_asm_pseudo_hex(RAsmOp *op, char *input) {
-	// assume input fits inside op->buf
-	int len = r_hex_str2bin (input, (ut8*)r_strbuf_get (&op->buf));
-	r_asm_op_set_hex (op, r_str_trim_head_tail (input));
-	if (len < 0) {
-		len = -len;
-		len--;
-		eprintf ("Buffer truncated at %d because of the RAsmOp abuse.\nInput: %s\n", len, input);
-	}
-	return len;
-}
-
 static inline int r_asm_pseudo_intN(RAsm *a, RAsmOp *op, char *input, int n) {
 	short s;
 	int i;
@@ -87,6 +88,9 @@ static inline int r_asm_pseudo_intN(RAsm *a, RAsmOp *op, char *input, int n) {
 	}
 	// XXX honor endian here
 	ut8 *buf = (ut8*)r_strbuf_get (&op->buf);
+	if (!buf) {
+		return 0;
+	}
 	if (n == 2) {
 		s = (short)s64;
 		r_write_ble16 (buf, s, a->big_endian);
@@ -99,7 +103,6 @@ static inline int r_asm_pseudo_intN(RAsm *a, RAsmOp *op, char *input, int n) {
 	} else {
 		return 0;
 	}
-	r_hex_bin2str (buf, n, r_strbuf_get (&op->buf_hex));
 	return n;
 }
 
@@ -160,6 +163,10 @@ static inline int r_asm_pseudo_incbin(RAsmOp *op, char *input) {
 	int skip = (int)r_num_math (NULL, r_str_word_get0 (input, 1));
 	int count = (int)r_num_math (NULL,r_str_word_get0 (input, 2));
 	char *content = r_file_slurp (input, &bytes_read);
+	if (!content) {
+		eprintf ("Could not open '%s'.\n", input);
+		return -1;
+	}
 	if (skip > 0) {
 		skip = skip > bytes_read ? bytes_read : skip;
 	}
@@ -203,15 +210,15 @@ R_API RAsm *r_asm_new() {
 	return a;
 }
 
-R_API int r_asm_setup(RAsm *a, const char *arch, int bits, int big_endian) {
-	int ret = 0;
-	ret |= !r_asm_use (a, arch);
-	ret |= !r_asm_set_bits (a, bits);
-	return ret;
+R_API bool r_asm_setup(RAsm *a, const char *arch, int bits, int big_endian) {
+	r_return_val_if_fail (a && arch, false);
+	bool ret = !r_asm_use (a, arch);
+	return ret | !r_asm_set_bits (a, bits);
 }
 
 // TODO: spagueti
 R_API int r_asm_filter_input(RAsm *a, const char *f) {
+	r_return_val_if_fail (a && f, false);
 	if (!a->ifilter) {
 		a->ifilter = r_parse_new ();
 	}
@@ -235,23 +242,23 @@ R_API int r_asm_filter_output(RAsm *a, const char *f) {
 	return true;
 }
 
-R_API RAsm *r_asm_free(RAsm *a) {
-	if (a) {
-		if (a->cur && a->cur->fini) {
-			a->cur->fini (a->cur->user);
-		}
-		if (a->plugins) {
-			r_list_free (a->plugins);
-			a->plugins = NULL;
-		}
-		r_syscall_free (a->syscall);
-		free (a->cpu);
-		sdb_free (a->pair);
-		ht_pp_free (a->flags);
-		a->pair = NULL;
-		free (a);
+R_API void r_asm_free(RAsm *a) {
+	if (!a) {
+		return;
 	}
-	return NULL;
+	if (a->cur && a->cur->fini) {
+		a->cur->fini (a->cur->user);
+	}
+	if (a->plugins) {
+		r_list_free (a->plugins);
+		a->plugins = NULL;
+	}
+	r_syscall_free (a->syscall);
+	free (a->cpu);
+	sdb_free (a->pair);
+	ht_pp_free (a->flags);
+	a->pair = NULL;
+	free (a);
 }
 
 R_API void r_asm_set_user_ptr(RAsm *a, void *user) {
@@ -314,7 +321,7 @@ R_API bool r_asm_use_assembler(RAsm *a, const char *name) {
 }
 
 // TODO: this can be optimized using r_str_hash()
-R_API int r_asm_use(RAsm *a, const char *name) {
+R_API bool r_asm_use(RAsm *a, const char *name) {
 	RAsmPlugin *h;
 	RListIter *iter;
 	if (!a || !name) {
@@ -325,11 +332,13 @@ R_API int r_asm_use(RAsm *a, const char *name) {
 			if (!a->cur || (a->cur && strcmp (a->cur->arch, h->arch))) {
 				char *r2prefix = r_str_r2_prefix (R2_SDB_OPCODES);
 				char *file = r_str_newf ("%s/%s.sdb", r_str_get (r2prefix), h->arch);
-				r_asm_set_cpu (a, NULL);
-				sdb_free (a->pair);
-				a->pair = sdb_new (NULL, file, 0);
+				if (file) {
+					r_asm_set_cpu (a, NULL);
+					sdb_free (a->pair);
+					a->pair = sdb_new (NULL, file, 0);
+					free (file);
+				}
 				free (r2prefix);
-				free (file);
 			}
 			a->cur = h;
 			return true;
@@ -360,9 +369,7 @@ R_API int r_asm_set_bits(RAsm *a, int bits) {
 }
 
 R_API bool r_asm_set_big_endian(RAsm *a, bool b) {
-	if (!a || !a->cur) {
-		return false;
-	}
+	r_return_val_if_fail (a && a->cur, false);
 	a->big_endian = false; //little endian by default
 	switch (a->cur->endian) {
 	case R_SYS_ENDIAN_NONE:
@@ -402,20 +409,19 @@ R_API int r_asm_set_pc(RAsm *a, ut64 pc) {
 	return true;
 }
 
-static bool isInvalid (RAsmOp *op) {
+static bool __isInvalid (RAsmOp *op) {
 	const char *buf_asm = r_strbuf_get (&op->buf_asm);
 	return (buf_asm && *buf_asm && !strcmp (buf_asm, "invalid"));
 }
 
 R_API int r_asm_disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
-	int ret;
-	if (!a || !buf || !op) {
-		return -1;
-	}
+	r_asm_op_init (op);
+	r_return_val_if_fail (a && buf && op, -1);
 	if (len < 1) {
 		return 0;
 	}
-	ret = op->payload = 0;
+
+	int ret = op->payload = 0;
 	op->size = 4;
 	op->bitsize = 0;
 	r_asm_op_set_asm (op, "");
@@ -424,9 +430,6 @@ R_API int r_asm_disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 		if (mod) {
 			op->size = a->pcalign - mod;
 			r_strbuf_set (&op->buf_asm, "unaligned");
-			r_strbuf_set (&op->buf_hex, "");
-			// assume op->size after pcalign-mod will be small enough to not need checks here
-			r_hex_bin2str (buf, op->size, r_strbuf_get (&op->buf_hex));
 			return -1;
 		}
 	}
@@ -456,7 +459,7 @@ R_API int r_asm_disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 		}
 	}
 
-	if (op->size < 1 || isInvalid (op)) {
+	if (op->size < 1 || __isInvalid (op)) {
 		if (a->invhex) {
 			if (a->bits == 16) {
 				ut16 b = r_read_le16 (buf);
@@ -471,8 +474,7 @@ R_API int r_asm_disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 		}
 	}
 	if (a->ofilter) {
-		char *buf_asm = r_strbuf_get (&op->buf_asm);
-		r_parse_parse (a->ofilter, buf_asm, buf_asm);
+		parseHeap (a->ofilter, &op->buf_asm);
 	}
 	int opsz = (op->size > 0)? R_MAX (0, R_MIN (len, op->size)): 1;
 	r_asm_op_set_buf (op, buf, opsz);
@@ -566,6 +568,7 @@ R_API void r_asm_list_directives() {
 }
 
 R_API int r_asm_assemble(RAsm *a, RAsmOp *op, const char *buf) {
+	r_return_val_if_fail (a && op && buf, 0);
 	int ret = 0;
 	char *b = strdup (buf);
 	if (!b) {
@@ -596,13 +599,10 @@ R_API int r_asm_assemble(RAsm *a, RAsmOp *op, const char *buf) {
 	}
 	// XXX delete this block, the ase thing should be setting asm, buf and hex
 	if (op && ret > 0) {
-		op->size = ret; // XXX shouldnt be necessary
-		r_asm_op_set_asm (op, b); // XXX ase should be updating this already, isnt?
-		ut8 *ophex = (ut8*)r_strbuf_get (&op->buf_hex);
-		if (!*ophex) {
-			ut8 *opbuf = (ut8*)r_strbuf_get (&op->buf);
-			r_asm_op_set_buf (op, opbuf, ret);
-		}
+		op->size = ret; // XXX shouldn't be necessary
+		r_asm_op_set_asm (op, b); // XXX ase should be updating this already, isn't?
+		ut8 *opbuf = (ut8*)r_strbuf_get (&op->buf);
+		r_asm_op_set_buf (op, opbuf, ret);
 	}
 	free (b);
 	return ret;
@@ -610,6 +610,8 @@ R_API int r_asm_assemble(RAsm *a, RAsmOp *op, const char *buf) {
 
 // TODO: Use RStrBuf api here pls
 R_API RAsmCode* r_asm_mdisassemble(RAsm *a, const ut8 *buf, int len) {
+	r_return_val_if_fail (a && buf && len >= 0, NULL);
+
 	RStrBuf *buf_asm;
 	RAsmCode *acode;
 	ut64 pc = a->pc;
@@ -621,14 +623,10 @@ R_API RAsmCode* r_asm_mdisassemble(RAsm *a, const ut8 *buf, int len) {
 	if (!(acode = r_asm_code_new ())) {
 		return NULL;
 	}
-	if (!(acode->buf = malloc (1 + len))) {
+	if (!(acode->bytes = malloc (1 + len))) {
 		return r_asm_code_free (acode);
 	}
-	memcpy (acode->buf, buf, len);
-	if (!(acode->buf_hex = calloc (2, len + 1))) {
-		return r_asm_code_free (acode);
-	}
-	r_hex_bin2str (buf, len, acode->buf_hex);
+	memcpy (acode->bytes, buf, len);
 	if (!(buf_asm = r_strbuf_new (NULL))) {
 		return r_asm_code_free (acode);
 	}
@@ -639,86 +637,81 @@ R_API RAsmCode* r_asm_mdisassemble(RAsm *a, const ut8 *buf, int len) {
 			ret = 1;
 		}
 		if (a->ofilter) {
-			char *op_buf_asm = r_strbuf_get (&op.buf_asm);
-			r_parse_parse (a->ofilter, op_buf_asm, op_buf_asm);
+			parseHeap (a->ofilter, &op.buf_asm);
 		}
 		r_strbuf_append (buf_asm, r_strbuf_get (&op.buf_asm));
 		r_strbuf_append (buf_asm, "\n");
 	}
-	acode->buf_asm = r_strbuf_drain (buf_asm);
+	acode->assembly = r_strbuf_drain (buf_asm);
 	acode->len = idx;
 	return acode;
 }
 
-R_API RAsmCode* r_asm_mdisassemble_hexstr(RAsm *a, const char *hexstr) {
-	RAsmCode *ret;
-	ut8 *buf;
-	int len;
-	if (!(buf = malloc (strlen (hexstr) + 1))) {
+R_API RAsmCode* r_asm_mdisassemble_hexstr(RAsm *a, RParse *p, const char *hexstr) {
+	ut8 *buf = malloc (strlen (hexstr) + 1);
+	if (!buf) {
 		return NULL;
 	}
-	len = r_hex_str2bin (hexstr, buf);
+	int len = r_hex_str2bin (hexstr, buf);
 	if (len < 1) {
 		free (buf);
 		return NULL;
 	}
-	ret = r_asm_mdisassemble (a, buf, (ut64)len);
-	if (ret && a->ofilter) {
-		r_parse_parse (a->ofilter, ret->buf_asm, ret->buf_asm);
+	RAsmCode *ret = r_asm_mdisassemble (a, buf, (ut64)len);
+	if (ret && p) {
+		// XXX this can crash
+		r_parse_parse (p, ret->assembly, ret->assembly);
 	}
 	free (buf);
 	return ret;
 }
 
-R_API RAsmCode* r_asm_assemble_file(RAsm *a, const char *file) {
-	RAsmCode *ac = NULL;
-	char *f = r_file_slurp (file, NULL);
-	if (f) {
-		ac = r_asm_massemble (a, f);
-		free (f);
-	}
-	return ac;
-}
-
-static void flag_free_kv(HtPPKv *kv) {
+static void __flag_free_kv(HtPPKv *kv) {
 	free (kv->key);
 	free (kv->value);
 }
 
-static void *dup_val(const void *v) {
+static void *__dup_val(const void *v) {
 	return (void *)strdup ((char *)v);
 }
 
-R_API RAsmCode *r_asm_massemble(RAsm *a, const char *buf) {
-	int labels = 0, num, stage, ret, idx, ctr, i, j, linenum = 0;
+R_API RAsmCode *r_asm_massemble(RAsm *a, const char *assembly) {
+	int num, stage, ret, idx, ctr, i, j, linenum = 0;
 	char *lbuf = NULL, *ptr2, *ptr = NULL, *ptr_start = NULL;
-	char *tokens[R_ASM_BUFSIZE], buf_token[R_ASM_BUFSIZE];
 	const char *asmcpu = NULL;
 	RAsmCode *acode = NULL;
 	RAsmOp op = {0};
 	ut64 off, pc;
-	if (!buf) {
+
+	char *buf_token = NULL;
+	size_t tokens_size = 32;
+	char **tokens = calloc (sizeof (char*), tokens_size);
+	if (!tokens) {
+		return NULL;
+	}
+	if (!assembly) {
+		free (tokens);
 		return NULL;
 	}
 	ht_pp_free (a->flags);
-	if (!(a->flags = ht_pp_new (dup_val, flag_free_kv, NULL))) {
+	if (!(a->flags = ht_pp_new (__dup_val, __flag_free_kv, NULL))) {
+		free (tokens);
 		return NULL;
 	}
 	if (!(acode = r_asm_code_new ())) {
+		free (tokens);
 		return NULL;
 	}
-	if (!(acode->buf_asm = malloc (strlen (buf) + 16))) {
+	if (!(acode->assembly = malloc (strlen (assembly) + 16))) {
+		free (tokens);
 		return r_asm_code_free (acode);
 	}
-	strncpy (acode->buf_asm, buf, sizeof (acode->buf_asm) - 1);
-	if (!(acode->buf_hex = malloc (64))) { // WTF unefficient
+	r_str_ncpy (acode->assembly, assembly, sizeof (acode->assembly) - 1);
+	if (!(acode->bytes = calloc (1, 64))) {
+		free (tokens);
 		return r_asm_code_free (acode);
 	}
-	*acode->buf_hex = 0;
-	if (!(acode->buf = calloc (1, 64))) {
-		return r_asm_code_free (acode);
-	}
-	lbuf = strdup (buf);
+	lbuf = strdup (assembly);
 	acode->code_align = 0;
 
 	/* consider ,, an alias for a newline */
@@ -732,7 +725,7 @@ R_API RAsmCode *r_asm_massemble(RAsm *a, const char *buf) {
 			}
 		}
 	}
-	// XXX: ops like mov eax, $pc+33 fail coz '+' is nov alid number!!!
+	// XXX: ops like mov eax, $pc+33 fail coz '+' is not a valid number!!!
 	// XXX: must be handled here to be global.. and not arch-specific
 	{
 		char val[32];
@@ -757,17 +750,31 @@ R_API RAsmCode *r_asm_massemble(RAsm *a, const char *buf) {
 			p = strstr (p + 5, "$sys.");
 		}
 	}
-	if (strchr (lbuf, ':')) {
-		labels = 1;
-	}
+	bool labels = !!strchr (lbuf, ':');
+
 	/* Tokenize */
 	for (tokens[0] = lbuf, ctr = 0;
-			ctr < R_ASM_BUFSIZE - 1 &&
 			((ptr = strchr (tokens[ctr], ';')) ||
 			(ptr = strchr (tokens[ctr], '\n')) ||
-			(ptr = strchr (tokens[ctr], '\r')));
-			tokens[++ctr] = ptr + 1) {
+			(ptr = strchr (tokens[ctr], '\r')));) {
+		if (ctr + 1 >= tokens_size) {
+			const size_t new_tokens_size = tokens_size * 2;
+			if (sizeof (char*) * new_tokens_size <= sizeof (char*) * tokens_size) {
+				// overflow
+				eprintf ("Too many tokens\n");
+				goto fail;
+			}
+			char **new_tokens = realloc (tokens, sizeof (char*) * new_tokens_size);
+			if (!new_tokens) {
+				eprintf ("Too many tokens\n");
+				goto fail;
+			}
+			tokens_size = new_tokens_size;
+			tokens = new_tokens;
+		}
+		ctr++;
 		*ptr = '\0';
+		tokens[ctr] = ptr + 1;
 	}
 
 #define isavrseparator(x) ((x)==' '||(x)=='\t'||(x)=='\n'||(x)=='\r'||(x)==' '|| \
@@ -786,9 +793,11 @@ R_API RAsmCode *r_asm_massemble(RAsm *a, const char *buf) {
 		}
 		inComment = false;
 		r_asm_set_pc (a, pc);
-		for (idx = ret = i = j = 0, off = a->pc, acode->buf_hex[0] = '\0';
-				i <= ctr; i++, idx += ret) {
-			strncpy (buf_token, tokens[i], R_ASM_BUFSIZE - 1);
+		for (idx = ret = i = j = 0, off = a->pc; i <= ctr; i++, idx += ret) {
+			buf_token = tokens[i];
+			if (!buf_token) {
+				continue;
+			}
 			if (inComment) {
 				if (!strncmp (ptr_start, "*/", 2)) {
 					inComment = false;
@@ -797,17 +806,9 @@ R_API RAsmCode *r_asm_massemble(RAsm *a, const char *buf) {
 			}
 			// XXX TODO remove arch-specific hacks
 			if (!strncmp (a->cur->arch, "avr", 3)) {
-				for (ptr_start = buf_token; *ptr_start &&
-							    isavrseparator (*ptr_start);
-					ptr_start++) {
-					;
-				}
+				for (ptr_start = buf_token; *ptr_start && isavrseparator (*ptr_start); ptr_start++);
 			} else {
-				for (ptr_start = buf_token; *ptr_start &&
-							    IS_SEPARATOR (*ptr_start);
-					ptr_start++) {
-					;
-				}
+				for (ptr_start = buf_token; *ptr_start && IS_SEPARATOR (*ptr_start); ptr_start++);
 			}
 			if (!strncmp (ptr_start, "/*", 2)) {
 				if (!strstr (ptr_start + 2, "*/")) {
@@ -815,9 +816,29 @@ R_API RAsmCode *r_asm_massemble(RAsm *a, const char *buf) {
 				}
 				continue;
 			}
-			ptr = strchr (ptr_start, '#'); /* Comments */
-			if (ptr && !R_BETWEEN ('0', ptr[1], '9') && ptr[1]!='-') {
-				*ptr = '\0';
+			/* Comments */ {
+				bool likely_comment = true;
+				char*cptr = strchr (ptr_start, ',');
+				ptr = strchr (ptr_start, '#');
+				// a comma is probably not followed by a comment
+				// 8051 often uses #symbol notation as 2nd arg
+				if (cptr && ptr && cptr < ptr) {
+					likely_comment = false;
+					for (cptr += 1; cptr < ptr ; cptr += 1) {
+						if ( ! isspace ( *cptr)) {
+							likely_comment = true;
+							break;
+						}
+					}
+				}
+				// # followed by number literal also
+				// isn't likely to be a comment
+				likely_comment = likely_comment && ptr
+					&& !R_BETWEEN ('0', ptr[1], '9')
+					&& ptr[1] != '-' ;
+				if (likely_comment) {
+					*ptr = '\0';
+				}
 			}
 			r_asm_set_pc (a, a->pc + ret);
 			off = a->pc;
@@ -839,29 +860,32 @@ R_API RAsmCode *r_asm_massemble(RAsm *a, const char *buf) {
 				}
 				if (is_a_label) {
 					//if (stage != 2) {
-					if (ptr_start[1] != 0 && ptr_start[1] != ' ') {
+					if (ptr_start[1] && ptr_start[1] != ' ') {
 						*ptr = 0;
+						char *p = strdup (ptr_start);
+						*ptr = ':';
 						if (acode->code_align) {
 							off += (acode->code_align - (off % acode->code_align));
 						}
-						char food[64];
-						snprintf (food, sizeof (food), "0x%"PFMT64x, off);
+						char *food = r_str_newf ("0x%"PFMT64x, off);
 						ht_pp_insert (a->flags, ptr_start, food);
-						// TODO: warning when redefined
-						r_asm_code_set_equ (acode, ptr_start, food);
+						r_asm_code_set_equ (acode, p, food);
+						free (p);
+						free (food);
 					}
 					//}
 					ptr_start = ptr + 1;
 				}
 				ptr = ptr_start;
 			}
-			if (*ptr_start == '\0') {
+			if (!*ptr_start) {
 				ret = 0;
 				continue;
 			}
 			if (*ptr_start == '.') { /* pseudo */
 				/* TODO: move into a separate function */
 				ptr = ptr_start;
+				r_str_trim (ptr);
 				if (!strncmp (ptr, ".intel_syntax", 13)) {
 					a->syntax = R_ASM_SYNTAX_INTEL;
 				} else if (!strncmp (ptr, ".att_syntax", 11)) {
@@ -877,7 +901,9 @@ R_API RAsmCode *r_asm_massemble(RAsm *a, const char *buf) {
 					ret = r_asm_pseudo_string (&op, ptr + 8, 1);
 				} else if (!strncmp (ptr, ".string ", 8)) {
 					r_str_trim (ptr + 8);
-					ret = r_asm_pseudo_string (&op, ptr + 8, 1);
+					char * str = strdup (ptr + 8);
+					ret = r_asm_pseudo_string (&op, str, 1);
+					free (str);
 				} else if (!strncmp (ptr, ".ascii", 6)) {
 					ret = r_asm_pseudo_string (&op, ptr + 7, 0);
 				} else if (!strncmp (ptr, ".align", 6)) {
@@ -903,19 +929,19 @@ R_API RAsmCode *r_asm_massemble(RAsm *a, const char *buf) {
 				} else if (!strncmp (ptr, ".os ", 4)) {
 					r_syscall_setup (a->syscall, a->cur->arch, a->bits, asmcpu, ptr + 4);
 				} else if (!strncmp (ptr, ".hex ", 5)) {
-					ret = r_asm_pseudo_hex (&op, ptr + 5);
+					ret = r_asm_op_set_hex (&op, ptr + 5);
 				} else if ((!strncmp (ptr, ".int16 ", 7)) || !strncmp (ptr, ".short ", 7)) {
-					ret = r_asm_pseudo_int16 (a, &op, ptr+7);
+					ret = r_asm_pseudo_int16 (a, &op, ptr + 7);
 				} else if (!strncmp (ptr, ".int32 ", 7)) {
-					ret = r_asm_pseudo_int32 (a, &op, ptr+7);
+					ret = r_asm_pseudo_int32 (a, &op, ptr + 7);
 				} else if (!strncmp (ptr, ".int64 ", 7)) {
-					ret = r_asm_pseudo_int64 (a, &op, ptr+7);
+					ret = r_asm_pseudo_int64 (a, &op, ptr + 7);
 				} else if (!strncmp (ptr, ".size", 5)) {
 					ret = true; // do nothing, ignored
 				} else if (!strncmp (ptr, ".section", 8)) {
 					ret = true; // do nothing, ignored
 				} else if ((!strncmp (ptr, ".byte ", 6)) || (!strncmp (ptr, ".int8 ", 6))) {
-					ret = r_asm_pseudo_byte (&op, ptr+6);
+					ret = r_asm_pseudo_byte (&op, ptr + 6);
 				} else if (!strncmp (ptr, ".glob", 5)) { // .global .globl
 									 //	eprintf (".global directive not yet implemented\n");
 					ret = 0;
@@ -935,8 +961,10 @@ R_API RAsmCode *r_asm_massemble(RAsm *a, const char *buf) {
 						eprintf ("Invalid syntax for '.equ': Use '.equ <word> <word>'\n");
 					}
 				} else if (!strncmp (ptr, ".org ", 5)) {
-					ret = r_asm_pseudo_org (a, ptr+5);
+					ret = r_asm_pseudo_org (a, ptr + 5);
 					off = a->pc;
+				} else if (r_str_startswith (ptr, ".offset ")) {
+					eprintf ("Invalid use of the .offset directory. This directive is only supported in r2 -c 'waf'.\n");
 				} else if (!strncmp (ptr, ".text", 5)) {
 					acode->code_offset = a->pc;
 				} else if (!strncmp (ptr, ".data", 5)) {
@@ -949,20 +977,18 @@ R_API RAsmCode *r_asm_massemble(RAsm *a, const char *buf) {
 					ret = r_asm_pseudo_incbin (&op, ptr + 8);
 				} else {
 					eprintf ("Unknown directive (%s)\n", ptr);
-					free(lbuf);
-					return r_asm_code_free (acode);
+					goto fail;
 				}
 				if (!ret) {
 					continue;
 				}
 				if (ret < 0) {
 					eprintf ("!!! Oops (%s)\n", ptr);
-					free (lbuf);
-					return r_asm_code_free (acode);
+					goto fail;
 				}
 			} else { /* Instruction */
 				char *str = ptr_start;
-				ptr_start = r_str_trim (str);
+				r_str_trim (str);
 				if (a->ifilter) {
 					r_parse_parse (a->ifilter, ptr_start, ptr_start);
 				}
@@ -983,47 +1009,34 @@ R_API RAsmCode *r_asm_massemble(RAsm *a, const char *buf) {
 			if (stage == STAGES - 1) {
 				if (ret < 1) {
 					eprintf ("Cannot assemble '%s' at line %d\n", ptr_start, linenum);
-					free (lbuf);
-					return r_asm_code_free (acode);
+					goto fail;
 				}
 				acode->len = idx + ret;
-				char *newbuf = realloc (acode->buf, (idx + ret) * 2);
+				char *newbuf = realloc (acode->bytes, (idx + ret) * 2);
 				if (!newbuf) {
-					free (lbuf);
-					return r_asm_code_free (acode);
+					goto fail;
 				}
-				acode->buf = (ut8*)newbuf;
-
-				const int buf_inc_size = op.buf_inc ? r_buf_size (op.buf_inc): 0;
-				int newlen = strlen (acode->buf_hex) + r_strbuf_length (&op.buf_hex)
-					+ buf_inc_size + 128;
-				newbuf = realloc (acode->buf_hex, newlen);
-				if (!newbuf) {
-					free (lbuf);
-					return r_asm_code_free (acode);
-				}
-				acode->buf_hex = newbuf;
-				acode->buf = malloc (4095);
-				memcpy (acode->buf + idx, r_strbuf_get (&op.buf), r_strbuf_length (&op.buf)); // ret);
-				// XXX slow. use strbuf pls
-				strcat (acode->buf_hex, r_strbuf_get (&op.buf_hex));
+				acode->bytes = (ut8*)newbuf;
+				memcpy (acode->bytes + idx, r_strbuf_get (&op.buf), r_strbuf_length (&op.buf));
+				memset (acode->bytes + idx + ret, 0, idx + ret);
 				if (op.buf_inc && r_buf_size (op.buf_inc) > 1) {
-					if (*acode->buf_hex) {
-						strcat (acode->buf_hex, "\n");
-					}
-					char *s = r_buf_free_to_string (op.buf_inc);
-					if (s) {
-						strcat (acode->buf_hex, s);
-						free (s);
+					char *inc = r_buf_to_string (op.buf_inc);
+					r_buf_free (op.buf_inc);
+					if (inc) {
+						ret += r_hex_str2bin (inc, acode->bytes + idx + ret);
+						free (inc);
 					}
 				}
-
 			}
 		}
 	}
 	free (lbuf);
-	r_hex_str2bin (acode->buf_hex, acode->buf);
+	free (tokens);
 	return acode;
+fail:
+	free (lbuf);
+	free (tokens);
+	return r_asm_code_free (acode);
 }
 
 R_API bool r_asm_modify(RAsm *a, ut8 *buf, int field, ut64 val) {
@@ -1051,11 +1064,12 @@ R_API bool r_asm_set_arch(RAsm *a, const char *name, int bits) {
 
 /* to ease the use of the native bindings (not used in r2) */
 R_API char *r_asm_to_string(RAsm *a, ut64 addr, const ut8 *b, int l) {
+	r_return_val_if_fail (a && b && l >= 0, NULL);
 	r_asm_set_pc (a, addr);
 	RAsmCode *code = r_asm_mdisassemble (a, b, l);
 	if (code) {
-		char *buf_asm = code->buf_asm;
-		code->buf_asm = NULL;
+		char *buf_asm = code->assembly;
+		code->assembly = NULL;
 		r_asm_code_free (code);
 		return buf_asm;
 	}
@@ -1066,7 +1080,7 @@ R_API ut8 *r_asm_from_string(RAsm *a, ut64 addr, const char *b, int *l) {
 	r_asm_set_pc (a, addr);
 	RAsmCode *code = r_asm_massemble (a, b);
 	if (code) {
-		ut8 *buf = code->buf;
+		ut8 *buf = code->bytes;
 		if (l) {
 			*l = code->len;
 		}
@@ -1077,6 +1091,7 @@ R_API ut8 *r_asm_from_string(RAsm *a, ut64 addr, const char *b, int *l) {
 }
 
 R_API int r_asm_syntax_from_string(const char *name) {
+	r_return_val_if_fail (name, -1);
 	if (!strcmp (name, "regnum")) {
 		return R_ASM_SYNTAX_REGNUM;
 	}
@@ -1096,14 +1111,16 @@ R_API int r_asm_syntax_from_string(const char *name) {
 }
 
 R_API char *r_asm_mnemonics(RAsm *a, int id, bool json) {
-	if (a && a->cur && a->cur->mnemonics) {
+	r_return_val_if_fail (a && a->cur, NULL);
+	if (a->cur->mnemonics) {
 		return a->cur->mnemonics (a, id, json);
 	}
 	return NULL;
 }
 
 R_API int r_asm_mnemonics_byname(RAsm *a, const char *name) {
-	if (a && a->cur && a->cur->mnemonics) {
+	r_return_val_if_fail (a && a->cur, 0);
+	if (a->cur->mnemonics) {
 		int i;
 		for (i = 0; i < 1024; i++) {
 			char *n = a->cur->mnemonics (a, i, false);
@@ -1117,6 +1134,7 @@ R_API int r_asm_mnemonics_byname(RAsm *a, const char *name) {
 }
 
 R_API RAsmCode* r_asm_rasm_assemble(RAsm *a, const char *buf, bool use_spp) {
+	r_return_val_if_fail (a && buf, NULL);
 	char *lbuf = strdup (buf);
 	if (!lbuf) {
 		return NULL;
