@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2013-2018 - pancake, astuder */
+/* radare2 - LGPL - Copyright 2013-2019 - pancake, astuder */
 
 #include <stdio.h>
 #include <stdarg.h>
@@ -8,13 +8,19 @@
 #include <r_lib.h>
 #include <r_asm.h>
 
-#include <8051_disas.h>
+#include <8051_ass.h>
+#include "../arch/8051/8051_disas.c"
 
 static int disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
-	int dlen = _8051_disas (a->pc, op, buf, len);
+	int dlen = 0;
+	char *s = r_8051_disas (a->pc, buf, len, &dlen);
 	if (dlen < 0) {
 		dlen = 0;
 	}
+	if (s) {
+		r_strbuf_set (&op->buf_asm, s);
+		free (s);
+	}
 	op->size = dlen;
 	return dlen;
 }
@@ -26,14 +32,15 @@ RAsmPlugin r_asm_plugin_8051 = {
 	.endian = R_SYS_ENDIAN_NONE,
 	.desc = "8051 Intel CPU",
 	.disassemble = &disassemble,
+	.assemble = &assemble_8051,
 	.license = "PD",
 	.cpus =
 		"8051-generic," // First one is default
 		"8051-shared-code-xdata"
 };
 
-#ifndef CORELIB
-RLibStruct radare_plugin = {
+#ifndef R2_PLUGIN_INCORE
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_8051,
 	.version = R2_VERSION
