@@ -217,7 +217,7 @@ RAsmPlugin r_asm_plugin_arm_gnu = {
 	.license = "GPL3"
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_arm_gnu,
