@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2017 - earada, pancake */
+/* radare - LGPL - Copyright 2009-2019 - earada, pancake, h4ng3r */
 
 #include <stdio.h>
 #include <string.h>
@@ -10,27 +10,29 @@
 #include <dalvik/opcode.h>
 
 static int dalvik_disassemble (RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
-	if (!a || !op || !buf || len < 1) {
-		return -1;
-	}
+	r_return_val_if_fail  (a && op && buf && len > 0, -1);
+
 	int vA, vB, vC, vD, vE, vF, vG, vH, payload = 0, i = (int) buf[0];
 	int size = dalvik_opcodes[i].len;
-	char str[1024], *strasm;
+	char str[1024], *strasm = NULL;
 	ut64 offset;
-	const char *flag_str; 
+	const char *flag_str;
 	a->dataalign = 2;
 
-	op->buf_asm[0] = 0;
+	const char *buf_asm = NULL;
 	if (buf[0] == 0x00) { /* nop */
+		if (len < 2) {
+			return -1;
+		}
 		switch (buf[1]) {
 		case 0x01: /* packed-switch-payload */
 			// ushort size
 			// int first_key
 			// int[size] = relative offsets
 			{
-				unsigned short array_size = buf[2] | (buf[3] << 8);
+				ut16 array_size = buf[2] | (buf[3] << 8);
 				int first_key = buf[4] | (buf[5] << 8) | (buf[6] << 16) | (buf[7] << 24);
-				snprintf (op->buf_asm, sizeof(op->buf_asm), "packed-switch-payload %d, %d", array_size, first_key);
+				buf_asm = sdb_fmt ("packed-switch-payload %d, %d", array_size, first_key);
 				size = 8;
 				payload = 2 * (array_size * 2);
 				len = 0;
@@ -41,10 +43,10 @@ static int dalvik_disassemble (RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 			// int[size] keys
 			// int[size] relative offsets
 			{
-				unsigned short array_size = buf[2] | (buf[3] << 8);
-				snprintf (op->buf_asm, sizeof (op->buf_asm), "sparse-switch-payload %d", array_size);
+				ut16 array_size = buf[2] | (buf[3] << 8);
+				buf_asm = sdb_fmt ("sparse-switch-payload %d", array_size);
 				size = 4;
-				payload = 2 * (array_size*4);
+				payload = 2 * (array_size * 4);
 				len = 0;
 			}
 			break;
@@ -53,11 +55,9 @@ static int dalvik_disassemble (RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 			// size = 4 bytes uint
 			// ([size*element_width+1)/2)+4
 			if (len > 7) {
-				unsigned short elem_width = buf[2] | (buf[3] << 8);
-				unsigned int array_size = buf[4] | (buf[5] << 8) | (buf[6] << 16) | (buf[7] << 24);
-				snprintf (op->buf_asm, sizeof (op->buf_asm),
-					"fill-array-data-payload %d, %d",
-					elem_width, array_size);
+				ut16 elem_width = buf[2] | (buf[3] << 8);
+				ut32 array_size = buf[4] | (buf[5] << 8) | (buf[6] << 16) | (buf[7] << 24);
+				buf_asm = sdb_fmt ("fill-array-data-payload %d, %d", elem_width, array_size);
 				payload = array_size * elem_width;
 			}
 			size = 8;
@@ -68,10 +68,12 @@ static int dalvik_disassemble (RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 			break;
 		}
 	}
+	if (buf_asm) {
+		r_strbuf_set (&op->buf_asm, buf_asm);
+	}
 	strasm = NULL;
 	if (size <= len) {
-		strncpy (op->buf_asm, dalvik_opcodes[i].name, sizeof (op->buf_asm) - 1);
-		strasm = strdup (op->buf_asm);
+		strasm = strdup (dalvik_opcodes[i].name);
 		size = dalvik_opcodes[i].len;
 		switch (dalvik_opcodes[i].fmt) {
 		case fmtop: break;
@@ -135,12 +137,10 @@ static int dalvik_disassemble (RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 			break;
 		case fmtopvAAcBBBBBBBBBBBBBBBB:
 			vA = (int) buf[1];
-			#define llint long long int
-			llint lB = (llint)buf[2] | ((llint)buf[3] << 8)|
-				((llint)buf[4] << 16) | ((llint)buf[5] << 24)|
-				((llint)buf[6] << 32) | ((llint)buf[7] << 40)|
-				((llint)buf[8] << 48) | ((llint)buf[9] << 56);
-			#undef llint
+			ut64 lB = (ut64)buf[2] | ((ut64)buf[3] << 8)|
+				((ut64)buf[4] << 16) | ((ut64)buf[5] << 24)|
+				((ut64)buf[6] << 32) | ((ut64)buf[7] << 40)|
+				((ut64)(buf[8]&0xff) << 48) | ((ut64)(buf[9]&0xff) << 56);
 			snprintf (str, sizeof (str), " v%i:v%i, 0x%"PFMT64x, vA, vA + 1, lB);
 			strasm = r_str_append (strasm, str);
 			break;
@@ -320,16 +320,14 @@ static int dalvik_disassemble (RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 				flag_str = R_ASM_GET_NAME (a, 'c', vC);
 				if (flag_str) {
 					snprintf (str, sizeof (str), " v%i, v%i, %s", vA, vB, flag_str);
-				}
-				else {
+				} else {
 					snprintf (str, sizeof (str), " v%i, v%i, class+%i", vA, vB, vC);
 				}
 			} else {
 				flag_str = R_ASM_GET_NAME (a, 'f', vC);
 				if (flag_str) {
 					snprintf (str, sizeof (str), " v%i, v%i, %s", vA, vB, flag_str);
-				}
-				else {
+				} else {
 					snprintf (str, sizeof (str), " v%i, v%i, field+%i", vA, vB, vC);
 				}
 			}
@@ -341,8 +339,7 @@ static int dalvik_disassemble (RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 			offset = R_ASM_GET_OFFSET (a, 's', vB);
 			if (offset == -1) {
 				snprintf (str, sizeof (str), " v%i, string+%i", vA, vB);
-			}
-			else {
+			} else {
 				snprintf (str, sizeof (str), " v%i, 0x%"PFMT64x, vA, offset);
 			}
 			strasm = r_str_append (strasm, str);
@@ -440,39 +437,40 @@ static int dalvik_disassemble (RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 			vH = (buf[7] << 8) | buf[6];
 
 			switch (vA) {
-				case 1:
-					snprintf (str, sizeof (str), " {v%d}", vC);
-					break;
-				case 2:
-					snprintf (str, sizeof (str), " {v%d, v%d}", vC, vD);
-					break;
-				case 3:
-					snprintf (str, sizeof (str), " {v%d, v%d, v%d}", vC, vD, vE);
-					break;
-				case 4:
-					snprintf (str, sizeof (str), " {v%d, v%d, v%d, v%d}", vC, vD, vE, vF);
-					break;
-				case 5:
-					snprintf (str, sizeof (str), " {v%d, v%d, v%d, v%d, v%d}", vC, vD, vE, vF, vG);
-					break;
+			case 1:
+				snprintf (str, sizeof (str), " {v%d}", vC);
+				break;
+			case 2:
+				snprintf (str, sizeof (str), " {v%d, v%d}", vC, vD);
+				break;
+			case 3:
+				snprintf (str, sizeof (str), " {v%d, v%d, v%d}", vC, vD, vE);
+				break;
+			case 4:
+				snprintf (str, sizeof (str), " {v%d, v%d, v%d, v%d}", vC, vD, vE, vF);
+				break;
+			case 5:
+				snprintf (str, sizeof (str), " {v%d, v%d, v%d, v%d, v%d}", vC, vD, vE, vF, vG);
+				break;
+			default:
+				snprintf (str, sizeof (str), " %d", vC);
+				break;
 			}
 			strasm = r_str_append (strasm, str);
 
 			flag_str = R_ASM_GET_NAME (a, 'm', vB);
 			if (flag_str) {
-				snprintf (str, sizeof (str), ", %s", flag_str);
+				strasm = r_str_appendf (strasm, ", %s", flag_str);
 			} else {
-				snprintf (str, sizeof (str), ", method+%i", vB);
+				strasm = r_str_appendf (strasm, ", method+%i", vB);
 			}
-			strasm = r_str_append (strasm, str);
 
 			flag_str = R_ASM_GET_NAME (a, 'p', vH);
 			if (flag_str) {
-				snprintf (str, sizeof (str), ", %s", flag_str);
+				strasm = r_str_appendf (strasm, ", %s", flag_str);
 			} else {
-				snprintf (str, sizeof (str), ", proto+%i", vH);
+				strasm = r_str_appendf (strasm, ", proto+%i", vH);
 			}
-			strasm = r_str_append (strasm, str);
 			break;
 		case fmtop4RCC:
 			vA = (int) buf[1];
@@ -482,8 +480,7 @@ static int dalvik_disassemble (RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 			flag_str = R_ASM_GET_NAME (a, 'm', vB);
 			if (flag_str) {
 				snprintf (str, sizeof (str), " {v%i..v%i}, %s", vC, vC + vA - 1, flag_str);
-			}
-			else {
+			} else {
 				snprintf (str, sizeof (str), " {v%i..v%i}, method+%i", vC, vC + vA - 1, vB);
 			}
 			strasm = r_str_append (strasm, str);
@@ -491,8 +488,7 @@ static int dalvik_disassemble (RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 			flag_str = R_ASM_GET_NAME (a, 'p', vH);
 			if (flag_str) {
 				snprintf (str, sizeof (str), ", %s", flag_str);
-			}
-			else {
+			} else {
 				snprintf (str, sizeof (str), ", proto+%i", vH);
 			}
 			strasm = r_str_append (strasm, str);
@@ -501,27 +497,32 @@ static int dalvik_disassemble (RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 		case fmtoptinvokeIR:
 		case fmt00:
 		default:
-			strcpy (op->buf_asm, "invalid ");
 			free (strasm);
 			strasm = NULL;
 			size = 2;
 		}
-		if (strasm) {
-			strncpy (op->buf_asm, strasm, sizeof (op->buf_asm) - 1);
-			op->buf_asm[sizeof (op->buf_asm) - 1] = 0;
-		} else {
-			//op->buf_asm[0] = 0;
-			strcpy (op->buf_asm , "invalid");
-		}
+		r_strbuf_set (&op->buf_asm, strasm? strasm: "invalid");
 	} else if (len > 0) {
-		strcpy (op->buf_asm, "invalid ");
+		r_strbuf_set (&op->buf_asm, "invalid");
 		op->size = len;
 		size = len;
 	}
-	op->payload = payload;
-	size += payload; // XXX
-	// align to 2
-	op->size = size;
+
+	if (payload < 0) {
+		op->payload = 0;
+	} else if (len > 0 && payload >= len) {
+		op->payload = len;
+	} else {
+		op->payload = payload;
+	}
+
+	if (size + op->payload < 0) {
+		op->size = 0;
+	} else if (size + op->payload >= len) {
+		op->size = len;
+	} else {
+		op->size = size + op->payload;
+	}
 	free (strasm);
 	return size;
 }
@@ -536,7 +537,9 @@ static int dalvik_assemble(RAsm *a, RAsmOp *op, const char *buf) {
 	// TODO: use a hashtable here
 	for (i = 0; i < 256; i++) {
 		if (!strcmp (dalvik_opcodes[i].name, buf)) {
-			r_write_ble32 (op->buf, i, a->big_endian);
+			ut8 buf[4];
+			r_write_ble32 (buf, i, a->big_endian);
+			r_strbuf_setbin (&op->buf, buf, sizeof (buf));
 			op->size = dalvik_opcodes[i].len;
 			return op->size;
 		}
@@ -555,8 +558,8 @@ RAsmPlugin r_asm_plugin_dalvik = {
 	.assemble = &dalvik_assemble,
 };
 
-#ifndef CORELIB
-RLibStruct radare_plugin = {
+#ifndef R2_PLUGIN_INCORE
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_dalvik,
 	.version = R2_VERSION
