@@ -20,8 +20,8 @@ RAsmPlugin r_asm_plugin_i4004 = {
 	.disassemble = &disassemble
 };
 
-#ifndef CORELIB
-RLibStruct radare_plugin = {
+#ifndef R2_PLUGIN_INCORE
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_i4004,
 	.version = R2_VERSION
