@@ -11,7 +11,9 @@
 
 static int disassemble(RAsm *a, RAsmOp *r_op, const ut8 *buf, int len) {
 	int dlen = spc700Disass(r_op, buf, len);
-	if(dlen < 0) dlen=0;
+	if (dlen < 0) {
+		dlen = 0;
+	}
 	r_op->size = dlen;
 	return dlen;
 }
@@ -26,8 +28,8 @@ RAsmPlugin r_asm_plugin_spc700 = {
 	.disassemble = &disassemble,
 };
 
-#ifndef CORELIB
-RLibStruct radare_plugin = {
+#ifndef R2_PLUGIN_INCORE
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_spc700,
 	.version = R2_VERSION
