@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2016 - pancake */
+/* radare - LGPL - Copyright 2009-2019 - pancake */
 
 #include <r_types.h>
 #include <r_util.h>
@@ -8,34 +8,36 @@
 static int assemble(RAsm *a, RAsmOp *op, const char *buf) {
 	char *ipath, *opath;
 	int ifd, ofd;
-	char asm_buf[R_ASM_BUFSIZE];
-	int len = 0;
 	if (a->syntax != R_ASM_SYNTAX_INTEL) {
 		eprintf ("asm.x86.nasm does not support non-intel syntax\n");
 		return -1;
 	}
 
 	ifd = r_file_mkstemp ("r_nasm", &ipath);
-	if (ifd == -1)
+	if (ifd == -1) {
 		return -1;
-		
+	}
+
 	ofd = r_file_mkstemp ("r_nasm", &opath);
 	if (ofd == -1) {
 		free (ipath);
 		return -1;
 	}
 
-	len = snprintf (asm_buf, sizeof (asm_buf),
-			"[BITS %i]\nORG 0x%"PFMT64x"\n%s\n", a->bits, a->pc, buf);
-	write (ifd, asm_buf, len);
+	char *asm_buf = r_str_newf ("[BITS %i]\nORG 0x%"PFMT64x"\n%s\n", a->bits, a->pc, buf);
+	if (asm_buf) {
+		write (ifd, asm_buf, strlen (asm_buf));
+		free (asm_buf);
+	}
 
 	close (ifd);
 
-	if ( !r_sys_cmdf ("nasm %s -o %s", ipath, opath)) {
-		len = read (ofd, op->buf, R_ASM_BUFSIZE);
+	if (!r_sys_cmdf ("nasm %s -o %s", ipath, opath)) {
+		ut8 buf[512]; // TODO: remove limits
+		op->size = read (ofd, buf, sizeof (buf));
+		r_asm_op_set_buf (op, buf, op->size);
 	} else {
 		eprintf ("Error running 'nasm'\n");
-		len = 0;
 	}
 
 	close (ofd);
@@ -44,8 +46,7 @@ static int assemble(RAsm *a, RAsmOp *op, const char *buf) {
 	free (ipath);
 	free (opath);
 
-	op->size = len;
-	return len;
+	return op->size;
 }
 
 RAsmPlugin r_asm_plugin_x86_nasm = {
@@ -59,8 +60,8 @@ RAsmPlugin r_asm_plugin_x86_nasm = {
 	.assemble = &assemble
 };
 
-#ifndef CORELIB
-RLibStruct radare_plugin = {
+#ifndef R2_PLUGIN_INCORE
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_x86_nasm,
 	.version = R2_VERSION
