@@ -28,7 +28,7 @@ RAsmPlugin r_asm_plugin_z80 = {
 	.assemble = &do_assemble,
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_z80,
