@@ -1,72 +1,66 @@
-/* radare - LGPL - Copyright 2009-2017 - pancake, h4ng3r */
+/* radare - LGPL - Copyright 2009-2019 - pancake, h4ng3r */
 
 #include <r_types.h>
 #include <r_util.h>
 #include "dex.h"
+#include "./dex_uleb128.inc"
 
 char* r_bin_dex_get_version(RBinDexObj *bin) {
-	if (bin) {
-		ut8* version = calloc (1, 8);
-		r_buf_read_at (bin->b, 4, version, 3);
-		return (char *)version;
+	r_return_val_if_fail (bin, NULL);
+	char* version = calloc (1, 8);
+	if (version) {
+		r_buf_read_at (bin->b, 4, (ut8*)version, 3);
+		return version;
 	}
 	return NULL;
 }
 
-#define FAIL(x) { eprintf(x"\n"); goto fail; }
 RBinDexObj *r_bin_dex_new_buf(RBuffer *buf) {
-	if (!buf) {
-		return NULL;
-	}
+	r_return_val_if_fail (buf, NULL);
 	RBinDexObj *bin = R_NEW0 (RBinDexObj);
 	int i;
-	ut8 *bufptr;
 	struct dex_header_t *dexhdr;
 	if (!bin) {
 		goto fail;
 	}
-	bin->size = buf->length;
-	bin->b = r_buf_new ();
-	if (!r_buf_set_bytes (bin->b, buf->buf, bin->size)) {
-		goto fail;
-	}
+	bin->size = r_buf_size (buf);
+	bin->b = r_buf_ref (buf);
 	/* header */
 	if (bin->size < sizeof (struct dex_header_t)) {
 		goto fail;
 	}
-	bufptr = bin->b->buf;
 	dexhdr = &bin->header;
 
-	//check boundaries of bufptr
 	if (bin->size < 112) {
 		goto fail;
 	}
 
-	memcpy (&dexhdr->magic, bufptr, 8);
-	dexhdr->checksum = r_read_le32 (bufptr + 8);
-	memcpy (&dexhdr->signature, bufptr + 12, 20);
-	dexhdr->size = r_read_le32 (bufptr + 32);
-	dexhdr->header_size = r_read_le32 (bufptr + 36);
-	dexhdr->endian = r_read_le32 (bufptr + 40);
+	r_buf_seek (bin->b, 0, R_BUF_SET);
+	r_buf_read (bin->b, (ut8 *)&dexhdr->magic, 8);
+	dexhdr->checksum = r_buf_read_le32 (bin->b);
+	r_buf_read (bin->b, (ut8 *)&dexhdr->signature, 20);
+	dexhdr->size = r_buf_read_le32 (bin->b);
+	dexhdr->header_size = r_buf_read_le32 (bin->b);
+	dexhdr->endian = r_buf_read_le32 (bin->b);
 	// TODO: this offsets and size will be used for checking,
 	// so they should be checked. Check overlap, < 0, > bin.size
-	dexhdr->linksection_size = r_read_le32 (bufptr + 44);
-	dexhdr->linksection_offset = r_read_le32 (bufptr + 48);
-	dexhdr->map_offset = r_read_le32 (bufptr + 52);
-	dexhdr->strings_size = r_read_le32 (bufptr + 56);
-	dexhdr->strings_offset = r_read_le32 (bufptr + 60);
-	dexhdr->types_size = r_read_le32 (bufptr + 64);
-	dexhdr->types_offset = r_read_le32 (bufptr + 68);
-	dexhdr->prototypes_size = r_read_le32 (bufptr + 72);
-	dexhdr->prototypes_offset = r_read_le32 (bufptr + 76);
-	dexhdr->fields_size = r_read_le32 (bufptr + 80);
-	dexhdr->fields_offset = r_read_le32 (bufptr + 84);
-	dexhdr->method_size = r_read_le32 (bufptr + 88);
-	dexhdr->method_offset = r_read_le32 (bufptr + 92);
-	dexhdr->class_size = r_read_le32 (bufptr + 96);
-	dexhdr->class_offset = r_read_le32 (bufptr + 100);
-	dexhdr->data_size = r_read_le32 (bufptr + 104);
-	dexhdr->data_offset = r_read_le32 (bufptr + 108);
+	dexhdr->linksection_size = r_buf_read_le32 (bin->b);
+	dexhdr->linksection_offset = r_buf_read_le32 (bin->b);
+	dexhdr->map_offset = r_buf_read_le32 (bin->b);
+	dexhdr->strings_size = r_buf_read_le32 (bin->b);
+	dexhdr->strings_offset = r_buf_read_le32 (bin->b);
+	dexhdr->types_size = r_buf_read_le32 (bin->b);
+	dexhdr->types_offset = r_buf_read_le32 (bin->b);
+	dexhdr->prototypes_size = r_buf_read_le32 (bin->b);
+	dexhdr->prototypes_offset = r_buf_read_le32 (bin->b);
+	dexhdr->fields_size = r_buf_read_le32 (bin->b);
+	dexhdr->fields_offset = r_buf_read_le32 (bin->b);
+	dexhdr->method_size = r_buf_read_le32 (bin->b);
+	dexhdr->method_offset = r_buf_read_le32 (bin->b);
+	dexhdr->class_size = r_buf_read_le32 (bin->b);
+	dexhdr->class_offset = r_buf_read_le32 (bin->b);
+	dexhdr->data_size = r_buf_read_le32 (bin->b);
+	dexhdr->data_offset = r_buf_read_le32 (bin->b);
 
 	/* strings */
 	#define STRINGS_SIZE ((dexhdr->strings_size + 1) * sizeof (ut32))
@@ -80,12 +74,11 @@ RBinDexObj *r_bin_dex_new_buf(RBuffer *buf) {
 	}
 	for (i = 0; i < dexhdr->strings_size; i++) {
 		ut64 offset = dexhdr->strings_offset + i * sizeof (ut32);
-		//make sure we can read from bufptr without oob
 		if (offset + 4 > bin->size) {
 			free (bin->strings);
 			goto fail;
 		}
-		bin->strings[i] = r_read_le32 (bufptr + offset);
+		bin->strings[i] = r_buf_read_le32_at (bin->b, offset);
 	}
 	/* classes */
 	// TODO: not sure about if that is needed
@@ -106,14 +99,15 @@ RBinDexObj *r_bin_dex_new_buf(RBuffer *buf) {
 			free (bin->classes);
 			goto fail;
 		}
-		bin->classes[i].class_id = r_read_le32 (bufptr + offset + 0);
-		bin->classes[i].access_flags = r_read_le32 (bufptr + offset + 4);
-		bin->classes[i].super_class = r_read_le32 (bufptr + offset + 8);
-		bin->classes[i].interfaces_offset = r_read_le32 (bufptr + offset + 12);
-		bin->classes[i].source_file = r_read_le32 (bufptr + offset + 16);
-		bin->classes[i].anotations_offset = r_read_le32 (bufptr + offset + 20);
-		bin->classes[i].class_data_offset = r_read_le32 (bufptr + offset + 24);
-		bin->classes[i].static_values_offset = r_read_le32 (bufptr + offset + 28);
+		r_buf_seek (bin->b, offset, R_BUF_SET);
+		bin->classes[i].class_id = r_buf_read_le32 (bin->b);
+		bin->classes[i].access_flags = r_buf_read_le32 (bin->b);
+		bin->classes[i].super_class = r_buf_read_le32 (bin->b);
+		bin->classes[i].interfaces_offset = r_buf_read_le32 (bin->b);
+		bin->classes[i].source_file = r_buf_read_le32 (bin->b);
+		bin->classes[i].anotations_offset = r_buf_read_le32 (bin->b);
+		bin->classes[i].class_data_offset = r_buf_read_le32 (bin->b);
+		bin->classes[i].static_values_offset = r_buf_read_le32 (bin->b);
 	}
 
 	/* methods */
@@ -134,9 +128,10 @@ RBinDexObj *r_bin_dex_new_buf(RBuffer *buf) {
 			free (bin->methods);
 			goto fail;
 		}
-		bin->methods[i].class_id = r_read_le16 (bufptr + offset + 0);
-		bin->methods[i].proto_id = r_read_le16 (bufptr + offset + 2);
-		bin->methods[i].name_id = r_read_le32 (bufptr + offset + 4);
+		r_buf_seek (bin->b, offset, R_BUF_SET);
+		bin->methods[i].class_id = r_buf_read_le16 (bin->b);
+		bin->methods[i].proto_id = r_buf_read_le16 (bin->b);
+		bin->methods[i].name_id = r_buf_read_le32 (bin->b);
 	}
 
 	/* types */
@@ -158,7 +153,7 @@ RBinDexObj *r_bin_dex_new_buf(RBuffer *buf) {
 			free (bin->types);
 			goto fail;
 		}
-		bin->types[i].descriptor_id = r_read_le32 (bufptr + offset);
+		bin->types[i].descriptor_id = r_buf_read_le32_at (bin->b, offset);
 	}
 
 	/* fields */
@@ -181,9 +176,10 @@ RBinDexObj *r_bin_dex_new_buf(RBuffer *buf) {
 			free (bin->fields);
 			goto fail;
 		}
-		bin->fields[i].class_id = r_read_le16 (bufptr + offset + 0);
-		bin->fields[i].type_id = r_read_le16 (bufptr + offset + 2);
-		bin->fields[i].name_id = r_read_le32 (bufptr + offset + 4);
+		r_buf_seek (bin->b, offset, R_BUF_SET);
+		bin->fields[i].class_id = r_buf_read_le16 (bin->b);
+		bin->fields[i].type_id = r_buf_read_le16 (bin->b);
+		bin->fields[i].name_id = r_buf_read_le32 (bin->b);
 	}
 
 	/* proto */
@@ -208,13 +204,12 @@ RBinDexObj *r_bin_dex_new_buf(RBuffer *buf) {
 			free (bin->protos);
 			goto fail;
 		}
-		bin->protos[i].shorty_id = r_read_le32 (bufptr + offset + 0);
-		bin->protos[i].return_type_id = r_read_le32 (bufptr + offset + 4);
-		bin->protos[i].parameters_off = r_read_le32 (bufptr + offset + 8);
+		r_buf_seek (bin->b, offset, R_BUF_SET);
+		bin->protos[i].shorty_id = r_buf_read_le32 (bin->b);
+		bin->protos[i].return_type_id = r_buf_read_le32 (bin->b);
+		bin->protos[i].parameters_off = r_buf_read_le32 (bin->b);
 	}
-
 	return bin;
-	
 fail:
 	if (bin) {
 		r_buf_free (bin->b);
@@ -222,73 +217,3 @@ fail:
 	}
 	return NULL;
 }
-
-// Move to r_util ??
-int dex_read_uleb128(const ut8 *ptr, int size) {
-	ut8 len = dex_uleb128_len (ptr, size);
-	if (len > size) {
-		return 0;
-	}
-	const ut8 *in = ptr + len - 1;
-	ut32 result = 0;
-	ut8 shift = 0;
-	ut8 byte;
-
-	while(shift < 29 && len > 0) {
-		byte = *(in--);
-		result |= (byte & 0x7f << shift);
-		if (byte > 0x7f) {
-			break;
-		}
-		shift += 7;
-		len--;
-	}
-	return result;
-}
-
-#define LEB_MAX_SIZE 6
-int dex_uleb128_len(const ut8 *ptr, int size) {
-	int i = 1, result = *(ptr++);
-	while (result > 0x7f && i <= LEB_MAX_SIZE && i < size) {
-		result = *(ptr++);
-		i++;
-	}
-	return i;
-}
-
-#define SIG_EXTEND(X,Y) X = ((X) << (Y)) >> Y
-int dex_read_sleb128(const char *ptr, int size) {
-	int cur, result;
-	ut8 len = dex_uleb128_len ((const ut8*)ptr, size);
-	if (len > size) {
-		return 0;
-	}
-	ptr += len - 1;
-	result = *(ptr--);
-
-	if (result <= 0x7f) {
-		SIG_EXTEND (result, 25);
-	} else {
-		cur = *(ptr--);
-		result = (result & 0x7f) | ((cur & 0x7f) << 7);
-		if (cur <= 0x7f) {
-			SIG_EXTEND (result, 18);
-		} else {
-			cur = *(ptr--);
-			result |= (cur & 0x7f) << 14;
-			if (cur <= 0x7f) {
-				SIG_EXTEND (result, 11);
-			} else {
-				cur = *(ptr--);
-				result |= (cur & 0x7f) << 21;
-				if (cur <= 0x7f) {
-					SIG_EXTEND (result, 4);
-				} else {
-					cur = *(ptr--);
-					result |= cur << 28;
-				}
-			}
-		}
-	}
-	return result;
-}
