@@ -38,6 +38,7 @@ struct r_bin_mdmp_obj {
 		RList *operations;
 		RList *thread_infos;
 		RList *threads;
+		RList *token_infos;
 		RList *unloaded_modules;
 		struct {
 			rva64_t base_rva;
@@ -49,16 +50,16 @@ struct r_bin_mdmp_obj {
 	RList *pe32_bins;
 	RList *pe64_bins;
 
-	struct r_buf_t *b;
+	RBuffer *b;
 	size_t size;
 	ut8 endian;
 	Sdb *kv;
 };
 
-struct r_bin_mdmp_obj *r_bin_mdmp_new_buf(struct r_buf_t *buf);
+struct r_bin_mdmp_obj *r_bin_mdmp_new_buf(RBuffer *buf);
 void r_bin_mdmp_free(struct r_bin_mdmp_obj *obj);
 ut64 r_bin_mdmp_get_paddr(struct r_bin_mdmp_obj *obj, ut64 vaddr);
-ut32 r_bin_mdmp_get_srwx(struct r_bin_mdmp_obj *obj, ut64 vaddr);
+ut32 r_bin_mdmp_get_perm(struct r_bin_mdmp_obj *obj, ut64 vaddr);
 struct minidump_memory_info *r_bin_mdmp_get_mem_info(struct r_bin_mdmp_obj *obj, ut64 vaddr);
 
 #endif /* MDMP_H */
