@@ -1,21 +1,22 @@
-/* radare - LGPL - Copyright 2010-2018 pancake, nibble */
+/* radare - LGPL - Copyright 2010-2019 pancake, nibble */
 
 #include <r_types.h>
 #include <r_util.h>
 #include "pe.h"
 
-bool PE_(r_bin_pe_section_perms)(struct PE_(r_bin_pe_obj_t) *bin, const char *name, int perms) {
-	PE_(image_section_header) *shdr = bin->section_header;
+// bool PE_(r_bin_pe_section_perms)(struct PE_(r_bin_pe_obj_t) *bin, const char *name, int perms) {
+bool PE_(r_bin_pe_section_perms)(RBinFile *bf, const char *name, int perms) {
+	struct PE_(r_bin_pe_obj_t) *pe = bf->o->bin_obj;
+	PE_(image_section_header) *shdr = pe->section_header;
 	int i;
 
 	if (!shdr) {
 		return false;
 	}
 
-	for (i = 0; i < bin->num_sections; i++) {
+	for (i = 0; i < pe->num_sections; i++) {
 		const char *sname = (const char*) shdr[i].Name;
 		if (!strncmp (name, sname, PE_IMAGE_SIZEOF_SHORT_NAME)) {
-			int patchoff;
 			ut32 newperms = shdr[i].Characteristics;
 			ut32 newperms_le;
 
@@ -41,12 +42,15 @@ bool PE_(r_bin_pe_section_perms)(struct PE_(r_bin_pe_obj_t) *bin, const char *na
 				newperms &= ~PE_IMAGE_SCN_MEM_SHARED;
 			}
 
-			patchoff = bin->section_header_offset;
+			int patchoff = pe->section_header_offset;
 			patchoff += i * sizeof (PE_(image_section_header));
 			patchoff += r_offsetof (PE_(image_section_header), Characteristics);
 			r_write_le32 (&newperms_le, newperms);
 			printf ("wx %02x @ 0x%x\n", newperms_le, patchoff);
-			r_buf_write_at (bin->b, patchoff, (ut8*)&newperms_le, sizeof (newperms_le));
+			int res = r_buf_write_at (bf->buf, patchoff, (ut8*)&newperms_le, sizeof (newperms_le));
+			if (res != sizeof (newperms_le)) {
+				return false;
+			}
 			return true;
 		}
 	}
