@@ -333,8 +333,13 @@ extern unsigned int xcrc32 (const unsigned char *, int, unsigned int);
    as new/delete and new[]/delete[].  */
 
 /* Scalar allocators.  */
+ #define XALLOCA(T)             ((T *) alloca (sizeof (T)))
+ #define XNEW(T)                        ((T *) xmalloc (sizeof (T)))
+ #define XCNEW(T)               ((T *) xcalloc (1, sizeof (T)))
+
 #ifdef _MSC_VER
 #include <stdlib.h>
+#define alloca(x) _alloca(x)
 #define xmalloc malloc
 #define xcalloc calloc
 #define xrealloc realloc
@@ -651,7 +656,7 @@ extern void setproctitle (const char *name, ...);
    also set/unset as it is often used to indicate whether code needs
    to call alloca(0).  */
 #ifndef _MSC_VER
-extern void *C_alloca (size_t) ATTRIBUTE_MALLOC;
+extern void *_alloca (size_t) ATTRIBUTE_MALLOC;
 #endif
 #undef alloca
 #if GCC_VERSION >= 2000 && !defined USE_C_ALLOCA
@@ -663,7 +668,7 @@ extern void *C_alloca (size_t) ATTRIBUTE_MALLOC;
    char *const libiberty_nptr = (char *const) alloca (libiberty_len); \
    (char *) memcpy (libiberty_nptr, libiberty_optr, libiberty_len); }))
 #else
-# define alloca(x) C_alloca(x)
+# define alloca(x) _alloca(x)
 # undef USE_C_ALLOCA
 # define USE_C_ALLOCA 1
 # undef C_ALLOCA
