@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2015 - ret2libc, pancake */
+/* radare - LGPL - Copyright 2009-2019 - ret2libc, pancake */
 
 #define R_BIN_CGC 1
 #include "bin_elf.inc"
@@ -6,8 +6,10 @@
 extern struct r_bin_dbginfo_t r_bin_dbginfo_elf;
 extern struct r_bin_write_t r_bin_write_elf;
 
-static bool check_bytes(const ut8 *buf, ut64 length) {
-	return buf && length > 4 && !memcmp (buf, CGCMAG, SCGCMAG) && buf[4] != 2;
+static bool check_buffer(RBuffer *buf) {
+	ut8 tmp[SCGCMAG + 1];
+	int r = r_buf_read_at (buf, 0, tmp, sizeof (tmp));
+	return r > SCGCMAG && !memcmp (tmp, CGCMAG, SCGCMAG) && tmp[4] != 2;
 }
 
 static RBuffer* create(RBin* bin, const ut8 *code, int codelen, const ut8 *data, int datalen, RBinArchOptions *opt) {
@@ -24,7 +26,7 @@ static RBuffer* create(RBin* bin, const ut8 *code, int codelen, const ut8 *data,
 #define H(x) r_buf_append_ut16(buf,x)
 #define Z(x) r_buf_append_nbytes(buf,x)
 #define W(x,y,z) r_buf_write_at(buf,x,(const ut8*)(y),z)
-#define WZ(x,y) p_tmp=buf->length;Z(x);W(p_tmp,y,strlen(y))
+#define WZ(x,y) p_tmp=r_buf_size (buf);Z(x);W(p_tmp,y,strlen(y))
 
 	B ("\x7F" "CGC" "\x01\x01\x01\x43", 8);
 	Z (8);
@@ -32,38 +34,38 @@ static RBuffer* create(RBin* bin, const ut8 *code, int codelen, const ut8 *data,
 	H (3); // e_machne = EM_I386
 
 	D (1);
-	p_start = buf->length;
+	p_start = r_buf_size (buf);
 	D (-1); // _start
-	p_phoff = buf->length;
+	p_phoff = r_buf_size (buf);
 	D (-1); // phoff -- program headers offset
 	D (0);  // shoff -- section headers offset
 	D (0);  // flags
-	p_ehdrsz = buf->length;
+	p_ehdrsz = r_buf_size (buf);
 	H (-1); // ehdrsz
-	p_phdrsz = buf->length;
+	p_phdrsz = r_buf_size (buf);
 	H (-1); // phdrsz
 	H (1);
 	H (0);
 	H (0);
 	H (0);
 	// phdr:
-	p_phdr = buf->length;
+	p_phdr = r_buf_size (buf);
 	D (1);
 	D (0);
-	p_vaddr = buf->length;
+	p_vaddr = r_buf_size (buf);
 	D (-1); // vaddr = $$
-	p_paddr = buf->length;
+	p_paddr = r_buf_size (buf);
 	D (-1); // paddr = $$
-	p_fs = buf->length;
+	p_fs = r_buf_size (buf);
 	D (-1); // filesize
-	p_fs2 = buf->length;
+	p_fs2 = r_buf_size (buf);
 	D (-1); // filesize
 	D (5); // flags
 	D (0x1000); // align
 
 	ehdrsz = p_phdr;
-	phdrsz = buf->length - p_phdr;
-	code_pa = buf->length;
+	phdrsz = r_buf_size (buf) - p_phdr;
+	code_pa = r_buf_size (buf);
 	code_va = code_pa + baddr;
 	phoff = 0x34;//p_phdr ;
 	filesize = code_pa + codelen + datalen;
@@ -96,11 +98,9 @@ RBinPlugin r_bin_plugin_cgc = {
 	.desc = "CGC format r_bin plugin",
 	.license = "LGPL3",
 	.get_sdb = &get_sdb,
-	.load = &load,
-	.load_bytes = &load_bytes,
-	.load_buffer= load_buffer,
+	.load_buffer = load_buffer,
 	.destroy = &destroy,
-	.check_bytes = &check_bytes,
+	.check_buffer = &check_buffer,
 	.baddr = &baddr,
 	.boffset = &boffset,
 	.binsym = &binsym,
