@@ -11,6 +11,8 @@
 #include "../format/mach0/mach0.h"
 #include "objc/mach0_classes.h"
 
+#define R_IS_PTR_AUTHENTICATED(x) B_IS_SET(x, 63)
+
 typedef struct {
 	ut8 version;
 	ut64 slide;
@@ -19,6 +21,19 @@ typedef struct {
 	ut64 start_of_data;
 } RDyldRebaseInfo;
 
+typedef struct {
+	ut8 version;
+	ut64 slide;
+	ut8 *one_page_buf;
+	ut32 page_size;
+	ut64 start_of_data;
+	ut16 *page_starts;
+	ut32 page_starts_count;
+	ut64 delta_mask;
+	ut32 delta_shift;
+	ut64 auth_value_add;
+} RDyldRebaseInfo3;
+
 typedef struct {
 	ut8 version;
 	ut64 slide;
@@ -63,6 +78,8 @@ typedef struct _r_bin_image {
 	ut64 header_at;
 } RDyldBinImage;
 
+static RList * pending_bin_files = NULL;
+
 static void free_bin(RDyldBinImage *bin) {
 	if (!bin) {
 		return;
@@ -72,6 +89,15 @@ static void free_bin(RDyldBinImage *bin) {
 	R_FREE (bin);
 }
 
+static void rebase_info3_free(RDyldRebaseInfo3 *rebase_info) {
+	if (!rebase_info) {
+		return;
+	}
+
+	R_FREE (rebase_info->page_starts);
+	R_FREE (rebase_info);
+}
+
 static void rebase_info2_free(RDyldRebaseInfo2 *rebase_info) {
 	if (!rebase_info) {
 		return;
@@ -105,6 +131,8 @@ static void rebase_info_free(RDyldRebaseInfo *rebase_info) {
 		rebase_info1_free ((RDyldRebaseInfo1*) rebase_info);
 	} else if (version == 2) {
 		rebase_info2_free ((RDyldRebaseInfo2*) rebase_info);
+	} else if (version == 3) {
+		rebase_info3_free ((RDyldRebaseInfo3*) rebase_info);
 	} else {
 		R_FREE (rebase_info);
 	}
@@ -115,21 +143,12 @@ static void r_dyldcache_free(RDyldCache *cache) {
 		return;
 	}
 
-	if (cache->bins) {
-		r_list_free (cache->bins);
-		cache->bins = NULL;
-	}
-
-	if (cache->buf) {
-		r_buf_free (cache->buf);
-		cache->buf = NULL;
-	}
-
-	if (cache->rebase_info) {
-		rebase_info_free (cache->rebase_info);
-		cache->rebase_info = NULL;
-	}
-
+	r_list_free (cache->bins);
+	cache->bins = NULL;
+	r_buf_free (cache->buf);
+	cache->buf = NULL;
+	rebase_info_free (cache->rebase_info);
+	cache->rebase_info = NULL;
 	R_FREE (cache->hdr);
 	R_FREE (cache->maps);
 	R_FREE (cache->accel);
@@ -210,7 +229,7 @@ static int prot2perm(int x) {
 
 static ut32 dumb_ctzll(ut64 x) {
 	ut64 result = 0;
-	int i,j;
+	int i, j;
 	for (i = 0; i < 64; i += 8) {
 		ut8 byte = (x >> i) & 0xff;
 		if (!byte) {
@@ -287,9 +306,13 @@ static ut64 estimate_slide(RBinFile *bf, RDyldCache *cache, ut64 value_mask) {
 		}
 
 		ut64 data_addr = sections[data_idx].addr;
+		ut64 data_tail = data_addr & 0xfff;
+		ut64 data_tail_end = (data_addr + sections[data_idx].size) & 0xfff;
 		for (i = 0; i < n_classes; i++) {
-			if ((classlist[i] & 0xfff) == (data_addr & 0xfff)) {
-				slide = (classlist[i] & value_mask) - (data_addr & value_mask);
+			ut64 cl_tail = classlist[i] & 0xfff;
+			if (cl_tail >= data_tail && cl_tail < data_tail_end) {
+				ut64 off = cl_tail - data_tail;
+				slide = ((classlist[i] - off) & value_mask) - (data_addr & value_mask);
 				found_sample = true;
 				break;
 			}
@@ -336,7 +359,58 @@ static RDyldRebaseInfo *get_rebase_info(RBinFile *bf, RDyldCache *cache) {
 		return NULL;
 	}
 
-	if (slide_info_version == 2) {
+	if (slide_info_version == 3) {
+		cache_slide3_t slide_info;
+		ut64 size = sizeof (cache_slide3_t);
+		if (r_buf_fread_at (cache_buf, offset, (ut8*) &slide_info, "4i1l", 1) < 20) {
+			return NULL;
+		}
+
+		ut64 page_starts_offset = offset + size;
+		ut64 page_starts_size = slide_info.page_starts_count * 2;
+
+		if (page_starts_size + size > cache->hdr->slideInfoSize) {
+			return NULL;
+		}
+
+		if (page_starts_size > 0) {
+			tmp_buf_1 = malloc (page_starts_size);
+			if (!tmp_buf_1) {
+				goto beach;
+			}
+			if (r_buf_fread_at (cache_buf, page_starts_offset, tmp_buf_1, "s", slide_info.page_starts_count) != page_starts_size) {
+				goto beach;
+			}
+		}
+
+		if (slide_info.page_size > 0) {
+			one_page_buf = malloc (slide_info.page_size);
+			if (!one_page_buf) {
+				goto beach;
+			}
+		}
+
+		RDyldRebaseInfo3 *rebase_info = R_NEW0 (RDyldRebaseInfo3);
+		if (!rebase_info) {
+			goto beach;
+		}
+
+		rebase_info->version = 3;
+		rebase_info->delta_mask = 0x3ff8000000000000ULL;
+		rebase_info->delta_shift = 48;
+		rebase_info->start_of_data = start_of_data;
+		rebase_info->page_starts = (ut16*) tmp_buf_1;
+		rebase_info->page_starts_count = slide_info.page_starts_count;
+		rebase_info->auth_value_add = slide_info.auth_value_add;
+		rebase_info->page_size = slide_info.page_size;
+		rebase_info->one_page_buf = one_page_buf;
+		rebase_info->slide = estimate_slide (bf, cache, 0x7ffffffffffffULL);
+		if (rebase_info->slide) {
+			eprintf ("dyldcache is slid: 0x%"PFMT64x"\n", rebase_info->slide);
+		}
+
+		return (RDyldRebaseInfo*) rebase_info;
+	} else if (slide_info_version == 2) {
 		cache_slide2_t slide_info;
 		ut64 size = sizeof (cache_slide2_t);
 		if (r_buf_fread_at (cache_buf, offset, (ut8*) &slide_info, "6i2l", 1) != size) {
@@ -466,7 +540,7 @@ static RDyldRebaseInfo *get_rebase_info(RBinFile *bf, RDyldCache *cache) {
 		rebase_info->start_of_data = start_of_data;
 		rebase_info->one_page_buf = one_page_buf;
 		rebase_info->page_size = 4096;
-		rebase_info->slide = estimate_slide (bf, cache, 0xffffffffffffffffL);
+		rebase_info->slide = estimate_slide (bf, cache, UT64_MAX);
 		rebase_info->toc = (ut16*) tmp_buf_1;
 		rebase_info->toc_count = slide_info.toc_count;
 		rebase_info->entries = tmp_buf_2;
@@ -477,6 +551,7 @@ static RDyldRebaseInfo *get_rebase_info(RBinFile *bf, RDyldCache *cache) {
 
 		return (RDyldRebaseInfo*) rebase_info;
 	} else {
+		eprintf ("unsupported slide info version %d\n", slide_info_version);
 		return NULL;
 	}
 
@@ -487,22 +562,22 @@ beach:
 	return NULL;
 }
 
-static bool check_bytes(const ut8 *buf, ut64 length) {
-	bool rc = false;
-	if (buf && length >= 32) {
-		char arch[9] = { 0 };
-		strncpy (arch, (const char *) buf + 9, R_MIN (length, sizeof (arch) - 1));
-		rc = !memcmp (buf, "dyld", 4);
-		if (rc) {
-			if (*arch) {
-				eprintf ("Arch: %s\n", arch);
-				if (!strstr (arch, "arm64")) {
-					return false;
-				}
-			}
-		}
+static bool check_buffer(RBuffer *buf) {
+	if (r_buf_size (buf) < 32) {
+		return false;
 	}
-	return rc;
+
+	ut8 hdr[4];
+	ut8 arch[9] = { 0 };
+	int rarch = r_buf_read_at (buf, 9, arch, sizeof (arch) - 1);
+	int rhdr = r_buf_read_at (buf, 0, hdr, sizeof (hdr));
+	if (rhdr != sizeof (hdr) || memcmp (hdr, "dyld", 4)) {
+		return false;
+	}
+	if (rarch > 0 && arch[0] && !strstr ((const char *)arch, "arm64")) {
+		return false;
+	}
+	return true;
 }
 
 static cache_img_t *read_cache_images(RBuffer *cache_buf, cache_hdr_t *hdr) {
@@ -578,7 +653,7 @@ static RList *create_cache_bins(RBinFile *bf, RBuffer *cache_buf, cache_hdr_t *h
 	char *target_libs = NULL;
 	target_libs = r_sys_getenv ("R_DYLDCACHE_FILTER");
 	if (target_libs) {
-		RList *target_lib_names = r_str_split_list (target_libs, ":");
+		RList *target_lib_names = r_str_split_list (target_libs, ":", 0);
 		if (!target_lib_names) {
 			R_FREE (target_libs);
 			r_list_free (bins);
@@ -799,12 +874,63 @@ next_page:
 	}
 }
 
+static void rebase_bytes_v3(RDyldRebaseInfo3 *rebase_info, ut8 *buf, ut64 offset, int count, ut64 start_of_write) {
+	int in_buf = 0;
+	while (in_buf < count) {
+		ut64 offset_in_data = offset - rebase_info->start_of_data;
+		ut64 page_index = offset_in_data / rebase_info->page_size;
+		ut64 page_offset = offset_in_data % rebase_info->page_size;
+		ut64 to_next_page = rebase_info->page_size - page_offset;
+
+		if (page_index >= rebase_info->page_starts_count) {
+			goto next_page;
+		}
+		ut64 delta = rebase_info->page_starts[page_index];
+
+		if (delta == DYLD_CACHE_SLIDE_V3_PAGE_ATTR_NO_REBASE) {
+			goto next_page;
+		}
+
+		ut64 first_rebase_off = delta;
+		if (first_rebase_off >= page_offset && first_rebase_off < page_offset + count) {
+			do {
+				ut64 position = in_buf + first_rebase_off - page_offset;
+				if (position >= count) {
+					break;
+				}
+				ut64 raw_value = r_read_le64 (buf + position);
+				delta = ((raw_value & rebase_info->delta_mask) >> rebase_info->delta_shift);
+				if (position >= start_of_write) {
+					ut64 new_value = 0;
+					if (R_IS_PTR_AUTHENTICATED (raw_value)) {
+						new_value = (raw_value & 0xFFFFFFFFULL) + rebase_info->auth_value_add;
+						// TODO: don't throw auth info away
+					} else {
+						new_value = ((raw_value << 13) & 0xFF00000000000000ULL) | (raw_value & 0x7ffffffffffULL);
+						new_value &= 0x00FFFFFFFFFFFFFFULL;
+					}
+					if (new_value != 0) {
+						new_value += rebase_info->slide;
+					}
+					r_write_le64 (buf + position, new_value);
+				}
+				first_rebase_off += delta;
+			} while (delta);
+		}
+next_page:
+		in_buf += to_next_page;
+		offset += to_next_page;
+	}
+}
+
 static void rebase_bytes(RDyldRebaseInfo *rebase_info, ut8 *buf, ut64 offset, int count, ut64 start_of_write) {
 	if (!rebase_info || !buf) {
 		return;
 	}
 
-	if (rebase_info->version == 2) {
+	if (rebase_info->version == 3) {
+		rebase_bytes_v3 ((RDyldRebaseInfo3*) rebase_info, buf, offset, count, start_of_write);
+	} else if (rebase_info->version == 2) {
 		rebase_bytes_v2 ((RDyldRebaseInfo2*) rebase_info, buf, offset, count, start_of_write);
 	} else if (rebase_info->version == 1) {
 		rebase_bytes_v1 ((RDyldRebaseInfo1*) rebase_info, buf, offset, count, start_of_write);
@@ -831,10 +957,35 @@ static int dyldcache_io_read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
 			} else {
 				cache = ((struct MACH0_(obj_t)*) bf->o->bin_obj)->user;
 			}
+			if (pending_bin_files) {
+				RListIter *to_remove = r_list_contains (pending_bin_files, bf);
+				if (to_remove) {
+					r_list_delete (pending_bin_files, to_remove);
+					if (r_list_empty (pending_bin_files)) {
+						r_list_free (pending_bin_files);
+						pending_bin_files = NULL;
+					}
+				}
+			}
 			break;
 		}
 	}
+	if (!cache) {
+		r_list_foreach (pending_bin_files, iter, bf) {
+			if (bf->fd == fd->fd && bf->o) {
+				if (!strncmp ((char*) bf->o->bin_obj, "dyldcac", 7)) {
+					cache = bf->o->bin_obj;
+				} else {
+					cache = ((struct MACH0_(obj_t)*) bf->o->bin_obj)->user;
+				}
+				break;
+			}
+		}
+	}
 	if (!cache || !cache->original_io_read) {
+		if (fd->plugin->read == &dyldcache_io_read) {
+			return -1;
+		}
 		return fd->plugin->read (io, fd, buf, count);
 	}
 
@@ -969,63 +1120,48 @@ static cache_accel_t *read_cache_accel(RBuffer *cache_buf, cache_hdr_t *hdr, cac
 	return accel;
 }
 
-static void *load_buffer(RBinFile *bf, RBuffer *buf, ut64 loadaddr, Sdb *sdb) {
-	RBuffer *fbuf = r_buf_new_with_io (&bf->rbin->iob, bf->fd);
-	ut8 bytes_to_check[32];
-	r_buf_read_at (fbuf, 0, (ut8*)bytes_to_check, 32);
-	if (!check_bytes (bytes_to_check, 32)) {
-		r_buf_free (fbuf);
-		return NULL;
-	}
-
+static bool load_buffer(RBinFile *bf, void **bin_obj, RBuffer *buf, ut64 loadaddr, Sdb *sdb) {
 	RDyldCache *cache = R_NEW0 (RDyldCache);
 	memcpy (cache->magic, "dyldcac", 7);
-	cache->buf = fbuf;
-	cache->hdr = read_cache_header (fbuf);
+	cache->buf = r_buf_ref (buf);
+	cache->hdr = read_cache_header (cache->buf);
 	if (!cache->hdr) {
 		r_dyldcache_free (cache);
-		return NULL;
+		return false;
 	}
-
-	cache->maps = read_cache_maps (fbuf, cache->hdr);
+	cache->maps = read_cache_maps (cache->buf, cache->hdr);
 	if (!cache->maps) {
 		r_dyldcache_free (cache);
-		return NULL;
+		return false;
 	}
-
-	cache->accel = read_cache_accel (fbuf, cache->hdr, cache->maps);
+	cache->accel = read_cache_accel (cache->buf, cache->hdr, cache->maps);
 	if (!cache->accel) {
 		r_dyldcache_free (cache);
-		return NULL;
+		return false;
 	}
-
-	cache->bins = create_cache_bins (bf, fbuf, cache->hdr, cache->maps, cache->accel);
+	cache->bins = create_cache_bins (bf, cache->buf, cache->hdr, cache->maps, cache->accel);
 	if (!cache->bins) {
 		r_dyldcache_free (cache);
-		return NULL;
+		return false;
 	}
-
 	cache->rebase_info = get_rebase_info (bf, cache);
 	if (!cache->rebase_info) {
 		r_dyldcache_free (cache);
-		return NULL;
+		return false;
 	}
-
 	if (!cache->rebase_info->slide) {
+		if (!pending_bin_files) {
+			pending_bin_files = r_list_new ();
+			if (!pending_bin_files) {
+				r_dyldcache_free (cache);
+				return false;
+			}
+		}
+		r_list_push (pending_bin_files, bf);
 		swizzle_io_read (cache, bf->rbin->iob.io);
 	}
-	return cache;
-}
-
-static bool load_bytes(RBinFile *bf, void **bin_obj, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb) {
-	return check_bytes (buf, sz);
-}
-
-static bool load(RBinFile *bf) {
-	const ut8 *bytes = bf ? r_buf_buffer (bf->buf) : NULL;
-	ut64 sz = bf ? r_buf_size (bf->buf): 0;
-	ut64 la = (bf && bf->o) ? bf->o->loadaddr: 0;
-	return load_bytes (bf, bf? &bf->o->bin_obj: NULL, bytes, sz, la, bf? bf->sdb: NULL);
+	*bin_obj = cache;
+	return true;
 }
 
 static RList *entries(RBinFile *bf) {
@@ -1089,7 +1225,8 @@ void symbols_from_bin(RList *ret, RBinFile *bf, RDyldBinImage *bin) {
 		return;
 	}
 
-	struct symbol_t *symbols = MACH0_(get_symbols) (mach0);
+	// const RList*symbols = MACH0_(get_symbols_list) (mach0);
+	const struct symbol_t *symbols = MACH0_(get_symbols) (mach0);
 	if (!symbols) {
 		return;
 	}
@@ -1108,7 +1245,7 @@ void symbols_from_bin(RList *ret, RBinFile *bf, RDyldBinImage *bin) {
 		sym->name = strdup (symbols[i].name);
 		sym->vaddr = symbols[i].addr;
 		if (sym->name[0] == '_') {
-			char *dn = r_bin_demangle (bf, sym->name, sym->name, sym->vaddr);
+			char *dn = r_bin_demangle (bf, sym->name, sym->name, sym->vaddr, false);
 			if (dn) {
 				sym->dname = dn;
 				char *p = strchr (dn, '.');
@@ -1126,31 +1263,34 @@ void symbols_from_bin(RList *ret, RBinFile *bf, RDyldBinImage *bin) {
 				}
 			}
 		}
-		sym->forwarder = r_str_const ("NONE");
-		sym->bind = r_str_const ((symbols[i].type == R_BIN_MACH0_SYMBOL_TYPE_LOCAL)?
-			R_BIN_BIND_LOCAL_STR: R_BIN_BIND_GLOBAL_STR);
-		sym->type = r_str_const (R_BIN_TYPE_FUNC_STR);
+		sym->forwarder = "NONE";
+		sym->bind = (symbols[i].type == R_BIN_MACH0_SYMBOL_TYPE_LOCAL)? R_BIN_BIND_LOCAL_STR: R_BIN_BIND_GLOBAL_STR;
+		sym->type = R_BIN_TYPE_FUNC_STR;
 		sym->paddr = symbols[i].offset + bf->o->boffset;
 		sym->size = symbols[i].size;
 		sym->ordinal = i;
 		r_list_append (ret, sym);
 	}
-	free (symbols);
 	MACH0_(mach0_free) (mach0);
 }
 
-static void handle_data_sections(RBinSection *sect) {
-	if (strstr (sect->name, "_cstring")) {
-		sect->is_data = true;
-	} else if (strstr (sect->name, "_os_log")) {
-		sect->is_data = true;
-	} else if (strstr (sect->name, "_objc_methname")) {
-		sect->is_data = true;
-	} else if (strstr (sect->name, "_objc_classname")) {
-		sect->is_data = true;
-	} else if (strstr (sect->name, "_objc_methtype")) {
-		sect->is_data = true;
+static bool __is_data_section(const char *name) {
+	if (strstr (name, "_cstring")) {
+		return true;
 	}
+	if (strstr (name, "_os_log")) {
+		return true;
+	}
+	if (strstr (name, "_objc_methname")) {
+		return true;
+	}
+	if (strstr (name, "_objc_classname")) {
+		return true;
+	}
+	if (strstr (name, "_objc_methtype")) {
+		return true;
+	}
+	return false;
 }
 
 static void sections_from_bin(RList *ret, RBinFile *bf, RDyldBinImage *bin) {
@@ -1166,8 +1306,8 @@ static void sections_from_bin(RList *ret, RBinFile *bf, RDyldBinImage *bin) {
 
 	int i;
 	for (i = 0; !sections[i].last; i++) {
-		RBinSection *ptr;
-		if (!(ptr = R_NEW0 (RBinSection))) {
+		RBinSection *ptr = R_NEW0 (RBinSection);
+		if (!ptr) {
 			break;
 		}
 		if (bin->file) {
@@ -1179,12 +1319,11 @@ static void sections_from_bin(RList *ret, RBinFile *bf, RDyldBinImage *bin) {
 			int len = sections[i].size / 8;
 			ptr->format = r_str_newf ("Cd %d[%d]", 8, len);
 		}
-		handle_data_sections (ptr);
+		ptr->is_data = __is_data_section (ptr->name);
 		ptr->size = sections[i].size;
 		ptr->vsize = sections[i].vsize;
 		ptr->paddr = sections[i].offset + bf->o->boffset;
 		ptr->vaddr = sections[i].addr;
-		ptr->add = true;
 		if (!ptr->vaddr) {
 			ptr->vaddr = ptr->paddr;
 		}
@@ -1221,9 +1360,10 @@ static RList *sections(RBinFile *bf) {
 		ptr->name = r_str_newf ("cache_map.%d", i);
 		ptr->size = cache->maps[i].size;
 		ptr->vsize = ptr->size;
-		ptr->paddr = cache->maps[i].fileOffset;// + bf->o->boffset;
+		ptr->paddr = cache->maps[i].fileOffset;
 		ptr->vaddr = cache->maps[i].address;
 		ptr->add = true;
+		ptr->is_segment = true;
 		ptr->perm = prot2perm (cache->maps[i].initProt);
 		r_list_append (ret, ptr);
 	}
@@ -1275,11 +1415,10 @@ static RList *symbols(RBinFile *bf) {
 	cache->original_io_read = NULL;
 } */
 
-static int destroy(RBinFile *bf) {
+static void destroy(RBinFile *bf) {
 	RDyldCache *cache = (RDyldCache*) bf->o->bin_obj;
 	// unswizzle_io_read (cache, bf->rbin->iob.io); // XXX io may be dead here
 	r_dyldcache_free (cache);
-	return true;
 }
 
 static RList *classes(RBinFile *bf) {
@@ -1315,7 +1454,11 @@ static RList *classes(RBinFile *bf) {
 			if (sections[i].size == 0) {
 				continue;
 			}
-			if (!strstr (sections[i].name, "__objc_classlist")) {
+
+			bool is_classlist = strstr (sections[i].name, "__objc_classlist");
+			bool is_catlist = strstr (sections[i].name, "__objc_catlist");
+
+			if (!is_classlist && !is_catlist) {
 				continue;
 			}
 
@@ -1343,7 +1486,11 @@ static RList *classes(RBinFile *bf) {
 
 				bf->o->bin_obj = mach0;
 				bf->buf = cache->buf;
-				MACH0_(get_class_t) ((ut64) pointer_to_class, bf, klass, false);
+				if (is_classlist) {
+					MACH0_(get_class_t) ((ut64) pointer_to_class, bf, klass, false, NULL);
+				} else {
+					MACH0_(get_category_t) ((ut64) pointer_to_class, bf, klass, NULL);
+				}
 				bf->o->bin_obj = cache;
 				bf->buf = orig_buf;
 
@@ -1434,7 +1581,7 @@ static void header(RBinFile *bf) {
 	bin->cb_printf ("\nslide info (v%d):\n", version);
 	bin->cb_printf ("slide: 0x%"PFMT64x"\n", slide);
 	if (version == 2) {
-		RDyldRebaseInfo2 * info2 = (RDyldRebaseInfo2*) cache->rebase_info;
+		RDyldRebaseInfo2 *info2 = (RDyldRebaseInfo2*) cache->rebase_info;
 		bin->cb_printf ("page_starts_count: 0x%"PFMT64x"\n", info2->page_starts_count);
 		bin->cb_printf ("page_extras_count: 0x%"PFMT64x"\n", info2->page_extras_count);
 		bin->cb_printf ("delta_mask: 0x%"PFMT64x"\n", info2->delta_mask);
@@ -1442,7 +1589,7 @@ static void header(RBinFile *bf) {
 		bin->cb_printf ("delta_shift: 0x%"PFMT64x"\n", info2->delta_shift);
 		bin->cb_printf ("page_size: 0x%"PFMT64x"\n", info2->page_size);
 	} else if (version == 1) {
-		RDyldRebaseInfo1 * info1 = (RDyldRebaseInfo1*) cache->rebase_info;
+		RDyldRebaseInfo1 *info1 = (RDyldRebaseInfo1*) cache->rebase_info;
 		bin->cb_printf ("toc_count: 0x%"PFMT64x"\n", info1->toc_count);
 		bin->cb_printf ("entries_size: 0x%"PFMT64x"\n", info1->entries_size);
 		bin->cb_printf ("page_size: 0x%"PFMT64x"\n", 4096);
@@ -1453,21 +1600,19 @@ RBinPlugin r_bin_plugin_dyldcache = {
 	.name = "dyldcache",
 	.desc = "dyldcache bin plugin",
 	.license = "LGPL3",
-	.load = &load,
-	.load_bytes = &load_bytes,
 	.load_buffer = &load_buffer,
 	.entries = &entries,
 	.baddr = &baddr,
 	.symbols = &symbols,
 	.sections = &sections,
-	.check_bytes = &check_bytes,
+	.check_buffer = &check_buffer,
 	.destroy = &destroy,
 	.classes = &classes,
 	.header = &header,
 	.info = &info,
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_BIN,
 	.data = &r_bin_plugin_dyldcache,
