@@ -1,37 +1,39 @@
+/* radare2 - LGPL - Copyright 2009-2019 - pancake, nibble, dso */
+
 #include "bin_elf.inc"
 
 static void headers32(RBinFile *bf) {
 #define p bf->rbin->cb_printf
-	const ut8 *buf = r_buf_get_at (bf->buf, 0, NULL);
-	p ("0x00000000  ELF MAGIC   0x%08x\n", r_read_le32 (buf));
-	p ("0x00000010  Type        0x%04x\n", r_read_le16 (buf + 0x10));
-	p ("0x00000012  Machine     0x%04x\n", r_read_le16 (buf + 0x12));
-	p ("0x00000014  Version     0x%08x\n", r_read_le32 (buf + 0x14));
-	p ("0x00000018  Entrypoint  0x%08x\n", r_read_le32 (buf + 0x18));
-	p ("0x0000001c  PhOff       0x%08x\n", r_read_le32 (buf + 0x1c));
-	p ("0x00000020  ShOff       0x%08x\n", r_read_le32 (buf + 0x20));
+	p ("0x00000000  ELF MAGIC   0x%08x\n", r_buf_read_le32_at (bf->buf, 0));
+	p ("0x00000010  Type        0x%04x\n", r_buf_read_le16_at (bf->buf, 0x10));
+	p ("0x00000012  Machine     0x%04x\n", r_buf_read_le16_at (bf->buf, 0x12));
+	p ("0x00000014  Version     0x%08x\n", r_buf_read_le32_at (bf->buf, 0x14));
+	p ("0x00000018  Entrypoint  0x%08x\n", r_buf_read_le32_at (bf->buf, 0x18));
+	p ("0x0000001c  PhOff       0x%08x\n", r_buf_read_le32_at (bf->buf, 0x1c));
+	p ("0x00000020  ShOff       0x%08x\n", r_buf_read_le32_at (bf->buf, 0x20));
 }
 
-static bool check_bytes(const ut8 *buf, ut64 length) {
-	return buf && length > 4 && memcmp (buf, ELFMAG, SELFMAG) == 0
-		&& buf[4] != 2;
+static bool check_buffer(RBuffer *buf) {
+	ut8 b[5] = {0};
+	r_buf_read_at (buf, 0, b, sizeof (b));
+	return !memcmp (b, ELFMAG, SELFMAG) && b[4] != 2;
 }
 
 extern struct r_bin_dbginfo_t r_bin_dbginfo_elf;
 extern struct r_bin_write_t r_bin_write_elf;
 
 static RBuffer* create(RBin* bin, const ut8 *code, int codelen, const ut8 *data, int datalen, RBinArchOptions *opt) {
+	r_return_val_if_fail (bin && opt && opt->arch, NULL);
+
 	ut32 filesize, code_va, code_pa, phoff;
 	ut32 p_start, p_phoff, p_phdr;
 	ut32 p_ehdrsz, p_phdrsz;
 	ut16 ehdrsz, phdrsz;
 	ut32 p_vaddr, p_paddr, p_fs, p_fs2;
 	ut32 baddr;
-	int is_arm = 0;
 	RBuffer *buf = r_buf_new ();
 
-	r_return_val_if_fail (bin && opt && opt->arch, NULL);
-	is_arm = !strcmp (opt->arch, "arm");
+	bool is_arm = !strcmp (opt->arch, "arm");
 	// XXX: hardcoded
 	if (is_arm) {
 		baddr = 0x40000;
@@ -44,7 +46,7 @@ static RBuffer* create(RBin* bin, const ut8 *code, int codelen, const ut8 *data,
 #define H(x) r_buf_append_ut16(buf,x)
 #define Z(x) r_buf_append_nbytes(buf,x)
 #define W(x,y,z) r_buf_write_at(buf,x,(const ut8*)(y),z)
-#define WZ(x,y) p_tmp=buf->length;Z(x);W(p_tmp,y,strlen(y))
+#define WZ(x,y) p_tmp=r_buf_size (buf);Z(x);W(p_tmp,y,strlen(y))
 
 	B ("\x7F" "ELF" "\x01\x01\x01\x00", 8);
 	Z (8);
@@ -56,38 +58,38 @@ static RBuffer* create(RBin* bin, const ut8 *code, int codelen, const ut8 *data,
 	}
 
 	D (1);
-	p_start = buf->length;
+	p_start = r_buf_size (buf);
 	D (-1); // _start
-	p_phoff = buf->length;
+	p_phoff = r_buf_size (buf);
 	D (-1); // phoff -- program headers offset
 	D (0);  // shoff -- section headers offset
-	D (0);  // flags
-	p_ehdrsz = buf->length;
+	D (0); // flags
+	p_ehdrsz = r_buf_size (buf);
 	H (-1); // ehdrsz
-	p_phdrsz = buf->length;
+	p_phdrsz = r_buf_size (buf);
 	H (-1); // phdrsz
 	H (1);
 	H (0);
 	H (0);
 	H (0);
 	// phdr:
-	p_phdr = buf->length;
+	p_phdr = r_buf_size (buf);
 	D (1);
 	D (0);
-	p_vaddr = buf->length;
+	p_vaddr = r_buf_size (buf);
 	D (-1); // vaddr = $$
-	p_paddr = buf->length;
+	p_paddr = r_buf_size (buf);
 	D (-1); // paddr = $$
-	p_fs = buf->length;
+	p_fs = r_buf_size (buf);
 	D (-1); // filesize
-	p_fs2 = buf->length;
+	p_fs2 = r_buf_size (buf);
 	D (-1); // filesize
 	D (5); // flags
 	D (0x1000); // align
 
 	ehdrsz = p_phdr;
-	phdrsz = buf->length - p_phdr;
-	code_pa = buf->length;
+	phdrsz = r_buf_size (buf) - p_phdr;
+	code_pa = r_buf_size (buf);
 	code_va = code_pa + baddr;
 	phoff = 0x34;//p_phdr ;
 	filesize = code_pa + codelen + datalen;
@@ -120,11 +122,9 @@ RBinPlugin r_bin_plugin_elf = {
 	.desc = "ELF format r2 plugin",
 	.license = "LGPL3",
 	.get_sdb = &get_sdb,
-	.load = &load,
-	.load_bytes = &load_bytes,
 	.load_buffer = &load_buffer,
 	.destroy = &destroy,
-	.check_bytes = &check_bytes,
+	.check_buffer = &check_buffer,
 	.baddr = &baddr,
 	.boffset = &boffset,
 	.binsym = &binsym,
@@ -148,7 +148,7 @@ RBinPlugin r_bin_plugin_elf = {
 	.maps = &maps,
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_BIN,
 	.data = &r_bin_plugin_elf,
