@@ -1,4 +1,4 @@
-/* radare - LGPL - 2018 - a0rtega */
+/* radare - LGPL - 2018-2019 - a0rtega */
 
 #include <r_types.h>
 #include <r_util.h>
@@ -10,31 +10,18 @@
 
 static struct n3ds_firm_hdr loaded_header;
 
-static bool check_bytes(const ut8 *buf, ut64 length) {
-	if (!buf || length < sizeof (struct n3ds_firm_hdr)) {
-		return false;
+static bool check_buffer(RBuffer *b) {
+	ut8 magic[4];
+	r_buf_read_at (b, 0, magic, sizeof (magic));
+	return (!memcmp (magic, "FIRM", 4));
+}
+
+static bool load_buffer(RBinFile *bf, void **bin_obj, RBuffer *b, ut64 loadaddr, Sdb *sdb) {
+	if (r_buf_read_at (b, 0, (ut8*)&loaded_header, sizeof (loaded_header)) == sizeof (loaded_header)) {
+		*bin_obj = &loaded_header;
+		return true;
 	}
-	return (!memcmp (buf, "FIRM", 4));
-}
-
-static bool load_bytes(RBinFile *bf, void **bin_obj, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb) {
-	return memcpy (&loaded_header, buf, sizeof (struct n3ds_firm_hdr));
-}
-
-static bool load(RBinFile *bf) {
-	const ut8 *bytes = bf? r_buf_buffer (bf->buf): NULL;
-	ut64 sz = bf? r_buf_size (bf->buf): 0;
-	if (!bf || !bf->o) {
-		return false;
-	}
-	load_bytes (bf, &bf->o->bin_obj, bytes, sz, bf->o->loadaddr, bf->sdb);
-	return check_bytes (bytes, sz);
-}
-
-static int destroy(RBinFile *bf) {
-	r_buf_free (bf->buf);
-	bf->buf = NULL;
-	return true;
+	return false;
 }
 
 static RList *sections(RBinFile *bf) {
@@ -144,16 +131,14 @@ RBinPlugin r_bin_plugin_nin3ds = {
 	.name = "nin3ds",
 	.desc = "Nintendo 3DS FIRM format r_bin plugin",
 	.license = "LGPL3",
-	.load = &load,
-	.load_bytes = &load_bytes,
-	.destroy = &destroy,
-	.check_bytes = &check_bytes,
+	.load_buffer = &load_buffer,
+	.check_buffer = &check_buffer,
 	.entries = &entries,
 	.sections = &sections,
 	.info = &info,
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_BIN,
 	.data = &r_bin_plugin_nin3ds,
