@@ -47,22 +47,29 @@ static ut64 baddr(RBinFile *bf) {
 	return 0x8000000;
 }
 
-static bool check_bytes(const ut8 *buf, ut64 length) {
-	if (buf && length >= 0x20) {
-		return fileType (buf + NSO_OFF (magic)) != NULL;
+static bool check_buffer(RBuffer *b) {
+	if (r_buf_size (b) >= 0x20) {
+		ut8 magic[4];
+		if (r_buf_read_at (b, 0, magic, sizeof (magic)) != 4) {
+			return false;
+		}
+		return fileType (magic) != NULL;
 	}
 	return false;
 }
 
 static RBinNXOObj *nso_new () {
 	RBinNXOObj *bin = R_NEW0 (RBinNXOObj);
-	bin->methods_list = r_list_newf ((RListFree)free);
-	bin->imports_list = r_list_newf ((RListFree)free);
-	bin->classes_list = r_list_newf ((RListFree)free);
+	if (bin) {
+		bin->methods_list = r_list_newf ((RListFree)free);
+		bin->imports_list = r_list_newf ((RListFree)free);
+		bin->classes_list = r_list_newf ((RListFree)free);
+	}
 	return bin;
 }
 
 static bool load_bytes(RBinFile *bf, void **bin_obj, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb) {
+	eprintf ("load_bytes in bin.nso must die\n");
 	RBin *rbin = bf->rbin;
 	ut32 toff = readLE32 (bf->buf, NSO_OFF (text_memoffset));
 	ut32 tsize = readLE32 (bf->buf, NSO_OFF (text_size));
@@ -73,26 +80,49 @@ static bool load_bytes(RBinFile *bf, void **bin_obj, const ut8 *buf, ut64 sz, ut
 	ut64 total_size = tsize + rosize + dsize;
 	RBuffer *newbuf = r_buf_new_empty (total_size);
 	ut64 ba = baddr (bf);
+	ut8 *tmp = NULL;
 
 	if (rbin->iob.io && !(rbin->iob.io->cached & R_PERM_W)) {
 		eprintf ("Please add \'-e io.cache=true\' option to r2 command. This is required to decompress the code.\n");
 		goto fail;
 	}
 	/* Decompress each sections */
-	if (decompress (buf + toff, r_buf_get_at (newbuf, 0, NULL), rooff - toff, tsize) != tsize) {
+	tmp = R_NEWS (ut8, tsize);
+	if (!tmp) {
+		goto fail;
+	}
+	if (decompress (buf + toff, tmp, rooff - toff, tsize) != tsize) {
 		eprintf ("decompression failure\n");
 		goto fail;
 	}
-	if (decompress (buf + rooff, r_buf_get_at (newbuf, tsize, NULL), doff - rooff, rosize) != rosize) {
+	r_buf_write_at (newbuf, 0, tmp, tsize);
+	R_FREE (tmp);
+
+	tmp = R_NEWS (ut8, rosize);
+	if (!tmp) {
+		goto fail;
+	}
+	if (decompress (buf + rooff, tmp, doff - rooff, rosize) != rosize) {
 		eprintf ("decompression2 failure\n");
 		goto fail;
 	}
-	if (decompress (buf + doff, r_buf_get_at (newbuf, tsize + rosize, NULL), r_buf_size (bf->buf) - doff, dsize) != dsize) {
+	r_buf_write_at (newbuf, tsize, tmp, rosize);
+	R_FREE (tmp);
+
+	tmp = R_NEWS (ut8, dsize);
+	if (!tmp) {
+		goto fail;
+	}
+	if (decompress (buf + doff, tmp, r_buf_size (bf->buf) - doff, dsize) != dsize) {
 		eprintf ("decompression3 failure\n");
 		goto fail;
 	}
+	r_buf_write_at (newbuf, tsize + rosize, tmp, dsize);
+	R_FREE (tmp);
+
 	/* Load unpacked binary */
-	r_io_write_at (rbin->iob.io, ba, r_buf_get_at (newbuf, 0, NULL), total_size);
+	const ut8 *tmpbuf = r_buf_data (newbuf, &total_size);
+	r_io_write_at (rbin->iob.io, ba, tmpbuf, total_size);
 	ut32 modoff = readLE32 (newbuf, NSO_OFFSET_MODMEMOFF);
 	RBinNXOObj *bin = nso_new ();
 	eprintf ("MOD Offset = 0x%"PFMT64x"\n", (ut64)modoff);
@@ -101,28 +131,18 @@ static bool load_bytes(RBinFile *bf, void **bin_obj, const ut8 *buf, ut64 sz, ut
 	*bin_obj = bin;
 	return true;
 fail:
+	free (tmp);
 	r_buf_free (newbuf);
 	*bin_obj = NULL;
 	return false;
 }
 
-static void *load_buffer(RBinFile *bf, RBuffer *buf, ut64 loadaddr, Sdb *sdb) {
+static bool load_buffer(RBinFile *bf, void **bin_obj, RBuffer *buf, ut64 loadaddr, Sdb *sdb) {
 	r_return_val_if_fail (bf && buf, NULL);
-	const ut64 sz = r_buf_size (buf);
-	ut8 *bytes = malloc (sz);
-	if (!bytes) {
-		return NULL;
-	}
-	r_buf_read_at (buf, 0, bytes, sz);
-	void *ptr = NULL;
 	const ut64 la = bf->loadaddr;
-	(void)load_bytes (bf, &ptr, bytes, sz, la, bf->sdb);
-	free (bytes);
-	return ptr;
-}
-
-static int destroy(RBinFile *bf) {
-	return true;
+	ut64 sz = 0;
+	const ut8 *bytes = r_buf_data (buf, &sz);
+	return load_bytes (bf, bin_obj, bytes, sz, la, bf->sdb);
 }
 
 static RBinAddr *binsym(RBinFile *bf, int type) {
@@ -231,7 +251,13 @@ static RBinInfo *info(RBinFile *bf) {
 	if (!ret) {
 		return NULL;
 	}
-	const char *ft = fileType (r_buf_get_at (bf->buf, NSO_OFF (magic), NULL));
+	ut8 magic[4];
+	if (r_buf_read_at (bf->buf, NSO_OFF (magic), magic, sizeof (magic)) != sizeof (magic)) {
+		free (ret);
+		return NULL;
+	}
+
+	const char *ft = fileType (magic);
 	if (!ft) {
 		ft = "nso";
 	}
@@ -258,8 +284,7 @@ RBinPlugin r_bin_plugin_nso = {
 	.desc = "Nintendo Switch NSO0 binaries",
 	.license = "MIT",
 	.load_buffer = &load_buffer,
-	.destroy = &destroy,
-	.check_bytes = &check_bytes,
+	.check_buffer = &check_buffer,
 	.baddr = &baddr,
 	.binsym = &binsym,
 	.entries = &entries,
@@ -268,7 +293,7 @@ RBinPlugin r_bin_plugin_nso = {
 	.info = &info,
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_BIN,
 	.data = &r_bin_plugin_nso,
