@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2013-2015 - xvilka */
+/* radare - LGPL - Copyright 2013-2019 - xvilka */
 
 #include <r_types.h>
 #include <r_util.h>
@@ -16,39 +16,24 @@ static Sdb *get_sdb(RBinFile *bf) {
 	return bin? bin->kv: NULL;
 }
 
-static void *load_bytes(RBinFile *bf, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb){
-	struct r_bin_te_obj_t *res = NULL;
-	RBuffer *tbuf = NULL;
-
-	if (!buf || sz == 0 || sz == UT64_MAX) {
-		return NULL;
+static bool load_buffer(RBinFile *bf, void **bin_obj, RBuffer *b, ut64 loadaddr, Sdb *sdb) {
+	r_return_val_if_fail (bf && bin_obj && b, false);
+	ut64 sz = r_buf_size (b);
+	if (sz == 0 || sz == UT64_MAX) {
+		return false;
 	}
-	tbuf = r_buf_new ();
-	r_buf_set_bytes (tbuf, buf, sz);
-	res = r_bin_te_new_buf (tbuf);
+	struct r_bin_te_obj_t *res = r_bin_te_new_buf (b);
 	if (res) {
 		sdb_ns_set (sdb, "info", res->kv);
 	}
-	r_buf_free (tbuf);
-	return res;
-}
-
-static bool load(RBinFile *bf) {
-	const ut8 *bytes = bf? r_buf_buffer (bf->buf): NULL;
-	ut64 sz = bf? r_buf_size (bf->buf): 0;
-
-	if (!bf || !bf->o) {
-		return false;
-	}
-	bf->o->bin_obj = load_bytes (bf, bytes, sz, bf->o->loadaddr, bf->sdb);
-	return bf->o->bin_obj? true: false;
-}
-
-static int destroy(RBinFile *bf) {
-	r_bin_te_free ((struct r_bin_te_obj_t *) bf->o->bin_obj);
+	*bin_obj = res;
 	return true;
 }
 
+static void destroy(RBinFile *bf) {
+	r_bin_te_free ((struct r_bin_te_obj_t *) bf->o->bin_obj);
+}
+
 static ut64 baddr(RBinFile *bf) {
 	return r_bin_te_get_image_base (bf->o->bin_obj);
 }
@@ -67,23 +52,19 @@ static RBinAddr *binsym(RBinFile *bf, int type) {
 }
 
 static RList *entries(RBinFile *bf) {
-	RList *ret;
-	RBinAddr *ptr = NULL;
-	RBinAddr *entry = NULL;
-
-	if (!(ret = r_list_new ())) {
-		return NULL;
+	RList *ret = r_list_newf (free);
+	if (ret) {
+		RBinAddr *entry = r_bin_te_get_entrypoint (bf->o->bin_obj);
+		if (entry) {
+			RBinAddr *ptr = R_NEW0 (RBinAddr);
+			if (ptr) {
+				ptr->paddr = entry->paddr;
+				ptr->vaddr = entry->vaddr;
+				r_list_append (ret, ptr);
+			}
+			free (entry);
+		}
 	}
-	ret->free = free;
-	if (!(entry = r_bin_te_get_entrypoint (bf->o->bin_obj))) {
-		return ret;
-	}
-	if ((ptr = R_NEW (RBinAddr))) {
-		ptr->paddr = entry->paddr;
-		ptr->vaddr = entry->vaddr;
-		r_list_append (ret, ptr);
-	}
-	free (entry);
 	return ret;
 }
 
@@ -105,36 +86,29 @@ static RList *sections(RBinFile *bf) {
 		if (!(ptr = R_NEW0 (RBinSection))) {
 			break;
 		}
-		if (sections[i].name[sizeof (sections[i].name) - 1]) {
-			memcpy (ptr->name, sections[i].name,
-				sizeof (sections[i].name));
-			ptr->name[sizeof (sections[i].name)] = 0;
-		} else {
-			strncpy (ptr->name, (char *) sections[i].name,
-				R_BIN_SIZEOF_STRINGS);
-		}
+		ptr->name = strdup ((char*)sections[i].name);
 		ptr->size = sections[i].size;
 		ptr->vsize = sections[i].vsize;
 		ptr->paddr = sections[i].paddr;
 		ptr->vaddr = sections[i].vaddr;
-		ptr->srwx = 0;
+		ptr->perm = 0;
 		ptr->add = true;
 		if (R_BIN_TE_SCN_IS_EXECUTABLE (sections[i].flags)) {
-			ptr->srwx |= R_BIN_SCN_EXECUTABLE;
+			ptr->perm |= R_PERM_X;
 		}
 		if (R_BIN_TE_SCN_IS_WRITABLE (sections[i].flags)) {
-			ptr->srwx |= R_BIN_SCN_WRITABLE;
+			ptr->perm |= R_PERM_W;
 		}
 		if (R_BIN_TE_SCN_IS_READABLE (sections[i].flags)) {
-			ptr->srwx |= R_BIN_SCN_READABLE;
+			ptr->perm |= R_PERM_R;
 		}
 		if (R_BIN_TE_SCN_IS_SHAREABLE (sections[i].flags)) {
-			ptr->srwx |= R_BIN_SCN_SHAREABLE;
+			ptr->perm |= R_PERM_SHAR;
 		}
 		/* All TE files have _TEXT_RE section, which is 16-bit, because of
 		 * CPU start in this mode */
 		if (!strncmp (ptr->name, "_TEXT_RE", 8)) {
-			ptr->bits = 16;
+			ptr->bits = R_SYS_BITS_16;
 		}
 		r_list_append (ret, ptr);
 	}
@@ -165,8 +139,12 @@ static RBinInfo *info(RBinFile *bf) {
 	return ret;
 }
 
-static bool check_bytes(const ut8 *buf, ut64 length) {
-	return (buf && length > 2 && !memcmp (buf, "\x56\x5a", 2));
+static bool check_buffer(RBuffer *b) {
+	ut8 buf[2];
+	if (r_buf_read_at (b, 0, buf, 2) == 2) {
+		return !memcmp (buf, "\x56\x5a", 2);
+	}
+	return false;
 }
 
 RBinPlugin r_bin_plugin_te = {
@@ -174,10 +152,9 @@ RBinPlugin r_bin_plugin_te = {
 	.desc = "TE bin plugin", // Terse Executable format
 	.license = "LGPL3",
 	.get_sdb = &get_sdb,
-	.load = &load,
-	.load_bytes = &load_bytes,
+	.load_buffer = &load_buffer,
 	.destroy = &destroy,
-	.check_bytes = &check_bytes,
+	.check_buffer = &check_buffer,
 	.baddr = &baddr,
 	.binsym = &binsym,
 	.entries = &entries,
@@ -186,8 +163,8 @@ RBinPlugin r_bin_plugin_te = {
 	.minstrlen = 4,
 };
 
-#ifndef CORELIB
-RLibStruct radare_plugin = {
+#ifndef R2_PLUGIN_INCORE
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_BIN,
 	.data = &r_bin_plugin_te,
 	.version = R2_VERSION
