@@ -472,7 +472,8 @@ static bool pdb7_parse(R_PDB *pdb) {
 	}
 	p_tmp = root_page_data;
 	for (i = 0; i < num_root_index_pages; i++) {
-		r_buf_seek (pdb->buf, root_index_pages[i] * page_size, 0);
+		r_buf_seek (pdb->buf, root_index_pages[i] * page_size,
+			   R_BUF_SET);
 		r_buf_read (pdb->buf, p_tmp, page_size);
 		p_tmp = (char *) p_tmp + page_size;
 	}
@@ -552,17 +553,17 @@ static void finish_pdb_parse(R_PDB *pdb) {
 		switch (i) {
 		case 1:
 			pdb_info_stream = (SPDBInfoStream *) r_list_iter_get (it);
-			pdb_info_stream->free_(pdb_info_stream);
+			free_pdb_stream (pdb_info_stream);
 			free (pdb_info_stream);
 			break;
 		case 2:
 			tpi_stream = (STpiStream *) r_list_iter_get (it);
-			tpi_stream->free_(tpi_stream);
+			free_pdb_stream (tpi_stream);
 			free (tpi_stream);
 			break;
 		case 3:
 			dbi_stream = (SDbiStream *) r_list_iter_get (it);
-			dbi_stream->free_(dbi_stream);
+			free_pdb_stream (dbi_stream);
 			free (dbi_stream);
 			break;
 		default:
@@ -571,7 +572,7 @@ static void finish_pdb_parse(R_PDB *pdb) {
 				break;
 			}
 			pdb_stream = (R_PDB_STREAM *) r_list_iter_get (it);
-			pdb_stream->free_(pdb_stream);
+			free_pdb_stream (pdb_stream);
 			free (pdb_stream);
 			break;
 		}
@@ -796,11 +797,11 @@ void build_command_field(ELeafType lt, char **command_field) {
 	switch (lt) {
 	case eLF_STRUCTURE:
 	case eLF_UNION:
-		*command_field = (char *) malloc (strlen ("pf.") + 1);
+		*command_field = (char *) malloc (strlen ("\"pf.") + 1);
 		if (!(*command_field)) {
 			break;
 		}
-		strcpy (*command_field, "pf.");
+		strcpy (*command_field, "\"pf.");
 		break;
 	case eLF_ENUM:
 		*command_field = (char *) malloc (strlen ("\"td enum ") + 1);
@@ -1020,6 +1021,7 @@ static void print_types(R_PDB *pdb, int mode) {
 			}
 
 			if (mode == 'r') {
+				r_name_filter (name_field, -1);
 				pdb->cb_printf ("%s%s ", command_field, name_field);
 				if (lt != eLF_ENUM) {
 					pdb->cb_printf ("%s ", flags_format_field);
@@ -1028,6 +1030,11 @@ static void print_types(R_PDB *pdb, int mode) {
 				}
 				sym = (lt == eLF_ENUM)? ',': ' ';
 				for (i = 0; i < members_amount; i++) {
+					char *eq = (lt == eLF_ENUM) ? strchr (members_name_field[i], '=') : NULL;
+					r_name_filter (members_name_field[i], -1);
+					if (eq) {
+						*eq = '=';
+					}
 					pdb->cb_printf ("%s", members_name_field[i]);
 					if ((i + 1) != members_amount) {
 						pdb->cb_printf ("%c", sym);
@@ -1036,7 +1043,7 @@ static void print_types(R_PDB *pdb, int mode) {
 				if (lt == eLF_ENUM) {
 					pdb->cb_printf (" };\"\n");
 				} else {
-					pdb->cb_printf ("\n");
+					pdb->cb_printf ("\"\n");
 				}
 			}
 			if (mode == 'j') {
@@ -1198,7 +1205,7 @@ R_API bool init_pdb_parser(R_PDB *pdb, const char *filename) {
 		goto error;
 	}
 
-	r_buf_seek (pdb->buf, 0, 0);
+	r_buf_seek (pdb->buf, 0, R_BUF_SET);
 
 	if (!memcmp (signature, PDB7_SIGNATURE, PDB7_SIGNATURE_LEN)) {
 		pdb->pdb_parse = pdb7_parse;
