@@ -35,9 +35,9 @@ static void stream_file_read_pages(R_STREAM_FILE *stream_file, int start_indx, i
 		if (page_offset < 1) {
 			return;
 		}
-		stream_file->buf->cur = page_offset;
+		r_buf_seek (stream_file->buf, page_offset, R_BUF_SET);
 		r_buf_read_at (stream_file->buf, page_offset,
-			(ut8*)res, stream_file->page_size);
+			(ut8 *)res, stream_file->page_size);
 		res += stream_file->page_size;
 	}
 }
