@@ -37,7 +37,7 @@ struct r_bp_plugin_t r_bp_plugin_arm = {
 	.bps = r_bp_plugin_arm_bps,
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_BP,
 	.data = &r_bp_plugin_arm,
