@@ -1,21 +1,20 @@
-/* radare - LGPL - Copyright 2006-2011 pancake<nopcode.org> */
+/* radare - LGPL - Copyright 2006-2019 - pancake */
 
 #include "r_config.h"
 
-// XXX spaguetti code lives here
-static int r_config_setter_q(void *data) {
+static bool r_config_setter_q(void *data) {
 	RConfigNode *node = data;
 	*(node->cb_ptr_q) = node->i_value;
 	return true;
 }
 
-static int r_config_setter_i(void *data) {
+static bool r_config_setter_i(void *data) {
 	RConfigNode *node = data;
 	*(node->cb_ptr_i) = node->i_value;
 	return true;
 }
 
-static int r_config_setter_s(void *data) {
+static bool r_config_setter_s(void *data) {
 	RConfigNode *node = data;
 	if (!node->value || !*node->value) {
 		free (*node->cb_ptr_s);
@@ -26,7 +25,7 @@ static int r_config_setter_s(void *data) {
 	return true;
 }
 
-R_API int r_config_set_setter_q(RConfig *cfg, const char *name, ut64 *ptr) {
+R_API bool r_config_set_setter_q(RConfig *cfg, const char *name, ut64 *ptr) {
 	RConfigNode *node = r_config_node_get (cfg, name);
 	if (node) {
 		node->cb_ptr_q = ptr;
@@ -36,7 +35,7 @@ R_API int r_config_set_setter_q(RConfig *cfg, const char *name, ut64 *ptr) {
 	return false;
 }
 
-R_API int r_config_set_setter_i(RConfig *cfg, const char *name, int *ptr) {
+R_API bool r_config_set_setter_i(RConfig *cfg, const char *name, int *ptr) {
 	RConfigNode *node = r_config_node_get (cfg, name);
 	if (node) {
 		node->cb_ptr_i = ptr;
@@ -46,7 +45,7 @@ R_API int r_config_set_setter_i(RConfig *cfg, const char *name, int *ptr) {
 	return false;
 }
 
-R_API int r_config_set_setter_s(RConfig *cfg, const char *name, char * *ptr) {
+R_API bool r_config_set_setter_s(RConfig *cfg, const char *name, char * *ptr) {
 	RConfigNode *node = r_config_node_get (cfg, name);
 	if (node) {
 		node->cb_ptr_s = ptr;
