@@ -1,7 +1,6 @@
-/* radare - LGPL - Copyright 2006-2018 - pancake */
+/* radare - LGPL - Copyright 2006-2019 - pancake */
 
 #include "r_config.h"
-#include "r_util.h" // r_str_hash, r_str_chop, ...
 
 R_API RConfigNode* r_config_node_new(const char *name, const char *value) {
 	if (IS_NULLSTR (name)) {
@@ -46,31 +45,18 @@ R_API void r_config_node_free(void *n) {
 	free (node);
 }
 
-static bool isBoolean(const char *val) {
-	if (!r_str_casecmp (val, "true") || !r_str_casecmp (val, "false")) {
-		return true;
-	}
-	if (!r_str_casecmp (val, "on") || !r_str_casecmp (val, "off")) {
-		return true;
-	}
-	if (!r_str_casecmp (val, "yes") || !r_str_casecmp (val, "no")) {
-		return true;
-	}
-	return false;
-}
-
 static void config_print_value_json(RConfig *cfg, RConfigNode *node) {
 	const char *val = node->value;
 	if (!val) {
 		val = "0";
 	}
 	char *sval = r_str_escape (val);
-	if (node->flags & CN_BOOL || node->flags & CN_INT || node->flags & CN_OFFT) {
+	if (r_config_node_is_bool (node) || r_config_node_is_int (node)) {
 		if (!strncmp (val, "0x", 2)) {
 			ut64 n = r_num_get (NULL, val);
 			cfg->cb_printf ("%"PFMT64d, n);
-		} else if (r_str_isnumber (val) || isBoolean (val)) {
-			cfg->cb_printf ("%s", val);
+		} else if (r_str_isnumber (val) || (*val /* HACK */ && r_str_is_bool (val))) {
+			cfg->cb_printf ("%s", val);  // TODO: always use true/false for bool json str
 		} else {
 			cfg->cb_printf ("\"%s\"", sval);
 		}
@@ -98,7 +84,7 @@ static void config_print_node(RConfig *cfg, RConfigNode *node, const char *pfx,
 				cfg->cb_printf ("\"desc\":\"%s\",", es);
 				free (es);
 			}
-			cfg->cb_printf ("\"ro\":%s", node->flags & CN_RO ? "true" : "false");
+			cfg->cb_printf ("\"ro\":%s", r_str_bool (r_config_node_is_ro (node)));
 			if (!r_list_empty (node->options)) {
 				isFirst = true;
 				cfg->cb_printf (",\"options\":[");
@@ -125,7 +111,7 @@ static void config_print_node(RConfig *cfg, RConfigNode *node, const char *pfx,
 		if (verbose) {
 			cfg->cb_printf ("%s%s = %s%s %s; %s", pfx,
 				node->name, node->value, sfx, 
-				node->flags & CN_RO ? "(ro)" : "", 
+				r_config_node_is_ro (node) ? "(ro)" : "",
 				node->desc);
 			if (!r_list_empty (node->options)) {
 				isFirst = true;
@@ -134,11 +120,11 @@ static void config_print_node(RConfig *cfg, RConfigNode *node, const char *pfx,
 					if (isFirst) {
 						isFirst = false;
 					} else {
-						cfg->cb_printf(", ");
+						cfg->cb_printf (", ");
 					}
-					cfg->cb_printf("%s", option);
+					cfg->cb_printf ("%s", option);
 				}
-				cfg->cb_printf("]");
+				cfg->cb_printf ("]");
 			}
 			cfg->cb_printf ("\n");
 		} else {
@@ -252,7 +238,6 @@ R_API void r_config_list(RConfig *cfg, const char *str, int rad) {
 		verbose = true;
 	/* fallthrou */
 	case 'j':
-		json = true;
 		isFirst = true;
 		if (verbose) {
 			cfg->cb_printf ("[");
@@ -267,7 +252,7 @@ R_API void r_config_list(RConfig *cfg, const char *str, int rad) {
 					} else {
 						cfg->cb_printf (",");
 					}
-					config_print_node (cfg, node, pfx, sfx, verbose, json);
+					config_print_node (cfg, node, pfx, sfx, verbose, true);
 				}
 			}
 		}
@@ -281,62 +266,53 @@ R_API void r_config_list(RConfig *cfg, const char *str, int rad) {
 }
 
 R_API RConfigNode* r_config_node_get(RConfig *cfg, const char *name) {
-	if (!cfg || IS_NULLSTR (name)) {
-		return NULL;
-	}
+	r_return_val_if_fail (cfg && name, NULL);
 	return ht_pp_find (cfg->ht, name, NULL);
 }
 
-R_API int r_config_set_getter(RConfig *cfg, const char *key, RConfigCallback cb) {
+R_API bool r_config_set_getter(RConfig *cfg, const char *key, RConfigCallback cb) {
 	RConfigNode *node = r_config_node_get (cfg, key);
 	if (node) {
 		node->getter = cb;
-		return 1;
+		return true;
 	}
-	return 0;
+	return false;
 }
 
-R_API int r_config_set_setter(RConfig *cfg, const char *key, RConfigCallback cb) {
+R_API bool r_config_set_setter(RConfig *cfg, const char *key, RConfigCallback cb) {
 	RConfigNode *node = r_config_node_get (cfg, key);
 	if (node) {
 		node->setter = cb;
-		return 1;
+		return true;
 	}
-	return 0;
+	return false;
 }
 
-static bool is_true(const char *s) {
-	return !r_str_casecmp ("yes", s) || !r_str_casecmp ("on", s) || !r_str_casecmp ("true", s) || !r_str_casecmp ("1", s);
-}
 
 static bool is_bool(const char *s) {
 	return !r_str_casecmp ("true", s) || !r_str_casecmp ("false", s);
 }
 
 R_API const char* r_config_get(RConfig *cfg, const char *name) {
-	if (!cfg || !name) {
-		return NULL;
-	}
+	r_return_val_if_fail (cfg && name, NULL);
 	RConfigNode *node = r_config_node_get (cfg, name);
 	if (node) {
 		if (node->getter) {
 			node->getter (cfg->user, node);
 		}
-		cfg->last_notfound = 0;
-		if (node->flags & CN_BOOL) {
-			return r_str_bool (is_true (node->value));
+		if (r_config_node_is_bool (node)) {
+			return r_str_bool (r_str_is_true (node->value));
 		}
 		return node->value;
 	} else {
 		eprintf ("r_config_get: variable '%s' not found\n", name);
 	}
-	cfg->last_notfound = 1;
 	return NULL;
 }
 
-R_API int r_config_toggle(RConfig *cfg, const char *name) {
+R_API bool r_config_toggle(RConfig *cfg, const char *name) {
 	RConfigNode *node = r_config_node_get (cfg, name);
-	if (node && node->flags & CN_BOOL) {
+	if (node && r_config_node_is_bool (node)) {
 		(void)r_config_set_i (cfg, name, !node->i_value);
 		return true;
 	}
@@ -361,20 +337,19 @@ R_API ut64 r_config_get_i(RConfig *cfg, const char *name) {
 }
 
 R_API const char* r_config_node_type(RConfigNode *node) {
-	if (node) {
-		int f = node->flags;
-		if (f & CN_BOOL) {
-			return "bool";
-		}
-		if (f & CN_STR) {
-			return "str";
-		}
-		if (f & CN_OFFT || f & CN_INT) {
-			if (!strncmp (node->value, "0x", 2)) {
-				return "addr";
-			}
-			return "int";
+	r_return_val_if_fail (node, "");
+
+	if (r_config_node_is_bool (node)) {
+		return "bool";
+	}
+	if (r_config_node_is_str (node)) {
+		return "str";
+	}
+	if (r_config_node_is_int (node)) {
+		if (!strncmp (node->value, "0x", 2)) {
+			return "addr";
 		}
+		return "int";
 	}
 	return "";
 }
@@ -410,7 +385,7 @@ R_API RConfigNode* r_config_set(RConfig *cfg, const char *name, const char *valu
 
 	node = r_config_node_get (cfg, name);
 	if (node) {
-		if (node->flags & CN_RO) {
+		if (r_config_node_is_ro (node)) {
 			eprintf ("(error: '%s' config key is read only)\n", name);
 			return node;
 		}
@@ -424,8 +399,8 @@ R_API RConfigNode* r_config_set(RConfig *cfg, const char *name, const char *valu
 			free (node->value);
 			node->value = strdup ("");
 		}
-		if (node->flags & CN_BOOL) {
-			bool b = is_true (value);
+		if (r_config_node_is_bool (node)) {
+			bool b = r_str_is_true (value);
 			node->i_value = b? 1: 0;
 			char *value = strdup (r_str_bool (b));
 			if (value) {
@@ -441,10 +416,9 @@ R_API RConfigNode* r_config_set(RConfig *cfg, const char *name, const char *valu
 				if (node->value == value) {
 					goto beach;
 				}
-				char *tmp = node->value;
+				free (node->value);
 				node->value = strdup (value);
-				free (tmp);
-				if (IS_DIGIT (*value)) {
+				if (IS_DIGIT (*value) || (value[0] == '-' && IS_DIGIT (value[1]))) {
 					if (strchr (value, '/')) {
 						node->i_value = r_num_get (cfg->num, value);
 					} else {
@@ -463,11 +437,10 @@ R_API RConfigNode* r_config_set(RConfig *cfg, const char *name, const char *valu
 			if (node) {
 				if (value && is_bool (value)) {
 					node->flags |= CN_BOOL;
-					node->i_value = is_true (value)? 1: 0;
+					node->i_value = r_str_is_true (value)? 1: 0;
 				}
 				ht_pp_insert (cfg->ht, node->name, node);
 				r_list_append (cfg->nodes, node);
-				cfg->n_nodes++;
 			} else {
 				eprintf ("r_config_set: unable to create a new RConfigNode\n");
 			}
@@ -477,14 +450,13 @@ R_API RConfigNode* r_config_set(RConfig *cfg, const char *name, const char *valu
 	}
 
 	if (node && node->setter) {
-		int ret = node->setter (cfg->user, node);
-		if (ret == false) {
+		if (!node->setter (cfg->user, node)) {
 			if (oi != UT64_MAX) {
 				node->i_value = oi;
 			}
 			free (node->value);
 			node->value = strdup (ov? ov: "");
-free (ov);
+			free (ov);
 			return NULL;
 		}
 	}
@@ -502,76 +474,66 @@ R_API const char* r_config_desc(RConfig *cfg, const char *name, const char *desc
 }
 
 R_API const char* r_config_node_desc(RConfigNode *node, const char *desc) {
-	if (node) {
-		if (desc) {
-			free (node->desc);
-			node->desc = strdup (desc);
-		}
-		return node->desc;
+	r_return_val_if_fail (node, NULL);
+	if (desc) {
+		free (node->desc);
+		node->desc = strdup (desc);
 	}
-	return NULL;
+	return node->desc;
 }
 
-R_API int r_config_rm(RConfig *cfg, const char *name) {
+R_API bool r_config_rm(RConfig *cfg, const char *name) {
 	RConfigNode *node = r_config_node_get (cfg, name);
 	if (node) {
 		ht_pp_delete (cfg->ht, node->name);
 		r_list_delete_data (cfg->nodes, node);
-		cfg->n_nodes--;
 		return true;
 	}
 	return false;
 }
 
+R_API void r_config_node_value_format_i(char *buf, size_t buf_size, const ut64 i, R_NULLABLE RConfigNode *node) {
+	if (node && r_config_node_is_bool (node)) {
+		r_str_ncpy (buf, r_str_bool ((int) i), buf_size);
+		return;
+	}
+	if (i < 1024) {
+		snprintf (buf, buf_size, "%" PFMT64d "", i);
+	} else {
+		snprintf (buf, buf_size, "0x%08" PFMT64x "", i);
+	}
+}
+
 R_API RConfigNode* r_config_set_i(RConfig *cfg, const char *name, const ut64 i) {
 	char buf[128], *ov = NULL;
-	if (!cfg || !name) {
-		return NULL;
-	}
+	r_return_val_if_fail (cfg && name, NULL);
 	RConfigNode *node = r_config_node_get (cfg, name);
 	if (node) {
-		if (node->flags & CN_RO) {
+		if (r_config_node_is_ro (node)) {
 			node = NULL;
 			goto beach;
 		}
-		if (node->value) {
-			ov = strdup (node->value);
-			if (!ov) {
-				node = NULL;
-				goto beach;
-			}
-			free (node->value);
-		}
-		if (node->flags & CN_BOOL) {
-			node->value = strdup (r_str_bool (i));
-		} else {
-			snprintf (buf, sizeof (buf) - 1, "%" PFMT64d, i);
-			node->value = strdup (buf);
-		}
+		ov = node->value;
+		r_config_node_value_format_i (buf, sizeof (buf), i, NULL);
+		node->value = strdup (buf);
 		if (!node->value) {
 			node = NULL;
 			goto beach;
 		}
-		//node->flags = CN_RW | CN_INT;
 		node->i_value = i;
 	} else {
 		if (!cfg->lock) {
-			if (i < 1024) {
-				snprintf (buf, sizeof (buf), "%" PFMT64d "", i);
-			} else {
-				snprintf (buf, sizeof (buf), "0x%08" PFMT64x "", i);
-			}
+			r_config_node_value_format_i (buf, sizeof (buf), i, NULL);
 			node = r_config_node_new (name, buf);
 			if (!node) {
 				node = NULL;
 				goto beach;
 			}
-			node->flags = CN_RW | CN_OFFT;
+			node->flags = CN_RW | CN_INT;
 			node->i_value = i;
 			ht_pp_insert (cfg->ht, node->name, node);
 			if (cfg->nodes) {
 				r_list_append (cfg->nodes, node);
-				cfg->n_nodes++;
 			}
 		} else {
 			eprintf ("(locked: no new keys can be created (%s))\n", name);
@@ -584,7 +546,8 @@ R_API RConfigNode* r_config_set_i(RConfig *cfg, const char *name, const ut64 i)
 		if (!ret) {
 			node->i_value = oi;
 			free (node->value);
-			node->value = strdup (ov? ov: "");
+			node->value = ov? ov: strdup ("");
+			ov = NULL;
 		}
 	}
 beach:
@@ -592,54 +555,62 @@ beach:
 	return node;
 }
 
-R_API int r_config_eval(RConfig *cfg, const char *str) {
-	char *ptr, *a, *b, name[1024];
-	unsigned int len;
-	if (!str || !cfg) {
-		return false;
+static void __evalString(RConfig *cfg, char *name) {
+	if (!*name) {
+		return;
 	}
-	len = strlen (str) + 1;
-	if (len >= sizeof (name)) {
-		return false;
-	}
-	memcpy (name, str, len);
-	str = r_str_trim (name);
-
-	if (!str) {
-		return false;
-	}
-
-	if (str[0] == '\0' || !strcmp (str, "help")) {
-		r_config_list (cfg, NULL, 0);
-		return false;
-	}
-
-	if (str[0] == '-') {
-		r_config_rm (cfg, str + 1);
-		return false;
-	}
-
-	ptr = strchr (str, '=');
-	if (ptr) {
-		/* set */
-		ptr[0] = '\0';
-		a = r_str_trim (name);
-		b = r_str_trim (ptr + 1);
-		(void) r_config_set (cfg, a, b);
+	char *eq = strchr (name, '=');
+	if (eq) {
+		*eq++ = 0;
+		r_str_trim (name);
+		r_str_trim (eq);
+		if (*name) {
+			(void) r_config_set (cfg, name, eq);
+		}
 	} else {
-		char *foo = r_str_trim (name);
-		if (foo[strlen (foo) - 1] == '.') {
+		if (r_str_endswith (name, ".")) {
 			r_config_list (cfg, name, 0);
-			return false;
 		} else {
-			/* get */
-			const char *str = r_config_get (cfg, foo);
-			if (str) {
-				cfg->cb_printf ("%s\n",
-					(((int) (size_t) str) == 1)? "true": str);
+			const char *v = r_config_get (cfg, name);
+			if (v) {
+				cfg->cb_printf ("%s\n", v);
+			} else {
+				eprintf ("Invalid config key %s\n", name);
 			}
 		}
 	}
+}
+
+R_API bool r_config_eval(RConfig *cfg, const char *str, bool many) {
+	r_return_val_if_fail (cfg && str, false);
+
+	char *s = r_str_trim_dup (str);
+
+	if (!*s || !strcmp (s, "help")) {
+		r_config_list (cfg, NULL, 0);
+		free (s);
+		return false;
+	}
+
+	if (*s == '-') {
+		r_config_rm (cfg, s + 1);
+		free (s);
+		return false;
+	}
+	if (many) {
+		// space separated list of k=v k=v,..
+		// if you want to use spaces go for base64 or e.
+		RList *list = r_str_split_list (s, ",", 0);
+		RListIter *iter;
+		char *name;
+		r_list_foreach (list, iter, name) {
+			__evalString (cfg, name);
+		}
+		free (s);
+		return true;
+	}
+	__evalString (cfg, s);
+	free (s);
 	return true;
 }
 
@@ -652,13 +623,13 @@ R_API void r_config_lock(RConfig *cfg, int l) {
 	cfg->lock = l;
 }
 
-R_API int r_config_readonly(RConfig *cfg, const char *key) {
+R_API bool r_config_readonly(RConfig *cfg, const char *key) {
 	RConfigNode *n = r_config_node_get (cfg, key);
-	if (!n) {
-		return false;
+	if (n) {
+		n->flags |= CN_RO;
+		return true;
 	}
-	n->flags |= CN_RO;
-	return true;
+	return false;
 }
 
 R_API RConfig* r_config_new(void *user) {
@@ -674,7 +645,6 @@ R_API RConfig* r_config_new(void *user) {
 	}
 	cfg->user = user;
 	cfg->num = NULL;
-	cfg->n_nodes = 0;
 	cfg->lock = 0;
 	cfg->cb_printf = (void *) printf;
 	return cfg;
@@ -691,115 +661,55 @@ R_API RConfig* r_config_clone(RConfig *cfg) {
 		RConfigNode *nn = r_config_node_clone (node);
 		ht_pp_insert (c->ht, node->name, nn);
 		r_list_append (c->nodes, nn);
-		c->n_nodes++;
 	}
 	c->lock = cfg->lock;
 	c->cb_printf = cfg->cb_printf;
 	return c;
 }
 
-R_API int r_config_free(RConfig *cfg) {
-	if (!cfg) {
-		return 0;
+R_API void r_config_free(RConfig *cfg) {
+	if (cfg) {
+		cfg->nodes->free = r_config_node_free; // damn
+		r_list_free (cfg->nodes);
+		ht_pp_free (cfg->ht);
+		free (cfg);
 	}
-	cfg->nodes->free = r_config_node_free; // damn
-	r_list_free (cfg->nodes);
-	ht_pp_free (cfg->ht);
-	free (cfg);
-	return 0;
 }
 
 R_API void r_config_visual_hit_i(RConfig *cfg, const char *name, int delta) {
 	RConfigNode *node = r_config_node_get (cfg, name);
-	if (node && (node->flags & CN_INT || node->flags & CN_OFFT)) {
+	if (node && r_config_node_is_int (node)) {
 		(void)r_config_set_i (cfg, name, r_config_get_i (cfg, name) + delta);
 	}
 }
 
 R_API void r_config_bump(RConfig *cfg, const char *key) {
 	char *orig = strdup (r_config_get (cfg, key));
-	r_config_set (cfg, key, orig);
-	free (orig);
+	if (orig) {
+		r_config_set (cfg, key, orig);
+		free (orig);
+	}
 }
 
-R_API bool r_config_save_char(RConfigHold *h, ...) {
-	va_list ap;
-	char *key;
-	if (!h->list_char) {
-		h->list_char = r_list_newf ((RListFree) free);
-		if (!h->list_char) {
-			return false;
-		}
-	}
-	va_start (ap, h);
-	while ((key = va_arg (ap, char *))) {
-		RConfigHoldChar *hc = R_NEW0 (RConfigHoldChar);
-		if (!hc) {
-			continue;
-		}
-		hc->key = key;
-		hc->value = r_config_get (h->cfg, key);
-		r_list_append (h->list_char, hc);
-	}
-	va_end (ap);
-	return true;
-}
-
-R_API bool r_config_save_num(RConfigHold *h, ...) {
-	va_list ap;
-	char *key;
-	if (!h) {
-		return false;
-	}
-	if (!h->list_num) {
-		h->list_num = r_list_newf ((RListFree) free);
-		if (!h->list_num) {
-			return false;
-		}
-	}
-	va_start (ap, h);
-	while ((key = va_arg (ap, char *))) {
-		RConfigHoldNum *hc = R_NEW0 (RConfigHoldNum);
-		if (!hc) {
-			continue;
-		}
-		hc->key = key;
-		hc->value = r_config_get_i (h->cfg, key);
-		r_list_append (h->list_num, hc);
-	}
-	va_end (ap);
-	return true;
-}
-
-R_API RConfigHold* r_config_hold_new(RConfig *cfg) {
-	if (cfg) {
-		RConfigHold *hold = R_NEW0 (RConfigHold);
-		if (hold) {
-			hold->cfg = cfg;
-			return hold;
-		}
-	}
-	return NULL;
-}
-
-R_API void r_config_restore(RConfigHold *h) {
+R_API void r_config_serialize(R_NONNULL RConfig *config, R_NONNULL Sdb *db) {
 	RListIter *iter;
-	RConfigHoldChar *hchar;
-	RConfigHoldNum *hnum;
-	if (h) {
-		r_list_foreach (h->list_num, iter, hnum) {
-			(void)r_config_set_i (h->cfg, hnum->key, hnum->value);
-		}
-		r_list_foreach (h->list_char, iter, hchar) {
-			(void)r_config_set (h->cfg, hchar->key, hchar->value);
-		}
+	RConfigNode *node;
+	r_list_foreach (config->nodes, iter, node) {
+		sdb_set (db, node->name, node->value, 0);
 	}
 }
 
-R_API void r_config_hold_free(RConfigHold *h) {
-	if (h) {
-		r_list_free (h->list_num);
-		r_list_free (h->list_char);
-		R_FREE (h);
+static int load_config_cb(void *user, const char *k, const char *v) {
+	RConfig *config = user;
+	RConfigNode *node = r_config_node_get (config, k);
+	if (!node) {
+		return 1;
 	}
+	r_config_set (config, k, v);
+	return 1;
+}
+
+R_API bool r_config_unserialize(R_NONNULL RConfig *config, R_NONNULL Sdb *db, R_NULLABLE char **err) {
+	sdb_foreach (db, load_config_cb, config);
+	return true;
 }
