@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2008-2018 - pancake, Jody Frankowski */
+/* radare2 - LGPL - Copyright 2008-2019 - pancake, Jody Frankowski */
 
 #include <r_cons.h>
 #include <r_util.h>
@@ -8,9 +8,6 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdarg.h>
-#if __UNIX__ || __CYGWIN__
-#include <signal.h>
-#endif
 
 #define COUNT_LINES 1
 #define CTX(x) I.context->x
@@ -35,6 +32,8 @@ typedef struct {
 	void *event_interrupt_data;
 } RConsBreakStack;
 
+static void cons_grep_reset(RConsGrep *grep);
+
 static void break_stack_free(void *ptr) {
 	RConsBreakStack *b = (RConsBreakStack*)ptr;
 	free (b);
@@ -53,35 +52,30 @@ static void cons_stack_free(void *ptr) {
 
 static RConsStack *cons_stack_dump(bool recreate) {
 	RConsStack *data = R_NEW0 (RConsStack);
-	if (!data) {
-		return NULL;
-	}
-
-	if (CTX (buffer)) {
-		data->buf = CTX (buffer);
-		data->buf_len = CTX (buffer_len);
-		data->buf_size = CTX (buffer_sz);
-	}
-
-	data->grep = R_NEW0 (RConsGrep);
-	if (data->grep) {
-		memcpy (data->grep, &I.context->grep, sizeof (RConsGrep));
-		if (I.context->grep.str) {
-			data->grep->str = strdup (I.context->grep.str);
+	if (data) {
+		if (CTX (buffer)) {
+			data->buf = CTX (buffer);
+			data->buf_len = CTX (buffer_len);
+			data->buf_size = CTX (buffer_sz);
+		}
+		data->grep = R_NEW0 (RConsGrep);
+		if (data->grep) {
+			memcpy (data->grep, &I.context->grep, sizeof (RConsGrep));
+			if (I.context->grep.str) {
+				data->grep->str = strdup (I.context->grep.str);
+			}
+		}
+		if (recreate && I.context->buffer_sz > 0) {
+			I.context->buffer = malloc (I.context->buffer_sz);
+			if (!I.context->buffer) {
+				I.context->buffer = data->buf;
+				free (data);
+				return NULL;
+			}
+		} else {
+			I.context->buffer = NULL;
 		}
 	}
-
-	if (recreate && I.context->buffer_sz > 0) {
-		I.context->buffer = malloc (I.context->buffer_sz);
-		if (!I.context->buffer) {
-			I.context->buffer = data->buf;
-			free (data);
-			return NULL;
-		}
-	} else {
-		I.context->buffer = NULL;
-	}
-
 	return data;
 }
 
@@ -100,36 +94,49 @@ static void cons_stack_load(RConsStack *data, bool free_current) {
 	}
 }
 
-static void cons_context_init(RConsContext *context) {
+static void cons_context_init(RConsContext *context, R_NULLABLE RConsContext *parent) {
 	context->breaked = false;
+	context->cmd_depth = R_CONS_CMD_DEPTH + 1;
 	context->buffer = NULL;
 	context->buffer_sz = 0;
 	context->lastEnabled = true;
 	context->buffer_len = 0;
+	context->is_interactive = false;
 	context->cons_stack = r_stack_newf (6, cons_stack_free);
 	context->break_stack = r_stack_newf (6, break_stack_free);
 	context->event_interrupt = NULL;
 	context->event_interrupt_data = NULL;
 	context->pageable = true;
 	context->log_callback = NULL;
+
+	if (parent) {
+		context->color_mode = parent->color_mode;
+		r_cons_pal_copy (context, parent);
+	} else {
+		context->color_mode = COLOR_MODE_DISABLED;
+		r_cons_pal_init (context);
+	}
+
+	cons_grep_reset (&context->grep);
 }
 
 static void cons_context_deinit(RConsContext *context) {
 	r_stack_free (context->cons_stack);
 	r_stack_free (context->break_stack);
+	r_cons_pal_free (context);
 }
 
-static void break_signal(int sig) {
+static void __break_signal(int sig) {
 	r_cons_context_break (&r_cons_context_default);
 }
 
-static inline void r_cons_write(const char *buf, int len) {
-#if __WINDOWS__ && !__CYGWIN__
+static inline void __cons_write_ll(const char *buf, int len) {
+#if __WINDOWS__
 	if (I.ansicon) {
 		(void) write (I.fdout, buf, len);
 	} else {
 		if (I.fdout == 1) {
-			r_cons_w32_print ((const ut8*)buf, len, 0);
+			r_cons_w32_print (buf, len, false);
 		} else {
 			(void) write (I.fdout, buf, len);
 		}
@@ -142,9 +149,20 @@ static inline void r_cons_write(const char *buf, int len) {
 #endif
 }
 
+static inline void __cons_write(const char *obuf, int olen) {
+	const unsigned int bucket = 64 * 1024;
+	unsigned int i;
+	for (i = 0; (i + bucket) < olen; i += bucket) {
+		__cons_write_ll (obuf + i, bucket);
+	}
+	if (i < olen) {
+		__cons_write_ll (obuf + i, olen - i);
+	}
+}
+
 R_API RColor r_cons_color_random(ut8 alpha) {
 	RColor rcolor = {0};
-	if (I.color > COLOR_MODE_16) {
+	if (I.context->color_mode > COLOR_MODE_16) {
 		rcolor.r = r_num_rand (0xff);
 		rcolor.g = r_num_rand (0xff);
 		rcolor.b = r_num_rand (0xff);
@@ -211,6 +229,46 @@ R_API void r_cons_strcat_justify(const char *str, int j, char c) {
 	}
 }
 
+R_API void r_cons_strcat_at(const char *_str, int x, char y, int w, int h) {
+	int i, o, len;
+	int cols = 0;
+	int rows = 0;
+	if (x < 0 || y < 0) {
+		int H, W = r_cons_get_size (&H);
+		if (x < 0) {
+			x += W;
+		}
+		if (y < 0) {
+			y += H;
+		}
+	}
+	char *str = r_str_ansi_crop (_str, 0, 0, w + 1, h);
+	r_cons_strcat (R_CONS_CURSOR_SAVE);
+	for (o = i = len = 0; str[i]; i++, len++) {
+		if (w < 0 || rows > w) {
+			break;
+		}
+		if (str[i] == '\n') {
+			r_cons_gotoxy (x, y + rows);
+			int ansilen = r_str_ansi_len (str + o);
+			cols = R_MIN (w, ansilen);
+			const char *end = r_str_ansi_chrn (str + o, cols);
+			cols = end - str + o;
+			r_cons_memcat (str + o, R_MIN (len, cols));
+			o = i + 1;
+			len = 0;
+			rows++;
+		}
+	}
+	if (len > 1) {
+		r_cons_gotoxy (x, y + rows);
+		r_cons_memcat (str + o, len);
+	}
+	r_cons_strcat (Color_RESET);
+	r_cons_strcat (R_CONS_CURSOR_RESTORE);
+	free (str);
+}
+
 R_API RCons *r_cons_singleton() {
 	return &I;
 }
@@ -230,9 +288,9 @@ R_API void r_cons_context_break_push(RConsContext *context, RConsBreak cb, void
 		return;
 	}
 	if (r_stack_is_empty (context->break_stack)) {
-#if __UNIX__ || __CYGWIN__
+#if __UNIX__
 		if (sig && r_cons_context_is_main ()) {
-			signal (SIGINT, break_signal);
+			r_sys_signal (SIGINT, __break_signal);
 		}
 #endif
 		context->breaked = false;
@@ -259,9 +317,9 @@ R_API void r_cons_context_break_pop(RConsContext *context, bool sig) {
 		break_stack_free (b);
 	} else {
 		//there is not more elements in the stack
-#if __UNIX__ || __CYGWIN__
+#if __UNIX__
 		if (sig && r_cons_context_is_main ()) {
-			signal (SIGINT, SIG_IGN);
+			r_sys_signal (SIGINT, SIG_IGN);
 		}
 #endif
 		context->breaked = false;
@@ -276,6 +334,14 @@ R_API void r_cons_break_pop() {
 	r_cons_context_break_pop (I.context, true);
 }
 
+R_API bool r_cons_is_interactive() {
+	return I.context->is_interactive;
+}
+
+R_API bool r_cons_default_context_is_interactive(void) {
+	return r_cons_context_default.is_interactive;
+}
+
 R_API bool r_cons_is_breaked() {
 	if (I.cb_break) {
 		I.cb_break (I.user);
@@ -290,28 +356,51 @@ R_API bool r_cons_is_breaked() {
 	return I.context->breaked;
 }
 
-R_API void r_cons_break_timeout(int timeout) {
-	if (!timeout && I.timeout) {
-		I.timeout = 0;
-	} else {
-		if (timeout) {
-			I.timeout = r_sys_now () + (timeout * 1000000);
-		} else {
-			I.timeout = 0;
+R_API int r_cons_get_cur_line() {
+	int curline = 0;
+#if __WINDOWS__
+	POINT point;
+		if (GetCursorPos (&point)) {
+			curline = point.y;
 		}
-	}
+#endif
+#if __UNIX__
+		char buf[8];
+		struct termios save,raw;
+		// flush the Arrow keys escape keys which was messing up the output
+		fflush (stdout);
+		(void) tcgetattr (0, &save);
+		cfmakeraw (&raw);
+		(void) tcsetattr (0, TCSANOW, &raw);
+		if (isatty (fileno (stdin))){
+			write (1, R_CONS_GET_CURSOR_POSITION, sizeof (R_CONS_GET_CURSOR_POSITION));
+			read (0, buf, sizeof (buf));
+			if (isdigit (buf[2])) {
+				curline = (buf[2] - '0');
+			} if (isdigit (buf[3])) {
+				curline = curline * 10 + (buf[3] - '0');
+			}
+		}
+		(void) tcsetattr (0, TCSANOW, &save);
+#endif
+	return curline;
+}
+
+R_API void r_cons_break_timeout(int timeout) {
+	I.timeout = (timeout && !I.timeout) 
+		? r_sys_now () + ((ut64) timeout << 20) : 0;
 }
 
 R_API void r_cons_break_end() {
 	I.context->breaked = false;
 	I.timeout = 0;
-#if __UNIX__ || __CYGWIN__
-	signal (SIGINT, SIG_IGN);
+#if __UNIX__
+	r_sys_signal (SIGINT, SIG_IGN);
 #endif
 	if (!r_stack_is_empty (I.context->break_stack)) {
-		//free all the stack
+		// free all the stack
 		r_stack_free (I.context->break_stack);
-		//create another one
+		// create another one
 		I.context->break_stack = r_stack_newf (6, break_stack_free);
 		I.context->event_interrupt_data = NULL;
 		I.context->event_interrupt = NULL;
@@ -326,43 +415,81 @@ R_API void *r_cons_sleep_begin(void) {
 }
 
 R_API void r_cons_sleep_end(void *user) {
-	if (!I.cb_sleep_end) {
-		return;
+	if (I.cb_sleep_end) {
+		I.cb_sleep_end (I.user, user);
 	}
-	I.cb_sleep_end (I.user, user);
 }
 
-#if __WINDOWS__ && !__CYGWIN__
+#if __WINDOWS__
 static HANDLE h;
 static BOOL __w32_control(DWORD type) {
 	if (type == CTRL_C_EVENT) {
-		break_signal (2); // SIGINT
+		__break_signal (2); // SIGINT
 		eprintf ("{ctrl+c} pressed.\n");
 		return true;
 	}
 	return false;
 }
-#elif __UNIX__ || __CYGWIN__
+#elif __UNIX__
 volatile sig_atomic_t sigwinchFlag;
 static void resize(int sig) {
 	sigwinchFlag = 1;
 }
-
+#endif
 void resizeWin(void) {
 	if (I.event_resize) {
 		I.event_resize (I.event_data);
 	}
 }
-#endif
+
+R_API void r_cons_set_click(int x, int y) {
+	I.click_x = x;
+	I.click_y = y;
+	I.click_set = true;
+	I.mouse_event = 1;
+}
+
+R_API bool r_cons_get_click(int *x, int *y) {
+	if (x) {
+		*x = I.click_x;
+	}
+	if (y) {
+		*y = I.click_y;
+	}
+	bool set = I.click_set;
+	I.click_set = false;
+	return set;
+}
+
+R_API void r_cons_enable_highlight(const bool enable) {
+        I.enable_highlight = enable;
+}
 
 R_API bool r_cons_enable_mouse(const bool enable) {
-#if __UNIX__ || __CYGWIN__
-	const char *code = enable
-		? "\x1b[?1001s" "\x1b[?1000h"
+#if __UNIX__
+	const char *click = enable
+		? "\x1b[?1000;1006;1015h"
 		: "\x1b[?1001r" "\x1b[?1000l";
+		// : "\x1b[?1000;1006;1015l";
+	// const char *old = enable ? "\x1b[?1001s" "\x1b[?1000h" : "\x1b[?1001r" "\x1b[?1000l";
 	bool enabled = I.mouse;
 	I.mouse = enable;
-	write (2, code, 16);
+	write (2, click, strlen (click));
+	return enabled;
+#elif __WINDOWS__
+	DWORD mode, mouse;
+	HANDLE h;
+	bool enabled = I.mouse;
+	if (enabled == enable) {
+		return enabled;
+	}
+	h = GetStdHandle (STD_INPUT_HANDLE);
+	GetConsoleMode (h, &mode);
+	mouse = (ENABLE_MOUSE_INPUT | ENABLE_EXTENDED_FLAGS);
+	mode = enable ? (mode | mouse) & ~ENABLE_QUICK_EDIT_MODE : (mode & ~mouse) | ENABLE_QUICK_EDIT_MODE;
+	if (SetConsoleMode (h, mode)) {
+		I.mouse = enable;
+	}
 	return enabled;
 #else
 	return false;
@@ -378,10 +505,10 @@ R_API RCons *r_cons_new() {
 	}
 	I.rgbstr = r_cons_rgb_str_off;
 	I.line = r_line_new ();
+	I.enable_highlight = true;
 	I.highlight = NULL;
 	I.is_wine = -1;
 	I.fps = 0;
-	I.color = COLOR_MODE_DISABLED;
 	I.blankline = true;
 	I.teefile = NULL;
 	I.fix_columns = 0;
@@ -391,7 +518,6 @@ R_API RCons *r_cons_new() {
 	I.force_columns = 0;
 	I.event_resize = NULL;
 	I.event_data = NULL;
-	I.is_interactive = true;
 	I.noflush = false;
 	I.linesleep = 0;
 	I.fdin = stdin;
@@ -400,17 +526,17 @@ R_API RCons *r_cons_new() {
 	I.lines = 0;
 
 	I.context = &r_cons_context_default;
-	cons_context_init (I.context);
+	cons_context_init (I.context, NULL);
 
 	r_cons_get_size (&I.pagesize);
 	I.num = NULL;
 	I.null = 0;
-#if __WINDOWS__ && !__CYGWIN__
-	I.ansicon = r_sys_getenv ("ANSICON");
+#if __WINDOWS__
+	I.ansicon = r_cons_is_ansicon ();
 #endif
 #if EMSCRIPTEN
 	/* do nothing here :? */
-#elif __UNIX__ || __CYGWIN__
+#elif __UNIX__
 	tcgetattr (0, &I.term_buf);
 	memcpy (&I.term_raw, &I.term_buf, sizeof (I.term_raw));
 	I.term_raw.c_iflag &= ~(BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON);
@@ -418,7 +544,7 @@ R_API RCons *r_cons_new() {
 	I.term_raw.c_cflag &= ~(CSIZE|PARENB);
 	I.term_raw.c_cflag |= CS8;
 	I.term_raw.c_cc[VMIN] = 1; // Solaris stuff hehe
-	signal (SIGWINCH, resize);
+	r_sys_signal (SIGWINCH, resize);
 #elif __WINDOWS__
 	h = GetStdHandle (STD_INPUT_HANDLE);
 	GetConsoleMode (h, &I.term_buf);
@@ -429,9 +555,10 @@ R_API RCons *r_cons_new() {
 #endif
 	I.pager = NULL; /* no pager by default */
 	I.mouse = 0;
+	I.onestream = false;
+	I.show_vals = false;
 	r_cons_reset ();
 	r_cons_rgb_init ();
-	r_cons_pal_init ();
 
 	r_print_set_is_interrupted_cb (r_cons_is_breaked);
 
@@ -443,14 +570,11 @@ R_API RCons *r_cons_free() {
 	if (I.refcnt != 0) {
 		return NULL;
 	}
-	r_cons_pal_free ();
 	if (I.line) {
 		r_line_free ();
 		I.line = NULL;
 	}
-	if (I.context->buffer) {
-		R_FREE (I.context->buffer);
-	}
+	R_FREE (I.context->buffer);
 	R_FREE (I.break_word);
 	cons_context_deinit (I.context);
 	R_FREE (I.context->lastOutput);
@@ -500,7 +624,11 @@ R_API int r_cons_eof() {
 }
 
 R_API void r_cons_gotoxy(int x, int y) {
+#if __WINDOWS__
+	r_cons_w32_gotoxy (1, x, y);
+#else
 	r_cons_printf ("\x1b[%d;%dH", y, x);
+#endif
 }
 
 R_API void r_cons_print_clear() {
@@ -557,19 +685,20 @@ R_API void r_cons_reset_colors() {
 }
 
 R_API void r_cons_clear() {
-	r_cons_strcat (Color_RESET R_CONS_CLEAR_SCREEN);
 	I.lines = 0;
+#if __WINDOWS__
+	r_cons_w32_clear ();
+#else
+	r_cons_strcat (Color_RESET R_CONS_CLEAR_SCREEN);
+#endif
 }
 
 static void cons_grep_reset(RConsGrep *grep) {
-	grep->strings[0][0] = '\0';
-	grep->nstrings = 0; // XXX
+	R_FREE (grep->str);
+	ZERO_FILL (*grep);
 	grep->line = -1;
 	grep->sort = -1;
 	grep->sort_invert = false;
-	R_FREE (grep->str);
-	ZERO_FILL (grep->tokens);
-	grep->tokens_used = 0;
 }
 
 R_API void r_cons_reset() {
@@ -639,36 +768,13 @@ R_API void r_cons_pop() {
 	cons_stack_free ((void *)data);
 }
 
-R_API RConsContext *r_cons_context_new(void) {
+R_API RConsContext *r_cons_context_new(R_NULLABLE RConsContext *parent) {
 	RConsContext *context = R_NEW0 (RConsContext);
 	if (!context) {
 		return NULL;
 	}
-	cons_context_init (context);
+	cons_context_init (context, parent);
 	return context;
-	/*
-	RStack *stack = r_stack_newf (6, cons_stack_free);
-	if (!stack) {
-		return NULL;
-	}
-
-	RConsStack *data = R_NEW0 (RConsStack);
-	if (!data) {
-		r_stack_free (stack);
-		return NULL;
-	}
-
-	data->grep = R_NEW0 (RConsGrep);
-	if (!data->grep) {
-		R_FREE (data);
-		r_stack_free (stack);
-		return NULL;
-	}
-	cons_grep_reset (data->grep);
-
-	r_stack_push (stack, data);
-
-	return stack;*/
 }
 
 R_API void r_cons_context_free(RConsContext *context) {
@@ -716,6 +822,26 @@ static bool lastMatters() {
 		!I.context->grep.json && !I.is_html);
 }
 
+R_API void r_cons_echo(const char *msg) {
+	static RStrBuf *echodata = NULL; // TODO: move into RConsInstance? maybe nope
+	if (msg) {
+		if (echodata) {
+			r_strbuf_append (echodata, msg);
+			r_strbuf_append (echodata, "\n");
+		} else {
+			echodata = r_strbuf_new (msg);
+		}
+	} else {
+		if (echodata) {
+			char *data = r_strbuf_drain (echodata);
+			r_cons_strcat (data);
+			r_cons_newline ();
+			echodata = NULL;
+			free (data);
+		}
+	}
+}
+
 R_API void r_cons_flush(void) {
 	const char *tee = I.teefile;
 	if (I.noflush) {
@@ -737,7 +863,7 @@ R_API void r_cons_flush(void) {
 		CTX (lastMode) = false;
 	}
 	r_cons_filter ();
-	if (I.is_interactive && I.fdout == 1) {
+	if (r_cons_is_interactive () && I.fdout == 1) {
 		/* Use a pager if the output doesn't fit on the terminal window. */
 		if (CTX (pageable) && CTX (buffer) && I.pager && *I.pager && CTX (buffer_len) > 0 && r_str_char_count (CTX (buffer), '\n') >= I.rows) {
 			I.context->buffer[I.context->buffer_len - 1] = 0;
@@ -790,7 +916,7 @@ R_API void r_cons_flush(void) {
 	r_cons_highlight (I.highlight);
 
 	// is_html must be a filter, not a write endpoint
-	if (I.is_interactive && !r_sandbox_enable (false)) {
+	if (r_cons_is_interactive () && !r_sandbox_enable (false)) {
 		if (I.linesleep > 0 && I.linesleep < 1000) {
 			int i = 0;
 			int pagesize = R_MAX (1, I.pagesize);
@@ -800,7 +926,7 @@ R_API void r_cons_flush(void) {
 			I.context->buffer[I.context->buffer_len] = 0;
 			r_cons_break_push (NULL, NULL);
 			while (nl && !r_cons_is_breaked ()) {
-				r_cons_write (ptr, nl - ptr + 1);
+				__cons_write (ptr, nl - ptr + 1);
 				if (!(i % pagesize)) {
 					r_sys_usleep (I.linesleep * 1000);
 				}
@@ -808,13 +934,13 @@ R_API void r_cons_flush(void) {
 				nl = strchr (ptr, '\n');
 				i++;
 			}
-			r_cons_write (ptr, I.context->buffer + len - ptr);
+			__cons_write (ptr, I.context->buffer + len - ptr);
 			r_cons_break_pop ();
 		} else {
-			r_cons_write (I.context->buffer, I.context->buffer_len);
+			__cons_write (I.context->buffer, I.context->buffer_len);
 		}
 	} else {
-		r_cons_write (I.context->buffer, I.context->buffer_len);
+		__cons_write (I.context->buffer, I.context->buffer_len);
 	}
 
 	r_cons_reset ();
@@ -831,11 +957,11 @@ R_API void r_cons_visual_flush() {
 	r_cons_highlight (I.highlight);
 	if (!I.null) {
 /* TODO: this ifdef must go in the function body */
-#if __WINDOWS__ && !__CYGWIN__
+#if __WINDOWS__
 		if (I.ansicon) {
 			r_cons_visual_write (I.context->buffer);
 		} else {
-			r_cons_w32_print ((const ut8*)I.context->buffer, I.context->buffer_len, 1);
+			r_cons_w32_print (I.context->buffer, I.context->buffer_len, true);
 		}
 #else
 		r_cons_visual_write (I.context->buffer);
@@ -843,25 +969,41 @@ R_API void r_cons_visual_flush() {
 	}
 	r_cons_reset ();
 	if (I.fps) {
-		int fps = 0, w = r_cons_get_size (NULL);
-		static ut64 prev = 0LL; //r_sys_now ();
-		fps = 0;
-		if (prev) {
-			ut64 now = r_sys_now ();
-			st64 diff = (st64)(now - prev);
-			if (diff < 0) {
-				fps = 0;
-			} else {
-				fps = (diff < 1000000)? (1000000.0/diff): 0;
-			}
-			prev = now;
-		} else {
-			prev = r_sys_now ();
-		}
-		eprintf ("\x1b[0;%dH[%d FPS] \n", w-10, fps);
+		r_cons_print_fps (0);
 	}
 }
 
+R_API void r_cons_print_fps (int col) {
+	int fps = 0, w = r_cons_get_size (NULL);
+	static ut64 prev = 0LL; //r_sys_now ();
+	fps = 0;
+	if (prev) {
+		ut64 now = r_sys_now ();
+		st64 diff = (st64)(now - prev);
+		if (diff < 0) {
+			fps = 0;
+		} else {
+			fps = (diff < 1000000)? (1000000.0/diff): 0;
+		}
+		prev = now;
+	} else {
+		prev = r_sys_now ();
+	}
+	if (col < 1) {
+		col = 12;
+	}
+#ifdef __WINDOWS__
+	if (I.ansicon) {
+		eprintf ("\x1b[0;%dH[%d FPS] \n", w - col, fps);
+	} else {
+		r_cons_w32_gotoxy (2, w - col, 0);
+		eprintf (" [%d FPS] \n", fps);
+	}
+#else
+	eprintf ("\x1b[0;%dH[%d FPS] \n", w - col, fps);
+#endif
+}
+
 static int real_strlen(const char *ptr, int len) {
 	int utf8len = r_str_len_utf8 (ptr);
 	int ansilen = r_str_ansi_len (ptr);
@@ -905,26 +1047,26 @@ R_API void r_cons_visual_write(char *buffer) {
 			len = endptr - ptr;
 			plen = ptr > buffer ? len : len - 1;
 			if (lines > 0) {
-				r_cons_write (pptr, plen);
+				__cons_write (pptr, plen);
 				if (len != olen) {
-					r_cons_write (Color_RESET, strlen (Color_RESET));
+					__cons_write (Color_RESET, strlen (Color_RESET));
 				}
 			}
 		} else {
 			if (lines > 0) {
 				int w = cols - (alen % cols == 0 ? cols : alen % cols);
-				r_cons_write (pptr, plen);
+				__cons_write (pptr, plen);
 				if (I.blankline && w > 0) {
 					if (w > sizeof (white) - 1) {
 						w = sizeof (white) - 1;
 					}
-					r_cons_write (white, w);
+					__cons_write (white, w);
 				}
 			}
 			// TRICK to empty columns.. maybe buggy in w32
 			if (r_mem_mem ((const ut8*)ptr, len, (const ut8*)"\x1b[0;0H", 6)) {
 				lines = I.rows;
-				r_cons_write (pptr, plen);
+				__cons_write (pptr, plen);
 			}
 		}
 		if (break_lines) {
@@ -940,7 +1082,7 @@ R_API void r_cons_visual_write(char *buffer) {
 			cols = sizeof (white);
 		}
 		while (--lines >= 0) {
-			r_cons_write (white, cols);
+			__cons_write (white, cols);
 		}
 	}
 }
@@ -957,18 +1099,20 @@ R_API void r_cons_printf_list(const char *format, va_list ap) {
 		return;
 	}
 	if (strchr (format, '%')) {
-		palloc (MOAR + strlen (format) * 20);
+		if (palloc (MOAR + strlen (format) * 20)) {
 club:
-		size = I.context->buffer_sz - I.context->buffer_len - 1; /* remaining space in I.context->buffer */
-		written = vsnprintf (I.context->buffer + I.context->buffer_len, size, format, ap3);
-		if (written >= size) { /* not all bytes were written */
-			palloc (written);
-			va_end (ap3);
-			va_copy (ap3, ap2);
-			goto club;
+			size = I.context->buffer_sz - I.context->buffer_len - 1; /* remaining space in I.context->buffer */
+			written = vsnprintf (I.context->buffer + I.context->buffer_len, size, format, ap3);
+			if (written >= size) { /* not all bytes were written */
+				if (palloc (written)) {
+					va_end (ap3);
+					va_copy (ap3, ap2);
+					goto club;
+				}
+			}
+			I.context->buffer_len += written;
+			I.context->buffer[I.context->buffer_len] = 0;
 		}
-		I.context->buffer_len += written;
-		I.context->buffer[I.context->buffer_len] = 0;
 	} else {
 		r_cons_strcat (format);
 	}
@@ -988,6 +1132,24 @@ R_API int r_cons_printf(const char *format, ...) {
 	return 0;
 }
 
+#if ONE_STREAM_HACK
+R_API int r_cons_onestream_printf(const char *format, ...) {
+	va_list ap;
+	if (!format || !*format) {
+		return -1;
+	}
+	va_start (ap, format);
+	if (I.onestream) {
+		r_cons_printf_list (format, ap);
+	} else {
+		vfprintf (stderr, format, ap);
+	}
+	va_end (ap);
+
+	return 0;
+}
+#endif
+
 R_API int r_cons_get_column() {
 	char *line = strrchr (I.context->buffer, '\n');
 	if (!line) {
@@ -1029,10 +1191,11 @@ R_API int r_cons_memcat(const char *str, int len) {
 
 R_API void r_cons_memset(char ch, int len) {
 	if (!I.null && len > 0) {
-		palloc (len + 1);
-		memset (I.context->buffer + I.context->buffer_len, ch, len);
-		I.context->buffer_len += len;
-		I.context->buffer[I.context->buffer_len] = 0;
+		if (palloc (len + 1)) {
+			memset (I.context->buffer + I.context->buffer_len, ch, len);
+			I.context->buffer_len += len;
+			I.context->buffer[I.context->buffer_len] = 0;
+		}
 	}
 }
 
@@ -1097,7 +1260,7 @@ R_API int r_cons_get_cursor(int *rows) {
 }
 
 R_API bool r_cons_isatty() {
-#if __UNIX__ || __CYGWIN__
+#if __UNIX__
 	struct winsize win = { 0 };
 	const char *tty;
 	struct stat sb;
@@ -1126,12 +1289,12 @@ R_API bool r_cons_isatty() {
 
 // XXX: if this function returns <0 in rows or cols expect MAYHEM
 R_API int r_cons_get_size(int *rows) {
-#if __WINDOWS__ && !__CYGWIN__
+#if __WINDOWS__
 	CONSOLE_SCREEN_BUFFER_INFO csbi;
-	GetConsoleScreenBufferInfo (GetStdHandle (STD_OUTPUT_HANDLE), &csbi);
-	I.columns = (csbi.srWindow.Right - csbi.srWindow.Left) - 1;
-	I.rows = csbi.srWindow.Bottom - csbi.srWindow.Top; // last row empty
- 	if (I.columns == -1 && I.rows == 0) {
+	bool ret = GetConsoleScreenBufferInfo (GetStdHandle (STD_OUTPUT_HANDLE), &csbi);
+	I.columns = csbi.srWindow.Right - csbi.srWindow.Left + 1;
+	I.rows = csbi.srWindow.Bottom - csbi.srWindow.Top + 1;
+ 	if (!ret || (I.columns == -1 && I.rows == 0)) {
 		// Stdout is probably redirected so we set default values
 		I.columns = 80;
 		I.rows = 23;
@@ -1139,7 +1302,7 @@ R_API int r_cons_get_size(int *rows) {
 #elif EMSCRIPTEN
 	I.columns = 80;
 	I.rows = 23;
-#elif __UNIX__ || __CYGWIN__
+#elif __UNIX__
 	struct winsize win = { 0 };
 	if (isatty (0) && !ioctl (0, TIOCGWINSZ, &win)) {
 		if ((!win.ws_col) || (!win.ws_row)) {
@@ -1205,14 +1368,57 @@ R_API int r_cons_get_size(int *rows) {
 	return R_MAX (0, I.columns);
 }
 
+#if __WINDOWS__
+R_API bool r_cons_is_ansicon(void) {
+	DWORD major;
+	DWORD minor;
+	DWORD release = 0;
+	bool win_support = false;
+	RSysInfo *info = r_sys_info ();
+	if (info && info->version) {
+		char *dot = strtok (info->version, ".");
+		major = atoi (dot);
+		dot = strtok (NULL, ".");
+		minor = atoi (dot);
+		if (info->release) {
+			release = atoi (info->release);
+		}
+		if (major > 10
+			|| (major == 10 && minor > 0)
+			|| (major == 10 && minor == 0 && release >= 1703)) {
+			win_support = true;
+		}
+	}
+	r_sys_info_free (info);
+	char *ansicon = r_sys_getenv ("ANSICON");
+	if (ansicon) {
+		free (ansicon);
+		win_support = true;
+	}
+	return win_support;
+}
+#endif
+
 R_API void r_cons_show_cursor(int cursor) {
-#if __WINDOWS__ && !__CYGWIN__
-	// TODO
-#else
-	if (cursor) {
-		write (1, "\x1b[?25h", 6);
+#if __WINDOWS__
+	if (I.ansicon) {
+#endif
+		write (1, cursor ? "\x1b[?25h" : "\x1b[?25l", 6);
+#if __WINDOWS__
 	} else {
-		write (1, "\x1b[?25l", 6);
+		static HANDLE hStdout = NULL;
+		static DWORD size = -1;
+		CONSOLE_CURSOR_INFO cursor_info;
+		if (!hStdout) {
+			hStdout = GetStdHandle (STD_OUTPUT_HANDLE);
+		}
+		if (size == -1) {
+			GetConsoleCursorInfo (hStdout, &cursor_info);
+			size = cursor_info.dwSize;
+		}
+		cursor_info.dwSize = size;
+		cursor_info.bVisible = cursor ? TRUE : FALSE;
+		SetConsoleCursorInfo (hStdout, &cursor_info);
 	}
 #endif
 }
@@ -1238,7 +1444,7 @@ R_API void r_cons_set_raw(bool is_raw) {
 	}
 #if EMSCRIPTEN
 	/* do nothing here */
-#elif __UNIX__ || __CYGWIN__
+#elif __UNIX__
 	// enforce echo off
 	if (is_raw) {
 		I.term_raw.c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
@@ -1259,6 +1465,34 @@ R_API void r_cons_set_raw(bool is_raw) {
 	oldraw = is_raw;
 }
 
+R_API void r_cons_set_utf8(bool b) {
+	I.use_utf8 = b;
+#if __WINDOWS__
+	if (b) {
+		if (IsValidCodePage (CP_UTF8)) {
+			if (!SetConsoleOutputCP (CP_UTF8)) {
+				r_sys_perror ("r_cons_set_utf8");
+			}
+#if UNICODE
+			UINT inCP = CP_UTF8;
+#else
+			UINT inCP = GetACP ();
+#endif
+			if (!SetConsoleCP (inCP)) {
+				r_sys_perror ("r_cons_set_utf8");
+			}
+		} else {
+			R_LOG_WARN ("UTF-8 Codepage not installed.\n");
+		}
+	} else {
+		UINT acp = GetACP ();
+		if (!SetConsoleCP (acp) || !SetConsoleOutputCP (acp)) {
+			r_sys_perror ("r_cons_set_utf8");
+		}
+	}
+#endif
+}
+
 R_API void r_cons_invert(int set, int color) {
 	r_cons_strcat (R_CONS_INVERT (set, color));
 }
@@ -1270,13 +1504,13 @@ R_API void r_cons_invert(int set, int color) {
   rmcup: enable terminal scrolling (normal mode)
 */
 R_API void r_cons_set_cup(int enable) {
-#if __UNIX__ || __CYGWIN__
+#if __UNIX__
 	const char *code = enable
 		? "\x1b[?1049h" "\x1b" "7\x1b[?47h"
 		: "\x1b[?1049l" "\x1b[?47l" "\x1b" "8";
 	write (2, code, strlen (code));
 	fflush (stdout);
-#elif __WINDOWS__ && !__CYGWIN__
+#elif __WINDOWS__
 	if (I.ansicon) {
 		if (enable) {
 			const char *code =
@@ -1309,15 +1543,16 @@ R_API void r_cons_column(int c) {
 	free (b);
 }
 
-static int lasti = 0; /* last interactive mode */
+//  XXX deprecate must be push/pop context state
+static bool lasti = false; /* last interactive mode */
 
 R_API void r_cons_set_interactive(bool x) {
-	lasti = r_cons_singleton ()->is_interactive;
-	r_cons_singleton ()->is_interactive = x;
+	lasti = r_cons_singleton ()->context->is_interactive;
+	r_cons_singleton ()->context->is_interactive = x;
 }
 
 R_API void r_cons_set_last_interactive() {
-	r_cons_singleton ()->is_interactive = lasti;
+	r_cons_singleton ()->context->is_interactive = lasti;
 }
 
 R_API void r_cons_set_title(const char *str) {
@@ -1343,11 +1578,16 @@ R_API void r_cons_highlight(const char *word) {
 		strlen (inv[1])
 	};
 
+	if (!I.enable_highlight) {
+		r_cons_enable_highlight (true);
+		return;
+	}
 	if (word && *word && I.context->buffer) {
 		int word_len = strlen (word);
 		char *orig;
 		clean = r_str_ndup (I.context->buffer, I.context->buffer_len);
 		l = r_str_ansi_filter (clean, &orig, &cpos, -1);
+		free (I.context->buffer);
 		I.context->buffer = orig;
 		if (I.highlight) {
 			if (strcmp (word, I.highlight)) {
@@ -1367,7 +1607,7 @@ R_API void r_cons_highlight(const char *word) {
 		strcpy (rword + linv[0], word);
 		strcpy (rword + linv[0] + word_len, inv[1]);
 		res = r_str_replace_thunked (I.context->buffer, clean, cpos,
-					     l, word, rword, 1);
+					l, word, rword, 1);
 		if (res) {
 			I.context->buffer = res;
 			I.context->buffer_len = I.context->buffer_sz = strlen (res);
@@ -1485,23 +1725,25 @@ R_API void r_cons_bind(RConsBind *bind) {
 	}
 	bind->get_size = r_cons_get_size;
 	bind->get_cursor = r_cons_get_cursor;
+	bind->cb_printf = r_cons_printf;
+	bind->cb_flush = r_cons_flush;
+	bind->cb_grep = r_cons_grep;
+	bind->is_breaked = r_cons_is_breaked;
 }
 
 R_API const char* r_cons_get_rune(const ut8 ch) {
-	if (ch >= RUNECODE_MIN && ch < RUNECODE_MAX) {
-		switch (ch) {
-		case RUNECODE_LINE_HORIZ: return RUNE_LINE_HORIZ;
-		case RUNECODE_LINE_VERT:  return RUNE_LINE_VERT;
-		case RUNECODE_LINE_CROSS: return RUNE_LINE_CROSS;
-		case RUNECODE_CORNER_TL:  return RUNE_CORNER_TL;
-		case RUNECODE_CORNER_TR:  return RUNE_CORNER_TR;
-		case RUNECODE_CORNER_BR:  return RUNE_CORNER_BR;
-		case RUNECODE_CORNER_BL:  return RUNE_CORNER_BL;
-		case RUNECODE_CURVE_CORNER_TL:  return RUNE_CURVE_CORNER_TL;
-		case RUNECODE_CURVE_CORNER_TR:  return RUNE_CURVE_CORNER_TR;
-		case RUNECODE_CURVE_CORNER_BR:  return RUNE_CURVE_CORNER_BR;
-		case RUNECODE_CURVE_CORNER_BL:  return RUNE_CURVE_CORNER_BL;
-		}
+	switch (ch) {
+	case RUNECODE_LINE_HORIZ: return RUNE_LINE_HORIZ;
+	case RUNECODE_LINE_VERT:  return RUNE_LINE_VERT;
+	case RUNECODE_LINE_CROSS: return RUNE_LINE_CROSS;
+	case RUNECODE_CORNER_TL:  return RUNE_CORNER_TL;
+	case RUNECODE_CORNER_TR:  return RUNE_CORNER_TR;
+	case RUNECODE_CORNER_BR:  return RUNE_CORNER_BR;
+	case RUNECODE_CORNER_BL:  return RUNE_CORNER_BL;
+	case RUNECODE_CURVE_CORNER_TL:  return RUNE_CURVE_CORNER_TL;
+	case RUNECODE_CURVE_CORNER_TR:  return RUNE_CURVE_CORNER_TR;
+	case RUNECODE_CURVE_CORNER_BR:  return RUNE_CURVE_CORNER_BR;
+	case RUNECODE_CURVE_CORNER_BL:  return RUNE_CURVE_CORNER_BL;
 	}
 	return NULL;
 }
@@ -1523,9 +1765,9 @@ R_API void r_cons_breakword(R_NULLABLE const char *s) {
  * "command2", "args2", "description"}; */
 R_API void r_cons_cmd_help(const char *help[], bool use_color) {
 	RCons *cons = r_cons_singleton ();
-	const char *pal_args_color = use_color ? cons->pal.args : "",
-			*pal_help_color = use_color ? cons->pal.help : "",
-			*pal_reset = use_color ? cons->pal.reset : "";
+	const char *pal_args_color = use_color ? cons->context->pal.args : "",
+			*pal_help_color = use_color ? cons->context->pal.help : "",
+			*pal_reset = use_color ? cons->context->pal.reset : "";
 	int i, max_length = 0;
 	const char *usage_str = "Usage:";
 
@@ -1559,7 +1801,7 @@ R_API void r_cons_cmd_help(const char *help[], bool use_color) {
 }
 
 R_API void r_cons_clear_buffer(void) {
-#if __UNIX__ || __CYGWIN__
+#if __UNIX__
 	write (1, "\x1b" "c\x1b[3J",  6);
 #endif
 }
