@@ -1,9 +1,9 @@
-/* radare - LGPL - Copyright 2013-2018 - pancake, sghctoma, xarkes */
+/* radare - LGPL - Copyright 2013-2019 - pancake, sghctoma, xarkes */
 
 #include <r_cons.h>
 
-#define RCOLOR_AT(i) (RColor *) (((ut8 *) &(r_cons_singleton ()->cpal)) + keys[i].coff)
-#define COLOR_AT(i) (char **) (((ut8 *) &(r_cons_singleton ()->pal)) + keys[i].off)
+#define RCOLOR_AT(i) (RColor *) (((ut8 *) &(r_cons_singleton ()->context->cpal)) + keys[i].coff)
+#define COLOR_AT(i) (char **) (((ut8 *) &(r_cons_singleton ()->context->pal)) + keys[i].off)
 
 static struct {
 	const char *name;
@@ -41,6 +41,8 @@ static struct {
 	{ "nop", r_offsetof (RConsPrintablePalette, nop), r_offsetof (RConsPalette, nop) },
 	{ "ret", r_offsetof (RConsPrintablePalette, ret), r_offsetof (RConsPalette, ret) },
 	{ "trap", r_offsetof (RConsPrintablePalette, trap), r_offsetof (RConsPalette, trap) },
+	{ "ucall", r_offsetof (RConsPrintablePalette, ucall), r_offsetof (RConsPalette, ucall) },
+	{ "ujmp", r_offsetof (RConsPrintablePalette, ujmp), r_offsetof (RConsPalette, ujmp) },
 	{ "swi", r_offsetof (RConsPrintablePalette, swi), r_offsetof (RConsPalette, swi) },
 	{ "cmp", r_offsetof (RConsPrintablePalette, cmp), r_offsetof (RConsPalette, cmp) },
 	{ "reg", r_offsetof (RConsPrintablePalette, reg), r_offsetof (RConsPalette, reg) },
@@ -70,6 +72,11 @@ static struct {
 	{ "graph.current", r_offsetof (RConsPrintablePalette, graph_current), r_offsetof (RConsPalette, graph_current) },
 	{ "graph.traced", r_offsetof (RConsPrintablePalette, graph_traced), r_offsetof (RConsPalette, graph_traced) },
 
+        { "graph.diff.unknown", r_offsetof (RConsPrintablePalette, graph_diff_unknown), r_offsetof (RConsPalette, graph_diff_unknown) },
+        { "graph.diff.new", r_offsetof (RConsPrintablePalette, graph_diff_new), r_offsetof (RConsPalette, graph_diff_new) },
+        { "graph.diff.match", r_offsetof (RConsPrintablePalette, graph_diff_match), r_offsetof (RConsPalette, graph_diff_match) },
+        { "graph.diff.unmatch", r_offsetof (RConsPrintablePalette, graph_diff_unmatch), r_offsetof (RConsPalette, graph_diff_unmatch) },
+
 	{ "gui.cflow", r_offsetof (RConsPrintablePalette, gui_cflow), r_offsetof (RConsPalette, gui_cflow) },
 	{ "gui.dataoffset", r_offsetof (RConsPrintablePalette, gui_dataoffset), r_offsetof (RConsPalette, gui_dataoffset) },
 	{ "gui.background", r_offsetof (RConsPrintablePalette, gui_background), r_offsetof (RConsPalette, gui_background) },
@@ -98,6 +105,14 @@ struct {
 	{ "cyan",     RColor_CYAN,     Color_CYAN,     Color_BGCYAN },
 	{ "blue",     RColor_BLUE,     Color_BLUE,     Color_BGBLUE },
 	{ "gray",     RColor_GRAY,     Color_GRAY,     Color_BGGRAY },
+	{ "bblack",   RColor_BBLACK,   Color_BBLACK,   Color_BBGBLACK },
+	{ "bred",     RColor_BRED,     Color_BRED,     Color_BBGRED },
+	{ "bwhite",   RColor_BWHITE,   Color_BWHITE,   Color_BBGWHITE },
+	{ "bgreen",   RColor_BGREEN,   Color_BGREEN,   Color_BBGGREEN },
+	{ "bmagenta", RColor_BMAGENTA, Color_BMAGENTA, Color_BBGMAGENTA },
+	{ "byellow",  RColor_BYELLOW,  Color_BYELLOW,  Color_BBGYELLOW },
+	{ "bcyan",    RColor_BCYAN,    Color_BCYAN,    Color_BBGCYAN },
+	{ "bblue",    RColor_BBLUE,    Color_BBLUE,    Color_BBGBLUE },
 	{ "none",     RColor_NULL,     Color_RESET,    Color_RESET },
 	{ NULL, RColor_NULL, NULL, NULL }
 };
@@ -109,102 +124,146 @@ static inline ut8 rgbnum(const char ch1, const char ch2) {
 	return r << 4 | r2;
 }
 
-R_API void r_cons_pal_init() {
-	RCons *cons = r_cons_singleton ();
-
-	memset (&cons->cpal, 0, sizeof (cons->cpal));
-	memset (&cons->pal, 0, sizeof (cons->pal));
-
-	cons->cpal.b0x00              = (RColor) RColor_GREEN;
-	cons->cpal.b0x7f              = (RColor) RColor_CYAN;
-	cons->cpal.b0xff              = (RColor) RColor_RED;
-	cons->cpal.args               = (RColor) RColor_YELLOW;
-	cons->cpal.bin                = (RColor) RColor_CYAN;
-	cons->cpal.btext              = (RColor) RColor_YELLOW;
-	cons->cpal.call               = (RColor) RColor_GREEN;
-	cons->cpal.call.attr          = R_CONS_ATTR_BOLD;
-	cons->cpal.cjmp               = (RColor) RColor_GREEN;
-	cons->cpal.cmp                = (RColor) RColor_CYAN;
-	cons->cpal.comment            = (RColor) RColor_RED;
-	cons->cpal.usercomment        = (RColor) RColor_WHITE;
-	cons->cpal.creg               = (RColor) RColor_CYAN;
-	cons->cpal.flag               = (RColor) RColor_CYAN;
-	cons->cpal.fline              = (RColor) RColor_CYAN;
-	cons->cpal.floc               = (RColor) RColor_CYAN;
-	cons->cpal.flow               = (RColor) RColor_CYAN;
-	cons->cpal.flow2              = (RColor) RColor_BLUE;
-	cons->cpal.fname              = (RColor) RColor_RED;
-	cons->cpal.help               = (RColor) RColor_GREEN;
-	cons->cpal.input              = (RColor) RColor_WHITE;
-	cons->cpal.invalid            = (RColor) RColor_RED;
-	cons->cpal.invalid.attr       = R_CONS_ATTR_BOLD;
-	cons->cpal.jmp                = (RColor) RColor_GREEN;
-	cons->cpal.label              = (RColor) RColor_CYAN;
-	cons->cpal.math               = (RColor) RColor_YELLOW;
-	cons->cpal.mov                = (RColor) RColor_WHITE;
-	cons->cpal.nop                = (RColor) RColor_BLUE;
-	cons->cpal.num                = (RColor) RColor_YELLOW;
-	cons->cpal.offset             = (RColor) RColor_GREEN;
-	cons->cpal.other              = (RColor) RColor_WHITE;
-	cons->cpal.pop                = (RColor) RColor_MAGENTA;
-	cons->cpal.pop.attr           = R_CONS_ATTR_BOLD;
-	cons->cpal.prompt             = (RColor) RColor_YELLOW;
-	cons->cpal.push               = (RColor) RColor_MAGENTA;
-	cons->cpal.crypto             = (RColor) RColor_BGBLUE;
-	cons->cpal.reg                = (RColor) RColor_CYAN;
-	cons->cpal.ret                = (RColor) RColor_RED;
-	cons->cpal.swi                = (RColor) RColor_MAGENTA;
-	cons->cpal.trap               = (RColor) RColor_RED;
-	cons->cpal.trap.attr          = R_CONS_ATTR_BOLD;
-
-	cons->cpal.ai_read            = (RColor) RColor_GREEN;
-	cons->cpal.ai_write           = (RColor) RColor_BLUE;
-	cons->cpal.ai_exec            = (RColor) RColor_RED;
-	cons->cpal.ai_seq             = (RColor) RColor_MAGENTA;
-	cons->cpal.ai_ascii           = (RColor) RColor_YELLOW;
-
-	cons->cpal.gui_cflow          = (RColor) RColor_YELLOW;
-	cons->cpal.gui_dataoffset     = (RColor) RColor_YELLOW;
-	cons->cpal.gui_background     = (RColor) RColor_BLACK;
-	cons->cpal.gui_alt_background = (RColor) RColor_WHITE;
-	cons->cpal.gui_border         = (RColor) RColor_BLACK;
-	cons->cpal.wordhl             = (RColor) RColor_BGRED;
-	cons->cpal.linehl             = (RColor) RCOLOR (ALPHA_BG, 0x00, 0x00, 0x7f, 0x00, 0x00, 0x00);
-
-	cons->cpal.func_var           = (RColor) RColor_WHITE;
-	cons->cpal.func_var_type      = (RColor) RColor_BLUE;
-	cons->cpal.func_var_addr      = (RColor) RColor_CYAN;
-
-	cons->cpal.widget_bg          = (RColor) RCOLOR (ALPHA_BG, 0x30, 0x30, 0x30, 0x00, 0x00, 0x00);
-	cons->cpal.widget_sel         = (RColor) RColor_BGRED;
-
-	cons->cpal.graph_box          = (RColor) RColor_NULL;
-	cons->cpal.graph_box2         = (RColor) RColor_BLUE;
-	cons->cpal.graph_box3         = (RColor) RColor_MAGENTA;
-	cons->cpal.graph_box4         = (RColor) RColor_GRAY;
-	cons->cpal.graph_true         = (RColor) RColor_GREEN;
-	cons->cpal.graph_false        = (RColor) RColor_RED;
-	cons->cpal.graph_trufae       = (RColor) RColor_BLUE; // single jump
-	cons->cpal.graph_traced       = (RColor) RColor_YELLOW;
-	cons->cpal.graph_current      = (RColor) RColor_BLUE;
-
-	cons->pal.rainbow = NULL;
-	cons->pal.rainbow_sz = 0;
-	r_cons_pal_free ();
-	cons->pal.reset = Color_RESET; // reset is not user accessible, const char* is ok
-
-	r_cons_pal_update_event ();
+static void __cons_pal_update_event(RConsContext *ctx) {
+	Sdb *db = sdb_new0 ();
+	int i, n = 0;
+	/* Compute cons->pal values */
+	for (i = 0; keys[i].name; i++) {
+		RColor *rcolor = (RColor *) (((ut8 *) &(ctx->cpal)) + keys[i].coff);
+		char **color = (char **) (((ut8 *) &(ctx->pal)) + keys[i].off);
+		// Color is dynamically allocated, needs to be freed
+		R_FREE (*color);
+		*color = r_cons_rgb_str_mode (ctx->color_mode, NULL, 0, rcolor);
+		const char *rgb = sdb_fmt ("rgb:%02x%02x%02x", rcolor->r, rcolor->g, rcolor->b);
+		sdb_set (db, rgb, "1", 0);
+	}
+	SdbList *list = sdb_foreach_list (db, true);
+	SdbListIter *iter;
+	SdbKv *kv;
+	r_cons_rainbow_free (ctx);
+	r_cons_rainbow_new (ctx, list->length);
+	ls_foreach (list, iter, kv) {
+		ctx->pal.rainbow[n++] = strdup (sdbkv_key (kv));
+	}
+	ctx->pal.rainbow_sz = n;
+	ls_free (list);
+	sdb_free (db);
 }
 
-R_API void r_cons_pal_free() {
+R_API void r_cons_pal_init(RConsContext *ctx) {
+	memset (&ctx->cpal, 0, sizeof (ctx->cpal));
+
+	ctx->cpal.b0x00              = (RColor) RColor_GREEN;
+	ctx->cpal.b0x7f              = (RColor) RColor_CYAN;
+	ctx->cpal.b0xff              = (RColor) RColor_RED;
+	ctx->cpal.args               = (RColor) RColor_YELLOW;
+	ctx->cpal.bin                = (RColor) RColor_CYAN;
+	ctx->cpal.btext              = (RColor) RColor_YELLOW;
+	ctx->cpal.call               = (RColor) RColor_BGREEN;
+	ctx->cpal.call.attr          = R_CONS_ATTR_BOLD;
+	ctx->cpal.ucall              = (RColor) RColor_GREEN;
+	ctx->cpal.ujmp               = (RColor) RColor_GREEN;
+	ctx->cpal.cjmp               = (RColor) RColor_GREEN;
+	ctx->cpal.cmp                = (RColor) RColor_CYAN;
+	ctx->cpal.comment            = (RColor) RColor_RED;
+	ctx->cpal.usercomment        = (RColor) RColor_WHITE;
+	ctx->cpal.creg               = (RColor) RColor_CYAN;
+	ctx->cpal.flag               = (RColor) RColor_CYAN;
+	ctx->cpal.fline              = (RColor) RColor_CYAN;
+	ctx->cpal.floc               = (RColor) RColor_CYAN;
+	ctx->cpal.flow               = (RColor) RColor_CYAN;
+	ctx->cpal.flow2              = (RColor) RColor_BLUE;
+	ctx->cpal.fname              = (RColor) RColor_RED;
+	ctx->cpal.help               = (RColor) RColor_GREEN;
+	ctx->cpal.input              = (RColor) RColor_WHITE;
+	ctx->cpal.invalid            = (RColor) RColor_BRED;
+	ctx->cpal.invalid.attr       = R_CONS_ATTR_BOLD;
+	ctx->cpal.jmp                = (RColor) RColor_GREEN;
+	ctx->cpal.label              = (RColor) RColor_CYAN;
+	ctx->cpal.math               = (RColor) RColor_YELLOW;
+	ctx->cpal.mov                = (RColor) RColor_WHITE;
+	ctx->cpal.nop                = (RColor) RColor_BLUE;
+	ctx->cpal.num                = (RColor) RColor_YELLOW;
+	ctx->cpal.offset             = (RColor) RColor_GREEN;
+	ctx->cpal.other              = (RColor) RColor_WHITE;
+	ctx->cpal.pop                = (RColor) RColor_BMAGENTA;
+	ctx->cpal.pop.attr           = R_CONS_ATTR_BOLD;
+	ctx->cpal.prompt             = (RColor) RColor_YELLOW;
+	ctx->cpal.push               = (RColor) RColor_MAGENTA;
+	ctx->cpal.crypto             = (RColor) RColor_BGBLUE;
+	ctx->cpal.reg                = (RColor) RColor_CYAN;
+	ctx->cpal.ret                = (RColor) RColor_RED;
+	ctx->cpal.swi                = (RColor) RColor_MAGENTA;
+	ctx->cpal.trap               = (RColor) RColor_BRED;
+	ctx->cpal.trap.attr          = R_CONS_ATTR_BOLD;
+
+	ctx->cpal.ai_read            = (RColor) RColor_GREEN;
+	ctx->cpal.ai_write           = (RColor) RColor_BLUE;
+	ctx->cpal.ai_exec            = (RColor) RColor_RED;
+	ctx->cpal.ai_seq             = (RColor) RColor_MAGENTA;
+	ctx->cpal.ai_ascii           = (RColor) RColor_YELLOW;
+
+	ctx->cpal.gui_cflow          = (RColor) RColor_YELLOW;
+	ctx->cpal.gui_dataoffset     = (RColor) RColor_YELLOW;
+	ctx->cpal.gui_background     = (RColor) RColor_BLACK;
+	ctx->cpal.gui_alt_background = (RColor) RColor_WHITE;
+	ctx->cpal.gui_border         = (RColor) RColor_BLACK;
+	ctx->cpal.wordhl             = (RColor) RColor_BGRED;
+	// No good choice for fallback ansi16 color
+#if __WINDOWS__
+	ctx->cpal.linehl             = (RColor) RCOLOR (ALPHA_BG, 0x00, 0x00, 0x50, 0x00, 0x00, 0x00, 4);
+#else
+	ctx->cpal.linehl             = (RColor) RCOLOR (ALPHA_BG, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 4);
+#endif
+
+	ctx->cpal.func_var           = (RColor) RColor_WHITE;
+	ctx->cpal.func_var_type      = (RColor) RColor_BLUE;
+	ctx->cpal.func_var_addr      = (RColor) RColor_CYAN;
+
+	ctx->cpal.widget_bg          = (RColor) RCOLOR (ALPHA_BG, 0x30, 0x30, 0x30, 0x00, 0x00, 0x00, 0);
+	ctx->cpal.widget_sel         = (RColor) RColor_BGRED;
+
+	ctx->cpal.graph_box          = (RColor) RColor_NULL;
+	ctx->cpal.graph_box2         = (RColor) RColor_BLUE;
+	ctx->cpal.graph_box3         = (RColor) RColor_MAGENTA;
+	ctx->cpal.graph_box4         = (RColor) RColor_GRAY;
+	ctx->cpal.graph_true         = (RColor) RColor_GREEN;
+	ctx->cpal.graph_false        = (RColor) RColor_RED;
+	ctx->cpal.graph_trufae       = (RColor) RColor_BLUE; // single jump
+	ctx->cpal.graph_traced       = (RColor) RColor_YELLOW;
+	ctx->cpal.graph_current      = (RColor) RColor_BLUE;
+	ctx->cpal.graph_diff_unknown = (RColor) RColor_MAGENTA;
+	ctx->cpal.graph_diff_new     =  (RColor) RColor_RED;
+	ctx->cpal.graph_diff_match   =  (RColor) RColor_GRAY;
+	ctx->cpal.graph_diff_unmatch =  (RColor) RColor_YELLOW;
+
+
+	r_cons_pal_free (ctx);
+	ctx->pal.reset = Color_RESET; // reset is not user accessible, const char* is ok
+	__cons_pal_update_event (ctx);
+}
+
+R_API void r_cons_pal_free(RConsContext *ctx) {
 	int i;
 	for (i = 0; keys[i].name; i++) {
-		char **color = COLOR_AT (i);
+		char **color = (char **) (((ut8 *) &(ctx->pal)) + keys[i].off);
 		if (color && *color) {
 			R_FREE (*color);
 		}
 	}
-	r_cons_rainbow_free ();
+	r_cons_rainbow_free (ctx);
+}
+
+R_API void r_cons_pal_copy(RConsContext *dst, RConsContext *src) {
+	memcpy (&dst->cpal, &src->cpal, sizeof (src->cpal));
+	memset (&dst->pal, 0, sizeof (dst->pal));
+
+	dst->pal.rainbow = NULL;
+	dst->pal.rainbow_sz = 0;
+
+	dst->pal.reset = Color_RESET; // reset is not user accessible, const char* is ok
+
+	__cons_pal_update_event (dst);
 }
 
 R_API void r_cons_pal_random() {
@@ -221,6 +280,7 @@ R_API void r_cons_pal_random() {
 R_API char *r_cons_pal_parse(const char *str, RColor *outcol) {
 	int i;
 	RColor rcolor = (RColor) RColor_BLACK;
+	rcolor.id16 = -1;
 	char *fgcolor;
 	char *bgcolor;
 	char *attr = NULL;
@@ -305,6 +365,7 @@ R_API char *r_cons_pal_parse(const char *str, RColor *outcol) {
 			rcolor.r = colors[i].rcolor.r;
 			rcolor.g = colors[i].rcolor.g;
 			rcolor.b = colors[i].rcolor.b;
+			rcolor.id16 = colors[i].rcolor.id16;
 			if (!outcol) {
 				strncat (out, colors[i].code,
 					sizeof (out) - strlen (out) - 1);
@@ -315,6 +376,7 @@ R_API char *r_cons_pal_parse(const char *str, RColor *outcol) {
 			rcolor.r2 = colors[i].rcolor.r; // Initial color doesn't
 			rcolor.g2 = colors[i].rcolor.g; // have r2, g2, b2
 			rcolor.b2 = colors[i].rcolor.b;
+			rcolor.id16 = colors[i].rcolor.id16;
 			if (!outcol) {
 				strncat (out, colors[i].bgcode,
 					sizeof (out) - strlen (out) - 1);
@@ -328,13 +390,13 @@ R_API char *r_cons_pal_parse(const char *str, RColor *outcol) {
 			if (!strncmp(p, "bold", 4)) {
 				rcolor.attr |= R_CONS_ATTR_BOLD;
 			} else if (!strncmp(p, "dim", 3)) {
-				rcolor.attr |= 1u << 2;
+				rcolor.attr |= R_CONS_ATTR_DIM;
 			} else if (!strncmp(p, "italic", 6)) {
-				rcolor.attr |= 1u << 3;
+				rcolor.attr |= R_CONS_ATTR_ITALIC;
 			} else if (!strncmp(p, "underline", 9)) {
-				rcolor.attr |= 1u << 4;
+				rcolor.attr |= R_CONS_ATTR_UNDERLINE;
 			} else if (!strncmp(p, "blink", 5)) {
-				rcolor.attr |= 1u << 5;
+				rcolor.attr |= R_CONS_ATTR_BLINK;
 			} else {
 				eprintf ("Failed to parse terminal attributes: %s\n", p);
 				break;
@@ -446,7 +508,7 @@ R_API void r_cons_pal_show() {
 			colors[i].bgcode,
 			colors[i].name);
 	}
-	switch (r_cons_singleton ()->color) {
+	switch (r_cons_singleton ()->context->color_mode) {
 	case COLOR_MODE_256: // 256 color palette
 		r_cons_pal_show_gs ();
 		r_cons_pal_show_256 ();
@@ -455,9 +517,16 @@ R_API void r_cons_pal_show() {
 		r_cons_pal_show_gs ();
 		r_cons_pal_show_rgb ();
 		break;
+	default:
+		break;
 	}
 }
 
+typedef struct {
+	int val;
+	const char *str;
+} RAttrStr;
+
 R_API void r_cons_pal_list(int rad, const char *arg) {
 	char *name, **color;
 	const char *hasnext;
@@ -503,8 +572,31 @@ R_API void r_cons_pal_list(int rad, const char *arg) {
 		case '*':
 		case 'r':
 		case 1:
-			r_cons_printf ("ec %s rgb:%02x%02x%02x\n",
+			r_cons_printf ("ec %s rgb:%02x%02x%02x",
 				keys[i].name, rcolor->r, rcolor->g, rcolor->b);
+			if (rcolor->a == ALPHA_FGBG) {
+				r_cons_printf (" rgb:%02x%02x%02x",
+					rcolor->r2, rcolor->g2, rcolor->b2);
+			}
+			if (rcolor->attr) {
+				const RAttrStr attrs[] = {
+				    { R_CONS_ATTR_BOLD, "bold" },
+				    { R_CONS_ATTR_DIM, "dim" },
+				    { R_CONS_ATTR_ITALIC, "italic" },
+				    { R_CONS_ATTR_UNDERLINE, "underline" },
+				    { R_CONS_ATTR_BLINK, "blink" }
+				};
+				int j;
+				if (rcolor->a != ALPHA_FGBG) {
+					r_cons_strcat (" .");
+				}
+				for (j = 0; j < R_ARRAY_SIZE (attrs); j++) {
+					if (rcolor->attr & attrs[j].val) {
+						r_cons_printf (" %s", attrs[j].str);
+					}
+				}
+			}
+			r_cons_newline ();
 			break;
 		default:
 			r_cons_printf (" %s##"Color_RESET"  %s\n", *color,
@@ -529,7 +621,7 @@ R_API int r_cons_pal_set(const char *key, const char *val) {
 			return true;
 		}
 	}
-	eprintf ("Invalid color %s\n", key);
+	eprintf ("r_cons_pal_set: Invalid color %s\n", key);
 	return false;
 }
 
@@ -561,64 +653,37 @@ R_API int r_cons_pal_len() {
 }
 
 R_API void r_cons_pal_update_event() {
-	Sdb *db = sdb_new0 ();
-	int i, n = 0;
-	char **color;
-	/* Compute cons->pal values */
-	for (i = 0; keys[i].name; i++) {
-		RColor *rcolor = RCOLOR_AT (i);
-		color = COLOR_AT (i);
-		// Color is dynamically allocated, needs to be freed
-		if (*color) {
-			R_FREE (*color);
-		}
-		*color = r_cons_rgb_str (NULL, 0, rcolor);
-		const char *rgb = sdb_fmt ("rgb:%02x%02x%02x", rcolor->r, rcolor->g, rcolor->b);
-		sdb_set (db, rgb, "1", 0);
-	}
-	SdbList *list = sdb_foreach_list (db, true);
-	SdbListIter *iter;
-	SdbKv *kv;
-	r_cons_rainbow_free ();
-	r_cons_rainbow_new (list->length);
-	ls_foreach (list, iter, kv) {
-		r_cons_singleton ()->pal.rainbow[n++] = strdup (sdbkv_key (kv));
-	}
-	r_cons_singleton ()->pal.rainbow_sz = n;
-	ls_free (list);
-	sdb_free (db);
+	__cons_pal_update_event (r_cons_singleton ()->context);
 }
 
-R_API void r_cons_rainbow_new(int sz) {
-	RCons *cons = r_cons_singleton ();
-	cons->pal.rainbow_sz = sz;
-	free (cons->pal.rainbow);
-	cons->pal.rainbow = calloc (sizeof (char *), sz);
+R_API void r_cons_rainbow_new(RConsContext *ctx, int sz) {
+	ctx->pal.rainbow_sz = sz;
+	free (ctx->pal.rainbow);
+	ctx->pal.rainbow = calloc (sizeof (char *), sz);
 }
 
-R_API void r_cons_rainbow_free() {
-	RCons *cons = r_cons_singleton ();
-	int i, sz = cons->pal.rainbow_sz;
-	if (cons->pal.rainbow) {
-		for (i = 0; i < sz ; i++) {
-			free (cons->pal.rainbow[i]);
+R_API void r_cons_rainbow_free(RConsContext *ctx) {
+	int i, sz = ctx->pal.rainbow_sz;
+	if (ctx->pal.rainbow) {
+		for (i = 0; i < sz; i++) {
+			free (ctx->pal.rainbow[i]);
 		}
 	}
-	cons->pal.rainbow_sz = 0;
-	R_FREE (cons->pal.rainbow);
+	ctx->pal.rainbow_sz = 0;
+	R_FREE (ctx->pal.rainbow);
 }
 
 R_API char *r_cons_rainbow_get(int idx, int last, bool bg) {
 	RCons *cons = r_cons_singleton ();
 	if (last < 0) {
-		last = cons->pal.rainbow_sz;
+		last = cons->context->pal.rainbow_sz;
 	}
-	if (idx < 0 || idx >= last || !cons->pal.rainbow) {
+	if (idx < 0 || idx >= last || !cons->context->pal.rainbow) {
 		return NULL;
 	}
-	int x = (last == cons->pal.rainbow_sz)
-		? idx : (cons->pal.rainbow_sz * idx) / (last + 1);
-	const char *a = cons->pal.rainbow[x];
+	int x = (last == cons->context->pal.rainbow_sz)
+		? idx : (cons->context->pal.rainbow_sz * idx) / (last + 1);
+	const char *a = cons->context->pal.rainbow[x];
 	if (bg) {
 		char *dup = r_str_newf ("%s %s", a, a);
 		char *res = r_cons_pal_parse (dup, NULL);
