@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2013-2018 - pancake, xarkes */
+/* radare - LGPL - Copyright 2013-2019 - pancake, xarkes */
 /* ansi 256 color extension for r_cons */
 /* https://en.wikipedia.org/wiki/ANSI_color */
 
@@ -42,7 +42,7 @@ static void init_color_table() {
 	}
 }
 
-static int lookup_rgb(int r, int g, int b) {
+static int __lookup_rgb(int r, int g, int b) {
 	int i, color = (r << 16) + (g << 8) + b;
 	// lookup extended colors only, coz non-extended can be changed by users.
 	for (i = 16; i < 256; ++i) {
@@ -53,7 +53,7 @@ static int lookup_rgb(int r, int g, int b) {
 	return -1;
 }
 
-static ut32 approximate_rgb(int r, int g, int b) {
+static ut32 __approximate_rgb(int r, int g, int b) {
 	bool grey = (r > 0 && r < 255 && r == g && r == b);
 	if (grey) {
 		return 232 + (int)((double)r / (255 / 24.1));
@@ -83,22 +83,22 @@ static ut32 approximate_rgb(int r, int g, int b) {
 }
 
 static int rgb(int r, int g, int b) {
-	int c = lookup_rgb (r, g, b);
+	int c = __lookup_rgb (r, g, b);
 	if (c == -1) {
-		return approximate_rgb (r, g, b);
+		return __approximate_rgb (r, g, b);
 	}
 	return c;
 }
 
-static void unrgb(int color, int *r, int *g, int *b) {
-	if (color < 0 || color > 256) {
+static void __unrgb(int color, int *r, int *g, int *b) {
+	if (color < 0 || color > 255) {
 		*r = *g = *b = 0;
-		return;
+	} else {
+		int rgb = color_table[color];
+		*r = (rgb >> 16) & 0xff;
+		*g = (rgb >> 8) & 0xff;
+		*b = rgb & 0xff;
 	}
-	int rgb = color_table[color];
-	*r = (rgb >> 16) & 0xff;
-	*g = (rgb >> 8) & 0xff;
-	*b = rgb & 0xff;
 }
 
 R_API void r_cons_rgb_init(void) {
@@ -130,7 +130,7 @@ R_API int r_cons_rgb_parse(const char *p, ut8 *r, ut8 *g, ut8 *b, ut8 *a) {
 		if (p[4] == '5')  { // \x1b[%d;5;%dm is 256 colors
 			int x, y, z;
 			int n = atoi (p + 6);
-			unrgb (n, &x, &y, &z);
+			__unrgb (n, &x, &y, &z);
 			SETRGB (x, y, z);
 		} else { // 16M colors (truecolor)
 			/* complex rgb */
@@ -175,6 +175,7 @@ R_API int r_cons_rgb_parse(const char *p, ut8 *r, ut8 *g, ut8 *b, ut8 *a) {
 
 R_API char *r_cons_rgb_str_off(char *outstr, size_t sz, ut64 off) {
 	RColor rc = RColor_BLACK;
+	rc.id16 = -1;
 	rc.r = (off >> 2) & 0xff;
 	rc.g = (off >> 6) & 0xff;
 	rc.b = (off >> 12) & 0xff;
@@ -182,7 +183,8 @@ R_API char *r_cons_rgb_str_off(char *outstr, size_t sz, ut64 off) {
 }
 
 /* Compute color string depending on cons->color */
-static void r_cons_rgb_gen(char *outstr, size_t sz, ut8 attr, ut8 a, ut8 r, ut8 g, ut8 b) {
+static void r_cons_rgb_gen(RConsColorMode mode, char *outstr, size_t sz, ut8 attr, ut8 a, ut8 r, ut8 g, ut8 b,
+                           st8 id16) {
 	ut8 fgbg = (a == ALPHA_BG)? 48: 38; // ANSI codes for Background/Foreground
 
 	if (sz < 4) { // must have at least room for "<esc>[m\0"
@@ -201,40 +203,49 @@ static void r_cons_rgb_gen(char *outstr, size_t sz, ut8 attr, ut8 a, ut8 r, ut8
 			return;
 		}
 		switch (attr & -attr) {
-		case 1u << 1: outstr[i] = '1'; break;
-		case 1u << 2: outstr[i] = '2'; break;
-		case 1u << 3: outstr[i] = '3'; break;
-		case 1u << 4: outstr[i] = '4'; break;
-		case 1u << 5: outstr[i] = '5'; break;
+		case R_CONS_ATTR_BOLD: outstr[i] = '1'; break;
+		case R_CONS_ATTR_DIM: outstr[i] = '2'; break;
+		case R_CONS_ATTR_ITALIC: outstr[i] = '3'; break;
+		case R_CONS_ATTR_UNDERLINE: outstr[i] = '4'; break;
+		case R_CONS_ATTR_BLINK: outstr[i] = '5'; break;
 		}
 		outstr[i + 1] = ';';
 		i += 2;
 	}
 
 	int written = -1;
-	switch (r_cons_singleton ()->color) {
+	switch (mode) {
 	case COLOR_MODE_256: // 256 color palette
 		written = snprintf (outstr + i, sz - i, "%d;5;%dm", fgbg, rgb (r, g, b));
 		break;
 	case COLOR_MODE_16M: // 16M (truecolor)
 		written = snprintf (outstr + i, sz - i, "%d;2;%d;%d;%dm", fgbg, r, g, b);
 		break;
-	case COLOR_MODE_16: // ansi 16 colors
-		{
+	case COLOR_MODE_16: { // ansi 16 colors
+		ut8 bright, c;
 		fgbg -= 8;
-		if (r == g && g == b) {
-			r = (r > 0x7f) ? 1 : 0;
-			g = (g > 0x7f) ? 1 : 0;
-			b = (b > 0x7f) ? 1 : 0;
+		if (id16 >= 0 && id16 <= 15) {
+			c = id16 % 8;
+			bright = id16 >= 8 ? 60 : 0;
 		} else {
-			ut8 k = (r + g + b) / 3;
-			r = (r >= k) ? 1 : 0;
-			g = (g >= k) ? 1 : 0;
-			b = (b >= k) ? 1 : 0;
-		}
-		ut8 c = (r ? 1 : 0) + (g ? (b ? 6 : 2) : (b ? 4 : 0));
-		written = snprintf (outstr + i, sz - i, "%dm", fgbg + c);
+			bright = (r == 0x80 && g == 0x80 && b == 0x80) ? 53
+			         : (r == 0xff || g == 0xff || b == 0xff) ? 60 : 0;  // eco bright-specific
+			if (r == g && g == b) {
+				r = (r > 0x7f) ? 1 : 0;
+				g = (g > 0x7f) ? 1 : 0;
+				b = (b > 0x7f) ? 1 : 0;
+			} else {
+				ut8 k = (r + g + b) / 3;
+				r = (r >= k) ? 1 : 0;
+				g = (g >= k) ? 1 : 0;
+				b = (b >= k) ? 1 : 0;
+			}
+			c = (r ? 1 : 0) + (g ? (b ? 6 : 2) : (b ? 4 : 0));
 		}
+		written = snprintf (outstr + i, sz - i, "%dm", fgbg + bright + c);
+		break;
+	}
+	default:
 		break;
 	}
 
@@ -243,8 +254,8 @@ static void r_cons_rgb_gen(char *outstr, size_t sz, ut8 attr, ut8 a, ut8 r, ut8
 	}
 }
 
-/* Return the computed color string for the specified color */
-R_API char *r_cons_rgb_str(char *outstr, size_t sz, RColor *rcolor) {
+/* Return the computed color string for the specified color in the specified mode */
+R_API char *r_cons_rgb_str_mode(RConsColorMode mode, char *outstr, size_t sz, RColor *rcolor) {
 	if (!rcolor) {
 		return NULL;
 	}
@@ -259,15 +270,21 @@ R_API char *r_cons_rgb_str(char *outstr, size_t sz, RColor *rcolor) {
 	}
 	// If the color handles both foreground and background, also add background
 	if (rcolor->a == ALPHA_FGBG) {
-		r_cons_rgb_gen (outstr, sz, 0, ALPHA_BG, rcolor->r2, rcolor->g2, rcolor->b2);
+		r_cons_rgb_gen (mode, outstr, sz, 0, ALPHA_BG, rcolor->r2, rcolor->g2, rcolor->b2, rcolor->id16);
 	}
 	// APPEND
 	size_t len = strlen (outstr);
-	r_cons_rgb_gen (outstr + len, sz - len, rcolor->attr, rcolor->a, rcolor->r, rcolor->g, rcolor->b);
+	r_cons_rgb_gen (mode, outstr + len, sz - len, rcolor->attr, rcolor->a, rcolor->r, rcolor->g, rcolor->b,
+	                rcolor->id16);
 
 	return outstr;
 }
 
+/* Return the computed color string for the specified color */
+R_API char *r_cons_rgb_str(char *outstr, size_t sz, RColor *rcolor) {
+	return r_cons_rgb_str_mode (r_cons_singleton ()->context->color_mode, outstr, sz, rcolor);
+}
+
 R_API char *r_cons_rgb_tostring(ut8 r, ut8 g, ut8 b) {
 	const char *str = NULL;
 	if (r == 0x00 && g == b && g == 0) {
