@@ -30,6 +30,43 @@ typedef struct fcn {
 	ut64 ends;
 } fcn_t;
 
+static bool __is_data_block_cb(RAnalBlock *block, void *user) {
+	bool *block_exists = user;
+	*block_exists = true;
+	return false;
+}
+
+static int __isdata(RCore *core, ut64 addr) {
+	if (!r_io_is_valid_offset (core->io, addr, false)) {
+		// eprintf ("Warning: Invalid memory address at 0x%08"PFMT64x"\n", addr);
+		return 4;
+	}
+
+	bool block_exists = false;
+	// This will just set block_exists = true if there is any basic block at this addr
+	r_anal_blocks_foreach_in (core->anal, addr, __is_data_block_cb, &block_exists);
+	if (block_exists) {
+		return 1;
+	}
+
+	RList *list = r_meta_find_list_in (core->anal, addr, -1, 4);
+	RListIter *iter;
+	RAnalMetaItem *meta;
+	int result = 0;
+	r_list_foreach (list, iter, meta) {
+		switch (meta->type) {
+		case R_META_TYPE_DATA:
+		case R_META_TYPE_STRING:
+		case R_META_TYPE_FORMAT:
+			result = meta->size - (addr - meta->from);
+			goto exit;
+		}
+	}
+exit:
+	r_list_free (list);
+	return result;
+}
+
 static bool fcnAddBB (fcn_t *fcn, bb_t* block) {
 	if (!fcn) {
 		eprintf ("No function given to add a basic block\n");
@@ -86,7 +123,7 @@ static void initBB (bb_t *bb, ut64 start, ut64 end, ut64 jump, ut64 fail, bb_typ
 	}
 }
 
-static bool addBB (RList *block_list, ut64 start, ut64 end, ut64 jump, ut64 fail, bb_type_t type, int score) {
+static bool addBB(RList *block_list, ut64 start, ut64 end, ut64 jump, ut64 fail, bb_type_t type, int score) {
 	bb_t *bb = (bb_t*) R_NEW0 (bb_t);
 	if (!bb) {
 		eprintf ("Failed to calloc mem for new basic block!\n");
@@ -180,7 +217,7 @@ static void createFunction(RCore *core, fcn_t* fcn, const char *name) {
 		pfx = "fcn";
 	}
 
-	RAnalFunction *f = r_anal_fcn_new ();
+	RAnalFunction *f = r_anal_function_new (core->anal);
 	if (!f) {
 		eprintf ("Failed to create new function\n");
 		return;
@@ -189,17 +226,18 @@ static void createFunction(RCore *core, fcn_t* fcn, const char *name) {
 	f->name = name? strdup (name): r_str_newf ("%s.%" PFMT64x, pfx, fcn->addr);
 	f->addr = fcn->addr;
 	f->bits = core->anal->bits;
-	f->cc = r_str_const (r_anal_cc_default (core->anal));
-	r_anal_fcn_set_size (NULL, f, fcn->size);
+	f->cc = r_str_constpool_get (&core->anal->constpool, r_anal_cc_default (core->anal));
 	f->type = R_ANAL_FCN_TYPE_FCN;
 
 	r_list_foreach (fcn->bbs, fcn_iter, cur) {
+		if (__isdata (core, cur->start)) {
+			continue;
+		}
 		r_anal_fcn_add_bb (core->anal, f, cur->start, (cur->end - cur->start), cur->jump, cur->fail, 0, NULL);
 	}
-	if (!r_anal_fcn_insert (core->anal, f)) {
+	if (!r_anal_add_function (core->anal, f)) {
 		// eprintf ("Failed to insert function\n");
-		r_anal_fcn_free (f);
-		//TODO free not added function
+		r_anal_function_free (f);
 		return;
 	}
 }
@@ -234,7 +272,8 @@ R_API bool core_anal_bbs(RCore *core, const char* input) {
 		eprintf ("Analyzing [0x%08"PFMT64x"-0x%08"PFMT64x"]\n", start, start + size);
 		eprintf ("Creating basic blocks\b");
 	}
-	while (cur < size) {
+	ut64 base = cur;
+	while (cur >= base && cur < size) {
 		if (r_cons_is_breaked ()) {
 			break;
 		}
@@ -242,7 +281,17 @@ R_API bool core_anal_bbs(RCore *core, const char* input) {
 		if (block_score < invalid_instruction_barrier) {
 			break;
 		}
-		op = r_core_anal_op (core, start + cur, R_ANAL_OP_MASK_BASIC);
+		ut64 dst = start + cur;
+		if (dst < start) {
+			// fix underflow issue
+			break;
+		}
+		int dsize = __isdata (core, dst);
+		if (dsize > 0) {
+			cur += dsize;
+			continue;
+		}
+		op = r_core_anal_op (core, dst, R_ANAL_OP_MASK_BASIC | R_ANAL_OP_MASK_DISASM);
 
 		if (!op || !op->mnemonic) {
 			block_score -= 10;
@@ -252,7 +301,7 @@ R_API bool core_anal_bbs(RCore *core, const char* input) {
 
 		if (op->mnemonic[0] == '?') {
 			eprintf ("? Bad op at: 0x%08"PFMT64x"\n", cur + start);
-			eprintf ("Cannot analyze opcode at %"PFMT64x"\n", start + cur);
+			eprintf ("Cannot analyze opcode at 0x%"PFMT64x"\n", start + cur);
 			block_score -= 10;
 			cur++;
 			continue;
@@ -278,7 +327,7 @@ R_API bool core_anal_bbs(RCore *core, const char* input) {
 			block_score = 0;
 			break;
 		case R_ANAL_OP_TYPE_TRAP:
-			// we dont want to add trap stuff
+			// we don't want to add trap stuff
 			if (b_start < start + cur) {
 				addBB (block_list, b_start, start + cur, UT64_MAX, UT64_MAX, NORMAL, block_score);
 			}
@@ -524,7 +573,7 @@ R_API bool core_anal_bbs_range (RCore *core, const char* input) {
 				}
 
 				bool bFound = false;
-				// check if offset dont have into block_list, to end branch analisys
+				// check if offset don't have into block_list, to end branch analisys
 				r_list_foreach (block_list, iter, block) {
 					if ( (block->type == END || block->type == NORMAL) && b_start + cur == block->start ) {
 						bFound = true;
@@ -533,7 +582,7 @@ R_API bool core_anal_bbs_range (RCore *core, const char* input) {
 				}
 
 				if (!bFound) {
-					op = r_core_anal_op (core, b_start + cur, R_ANAL_OP_MASK_BASIC);
+					op = r_core_anal_op (core, b_start + cur, R_ANAL_OP_MASK_BASIC | R_ANAL_OP_MASK_DISASM);
 
 					if (!op || !op->mnemonic) {
 						block_score -= 10;
@@ -589,7 +638,7 @@ R_API bool core_anal_bbs_range (RCore *core, const char* input) {
 					op = NULL;
 				}
 				else {
-					// we have this offset into previous analized block, exit from this path flow.
+					// we have this offset into previous analyzed block, exit from this path flow.
 					break;
 				}
 			}
@@ -681,9 +730,9 @@ R_API bool core_anal_bbs_range (RCore *core, const char* input) {
 		r_list_append (result, block);
 	}
 
-	// finally add bb to fuction
+	// finally add bb to function
 	// we simply assume that non reached blocks
-	// dont are part of the created function
+	// don't are part of the created function
 	if (debug) {
 		eprintf ("Trying to create functions\n");
 	}
@@ -746,9 +795,9 @@ R_API bool core_anal_bbs_range (RCore *core, const char* input) {
 
 			// function creation complete
 			if (current_function) {
-				// check for supply fuction address match with current block
+				// check for supply function address match with current block
 				if (current_function->addr == start) {
-					// set supply fuction size
+					// set supply function size
 					current_function->size = size;
 					if (checkFunction (current_function)) {
 						if (input[0] == '*') {
