@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2009-2017 - pancake */
+/* radare2 - LGPL - Copyright 2009-2019 - pancake */
 
 #include "r_core.h"
 
@@ -52,6 +52,7 @@ R_API int r_core_setup_debugger (RCore *r, const char *debugbackend, bool attach
 		}
 	}
 	r_config_set (r->config, "cmd.vprompt", ".dr*");
+	r_config_set (r->config, "cmd.gprompt", ".dr*");
 	return true;
 }
 
@@ -76,8 +77,9 @@ R_API bool r_core_dump(RCore *core, const char *file, ut64 addr, ut64 size, int
 		return false;
 	}
 	/* some io backends seems to be buggy in those cases */
-	if (bs > 4096)
+	if (bs > 4096) {
 		bs = 4096;
+	}
 	buf = malloc (bs);
 	if (!buf) {
 		eprintf ("Cannot alloc %d byte(s)\n", bs);
@@ -104,15 +106,46 @@ R_API bool r_core_dump(RCore *core, const char *file, ut64 addr, ut64 size, int
 	return true;
 }
 
-R_API int r_core_write_op(RCore *core, const char *arg, char op) {
-	int i, j, len, ret = false;
+static bool __endian_swap(ut8 *buf, ut32 blocksize, ut8 len) {
+	ut32 i;
+	ut16 v16;
+	ut32 v32;
+	ut64 v64;
+	if (len != 8 && len != 4 && len != 2 && len != 1) {
+		eprintf ("Invalid word size. Use 1, 2, 4 or 8\n");
+		return false;
+	}
+	if (len == 1) {
+		return true;
+	}
+	for (i = 0; i < blocksize; i += len) {
+		switch (len) {
+		case 8:
+			v64 = r_read_at_be64 (buf, i);
+			r_write_at_le64 (buf, v64, i);
+			break;
+		case 4:
+			v32 = r_read_at_be32 (buf, i);
+			r_write_at_le32 (buf, v32, i);
+			break;
+		case 2:
+			v16 = r_read_at_be16 (buf, i);
+			r_write_at_le16 (buf, v16, i);
+			break;
+		}
+	}
+	return true;
+}
+
+R_API ut8* r_core_transform_op(RCore *core, const char *arg, char op) {
+	int i, j;
+	ut64 len;
 	char *str = NULL;
 	ut8 *buf;
 
-	// XXX we can work with config.block instead of dupping it
 	buf = (ut8 *)malloc (core->blocksize);
 	if (!buf) {
-		goto beach;
+		return NULL;
 	}
 	memcpy (buf, core->block, core->blocksize);
 
@@ -122,8 +155,9 @@ R_API int r_core_write_op(RCore *core, const char *arg, char op) {
 			// r_hex_str2bin() is guaranteed to output maximum half the
 			// input size, or 1 byte if there is just a single nibble.
 			str = (char *)malloc (strlen (arg) / 2 + 1);
-			if (!str)
+			if (!str) {
 				goto beach;
+			}
 			len = r_hex_str2bin (arg, (ut8 *)str);
 			// Output is invalid if there was just a single nibble,
 			// but in that case, len is negative (-1).
@@ -132,26 +166,22 @@ R_API int r_core_write_op(RCore *core, const char *arg, char op) {
 				goto beach;
 			}
 		} else {  // use clipboard as key
-			len = core->yank_buf->length;
-			if (len <= 0) {
-				eprintf ("Clipboard is empty and no value argument(s) given\n");
+			const ut8 *tmp = r_buf_data (core->yank_buf, &len);
+			str = r_mem_dup (tmp, len);
+			if (!str) {
 				goto beach;
 			}
-			str = r_mem_dup (core->yank_buf->buf, len);
-			if (!str)
-				goto beach;
 		}
-	} else len = 0;
+	} else {
+		len = 0;
+	}
 
 	// execute the operand
 	if (op=='e') {
 		int wordsize = 1;
 		char *os, *p, *s = strdup (arg);
-		int n, from = 0, to = 0, dif = 0, step = 1;
-		n = from = to;
+		int n = 0, from = 0, to = UT8_MAX, dif = 0, step = 1;
 		os = s;
-		to = UT8_MAX;
-		//
 		p = strchr (s, ' ');
 		if (p) {
 			*p = 0;
@@ -177,9 +207,6 @@ R_API int r_core_write_op(RCore *core, const char *arg, char op) {
 		eprintf ("from %d to %d step %d size %d\n", from, to, step, wordsize);
 		dif = (to <= from)? UT8_MAX: to - from + 1;
 		if (wordsize == 1) {
-			if (to < 1 || to > UT8_MAX) {
-				to = UT8_MAX;
-			}
 			from %= (UT8_MAX + 1);
 		}
 		if (dif < 1) {
@@ -213,70 +240,141 @@ R_API int r_core_write_op(RCore *core, const char *arg, char op) {
 		} else {
 			eprintf ("Invalid word size. Use 1, 2, 4 or 8\n");
 		}
-	} else if (op=='2' || op=='4') {
-		op -= '0';
-		// if i < core->blocksize would pass the test but buf[i+3] goes beyond the buffer
-		if (core->blocksize > 3) {
-			for (i=0; i<core->blocksize-3; i+=op) {
-				/* endian swap */
-				ut8 tmp = buf[i];
+	} else if (op == '2' || op == '4' || op == '8') { // "wo2" "wo4" "wo8"
+		int inc = op - '0';
+		ut8 tmp;
+		if (inc < 1 || inc > 8) {
+			goto beach;
+		}
+		for (i = 0; (i + inc) <= core->blocksize; i += inc) {
+			if (inc == 2) {
+				tmp = buf[i];
+				buf[i] = buf[i+1];
+				buf[i+1] = tmp;
+			} else if (inc == 4) {
+				tmp = buf[i];
 				buf[i] = buf[i+3];
 				buf[i+3] = tmp;
-				if (op == 4) {
-					tmp = buf[i + 1];
-					buf[i + 1] = buf[i + 2];
-					buf[i + 2] = tmp;
-				}
+				tmp = buf[i+1];
+				buf[i+1] = buf[i+2];
+				buf[i+2] = tmp;
+			} else if (inc == 8) {
+				tmp = buf[i];
+				buf[i] = buf[i+7];
+				buf[i+7] = tmp;
+
+				tmp = buf[i+1];
+				buf[i+1] = buf[i+6];
+				buf[i+6] = tmp;
+
+				tmp = buf[i+2];
+				buf[i+2] = buf[i+5];
+				buf[i+5] = tmp;
+
+				tmp = buf[i+3];
+				buf[i+3] = buf[i+4];
+				buf[i+4] = tmp;
+			} else {
+				eprintf ("Invalid inc, use 2, 4 or 8.\n");
+				break;
 			}
 		}
 	} else {
-		for (i=j=0; i<core->blocksize; i++) {
+		bool be = r_config_get_i (core->config, "cfg.bigendian");
+		if (!be) {
+			if (!__endian_swap ((ut8*)str, len, len)) {
+				goto beach;
+			}
+		}
+		for (i = j = 0; i < core->blocksize; i++) {
 			switch (op) {
 			case 'x': buf[i] ^= str[j]; break;
 			case 'a': buf[i] += str[j]; break;
 			case 's': buf[i] -= str[j]; break;
 			case 'm': buf[i] *= str[j]; break;
 			case 'w': buf[i] = str[j]; break;
-			case 'd': buf[i] = (str[j])? buf[i] / str[j]: 0; break;
+			case 'd': buf[i] = (str[j])? (buf[i] / str[j]): 0; break;
 			case 'r': buf[i] >>= str[j]; break;
 			case 'l': buf[i] <<= str[j]; break;
 			case 'o': buf[i] |= str[j]; break;
 			case 'A': buf[i] &= str[j]; break;
 			}
-			j++; if (j>=len) j=0; /* cyclic key */
+			j++;
+			if (j >= len) {
+				j = 0; /* cyclic key */
+			}
 		}
 	}
 
-	ret = r_core_write_at (core, core->offset, buf, core->blocksize);
-beach:
-	free (buf);
 	free (str);
+	return buf;
+beach:
+	free (str);
+	free (buf);
+	return NULL;
+}
+
+R_API int r_core_write_op(RCore *core, const char *arg, char op) {
+	ut8 *buf = r_core_transform_op(core, arg, op);
+	if (!buf) {
+		return false;
+	}
+	int ret = r_core_write_at (core, core->offset, buf, core->blocksize);
+	free (buf);
 	return ret;
 }
 
-
-static void choose_bits_anal_hints(RCore *core, ut64 addr, int *bits) {
-	RAnalRange *range;
-	RListIter *iter;
-	r_list_foreach (core->anal->bits_ranges, iter, range) {
-		if (addr >= range->from && addr < range->to) {
-			*bits = range->bits;
-			return;
+static void __choose_bits_anal_hints(RCore *core, ut64 addr, int *bits) {
+	if (core->anal) {
+		int ret =  r_anal_range_tree_find_bits_at (core->anal->rb_hints_ranges, addr);
+		if (ret) {
+			*bits = ret;
 		}
 	}
 }
 
-R_API void r_core_seek_archbits(RCore *core, ut64 addr) {
-	int bits = 0;
-	const char *arch = r_io_section_get_archbits (core->io, addr, &bits);
-	if (!bits && !core->fixedbits) {
-		//if we found bits related with anal hints pick it up
-		choose_bits_anal_hints (core, addr, &bits);
+// Get address-specific bits and arch at a certain address.
+// If there are no specific infos (i.e. asm.bits and asm.arch should apply), the bits and arch will be 0 or NULL respectively!
+R_API void r_core_arch_bits_at(RCore *core, ut64 addr, R_OUT R_NULLABLE int *bits, R_OUT R_BORROW R_NULLABLE const char **arch) {
+	int bitsval = 0;
+	const char *archval = NULL;
+	RBinObject *o = r_bin_cur_object (core->bin);
+	RBinSection *s = o ? r_bin_get_section_at (o, addr, core->io->va) : NULL;
+	if (s) {
+		if (!core->fixedarch) {
+			archval = s->arch;
+		}
+		if (!core->fixedbits && s->bits) {
+			// only enforce if there's one bits set
+			switch (s->bits) {
+			case R_SYS_BITS_16:
+			case R_SYS_BITS_32:
+			case R_SYS_BITS_64:
+				bitsval = s->bits * 8;
+				break;
+			}
+		}
 	}
-	if (bits && !core->fixedbits) {
+	if (bits && !bitsval && !core->fixedbits) {
+		//if we found bits related with anal hints pick it up
+		__choose_bits_anal_hints (core, addr, &bitsval);
+	}
+	if (bits && bitsval) {
+		*bits = bitsval;
+	}
+	if (arch && archval) {
+		*arch = archval;
+	}
+}
+
+R_API void r_core_seek_arch_bits(RCore *core, ut64 addr) {
+	int bits = 0;
+	const char *arch = NULL;
+	r_core_arch_bits_at (core, addr, &bits, &arch);
+	if (bits) {
 		r_config_set_i (core->config, "asm.bits", bits);
 	}
-	if (arch && !core->fixedarch) {
+	if (arch) {
 		r_config_set (core->config, "asm.arch", arch);
 	}
 }
@@ -286,12 +384,21 @@ R_API bool r_core_seek(RCore *core, ut64 addr, bool rb) {
 	if (rb) {
 		r_core_block_read (core);
 	}
+	if (core->binat) {
+		RBinFile *bf = r_bin_file_at (core->bin, core->offset);
+		if (bf) {
+			core->bin->cur = bf;
+			r_bin_select_bfid (core->bin, bf->id);
+			// XXX r_core_cmdf (core, "obb %d", bf->id);
+		} else {
+			core->bin->cur = NULL;
+		}
+	}
 	return core->offset == addr;
 }
 
 R_API int r_core_seek_delta(RCore *core, st64 addr) {
 	ut64 tmp = core->offset;
-	int ret;
 	if (addr == 0) {
 		return true;
 	}
@@ -307,20 +414,16 @@ R_API int r_core_seek_delta(RCore *core, st64 addr) {
 		}
 	}
 	core->offset = addr;
-	ret = r_core_seek (core, addr, 1);
-	//ret = r_core_block_read (core);
-	//if (ret == -1)
-	//	memset (core->block, 0xff, core->blocksize);
-	//	core->offset = tmp;
-	return ret;
+	return r_core_seek (core, addr, 1);
 }
 
+// TODO: kill this wrapper
 R_API bool r_core_write_at(RCore *core, ut64 addr, const ut8 *buf, int size) {
-	bool ret;
-	if (!core) {
+	r_return_val_if_fail (core && buf && addr != UT64_MAX, false);
+	if (size < 1) {
 		return false;
 	}
-	ret = r_io_write_at (core->io, addr, buf, size);
+	bool ret = r_io_write_at (core->io, addr, buf, size);
 	if (addr >= core->offset && addr <= core->offset + core->blocksize - 1) {
 		r_core_block_read (core);
 	}
@@ -353,7 +456,7 @@ R_API int r_core_shift_block(RCore *core, ut64 addr, ut64 b_size, st64 dist) {
 	}
 
 	if (b_size == 0 || b_size == (ut64) -1) {
-		res = r_io_use_fd (core->io, core->file->fd);
+		r_io_use_fd (core->io, core->file->fd);
 		file_sz = r_io_size (core->io);
 		if (file_sz == UT64_MAX) {
 			file_sz = 0;
@@ -418,7 +521,7 @@ R_API int r_core_is_valid_offset (RCore *core, ut64 offset) {
 	if (!core) {
 		eprintf ("r_core_is_valid_offset: core is NULL\n");
 		r_sys_backtrace ();
-		return R_FAIL;
+		return -1;
 	}
 	return r_io_is_valid_offset (core->io, offset, 0);
 }
