@@ -1,30 +1,35 @@
-/* radare - LGPL - Copyright 2009-2015 - pancake */
+/* radare - LGPL - Copyright 2009-2019 - pancake */
 
 #include "r_core.h"
 
 static const char *help_msg_c[] = {
 	"Usage:", "c[?dfx] [argument]", " # Compare",
 	"c", " [string]", "Compare a plain with escaped chars string",
-	"c*", " [string]", "Compare a plain with escaped chars string (output r2 commands)",
+	"c*", " [string]", "Same as above, but printing r2 commands instead",
+	"c1", " [addr]", "Compare 8 bits from current offset",
+	"c2", " [value]", "Compare a word from a math expression",
 	"c4", " [value]", "Compare a doubleword from a math expression",
 	"c8", " [value]", "Compare a quadword from a math expression",
 	"cat", " [file]", "Show contents of file (see pwd, ls)",
-	"cc", " [at] [(at)]", "Compares in two hexdump columns of block size",
-	"ccc", " [at] [(at)]", "Same as above, but only showing different lines",
-	"ccd", " [at] [(at)]", "Compares in two disasm columns of block size",
+	"cc", " [at]", "Compares in two hexdump columns of block size",
+	"ccc", " [at]", "Same as above, but only showing different lines",
+	"ccd", " [at]", "Compares in two disasm columns of block size",
+	"ccdd", " [at]", "Compares decompiler output (e cmd.pdc=pdg|pdd)",
 	// "cc", " [offset]", "code bindiff current block against offset"
 	// "cD", " [file]", "like above, but using radiff -b",
 	"cf", " [file]", "Compare contents of file at current seek",
 	"cg", "[?] [o] [file]", "Graphdiff current file and [file]",
-	"cl|cls|clear", "", "Clear screen, (clear0 to goto 0, 0 only)",
 	"cu", "[?] [addr] @at", "Compare memory hexdumps of $$ and dst in unified diff",
 	"cud", " [addr] @at", "Unified diff disasm from $$ and given address",
-	"cv", "[1248] [hexpairs] @at", "Compare 1,2,4,8-byte value",
-	"cV", "[1248] [addr] @at", "Compare 1,2,4,8-byte address contents",
+	"cv", "[1248] [hexpairs] @at", "Compare 1,2,4,8-byte (silent return in $?)",
+	"cV", "[1248] [addr] @at", "Compare 1,2,4,8-byte address contents (silent, return in $?)",
 	"cw", "[?] [us?] [...]", "Compare memory watchers",
 	"cx", " [hexpair]", "Compare hexpair string (use '.' as nibble wildcard)",
 	"cx*", " [hexpair]", "Compare hexpair string (output r2 commands)",
 	"cX", " [addr]", "Like 'cc' but using hexdiff output",
+	"", "", "",
+	"cd", " [dir]", "chdir",
+	"cl|cls|clear", "", "Clear screen, (clear0 to goto 0, 0 only)",
 	NULL
 };
 
@@ -148,6 +153,55 @@ R_API int r_core_cmpwatch_revert(RCore *core, ut64 addr) {
 	return ret;
 }
 
+static int radare_compare_words(RCore *core, ut64 of, ut64 od, int len, int ws) {
+	int i;
+	bool useColor = r_config_get_i (core->config, "scr.color") != 0;
+	utAny v0, v1;
+	RConsPrintablePalette *pal = &r_cons_singleton ()->context->pal;
+	for (i = 0; i < len; i+=ws) {
+		memset (&v0, 0, sizeof (v0));
+		memset (&v1, 0, sizeof (v1));
+		r_io_read_at (core->io, of + i, (ut8*)&v0, ws);
+		r_io_read_at (core->io, od + i, (ut8*)&v1, ws);
+		char ch = (v0.v64 == v1.v64)? '=': '!';
+		const char *color = useColor? ch == '='? "": pal->graph_false: "";
+		const char *colorEnd = useColor? Color_RESET: "";
+
+		if (useColor) {
+			r_cons_printf ("%s0x%08" PFMT64x"  "Color_RESET, pal->offset, of + i);
+		} else {
+			r_cons_printf ("0x%08" PFMT64x"  ", of + i);
+		}
+		switch (ws) {
+		case 1:
+			r_cons_printf ("%s0x%02x %c 0x%02x%s\n", color,
+				(ut32)(v0.v8 & 0xff), ch, (ut32)(v1.v8 & 0xff), colorEnd);
+			break;
+		case 2:
+			r_cons_printf ("%s0x%04hx %c 0x%04hx%s\n", color,
+				v0.v16, ch, v1.v16, colorEnd);
+			break;
+		case 4:
+			r_cons_printf ("%s0x%08"PFMT32x" %c 0x%08"PFMT32x"%s\n", color,
+				v0.v32, ch, v1.v32, colorEnd);
+			//r_core_cmdf (core, "fd@0x%"PFMT64x, v0.v32);
+			if (v0.v32 != v1.v32) {
+			//	r_core_cmdf (core, "fd@0x%"PFMT64x, v1.v32);
+			}
+			break;
+		case 8:
+			r_cons_printf ("%s0x%016"PFMT64x" %c 0x%016"PFMT64x"%s\n",
+				color, v0.v64, ch, v1.v64, colorEnd);
+			//r_core_cmdf (core, "fd@0x%"PFMT64x, v0.v64);
+			if (v0.v64 != v1.v64) {
+			//	r_core_cmdf (core, "fd@0x%"PFMT64x, v1.v64);
+			}
+			break;
+		}
+	}
+	return 0;
+}
+
 static int radare_compare_unified(RCore *core, ut64 of, ut64 od, int len) {
 	int i, min, inc = 16;
 	ut8 *f, *d;
@@ -189,9 +243,19 @@ static int radare_compare_unified(RCore *core, ut64 of, ut64 od, int len) {
 
 static int radare_compare(RCore *core, const ut8 *f, const ut8 *d, int len, int mode) {
 	int i, eq = 0;
+	PJ *pj = NULL;
 	if (len < 1) {
 		return 0;
 	}
+	if (mode == 'j') {
+		pj = pj_new ();
+		if (!pj) {
+			return -1;
+		}
+		pj_o (pj);
+		pj_k (pj, "diff_bytes");
+		pj_a (pj);
+	}
 	for (i = 0; i < len; i++) {
 		if (f[i] == d[i]) {
 			eq++;
@@ -210,11 +274,26 @@ static int radare_compare(RCore *core, const ut8 *f, const ut8 *d, int len, int
 				d[i],
 				core->offset + i);
 			break;
+		case 'j':
+			pj_o (pj);
+			pj_kn (pj, "offset", core->offset + i);
+			pj_ki (pj, "rel_offset", i);
+			pj_ki (pj, "value", (int)f[i]);
+			pj_ki (pj, "cmp_value", (int)d[i]);
+			pj_end (pj);
+			break;
 
 		}
 	}
 	if (mode == 0) {
 		eprintf ("Compare %d/%d equal bytes (%d%%)\n", eq, len, (eq / len) * 100);
+	} else if (mode == 'j') {
+		pj_end (pj);
+		pj_ki (pj, "equal_bytes", eq);
+		pj_ki (pj, "total_bytes", len);
+		pj_end (pj); // End array
+		pj_end (pj); // End object
+		r_cons_println (pj_string (pj));
 	}
 	return len - eq;
 }
@@ -276,17 +355,37 @@ static void cmd_cmp_watcher(RCore *core, const char *input) {
 
 static int cmd_cmp_disasm(RCore *core, const char *input, int mode) {
 	RAsmOp op, op2;
-	int i, j, iseq;
+	int i, j;
 	char colpad[80];
 	int hascolor = r_config_get_i (core->config, "scr.color");
 	int cols = r_config_get_i (core->config, "hex.cols") * 2;
 	ut64 off = r_num_math (core->num, input);
 	ut8 *buf = calloc (core->blocksize + 32, 1);
+	RConsPrintablePalette *pal = &r_cons_singleton ()->context->pal;
 	if (!buf) {
 		return false;
 	}
-	r_core_read_at (core, off, buf, core->blocksize + 32);
+	r_io_read_at (core->io, off, buf, core->blocksize + 32);
 	switch (mode) {
+	case 'd': // decompiler
+		{
+#if 0
+		char *a = r_core_cmd_strf (core, "pdc @ 0x%"PFMT64x, off);
+		char *b = r_core_cmd_strf (core, "pdc @ 0x%"PFMT64x, core->offset);
+		RDiff *d = r_diff_new ();
+		char *s = r_diff_buffers_unified (d, a, strlen(a), b, strlen(b));
+		r_cons_printf ("%s\n", s);
+		free (a);
+		free (b);
+		free (s);
+		r_diff_free (d);
+#else
+		r_core_cmdf (core, "pdc @ 0x%"PFMT64x">$a", off);
+		r_core_cmdf (core, "pdc @ 0x%"PFMT64x">$b", core->offset);
+		r_core_cmd0 (core, "diff $a $b;rm $a;rm $b");
+#endif
+		}
+		break;
 	case 'c': // columns
 		for (i = j = 0; i < core->blocksize && j < core->blocksize;) {
 			// dis A
@@ -300,20 +399,20 @@ static int cmd_cmp_disasm(RCore *core, const char *input, int mode) {
 				buf + j, core->blocksize - j);
 
 			// show output
-			iseq = (!strcmp (op.buf_asm, op2.buf_asm));
-			memset (colpad, ' ', sizeof(colpad));
+			bool iseq = r_strbuf_equals (&op.buf_asm, &op2.buf_asm);
+			memset (colpad, ' ', sizeof (colpad));
 			{
-				int pos = strlen (op.buf_asm);
+				int pos = strlen (r_strbuf_get (&op.buf_asm));
 				pos = (pos > cols)? 0: cols - pos;
 				colpad[pos] = 0;
 			}
 			if (hascolor) {
-				r_cons_printf (iseq? Color_GREEN: Color_RED);
+				r_cons_printf (iseq? pal->graph_true: pal->graph_false);
 			}
 			r_cons_printf (" 0x%08"PFMT64x "  %s %s",
-				core->offset + i, op.buf_asm, colpad);
+				core->offset + i, r_strbuf_get (&op.buf_asm), colpad);
 			r_cons_printf ("%c 0x%08"PFMT64x "  %s\n",
-				iseq? '=': '!', off + j, op2.buf_asm);
+				iseq? '=': '!', off + j, r_strbuf_get (&op2.buf_asm));
 			if (hascolor) {
 				r_cons_printf (Color_RESET);
 			}
@@ -340,21 +439,21 @@ static int cmd_cmp_disasm(RCore *core, const char *input, int mode) {
 				buf + j, core->blocksize - j);
 
 			// show output
-			iseq = (!strcmp (op.buf_asm, op2.buf_asm));
+			bool iseq = r_strbuf_equals (&op.buf_asm, &op2.buf_asm); // (!strcmp (op.buf_asm, op2.buf_asm));
 			if (iseq) {
 				r_cons_printf (" 0x%08"PFMT64x "  %s\n",
-					core->offset + i, op.buf_asm);
+					core->offset + i, r_strbuf_get (&op.buf_asm));
 			} else {
 				if (hascolor) {
-					r_cons_printf (Color_RED);
+					r_cons_printf (pal->graph_false);
 				}
 				r_cons_printf ("-0x%08"PFMT64x "  %s\n",
-					core->offset + i, op.buf_asm);
+					core->offset + i, r_strbuf_get (&op.buf_asm));
 				if (hascolor) {
-					r_cons_printf (Color_GREEN);
+					r_cons_printf (pal->graph_true);
 				}
 				r_cons_printf ("+0x%08"PFMT64x "  %s\n",
-					off + j, op2.buf_asm);
+					off + j, r_strbuf_get (&op2.buf_asm));
 				if (hascolor) {
 					r_cons_printf (Color_RESET);
 				}
@@ -374,28 +473,79 @@ static int cmd_cmp_disasm(RCore *core, const char *input, int mode) {
 }
 
 static int cmd_cp(void *data, const char *input) {
-	char *src, *dst;
+	RCore *core = (RCore *)data;
+	if (input[1] == '.') {
+		char *file = r_core_cmd_strf (core, "ij~{core.file}");
+		r_str_trim (file);
+		char *newfile = r_str_newf ("%s.%s", file, input + 2);
+		r_file_copy (file, newfile);
+		free (file);
+		free (newfile);
+		return true;
+	}
 	if (strlen (input) < 3) {
 		eprintf ("Usage: cp src dst\n");
+		eprintf ("Usage: cp.orig  # cp $file $file.orig\n");
 		return false;
 	}
-	src = strdup (input + 2);
-	dst = strchr (src, ' ');
-	if (dst) {
-		*dst++ = 0;
-		bool rc = r_file_copy (src, dst);
-		free (src);
-		return rc;
+	char *cmd = strdup (input + 2);
+	if (cmd) {
+		char **files = r_str_argv (cmd, NULL);
+		if (files[0] && files[1]) {
+			bool rc = r_file_copy (files[0], files[1]);
+			free (cmd);
+			r_str_argv_free (files);
+			return rc;
+		}
+		r_str_argv_free (files);
 	}
 	eprintf ("Usage: cp src dst\n");
-	free (src);
 	return false;
 }
 
+static void __core_cmp_bits (RCore *core, ut64 addr) {
+	const bool scr_color = r_config_get_i (core->config, "scr.color");
+	int i;
+	ut8 a, b;
+	r_io_read_at (core->io, core->offset, &a, 1);
+	r_io_read_at (core->io, addr, &b, 1);
+	RConsPrintablePalette *pal = &r_cons_singleton ()->context->pal;
+	const char *color = scr_color? pal->offset: "";
+	const char *color_end = scr_color? Color_RESET: "";
+	if (r_config_get_i (core->config, "hex.header")) {
+		char *n = r_str_newf ("0x%08"PFMT64x, core->offset);
+		const char *extra = r_str_pad (' ', strlen (n) - 10);
+		free (n);
+		r_cons_printf ("%s- offset -%s  7 6 5 4 3 2 1 0%s\n", color, extra, color_end);
+	}
+	color = scr_color? pal->graph_false: "";
+	color_end = scr_color? Color_RESET: "";
+
+	r_cons_printf ("%s0x%08"PFMT64x"%s  ", color, core->offset, color_end);
+	for (i = 7; i >= 0; i--) {
+		bool b0 = (a & 1<<i)? 1: 0;
+		bool b1 = (b & 1<<i)? 1: 0;
+		color = scr_color? (b0 == b1)? "": b0? pal->graph_true:pal->graph_false: "";
+		color_end = scr_color ? Color_RESET: "";
+		r_cons_printf ("%s%d%s ", color, b0, color_end);
+	}
+	color = scr_color? pal->graph_true: "";
+	color_end = scr_color? Color_RESET: "";
+	r_cons_printf ("\n%s0x%08"PFMT64x"%s  ", color, addr, color_end);
+	for (i = 7; i >= 0; i--) {
+		bool b0 = (a & 1<<i)? 1: 0;
+		bool b1 = (b & 1<<i)? 1: 0;
+		color = scr_color? (b0 == b1)? "": b1? pal->graph_true: pal->graph_false: "";
+		color_end = scr_color ? Color_RESET: "";
+		r_cons_printf ("%s%d%s ", color, b1, color_end);
+	}
+	r_cons_newline ();
+}
+
 static int cmd_cmp(void *data, const char *input) {
 	static char *oldcwd = NULL;
 	int ret = 0, i, mode = 0;
-	RCore *core = data;
+	RCore *core = (RCore *)data;
 	ut64 val = UT64_MAX;
 	char *filled;
 	ut8 *buf;
@@ -403,6 +553,7 @@ static int cmd_cmp(void *data, const char *input) {
 	ut32 v32;
 	ut64 v64;
 	FILE *fd;
+	const ut8* block = core->block;
 
 	switch (*input) {
 	case 'p':
@@ -411,13 +562,20 @@ static int cmd_cmp(void *data, const char *input) {
 	case 'a': // "cat"
 		if (input[1] == 't') {
 			const char *path = r_str_trim_ro (input + 2);
-			if (r_fs_check (core->fs, path)) {
-				r_core_cmdf (core, "mg %s", path);
+			if (*path == '$') {
+				const char *oldText = r_cmd_alias_get (core->rcmd, path, 1);
+				if (oldText) {
+					r_cons_printf ("%s\n", oldText + 1);
+				}
 			} else {
-				char *res = r_syscmd_cat (path);
-				if (res) {
-					r_cons_print (res);
-					free (res);
+				if (r_fs_check (core->fs, path)) {
+					r_core_cmdf (core, "mg %s", path);
+				} else {
+					char *res = r_syscmd_cat (path);
+					if (res) {
+						r_cons_print (res);
+						free (res);
+					}
 				}
 			}
 		}
@@ -431,17 +589,29 @@ static int cmd_cmp(void *data, const char *input) {
 			return 0;
 		}
 
-		val = radare_compare (core, core->block, (ut8 *) input + 2,
+		val = radare_compare (core, block, (ut8 *) input + 2,
 			strlen (input + 2) + 1, '*');
 		break;
 	case ' ':
 	{
 		char *str = strdup (input + 1);
 		int len = r_str_unescape (str);
-		val = radare_compare (core, core->block, (ut8 *) str, len, 0);
+		val = radare_compare (core, block, (ut8 *) str, len, 0);
 		free (str);
 	}
 	break;
+	case 'j':
+	{
+		if (input[1] != ' ') {
+			eprintf ("Usage: cj [string]\n");
+		} else {
+			char *str = strdup (input + 2);
+			int len = r_str_unescape (str);
+			val = radare_compare (core, block, (ut8 *) str, len, 'j');
+			free (str);
+		}
+	}
+	break;
 	case 'x':
 		switch (input[1]) {
 		case ' ':
@@ -468,7 +638,7 @@ static int cmd_cmp(void *data, const char *input) {
 			free (filled);
 			return false;
 		}
-		ret = r_hex_bin2str (core->block, strlen (input) / 2, (char *) buf);
+		ret = r_hex_bin2str (block, strlen (input) / 2, (char *) buf);
 		for (i = 0; i < ret * 2; i++) {
 			if (filled[i] == '.') {
 				filled[i] = buf[i];
@@ -479,7 +649,7 @@ static int cmd_cmp(void *data, const char *input) {
 		if (ret < 1) {
 			eprintf ("Cannot parse hexpair\n");
 		} else {
-			val = radare_compare (core, core->block, buf, ret, mode);
+			val = radare_compare (core, block, buf, ret, mode);
 		}
 		free (buf);
 		free (filled);
@@ -491,7 +661,7 @@ static int cmd_cmp(void *data, const char *input) {
 					    input + 1), buf, core->blocksize)) {
 				eprintf ("Cannot read hexdump\n");
 			} else {
-				val = radare_compare (core, core->block, buf, ret, mode);
+				val = radare_compare (core, block, buf, ret, mode);
 			}
 			free (buf);
 		}
@@ -512,8 +682,7 @@ static int cmd_cmp(void *data, const char *input) {
 			if (fread (buf, 1, core->blocksize, fd) < 1) {
 				eprintf ("Cannot read file %s\n", input + 2);
 			} else {
-				val = radare_compare (core, core->block,
-					buf, core->blocksize, 0);
+				val = radare_compare (core, block, buf, core->blocksize, 0);
 			}
 			fclose (fd);
 			free (buf);
@@ -522,7 +691,7 @@ static int cmd_cmp(void *data, const char *input) {
 			return false;
 		}
 		break;
-	case 'd':
+	case 'd': // "cd"
 		while (input[1] == ' ') input++;
 		if (input[1]) {
 			if (!strcmp (input + 1, "-")) {
@@ -568,21 +737,30 @@ static int cmd_cmp(void *data, const char *input) {
 			free (home);
 		}
 		break;
-	case '2':
+	case '1': // "c1"
+		__core_cmp_bits (core, r_num_math (core->num, input + 1));
+		break;
+	case '2': // "c2"
 		v16 = (ut16) r_num_math (core->num, input + 1);
-		val = radare_compare (core, core->block, (ut8 *) &v16, sizeof (v16), 0);
+		val = radare_compare (core, block, (ut8 *) &v16, sizeof (v16), 0);
 		break;
-	case '4':
+	case '4': // "c4"
 		v32 = (ut32) r_num_math (core->num, input + 1);
-		val = radare_compare (core, core->block, (ut8 *) &v32, sizeof (v32), 0);
+		val = radare_compare (core, block, (ut8 *) &v32, sizeof (v32), 0);
 		break;
-	case '8':
+	case '8': // "c8"
 		v64 = (ut64) r_num_math (core->num, input + 1);
-		val = radare_compare (core, core->block, (ut8 *) &v64, sizeof (v64), 0);
+		val = radare_compare (core, block, (ut8 *) &v64, sizeof (v64), 0);
 		break;
 	case 'c': // "cc"
-		if (input[1] == 'd') {
-			cmd_cmp_disasm (core, input + 2, 'c');
+		if (input[1] == '?') { // "cc?"
+			r_core_cmd0 (core, "c?~cc");
+		} else if (input[1] == 'd') { // "ccd"
+			if (input[2] == 'd') { // "ccdd"
+				cmd_cmp_disasm (core, input + 3, 'd');
+			} else {
+				cmd_cmp_disasm (core, input + 2, 'c');
+			}
 		} else {
 			ut32 oflags = core->print->flags;
 			ut64 addr = 0; // TOTHINK: Not sure what default address should be
@@ -598,8 +776,8 @@ static int cmd_cmp(void *data, const char *input) {
 			ut8 *b = malloc (core->blocksize);
 			if (b != NULL) {
 				memset (b, 0xff, core->blocksize);
-				r_core_read_at (core, addr, b, core->blocksize);
-				r_print_hexdiff (core->print, core->offset, core->block,
+				r_io_read_at (core->io, addr, b, core->blocksize);
+				r_print_hexdiff (core->print, core->offset, block,
 					addr, b, core->blocksize, col);
 				free (b);
 			}
@@ -650,15 +828,14 @@ static int cmd_cmp(void *data, const char *input) {
 		}
 		r_core_loadlibs (core2, R_CORE_LOADLIBS_ALL, NULL);
 		core2->io->va = core->io->va;
-		core2->anal->split = core->anal->split;
 		if (!r_core_file_open (core2, file2, 0, 0LL)) {
 			eprintf ("Cannot open diff file '%s'\n", file2);
 			r_core_free (core2);
+			r_core_bind_cons (core);
 			return false;
 		}
 		// TODO: must replicate on core1 too
 		r_config_set_i (core2->config, "io.va", true);
-		r_config_set_i (core2->config, "anal.split", true);
 		r_anal_diff_setup (core->anal, diffops, -1, -1);
 		r_anal_diff_setup (core2->anal, diffops, -1, -1);
 
@@ -669,24 +846,38 @@ static int cmd_cmp(void *data, const char *input) {
 		/* exchange a segfault with a memleak */
 		core2->config = NULL;
 		r_core_free (core2);
+		r_core_bind_cons (core);
 	}
 	break;
-	case 'u':
+	case 'u': // "cu"
 		switch (input[1]) {
+		case '.':
 		case ' ':
 			radare_compare_unified (core, core->offset,
-				r_num_math (core->num, input + 1),
+				r_num_math (core->num, input + 2),
 				core->blocksize);
 			break;
+		case '1':
+		case '2':
+		case '4':
+		case '8':
+			radare_compare_words (core, core->offset,
+				r_num_math (core->num, input + 2),
+				core->blocksize, input[1] - '0');
+			break;
 		case 'd':
 			cmd_cmp_disasm (core, input + 2, 'u');
 			break;
 		default: {
 			const char *help_msg[] = {
-				"Usage: cu", " [offset]", "# Creates a unified hex patch",
-				"cu", " $$+1 > p", "Compare current seek and +1",
+				"Usage: cu", " [offset]", "# Prints unified comparison to make hexpatches",
+				"cu", " $$+1 > p", "Compare hexpairs from  current seek and +1",
+				"cu1", " $$+1 > p", "Compare bytes from current seek and +1",
+				"cu2", " $$+1 > p", "Compare words (half, 16bit) from current seek and +1",
+				"cu4", " $$+1 > p", "Compare dwords from current seek and +1",
+				"cu8", " $$+1 > p", "Compare qwords from current seek and +1",
 				"cud", " $$+1 > p", "Compare disasm current seek and +1",
-				"wu", " p", "Apply unified hex patch",
+				"wu", " p", "Apply unified hex patch (see output of cu)",
 				NULL
 			};
 			r_core_cmd_help (core, help_msg);
@@ -696,8 +887,7 @@ static int cmd_cmp(void *data, const char *input) {
 	case '?':
 		r_core_cmd_help (core, help_msg_c);
 		break;
-	case 'v': // "cv"
-	{
+	case 'v': { // "cv"
 		int sz = input[1];
 		if (sz == ' ') {
 			switch (r_config_get_i (core->config, "asm.bits")) {
@@ -710,38 +900,42 @@ static int cmd_cmp(void *data, const char *input) {
 		}
 		// TODO: honor endian
 		switch (sz) {
-		case '1':
-		{
+		case '1': { // "cv1"
 			ut8 n = (ut8) r_num_math (core->num, input + 2);
-			if (core->block[0] == n) {
+			core->num->value = 1;
+			if (block[0] == n) {
 				r_cons_printf ("0x%08"PFMT64x "\n", core->offset);
+				core->num->value = 0;
 			}
+			break;
 		}
-		break;
-		case '2':
-		{
-			ut16 *b = (ut16 *) core->block, n = (ut16) r_num_math (core->num, input + 2);
-			if (*b == n) {
+		case '2': { // "cv2"
+			ut16 n = (ut16) r_num_math (core->num, input + 2);
+			core->num->value = 1;
+			if (core->blocksize >= 2 && *(ut16*)block == n) {
 				r_cons_printf ("0x%08"PFMT64x "\n", core->offset);
+				core->num->value = 0;
 			}
+			break;
 		}
-		break;
-		case '4':
-		{
-			ut32 *b = (ut32 *) core->block, n = (ut32) r_num_math (core->num, input + 2);
-			if (*b == n) {
+		case '4': { // "cv4"
+			ut32 n = (ut32) r_num_math (core->num, input + 2);
+			core->num->value = 1;
+			if (core->blocksize >= 4 && *(ut32*)block == n) {
 				r_cons_printf ("0x%08"PFMT64x "\n", core->offset);
+				core->num->value = 0;
 			}
+			break;
 		}
-		break;
-		case '8':
-		{
-			ut64 *b = (ut64 *) core->block, n = (ut64) r_num_math (core->num, input + 2);
-			if (*b == n) {
+		case '8': { // "cv8"
+			ut64 n = (ut64) r_num_math (core->num, input + 2);
+			core->num->value = 1;
+			if (core->blocksize >= 8 && *(ut64*)block == n) {
 				r_cons_printf ("0x%08"PFMT64x "\n", core->offset);
+				core->num->value = 0;
 			}
+			break;
 		}
-		break;
 		default:
 		case '?':
 			eprintf ("Usage: cv[1248] [num]\n"
@@ -754,8 +948,7 @@ static int cmd_cmp(void *data, const char *input) {
 		}
 	}
 	break;
-	case 'V': // "cV"
-	{
+	case 'V': { // "cV"
 		int sz = input[1];
 		if (sz == ' ') {
 			switch (r_config_get_i (core->config, "asm.bits")) {
@@ -776,9 +969,9 @@ static int cmd_cmp(void *data, const char *input) {
 			r_io_read_at (core->io, at, buf, sizeof (buf));
 			core->num->value = memcmp (buf, core->block, sz)? 1: 0;
 		}
-	}
 		break;
-	case 'l':
+	}
+	case 'l': // "cl"
 		if (strchr (input, 'f')) {
 			r_cons_flush ();
 		} else if (input[1] == 0) {
