@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2018 - pancake */
+/* radare - LGPL - Copyright 2009-2019 - pancake */
 
 #include <string.h>
 #include "r_bin.h"
@@ -20,7 +20,10 @@ static const char *help_msg_i[] = {
 	"ib", "", "Reload the current buffer for setting of the bin (use once only)",
 	"ic", "", "List classes, methods and fields",
 	"icc", "", "List classes, methods and fields in Header Format",
-	"iC", "", "Show signature info (entitlements, ...)",
+	"icg", "", "List classes as agn/age commands to create class hirearchy graphs",
+	"icq", "", "List classes, in quiet mode (just the classname)",
+	"icqq", "", "List classes, in quieter mode (only show non-system classnames)",
+	"iC", "[j]", "Show signature info (entitlements, ...)",
 	"id", "[?]", "Debug information (source lines)",
 	"idp", "", "Load pdb file information",
 	"iD", " lang sym", "demangle symbolname for given language",
@@ -47,12 +50,15 @@ static const char *help_msg_i[] = {
 	"iS.", "", "Current section",
 	"iS=", "", "Show ascii-art color bars with the section ranges",
 	"iSS", "", "List memory segments (maps with om)",
+	"it", "", "File hashes",
+	"iT", "", "File signature",
 	"iV", "", "Display file version info",
+	"iw", "", "try/catch blocks",
 	"iX", "", "Display source files used (via dwarf)",
 	"iz|izj", "", "Strings in data sections (in JSON/Base64)",
 	"izz", "", "Search for Strings in the whole binary",
 	"izzz", "", "Dump Strings from whole binary to r2 shell (for huge files)",
-	"iz-", " [addr]", "Purge string via bin.strpurge",
+	"iz-", " [addr]", "Purge string via bin.str.purge",
 	"iZ", "", "Guess size of binary program",
 	NULL
 };
@@ -114,23 +120,85 @@ static bool demangle_internal(RCore *core, const char *lang, const char *s) {
 	return true;
 }
 
-static int demangle(RCore *core, const char *s) {
-	char *p, *q;
+static bool demangle(RCore *core, const char *s) {
+	r_return_val_if_fail (core && s, false);
 	const char *ss = strchr (s, ' ');
 	if (!*s) {
-		return 0;
+		return false;
 	}
 	if (!ss) {
 		const char *lang = r_config_get (core->config, "bin.lang");
 		demangle_internal (core, lang, s);
-		return 1;
+		return true;
 	}
-	p = strdup (s);
-	q = p + (ss - s);
+	char *p = strdup (s);
+	char *q = p + (ss - s);
 	*q = 0;
 	demangle_internal (core, p, q + 1);
 	free (p);
-	return 1;
+	return true;
+}
+
+static void cmd_info_here(RCore *core, int mode) {
+	RCoreItem *item = r_core_item_at (core, core->offset);
+	if (item) {
+		PJ *pj = pj_new ();
+		pj_o (pj);
+
+		pj_ks (pj, "type", item->type);
+		pj_ks (pj, "perm", r_str_rwx_i (item->perm));
+		pj_kn (pj, "size", item->size);
+		pj_kn (pj, "addr", item->addr);
+		pj_kn (pj, "next", item->next);
+		pj_kn (pj, "prev", item->prev);
+		if (item->fcnname) {
+			pj_ks (pj, "fcnname", item->fcnname);
+		}
+		if (item->sectname) {
+			pj_ks (pj, "sectname", item->sectname);
+		}
+		if (item->comment) {
+			pj_ks (pj, "comment", item->comment);
+		}
+		RListIter *iter;
+		RAnalRef *ref;
+		if (item->data) {
+			pj_ks (pj, "data", item->data);
+		}
+		{
+			RList *refs = r_anal_refs_get (core->anal, core->offset);
+			if (refs && r_list_length (refs) > 0) {
+				pj_k (pj, "refs");
+				pj_a (pj);
+				r_list_foreach (refs, iter, ref) {
+					pj_o (pj);
+					pj_ks (pj, "type", r_anal_ref_type_tostring (ref->type));
+					pj_kn (pj, "addr", ref->addr);
+					pj_end (pj);
+				}
+				pj_end (pj);
+			}
+		}
+		{
+			RList *refs = r_anal_xrefs_get (core->anal, core->offset);
+			if (refs && r_list_length (refs) > 0) {
+				pj_k (pj, "xrefs");
+				pj_a (pj);
+				r_list_foreach (refs, iter, ref) {
+					pj_o (pj);
+					pj_ks (pj, "type", r_anal_ref_type_tostring (ref->type));
+					pj_kn (pj, "addr", ref->addr);
+					pj_end (pj);
+				}
+				pj_end (pj);
+			}
+		}
+		pj_end (pj);
+		char *s = pj_drain (pj);
+		r_cons_printf ("%s\n", s);
+		free (s);
+		r_core_item_free (item);
+	}
 }
 
 #define STR(x) (x)? (x): ""
@@ -139,7 +207,7 @@ static void r_core_file_info(RCore *core, int mode) {
 	int dbg = r_config_get_i (core->config, "cfg.debug");
 	bool io_cache = r_config_get_i (core->config, "io.cache");
 	RBinInfo *info = r_bin_get_info (core->bin);
-	RBinFile *binfile = r_core_bin_cur (core);
+	RBinFile *binfile = r_bin_cur (core->bin);
 	int fd = r_io_fd_get_current (core->io);
 	RIODesc *desc = r_io_desc_get (core->io, fd);
 	RBinPlugin *plugin = r_bin_file_cur_plugin (binfile);
@@ -152,15 +220,17 @@ static void r_core_file_info(RCore *core, int mode) {
 	if (mode == R_MODE_SIMPLE) {
 		return;
 	}
+	const char *comma = "";
 	if (info) {
 		fn = info->file;
 		if (mode == R_MODE_JSON) {
-			r_cons_printf ("\"type\":\"%s\",", STR (info->type));
+			comma = ",";
+			r_cons_printf ("\"type\":\"%s\"", STR (info->type));
 		}
 	} else {
 		fn = desc ? desc->name: NULL;
 	}
-	if (desc && mode == R_MODE_JSON) {
+	if (mode == R_MODE_JSON) {
 		const char *uri = fn;
 		if (!uri) {
 			if (desc && desc->uri && *desc->uri) {
@@ -171,7 +241,8 @@ static void r_core_file_info(RCore *core, int mode) {
 		}
 		{
 			char *escapedFile = r_str_escape_utf8_for_json (uri, -1);
-			r_cons_printf ("\"file\":\"%s\"", escapedFile);
+			r_cons_printf ("%s\"file\":\"%s\"", comma, escapedFile);
+			comma = ",";
 			free (escapedFile);
 		}
 		if (dbg) {
@@ -179,7 +250,8 @@ static void r_core_file_info(RCore *core, int mode) {
 		}
 		if (desc) {
 			ut64 fsz = r_io_desc_size (desc);
-			r_cons_printf (",\"fd\":%d", desc->fd);
+			r_cons_printf ("%s\"fd\":%d", comma, desc->fd);
+			comma = ",";
 			if (fsz != UT64_MAX) {
 				char humansz[8];
 				r_cons_printf (",\"size\":%"PFMT64d, fsz);
@@ -193,7 +265,7 @@ static void r_core_file_info(RCore *core, int mode) {
 				r_cons_printf (",\"referer\":\"%s\"", desc->referer);
 			}
 		}
-		r_cons_printf (",\"block\":%d", core->blocksize);
+		r_cons_printf ("%s\"block\":%d", comma, core->blocksize);
 		if (binfile) {
 			if (binfile->curxtr) {
 				r_cons_printf (",\"packet\":\"%s\"",
@@ -210,29 +282,12 @@ static void r_core_file_info(RCore *core, int mode) {
 		if (dbg) {
 			dbg = R_PERM_WX;
 		}
-		if (desc) {
-			pair ("blksz", sdb_fmt ("0x%"PFMT64x, (ut64) core->io->desc->obsz));
-		}
-		pair ("block", sdb_fmt ("0x%x", core->blocksize));
 		if (desc) {
 			pair ("fd", sdb_fmt ("%d", desc->fd));
 		}
 		if (fn || (desc && desc->uri)) {
 			pair ("file", fn? fn: desc->uri);
 		}
-		if (plugin) {
-			pair ("format", plugin->name);
-		}
-		if (desc) {
-			pair ("iorw", r_str_bool (io_cache || desc->perm & R_PERM_W));
-			pair ("mode", r_str_rwx_i (desc->perm & R_PERM_RWX));
-		}
-		if (binfile && binfile->curxtr) {
-			pair ("packet", binfile->curxtr->name);
-		}
-		if (desc && desc->referer && *desc->referer) {
-			pair ("referer", desc->referer);
-		}
 		if (desc) {
 			ut64 fsz = r_io_desc_size (desc);
 			if (fsz != UT64_MAX) {
@@ -242,6 +297,27 @@ static void r_core_file_info(RCore *core, int mode) {
 				pair ("humansz", humansz);
 			}
 		}
+		if (desc) {
+			pair ("mode", r_str_rwx_i (desc->perm & R_PERM_RWX));
+		}
+		if (plugin) {
+			pair ("format", plugin->name);
+		}
+		if (desc) {
+			pair ("iorw", r_str_bool (io_cache || desc->perm & R_PERM_W));
+		}
+		if (desc) {
+			pair ("blksz", sdb_fmt ("0x%"PFMT64x, (ut64) core->io->desc->obsz));
+		}
+		pair ("block", sdb_fmt ("0x%x", core->blocksize));
+		
+		if (binfile && binfile->curxtr) {
+			pair ("packet", binfile->curxtr->name);
+		}
+		if (desc && desc->referer && *desc->referer) {
+			pair ("referer", desc->referer);
+		}
+		
 		if (info) {
 			pair ("type", info->type);
 		}
@@ -284,7 +360,7 @@ static void cmd_info_bin(RCore *core, int va, int mode) {
 			}
 			r_core_bin_info (core, R_CORE_BIN_ACC_INFO, mode, va, NULL, NULL);
 		}
-		if (mode == R_MODE_JSON && array == 0) {
+		if ((mode & R_MODE_JSON) && array == 0) {
 			r_cons_strcat ("}\n");
 		}
 	} else {
@@ -306,15 +382,73 @@ static void playMsg(RCore *core, const char *n, int len) {
 	}
 }
 
+static bool is_equal_file_hashes(RList *lfile_hashes, RList *rfile_hashes, bool *equal) {
+	r_return_val_if_fail (lfile_hashes, false);
+	r_return_val_if_fail (rfile_hashes, false);
+	r_return_val_if_fail (equal, false);
+
+	*equal = true;
+	RBinFileHash *fh_l, *fh_r;
+	RListIter *hiter_l, *hiter_r;
+	r_list_foreach (lfile_hashes, hiter_l, fh_l) {
+		r_list_foreach (rfile_hashes, hiter_r, fh_r) {
+			if (strcmp (fh_l->type, fh_r->type)) {
+				continue;
+			}
+			if (!!strcmp (fh_l->hex, fh_r->hex)) {
+				*equal = false;
+				return true;
+			}
+		}
+	}
+	return true;
+}
+
+static int __r_core_bin_reload(RCore *r, const char *file, ut64 baseaddr) {
+	int result = 0;
+	RCoreFile *cf = r_core_file_cur (r);
+	if (cf) {
+		RBinFile *bf = r_bin_file_find_by_fd (r->bin, cf->fd);
+		result = r_bin_reload (r->bin, bf->id, baseaddr);
+	}
+	r_core_bin_set_env (r, r_bin_cur (r->bin));
+	return result;
+}
+
+static bool isKnownPackage(const char *cn) {
+	if (*cn == 'L') {
+		if (r_str_startswith (cn, "Lkotlin")) {
+			return true;
+		}
+		if (r_str_startswith (cn, "Lcom/google")) {
+			return true;
+		}
+		if (r_str_startswith (cn, "Lcom/facebook")) {
+			return true;
+		}
+		if (r_str_startswith (cn, "Lokhttp")) {
+			return true;
+		}
+		if (r_str_startswith (cn, "Landroid")) {
+			return true;
+		}
+		if (r_str_startswith (cn, "Lokio")) {
+			return true;
+		}
+	}
+	return false;
+}
+
 static int cmd_info(void *data, const char *input) {
 	RCore *core = (RCore *) data;
-	bool newline = r_config_get_i (core->config, "scr.interactive");
+	bool newline = r_cons_is_interactive ();
 	int fd = r_io_fd_get_current (core->io);
 	RIODesc *desc = r_io_desc_get (core->io, fd);
 	int i, va = core->io->va || core->io->debug;
 	int mode = 0; //R_MODE_SIMPLE;
 	bool rdump = false;
 	int is_array = 0;
+	bool is_izzzj = false;
 	Sdb *db;
 
 	for (i = 0; input[i] && input[i] != ' '; i++)
@@ -329,14 +463,17 @@ static int cmd_info(void *data, const char *input) {
 	if (mode == R_MODE_JSON) {
 		int suffix_shift = 0;
 		if (!strncmp (input, "SS", 2) || !strncmp (input, "ee", 2)
-			|| !strncmp (input, "zz", 2)) {
+		    || !strncmp (input, "zz", 2)) {
 			suffix_shift = 1;
 		}
 		if (strlen (input + 1 + suffix_shift) > 1) {
 			is_array = 1;
 		}
+		if (!strncmp (input, "zzz", 2)) {
+			is_izzzj = true;
+		}
 	}
-	if (is_array) {
+	if (is_array && !is_izzzj) {
 		r_cons_printf ("{");
 	}
 	if (!*input) {
@@ -347,12 +484,23 @@ static int cmd_info(void *data, const char *input) {
 		input = "I*";
 	}
 	char *question = strchr (input, '?');
-	if (question > input) {
+	const char *space = strchr (input, ' ');
+	if (!space && question) {
+		space = question + 1;
+	}
+	if (question < space && question > input) {
 		question--;
-		r_core_cmdf (core, "i?~ i%c", *question);
+		r_core_cmdf (core, "i?~& i%c", *question);
 		goto done;
 	}
+	R_FREE (core->table_query);
+	if (space && *space == ' ') {
+		core->table_query = r_str_trim_dup (space + 1);
+	}
 	while (*input) {
+		if (*input == ' ') {
+			break;
+		}
 		switch (*input) {
 		case 'b': // "ib"
 		{
@@ -364,7 +512,7 @@ static int cmd_info(void *data, const char *input) {
 			// An assumption is made that assumes there is an underlying
 			// plugin that will be used to load the bin (e.g. malloc://)
 			// TODO: Might be nice to reload a bin at a specified offset?
-			r_core_bin_reload (core, NULL, baddr);
+			__r_core_bin_reload (core, NULL, baddr);
 			r_core_block_read (core);
 			newline = false;
 		}
@@ -435,12 +583,9 @@ static int cmd_info(void *data, const char *input) {
 				r_core_bin_info (core, x, mode, va, NULL, y);
 		case 'A': // "iA"
 			newline = false;
-			if (input[1] == 'j') {
-				r_cons_printf ("{");
-				r_bin_list_archs (core->bin, 'j');
-				r_cons_printf ("}\n");
-			} else {
-				r_bin_list_archs (core->bin, 1);
+			{
+				int mode = (input[1] == 'j')? 'j': 1;
+				r_bin_list_archs (core->bin, mode);
 			}
 			break;
 		case 'E': // "iE"
@@ -455,6 +600,109 @@ static int cmd_info(void *data, const char *input) {
 			input--;
 			break;
 		}
+		case 't': // "it"
+			{
+				ut64 limit = r_config_get_i (core->config, "bin.hashlimit");
+				const char *fileName;
+				RBinInfo *info = r_bin_get_info (core->bin);
+				if (info) {
+					fileName = info->file;
+				} else {
+					int fd = r_io_fd_get_current (core->io);
+					RIODesc *desc = r_io_desc_get (core->io, fd);
+					fileName = desc? desc->name: NULL;
+				}
+				const bool is_json = input[1] == 'j'; // "itj"
+				RList *old_file_hashes = NULL;
+				if (!r_bin_file_hash (core->bin, limit, fileName, &old_file_hashes)) {
+					eprintf ("r_bin_file_hash: Cannot get file hashes");
+					r_list_free (old_file_hashes);
+					return 0;
+				}
+
+				if (old_file_hashes && r_list_empty (old_file_hashes)) {
+					// clean the old hashes list to reduce comparison operations in case it is allocated but empty
+					r_list_free (old_file_hashes);
+					old_file_hashes = NULL;
+				}
+				if (!info) {
+					info = r_bin_get_info (core->bin);
+				}
+				if (!info) {
+					eprintf ("r_bin_get_info: Cannot get bin info");
+					r_list_free (old_file_hashes);
+					return 0;
+				}
+
+				bool equal = true;
+				// check are hashes changed
+				if (!r_list_empty (old_file_hashes) && !r_list_empty (info->file_hashes)) {
+					if (!is_equal_file_hashes (old_file_hashes, info->file_hashes, &equal)) {
+						eprintf ("is_equal_file_hashes: Cannot compare file hashes");
+						r_list_free (old_file_hashes);
+						return 0;
+					}
+				}
+
+				RBinFileHash *fh_old, *fh_new;
+				RListIter *hiter_old, *hiter_new;
+
+				if (is_json) { // "itj"
+					PJ *pj = pj_new ();
+					if (!pj) {
+						eprintf ("JSON mode failed\n");
+						r_list_free (old_file_hashes);
+						return 0;
+					}
+					pj_o (pj);
+					r_list_foreach (info->file_hashes, hiter_new, fh_new) {
+						pj_ks (pj, fh_new->type ? fh_new->type : "", fh_new->hex ? fh_new->hex : "");
+					}
+					if (!equal) {
+						// print old hashes prefixed with `o` character like `omd5` and `osha1`
+						r_list_foreach (old_file_hashes, hiter_old, fh_old) {
+							char *key = r_str_newf ("o%s", fh_old->type ? fh_old->type : "");
+							pj_ks (pj, key, fh_old->hex ? fh_old->hex : "");
+							free (key);
+						}
+					}
+					pj_end (pj);
+					r_cons_printf ("%s", pj_string (pj));
+					pj_free (pj);
+				} else { // "it"
+					if (!equal) {
+						eprintf ("File has been modified.\n");
+						hiter_new = r_list_iterator (info->file_hashes);
+						hiter_old = r_list_iterator (old_file_hashes);
+						while (r_list_iter_next(hiter_new) && r_list_iter_next (hiter_old)) {
+							fh_new = (RBinFileHash*)r_list_iter_get (hiter_new);
+							fh_old = (RBinFileHash*)r_list_iter_get (hiter_old);
+							if (strcmp (fh_new->type, fh_old->type)) {
+								eprintf ("Wrong file hashes structure");
+								break;
+							}
+							if (!strcmp (fh_new->hex, fh_old->hex)) {
+								eprintf ("= %s %s\n", fh_new->type, fh_new->hex);  // output one line because hash remains same `= hashtype hashval`
+							} else {
+								// output diff-like two lines, one with old hash val `- hashtype hashval` and one with new `+ hashtype hashval`
+								eprintf ("- %s %s\n"
+									"+ %s %s\n",
+									fh_old->type, fh_old->hex,
+									fh_new->type, fh_new->hex);
+							}
+						}
+					} else { // hashes are equal
+						r_list_foreach (info->file_hashes, hiter_new, fh_new) {
+							r_cons_printf ("%s %s\n", fh_new->type ? fh_new->type : "", fh_new->hex ? fh_new->hex : "");
+						}
+					}
+				}
+
+				if (!r_list_empty (old_file_hashes)) {
+					r_list_free (old_file_hashes);
+				}
+			}
+			break;
 		case 'Z': // "iZ"
 			RBININFO ("size", R_CORE_BIN_ACC_SIZE, NULL, 0);
 			break;
@@ -480,12 +728,15 @@ static int cmd_info(void *data, const char *input) {
 				int param_shift = 0;
 				if (input[1] == 'S') {
 					name = "segments";
+					input++;
 					action = R_CORE_BIN_ACC_SEGMENTS;
 					param_shift = 1;
 				}
 				// case for iS=
 				if (input[1] == '=') {
 					mode = R_MODE_EQUAL;
+				} else if (input[1] == '*') {
+					mode = R_MODE_RADARE;
 				} else if (input[1] == 'q' && input[2] == '.') {
 					mode = R_MODE_SIMPLE;
 				} else if (input[1] == 'j' && input[2] == '.') {
@@ -493,12 +744,12 @@ static int cmd_info(void *data, const char *input) {
 				}
 				RBinObject *obj = r_bin_cur_object (core->bin);
 				if (mode == R_MODE_RADARE || mode == R_MODE_JSON || mode == R_MODE_SIMPLE) {
-					RBININFO (name, action, input + 2 + param_shift,
-						(obj && obj->sections)? r_list_length (obj->sections): 0);
-				} else {
-					RBININFO (name, action, input + 1 + param_shift,
-						(obj && obj->sections)? r_list_length (obj->sections): 0);
+					if (input[param_shift + 1]) {
+						param_shift ++;
+					}
 				}
+				RBININFO (name, action, input + 1 + param_shift,
+					(obj && obj->sections)? r_list_length (obj->sections): 0);
 			}
 			//we move input until get '\0'
 			while (*(++input)) ;
@@ -541,6 +792,12 @@ static int cmd_info(void *data, const char *input) {
 			if (input[1] == 'j' && input[2] == '.') {
 				mode = R_MODE_JSON;
 				RBININFO ("symbols", R_CORE_BIN_ACC_SYMBOLS, input + 2, (obj && obj->symbols)? r_list_length (obj->symbols): 0);
+			} else if (input[1] == 'q' && input[2] == 'q') {
+				mode = R_MODE_SIMPLEST;
+				RBININFO ("symbols", R_CORE_BIN_ACC_SYMBOLS, input + 1, (obj && obj->symbols)? r_list_length (obj->symbols): 0);
+			} else if (input[1] == 'q' && input[2] == '.') {
+				mode = R_MODE_SIMPLE;
+				RBININFO ("symbols", R_CORE_BIN_ACC_SYMBOLS, input + 2, 0);
 			} else {
 				RBININFO ("symbols", R_CORE_BIN_ACC_SYMBOLS, input + 1, (obj && obj->symbols)? r_list_length (obj->symbols): 0);
 			}
@@ -584,7 +841,7 @@ static int cmd_info(void *data, const char *input) {
 					pdbopts.symbol_store_path = (char*) r_config_get (core->config, "pdb.symstore");
 					int r = r_bin_pdb_download (core, 0, NULL, &pdbopts);
 					if (r > 0) {
-						eprintf ("Error while downloading pdb file");
+						eprintf ("Error while downloading pdb file\n");
 					}
 					input++;
 					break;
@@ -599,7 +856,7 @@ static int cmd_info(void *data, const char *input) {
 					} else {
 						/* Autodetect local file */
 						if (!info || !info->debug_file_name) {
-							eprintf ("Cannot get file's debug information");
+							eprintf ("Cannot get file's debug information\n");
 							break;
 						}
 						// Check raw path for debug filename
@@ -636,7 +893,7 @@ static int cmd_info(void *data, const char *input) {
 					}
 
 					if (!file_found) {
-						eprintf ("File '%s' not found in file directory or symbol store", r_file_basename (info->debug_file_name));
+						eprintf ("File '%s' not found in file directory or symbol store\n", r_file_basename (info->debug_file_name));
 						free (filename);
 						break;
 					}
@@ -686,10 +943,14 @@ static int cmd_info(void *data, const char *input) {
 		case 'm': // "im"
 			RBININFO ("memory", R_CORE_BIN_ACC_MEM, NULL, 0);
 			break;
+		case 'w': // "iw"
+			RBININFO ("trycatch", R_CORE_BIN_ACC_TRYCATCH, NULL, 0);
+			break;
 		case 'V': // "iV"
 			RBININFO ("versioninfo", R_CORE_BIN_ACC_VERSIONINFO, NULL, 0);
 			break;
-		case 'C': // "iC"
+		case 'T': // "iT"
+		case 'C': // "iC" // rabin2 -C create // should be deprecated and just use iT (or find a better name)
 			RBININFO ("signature", R_CORE_BIN_ACC_SIGNATURE, NULL, 0);
 			break;
 		case 'z': // "iz"
@@ -707,7 +968,7 @@ static int cmd_info(void *data, const char *input) {
 					}
 				}
 				core->tmpseek = false;
-				r_core_cmdf (core, "e bin.strpurge=%s%s0x%" PFMT64x,
+				r_core_cmdf (core, "e bin.str.purge=%s%s0x%" PFMT64x,
 				             strpurge ? strpurge : "",
 				             strpurge && *strpurge ? "," : "",
 				             addr);
@@ -738,7 +999,7 @@ static int cmd_info(void *data, const char *input) {
 				}
 				input++;
 				if (rdump) {
-					RBinFile *bf = r_core_bin_cur (core);
+					RBinFile *bf = r_bin_cur (core->bin);
 					int min = r_config_get_i (core->config, "bin.minstr");
 					if (bf) {
 						bf->strmode = mode;
@@ -762,102 +1023,157 @@ static int cmd_info(void *data, const char *input) {
 			}
 			break;
 		case 'c': // "ic"
+		// XXX this is dupe of cbin.c:bin_classes()
 			if (input[1] == '?') {
-				eprintf ("Usage: ic[ljqc*] [class-index or name]\n");
-			} else if (input[1] == ' ' || input[1] == 'q' || input[1] == 'j' || input[1] == 'l' || input[1] == 'c') {
+				eprintf ("Usage: ic[gljqc**] [class-index or name]\n");
+			} else if (input[1] == 'g') {
+				RBinClass *cls;
+				RListIter *iter;
+				RBinObject *obj = r_bin_cur_object (core->bin);
+				if (!obj) {
+					break;
+				}
+				bool fullGraph = true;
+				if (fullGraph) {
+					r_list_foreach (obj->classes, iter, cls) {
+						if (cls->super) {
+							r_cons_printf ("agn %s\n", cls->super);
+							r_cons_printf ("agn %s\n", cls->name);
+							r_cons_printf ("age %s %s\n", cls->super, cls->name);
+						} else {
+							r_cons_printf ("agn %s\n", cls->name);
+						}
+					}
+				} else {
+					r_list_foreach (obj->classes, iter, cls) {
+						if (cls->super && !strstr (cls->super, "NSObject")) {
+							r_cons_printf ("agn %s\n", cls->super);
+							r_cons_printf ("agn %s\n", cls->name);
+							r_cons_printf ("age %s %s\n", cls->super, cls->name);
+						}
+					}
+				}
+				goto done;
+			} else if (input[1] == ' ' || input[1] == 'q' || input[1] == 'j' || input[1] == 'l' || input[1] == 'c' || input[1] == '*') {
 				RBinClass *cls;
 				RBinSymbol *sym;
 				RListIter *iter, *iter2;
 				RBinObject *obj = r_bin_cur_object (core->bin);
-				if (obj) {
-					if (input[2]) {
-						int idx = -1;
-						const char * cls_name = NULL;
-						if (r_num_is_valid_input (core->num, input + 2)) {
-							idx = r_num_math (core->num, input + 2);
-						} else {
-							const char * first_char = input + ((input[1] == ' ') ? 1 : 2);
-							int not_space = strspn (first_char, " ");
-							if (first_char[not_space]) {
-								cls_name = first_char + not_space;
-							}
-						}
-						int count = 0;
-						r_list_foreach (obj->classes, iter, cls) {
-							if ((idx >= 0 && idx != count++) ||
-							   (cls_name && strcmp (cls_name, cls->name) != 0)){
-								continue;
-							}
-							switch (input[1]) {
-							case '*':
-								r_list_foreach (cls->methods, iter2, sym) {
-									r_cons_printf ("f sym.%s @ 0x%"PFMT64x "\n",
-										sym->name, sym->vaddr);
-								}
-								input++;
-								break;
-							case 'l':
-								r_list_foreach (cls->methods, iter2, sym) {
-									const char *comma = iter2->p? " ": "";
-									r_cons_printf ("%s0x%"PFMT64d, comma, sym->vaddr);
-								}
-								r_cons_newline ();
-								input++;
-								break;
-							case 'j':
-								input++;
-								r_cons_printf ("\"class\":\"%s\"", cls->name);
-								r_cons_printf (",\"methods\":[");
-								r_list_foreach (cls->methods, iter2, sym) {
-									const char *comma = iter2->p? ",": "";
-
-									if (sym->method_flags) {
-										char *flags = r_core_bin_method_flags_str (sym->method_flags, R_MODE_JSON);
-										r_cons_printf ("%s{\"name\":\"%s\",\"flags\":%s,\"vaddr\":%"PFMT64d "}",
-											comma, sym->name, flags, sym->vaddr);
-										R_FREE (flags);
-									} else {
-										r_cons_printf ("%s{\"name\":\"%s\",\"vaddr\":%"PFMT64d "}",
-											comma, sym->name, sym->vaddr);
-									}
-								}
-								r_cons_printf ("]");
-								break;
-							default:
-								r_cons_printf ("class %s\n", cls->name);
-								r_list_foreach (cls->methods, iter2, sym) {
-									char *flags = r_core_bin_method_flags_str (sym->method_flags, 0);
-									r_cons_printf ("0x%08"PFMT64x " method %s %s %s\n",
-										sym->vaddr, cls->name, flags, sym->name);
-									R_FREE (flags);
-								}
-								break;
-							}
-							goto done;
-						}
-						goto done;
+				if (!obj) {
+					break;
+				}
+				if (input[2] && input[2] != 'j' && !strstr (input, "qq")) {
+					bool radare2 = strstr (input, "**") != NULL;
+					int idx = -1;
+					const char * cls_name = NULL;
+					if (radare2) {
+						input ++;
+					}
+					if (r_num_is_valid_input (core->num, input + 2)) {
+						idx = r_num_math (core->num, input + 2);
 					} else {
-						playMsg (core, "classes", r_list_length (obj->classes));
-						if (input[1] == 'l' && obj) { // "icl"
-							r_list_foreach (obj->classes, iter, cls) {
-								r_list_foreach (cls->methods, iter2, sym) {
-									const char *comma = iter2->p? " ": "";
-									r_cons_printf ("%s0x%"PFMT64d, comma, sym->vaddr);
-								}
-								if (!r_list_empty (cls->methods)) {
-									r_cons_newline ();
-								}
-							}
-						} else if (input[1] == 'c' && obj) { // "icc"
-                					mode = R_MODE_CLASSDUMP;
-							RBININFO ("classes", R_CORE_BIN_ACC_CLASSES, NULL, r_list_length (obj->classes));
-							input = " ";
-						} else {
-							RBININFO ("classes", R_CORE_BIN_ACC_CLASSES, NULL, r_list_length (obj->classes));
+						const char * first_char = input + ((input[1] == ' ') ? 1 : 2);
+						int not_space = strspn (first_char, " ");
+						if (first_char[not_space]) {
+							cls_name = first_char + not_space;
 						}
 					}
-        			}
-			} else {
+					if (radare2) {
+						input++;
+					}
+					int count = 0;
+					int mode = input[1];
+					r_list_foreach (obj->classes, iter, cls) {
+						if (radare2) {
+							r_cons_printf ("ac %s\n", cls->name);
+							r_list_foreach (cls->methods, iter2, sym) {
+								r_cons_printf ("ac %s %s 0x%08"PFMT64x"\n", cls->name, sym->name, sym->vaddr);
+							}
+							continue;
+						}
+						if ((idx >= 0 && idx != count++) ||
+						   (cls_name && *cls_name && strcmp (cls_name, cls->name) != 0)) {
+							continue;
+						}
+						switch (mode) {
+						case '*':
+							r_list_foreach (cls->methods, iter2, sym) {
+								r_cons_printf ("f sym.%s @ 0x%"PFMT64x "\n",
+									sym->name, sym->vaddr);
+							}
+							input++;
+							break;
+						case 'l':
+							r_list_foreach (cls->methods, iter2, sym) {
+								const char *comma = iter2->p? " ": "";
+								r_cons_printf ("%s0x%"PFMT64d, comma, sym->vaddr);
+							}
+							r_cons_newline ();
+							input++;
+							break;
+						case 'j':
+							input++;
+							r_cons_printf ("\"class\":\"%s\"", cls->name);
+							r_cons_printf (",\"methods\":[");
+							r_list_foreach (cls->methods, iter2, sym) {
+								const char *comma = iter2->p? ",": "";
+
+								if (sym->method_flags) {
+									char *flags = r_core_bin_method_flags_str (sym->method_flags, R_MODE_JSON);
+									r_cons_printf ("%s{\"name\":\"%s\",\"flags\":%s,\"vaddr\":%"PFMT64d "}",
+										comma, sym->name, flags, sym->vaddr);
+									R_FREE (flags);
+								} else {
+									r_cons_printf ("%s{\"name\":\"%s\",\"vaddr\":%"PFMT64d "}",
+										comma, sym->name, sym->vaddr);
+								}
+							}
+							r_cons_printf ("]");
+							break;
+						default:
+							r_cons_printf ("class %s\n", cls->name);
+							r_list_foreach (cls->methods, iter2, sym) {
+								char *flags = r_core_bin_method_flags_str (sym->method_flags, 0);
+								r_cons_printf ("0x%08"PFMT64x " method %s %s %s\n",
+									sym->vaddr, cls->name, flags, sym->name);
+								R_FREE (flags);
+							}
+							break;
+						}
+						goto done;
+					}
+					goto done;
+				} else if (obj->classes) {
+					playMsg (core, "classes", r_list_length (obj->classes));
+					if (strstr (input, "qq")) { // "icqq"
+						r_list_foreach (obj->classes, iter, cls) {
+							if (!isKnownPackage (cls->name)) {
+								r_cons_printf ("%s\n", cls->name);
+							}
+						}
+					} else if (input[1] == 'l' && obj) { // "icl"
+						r_list_foreach (obj->classes, iter, cls) {
+							r_list_foreach (cls->methods, iter2, sym) {
+								const char *comma = iter2->p? " ": "";
+								r_cons_printf ("%s0x%"PFMT64d, comma, sym->vaddr);
+							}
+							if (!r_list_empty (cls->methods)) {
+								r_cons_newline ();
+							}
+						}
+					} else if (input[1] == 'c' && obj) { // "icc"
+						mode = R_MODE_CLASSDUMP;
+						RBININFO ("classes", R_CORE_BIN_ACC_CLASSES, NULL, r_list_length (obj->classes));
+						input = " ";
+					} else { // "icq"
+						if (input[2] == 'j') {
+							mode |= R_MODE_JSON; // default mode is R_MODE_SIMPLE
+						}
+						RBININFO ("classes", R_CORE_BIN_ACC_CLASSES, NULL, r_list_length (obj->classes));
+					}
+					goto done;
+				}
+			} else { // "ic"
 				RBinObject *obj = r_bin_cur_object (core->bin);
 				if (obj && obj->classes) {
 					int len = r_list_length (obj->classes);
@@ -882,7 +1198,12 @@ static int cmd_info(void *data, const char *input) {
 			r_core_cmd_help (core, help_msg_i);
 			goto redone;
 		case '*': // "i*"
-			mode = R_MODE_RADARE;
+			if (mode == R_MODE_RADARE) {
+				// TODO:handle ** submodes
+				mode = R_MODE_RADARE;
+			} else {
+				mode = R_MODE_RADARE;
+			}
 			goto done;
 		case 'q': // "iq"
 			mode = R_MODE_SIMPLE;
@@ -895,6 +1216,9 @@ static int cmd_info(void *data, const char *input) {
 			}
 			cmd_info_bin (core, va, mode);
 			goto done;
+		case '.': // "i."
+			cmd_info_here (core, input[1]);
+			goto done;
 		default:
 			cmd_info_bin (core, va, mode);
 			break;
@@ -910,7 +1234,7 @@ static int cmd_info(void *data, const char *input) {
 		}
 	}
 done:
-	if (is_array) {
+	if (is_array && !is_izzzj) {
 		r_cons_printf ("}\n");
 	}
 	if (newline) {
