@@ -1,6 +1,6 @@
-/* radare - LGPL - Copyright 2009-2018 - pancake */
+/* radare - LGPL - Copyright 2009-2019 - pancake */
 
-#include "r_core.h"
+#include <r_core.h>
 
 /* ugly global vars */
 static int magicdepth = 99; //XXX: do not use global var here
@@ -12,12 +12,16 @@ static void r_core_magic_reset(RCore *core) {
 	kw_count = 0;
 }
 
-static int r_core_magic_at(RCore *core, const char *file, ut64 addr, int depth, int v, bool json) {
+static int r_core_magic_at(RCore *core, const char *file, ut64 addr, int depth, int v, bool json, int *hits) {
 	const char *fmt;
 	char *q, *p;
 	const char *str;
 	int found = 0, delta = 0, adelta = 0, ret;
 	ut64 curoffset = core->offset;
+	int maxHits = r_config_get_i (core->config, "search.maxhits");
+	if (maxHits > 0 && *hits >= maxHits) {
+		return 0;
+	}
 #define NAH 32
 
 	if (--depth<0) {
@@ -63,7 +67,7 @@ static int r_core_magic_at(RCore *core, const char *file, ut64 addr, int depth,
 		if (file) {
 			free (ofile);
 			ofile = strdup (file);
-			if (r_magic_load (ck, file) == -1) {
+			if (!r_magic_load (ck, file)) {
 				eprintf ("failed r_magic_load (\"%s\") %s\n", file, r_magic_error (ck));
 				ck = NULL;
 				ret = -1;
@@ -71,7 +75,7 @@ static int r_core_magic_at(RCore *core, const char *file, ut64 addr, int depth,
 			}
 		} else {
 			const char *magicpath = r_config_get (core->config, "dir.magic");
-			if (r_magic_load (ck, magicpath) == -1) {
+			if (!r_magic_load (ck, magicpath)) {
 				ck = NULL;
 				eprintf ("failed r_magic_load (dir.magic) %s\n", r_magic_error (ck));
 				ret = -1;
@@ -113,6 +117,7 @@ static int r_core_magic_at(RCore *core, const char *file, ut64 addr, int depth,
 				strcpy (q + 1, q + ((q[2] == ' ')? 3: 2));
 			}
 		}
+		(*hits)++;
 		cmdhit = r_config_get (core->config, "cmd.hit");
 		if (cmdhit && *cmdhit) {
 			r_core_cmd0 (core, cmdhit);
@@ -152,7 +157,7 @@ static int r_core_magic_at(RCore *core, const char *file, ut64 addr, int depth,
 					if (!fmt || !*fmt) {
 						fmt = file;
 					}
-					r_core_magic_at (core, fmt, addr, depth, 1, json);
+					r_core_magic_at (core, fmt, addr, depth, 1, json, hits);
 					*q = '@';
 				}
 				break;
@@ -167,10 +172,6 @@ static int r_core_magic_at(RCore *core, const char *file, ut64 addr, int depth,
 	}
 	adelta ++;
 	delta ++;
-#if 0
-	if((core->blocksize-delta)>16)
-		goto repeat;
-#endif
 #if 0
 	r_magic_free (ck);
 	ck = NULL;
@@ -189,10 +190,14 @@ seek_exit:
 	return ret;
 }
 
-static void r_core_magic(RCore *core, const char *file, int v) {
+static void r_core_magic(RCore *core, const char *file, int v, int json) {
 	ut64 addr = core->offset;
+	int hits = 0;
 	magicdepth = r_config_get_i (core->config, "magic.depth"); // TODO: do not use global var here
-	r_core_magic_at (core, file, addr, magicdepth, v, false);
+	r_core_magic_at (core, file, addr, magicdepth, v, json, &hits);
+	if (json) {
+		r_cons_newline ();
+	}
 	if (addr != core->offset) {
 		r_core_seek (core, addr, true);
 	}
