@@ -1,10 +1,10 @@
-/* radare2 - LGPL - Copyright 2009-2018 - pancake */
+/* radare2 - LGPL - Copyright 2009-2019 - pancake */
 
 #include "r_anal.h"
 #include "r_bin.h"
 #include "r_cons.h"
 #include "r_core.h"
-#include "r_print.h"
+#include "r_util.h"
 #include "r_types.h"
 #include "sdb/sdb.h"
 
@@ -12,42 +12,55 @@ static const char *help_msg_C[] = {
 	"Usage:", "C[-LCvsdfm*?][*?] [...]", " # Metadata management",
 	"C", "", "list meta info in human friendly form",
 	"C*", "", "list meta info in r2 commands",
-	"C[Cthsdmf]", "", "list comments/types/hidden/strings/data/magic/formatted in human friendly form",
-	"C[Cthsdmf]*", "", "list comments/types/hidden/strings/data/magic/formatted in r2 commands",
+	"C*.", "", "list meta info of current offset in r2 commands",
 	"C-", " [len] [[@]addr]", "delete metadata at given address range",
-	"CL", "[-][*] [file:line] [addr]", "show or add 'code line' information (bininfo)",
-	"CS", "[-][space]", "manage meta-spaces to filter comments, etc..",
-	"CC", "[?] [-] [comment-text] [@addr]", "add/remove comment",
-	"Ct", "[?] [-] [comment-text] [@addr]", "add/remove type analysis comment",
-	"CC.", "[addr]", "show comment in current address",
+	"C.", "", "list meta info of current offset in human friendly form",
 	"CC!", " [@addr]", "edit comment with $EDITOR",
+	"CC", "[?] [-] [comment-text] [@addr]", "add/remove comment",
+	"CC.", "[addr]", "show comment in current address",
 	"CCa", "[-at]|[at] [text] [@addr]", "add/remove comment at given address",
 	"CCu", " [comment-text] [@addr]", "add unique comment",
-	"Cv", "[bsr][?]", "add comments to args",
-	"Cs", "[?] [-] [size] [@addr]", "add string",
-	"Cz", "[@addr]", "add string (see Cs?)",
-	"Ch", "[-] [size] [@addr]", "hide data",
+	"CF", "[sz] [fcn-sign..] [@addr]", "function signature",
+	"CL", "[-][*] [file:line] [addr]", "show or add 'code line' information (bininfo)",
+	"CS", "[-][space]", "manage meta-spaces to filter comments, etc..",
+	"C[Cthsdmf]", "", "list comments/types/hidden/strings/data/magic/formatted in human friendly form",
+	"C[Cthsdmf]*", "", "list comments/types/hidden/strings/data/magic/formatted in r2 commands",
 	"Cd", "[-] [size] [repeat] [@addr]", "hexdump data array (Cd 4 10 == dword [10])",
 	"Cd.", " [@addr]", "show size of data at current address",
 	"Cf", "[?][-] [sz] [0|cnt][fmt] [a0 a1...] [@addr]", "format memory (see pf?)",
-	"CF", "[sz] [fcn-sign..] [@addr]", "function signature",
+	"Ch", "[-] [size] [@addr]", "hide data",
 	"Cm", "[-] [sz] [fmt..] [@addr]", "magic parse (see pm?)",
+	"Cs", "[?] [-] [size] [@addr]", "add string",
+	"Ct", "[?] [-] [comment-text] [@addr]", "add/remove type analysis comment",
+	"Ct.", "[@addr]", "show comment at current or specified address",
+	"Cv", "[bsr][?]", "add comments to args",
+	"Cz", "[@addr]", "add string (see Cs?)",
 	NULL
 };
 
 static const char *help_msg_CC[] = {
 	"Usage:", "CC[-+!*au] [base64:..|str] @ addr", "",
+	"CC!", "", "edit comment using cfg.editor (vim, ..)",
+	"CC", " [text]", "append comment at current address",
 	"CC", "", "list all comments in human friendly form",
 	"CC*", "", "list all comments in r2 commands",
-	"CC.", "", "show comment at current offset",
-	"CC,", " [file]", "show or set comment file",
-	"CC", " [text]", "append comment at current address",
-	"CCf", "", "list comments in function",
 	"CC+", " [text]", "append comment at current address",
-	"CC!", "", "edit comment using cfg.editor (vim, ..)",
+	"CC,", " [file]", "show or set comment file",
 	"CC-", " @ cmt_addr", "remove comment at given address",
-	"CCu", " good boy @ addr", "add good boy comment at given address",
+	"CC.", "", "show comment at current offset",
+	"CCf", "", "list comments in function",
+	"CCf-", "", "delete all comments in current function",
 	"CCu", " base64:AA== @ addr", "add comment in base64",
+	"CCu", " good boy @ addr", "add good boy comment at given address",
+	NULL
+};
+
+static const char *help_msg_Ct[] = {
+	"Usage: Ct", "[.|-] [@ addr]", " # Manage comments for variable types",
+	"Ct", "", "list all variable type comments",
+	"Ct", " comment-text [@ addr]", "place comment at current or specified address",
+	"Ct.", " [@ addr]", "show comment at current or specified address",
+	"Ct-", " [@ addr]", "remove comment at current or specified address",
 	NULL
 };
 
@@ -69,16 +82,17 @@ static const char *help_msg_Cs[] = {
 	"Usage:", "Cs[ga-*.] [size] [@addr]", "",
 	"NOTE:", " size", "1 unit in bytes == width in bytes of smallest possible char in encoding,",
 	"", "", "  so ascii/latin1/utf8 = 1, utf16le = 2",
+	" Cz", " [size] [@addr]", "ditto",
+	"Cs", " [size] @addr", "add string (guess latin1/utf16le)",
 	"Cs", "", "list all strings in human friendly form",
 	"Cs*", "", "list all strings in r2 commands",
-	"Cs", " [size] @addr", "add string (guess latin1/utf16le)",
-	"Csg", " [size] [@addr]", "as above but addr not needed",
-	" Cz", " [size] [@addr]", "ditto",
-	"Csa", " [size] [@addr]", "add ascii/latin1 string",
-	"Cs8", " [size] [@addr]", "add utf8 string",
 	"Cs-", " [@addr]", "remove string",
 	"Cs.", "", "show string at current address",
 	"Cs..", "", "show string + info about it at current address",
+	"Cs.j", "", "show string at current address in JSON",
+	"Cs8", " [size] [@addr]", "add utf8 string",
+	"Csa", " [size] [@addr]", "add ascii/latin1 string",
+	"Csg", " [size] [@addr]", "as above but addr not needed",
 	NULL
 };
 
@@ -108,13 +122,13 @@ static const char *help_msg_Cvr[] = {
 
 static const char *help_msg_Cvs[] = {
 	"Usage:", "Cvs", "[name] [comment]",
-	"Cvs?", "", "show this help",
+	"Cvs!", "[name]", "edit comment using cfg editor",
 	"Cvs", "", "list all stack based args/vars comments in human friendly format",
+	"Cvs", "[name] [comment]", "add/append comment for the variable",
+	"Cvs", "[name]", "Show comments for stack pointer var/arg with that name",
 	"Cvs*", "", "list all stack based args/vars comments in r2 format",
 	"Cvs-", "[name]", "delete comments for stack pointer var/arg with that name",
-	"Cvs", "[name]", "Show comments for stack pointer var/arg with that name",
-	"Cvs", "[name] [comment]", "add/append comment for the variable",
-	"Cvs!", "[name]", "edit comment using cfg editor",
+	"Cvs?", "", "show this help",
 	NULL
 };
 
@@ -138,13 +152,13 @@ static int remove_meta_offset(RCore *core, ut64 offset) {
 	return sdb_unset (core->bin->cur->sdb_addrinfo, aoffsetptr, 0);
 }
 
-static void print_meta_offset(RCore *core, ut64 offset) {
-	int ret, line, line_old, i;
+static bool print_meta_offset(RCore *core, ut64 addr) {
+	int line, line_old, i;
 	char file[1024];
 
-	ret = r_bin_addr2line (core->bin, offset, file, sizeof (file)-1, &line);
+	int ret = r_bin_addr2line (core->bin, addr, file, sizeof (file) - 1, &line);
 	if (ret) {
-		r_cons_printf ("file %s\nline %d\n", file, line);
+		r_cons_printf ("file: %s\nline: %d\n", file, line);
 		line_old = line;
 		if (line >= 2) {
 			line -= 2;
@@ -160,12 +174,11 @@ static void print_meta_offset(RCore *core, ut64 offset) {
 		} else {
 			eprintf ("Cannot open '%s'\n", file);
 		}
-	} else {
-		eprintf ("Cannot find meta information at 0x%08"
-			PFMT64x"\n", offset);
 	}
+	return ret;
 }
 
+#if 0
 static int remove_meta_fileline(RCore *core, const char *file_line) {
 	return sdb_unset (core->bin->cur->sdb_addrinfo, file_line, 0);
 }
@@ -179,21 +192,35 @@ static int print_meta_fileline(RCore *core, const char *file_line) {
 	}
 	return 0;
 }
+#endif
+
+static ut64 filter_offset = UT64_MAX;
+static int filter_format = 0;
+static size_t filter_count = 0;
 
 static int print_addrinfo (void *user, const char *k, const char *v) {
-	char *colonpos, *subst;
-
 	ut64 offset = sdb_atoi (k);
-	if (!offset) {
+	if (!offset || offset == UT64_MAX) {
 		return true;
 	}
-	subst = strdup (v);
-	colonpos = strchr (subst, '|');
-
-	if (colonpos) {
-		*colonpos = ':';
+	char *subst = strdup (v);
+	char *colonpos = strchr (subst, '|'); // XXX keep only : for simplicity?
+	if (!colonpos) {
+		colonpos = strchr (subst, ':');
+	}
+	if (!colonpos) {
+		r_cons_printf ("%s\n", subst);
+	}
+	if (colonpos && (filter_offset == UT64_MAX || filter_offset == offset)) {
+		if (filter_format) {
+			*colonpos = ':';
+			r_cons_printf ("CL %s %s\n", k, subst);
+		} else {
+			*colonpos = 0;
+			r_cons_printf ("file: %s\nline: %s\n", subst, colonpos + 1);
+		}
+		filter_count++;
 	}
-	r_cons_printf ("CL %s %s\n", subst, k);
 	free (subst);
 
 	return true;
@@ -218,24 +245,39 @@ static int cmd_meta_add_fileline(Sdb *s, char *fileline, ut64 offset) {
 static int cmd_meta_lineinfo(RCore *core, const char *input) {
 	int ret;
 	ut64 offset = UT64_MAX; // use this as error value
-	int remove = false;
+	bool remove = false;
 	int all = false;
 	const char *p = input;
-	char *colon, *space, *file_line = 0;
+	char *file_line = NULL;
+	char *pheap = NULL;
 
 	if (*p == '?') {
-		eprintf ("Usage: CL[-][*] [file:line] [addr]");
+		eprintf ("Usage: CL[.-*?] [addr] [file:line]\n");
+		eprintf ("or: CL [addr] base64:[string]\n");
+		free (pheap);
 		return 0;
 	}
-
 	if (*p == '-') {
 		p++;
 		remove = true;
 	}
-
-	if (*p == '*') {
+	if (*p == '.') {
+		p++;
+		offset = core->offset;
+	}
+	if (*p == ' ') {
+		p = r_str_trim_ro (p + 1);
+		char *arg = strchr (p, ' ');
+		if (!arg) {
+			offset = r_num_math (core->num, p);
+			p = "";
+		}
+	} else if (*p == '*') {
 		p++;
 		all = true;
+		filter_format = '*';
+	} else {
+		filter_format = 0;
 	}
 
 	if (all) {
@@ -244,81 +286,55 @@ static int cmd_meta_lineinfo(RCore *core, const char *input) {
 		} else {
 			sdb_foreach (core->bin->cur->sdb_addrinfo, print_addrinfo, NULL);
 		}
+		free (pheap);
 		return 0;
 	}
 
-	while (*p == ' ') {
-		p++;
-	}
+	p = r_str_trim_ro (p);
+	char *myp = strdup (p);
+	char *sp = strchr (myp, ' ');
+	if (sp) {
+		*sp = 0;
+		sp++;
+		if (offset == UT64_MAX) {
+			offset = r_num_math (core->num, myp);
+		}
 
-	if (*p) {
-		offset = r_num_math (core->num, p);
-		if (!offset)
-			offset = core->offset;
-	} else {
-		offset = core->offset;
-	}
-	colon = strchr (p, ':');
-	if (colon) {
-		space = strchr (p, ' ');
-		if (!space) {
-			file_line = strdup (p);
-		} else if (space > colon) {
-			file_line = r_str_ndup (p, space - p);
-		} else {
-			goto error;
-		}
-		if (!file_line) {
-			return -1;
-		}
-		colon = strchr (file_line, ':');
-		if (!colon) {
-			goto error;
-		}
-		*colon = '|';
-		while (*p && *p != ' ') {
-			p++;
-		}
-		while (*p == ' ') {
-			p++;
-		}
-		if (*p != '\0') {
-			// TODO: use r_num_math here or something less rusty than sscanf
-			ret = sscanf (p, "0x%"PFMT64x, &offset);
-			if (ret != 1) {
-				remove = 0;
-				eprintf ("Failed to parse addr at %s\n", p);
-				// goto error;
-			} else {
-				ret = cmd_meta_add_fileline (core->bin->cur->sdb_addrinfo,
-						file_line, offset);
-				goto error;
+		if (!strncmp (sp, "base64:", 7)) {
+			int len = 0;
+			ut8 *o = sdb_decode (sp + 7, &len);
+			if (!o) {
+				eprintf ("Invalid base64\n");
+				return 0;
 			}
+			sp = pheap = (char *)o;
 		}
-		if (remove) {
-			remove_meta_fileline (core, file_line);
+		RBinFile *bf = r_bin_cur (core->bin);
+		ret = 0;
+		if (bf && bf->sdb_addrinfo) {
+			ret = cmd_meta_add_fileline (bf->sdb_addrinfo, sp, offset);
 		} else {
-			print_meta_fileline (core, file_line);
+			eprintf ("TODO: Support global SdbAddrinfo or dummy rbinfile to handlee this case\n");
 		}
 		free (file_line);
-		return 0;
+		free (myp);
+		free (pheap);
+		return ret;
 	}
-	offset = core->offset;
-
-	if (offset != UT64_MAX) {
-		if (remove) {
-			remove_meta_offset (core, offset);
-		} else {
+	free (myp);
+	if (remove) {
+		remove_meta_offset (core, offset);
+	} else {
+		// taken from r2 // TODO: we should move this addrinfo sdb logic into RBin.. use HT
+		filter_offset = offset;
+		filter_count = 0;
+		sdb_foreach (core->bin->cur->sdb_addrinfo, print_addrinfo, NULL);
+		if (filter_count == 0) {
 			print_meta_offset (core, offset);
 		}
-	} else {
-		goto error;
 	}
+	free (pheap);
 	return 0;
-
-error:
-	free (file_line);
-	return -1;
 }
 
 static int cmd_meta_comment(RCore *core, const char *input) {
@@ -375,11 +391,34 @@ static int cmd_meta_comment(RCore *core, const char *input) {
 		break;
 	case 'f': // "CCf"
 		switch (input[2]) {
+		case '-': // "CCf-"
+			{
+				ut64 arg = r_num_math (core->num, input + 2);
+				if (!arg) {
+					arg = core->offset;
+				}
+				RAnalFunction *fcn = r_anal_get_fcn_in (core->anal, arg, 0);
+				if (fcn) {
+					RAnalBlock *bb;
+					RListIter *iter;
+					r_list_foreach (fcn->bbs, iter, bb) {
+						int i;
+						for (i = 0; i < bb->size; i++) {
+							ut64 addr = bb->addr + i;
+							r_meta_del (core->anal, R_META_TYPE_COMMENT, addr, 1);
+						}
+					}
+				}
+			}
+			break;
 		case 'j': // "CCfj"
 			r_meta_list_at (core->anal, R_META_TYPE_COMMENT, 'j', core->offset);
 			break;
+		case '*': // "CCf*"
+			r_meta_list_at (core->anal, R_META_TYPE_COMMENT, 1, core->offset);
+			break;
 		default:
-			r_meta_list_at (core->anal, R_META_TYPE_COMMENT, 'f', core->offset);
+			r_meta_list_at (core->anal, R_META_TYPE_COMMENT, 0, core->offset);
 			break;
 		}
 		break;
@@ -416,6 +455,7 @@ static int cmd_meta_comment(RCore *core, const char *input) {
 				strcat (text, " ");
 				strcat (text, nc);
 				r_meta_set_string (core->anal, R_META_TYPE_COMMENT, addr, text);
+				free (comment);
 				free (text);
 			} else {
 				r_sys_perror ("malloc");
@@ -426,13 +466,20 @@ static int cmd_meta_comment(RCore *core, const char *input) {
 		free (nc);
 		}
 		break;
-	case '*':
+	case '*': // "CC*"
 		r_meta_list (core->anal, R_META_TYPE_COMMENT, 1);
 		break;
 	case '-': // "CC-"
-		r_meta_del (core->anal, R_META_TYPE_COMMENT, core->offset, 1);
+		if (input[2] == '*') { // "CC-*"
+			r_meta_del (core->anal, R_META_TYPE_COMMENT, UT64_MAX, UT64_MAX);
+		} else if (input[2]) { // "CC-$$+32"
+			ut64 arg = r_num_math (core->num, input + 2);
+			r_meta_del (core->anal, R_META_TYPE_COMMENT, arg, 1);
+		} else { // "CC-"
+			r_meta_del (core->anal, R_META_TYPE_COMMENT, core->offset, 1);
+		}
 		break;
-	case 'u':
+	case 'u': // "CCu"
 		//
 		{
 		char *newcomment;
@@ -460,7 +507,7 @@ static int cmd_meta_comment(RCore *core, const char *input) {
 		}
 		}
 		break;
-	case 'a':
+	case 'a': // "CCa"
 		{
 		char *s, *p;
 		s = strchr (input, ' ');
@@ -515,6 +562,60 @@ static int cmd_meta_comment(RCore *core, const char *input) {
 		return true;
 		}
 	}
+	return true;
+}
+
+static int cmd_meta_vartype_comment(RCore *core, const char *input) {
+	ut64 addr = core->offset;
+	switch (input[1]) {
+	case '?': // "Ct?"
+		r_core_cmd_help (core, help_msg_Ct);
+		break;
+	case 0: // "Ct"
+		r_meta_list (core->anal, R_META_TYPE_VARTYPE, 0);
+		break;
+	case ' ': // "Ct <vartype comment> @ addr"
+		{
+		const char* newcomment = r_str_trim_ro (input + 2);
+		char *text, *comment = r_meta_get_string (core->anal, R_META_TYPE_VARTYPE, addr);
+		char *nc = strdup (newcomment);
+		r_str_unescape (nc);
+		if (comment) {
+			text = malloc (strlen (comment)+ strlen (newcomment)+2);
+			if (text) {
+				strcpy (text, comment);
+				strcat (text, " ");
+				strcat (text, nc);
+				r_meta_set_string (core->anal, R_META_TYPE_VARTYPE, addr, text);
+				free (comment);
+				free (text);
+			} else {
+				r_sys_perror ("malloc");
+			}
+		} else {
+			r_meta_set_string (core->anal, R_META_TYPE_VARTYPE, addr, nc);
+		}
+		free (nc);
+		}
+		break;
+	case '.': // "Ct. @ addr"
+		{
+		ut64 at = input[2]? r_num_math (core->num, input + 2): addr;
+		char *comment = r_meta_get_string (
+				core->anal, R_META_TYPE_VARTYPE, at);
+		if (comment) {
+			r_cons_println (comment);
+			free (comment);
+		}
+		}
+		break;
+	case '-': // "Ct-"
+		r_meta_del (core->anal, R_META_TYPE_VARTYPE, core->offset, 1);
+		break;
+	default:
+		r_core_cmd_help (core, help_msg_Ct);
+		break;
+	}
 
 	return true;
 }
@@ -598,10 +699,19 @@ static int cmd_meta_others(RCore *core, const char *input) {
 		}
 		break;
 	case '.':
-		if (input[2] == '.') {
+		if (input[2] == '.') { // "Cs.."
 			RAnalMetaItem *mi = r_meta_find (core->anal, addr, type, R_META_WHERE_HERE);
 			if (mi) {
-				r_meta_print (core->anal, mi, 0, false);
+				r_meta_print (core->anal, mi, input[3], NULL, false);
+				r_meta_item_free (mi);
+			}
+			break;
+		} else if (input[2] == 'j') { // "Cs.j"
+			RAnalMetaItem *mi = r_meta_find (core->anal, addr, type, R_META_WHERE_HERE);
+			if (mi) {
+				r_meta_print (core->anal, mi, input[2], NULL, false);
+				r_cons_newline ();
+				r_meta_item_free (mi);
 			}
 			break;
 		}
@@ -615,7 +725,7 @@ static int cmd_meta_others(RCore *core, const char *input) {
 		if (!val) {
 			break;
 		}
-		if (!r_meta_deserialize_val (&mi, type, addr, val)) {
+		if (!r_meta_deserialize_val (core->anal, &mi, type, addr, val)) {
 			break;
 		}
 		if (!mi.str) {
@@ -663,8 +773,10 @@ static int cmd_meta_others(RCore *core, const char *input) {
 			if (!rep) {
 				rep = strchr (input + len, ' ');
 			}
-			if (rep) {
-				repeat = r_num_math (core->num, rep + 1);
+			if (*input == 'd') {
+				if (rep) {
+					repeat = r_num_math (core->num, rep + 1);
+				}
 			}
 		}
 		int repcnt = 0;
@@ -682,13 +794,22 @@ static int cmd_meta_others(RCore *core, const char *input) {
 				if (type == 'f') { // "Cf"
 					p = strchr (t, ' ');
 					if (p) {
+						p = (char *)r_str_trim_ro (p);
+						if (*p == '.') {
+							const char *realformat = r_print_format_byname (core->print, p + 1);
+							if (realformat) {
+								p = (char *)realformat;
+							} else {
+								eprintf ("Cannot resolve format '%s'\n", p + 1);
+								break;
+							}
+						}
 						if (n < 1) {
-							n = r_print_format_struct_size (p + 1, core->print, 0, 0);
+							n = r_print_format_struct_size (core->print, p, 0, 0);
 							if (n < 1) {
-								eprintf ("Cannot resolve struct size\n");
+								eprintf ("Warning: Cannot resolve struct size for '%s'\n", p);
 								n = 32; //
 							}
-p = t;
 						}
 						//make sure we do not overflow on r_print_format
 						if (n > core->blocksize) {
@@ -746,12 +867,15 @@ p = t;
 					RFlagItem *fi;
 					p = strchr (t, ' ');
 					if (p) {
-						*p = '\0';
-						strncpy (name, p + 1, sizeof (name)-1);
+						*p++ = '\0';
+						p = (char *)r_str_trim_ro (p);
+						strncpy (name, p, sizeof (name)-1);
 					} else {
 						if (type != 's') {
 							fi = r_flag_get_i (core->flags, addr);
-							if (fi) strncpy (name, fi->name, sizeof (name)-1);
+							if (fi) {
+								strncpy (name, fi->name, sizeof (name)-1);
+							}
 						}
 					}
 				}
@@ -817,7 +941,7 @@ void r_comment_vars(RCore *core, const char *input) {
 		return;
 	}
 	if (!fcn) {
-		eprintf ("Cant find function here\n");
+		eprintf ("Can't find function here\n");
 		return;
 	}
 	oname = name = strdup (input + 2);
@@ -825,8 +949,8 @@ void r_comment_vars(RCore *core, const char *input) {
 		name++;
 	}
 	switch (input[1]) {
-	case '*':
-	case '\0': {
+	case '*': // "Cv*"
+	case '\0': { // "Cv"
 		RList *var_list;
 		RListIter *iter;
 		var_list = r_anal_var_list (core->anal, fcn, input[0]);
@@ -844,9 +968,9 @@ void r_comment_vars(RCore *core, const char *input) {
 		}
 		}
 		break;
-	case ' ': {
+	case ' ': { // "Cv "
 		// TODO check that idx exist
-		char *comment = strstr (name, " ");
+		char *comment = strchr (name, ' ');
 		if (comment) { // new comment given
 			if (*comment) {
 				*comment++ = 0;
@@ -864,13 +988,13 @@ void r_comment_vars(RCore *core, const char *input) {
 		} else if (!strncmp (name, "-0x", 3)) {
 			idx = -(int) r_num_get (NULL, name+1);
 		} else {
-			eprintf ("cant find variable named `%s`\n",name);
+			eprintf ("can't find variable named `%s`\n",name);
 			free (heap_comment);
 			break;
 		}
 		r_anal_var_free (var);
 		if (!r_anal_var_get (core->anal, fcn->addr, input[0], 1, idx)) {
-			eprintf ("cant find variable at given offset\n");
+			eprintf ("can't find variable at given offset\n");
 		} else {
 			oldcomment = r_meta_get_var_comment (core->anal, input[0], idx, fcn->addr);
 			if (oldcomment) {
@@ -888,7 +1012,7 @@ void r_comment_vars(RCore *core, const char *input) {
 		free (heap_comment);
 		}
 		break;
-	case '-':
+	case '-': // "Cv-"
 		var = r_anal_var_get_byname (core->anal, fcn->addr, name);
 		if (var) {
 			idx = var->delta;
@@ -897,22 +1021,22 @@ void r_comment_vars(RCore *core, const char *input) {
 		} else if (!strncmp (name, "-0x", 3)) {
 			idx = -(int) r_num_get (NULL, name+1);
 		 }else {
-			eprintf ("cant find variable named `%s`\n",name);
+			eprintf ("can't find variable named `%s`\n",name);
 			break;
 		}
 		r_anal_var_free (var);
 		//XXX TODO here we leak a var
 		if (!r_anal_var_get (core->anal, fcn->addr, input[0],1,idx)) {
-			eprintf ("cant find variable at given offset\n");
+			eprintf ("can't find variable at given offset\n");
 			break;
 		}
 		r_meta_var_comment_del (core->anal, input[0], idx, fcn->addr);
 		break;
-	case '!': {
+	case '!': { // "Cv!"
 		char *comment;
 		var = r_anal_var_get_byname (core->anal, fcn->addr, name);
 		if (!var) {
-			eprintf ("cant find variable named `%s`\n",name);
+			eprintf ("can't find variable named `%s`\n",name);
 			break;
 		}
 		oldcomment = r_meta_get_var_comment (core->anal, input[0], var->delta, fcn->addr);
@@ -936,37 +1060,50 @@ static int cmd_meta(void *data, const char *input) {
 	int i;
 
 	switch (*input) {
-	case 'v': // Cr
+	case 'v': // "Cv"
 		r_comment_vars (core, input + 1);
 		break;
-	case '\0':
-	case 'j':
-	case '*':
-		r_meta_list (core->anal, R_META_TYPE_ANY, *input);
+	case '\0': // "C"
+		r_meta_list (core->anal, R_META_TYPE_ANY, 0);
 		break;
+	case 'j': // "Cj"
+	case '*': { // "C*"
+		if (!input[0] || input[1] == '.') {
+			r_meta_list_offset (core->anal, core->offset, *input);
+		} else {
+			r_meta_list (core->anal, R_META_TYPE_ANY, *input);
+		}
+		break;
+	}
+	case '.': { // "C."
+		r_meta_list_offset (core->anal, core->offset, 0);
+		break;
+	}
 	case 'L': // "CL"
 		cmd_meta_lineinfo (core, input + 1);
 		break;
 	case 'C': // "CC"
 		cmd_meta_comment (core, input);
 		break;
-	case 'r': /* Cr run command*/
-	case 'h': /* Ch comment */
-	case 's': /* Cs string */
-	case 'z': /* Cz zero-terminated string */
-	case 'd': /* "Cd" data */
-	case 'm': /* Cm magic */
-	case 'f': /* Cf formatted */
-	case 't': /* Ct type analysis commnets */
+	case 't': // "Ct" type analysis commnets
+		cmd_meta_vartype_comment (core, input);
+		break;
+	case 'r': // "Cr" run command
+	case 'h': // "Ch" comment
+	case 's': // "Cs" string
+	case 'z': // "Cz" zero-terminated string
+	case 'd': // "Cd" data
+	case 'm': // "Cm" magic
+	case 'f': // "Cf" formatted
 		cmd_meta_others (core, input);
 		break;
-	case '-':
+	case '-': // "C-"
 		if (input[1] != '*') {
 			i = input[1] ? r_num_math (core->num, input + (input[1] == ' ' ? 2 : 1)) : 1;
 			r_meta_del (core->anal, R_META_TYPE_ANY, core->offset, i);
 		} else r_meta_cleanup (core->anal, 0LL, UT64_MAX);
 		break;
-	case '?':
+	case '?': // "C?"
 		r_core_cmd_help (core, help_msg_C);
 		break;
 	case 'F': // "CF"
@@ -982,60 +1119,41 @@ static int cmd_meta(void *data, const char *input) {
 		ms = &core->anal->meta_spaces;
 		/** copypasta from `fs`.. this must be refactorized to be shared */
 		switch (input[1]) {
-		case '?':
+		case '?': // "CS?"
 			r_core_cmd_help (core, help_msg_CS);
 			break;
-		case '+':
-			r_space_push (ms, input + 2);
+		case '+': // "CS+"
+			r_spaces_push (ms, input + 2);
 			break;
-		case 'r':
+		case 'r': // "CSr"
 			if (input[2] == ' ') {
-				r_space_rename (ms, NULL, input+2);
+				r_spaces_rename (ms, NULL, input+2);
 			} else {
 				eprintf ("Usage: CSr [newname]\n");
 			}
 			break;
-		case '-':
+		case '-': // "CS-"
 			if (input[2]) {
 				if (input[2]=='*') {
-					r_space_unset (ms, NULL);
+					r_spaces_unset (ms, NULL);
 				} else {
-					r_space_unset (ms, input+2);
+					r_spaces_unset (ms, input+2);
 				}
 			} else {
-				r_space_pop (ms);
+				r_spaces_pop (ms);
 			}
 			break;
-		case 'j':
-		case '\0':
-		case '*':
-			r_space_list (ms, input[1]);
+		case 'j': // "CSj"
+		case '\0': // "CS"
+		case '*': // "CS*"
+			spaces_list (ms, input[1]);
 			break;
-		case ' ':
-			r_space_set (ms, input + 2);
+		case ' ': // "CS "
+			r_spaces_set (ms, input + 2);
 			break;
-#if 0
-		case 'm':
-			{ RFlagItem *f;
-				ut64 off = core->offset;
-				if (input[2] == ' ')
-					off = r_num_math (core->num, input+2);
-				f = r_flag_get_i (core->flags, off);
-				if (f) {
-					f->space = core->flags->space_idx;
-				} else eprintf ("Cannot find any flag at 0x%"PFMT64x".\n", off);
-			}
+		default:
+			spaces_list (ms, 0);
 			break;
-#endif
-		default: {
-				 int i, j = 0;
-				 for (i = 0; i < R_FLAG_SPACES_MAX; i++) {
-					 if (!ms->spaces[i]) continue;
-					 r_cons_printf ("%02d %c %s\n", j++,
-						 (i == ms->space_idx)?'*':' ',
-						 ms->spaces[i]);
-				 }
-			 } break;
 		}
 		break;
 	}
