@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2018 - pancake */
+/* radare - LGPL - Copyright 2009-2019 - pancake */
 
 #include "r_crypto.h"
 #include "r_config.h"
@@ -14,6 +14,7 @@ static const char *help_msg_w[] = {
 	"w6","[de] base64/hex","write base64 [d]ecoded or [e]ncoded string",
 	"wa","[?] push ebp","write opcode, separated by ';' (use '\"' around the command)",
 	"waf"," f.asm","assemble file and write bytes",
+	"waF"," f.asm","assemble file and write bytes and show 'wx' op with hexpair bytes of assembled code",
 	"wao","[?] op","modify opcode (change conditional of jump. nop, etc)",
 	"wA","[?] r 0","alter/modify opcode at current seek (see wA?)",
 	"wb"," 010203","fill current block with cyclic hexpairs",
@@ -41,9 +42,12 @@ static const char *help_msg_w[] = {
 static const char *help_msg_wa[] = {
 	"Usage:", "wa[of*] [arg]", "",
 	"wa", " nop", "write nopcode using asm.arch and asm.bits",
+	"wai", " jmp 0x8080", "write inside this op (fill with nops or error if doesnt fit)",
 	"wa*", " mov eax, 33", "show 'wx' op with hexpair bytes of assembled opcode",
 	"\"wa nop;nop\"", "" , "assemble more than one instruction (note the quotes)",
 	"waf", " f.asm" , "assemble file and write bytes",
+	"waF"," f.asm","assemble file and write bytes and show 'wx' op with hexpair bytes of assembled code",
+	"waF*"," f.asm","assemble file and show 'wx' op with hexpair bytes of assembled code",
 	"wao?", "", "show help for assembler operation on current opcode (hack)",
 	NULL
 };
@@ -88,8 +92,9 @@ static const char *help_msg_we[] = {
 static const char *help_msg_wo[] = {
 	"Usage:","wo[asmdxoArl24]"," [hexpairs] @ addr[!bsize]",
 	"wo[24aAdlmorwx]","", "without hexpair values, clipboard is used",
-	"wo2"," [val]","2=  2 byte endian swap",
-	"wo4"," [val]", "4=  4 byte endian swap",
+	"wo2"," [val]","2=  2 byte endian swap (word)",
+	"wo4"," [val]", "4=  4 byte endian swap (dword)",
+	"wo8"," [val]", "8=  8 byte endian swap (qword)",
 	"woa"," [val]", "+=  addition (f.ex: woa 0102)",
 	"woA"," [val]","&=  and",
 	"wod"," [val]", "/=  divide",
@@ -178,22 +183,29 @@ static void cmd_write_init(RCore *core) {
 	DEFINE_CMD_DESCRIPTOR (core, wx);
 }
 
-static void cmd_write_fail() {
+static void cmd_write_fail(RCore *core) {
 	eprintf ("Failed to write\n");
+	core->num->value = 1;
 }
 
 R_API int cmd_write_hexpair(RCore* core, const char* pairs) {
+	r_return_val_if_fail (core && pairs, 0);
 	ut8 *buf = malloc (strlen (pairs) + 1);
+	if (!buf) {
+		return 0;
+	}
 	int len = r_hex_str2bin (pairs, buf);
 	if (len != 0) {
 		if (len < 0) {
 			len = -len;
 			if (len < core->blocksize) {
-				buf[len-1] |= core->block[len-1] & 0xf;
+				buf[len - 1] |= core->block[len - 1] & 0xf;
 			}
 		}
+		core->num->value = 0;
 		if (!r_core_write_at (core, core->offset, buf, len)) {
-			cmd_write_fail ();
+			cmd_write_fail (core);
+			core->num->value = 1;
 		}
 		if (r_config_get_i (core->config, "cfg.wseek")) {
 			r_core_seek_delta (core, len);
@@ -201,6 +213,7 @@ R_API int cmd_write_hexpair(RCore* core, const char* pairs) {
 		r_core_block_read (core);
 	} else {
 		eprintf ("Error: invalid hexpair string\n");
+		core->num->value = 1;
 	}
 	free (buf);
 	return len;
@@ -220,6 +233,7 @@ static bool encrypt_or_decrypt_block(RCore *core, const char *algo, const char *
 	}
 	if (!no_key_mode && keylen < 1) {
 		eprintf ("%s key not defined. Use -S [key]\n", ((!direction) ? "Encryption" : "Decryption"));
+		free (binkey);
 		return false;
 	}
 	RCrypto *cry = r_crypto_new ();
@@ -248,7 +262,9 @@ static bool encrypt_or_decrypt_block(RCore *core, const char *algo, const char *
 			int result_size = 0;
 			ut8 *result = r_crypto_get_output (cry, &result_size);
 			if (result) {
-				r_io_write_at (core->io, core->offset, result, result_size);
+				if (!r_io_write_at (core->io, core->offset, result, result_size)) {
+					eprintf ("r_io_write_at failed at 0x%08"PFMT64x"\n", core->offset);
+				}
 				eprintf ("Written %d byte(s)\n", result_size);
 				free (result);
 			}
@@ -275,7 +291,7 @@ static void cmd_write_bits(RCore *core, int set, ut64 val) {
 		ret = orig & (~(val));
 	}
 	if (!r_core_write_at (core, core->offset, (const ut8*)&ret, sizeof (ret))) {
-		cmd_write_fail ();
+		cmd_write_fail (core);
 	}
 }
 
@@ -291,7 +307,9 @@ static void cmd_write_inc(RCore *core, int size, st64 num) {
 	case 8: v64 = (ut64*)core->block; *v64 += num; break;
 	}
 	// TODO: obey endian here
-        r_core_write_at (core, core->offset, core->block, size);
+	if (!r_core_write_at (core, core->offset, core->block, size)) {
+		cmd_write_fail (core);
+	}
 }
 
 static void cmd_write_op (RCore *core, const char *input) {
@@ -317,15 +335,15 @@ static void cmd_write_op (RCore *core, const char *input) {
 	case 'd':
 	case 'o':
 	case 'w':
-	case '2':
-	case '4':
+	case '2': // "wo2"
+	case '4': // "wo4"
+	case '8': // "wo8"
 		if (input[2]) {  // parse val from arg
-			r_core_write_op (core, input+3, input[1]);
-			r_core_block_read (core);
+			r_core_write_op (core, input + 3, input[1]);
 		} else {  // use clipboard instead of val
 			r_core_write_op (core, NULL, input[1]);
-			r_core_block_read (core);
 		}
+		r_core_block_read (core);
 		break;
 	case 'R':
 		r_core_cmd0 (core, "wr $b");
@@ -404,6 +422,9 @@ static void cmd_write_op (RCore *core, const char *input) {
 					} else {
 						while (true) {
 							int res = r_core_write_at (core, addr, ptr, len);
+							if (res != 0) {
+								cmd_write_fail (core);
+							}
 							if (res < 1 || len == res) {
 								break;
 							}
@@ -452,8 +473,10 @@ static void cmd_write_value (RCore *core, const char *input) {
 	int wseek = r_config_get_i (core->config, "cfg.wseek");
 	bool be = r_config_get_i (core->config, "cfg.bigendian");
 
-	if (!input)
+	core->num->value = 0;
+	if (!input) {
 		return;
+	}
 
 	if (input[0])
 	switch (input[1]) {
@@ -479,7 +502,7 @@ static void cmd_write_value (RCore *core, const char *input) {
 	case 1:
 		r_write_ble8 (buf, (ut8)(off & UT8_MAX));
 		if (!r_io_write (core->io, buf, 1)) {
-			cmd_write_fail ();
+			cmd_write_fail (core);
 		} else {
 			WSEEK (core, 1);
 		}
@@ -487,7 +510,7 @@ static void cmd_write_value (RCore *core, const char *input) {
 	case 2:
 		r_write_ble16 (buf, (ut16)(off & UT16_MAX), be);
 		if (!r_io_write (core->io, buf, 2)) {
-			cmd_write_fail ();
+			cmd_write_fail (core);
 		} else {
 			WSEEK (core, 2);
 		}
@@ -495,7 +518,7 @@ static void cmd_write_value (RCore *core, const char *input) {
 	case 4:
 		r_write_ble32 (buf, (ut32)(off & UT32_MAX), be);
 		if (!r_io_write (core->io, buf, 4)) {
-			cmd_write_fail ();
+			cmd_write_fail (core);
 		} else {
 			WSEEK (core, 4);
 		}
@@ -503,7 +526,7 @@ static void cmd_write_value (RCore *core, const char *input) {
 	case 8:
 		r_write_ble64 (buf, off, be);
 		if (!r_io_write (core->io, buf, 8)) {
-			cmd_write_fail ();
+			cmd_write_fail (core);
 		} else {
 			WSEEK (core, 8);
 		}
@@ -518,7 +541,7 @@ static bool cmd_wff(RCore *core, const char *input) {
 	// XXX: file names cannot contain spaces
 	const char *arg = input + ((input[1] == ' ') ? 2 : 1);
 	int wseek = r_config_get_i (core->config, "cfg.wseek");
-	char *p, *a = r_str_trim (strdup (arg));
+	char *p, *a = r_str_trim_dup (arg);
 	p = strchr (a, ' ');
 	if (p) {
 		*p++ = 0;
@@ -530,8 +553,10 @@ static bool cmd_wff(RCore *core, const char *input) {
 	if (!strcmp (arg, "-")) {
 		char *out = r_core_editor (core, NULL, NULL);
 		if (out) {
-			r_io_write_at (core->io, core->offset,
-				(ut8*)out, strlen (out));
+			if (!r_io_write_at (core->io, core->offset,
+				(ut8*)out, strlen (out))) {
+					eprintf ("r_io_write_at failed at 0x%08"PFMT64x"\n", core->offset);
+			}
 			r_core_block_read (core);
 			free (out);
 		}
@@ -550,7 +575,9 @@ static bool cmd_wff(RCore *core, const char *input) {
 			}
 		}
 		r_io_use_fd (core->io, core->file->fd);
-		r_io_write_at (core->io, core->offset, buf + u_offset, u_size);
+		if (!r_io_write_at (core->io, core->offset, buf + u_offset, u_size)) {
+			eprintf ("r_io_write_at failed at 0x%08"PFMT64x"\n", core->offset);
+		}
 		WSEEK (core, size);
 		free (buf);
 		r_core_block_read (core);
@@ -582,7 +609,7 @@ static bool ioMemcpy (RCore *core, ut64 dst, ut64 src, int len) {
 }
 
 static bool cmd_wfs(RCore *core, const char *input) {
-	char * args = r_str_trim (strdup (input + 1));
+	char * args = r_str_trim_dup (input + 1);
 	char *arg = strchr (args, ' ');
 	int len = core->blocksize;
 	if (arg) {
@@ -627,7 +654,7 @@ static bool cmd_wf(RCore *core, const char *input) {
 	if (input[1] == 'f') { // "wff"
 		return cmd_wff (core, input + 1);
 	}
-	char *args = r_str_trim (strdup (input + 1));
+	char *args = r_str_trim_dup (input + 1);
 	char *arg = strchr (args, ' ');
 	int len = core->blocksize;
 	if (arg) {
@@ -744,7 +771,9 @@ static int cmd_write(void *data, const char *input) {
 			if (len>0) {
 				ut8 *buf = calloc (1, len);
 				if (buf) {
-					r_io_write_at (core->io, core->offset, buf, len);
+					if (!r_io_write_at (core->io, core->offset, buf, len)) {
+						eprintf ("r_io_write_at failed at 0x%08"PFMT64x"\n", core->offset);
+					}
 					r_core_block_read (core);
 					free (buf);
 				} else eprintf ("Cannot allocate %d byte(s)\n", (int)len);
@@ -826,7 +855,9 @@ static int cmd_write(void *data, const char *input) {
 			}
 		}
 		if (!fail) {
-			r_core_write_at (core, core->offset, buf, len);
+			if (!r_core_write_at (core, core->offset, buf, len)) {
+				cmd_write_fail (core);
+			}
 			WSEEK (core, len);
 			r_core_block_read (core);
 			free (buf);
@@ -895,7 +926,9 @@ static int cmd_write(void *data, const char *input) {
 					ut64 cur_off = core->offset;
 					cmd_suc = r_core_extend_at (core, cur_off, len);
 					if (cmd_suc) {
-						r_core_write_at (core, cur_off, bytes, len);
+						if (!r_core_write_at (core, cur_off, bytes, len)) {
+							cmd_write_fail (core);
+						}
 					}
 					core->offset = cur_off;
 					r_core_block_read (core);
@@ -944,7 +977,9 @@ static int cmd_write(void *data, const char *input) {
 					//ut64 cur_off = core->offset;
 					cmd_suc = r_core_extend_at (core, addr, len);
 					if (cmd_suc) {
-						r_core_write_at (core, addr, bytes, len);
+						if (!r_core_write_at (core, addr, bytes, len)) {
+							cmd_write_fail (core);
+						}
 					}
 					core->offset = addr;
 					r_core_block_read (core);
@@ -1044,7 +1079,9 @@ static int cmd_write(void *data, const char *input) {
 				r_num_irand ();
 				for (i=0; i<len; i++)
 					buf[i] = r_num_rand (256);
-				r_core_write_at (core, core->offset, buf, len);
+				if (!r_core_write_at (core, core->offset, buf, len)) {
+					cmd_write_fail (core);
+				}
 				WSEEK (core, len);
 				free (buf);
 			} else eprintf ("Cannot allocate %d byte(s)\n", len);
@@ -1060,7 +1097,9 @@ static int cmd_write(void *data, const char *input) {
 					r_num_math (core->num, input+4));
 				eprintf ("len=%d\n", len);
 				if (len>0) {
-					r_core_write_at (core, core->offset, core->block, len);
+					if (!r_core_write_at (core, core->offset, core->block, len)) {
+						cmd_write_fail (core);
+					}
 					WSEEK (core, len);
 				} else eprintf ("r_asm_modify = %d\n", len);
 			} else eprintf ("Usage: wA [type] [value]\n");
@@ -1086,7 +1125,7 @@ static int cmd_write(void *data, const char *input) {
 			break;
 		case '+': // "wc+"
 			if (input[2]=='*') { // "wc+*"
-				//r_io_cache_reset (core->io, true);
+				//r_io_cache_reset (core->io, core->io->cached);
 				eprintf ("TODO\n");
 			} else if (input[2]==' ') { // "wc+ "
 				char *p = strchr (input+3, ' ');
@@ -1110,7 +1149,7 @@ static int cmd_write(void *data, const char *input) {
 			break;
 		case '-': { // "wc-"
 			if (input[2]=='*') { // "wc-*"
-				r_io_cache_reset (core->io, true);
+				r_io_cache_reset (core->io, core->io->cached);
 				break;
 			}
 			ut64 from, to;
@@ -1149,7 +1188,7 @@ static int cmd_write(void *data, const char *input) {
 			cmd_write_pcache (core, &input[2]);
 			break;
 		case 'r': // "wcr"
-			r_io_cache_reset (core->io, true);
+			r_io_cache_reset (core->io, core->io->cached);
 			/* Before loading the core block we have to make sure that if
 			 * the cache wrote past the original EOF these changes are no
 			 * longer displayed. */
@@ -1161,7 +1200,9 @@ static int cmd_write(void *data, const char *input) {
 	case ' ': // "w"
 		/* write string */
 		len = r_str_unescape (str);
-		r_core_write_at (core, core->offset, (const ut8*)str, len);
+		if (!r_core_write_at (core, core->offset, (const ut8*)str, len)) {
+			cmd_write_fail (core);
+		}
 #if 0
 		r_io_use_desc (core->io, core->file->desc);
 		r_io_write_at (core->io, core->offset, (const ut8*)str, len);
@@ -1172,7 +1213,9 @@ static int cmd_write(void *data, const char *input) {
 	case 'z': // "wz"
 		/* write zero-terminated string */
 		len = r_str_unescape (str);
-		r_core_write_at (core, core->offset, (const ut8*)str + 1, len);
+		if (!r_core_write_at (core, core->offset, (const ut8*)str + 1, len)) {
+			cmd_write_fail (core);
+		}
 		if (len > 0) {
 			core->num->value = len;
 		} else {
@@ -1249,21 +1292,22 @@ static int cmd_write(void *data, const char *input) {
 						r_core_cmd_help (core, help_msg_wt);
 						return 0;
 					}
-					RIOSection *s = r_io_section_vget (core->io, poff);
+					RIOMap *map = r_io_map_get (core->io, poff);
 					toend = true;
 					//use physical address
-					poff =  s ? poff - s->vaddr + s->paddr : poff;
+					poff = map ? poff - map->itv.addr + map->delta : poff;
 					str++;
 				}
-				if (*str == 'f') {
+				if (*str == 'f') { // "wtff"
 					if (str[1] == '?') {
 						r_core_cmd_help (core, help_msg_wt);
 						return 0;
 					}
-					const char *prefix = r_str_trim_ro (str + 1);
+					const char *prefix = r_str_trim_ro (str + 2);
 					if (!*prefix) {
 						prefix = "dump";
 					}
+					str++;
 					filename = r_str_newf ("%s-0x%08"PFMT64x, prefix, core->offset);
 				} else {
 					if (*str) {
@@ -1302,6 +1346,9 @@ static int cmd_write(void *data, const char *input) {
 			if (tmp) {
 				if (toend) {
 					sz = r_io_fd_size (core->io, core->file->fd) - core->offset;
+					if (sz < 0) {
+						eprintf ("Warning: File size is unknown.");
+					}
 				} else {
 					sz = (st64) r_num_math (core->num, tmp + 1);
 					*tmp = 0;
@@ -1315,6 +1362,9 @@ static int cmd_write(void *data, const char *input) {
 			} else {
 				if (toend) {
 					sz = r_io_fd_size (core->io, core->file->fd);
+					if (sz < 0) {
+						eprintf ("Warning: File size is unknown.");
+					}
 					if (sz != -1 && core->offset <= sz) {
 						sz -= core->offset;
 						if (!r_core_dump (core, filename, core->offset, (ut64)sz, append)) {
@@ -1340,6 +1390,11 @@ static int cmd_write(void *data, const char *input) {
 		cmd_wf (core, input);
 		break;
 	case 'w': // "ww"
+		len = r_str_unescape (str);
+		if (len < 1) {
+			break;
+		}
+		len++;
 		str++;
 		len = (len - 1) << 1;
 		tmp = (len > 0) ? malloc (len + 1) : NULL;
@@ -1352,7 +1407,9 @@ static int cmd_write(void *data, const char *input) {
 			if (core->file) {
 				r_io_use_fd (core->io, core->file->fd);
 			}
-			r_io_write_at (core->io, core->offset, (const ut8*)str, len);
+			if (!r_io_write_at (core->io, core->offset, (const ut8*)str, len)) {
+				eprintf ("r_io_write_at failed at 0x%08"PFMT64x"\n", core->offset);
+			}
 			WSEEK (core, len);
 			r_core_block_read (core);
 			free (tmp);
@@ -1376,7 +1433,9 @@ static int cmd_write(void *data, const char *input) {
 					if (out) {
 						len = r_hex_str2bin (in, out);
 						if (len > 0) {
-							r_io_write_at (core->io, core->offset, out, len);
+							if (!r_io_write_at (core->io, core->offset, out, len)) {
+								eprintf ("r_io_write_at failed at 0x%08"PFMT64x"\n", core->offset);
+							}
 							core->num->value = len;
 						} else {
 							core->num->value = 0;
@@ -1391,6 +1450,8 @@ static int cmd_write(void *data, const char *input) {
 					if (r_io_write_at (core->io, core->offset, buf, size) > 0) {
 						core->num->value = size;
 						WSEEK (core, size);
+					} else {
+						eprintf ("r_io_write_at failed at 0x%08"PFMT64x"\n", core->offset);
 					}
 					free (buf);
 					r_core_block_read (core);
@@ -1421,7 +1482,7 @@ static int cmd_write(void *data, const char *input) {
 		switch (input[1]) {
 		case 'o': // "wao"
 			if (input[2] == ' ') {
-				char *op = r_str_trim (strdup (input + 3));
+				char *op = r_str_trim_dup (input + 3);
 				if (op) {
 					r_core_hack (core, op);
 					free (op);
@@ -1431,52 +1492,125 @@ static int cmd_write(void *data, const char *input) {
 			}
 			break;
 		case ' ':
+		case 'i':
 		case '*': {
-			const char *file = input[1]=='*'? input + 2: input + 1;
+			const char *file = r_str_trim_ro (input + 2);
 			RAsmCode *acode;
 			r_asm_set_pc (core->assembler, core->offset);
 			acode = r_asm_massemble (core->assembler, file);
 			if (acode) {
-				if (input[1] == '*') {
-					cmd_write_hexpair (core, acode->buf_hex);
-				} else {
-					if (!r_core_write_at (core, core->offset, acode->buf, acode->len)) {
-						cmd_write_fail ();
-					} else {
-						if (r_config_get_i (core->config, "scr.prompt")) {
-							eprintf ("Written %d byte(s) (%s) = wx %s\n", acode->len, input+2, acode->buf_hex);
-						}
-						WSEEK (core, acode->len);
+				if (input[1] == 'i') { // "wai"
+					RAnalOp analop;
+					if (!r_anal_op (core->anal, &analop, core->offset, core->block, core->blocksize, R_ANAL_OP_MASK_BASIC)) {
+						eprintf ("Invalid instruction?\n");
+						break;
 					}
-					r_core_block_read (core);
+					if (analop.size < acode->len) {
+						eprintf ("Doesnt fit\n");
+						r_anal_op_fini (&analop);
+						r_asm_code_free (acode);
+						break;
+					}
+					r_anal_op_fini (&analop);
+					r_core_cmd0 (core, "wao nop");
 				}
-				r_asm_code_free (acode);
-			}
-			break;
-		}
-		case 'f': // "waf"
-			if ((input[2]==' '||input[2]=='*')) {
-				const char *file = input[2]=='*'? input+4: input+3;
-				RAsmCode *acode;
-				r_asm_set_pc (core->assembler, core->offset);
-				acode = r_asm_assemble_file (core->assembler, file);
-				if (acode) {
-					if (input[2]=='*') {
-						cmd_write_hexpair(core, acode->buf_hex);
+				if (acode->len > 0) {
+					char* hex = r_asm_code_get_hex (acode);
+					if (input[1] == '*') {
+						r_cons_printf ("wx %s\n", hex);
 					} else {
-						if (r_config_get_i (core->config, "scr.prompt")) {
-							eprintf ("Written %d byte(s) (%s)=wx %s\n", acode->len, input+1, acode->buf_hex);
-						}
-						if (!r_core_write_at (core, core->offset, acode->buf, acode->len)) {
-							cmd_write_fail ();
+						if (!r_core_write_at (core, core->offset, acode->bytes, acode->len)) {
+							cmd_write_fail (core);
 						} else {
+							if (r_config_get_i (core->config, "scr.prompt")) {
+								eprintf ("Written %d byte(s) (%s) = wx %s\n", acode->len, input+2, hex);
+							}
 							WSEEK (core, acode->len);
 						}
 						r_core_block_read (core);
 					}
-					r_asm_code_free (acode);
-				} else eprintf ("Cannot assemble file\n");
-			} else eprintf ("Wrong argument\n");
+					free (hex);
+				} else {
+					eprintf ("Nothing to do.\n");
+				}
+				r_asm_code_free (acode);
+			}
+		}
+			break;
+		case 'f': // "waf"
+			if ((input[2] == ' ' || input[2] == '*')) {
+				const char *file = input + ((input[2] == '*')? 4: 3);
+				r_asm_set_pc (core->assembler, core->offset);
+
+				char *src = r_file_slurp (file, NULL);
+				if (src) {
+					ut64 addr = core->offset, nextaddr = addr;
+					char *a, *b = src;
+					do {
+						a = strstr (b, ".offset ");
+						if (a) {
+							*a = 0;
+							a += strlen (".offset ");
+							nextaddr = r_num_math (core->num, a);
+							char *nl = strchr (a, '\n');
+							if (nl) {
+								*nl = 0;
+								a = nl + 1;
+							} else {
+								break;
+							}
+						}
+						if (*b) {
+							RAsmCode *ac = r_asm_massemble (core->assembler, b);
+							if (ac) {
+								r_io_write_at (core->io, addr, ac->bytes, ac->len);
+								r_asm_code_free (ac);
+							}
+						}
+						b = a;
+						addr = nextaddr;
+					} while (a);
+					free (src);
+				} else {
+					eprintf ("Cannot open '%s'\n", file);
+				}
+			} else {
+				eprintf ("Wrong argument\n");
+			}
+			break;
+		case 'F': // "waF"
+			if ((input[2] == ' ' || input[2] == '*')) {
+				const char *file = input + ((input[2] == '*')? 4: 3);
+				r_asm_set_pc (core->assembler, core->offset);
+				char *f = r_file_slurp (file, NULL);
+				if (f) {
+					RAsmCode *acode = r_asm_massemble (core->assembler, f);
+					if (acode) {
+						char* hex = r_asm_code_get_hex (acode);
+						if (input[2] == '*') {
+							r_cons_printf ("wx %s\n", hex);
+						} else {
+							if (r_config_get_i (core->config, "scr.prompt")) {
+								eprintf ("Written %d byte(s) (%s)=wx %s\n", acode->len, input+1, hex);
+							}
+							if (!r_core_write_at (core, core->offset, acode->bytes, acode->len)) {
+								cmd_write_fail (core);
+							} else {
+								WSEEK (core, acode->len);
+							}
+							r_core_block_read (core);
+						}
+						free (hex);
+						r_asm_code_free (acode);
+					} else {
+						eprintf ("Cannot assemble file\n");
+					}
+				} else {
+					eprintf ("Cannot slurp '%s'\n", file);
+				}
+			} else {
+				eprintf ("Wrong argument\n");
+			}
 			break;
 		default:
 			r_core_cmd_help (core, help_msg_wa);
@@ -1491,7 +1625,7 @@ static int cmd_write(void *data, const char *input) {
 			if (len > 0) {
 				r_mem_copyloop (core->block, buf, core->blocksize, len);
 				if (!r_core_write_at (core, core->offset, core->block, core->blocksize)) {
-					cmd_write_fail ();
+					cmd_write_fail (core);
 				} else {
 					WSEEK (core, core->blocksize);
 				}
@@ -1543,7 +1677,9 @@ static int cmd_write(void *data, const char *input) {
 				ut64 len = r_num_math (core->num, arg+1);
 				ut8 *data = malloc (len);
 				r_io_read_at (core->io, addr, data, len);
-				r_io_write_at (core->io, core->offset, data, len);
+				if (!r_io_write_at (core->io, core->offset, data, len)) {
+					eprintf ("r_io_write_at failed at 0x%08"PFMT64x"\n", core->offset);
+				}
 				free (data);
 			} else eprintf ("See wd?\n");
 			free (inp);
@@ -1557,8 +1693,8 @@ static int cmd_write(void *data, const char *input) {
 			} else {
 				ut8 ulen = (ut8)len;
 				if (!r_core_write_at (core, core->offset, &ulen, 1) ||
-						!r_core_write_at (core, core->offset+1, (const ut8*)str+1, len)) {
-					cmd_write_fail ();
+				!r_core_write_at (core, core->offset + 1, (const ut8*)str + 1, len)) {
+					cmd_write_fail (core);
 				} else {
 					WSEEK (core, len);
 				}
@@ -1568,15 +1704,7 @@ static int cmd_write(void *data, const char *input) {
 		break;
 	default:
 	case '?': // "w?"
-		if (core->oobi) {
-			eprintf ("Writing oobi buffer!\n");
-			r_io_use_fd (core->io, core->file->fd);
-			r_io_write (core->io, core->oobi, core->oobi_len);
-			WSEEK (core, core->oobi_len);
-			r_core_block_read (core);
-		} else {
-			r_core_cmd_help (core, help_msg_w);
-		}
+		r_core_cmd_help (core, help_msg_w);
 		break;
 	}
 	R_FREE (ostr);
