@@ -1,16 +1,17 @@
-/* radare - LGPL - Copyright 2011-2012 - pancake */
+/* radare - LGPL - Copyright 2011-2019 - pancake */
 
 #include <r_core.h>
 
 /* We can not use some kind of structure type with
  * a string for each case, because some architectures (like ARM)
- * have several modes/alignement requirements.
+ * have several modes/alignment requirements.
  */
 
 void r_core_hack_help(const RCore *core) {
 	const char* help_msg[] = {
 		"wao", " [op]", "performs a modification on current opcode",
 		"wao", " nop", "nop current opcode",
+		"wao", " jinf", "assemble an infinite loop",
 		"wao", " jz", "make current opcode conditional (zero)",
 		"wao", " jnz", "make current opcode conditional (not zero)",
 		"wao", " ret1", "make the current opcode return 1",
@@ -19,12 +20,28 @@ void r_core_hack_help(const RCore *core) {
 		"wao", " nocj", "remove conditional operation from branch (make it unconditional)",
 		"wao", " trap", "make the current opcode a trap",
 		"wao", " recj", "reverse (swap) conditional branch instruction",
-		"NOTE:", "", "those operations are only implemented for x86 and arm atm.", //TODO
+		"WIP:", "", "not all archs are supported and not all commands work on all archs",
 		NULL
 	};
 	r_core_cmd_help (core, help_msg);
 }
 
+R_API bool r_core_hack_dalvik(RCore *core, const char *op, const RAnalOp *analop) {
+	if (!strcmp (op, "nop")) {
+		r_core_cmdf (core, "wx 0000");
+	} else if (!strcmp (op, "ret2")) {
+		r_core_cmdf (core, "wx 12200f00"); // mov v0, 2;ret v0
+	} else if (!strcmp (op, "ret1")) {
+		r_core_cmdf (core, "wx 12100f00"); // mov v0, 1;ret v0
+	} else if (!strcmp (op, "ret0")) {
+		r_core_cmdf (core, "wx 12000f00"); // mov v0, 0;ret v0
+	} else {
+		eprintf ("Unsupported operation '%s'\n", op);
+		return false;
+	}
+	return true;
+}
+
 R_API bool r_core_hack_arm64(RCore *core, const char *op, const RAnalOp *analop) {
 	if (!strcmp (op, "nop")) {
 		r_core_cmdf (core, "wx 1f2003d5");
@@ -35,6 +52,8 @@ R_API bool r_core_hack_arm64(RCore *core, const char *op, const RAnalOp *analop)
 	} else if (!strcmp (op, "jz")) {
 		eprintf ("ARM jz hack not supported\n");
 		return false;
+	} else if (!strcmp (op, "jinf")) {
+		r_core_cmdf (core, "wx 00000014");
 	} else if (!strcmp (op, "jnz")) {
 		eprintf ("ARM jnz hack not supported\n");
 		return false;
@@ -82,6 +101,8 @@ R_API bool r_core_hack_arm(RCore *core, const char *op, const RAnalOp *analop) {
 		str[len*2] = '\0';
 		r_core_cmdf (core, "wx %s\n", str);
 		free (str);
+	} else if (!strcmp (op, "jinf")) {
+		r_core_cmdf (core, "wx %s\n", (bits==16)? "fee7": "feffffea");
 	} else if (!strcmp (op, "trap")) {
 		const char* trapcode = (bits==16)? "bebe": "fedeffe7";
 		r_core_cmdf (core, "wx %s\n", trapcode);
@@ -149,20 +170,23 @@ R_API bool r_core_hack_arm(RCore *core, const char *op, const RAnalOp *analop) {
 		eprintf ("TODO: use jnz or jz\n");
 		return false;
 	} else if (!strcmp (op, "ret1")) {
-		if (bits == 16)
+		if (bits == 16) {
 			r_core_cmd0 (core, "wx 01207047 @@ $$+1\n"); // mov r0, 1; bx lr
-		else
+		} else {
 			r_core_cmd0 (core, "wx 0100b0e31eff2fe1 @@ $$+1\n"); // movs r0, 1; bx lr
+		}
 	} else if (!strcmp (op, "ret0")) {
-		if (bits == 16)
+		if (bits == 16) {
 			r_core_cmd0 (core, "wx 00207047 @@ $$+1\n"); // mov r0, 0; bx lr
-		else
+		} else {
 			r_core_cmd0 (core, "wx 0000a0e31eff2fe1 @@ $$+1\n"); // movs r0, 0; bx lr
+		}
 	} else if (!strcmp (op, "retn")) {
-		if (bits == 16)
+		if (bits == 16) {
 			r_core_cmd0 (core, "wx ff207047 @@ $$+1\n"); // mov r0, -1; bx lr
-		else
+		} else {
 			r_core_cmd0 (core, "wx ff00a0e31eff2fe1 @@ $$+1\n"); // movs r0, -1; bx lr
+		}
 	} else {
 		eprintf ("Invalid operation\n");
 		return false;
@@ -174,13 +198,16 @@ R_API bool r_core_hack_x86(RCore *core, const char *op, const RAnalOp *analop) {
 	const ut8 *b = core->block;
 	int i, size = analop->size;
 	if (!strcmp (op, "nop")) {
-		if (size * 2 + 1 < size) return false;
+		if (size * 2 + 1 < size) {
+			return false;
+		}
 		char *str = malloc (size * 2 + 1);
 		if (!str) {
 			return false;
 		}
-		for (i = 0; i < size; i++)
+		for (i = 0; i < size; i++) {
 			memcpy (str + (i * 2), "90", 2);
+		}
 		str[size*2] = '\0';
 		r_core_cmdf (core, "wx %s\n", str);
 		free (str);
@@ -193,6 +220,8 @@ R_API bool r_core_hack_x86(RCore *core, const char *op, const RAnalOp *analop) {
 			eprintf ("Current opcode is not conditional\n");
 			return false;
 		}
+	} else if (!strcmp (op, "jinf")) {
+		r_core_cmd0 (core, "wx ebfe\n");
 	} else if (!strcmp (op, "jnz")) {
 		if (b[0] == 0x74) {
 			r_core_cmd0 (core, "wx 75\n");
@@ -242,6 +271,8 @@ R_API int r_core_hack(RCore *core, const char *op) {
 	}
 	if (strstr (asmarch, "x86")) {
 		hack = r_core_hack_x86;
+	} else if (strstr (asmarch, "dalvik")) {
+		hack = r_core_hack_dalvik;
 	} else if (strstr (asmarch, "arm")) {
 		if (asmbits == 64) {
 			hack = r_core_hack_arm64;
