@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2014 - pancake & dso */
+/* radare - LGPL - Copyright 2009-2019 - pancake & dso */
 
 #include "r_core.h"
 #include "r_util.h"
@@ -109,8 +109,10 @@ static int perform_mapped_file_yank(RCore *core, ut64 offset, ut64 len, const ch
 R_API int r_core_yank_set(RCore *core, ut64 addr, const ut8 *buf, ut32 len) {
 	// free (core->yank_buf);
 	if (buf && len) {
+		// FIXME: direct access to base should be avoided (use _sparse
+		// when you need buffer that starts at given addr)
 		r_buf_set_bytes (core->yank_buf, buf, len);
-		core->yank_buf->base = addr;
+		core->yank_addr = addr;
 		return true;
 	}
 	return false;
@@ -119,9 +121,10 @@ R_API int r_core_yank_set(RCore *core, ut64 addr, const ut8 *buf, ut32 len) {
 // Call set and then null terminate the bytes.
 R_API int r_core_yank_set_str(RCore *core, ut64 addr, const char *str, ut32 len) {
 	// free (core->yank_buf);
-	int res = r_core_yank_set (core, addr, (ut8 *) str, len);
+	int res = r_core_yank_set (core, addr, (ut8 *)str, len);
 	if (res == true) {
-		core->yank_buf->buf[len - 1] = 0;
+		ut8 zero = 0;
+		r_buf_write_at (core->yank_buf, len - 1, &zero, sizeof (zero));
 	}
 	return res;
 }
@@ -188,10 +191,17 @@ R_API int r_core_yank_paste(RCore *core, ut64 addr, int len) {
 	if (len < 0) {
 		return false;
 	}
-	if (len == 0 || len >= core->yank_buf->length) {
-		len = core->yank_buf->length;
+	if (len == 0 || len >= r_buf_size (core->yank_buf)) {
+		len = r_buf_size (core->yank_buf);
+	}
+	ut8 *buf = R_NEWS (ut8, len);
+	if (!buf) {
+		return false;
+	}
+	r_buf_read_at (core->yank_buf, 0, buf, len);
+	if (!r_core_write_at (core, addr, buf, len)) {
+		return false;
 	}
-	r_core_write_at (core, addr, core->yank_buf->buf, len);
 	return true;
 }
 
@@ -228,37 +238,72 @@ R_API int r_core_yank_to(RCore *core, const char *_arg) {
 	return res;
 }
 
-R_API int r_core_yank_dump(RCore *core, ut64 pos) {
-	int res = false, i = 0;
-	int ybl = core->yank_buf->length;
+R_API bool r_core_yank_dump(RCore *core, ut64 pos, int format) {
+	bool res = false;
+	int i = 0;
+	int ybl = r_buf_size (core->yank_buf);
 	if (ybl > 0) {
 		if (pos < ybl) {
-			r_cons_printf ("0x%08"PFMT64x " %d ",
-				core->yank_buf->base + pos,
-				core->yank_buf->length - pos);
-			for (i = pos; i < core->yank_buf->length; i++) {
-				r_cons_printf ("%02x",
-					core->yank_buf->buf[i]);
+			switch (format) {
+			case 'q':
+				for (i = pos; i < r_buf_size (core->yank_buf); i++) {
+					r_cons_printf ("%02x", r_buf_read8_at (core->yank_buf, i));
+				}
+				r_cons_newline ();
+				break;
+			case 'j':
+				{
+					r_cons_printf ("{\"addr\":%"PFMT64u",\"bytes\":\"", core->yank_addr);
+					for (i = pos; i < r_buf_size (core->yank_buf); i++) {
+						r_cons_printf ("%02x", r_buf_read8_at (core->yank_buf, i));
+					}
+					r_cons_printf ("\"}\n");
+				}
+				break;
+			case '*':
+				//r_cons_printf ("yfx ");
+				r_cons_printf ("wx ");
+				for (i = pos; i < r_buf_size (core->yank_buf); i++) {
+					r_cons_printf ("%02x", r_buf_read8_at (core->yank_buf, i));
+				}
+				//r_cons_printf (" @ 0x%08"PFMT64x, core->yank_addr);
+				r_cons_newline ();
+				break;
+			default:
+				r_cons_printf ("0x%08" PFMT64x " %d ",
+						core->yank_addr + pos,
+						r_buf_size (core->yank_buf) - pos);
+				for (i = pos; i < r_buf_size (core->yank_buf); i++) {
+					r_cons_printf ("%02x", r_buf_read8_at (core->yank_buf, i));
+				}
+				r_cons_newline ();
 			}
-			r_cons_newline ();
 			res = true;
 		} else {
 			eprintf ("Position exceeds buffer length.\n");
 		}
 	} else {
-		eprintf ("No buffer yanked already\n");
+		if (format == 'j') {
+			r_cons_printf ("{}\n");
+		} else {
+			eprintf ("No buffer yanked already\n");
+		}
 	}
 	return res;
 }
 
 R_API int r_core_yank_hexdump(RCore *core, ut64 pos) {
 	int res = false;
-	int ybl = core->yank_buf->length;
+	int ybl = r_buf_size (core->yank_buf);
 	if (ybl > 0) {
 		if (pos < ybl) {
+			ut8 *buf = R_NEWS (ut8, ybl - pos);
+			if (!buf) {
+				return false;
+			}
+			r_buf_read_at (core->yank_buf, pos, buf, ybl - pos);
 			r_print_hexdump (core->print, pos,
-				core->yank_buf->buf + pos,
-				ybl - pos, 16, 1, 1);
+				buf, ybl - pos, 16, 1, 1);
 			res = true;
 		} else {
 			eprintf ("Position exceeds buffer length.\n");
@@ -270,11 +315,16 @@ R_API int r_core_yank_hexdump(RCore *core, ut64 pos) {
 }
 
 R_API int r_core_yank_cat(RCore *core, ut64 pos) {
-	int ybl = core->yank_buf->length;
+	int ybl = r_buf_size (core->yank_buf);
 	if (ybl > 0) {
 		if (pos < ybl) {
-			r_cons_memcat ((const char *) core->yank_buf->buf + pos,
-				core->yank_buf->length - pos);
+			ut64 sz = ybl - pos;
+			char *buf = R_NEWS (char, sz);
+			if (!buf) {
+				return false;
+			}
+			r_buf_read_at (core->yank_buf, pos, (ut8 *)buf, sz);
+			r_cons_memcat (buf, sz);
 			r_cons_newline ();
 			return true;
 		}
@@ -286,11 +336,17 @@ R_API int r_core_yank_cat(RCore *core, ut64 pos) {
 }
 
 R_API int r_core_yank_cat_string(RCore *core, ut64 pos) {
-	int ybl = core->yank_buf->length;
+	int ybl = r_buf_size (core->yank_buf);
 	if (ybl > 0) {
 		if (pos < ybl) {
-			int len = r_str_nlen ((const char *) core->yank_buf->buf + pos, ybl - pos);
-			r_cons_memcat ((const char *) core->yank_buf->buf + pos, len);
+			ut64 sz = ybl - pos;
+			char *buf = R_NEWS (char, sz);
+			if (!buf) {
+				return false;
+			}
+			r_buf_read_at (core->yank_buf, pos, (ut8 *)buf, sz);
+			int len = r_str_nlen (buf, sz);
+			r_cons_memcat (buf, len);
 			r_cons_newline ();
 			return true;
 		}
@@ -332,6 +388,19 @@ R_API int r_core_yank_hud_path(RCore *core, const char *input, int dir) {
 	return res;
 }
 
+R_API bool r_core_yank_hexpair(RCore *core, const char *input) {
+	if (!input || !*input) {
+		return false;
+	}
+	char *out = strdup (input);
+	int len = r_hex_str2bin (input, (ut8 *)out);
+	if (len > 0) {
+		r_core_yank_set (core, core->offset, (ut8 *)out, len);
+	}
+	free (out);
+	return true;
+}
+
 R_API bool r_core_yank_file_ex(RCore *core, const char *input) {
 	ut64 len = 0, adv = 0, addr = 0;
 	bool res = false;
