@@ -73,7 +73,7 @@ RCryptoPlugin r_crypto_plugin_rol = {
 	.final = update,
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_CRYPTO,
 	.data = &r_crypto_plugin_rol,
