@@ -16,19 +16,18 @@ R_API ut64 r_debug_arg_get (RDebug *dbg, int cctype, int num) {
 	case R_ANAL_CC_TYPE_PASCAL:
 		sp = r_debug_reg_get (dbg, "SP");
 		if (dbg->bits == 64) {
-			sp += 8; // skip return address, asume we are inside the call
+			sp += 8; // skip return address, assume we are inside the call
 			sp += 8 * num;
 			dbg->iob.read_at (dbg->iob.io, sp, (ut8*)&n64, sizeof(ut64));
 			// TODO: honor endianness of platform
 			return (ut64)n64;
 		} else {
-			sp += 4; // skip return address, asume we are inside the call
+			sp += 4; // skip return address, assume we are inside the call
 			sp += 4 * num;
 			dbg->iob.read_at (dbg->iob.io, sp, (ut8*)&n32, sizeof(ut32));
 			// TODO: honor endianness of platform
 			return (ut64)n32;
 		}
-		break;
 	}
 	snprintf (reg, sizeof (reg)-1, "A%d", num);
 	return r_debug_reg_get (dbg, reg);
