@@ -1,13 +1,10 @@
 /* radare - LGPL - Copyright 2009-2018 - pancake, jduck, TheLemonMan, saucec0de */
 
 #include <r_debug.h>
+#include <r_drx.h>
 #include <r_core.h>
 #include <signal.h>
 
-#if __WINDOWS__
-void w32_break_process(void *);
-#endif
-
 R_LIB_VERSION(r_debug);
 
 // Size of the lookahead buffers used in r_debug functions
@@ -45,6 +42,13 @@ R_API void r_debug_bp_update(RDebug *dbg) {
 	}
 }
 
+static int r_debug_drx_at(RDebug *dbg, ut64 addr) {
+	if (dbg && dbg->h && dbg->h->drx) {
+		return dbg->h->drx (dbg, 0, addr, 0, 0, 0, DRX_API_GET_BP);
+	}
+	return -1;
+}
+
 /*
  * Recoiling after a breakpoint has two stages:
  * 1. remove the breakpoint and fix the program counter.
@@ -102,6 +106,12 @@ static int r_debug_bp_hit(RDebug *dbg, RRegItem *pc_ri, ut64 pc, RBreakpointItem
 			/* Some targets set pc to breakpoint */
 			b = r_bp_get_at (dbg->bp, pc);
 			if (!b) {
+				/* handle the case of hw breakpoints - notify the user */
+				int drx_reg_idx = r_debug_drx_at (dbg, pc);
+				if (drx_reg_idx != -1) {
+					eprintf ("hit hardware breakpoint %d at: %" PFMT64x "\n",
+						drx_reg_idx, pc);
+				}
 				/* Couldn't find the break point. Nothing more to do... */
 				return true;
 			}
@@ -380,6 +390,7 @@ R_API RDebug *r_debug_new(int hard) {
 		dbg->bp = r_bp_new ();
 		r_debug_plugin_init (dbg);
 		dbg->bp->iob.init = false;
+		dbg->bp->baddr = 0;
 	}
 	return dbg;
 }
@@ -561,13 +572,22 @@ R_API int r_debug_start(RDebug *dbg, const char *cmd) {
 }
 
 R_API int r_debug_detach(RDebug *dbg, int pid) {
+	int ret = 0;
 	if (dbg->h && dbg->h->detach) {
-		return dbg->h->detach (dbg, pid);
+		ret = dbg->h->detach (dbg, pid);
+		if (dbg->pid == pid) {
+			dbg->pid = -1;
+			dbg->tid = -1;
+		}
 	}
-	return false;
+	return ret;
 }
 
 R_API bool r_debug_select(RDebug *dbg, int pid, int tid) {
+	ut64 pc = 0;
+	int prev_pid = dbg->pid;
+	int prev_tid = dbg->tid;
+
 	if (pid < 0) {
 		return false;
 	}
@@ -587,16 +607,30 @@ R_API bool r_debug_select(RDebug *dbg, int pid, int tid) {
 		return false;
 	}
 
-	if (dbg->h && dbg->h->select && !dbg->h->select (pid, tid)) {
+	if (dbg->h && dbg->h->select && !dbg->h->select (dbg, pid, tid)) {
 		return false;
 	}
 
-	r_io_system (dbg->iob.io, sdb_fmt ("pid %d", pid));
+	// Don't change the pid/tid if the plugin already modified it due to internal constraints
+	if (dbg->pid == prev_pid) {
+		dbg->pid = pid;
+	}
+	if (dbg->tid == prev_tid) {
+		dbg->tid = tid;
+	}
 
-	dbg->pid = pid;
-	dbg->tid = tid;
+	r_io_system (dbg->iob.io, sdb_fmt ("pid %d", dbg->tid));
 
-	r_debug_reg_sync (dbg, R_REG_TYPE_GPR, false);
+	// Synchronize with the current thread's data
+	if (dbg->corebind.core) {
+		RCore *core = (RCore *)dbg->corebind.core;
+
+		r_reg_arena_swap (core->dbg->reg, true);
+		r_debug_reg_sync (dbg, R_REG_TYPE_ALL, false);
+
+		pc = r_debug_reg_get (dbg, "PC");
+		core->offset = pc;
+	}
 
 	return true;
 }
@@ -667,6 +701,10 @@ R_API RDebugReasonType r_debug_wait(RDebug *dbg, RBreakpointItem **bp) {
 		reason = dbg->h->wait (dbg, dbg->pid);
 		if (reason == R_DEBUG_REASON_DEAD) {
 			eprintf ("\n==> Process finished\n\n");
+			REventDebugProcessFinished event = {
+				.pid = dbg->pid
+			};
+			r_event_send (dbg->ev, R_EVENT_DEBUG_PROCESS_FINISHED, &event);
 			// XXX(jjd): TODO: handle fallback or something else
 			//r_debug_select (dbg, -1, -1);
 			return R_DEBUG_REASON_DEAD;
@@ -811,8 +849,7 @@ R_API int r_debug_step_soft(RDebug *dbg) {
 		} else {
 			r = 0;
 		}
-		if (!dbg->iob.read_at (dbg->iob.io,
-		      r*op.scale + op.disp, (ut8*)&memval, 8)) {
+		if (!dbg->iob.read_at (dbg->iob.io, r*op.scale + op.disp, (ut8*)&memval, 8)) {
 			next[0] = op.addr + op.size;
 		} else {
 			next[0] = (dbg->bits == R_SYS_BITS_32) ? memval.r32[0] : memval.r64;
@@ -868,11 +905,16 @@ R_API int r_debug_step_hard(RDebug *dbg) {
 		return false;
 	}
 	reason = r_debug_wait (dbg, NULL);
-	/* TODO: handle better */
-	if (reason == R_DEBUG_REASON_ERROR) {
+	if (reason == R_DEBUG_REASON_DEAD || r_debug_is_dead (dbg)) {
 		return false;
 	}
-	if (reason == R_DEBUG_REASON_DEAD || r_debug_is_dead (dbg)) {
+	// Unset breakpoints before leaving
+	if (reason != R_DEBUG_REASON_BREAKPOINT && reason != R_DEBUG_REASON_COND &&
+		reason != R_DEBUG_REASON_TRACEPOINT) {
+		r_bp_restore (dbg->bp, false);
+	}
+	/* TODO: handle better */
+	if (reason == R_DEBUG_REASON_ERROR) {
 		return false;
 	}
 	return true;
@@ -913,6 +955,17 @@ R_API int r_debug_step(RDebug *dbg, int steps) {
 	return steps_taken;
 }
 
+static bool isStepOverable(ut64 opType) {
+	switch (opType & R_ANAL_OP_TYPE_MASK) {
+	case R_ANAL_OP_TYPE_SWI:
+	case R_ANAL_OP_TYPE_CALL:
+	case R_ANAL_OP_TYPE_UCALL:
+	case R_ANAL_OP_TYPE_RCALL:
+		return true;
+	}
+	return false;
+}
+
 R_API int r_debug_step_over(RDebug *dbg, int steps) {
 	RAnalOp op;
 	ut64 buf_pc, pc, ins_size;
@@ -953,7 +1006,7 @@ R_API int r_debug_step_over(RDebug *dbg, int steps) {
 		}
 		// Analyze the opcode
 		if (!r_anal_op (dbg->anal, &op, pc, buf + (pc - buf_pc), sizeof (buf) - (pc - buf_pc), R_ANAL_OP_MASK_BASIC)) {
-			eprintf ("Decode error at %"PFMT64x"\n", pc);
+			eprintf ("debug-step-over: Decode error at %"PFMT64x"\n", pc);
 			return steps_taken;
 		}
 		if (op.fail == -1) {
@@ -963,8 +1016,7 @@ R_API int r_debug_step_over(RDebug *dbg, int steps) {
 			ins_size = op.fail;
 		}
 		// Skip over all the subroutine calls
-		if ((op.type & R_ANAL_OP_TYPE_MASK) == R_ANAL_OP_TYPE_CALL ||
-			(op.type & R_ANAL_OP_TYPE_MASK) == R_ANAL_OP_TYPE_UCALL) {
+		if (isStepOverable (op.type)) {
 			if (!r_debug_continue_until (dbg, ins_size)) {
 				eprintf ("Could not step over call @ 0x%"PFMT64x"\n", pc);
 				return steps_taken;
@@ -1085,26 +1137,21 @@ R_API bool r_debug_step_back(RDebug *dbg) {
 }
 
 R_API int r_debug_continue_kill(RDebug *dbg, int sig) {
-	RDebugReasonType reason, ret = false;
+	RDebugReasonType reason = R_DEBUG_REASON_NONE;
+	int ret = 0;
 	RBreakpointItem *bp = NULL;
 
 	if (!dbg) {
-		return false;
+		return 0;
 	}
-#if __WINDOWS__
-	r_cons_break_push (w32_break_process, dbg);
-#endif
 repeat:
 	if (r_debug_is_dead (dbg)) {
-		return false;
+		return 0;
 	}
 	if (dbg->h && dbg->h->cont) {
 		/* handle the stage-2 of breakpoints */
 		if (!r_debug_recoil (dbg, R_DBG_RECOIL_CONTINUE)) {
-#if __WINDOWS__
-			r_cons_break_pop ();
-#endif
-			return false;
+			return 0;
 		}
 		/* tell the inferior to go! */
 		ret = dbg->h->cont (dbg, dbg->pid, dbg->tid, sig);
@@ -1134,12 +1181,12 @@ repeat:
 #if DEBUGGER
 			/// if the plugin is not compiled link fails, so better do runtime linking
 			/// until this code gets fixed
-			static void (*linux_attach_new_process) (RDebug *dbg) = NULL;
+			static bool (*linux_attach_new_process) (RDebug *dbg, int pid) = NULL;
 			if (!linux_attach_new_process) {
 				linux_attach_new_process = r_lib_dl_sym (NULL, "linux_attach_new_process");
 			}
 			if (linux_attach_new_process) {
-				linux_attach_new_process (dbg);
+				linux_attach_new_process (dbg, dbg->forked_pid);
 			}
 #endif
 			goto repeat;
@@ -1156,14 +1203,14 @@ repeat:
 			goto repeat;
 		}
 #endif
-#if __WINDOWS__
 		if (reason != R_DEBUG_REASON_DEAD) {
-			// XXX(jjd): returning a thread id?!
 			ret = dbg->tid;
 		}
+#if __WINDOWS__
 		if (reason == R_DEBUG_REASON_NEW_LIB ||
 			reason == R_DEBUG_REASON_EXIT_LIB ||
 			reason == R_DEBUG_REASON_NEW_TID ||
+			reason == R_DEBUG_REASON_NONE ||
 			reason == R_DEBUG_REASON_EXIT_TID ) {
 			goto repeat;
 		}
@@ -1180,10 +1227,7 @@ repeat:
 		/* if continuing killed the inferior, we won't be able to get
 		 * the registers.. */
 		if (reason == R_DEBUG_REASON_DEAD || r_debug_is_dead (dbg)) {
-#if __WINDOWS__
-			r_cons_break_pop ();
-#endif
-			return false;
+			return 0;
 		}
 
 		/* if we hit a tracing breakpoint, we need to continue in
@@ -1195,7 +1239,9 @@ repeat:
 
 		/* choose the thread that was returned from the continue function */
 		// XXX(jjd): there must be a cleaner way to do this...
-		r_debug_select (dbg, dbg->pid, ret);
+		if (ret != dbg->tid) {
+			r_debug_select (dbg, dbg->pid, ret);
+		}
 		sig = 0; // clear continuation after signal if needed
 
 		/* handle general signals here based on the return from the wait
@@ -1228,16 +1274,24 @@ repeat:
 	}
 #if __WINDOWS__
 	r_cons_break_pop ();
+#elif __linux__
+	// Letting threads continue after the debugger breaks is currently problematic in linux
+	if (dbg->continue_all_threads) {
+		r_debug_stop (dbg);
+	}
 #endif
+	// Unset breakpoints before leaving
+	if (reason != R_DEBUG_REASON_BREAKPOINT) {
+		r_bp_restore (dbg->bp, false);
+	}
 	return ret;
-
 }
 
 R_API int r_debug_continue(RDebug *dbg) {
 	return r_debug_continue_kill (dbg, 0); //dbg->reason.signum);
 }
 
-#if __WINDOWS__ && !__CYGWIN__
+#if __WINDOWS__
 R_API int r_debug_continue_pass_exception(RDebug *dbg) {
 	return r_debug_continue_kill (dbg, DBG_EXCEPTION_NOT_HANDLED);
 }
@@ -1270,7 +1324,7 @@ R_API int r_debug_continue_until_optype(RDebug *dbg, int type, int over) {
 	buf_pc = r_debug_reg_get (dbg, dbg->reg->name[R_REG_NAME_PC]);
 	dbg->iob.read_at (dbg->iob.io, buf_pc, buf, sizeof (buf));
 
-	// step first, we dont want to check current optype
+	// step first, we don't want to check current optype
 	for (;;) {
 		if (!r_debug_reg_sync (dbg, R_REG_TYPE_GPR, false)) {
 			break;
@@ -1543,7 +1597,7 @@ R_API int r_debug_kill(RDebug *dbg, int pid, int tid, int sig) {
 		return false;
 	}
 	if (dbg->h && dbg->h->kill) {
-		if (pid > 0 && tid > 0) {
+		if (pid > 0) {
 			return dbg->h->kill (dbg, pid, tid, sig);
 		}
 		return -1;
@@ -1604,28 +1658,25 @@ R_API int r_debug_map_protect(RDebug *dbg, ut64 addr, int size, int perms) {
 
 R_API void r_debug_drx_list(RDebug *dbg) {
 	if (dbg && dbg->h && dbg->h->drx) {
-		dbg->h->drx (dbg, 0, 0, 0, 0, 0);
+		dbg->h->drx (dbg, 0, 0, 0, 0, 0, DRX_API_LIST);
 	}
 }
 
 R_API int r_debug_drx_set(RDebug *dbg, int idx, ut64 addr, int len, int rwx, int g) {
 	if (dbg && dbg->h && dbg->h->drx) {
-		return dbg->h->drx (dbg, idx, addr, len, rwx, g);
+		return dbg->h->drx (dbg, idx, addr, len, rwx, g, DRX_API_SET_BP);
 	}
 	return false;
 }
 
 R_API int r_debug_drx_unset(RDebug *dbg, int idx) {
 	if (dbg && dbg->h && dbg->h->drx) {
-		return dbg->h->drx (dbg, idx, 0, -1, 0, 0);
+		return dbg->h->drx (dbg, idx, 0, -1, 0, 0, DRX_API_REMOVE_BP);
 	}
 	return false;
 }
 
 R_API ut64 r_debug_get_baddr(RDebug *dbg, const char *file) {
-	char *abspath;
-	RListIter *iter;
-	RDebugMap *map;
 	if (!dbg || !dbg->iob.io || !dbg->iob.io->desc) {
 		return 0LL;
 	}
@@ -1643,14 +1694,21 @@ R_API ut64 r_debug_get_baddr(RDebug *dbg, const char *file) {
 	}
 #if __WINDOWS__
 	ut64 base;
-	return r_io_desc_get_base (dbg->iob.io->desc, &base), base;
-#else
+	bool ret = r_io_desc_get_base (dbg->iob.io->desc, &base);
+	if (ret) {
+		return base;
+	}
+#endif
+	RListIter *iter;
+	RDebugMap *map;
 	r_debug_select (dbg, pid, tid);
 	r_debug_map_sync (dbg);
-	abspath = r_sys_pid_to_path (pid);
+	char *abspath = r_sys_pid_to_path (pid);
+#if !__WINDOWS__
 	if (!abspath) {
 		abspath = r_file_abspath (file);
 	}
+#endif
 	if (!abspath) {
 		abspath = strdup (file);
 	}
@@ -1664,12 +1722,25 @@ R_API ut64 r_debug_get_baddr(RDebug *dbg, const char *file) {
 		free (abspath);
 	}
 	// fallback resolution (osx/w32?)
-	// we asume maps to be loaded in order, so lower addresses come first
+	// we assume maps to be loaded in order, so lower addresses come first
 	r_list_foreach (dbg->maps, iter, map) {
 		if (map->perm == 5) { // r-x
 			return map->addr;
 		}
 	}
 	return 0LL;
-#endif
+}
+
+R_API void r_debug_bp_rebase(RDebug *dbg, ut64 old_base, ut64 new_base) {
+	RBreakpointItem *bp;
+	RListIter *iter;
+	ut64 diff = new_base - old_base;
+	// update bp->baddr
+	dbg->bp->baddr = new_base;
+
+	// update bp's address
+	r_list_foreach (dbg->bp->bps, iter, bp) {
+		bp->addr += diff;
+		bp->delta = bp->addr - dbg->bp->baddr;
+	}
 }
