@@ -10,7 +10,7 @@ R_API int r_debug_reg_sync(RDebug *dbg, int type, int write) {
 	if (!dbg || !dbg->reg || !dbg->h) {
 		return false;
 	}
-	// Theres no point in syncing a dead target
+	// There's no point in syncing a dead target
 	if (r_debug_is_dead (dbg)) {
 		return false;
 	}
@@ -24,7 +24,7 @@ R_API int r_debug_reg_sync(RDebug *dbg, int type, int write) {
 	// Sync all the types sequentially if asked
 	i = (type == R_REG_TYPE_ALL)? R_REG_TYPE_GPR: type;
 	// Check to get the correct arena when using @ into reg profile (arena!=type)
-	// if request type is positive and the request regset dont have regs
+	// if request type is positive and the request regset don't have regs
 	if (i >= R_REG_TYPE_GPR && dbg->reg->regset[i].regs && !dbg->reg->regset[i].regs->length) {
 		// seek into the other arena for redirections.
 		for (n = R_REG_TYPE_GPR; n < R_REG_TYPE_LAST; n++) {
@@ -76,7 +76,7 @@ R_API int r_debug_reg_sync(RDebug *dbg, int type, int write) {
 		}
 		// DO NOT BREAK R_REG_TYPE_ALL PLEASE
 		//   break;
-		// Continue the syncronization or just stop if it was asked only for a single type of regs
+		// Continue the synchronization or just stop if it was asked only for a single type of regs
 		i++;
 	} while ((type == R_REG_TYPE_ALL) && (i < R_REG_TYPE_LAST));
 	return true;
@@ -196,6 +196,10 @@ R_API int r_debug_reg_list(RDebug *dbg, int type, int size, int rad, const char
 				case 128:
 					snprintf (strvalue, sizeof (strvalue), "0x%016"PFMT64x"%016"PFMT64x"", valueBig.v128.High, valueBig.v128.Low);
 					break;
+				case 256:
+					snprintf (strvalue, sizeof (strvalue), "0x%016"PFMT64x"%016"PFMT64x"%016"PFMT64x"%016"PFMT64x"",
+						valueBig.v256.High.High, valueBig.v256.High.Low, valueBig.v256.Low.High, valueBig.v256.Low.Low);
+					break;
 				default:
 					snprintf (strvalue, sizeof (strvalue), "ERROR");
 				}
@@ -217,7 +221,10 @@ R_API int r_debug_reg_list(RDebug *dbg, int type, int size, int rad, const char
 				break;
 			case 1:
 			case '*':
-				dbg->cb_printf ("f %s 1 %s\n", item->name, strvalue);
+				dbg->cb_printf ("f %s %d %s\n", item->name, item->size / 8, strvalue);
+				break;
+			case '.':
+				dbg->cb_printf ("dr %s=%s\n", item->name, strvalue);
 				break;
 			case 'd':
 			case 2:
