@@ -8,8 +8,9 @@
 static ut8 bfvm_op(BfvmCPU *c) {
 	// XXX: this is slow :(
 	ut8 buf[4] = {0};
-	if (c && c->iob.read_at && !c->iob.read_at (c->iob.io, c->eip, buf, 4))
+	if (c && c->iob.read_at && !c->iob.read_at (c->iob.io, c->eip, buf, 4)) {
 		return 0xff;
+	}
 	return buf[0];
 }
 
@@ -42,8 +43,9 @@ R_API int bfvm_init(BfvmCPU *c, ut32 size, int circular) {
 
 	/* data */
 	c->mem = (ut8 *)malloc (size);
-	if (!c->mem)
+	if (!c->mem) {
 		return 0;
+	}
 	memset (c->mem, '\0', size);
 
 	/* setup */
@@ -101,26 +103,30 @@ R_API ut8 bfvm_get(BfvmCPU *c) {
 
 R_API void bfvm_inc(BfvmCPU *c) {
 	ut8 *mem = bfvm_get_ptr (c);
-	if (mem != NULL)
+	if (mem != NULL) {
 		mem[0]++;
+	}
 }
 
 R_API void bfvm_dec(BfvmCPU *c) {
 	ut8 *mem = bfvm_get_ptr (c);
-	if (mem != NULL)
+	if (mem != NULL) {
 		mem[0]--;
+	}
 }
 
 R_API int bfvm_reg_set(BfvmCPU *c, const char *str) {
 	char *ptr = strchr (str, ' ');
-	if (!ptr)
+	if (!ptr) {
 		return 0;
-	if (strstr (str, "eip"))
+	}
+	if (strstr (str, "eip")) {
 		c->eip = r_num_math (NULL, ptr+1);
-	else if (strstr (str, "esp"))
+	} else if (strstr (str, "esp")) {
 		c->esp = r_num_math (NULL, ptr+1);
-	else if (strstr (str, "ptr"))
-		c->ptr = r_num_math (NULL, ptr+1);
+	} else if (strstr (str, "ptr")) {
+		c->ptr = r_num_math (NULL, ptr + 1);
+	}
 	return 1;
 }
 
@@ -129,8 +135,9 @@ R_API void bfvm_peek(BfvmCPU *c) {
 	int idx = c->input_idx;
 	ut8 *ptr = bfvm_get_ptr (c);
 
-	if (idx >= c->input_size)
+	if (idx >= c->input_size) {
 		idx = 0;
+	}
 
 	if (ptr) {
 		*ptr = c->input_buf[idx];
@@ -161,8 +168,9 @@ R_API int bfvm_trace_op(BfvmCPU *c, ut8 op) {
 	case ']':
 		g = bfvm_get (c);
 		eprintf ("%c  ; [ptr] = %d\n", op, g);
-		if (g!= 0)
+		if (g != 0) {
 			eprintf ("[");
+		}
 		break;
 	}
 	return 0;
