@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2011-2015 - pancake */
+/* radare - LGPL - Copyright 2011-2019 - pancake */
 
 #include <r_asm.h>
 #include <r_debug.h>
@@ -27,11 +27,14 @@ struct bfvm_regs {
 
 static struct bfvm_regs r;
 
-static int is_io_bf(RDebug *dbg) {
+static bool is_io_bf(RDebug *dbg) {
 	RIODesc *d = dbg->iob.io->desc;
-	if (d && d->plugin && d->plugin->name)
-		if (!strcmp ("bdescbg", d->plugin->name))
+	if (d && d->plugin && d->plugin->name) {
+		if (!strcmp ("bfdbg", d->plugin->name)) {
 			return true;
+		}
+	}
+	eprintf ("error: the iodesc data is not brainfuck friendly\n");
 	return false;
 }
 
@@ -40,10 +43,12 @@ static int r_debug_bf_step_over(RDebug *dbg) {
 	int op, oop = 0;
 	for (;;) {
 		op = bfvm_op (o->bfvm);
-		if (oop != 0 && op != oop)
+		if (oop != 0 && op != oop) {
 			break;
-		if (bfvm_in_trap (o->bfvm))
+		}
+		if (bfvm_in_trap (o->bfvm)) {
 			break;
+		}
 		bfvm_step (o->bfvm, 0);
 		oop = op;
 	}
@@ -57,14 +62,14 @@ static int r_debug_bf_step(RDebug *dbg) {
 }
 
 static int r_debug_bf_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
-	RIOBdescbg *o;
-	if (!dbg)
-		return false;
-	if (!is_io_bf (dbg))
+	r_return_val_if_fail (dbg && buf && size > 0, -1);
+	if (!is_io_bf (dbg)) {
 		return 0;
-	if (!(dbg->iob.io) || !(dbg->iob.io->desc) || !(dbg->iob.io->desc->data))
+	}
+	if (!(dbg->iob.io) || !(dbg->iob.io->desc) || !(dbg->iob.io->desc->data)) {
 		return 0;
-	o = dbg->iob.io->desc->data;
+	}
+	RIOBdescbg *o = dbg->iob.io->desc->data;
 	r.pc = o->bfvm->eip;
 	r.ptr = o->bfvm->ptr;
 	r.sp = o->bfvm->esp;
@@ -80,14 +85,16 @@ static int r_debug_bf_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 }
 
 static int r_debug_bf_reg_write(RDebug *dbg, int type, const ut8 *buf, int size) {
-	RIOBdescbg *o;
-	if (!dbg)
+	if (!dbg) {
 		return false;
-	if (!is_io_bf (dbg))
+	}
+	if (!is_io_bf (dbg)) {
 		return 0;
-	if (!(dbg->iob.io) || !(dbg->iob.io->desc) || !(dbg->iob.io->desc->data))
+	}
+	if (!(dbg->iob.io) || !(dbg->iob.io->desc) || !(dbg->iob.io->desc->data)) {
 		return 0;
-	o = dbg->iob.io->desc->data;
+	}
+	RIOBdescbg *o = dbg->iob.io->desc->data;
 	memcpy (&r, buf, sizeof (r));
 	o->bfvm->eip = r.pc;
 	o->bfvm->ptr = r.ptr; // dup
@@ -119,15 +126,9 @@ static int r_debug_bf_wait(RDebug *dbg, int pid) {
 }
 
 static int r_debug_bf_attach(RDebug *dbg, int pid) {
-	if (!is_io_bf (dbg))
+	if (!is_io_bf (dbg)) {
 		return false;
-#if 0
-	RIOBdescbg *o;
-	o = dbg->iob.io->desc->data;
-eprintf ("base = %llx\n", o->bfvm->base);
-eprintf ("screen = %llx\n", o->bfvm->screen);
-eprintf ("input = %llx\n", o->bfvm->input);
-#endif
+	}
 	return true;
 }
 
@@ -153,22 +154,32 @@ static char *r_debug_bf_reg_profile(RDebug *dbg) {
 	);
 }
 
-static int r_debug_bf_breakpoint (void *bp, RBreakpointItem *b, bool set) {
+static int r_debug_bf_breakpoint (struct r_bp_t *bp, RBreakpointItem *b, bool set) {
 	//r_io_system (dbg->iob.io, "db");
 	return false;
 }
 
 static bool r_debug_bf_kill(RDebug *dbg, int pid, int tid, int sig) {
+	if (!is_io_bf (dbg)) {
+		return false;
+	}
 	RIOBdescbg *o = dbg->iob.io->desc->data;
-	bfvm_reset (o->bfvm);
+	if (o) {
+		bfvm_reset (o->bfvm);
+	}
 	return true;
 }
 
 static RList *r_debug_native_map_get(RDebug *dbg) {
+	if (!is_io_bf (dbg)) {
+		return false;
+	}
 	RIOBdescbg *o = dbg->iob.io->desc->data;
 	BfvmCPU *c = o->bfvm;
 	RList *list = r_list_newf ((RListFree)r_debug_map_free);
-	if (!list) return NULL;
+	if (!list) {
+		return NULL;
+	}
 	r_list_append (list, r_debug_map_new (
 		"code", 0, 4096, 6, 0));
 	r_list_append (list, r_debug_map_new (
@@ -181,6 +192,9 @@ static RList *r_debug_native_map_get(RDebug *dbg) {
 }
 
 static int r_debug_bf_stop(RDebug *dbg) {
+	if (!is_io_bf (dbg)) {
+		return false;
+	}
 	RIOBdescbg *o = dbg->iob.io->desc->data;
 	BfvmCPU *c = o->bfvm;
 	c->breaked = true;
@@ -208,8 +222,8 @@ RDebugPlugin r_debug_plugin_bf = {
 	.map_get = r_debug_native_map_get,
 };
 
-#ifndef CORELIB
-RLibStruct radare_plugin = {
+#ifndef R2_PLUGIN_INCORE
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_DBG,
 	.data = &r_debug_plugin_bf,
 	.version = R2_VERSION
