@@ -1,5 +1,6 @@
 /* radare - LGPL - Copyright 2009-2018 - pancake, defragger */
 
+#include <r_core.h>
 #include <r_asm.h>
 #include <r_debug.h>
 #include <libgdbr.h>
@@ -13,6 +14,7 @@ typedef struct {
 #define UNSUPPORTED 0
 #define SUPPORTED 1
 
+static RIOGdb ** origriogdb = NULL;
 static libgdbr_t *desc = NULL;
 static ut8* reg_buf = NULL;
 static int buf_size = 0;
@@ -31,7 +33,7 @@ static int r_debug_gdb_step(RDebug *dbg) {
 	if (!desc) {
 		return R_DEBUG_REASON_UNKNOWN;
 	}
-	gdbr_step (desc, -1); // TODO handle thread specific step?
+	gdbr_step (desc, dbg->tid);
 	return true;
 }
 
@@ -43,6 +45,14 @@ static RList* r_debug_gdb_threads(RDebug *dbg, int pid) {
 	return list;
 }
 
+static RList* r_debug_gdb_pids(RDebug *dbg, int pid) {
+	RList *list;
+	if ((list = gdbr_pids_list (desc, pid))) {
+		list->free = (RListFree) &r_debug_pid_free;
+	}
+	return list;
+}
+
 static int r_debug_gdb_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 	int copy_size;
 	int buflen = 0;
@@ -289,7 +299,7 @@ static int r_debug_gdb_reg_write(RDebug *dbg, int type, const ut8 *buf, int size
 	free (r_reg_get_bytes (dbg->reg, type, &buflen));
 	// some implementations of the gdb protocol are acting weird.
 	// so winedbg is not able to write registers through the <G> packet
-	// and also it does not return the whole gdb register profile after\n"
+	// and also it does not return the whole gdb register profile after
 	// calling <g>
 	// so this workaround resizes the small register profile buffer
 	// to the whole set and fills the rest with 0
@@ -303,14 +313,15 @@ static int r_debug_gdb_reg_write(RDebug *dbg, int type, const ut8 *buf, int size
 	}
 
 	RRegItem* current = NULL;
+	// We default to little endian if there's no way to get the configuration,
+	// since this was the behaviour prior to the change.
+	RRegArena *arena = dbg->reg->regset[type].arena;
 	for (;;) {
 		current = r_reg_next_diff (dbg->reg, type, reg_buf, buflen, current, bits);
 		if (!current) {
 			break;
 		}
-		ut64 val = r_reg_get_value (dbg->reg, current);
-		int bytes = bits / 8;
-		gdbr_write_reg (desc, current->name, (char*)&val, bytes);
+		gdbr_write_reg (desc, current->name, (char*)arena->bytes + (current->offset / 8), current->size / 8);
 	}
 	return true;
 }
@@ -363,6 +374,7 @@ static int r_debug_gdb_attach(RDebug *dbg, int pid) {
 	if (d && d->plugin && d->plugin->name && d->data) {
 		if (!strcmp ("gdb", d->plugin->name)) {
 			RIOGdb *g = d->data;
+			origriogdb = (RIOGdb **)&d->data;	//TODO bit of a hack, please improve
 			support_sw_bp = UNKNOWN;
 			support_hw_bp = UNKNOWN;
 			int arch = r_sys_arch_id (dbg->arch);
@@ -416,17 +428,24 @@ static int r_debug_gdb_attach(RDebug *dbg, int pid) {
 				break;
 			}
 		} else {
-			eprintf ("ERROR: Underlaying IO descriptor is not a GDB one..\n");
+			eprintf ("ERROR: Underlying IO descriptor is not a GDB one..\n");
 		}
 	}
 	return true;
 }
 
 static int r_debug_gdb_detach(RDebug *dbg, int pid) {
+	int ret = 0;
+
 	if (pid <= 0 || !desc->stub_features.multiprocess) {
-		return gdbr_detach (desc);
+		ret = gdbr_detach (desc);
 	}
-	return gdbr_detach_pid (desc, pid);
+	ret = gdbr_detach_pid (desc, pid);
+
+	if (dbg->pid == pid) {
+		desc = NULL;
+	}
+	return ret;
 }
 
 static const char *r_debug_gdb_reg_profile(RDebug *dbg) {
@@ -480,17 +499,15 @@ static const char *r_debug_gdb_reg_profile(RDebug *dbg) {
 				"gpr	foseg	.32	164	0\n"
 				"gpr	fooff	.32	168	0\n"
 				"gpr	fop	.32	172	0\n"
-			/* Commented until the long registers will be implemented
-				"gpr	xmm0	.128	176	0\n"
-				"gpr	xmm1	.128	192	0\n"
-				"gpr	xmm2	.128	208	0\n"
-				"gpr	xmm3	.128	224	0\n"
-				"gpr	xmm4	.128	240	0\n"
-				"gpr	xmm5	.128	256	0\n"
-				"gpr	xmm6	.128	272	0\n"
-				"gpr	xmm7	.128	288	0\n"
+				"fpu	xmm0	.128	176	0\n"
+				"fpu	xmm1	.128	192	0\n"
+				"fpu	xmm2	.128	208	0\n"
+				"fpu	xmm3	.128	224	0\n"
+				"fpu	xmm4	.128	240	0\n"
+				"fpu	xmm5	.128	256	0\n"
+				"fpu	xmm6	.128	272	0\n"
+				"fpu	xmm7	.128	288	0\n"
 				"gpr	mxcsr	.32	304	0\n"
-			*/
 				);
 		} else if (dbg->anal->bits == 64) {
 			return strdup (
@@ -543,25 +560,23 @@ static const char *r_debug_gdb_reg_profile(RDebug *dbg) {
 				"gpr	foseg	.32	264	0\n"
 				"gpr	fooff	.32	268	0\n"
 				"gpr	fop	.32	272	0\n"
-			/* Commented until the long registers will be implemented
-				"gpr	xmm0	.128	276	0\n"
-				"gpr	xmm1	.128	292	0\n"
-				"gpr	xmm2	.128	308	0\n"
-				"gpr	xmm3	.128	324	0\n"
-				"gpr	xmm4	.128	340	0\n"
-				"gpr	xmm5	.128	356	0\n"
-				"gpr	xmm6	.128	372	0\n"
-				"gpr	xmm7	.128	388	0\n"
-				"gpr	xmm8	.128	404	0\n"
-				"gpr	xmm9	.128	420	0\n"
-				"gpr	xmm10	.128	436	0\n"
-				"gpr	xmm11	.128	452	0\n"
-				"gpr	xmm12	.128	468	0\n"
-				"gpr	xmm13	.128	484	0\n"
-				"gpr	xmm14	.128	500	0\n"
-				"gpr	xmm15	.128	516	0\n"
-				"gpr	mxcsr	.32	532	0\n"
-			*/
+				"fpu	xmm0	.128	276	0\n"
+				"fpu	xmm1	.128	292	0\n"
+				"fpu	xmm2	.128	308	0\n"
+				"fpu	xmm3	.128	324	0\n"
+				"fpu	xmm4	.128	340	0\n"
+				"fpu	xmm5	.128	356	0\n"
+				"fpu	xmm6	.128	372	0\n"
+				"fpu	xmm7	.128	388	0\n"
+				"fpu	xmm8	.128	404	0\n"
+				"fpu	xmm9	.128	420	0\n"
+				"fpu	xmm10	.128	436	0\n"
+				"fpu	xmm11	.128	452	0\n"
+				"fpu	xmm12	.128	468	0\n"
+				"fpu	xmm13	.128	484	0\n"
+				"fpu	xmm14	.128	500	0\n"
+				"fpu	xmm15	.128	516	0\n"
+				"fpu	mxcsr	.32	532	0\n"
 			);
 		} else {
 			return strdup (
@@ -1038,15 +1053,19 @@ static int r_debug_gdb_breakpoint (RBreakpoint *bp, RBreakpointItem *b, bool set
 static bool r_debug_gdb_kill(RDebug *dbg, int pid, int tid, int sig) {
 	// TODO kill based on pid and signal
 	if (sig != 0) {
-		return gdbr_kill (desc);
+		if (gdbr_kill (desc) < 0) {
+			return false;
+		}
 	}
 	return true;
 }
 
-static int r_debug_gdb_select(int pid, int tid) {
-	if (!desc) {
+static int r_debug_gdb_select(RDebug *dbg, int pid, int tid) {
+	if (!desc || !*origriogdb) {
+		desc = NULL;	//TODO hacky fix, please improve. I would suggest using a **desc instead of a *desc, so it is automatically updated
 		return false;
 	}
+
 	return gdbr_select (desc, pid, tid) >= 0;
 }
 
@@ -1098,13 +1117,14 @@ RDebugPlugin r_debug_plugin_gdb = {
 	.name = "gdb",
 	/* TODO: Add support for more architectures here */
 	.license = "LGPL3",
-	.arch = "x86,arm,sh,mips,avr,lm32,v850",
+	.arch = "x86,arm,sh,mips,avr,lm32,v850,ba2",
 	.bits = R_SYS_BITS_16 | R_SYS_BITS_32 | R_SYS_BITS_64,
 	.step = r_debug_gdb_step,
 	.cont = r_debug_gdb_continue,
 	.attach = &r_debug_gdb_attach,
 	.detach = &r_debug_gdb_detach,
 	.threads = &r_debug_gdb_threads,
+	.pids = &r_debug_gdb_pids,
 	.canstep = 1,
 	.wait = &r_debug_gdb_wait,
 	.map_get = r_debug_gdb_map_get,
@@ -1121,7 +1141,7 @@ RDebugPlugin r_debug_plugin_gdb = {
 	//.bp_read = &r_debug_gdb_bp_read,
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_DBG,
 	.data = &r_debug_plugin_gdb,
