@@ -160,7 +160,7 @@ RDebugPlugin r_debug_plugin_io = {
 #endif
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_DBG,
 	.data = &r_debug_plugin_io,
