@@ -37,7 +37,7 @@ static void pidlist_cb (void *ctx, pid_t pid, char *name) {
 	r_list_append (list, __r_debug_pid_new (name, pid, 's', 0));
 }
 
-static int r_debug_qnx_select (int pid, int tid) {
+static int r_debug_qnx_select (RDebug *dbg, int pid, int tid) {
 	return qnxr_select (desc, pid, tid);
 }
 
@@ -362,7 +362,7 @@ RDebugPlugin r_debug_plugin_qnx = {
 	.reg_profile = (void *)r_debug_qnx_reg_profile,
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_DBG,
 	.data = &r_debug_plugin_qnx,
