@@ -39,38 +39,44 @@ R_API int r_debug_pid_list(RDebug *dbg, int pid, char fmt) {
 		if (!list) {
 			return false;
 		}
-		if (fmt == 'j') {
-			dbg->cb_printf ("[");
-		}
+		PJ *j = pj_new ();
+		pj_a (j);
 		r_list_foreach (list, iter, p) {
 			switch (fmt) {
 			case 'j':
-				dbg->cb_printf ("{\"pid\":%d,"
-					"\"uid\":%d,"
-					"\"status\":\"%c\","
-					"\"path\":\"%s\"}%s",
-					p->pid, p->uid, p->status, p->path,
-					iter->n?",":"");
+				pj_o (j);
+				pj_kb (j, "current", dbg->pid == p->pid);
+				pj_ki (j, "ppid", p->ppid);
+				pj_ki (j, "pid", p->pid);
+				pj_ki (j, "uid", p->uid);
+				pj_ks (j, "status", &p->status);
+				pj_ks (j, "path", p->path);
+				pj_end (j);
 				break;
 			default:
-				dbg->cb_printf (" %c %d uid:%d %c %s\n",
+				dbg->cb_printf (" %c %d ppid:%d uid:%d %c %s\n",
 					dbg->pid == p->pid? '*': '-',
-					p->pid, p->uid, p->status, p->path);
+					p->pid, p->ppid, p->uid, p->status, p->path);
 				break;
 			}
 		}
+		pj_end (j);
 		if (fmt == 'j') {
-			dbg->cb_printf ("]\n");
+			dbg->cb_printf (pj_string (j));
 		}
+		pj_free (j);
 		r_list_free (list);
 	}
 	return false;
 }
 
-R_API int r_debug_thread_list(RDebug *dbg, int pid) {
+R_API int r_debug_thread_list(RDebug *dbg, int pid, char fmt) {
 	RList *list;
 	RListIter *iter;
 	RDebugPid *p;
+	RAnalFunction *fcn = NULL;
+	RDebugMap *map = NULL;
+	RStrBuf *path = NULL;
 	if (pid == -1) {
 		return false;
 	}
@@ -79,23 +85,45 @@ R_API int r_debug_thread_list(RDebug *dbg, int pid) {
 		if (!list) {
 			return false;
 		}
-		if (pid == -'j') {
-			dbg->cb_printf ("[");
-			r_list_foreach (list, iter, p) {
-				dbg->cb_printf ("{\"pid\":%d,"
-						"\"status\":\"%s\","
-						"\"path\":\"%s\"}%s",
-						p->pid, p->status, p->path,
-						iter->n?",":"");
+		PJ *j = pj_new ();
+		pj_a (j);
+		r_list_foreach (list, iter, p) {
+			path = r_strbuf_new ("");
+			if (p->pc != 0) {
+				map = r_debug_map_get (dbg, p->pc);
+				if (map && map->name && map->name[0]) {
+					r_strbuf_appendf (path, "%s ", map->name);
+				}
+
+				r_strbuf_appendf (path, "(0x%" PFMT64x ")", p->pc);
+
+				fcn = r_anal_get_fcn_in (dbg->anal, p->pc, 0);
+				if (fcn) {
+					r_strbuf_appendf (path, " in %s+0x%" PFMT64x, fcn->name, (p->pc - fcn->addr));
+				}
 			}
-			dbg->cb_printf ("]\n");
-		} else {
-			r_list_foreach (list, iter, p) {
+			switch (fmt) {
+			case 'j':
+				pj_o (j);
+				pj_kb (j, "current", dbg->tid == p->pid);
+				pj_ki (j, "pid", p->pid);
+				pj_ks (j, "status", &p->status);
+				pj_ks (j, "path", r_strbuf_get (path));
+				pj_end (j);
+				break;
+			default:
 				dbg->cb_printf (" %c %d %c %s\n",
-						dbg->tid == p->pid ? '*' : '-',
-						p->pid, p->status, p->path);
+					dbg->tid == p->pid? '*': '-',
+					p->pid, p->status, r_strbuf_get (path));
+				break;
 			}
+			r_strbuf_free (path);
 		}
+		pj_end (j);
+		if (fmt == 'j') {
+			dbg->cb_printf (pj_string (j));
+		}
+		pj_free (j);
 		r_list_free (list);
 	}
 	return false;
