@@ -13,6 +13,7 @@ extern "C" {
 R_LIB_VERSION_HEADER(r_search);
 
 enum {
+	R_SEARCH_ESIL,
 	R_SEARCH_KEYWORD,
 	R_SEARCH_REGEXP,
 	R_SEARCH_PATTERN,
@@ -61,7 +62,6 @@ typedef struct r_search_t {
 	ut64 nhits;
 	ut64 maxhits; // search.maxhits
 	RList *hits;
-	RMemoryPool *pool;
 	int distance;
 	int inverse;
 	bool overlap; // whether two matches can overlap
@@ -110,7 +110,7 @@ R_API int r_search_bmh(const RSearchKeyword *kw, const ut64 from, const ut8 *buf
 // TODO: is this an internal API?
 R_API int r_search_mybinparse_update(RSearch *s, ut64 from, const ut8 *buf, int len);
 R_API int r_search_aes_update(RSearch *s, ut64 from, const ut8 *buf, int len);
-R_API int r_search_rsa_update(RSearch *s, ut64 from, const ut8 *buf, int len);
+R_API int r_search_privkey_update(RSearch *s, ut64 from, const ut8 *buf, int len);
 R_API int r_search_magic_update(RSearch *_s, ut64 from, const ut8 *buf, int len);
 R_API int r_search_deltakey_update(RSearch *s, ut64 from, const ut8 *buf, int len);
 R_API int r_search_strings_update(RSearch *s, ut64 from, const ut8 *buf, int len);
