@@ -3,7 +3,8 @@
 
 // defines like IS_DIGIT, etc'
 #include "r_util/r_str_util.h"
-#include "r_userconf.h"
+#include <r_userconf.h>
+#include <stddef.h>
 
 // TODO: fix this to make it crosscompile-friendly: R_SYS_OSTYPE ?
 /* operating system */
@@ -12,6 +13,31 @@
 #undef __UNIX__
 #undef __WINDOWS__
 
+#define R_MODE_PRINT 0x000
+#define R_MODE_RADARE 0x001
+#define R_MODE_SET 0x002
+#define R_MODE_SIMPLE 0x004
+#define R_MODE_JSON 0x008
+#define R_MODE_ARRAY 0x010
+#define R_MODE_SIMPLEST 0x020
+#define R_MODE_CLASSDUMP 0x040
+#define R_MODE_EQUAL 0x080
+
+#define R_IN /* do not use, implicit */
+#define R_OUT /* parameter is written, not read */
+#define R_INOUT /* parameter is read and written */
+#define R_OWN /* pointer ownership is transferred */
+#define R_BORROW /* pointer ownership is not transferred, it must not be freed by the receiver */
+#define R_NONNULL /* pointer can not be null */
+#define R_NULLABLE /* pointer can be null */
+#define R_DEPRECATE /* should not be used in new code and should/will be removed in the future */
+#define R_IFNULL(x) /* default value for the pointer when null */
+#ifdef __GNUC__
+#define R_UNUSED __attribute__((__unused__))
+#else
+#define R_UNUSED /* unused */
+#endif
+
 #ifdef R_NEW
 #undef R_NEW
 #endif
@@ -28,6 +54,19 @@
 #undef R_NEWCOPY
 #endif
 
+// used in debug, io, bin, anal, ...
+#define R_PERM_R	4
+#define R_PERM_W	2
+#define R_PERM_X	1
+#define R_PERM_RW	(R_PERM_R|R_PERM_W)
+#define R_PERM_RX	(R_PERM_R|R_PERM_X)
+#define R_PERM_RWX	(R_PERM_R|R_PERM_W|R_PERM_X)
+#define R_PERM_WX	(R_PERM_W|R_PERM_X)
+#define R_PERM_SHAR	8
+#define R_PERM_PRIV	16
+#define R_PERM_ACCESS	32
+#define R_PERM_CREAT	64
+
 // HACK to fix capstone-android-mips build
 #undef mips
 #define mips mips
@@ -73,6 +112,10 @@
 #  define UNUSED_FUNCTION(x) UNUSED_ ## x
 #endif
 
+#ifdef __EMSCRIPTEN__
+# define __UNIX__ 1
+#endif
+
 #ifdef __HAIKU__
 # define __UNIX__ 1
 #endif
@@ -83,38 +126,39 @@
 #define __KFBSD__ 0
 #endif
 
-#if __MINGW32__ || __MINGW64__
-#undef MINGW32
-#define MINGW32 1
-#endif
-
 #ifdef _MSC_VER
+  #define restrict
   #define strcasecmp stricmp
   #define strncasecmp strnicmp
   #define __WINDOWS__ 1
+
+  #include <time.h>
+  static inline struct tm *gmtime_r(const time_t *t, struct tm *r) { return (gmtime_s(r, t))? NULL : r; }
 #endif
 
 #if defined(EMSCRIPTEN) || defined(__linux__) || defined(__APPLE__) || defined(__GNU__) || defined(__ANDROID__) || defined(__QNX__) || defined(__sun)
   #define __BSD__ 0
   #define __UNIX__ 1
 #endif
-#if __KFBSD__ || defined(__NetBSD__) || defined(__OpenBSD__)
+#if __KFBSD__ || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__DragonFly__)
   #define __BSD__ 1
   #define __UNIX__ 1
 #endif
-#if __WINDOWS__ || _WIN32 || MINGW32 && !(__MINGW64__ || __CYGWIN__)
+#if __WINDOWS__ || _WIN32
   #ifdef _MSC_VER
   /* Must be included before windows.h */
   #include <winsock2.h>
+  #ifndef WIN32_LEAN_AND_MEAN
   #define WIN32_LEAN_AND_MEAN
   #endif
+  #endif
   typedef int socklen_t;
   #undef USE_SOCKETS
   #define __WINDOWS__ 1
   #undef __UNIX__
   #undef __BSD__
 #endif
-#if __WINDOWS__ || _WIN32 || __CYGWIN__ || MINGW32
+#if __WINDOWS__ || _WIN32
   #define __addr_t_defined
   #include <windows.h>
 #endif
@@ -132,6 +176,7 @@
   #define FUNC_ATTR_ALLOC_ALIGN(x) __attribute__((alloc_align(x)))
   #define FUNC_ATTR_PURE __attribute__ ((pure))
   #define FUNC_ATTR_CONST __attribute__((const))
+  #define FUNC_ATTR_USED __attribute__((used))
   #define FUNC_ATTR_WARN_UNUSED_RESULT __attribute__((warn_unused_result))
   #define FUNC_ATTR_ALWAYS_INLINE __attribute__((always_inline))
 
@@ -149,11 +194,11 @@
   #define FUNC_ATTR_ALLOC_ALIGN(x)
   #define FUNC_ATTR_PURE
   #define FUNC_ATTR_CONST
+  #define FUNC_ATTR_USED
   #define FUNC_ATTR_WARN_UNUSED_RESULT
   #define FUNC_ATTR_ALWAYS_INLINE
 #endif
 
-#include <r_userconf.h>
 #include <r_types_base.h>
 
 #undef _FILE_OFFSET_BITS
@@ -179,35 +224,27 @@ extern "C" {
 #if __WINDOWS__
 #define FS "\\"
 #define R_SYS_DIR "\\"
+#define R_SYS_ENVSEP ";"
 #define R_SYS_HOME "USERPROFILE"
-#define R2_HOMEDIR ".config\\radare2"
+#define R_SYS_TMP "TEMP"
 #else
 #define FS "/"
 #define R_SYS_DIR "/"
+#define R_SYS_ENVSEP ":"
 #define R_SYS_HOME "HOME"
-#define R2_HOMEDIR ".config/radare2"
+#define R_SYS_TMP "TMPDIR"
 #endif
 
+#define R_JOIN_2_PATHS(p1, p2) p1 R_SYS_DIR p2
+#define R_JOIN_3_PATHS(p1, p2, p3) p1 R_SYS_DIR p2 R_SYS_DIR p3
+#define R_JOIN_4_PATHS(p1, p2, p3, p4) p1 R_SYS_DIR p2 R_SYS_DIR p3 R_SYS_DIR p4
+#define R_JOIN_5_PATHS(p1, p2, p3, p4, p5) p1 R_SYS_DIR p2 R_SYS_DIR p3 R_SYS_DIR p4 R_SYS_DIR p5
+
 #ifndef __packed
 #define __packed __attribute__((__packed__))
 #endif
 
-#ifndef UNUSED
-#ifdef __GNUC__
-#define UNUSED __attribute__((__unused__))
-#else
-#define UNUSED
-#endif
-#endif
-
-typedef void (*PrintfCallback)(const char *str, ...);
-
-// TODO NOT USED. DEPREACATE
-#if R_RTDEBUG
-#define IFRTDBG if (getenv ("LIBR_DEBUG"))
-#else
-#define IFRTDBG if (0)
-#endif
+typedef int (*PrintfCallback)(const char *str, ...);
 
 /* compile-time introspection helpers */
 #define CTO(y,z) ((size_t) &((y*)0)->z)
@@ -219,6 +256,8 @@ typedef void (*PrintfCallback)(const char *str, ...);
 #undef R_IPI
 #endif
 
+#define R_IPI
+
 #ifdef R_HEAP
 #undef R_HEAP
 #endif
@@ -244,12 +283,12 @@ typedef void (*PrintfCallback)(const char *str, ...);
 #define R_LIB_VERSION_HEADER(x) \
 R_API const char *x##_version(void)
 #define R_LIB_VERSION(x) \
-R_API const char *x##_version () { return "" R2_GITTAP; }
+R_API const char *x##_version() { return "" R2_GITTAP; }
 
 #define BITS2BYTES(x) (((x)/8)+(((x)%8)?1:0))
 #define ZERO_FILL(x) memset (&x, 0, sizeof (x))
 #define R_NEWS0(x,y) (x*)calloc(y,sizeof(x))
-#define R_NEWS(x,y) (x*)malloc(sizeof(x)*y)
+#define R_NEWS(x,y) (x*)malloc(sizeof(x)*(y))
 #define R_NEW0(x) (x*)calloc(1,sizeof(x))
 #define R_NEW(x) (x*)malloc(sizeof(x))
 #define R_NEWCOPY(x,y) (x*)r_new_copy(sizeof(x), y)
@@ -264,7 +303,7 @@ static inline void *r_new_copy(int size, void *data) {
 // TODO: Make R_NEW_COPY be 1 arg, not two
 #define R_NEW_COPY(x,y) x=(void*)malloc(sizeof(y));memcpy(x,y,sizeof(y))
 #define R_MEM_ALIGN(x) ((void *)(size_t)(((ut64)(size_t)x) & 0xfffffffffffff000LL))
-#define R_ARRAY_SIZE(x) (sizeof (x) / sizeof (x[0]))
+#define R_ARRAY_SIZE(x) (sizeof (x) / sizeof ((x)[0]))
 #define R_PTR_MOVE(d,s) d=s;s=NULL;
 
 #define R_PTR_ALIGN(v,t) \
@@ -314,7 +353,6 @@ static inline void *r_new_copy(int size, void *data) {
 
 #ifndef HAVE_EPRINTF
 #define eprintf(...) fprintf(stderr,__VA_ARGS__)
-#define eprint(x) fprintf(stderr,"%s\n",x)
 #define HAVE_EPRINTF 1
 #endif
 
@@ -322,8 +360,15 @@ static inline void *r_new_copy(int size, void *data) {
 #define typeof(arg) __typeof__(arg)
 #endif
 
-#undef r_offsetof
+#if 1
+#define r_offsetof(type, member) offsetof(type, member)
+#else
+#if __SDB_WINDOWS__
+#define r_offsetof(type, member) ((unsigned long) (ut64)&((type*)0)->member)
+#else
 #define r_offsetof(type, member) ((unsigned long) &((type*)0)->member)
+#endif
+#endif
 
 #define R_BETWEEN(x,y,z) (((y)>=(x)) && ((y)<=(z)))
 #define R_ROUND(x,y) ((x)%(y))?(x)+((y)-((x)%(y))):(x)
@@ -339,7 +384,7 @@ static inline void *r_new_copy(int size, void *data) {
 #define R_ABS(x) (((x)<0)?-(x):(x))
 #define R_BTW(x,y,z) (((x)>=(y))&&((y)<=(z)))?y:x
 
-#define R_FREE(x) { free(x); x = NULL; }
+#define R_FREE(x) { free((void *)x); x = NULL; }
 
 #if __WINDOWS__
 #define HAVE_REGEXP 0
@@ -347,11 +392,15 @@ static inline void *r_new_copy(int size, void *data) {
 #define HAVE_REGEXP 1
 #endif
 
-#if (__WINDOWS__ || MINGW32) && !__CYGWIN__
+#if __WINDOWS__
 #define PFMT64x "I64x"
 #define PFMT64d "I64d"
 #define PFMT64u "I64u"
 #define PFMT64o "I64o"
+#define PFMTSZx "Ix"
+#define PFMTSZd "Id"
+#define PFMTSZu "Iu"
+#define PFMTSZo "Io"
 #define LDBLFMT "f"
 #define HHXFMT  "x"
 #else
@@ -359,6 +408,10 @@ static inline void *r_new_copy(int size, void *data) {
 #define PFMT64d "lld"
 #define PFMT64u "llu"
 #define PFMT64o "llo"
+#define PFMTSZx "zx"
+#define PFMTSZd "zd"
+#define PFMTSZu "zu"
+#define PFMTSZo "zo"
 #define LDBLFMT "Lf"
 #define HHXFMT  "hhx"
 #endif
@@ -395,6 +448,10 @@ static inline void *r_new_copy(int size, void *data) {
 #define R_SYS_ARCH "x86"
 #define R_SYS_BITS R_SYS_BITS_32
 #define R_SYS_ENDIAN 0
+#elif __EMSCRIPTEN__
+#define R_SYS_ARCH "wasm"
+#define R_SYS_BITS (R_SYS_BITS_32 | R_SYS_BITS_64)
+#define R_SYS_ENDIAN 0
 #elif __x86_64__
 #define R_SYS_ARCH "x86"
 #define R_SYS_BITS (R_SYS_BITS_32 | R_SYS_BITS_64)
@@ -435,6 +492,14 @@ static inline void *r_new_copy(int size, void *data) {
 /* we should default to wasm when ready */
 #define R_SYS_ARCH "x86"
 #define R_SYS_BITS R_SYS_BITS_32
+#elif __riscv__ || __riscv
+# define R_SYS_ARCH "riscv"
+# define R_SYS_ENDIAN 0
+# if __riscv_xlen == 32
+#  define R_SYS_BITS R_SYS_BITS_32
+# else
+#  define R_SYS_BITS (R_SYS_BITS_32 | R_SYS_BITS_64)
+# endif
 #else
 #ifdef _MSC_VER
 #ifdef _WIN64
@@ -496,6 +561,12 @@ enum {
 	R_SYS_ARCH_LM32 =  0x80000000LL, // 1<<31
 };
 
+#if HAVE_CLOCK_NANOSLEEP && CLOCK_MONOTONIC && (__linux__ || (__FreeBSD__ && __FreeBSD_version >= 1101000) || (__NetBSD__ && __NetBSD_Version__ >= 700000000))
+#define HAS_CLOCK_NANOSLEEP 1
+#else
+#define HAS_CLOCK_NANOSLEEP 0
+#endif
+
 /* os */
 #if defined (__QNX__)
 #define R_SYS_OS "qnx"
@@ -505,7 +576,7 @@ enum {
 #define R_SYS_OS "darwin"
 #elif defined (__linux__)
 #define R_SYS_OS "linux"
-#elif defined (__WINDOWS__) || defined (__CYGWIN__) || defined (MINGW32)
+#elif defined (__WINDOWS__)
 #define R_SYS_OS "windows"
 #elif defined (__NetBSD__ )
 #define R_SYS_OS "netbsd"
@@ -517,83 +588,66 @@ enum {
 #define R_SYS_OS "unknown"
 #endif
 
-#if __GNUC__
-#  define r_sys_trap() __builtin_trap()
-#else
-#  if __i386__ || __x86_64__
-#    define r_sys_trap() __asm__ __volatile__ ("int3")
-#  elif __arm__
-#    define r_sys_trap() __asm__ __volatile__ ("bkpt")
-#  elif __arm64__
-#    define r_sys_trap() __asm__ __volatile__ ("brk #1")
-#  else
-#    define r_sys_trap() __asm__ __volatile__ (".word 0");
-#  endif
-#endif
-
 #ifdef __cplusplus
 }
 #endif
 
-static inline void r_run_call1 (void *fcn, void *arg1) {
+static inline void r_run_call1(void *fcn, void *arg1) {
 	((void (*)(void *))(fcn))(arg1);
 }
 
-static inline void r_run_call2 (void *fcn, void *arg1, void *arg2) {
+static inline void r_run_call2(void *fcn, void *arg1, void *arg2) {
 	((void (*)(void *, void *))(fcn))(arg1, arg2);
 }
 
-static inline void r_run_call3 (void *fcn, void *arg1, void *arg2, void *arg3) {
+static inline void r_run_call3(void *fcn, void *arg1, void *arg2, void *arg3) {
 	((void (*)(void *, void *, void *))(fcn))(arg1, arg2, arg3);
 }
 
-static inline void r_run_call4 (void *fcn, void *arg1, void *arg2, void *arg3, void *arg4) {
+static inline void r_run_call4(void *fcn, void *arg1, void *arg2, void *arg3, void *arg4) {
 	((void (*)(void *, void *, void *, void *))(fcn))(arg1, arg2, arg3, arg4);
 }
 
-static inline void r_run_call5 (void *fcn, void *arg1, void *arg2, void *arg3, void *arg4, void *arg5) {
+static inline void r_run_call5(void *fcn, void *arg1, void *arg2, void *arg3, void *arg4, void *arg5) {
 	((void (*)(void *, void *, void *, void *, void *))(fcn))(arg1, arg2, arg3, arg4, arg5);
 }
 
-static inline void r_run_call6 (void *fcn, void *arg1, void *arg2, void *arg3, void *arg4, void *arg5,
+static inline void r_run_call6(void *fcn, void *arg1, void *arg2, void *arg3, void *arg4, void *arg5,
 	void *arg6) {
 	((void (*)(void *, void *, void *, void *, void *, void *))(fcn))
 		(arg1, arg2, arg3, arg4, arg5, arg6);
 }
 
-static inline void r_run_call7 (void *fcn, void *arg1, void *arg2, void *arg3, void *arg4, void *arg5,
+static inline void r_run_call7(void *fcn, void *arg1, void *arg2, void *arg3, void *arg4, void *arg5,
 	void *arg6, void *arg7) {
 	((void (*)(void *, void *, void *, void *, void *, void *, void *))(fcn))
 		(arg1, arg2, arg3, arg4, arg5, arg6, arg7);
 }
 
-static inline void r_run_call8 (void *fcn, void *arg1, void *arg2, void *arg3, void *arg4, void *arg5,
+static inline void r_run_call8(void *fcn, void *arg1, void *arg2, void *arg3, void *arg4, void *arg5,
 	void *arg6, void *arg7, void *arg8) {
 	((void (*)(void *, void *, void *, void *, void *, void *, void *, void *))(fcn))
 		(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8);
 }
 
-static inline void r_run_call9 (void *fcn, void *arg1, void *arg2, void *arg3, void *arg4, void *arg5,
+static inline void r_run_call9(void *fcn, void *arg1, void *arg2, void *arg3, void *arg4, void *arg5,
 	void *arg6, void *arg7, void *arg8, void *arg9) {
 	((void (*)(void *, void *, void *, void *, void *, void *, void *, void *, void *))(fcn))
 		(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9);
 }
 
-static inline void r_run_call10 (void *fcn, void *arg1, void *arg2, void *arg3, void *arg4, void *arg5,
+static inline void r_run_call10(void *fcn, void *arg1, void *arg2, void *arg3, void *arg4, void *arg5,
 	void *arg6, void *arg7, void *arg8, void *arg9, void *arg10) {
 	((void (*)(void *, void *, void *, void *, void *, void *, void *, void *, void *, void *))(fcn))
 		(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10);
 }
 
-#endif // R2_TYPES_H
-
-// Usage: R_DEFINE_OBJECT(r_asm);
-#if 0
-#define R_DEFINE_OBJECT(type) \
- R_API struct type##_t* type##_new() { \
-    return type##_init(R_NEW(struct type##_t)); \
- } \
- R_API struct type##_t* type##_free(struct type##_t *foo) { \
-    return (type##_deinit(foo), free(foo), NULL); \
- }
+#ifndef container_of
+# ifdef _MSC_VER
+#  define container_of(ptr, type, member) ((type *)((char *)(ptr) - offsetof(type, member)))
+# else
+#  define container_of(ptr, type, member) ((type *)((char *)(__typeof__(((type *)0)->member) *){ptr} - offsetof(type, member)))
+# endif
 #endif
+
+#endif // R2_TYPES_H
