@@ -1,7 +1,9 @@
 #ifndef R_PJ_H
 #define R_PJ_H 1
-
 #define R_PRINT_JSON_DEPTH_LIMIT 128
+
+#include <r_util/r_strbuf.h>
+
 typedef struct pj_t {
 	RStrBuf *sb;
 	bool is_first;
@@ -11,7 +13,7 @@ typedef struct pj_t {
 } PJ;
 
 /* lifecycle */
-R_API PJ *pj_new();
+R_API PJ *pj_new(void);
 R_API void pj_free(PJ *j);
 R_API char *pj_drain(PJ *j);
 R_API const char *pj_string(PJ *pj);
@@ -26,16 +28,22 @@ R_API PJ *pj_o(PJ *j);
 R_API PJ *pj_a(PJ *j);
 /* keys, values */
 R_API PJ *pj_k(PJ *j, const char *k);
+R_API PJ *pj_knull(PJ *j, const char *k);
 R_API PJ *pj_kn(PJ *j, const char *k, ut64 n);
+R_API PJ *pj_kN(PJ *j, const char *k, st64 n);
 R_API PJ *pj_ks(PJ *j, const char *k, const char *v);
 R_API PJ *pj_ki(PJ *j, const char *k, int d);
 R_API PJ *pj_kd(PJ *j, const char *k, double d);
+R_API PJ *pj_kf(PJ *j, const char *k, float d);
 R_API PJ *pj_kb(PJ *j, const char *k, bool v);
+R_API PJ *pj_null(PJ *j);
 R_API PJ *pj_b(PJ *j, bool v);
 R_API PJ *pj_s(PJ *j, const char *k);
 R_API PJ *pj_n(PJ *j, ut64 n);
+R_API PJ *pj_N(PJ *j, st64 n);
 R_API PJ *pj_d(PJ *j, double d);
+R_API PJ *pj_f(PJ *j, float d);
 R_API PJ *pj_i(PJ *j, int d);
-
+R_API PJ *pj_j(PJ *j, const char *k);
 #endif
 
