@@ -72,7 +72,7 @@ typedef struct r_pkcs7_container_t {
 R_API RCMS *r_pkcs7_parse_cms(const ut8 *buffer, ut32 length);
 R_API void r_pkcs7_free_cms(RCMS* container);
 R_API char *r_pkcs7_cms_to_string(RCMS* container);
-R_API RJSVar *r_pkcs7_cms_json(RCMS* container);
+R_API PJ *r_pkcs7_cms_json(RCMS* container);
 
 #ifdef __cplusplus
 }
