@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2008-2018 - condret, pancake, alvaro_fe */
+/* radare2 - LGPL - Copyright 2008-2019 - condret, pancake, alvaro_fe */
 
 #include <r_io.h>
 #include <sdb.h>
@@ -59,6 +59,9 @@ static st64 on_map_skyline(RIO *io, ut64 vaddr, ut8 *buf, int len, int match_flg
 		}
 		// Now left endpoint <= addr < right endpoint
 		ut64 len1 = R_MIN (vaddr + len - addr, r_itv_end (part->itv) - addr);
+		if (len1 < 1) {
+			break;
+		}
 		// The map satisfies the permission requirement or p_cache is enabled
 		if (((part->map->perm & match_flg) == match_flg || io->p_cache)) {
 			st64 result = op (io, part->map->fd, part->map->delta + addr - part->map->itv.addr,
@@ -106,55 +109,33 @@ R_API RIO* r_io_init(RIO* io) {
 	r_pvector_init (&io->map_skyline, free);
 	r_pvector_init (&io->map_skyline_shadow, free);
 	r_io_map_init (io);
-	r_io_section_init (io);
 	r_io_cache_init (io);
 	r_io_plugin_init (io);
 	r_io_undo_init (io);
 	return io;
 }
 
-R_API RBuffer *r_io_read_buf(RIO *io, ut64 addr, int len) {
-	RBuffer *b = R_NEW0 (RBuffer);
-	if (!b) {
-		return NULL;
-	}
-	b->buf = malloc (len);
-	if (!b->buf) {
-		free (b);
-		return NULL;
-	}
-	len = r_io_read_at (io, addr, b->buf, len);
-	b->length = (len < 0)? 0: len;
-	return b;
-}
-
-R_API int r_io_write_buf(RIO *io, struct r_buf_t *b) {
-	return r_io_write_at (io, b->base, b->buf, b->length);
-}
-
 R_API void r_io_free(RIO *io) {
-	if (!io) {
-		return;
+	if (io) {
+		r_io_fini (io);
+		r_cache_free (io->buffer);
+		free (io);
 	}
-	r_io_fini (io);
-	r_cache_free (io->buffer);
-	free (io);
 }
 
 R_API RIODesc *r_io_open_buffer(RIO *io, RBuffer *b, int perm, int mode) {
-	const int bufSize = r_buf_size (b);
+	ut64 bufSize = r_buf_size (b);
 	char *uri = r_str_newf ("malloc://%d", bufSize);
 	RIODesc *desc = r_io_open_nomap (io, uri, perm, mode);
 	if (desc) {
-		r_io_desc_write (desc, r_buf_get_at (b, 0, NULL), bufSize);
+		const ut8 *tmp = r_buf_data (b, &bufSize);
+		r_io_desc_write (desc, tmp, bufSize);
 	}
 	return desc;
 }
 
 R_API RIODesc *r_io_open_nomap(RIO *io, const char *uri, int perm, int mode) {
-	if (!io || !uri) {
-		return NULL;
-	}
+	r_return_val_if_fail (io && uri, NULL);
 	RIODesc *desc = r_io_desc_open (io, uri, perm, mode);
 	if ((io->autofd || !io->desc) && desc) {
 		io->desc = desc;
@@ -167,23 +148,21 @@ R_API RIODesc *r_io_open_nomap(RIO *io, const char *uri, int perm, int mode) {
 R_API RIODesc* r_io_open(RIO* io, const char* uri, int perm, int mode) {
 	r_return_val_if_fail (io && io->maps, NULL);
 	RIODesc* desc = r_io_open_nomap (io, uri, perm, mode);
-	if (!desc) {
-		return NULL;
+	if (desc) {
+		r_io_map_new (io, desc->fd, desc->perm, 0LL, 0LL, r_io_desc_size (desc));
 	}
-	r_io_map_new (io, desc->fd, desc->perm, 0LL, 0LL, r_io_desc_size (desc));
 	return desc;
 }
 
 /* opens a file and maps it to an offset specified by the "at"-parameter */
 R_API RIODesc* r_io_open_at(RIO* io, const char* uri, int perm, int mode, ut64 at) {
-	r_return_val_if_fail (io && io->maps, NULL);
-	RIODesc* desc;
-	ut64 size;
-	desc = r_io_open_nomap (io, uri, perm, mode);
+	r_return_val_if_fail (io && io->maps && uri, NULL);
+
+	RIODesc* desc = r_io_open_nomap (io, uri, perm, mode);
 	if (!desc) {
 		return NULL;
 	}
-	size = r_io_desc_size (desc);
+	ut64 size = r_io_desc_size (desc);
 	// second map
 	if (size && ((UT64_MAX - size + 1) < at)) {
 		// split map into 2 maps if only 1 big map results into interger overflow
@@ -235,14 +214,15 @@ R_API bool r_io_reopen(RIO* io, int fd, int perm, int mode) {
 	}
 	//does this really work, or do we have to handler debuggers ugly
 	uri = old->referer? old->referer: old->uri;
-#if __WINDOWS__ //TODO: workaround, see https://github.com/radare/radare2/issues/8840
-	if (!r_io_desc_close (old)) {
+#if __WINDOWS__ //TODO: workaround, see https://github.com/radareorg/radare2/issues/8840
+	if (old->plugin->close && old->plugin->close (old)) {
 		return false; // TODO: this is an unrecoverable scenario
 	}
 	if (!(new = r_io_open_nomap (io, uri, perm, mode))) {
 		return false;
 	}
 	r_io_desc_exchange (io, old->fd, new->fd);
+	r_io_desc_del (io, old->fd);
 	return true;
 #else
 	if (!(new = r_io_open_nomap (io, uri, perm, mode))) {
@@ -259,18 +239,16 @@ R_API int r_io_close_all(RIO* io) { // what about undo?
 	}
 	r_io_desc_fini (io);
 	r_io_map_fini (io);
-	r_io_section_fini (io);
 	ls_free (io->plugins);
 	r_io_desc_init (io);
 	r_io_map_init (io);
-	r_io_section_init (io);
 	r_io_cache_fini (io);
 	r_io_plugin_init (io);
 	return true;
 }
 
 R_API int r_io_pread_at(RIO* io, ut64 paddr, ut8* buf, int len) {
-	r_return_val_if_fail (io && buf && len > 0, -1);
+	r_return_val_if_fail (io && buf && len >= 0, -1);
 	if (io->ff) {
 		memset (buf, io->Oxff, len);
 	}
@@ -301,7 +279,10 @@ static bool r_io_vwrite_at(RIO* io, ut64 vaddr, const ut8* buf, int len) {
 // For physical mode, the interface is broken because the actual read bytes are
 // not available. This requires fixes in all call sites.
 R_API bool r_io_read_at(RIO *io, ut64 addr, ut8 *buf, int len) {
-	r_return_val_if_fail (io && buf && len > 0, false);
+	r_return_val_if_fail (io && buf && len >= 0, false);
+	if (len == 0) {
+		return false;
+	}
 	bool ret = (io->va)
 		? r_io_vread_at_mapped (io, addr, buf, len)
 		: r_io_pread_at (io, addr, buf, len) > 0;
@@ -337,7 +318,10 @@ R_API bool r_io_read_at_mapped(RIO *io, ut64 addr, ut8 *buf, int len) {
 // Returns -1 on error.
 R_API int r_io_nread_at(RIO *io, ut64 addr, ut8 *buf, int len) {
 	int ret;
-	r_return_val_if_fail (io && buf && len > 0, -1);
+	r_return_val_if_fail (io && buf && len >= 0, -1);
+	if (len == 0) {
+		return 0;
+	}
 	if (io->va) {
 		if (io->ff) {
 			memset (buf, io->Oxff, len);
@@ -496,6 +480,23 @@ R_API bool r_io_set_write_mask(RIO* io, const ut8* mask, int len) {
 	return true;
 }
 
+R_API ut64 r_io_p2v(RIO *io, ut64 pa) {
+	RIOMap *map = r_io_map_get_paddr (io, pa);
+	if (map) {
+		return pa - map->delta + map->itv.addr;
+	}
+	return UT64_MAX;
+}
+
+R_API ut64 r_io_v2p(RIO *io, ut64 va) {
+	RIOMap *map = r_io_map_get (io, va);
+	if (map) {
+		st64 delta = va - map->itv.addr;
+		return map->itv.addr + map->delta + delta;
+	}
+	return UT64_MAX;
+}
+
 R_API void r_io_bind(RIO *io, RIOBind *bnd) {
 	r_return_if_fail (io && bnd);
 
@@ -504,6 +505,8 @@ R_API void r_io_bind(RIO *io, RIOBind *bnd) {
 	bnd->desc_use = r_io_use_fd;
 	bnd->desc_get = r_io_desc_get;
 	bnd->desc_size = r_io_desc_size;
+	bnd->p2v = r_io_p2v;
+	bnd->v2p = r_io_v2p;
 	bnd->open = r_io_open_nomap;
 	bnd->open_at = r_io_open_at;
 	bnd->close = r_io_fd_close;
@@ -514,6 +517,7 @@ R_API void r_io_bind(RIO *io, RIOBind *bnd) {
 	bnd->fd_close = r_io_fd_close;
 	bnd->fd_seek = r_io_fd_seek;
 	bnd->fd_size = r_io_fd_size;
+	bnd->fd_resize = r_io_fd_resize;
 	bnd->fd_read = r_io_fd_read;
 	bnd->fd_write = r_io_fd_write;
 	bnd->fd_read_at = r_io_fd_read_at;
@@ -523,10 +527,10 @@ R_API void r_io_bind(RIO *io, RIOBind *bnd) {
 	bnd->fd_get_map = r_io_map_get_for_fd;
 	bnd->fd_remap = r_io_map_remap_fd;
 	bnd->is_valid_offset = r_io_is_valid_offset;
+	bnd->map_get = r_io_map_get;
+	bnd->map_get_paddr = r_io_map_get_paddr;
 	bnd->addr_is_mapped = r_io_addr_is_mapped;
-	bnd->sections_vget = r_io_sections_vget;
-	bnd->section_add = r_io_section_add;
-	bnd->sect_vget = r_io_section_vget;
+	bnd->map_add = r_io_map_add;
 #if HAVE_PTRACE
 	bnd->ptrace = r_io_ptrace;
 	bnd->ptrace_func = r_io_ptrace_func;
@@ -660,7 +664,6 @@ R_API int r_io_fini(RIO* io) {
 	r_io_desc_cache_fini_all (io);
 	r_io_desc_fini (io);
 	r_io_map_fini (io);
-	r_io_section_fini (io);
 	ls_free (io->plugins);
 	r_list_free (io->cache);
 	r_list_free (io->undo.w_list);
