@@ -179,8 +179,9 @@ static RIODesc *__open(RIO *io, const char *pathname, int rw, int mode) {
 
 RIOPlugin r_io_plugin_gzip = {
 	.name = "gzip",
-	.desc = "read/write gzipped files",
+	.desc = "Read/write gzipped files",
 	.license = "LGPL3",
+	.uris = "gzip://",
 	.open = __open,
 	.close = __close,
 	.read = __read,
@@ -190,7 +191,7 @@ RIOPlugin r_io_plugin_gzip = {
 	.resize = __resize,
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_IO,
 	.data = &r_io_plugin_gzip,
