@@ -18,7 +18,7 @@ static inline ut32 _io_malloc_sz(RIODesc *desc) {
 	}
 	RIOMalloc *mal = (RIOMalloc*)desc->data;
 	return mal? mal->size: 0;
-} 
+}
 
 static inline void _io_malloc_set_sz(RIODesc *desc, ut32 sz) {
 	if (!desc) {
@@ -28,7 +28,7 @@ static inline void _io_malloc_set_sz(RIODesc *desc, ut32 sz) {
 	if (mal) {
 		mal->size = sz;
 	}
-} 
+}
 
 static inline ut8* _io_malloc_buf(RIODesc *desc) {
 	if (!desc) {
@@ -117,6 +117,7 @@ static int __read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
 		count = mallocsz - _io_malloc_off (fd);
 	}
 	memcpy (buf, _io_malloc_buf (fd) + _io_malloc_off (fd), count);
+	_io_malloc_set_off (fd, _io_malloc_off (fd) + count);
 	return count;
 }
 
@@ -157,7 +158,7 @@ static bool __check(RIO *io, const char *pathname, bool many) {
 }
 
 static RIODesc *__open(RIO *io, const char *pathname, int rw, int mode) {
-	if (__check (io, pathname,0)) {
+	if (__check (io, pathname, 0)) {
 		RIOMalloc *mal = R_NEW0 (RIOMalloc);
 		if (!strncmp (pathname, "hex://", 6)) {
 			mal->size = strlen (pathname);
@@ -182,7 +183,7 @@ static RIODesc *__open(RIO *io, const char *pathname, int rw, int mode) {
 			mal->buf = calloc (1, mal->size + 1);
 		}
 		if (mal->buf) {
-			return r_io_desc_new (io, &r_io_plugin_malloc, pathname, R_IO_RW | rw, mode, mal);
+			return r_io_desc_new (io, &r_io_plugin_malloc, pathname, R_PERM_RW | rw, mode, mal);
 		}
 		eprintf ("Cannot allocate (%s) %d byte(s)\n", pathname + 9, mal->size);
 		free (mal);
@@ -192,7 +193,8 @@ static RIODesc *__open(RIO *io, const char *pathname, int rw, int mode) {
 
 RIOPlugin r_io_plugin_malloc = {
 	.name = "malloc",
-	.desc = "memory allocation (malloc://1024 hex://cd8090)",
+	.desc = "Memory allocation plugin",
+	.uris = "malloc://,hex://",
 	.license = "LGPL3",
 	.open = __open,
 	.close = __close,
@@ -203,8 +205,8 @@ RIOPlugin r_io_plugin_malloc = {
 	.resize = __resize,
 };
 
-#ifndef CORELIB
-RLibStruct radare_plugin = {
+#ifndef R2_PLUGIN_INCORE
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_IO,
 	.data = &r_io_plugin_malloc,
 	.version = R2_VERSION
