@@ -82,9 +82,10 @@ static int __read(RIO *io, RIODesc *desc, ut8 *buf, int len) {
 	/* reopen procpidmem if necessary */
 #if USE_PROC_PID_MEM
 	fd = RIOPTRACE_FD (desc);
-	if (RIOPTRACE_PID(desc) != RIOPTRACE_OPID(desc)) {
-		if (fd != -1)
+	if (RIOPTRACE_PID (desc) != RIOPTRACE_OPID (desc)) {
+		if (fd != -1) {
 			close (fd);
+		}
 		open_pidmem ((RIOPtrace*)desc->data);
 		fd = RIOPTRACE_FD (desc);
 		RIOPTRACE_OPID(desc) = RIOPTRACE_PID(desc);
@@ -93,13 +94,21 @@ static int __read(RIO *io, RIODesc *desc, ut8 *buf, int len) {
 	if (fd != -1) {
 		ret = lseek (fd, addr, SEEK_SET);
 		if (ret >=0) {
-			ret = read (fd, buf, len);
 			// Workaround for the buggy Debian Wheeze's /proc/pid/mem
-			if (ret != -1) return ret;
+			if (read (fd, buf, len) != -1) {
+				return ret;
+			}
 		}
 	}
 #endif
-	return debug_os_read_at (io, RIOPTRACE_PID (desc), (ut32*)buf, len, addr);
+	ut32 *aligned_buf = (ut32*)r_malloc_aligned (len, sizeof (ut32));
+	if (aligned_buf) {
+		int res = debug_os_read_at (io, RIOPTRACE_PID (desc), (ut32*)aligned_buf, len, addr);
+		memcpy (buf, aligned_buf, len);
+		r_free_aligned (aligned_buf);
+		return res;
+	}
+	return -1;
 }
 
 static int ptrace_write_at(RIO *io, int pid, const ut8 *pbuf, int sz, ut64 addr) {
@@ -139,8 +148,9 @@ static void open_pidmem (RIOPtrace *iop) {
 	char pidmem[32];
 	snprintf (pidmem, sizeof (pidmem), "/proc/%d/mem", iop->pid);
 	iop->fd = open (pidmem, O_RDWR);
-	if (iop->fd == -1)
+	if (iop->fd == -1) {
 		iop->fd = open (pidmem, O_RDONLY);
+	}
 #if 0
 	if (iop->fd == -1)
 		eprintf ("Warning: Cannot open /proc/%d/mem. "
@@ -168,15 +178,41 @@ static bool __plugin_open(RIO *io, const char *file, bool many) {
 	return false;
 }
 
+static inline bool is_pid_already_attached (RIO *io, int pid, void *data) {
+#if defined(__linux__)
+	siginfo_t *sig = (siginfo_t *)data;
+	return -1 != r_io_ptrace (io, PTRACE_GETSIGINFO, pid, NULL, sig);
+#elif defined(__FreeBSD__)
+	struct ptrace_lwpinfo *info = (struct ptrace_lwpinfo *)data;
+	int len = (int)sizeof (*info);
+	return -1 != r_io_ptrace (io, PT_LWPINFO, pid, info, len);
+#else
+	return false;
+#endif
+}
+
 static RIODesc *__open(RIO *io, const char *file, int rw, int mode) {
 	RIODesc *desc = NULL;
 	int ret = -1;
-	if (__plugin_open (io, file,0)) {
-		int pid = atoi (file+9);
+#if defined(__linux__)
+	siginfo_t sig = { 0 };
+#elif defined(__FreeBSD__)
+	struct ptrace_lwpinfo sig = { 0 };
+#else
+	int sig = 0;
+#endif
+
+	if (!__plugin_open (io, file, 0)) {
+		return NULL;
+	}
+
+	int pid = atoi (file + 9);
+
+	// Safely check if the PID has already been attached to avoid printing errors
+	// and attempt attaching on failure
+	if (!is_pid_already_attached(io, pid, &sig)) {
 		ret = r_io_ptrace (io, PTRACE_ATTACH, pid, 0, 0);
-		if (file[0] == 'p') { //ptrace
-			ret = 0;
-		} else if (ret == -1) {
+		if (ret == -1) {
 #ifdef __ANDROID__
 			eprintf ("ptrace_attach: Operation not permitted\n");
 #else
@@ -191,36 +227,38 @@ static RIODesc *__open(RIO *io, const char *file, int rw, int mode) {
 				break;
 			}
 #endif
+			return NULL;
 		} else if (__waitpid (pid)) {
 			ret = pid;
 		} else {
 			eprintf ("Error in waitpid\n");
-		}
-		if (ret != -1) {
-			RIOPtrace *riop = R_NEW0 (RIOPtrace);
-			if (!riop) {
-				return NULL;
-			}
-			riop->pid = riop->tid = pid;
-			open_pidmem (riop);
-			desc = r_io_desc_new (io, &r_io_plugin_ptrace, file, rw | R_PERM_X, mode, riop);
-			desc->name = r_sys_pid_to_path (pid);
+			return NULL;
 		}
 	}
+
+	RIOPtrace *riop = R_NEW0 (RIOPtrace);
+	if (!riop) {
+		return NULL;
+	}
+
+	riop->pid = riop->tid = pid;
+	open_pidmem (riop);
+	desc = r_io_desc_new (io, &r_io_plugin_ptrace, file, rw | R_PERM_X, mode, riop);
+	desc->name = r_sys_pid_to_path (pid);
+
 	return desc;
 }
 
 static ut64 __lseek(RIO *io, RIODesc *fd, ut64 offset, int whence) {
 	switch (whence) {
-	case 0: // abs
+	case R_IO_SEEK_SET:
 		io->off = offset;
 		break;
-	case 1: // cur
-		io->off += (int)offset;
-		break;
-	case 2: // end
-		io->off = UT64_MAX;
+	case R_IO_SEEK_CUR:
+		io->off += offset;
 		break;
+	case R_IO_SEEK_END:
+		io->off = ST64_MAX;
 	}
 	return io->off;
 }
@@ -246,6 +284,9 @@ static char *__system(RIO *io, RIODesc *fd, const char *cmd) {
 	RIOPtrace *iop = (RIOPtrace*)fd->data;
 	//printf("ptrace io command (%s)\n", cmd);
 	/* XXX ugly hack for testing purposes */
+	if (!strcmp (cmd, "")) {
+		return NULL;
+	}
 	if (!strcmp (cmd, "help")) {
 		eprintf ("Usage: =!cmd args\n"
 			" =!ptrace   - use ptrace io\n"
@@ -290,8 +331,9 @@ static int __getpid (RIODesc *fd) {
 // TODO: rename ptrace to io_ptrace .. err io.ptrace ??
 RIOPlugin r_io_plugin_ptrace = {
 	.name = "ptrace",
-	.desc = "ptrace and /proc/pid/mem (if available) io",
+	.desc = "Ptrace and /proc/pid/mem (if available) io plugin",
 	.license = "LGPL3",
+	.uris = "ptrace://,attach://",
 	.open = __open,
 	.close = __close,
 	.read = __read,
@@ -309,7 +351,7 @@ struct r_io_plugin_t r_io_plugin_ptrace = {
 };
 #endif
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_IO,
 	.data = &r_io_plugin_ptrace,
