@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2008-2015 pancake */
+/* radare - LGPL - Copyright 2008-2019 pancake */
 
 #include "r_io.h"
 #include "r_lib.h"
@@ -9,9 +9,15 @@
 #define __UNIX__ 0
 #endif
 
+// linux requires -lrt for this, but still it seems to not work as expected
+// better not to enable it by default until we get enough time to properly
+// make this work across all unixes without adding extra depenencies
+#define USE_SHM_OPEN 0
+
 #if __UNIX__ && !defined (__QNX__) && !defined (__HAIKU__)
 #include <sys/ipc.h>
 #include <sys/shm.h>
+#include <sys/mman.h>
 
 typedef struct {
 	int fd;
@@ -24,61 +30,56 @@ typedef struct {
 #define SHMATSZ 0x9000; // 32*1024*1024; /* 32MB : XXX not used correctly? */
 
 static int shm__write(RIO *io, RIODesc *fd, const ut8 *buf, int count) {
-	RIOShm *shm;
-	if (!fd || !fd->data) {
-		return -1;
-	}
-	shm = fd->data;
-	if (shm->buf != NULL) {
-		(void)memcpy (shm->buf+io->off, buf, count);
+	r_return_val_if_fail (fd && fd->data, -1);
+	RIOShm *shm = fd->data;
+	if (shm->buf) {
+		(void)memcpy (shm->buf + io->off, buf, count);
 		return count;
 	}
-	return -1;
+	return write (shm->fd, buf, count);
 }
 
 static int shm__read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
-	RIOShm *shm;
-	if (!fd || !fd->data) {
-		return -1;
-	}
-	shm = fd->data;
-	if (io->off+count >= shm->size) {
+	r_return_val_if_fail (fd && fd->data, -1);
+	RIOShm *shm = fd->data;
+	if (io->off + count >= shm->size) {
 		if (io->off > shm->size) {
 			return -1;
 		}
 		count = shm->size - io->off;
 	}
-	if (count > 32) {
-		count = 32;
+	if (shm->buf) {
+		memcpy (buf, shm->buf+io->off , count);
+		return count;
 	}
-	memcpy (buf, shm->buf+io->off , count);
-	return count;
+	return read (shm->fd, buf, count);
 }
 
 static int shm__close(RIODesc *fd) {
+	r_return_val_if_fail (fd && fd->data, -1);
 	int ret;
-	if (!fd || !fd->data) {
-		return -1;
+	RIOShm *shm = fd->data;
+	if (shm->buf) {
+		ret = shmdt (((RIOShm*)(fd->data))->buf);
+	} else {
+		ret = close (shm->fd);
 	}
-	ret = shmdt (((RIOShm*)(fd->data))->buf);
 	R_FREE (fd->data);
 	return ret;
 }
 
 static ut64 shm__lseek(RIO *io, RIODesc *fd, ut64 offset, int whence) {
-	RIOShm *shm;
-	if (!fd || !fd->data) {
-		return -1;
-	}
-	shm = fd->data;
+	r_return_val_if_fail (fd && fd->data, -1);
+	RIOShm *shm = fd->data;
 	switch (whence) {
 	case SEEK_SET:
-		return offset;
+		return io->off = offset;
 	case SEEK_CUR:
 		if (io->off + offset > shm->size) {
-			return shm->size;
+			return io->off = shm->size;
 		}
-		return io->off + offset;
+		io->off += offset;
+		return io->off;
 	case SEEK_END:
 		return 0xffffffff;
 	}
@@ -89,12 +90,8 @@ static bool shm__plugin_open(RIO *io, const char *pathname, bool many) {
 	return (!strncmp (pathname, "shm://", 6));
 }
 
-static inline int getshmid (const char *str) {
-	return atoi (str);
-}
-
 static inline int getshmfd (RIOShm *shm) {
-	return (int)(size_t)shm->buf;
+	return (((int)(size_t)shm->buf) >> 4) & 0xfff;
 }
 
 static RIODesc *shm__open(RIO *io, const char *pathname, int rw, int mode) {
@@ -103,10 +100,23 @@ static RIODesc *shm__open(RIO *io, const char *pathname, int rw, int mode) {
 		if (!shm) {
 			return NULL;
 		}
-		const char *ptr = pathname+6;
-		shm->id = getshmid (ptr);
+		const char *ptr = pathname + 6;
+		shm->id = atoi (ptr);
+		if (!shm->id) {
+			shm->id = r_str_hash (ptr);
+		}
 		shm->buf = shmat (shm->id, 0, 0);
-		shm->fd = getshmfd (shm);
+		if (shm->buf == (void*)(size_t)-1) {
+#if USE_SHM_OPEN
+			shm->buf = NULL;
+			shm->fd = shm_open (ptr, O_CREAT | (rw?O_RDWR:O_RDONLY), 0644);
+#else
+			shm->fd = -1;
+#endif
+
+		} else {
+			shm->fd = getshmfd (shm);
+		}
 		shm->size = SHMATSZ;
 		if (shm->fd != -1) {
 			eprintf ("Connected to shared memory 0x%08x\n", shm->id);
@@ -120,8 +130,9 @@ static RIODesc *shm__open(RIO *io, const char *pathname, int rw, int mode) {
 
 RIOPlugin r_io_plugin_shm = {
 	.name = "shm",
-	.desc = "shared memory resources (shm://key)",
-	.license = "LGPL3",
+	.desc = "Shared memory resources plugin",
+	.uris = "shm://",
+	.license = "MIT",
 	.open = shm__open,
 	.close = shm__close,
 	.read = shm__read,
@@ -131,13 +142,13 @@ RIOPlugin r_io_plugin_shm = {
 };
 
 #else
-struct r_io_plugin_t r_io_plugin_shm = {
+RIOPlugin r_io_plugin_shm = {
 	.name = "shm",
 	.desc = "shared memory resources (not for w32)",
 };
 #endif
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_IO,
 	.data = &r_io_plugin_shm,
