@@ -101,7 +101,8 @@ static RIODesc *__open(RIO *io, const char *pathname, int rw, int mode) {
 
 RIOPlugin r_io_plugin_sparse = {
 	.name = "sparse",
-	.desc = "sparse buffer allocation (sparse://1024 sparse://)",
+	.desc = "Sparse buffer allocation plugin",
+	.uris = "sparse://",
 	.license = "LGPL3",
 	.open = __open,
 	.close = __close,
@@ -112,7 +113,7 @@ RIOPlugin r_io_plugin_sparse = {
 	.resize = NULL,
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_IO,
 	.data = &r_io_plugin_sparse,
