@@ -54,11 +54,7 @@ static int __write(RIO *io, RIODesc *fd, const ut8 *buf, int count) {
 		return -1;
 	}
 	if (windbg_get_target (fd->data)) {
-		ut64 va;
-		if (!windbg_va_to_pa (fd->data, io->off, &va)) {
-			return -1;
-		}
-		return windbg_write_at_phys (fd->data, buf, va, count);
+		return windbg_write_at_uva (fd->data, buf, io->off, count);
 	}
 	return windbg_write_at (fd->data, buf, io->off, count);
 }
@@ -66,11 +62,11 @@ static int __write(RIO *io, RIODesc *fd, const ut8 *buf, int count) {
 static ut64 __lseek(RIO *io, RIODesc *fd, ut64 offset, int whence) {
 	switch (whence) {
 	case R_IO_SEEK_SET:
-		return offset;
+		return io->off = offset;
 	case R_IO_SEEK_CUR:
 		return io->off + offset;
 	case R_IO_SEEK_END:
-		return UT64_MAX;
+		return ST64_MAX;
 	default:
 		return offset;
 	}
@@ -82,24 +78,21 @@ static int __read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
 	}
 
 	if (windbg_get_target (fd->data)) {
-		ut64 va;
-		if (!windbg_va_to_pa (fd->data, io->off, &va)) {
-			return -1;
-		}
-		return windbg_read_at_phys (fd->data, buf, va, count);
+		return windbg_read_at_uva (fd->data, buf, io->off, count);
 	}
 
 	return windbg_read_at (fd->data, buf, io->off, count);
 }
 
 static int __close(RIODesc *fd) {
-	windbg_ctx_free (fd->data);
+	windbg_ctx_free ((WindCtx**)&fd->data);
 	return true;
 }
 
 RIOPlugin r_io_plugin_windbg = {
 	.name = "windbg",
-	.desc = "Attach to a KD debugger (windbg://socket)",
+	.desc = "Attach to a KD debugger",
+	.uris = "windbg://",
 	.license = "LGPL3",
 	.open = __open,
 	.close = __close,
@@ -110,7 +103,7 @@ RIOPlugin r_io_plugin_windbg = {
 	.isdbg = true
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_IO,
 	.data = &r_io_plugin_windbg,
