@@ -30,13 +30,11 @@ R_API void r_io_undo_enable(RIO *io, int s, int w) {
 /* undo seekz */
 
 R_API RIOUndos *r_io_sundo(RIO *io, ut64 offset) {
-	RIOUndos *undo;
-	RIOSection *sec;
-
 	if (!io->undo.s_enable || !io->undo.undos) {
 		return NULL;
 	}
 
+	RIOUndos *undo;
 	/* No redos yet, store the current seek so we can redo to it. */
 	if (!io->undo.redos) {
 		undo = &io->undo.seek[io->undo.idx];
@@ -49,18 +47,18 @@ R_API RIOUndos *r_io_sundo(RIO *io, ut64 offset) {
 	io->undo.redos++;
 
 	undo = &io->undo.seek[io->undo.idx];
-	sec = r_io_section_vget (io, undo->off);
-	if (!sec || (sec->paddr == sec->vaddr)) {
+	RIOMap *map = r_io_map_get (io, undo->off);
+	if (!map || (map->delta == map->itv.addr)) {
 		io->off = undo->off;
 	} else {
-		io->off = undo->off - sec->vaddr + sec->paddr;
+		io->off = undo->off - (map->itv.addr + map->delta);
 	}
 	return undo;
 }
 
 R_API RIOUndos *r_io_sundo_redo(RIO *io) {
 	RIOUndos *undo;
-	RIOSection *sec;
+	RIOMap *map;
 
 	if (!io->undo.s_enable || !io->undo.redos) {
 		return NULL;
@@ -71,11 +69,11 @@ R_API RIOUndos *r_io_sundo_redo(RIO *io) {
 	io->undo.redos--;
 
 	undo = &io->undo.seek[io->undo.idx];
-	sec = r_io_section_vget (io, undo->off);
-	if (!sec || (sec->paddr == sec->vaddr)) {
+	map = r_io_map_get (io, undo->off);
+	if (!map || (map->delta == map->itv.addr)) {
 		io->off = undo->off;
 	} else {
-		io->off = undo->off - sec->vaddr + sec->paddr;
+		io->off = undo->off - map->itv.addr + map->delta;
 	}
 	return undo;
 }
@@ -85,9 +83,9 @@ R_API void r_io_sundo_push(RIO *io, ut64 off, int cursor) {
 	if (!io->undo.s_enable) {
 		return;
 	}
-	// the first insert
-	if (io->undo.idx > 0) {
-		undo = &io->undo.seek[io->undo.idx - 1];
+	// don't push duplicate seek
+	if (io->undo.undos > 0) {
+		undo = &io->undo.seek[(io->undo.idx - 1 + R_IO_UNDOS) % R_IO_UNDOS];
 		if (undo->off == off && undo->cursor == cursor) {
 			return;
 		}
@@ -129,7 +127,7 @@ R_API RList *r_io_sundo_list(RIO *io, int mode) {
 
 	idx = io->undo.idx;
 	start = (idx - undos + R_IO_UNDOS) % R_IO_UNDOS;
-	end = (idx + redos + 1) % R_IO_UNDOS;
+	end = (idx + redos + 1 - 1) % R_IO_UNDOS; // +1 slot for current position, -1 due to inclusive end
 
 	j = 0;
 	switch (mode) {
@@ -140,24 +138,17 @@ R_API RList *r_io_sundo_list(RIO *io, int mode) {
 		list = r_list_newf (free);
 		break;
 	}
-	const char *comma = "";
-	for (i = start; i < end || j == 0; i = (i + 1) % R_IO_UNDOS) {
+	for (i = start;/* condition at the end of loop */; i = (i + 1) % R_IO_UNDOS) {
 		int idx = (j < undos)? undos - j - 1: j - undos - 1;
 		RIOUndos *undo = &io->undo.seek[i];
 		ut64 addr = undo->off;
-		ut64 notLast = (j + 1 < undos) && (i != end - 1);
+		bool notLast = (j + 1 < undos);
 		switch (mode) {
 		case '=':
 			if (j < undos) {
 				io->cb_printf ("0x%"PFMT64x"%s", addr, notLast? " > ": "");
 			}
 			break;
-		case 'j':
-			if (j < undos) {
-				io->cb_printf ("%"PFMT64d"%s", addr, notLast? ",": "");
-				comma = ",";
-			}
-			break;
 		case '*':
 			if (j < undos) {
 				io->cb_printf ("f undo_%d @ 0x%"PFMT64x"\n", idx, addr);
@@ -171,21 +162,27 @@ R_API RList *r_io_sundo_list(RIO *io, int mode) {
 			if (list) {
 				RIOUndos  *u = R_NEW0 (RIOUndos);
 				if (u) {
-					memcpy (u, undo, sizeof (RIOUndos));
+					if (!(j == undos && redos == 0)) {
+						// Current position gets pushed before seek, so there
+						// is no valid offset when we are at the end of list.
+						memcpy (u, undo, sizeof (RIOUndos));
+					} else {
+						u->off = io->off;
+					}
 					r_list_append (list, u);
 				}
 			}
 			break;
 		}
 		j++;
+		if (i == end) {
+			break;
+		}
 	}
 	switch (mode) {
 	case '=':
 		io->cb_printf ("\n");
 		break;
-	case 'j':
-		io->cb_printf ("%s%"PFMT64d"]\n", comma, io->off);
-		break;
 	}
 	return list;
 }
@@ -194,8 +191,9 @@ R_API RList *r_io_sundo_list(RIO *io, int mode) {
 
 R_API void r_io_wundo_new(RIO *io, ut64 off, const ut8 *data, int len) {
 	RIOUndoWrite *uw;
-	if (!io->undo.w_enable)
+	if (!io->undo.w_enable) {
 		return;
+	}
 	/* undo write changes */
 	uw = R_NEW0 (RIOUndoWrite);
 	if (!uw) {
@@ -237,17 +235,26 @@ R_API void r_io_wundo_list(RIO *io) {
 	RIOUndoWrite *u;
 	int i = 0, j, len;
 
-	if (io->undo.w_init)
-	r_list_foreach (io->undo.w_list, iter, u) {
-		io->cb_printf ("%02d %c %d %08"PFMT64x": ", i, u->set?'+':'-', u->len, u->off);
-		len = (u->len>BW)?BW:u->len;
-		for (j=0;j<len;j++) io->cb_printf ("%02x ", u->o[j]);
-		if (len == BW) io->cb_printf (".. ");
-		io->cb_printf ("=> ");
-		for (j=0;j<len;j++) io->cb_printf ("%02x ", u->n[j]);
-		if (len == BW) io->cb_printf (".. ");
-		io->cb_printf ("\n");
-		i++;
+	if (io->undo.w_init) {
+		r_list_foreach (io->undo.w_list, iter, u) {
+			io->cb_printf ("%02d %c %d %08" PFMT64x ": ", i, u->set ? '+' : '-', u->len, u->off);
+			len = (u->len > BW) ? BW : u->len;
+			for (j = 0; j < len; j++) {
+				io->cb_printf ("%02x ", u->o[j]);
+			}
+			if (len == BW) {
+				io->cb_printf (".. ");
+			}
+			io->cb_printf ("=> ");
+			for (j = 0; j < len; j++) {
+				io->cb_printf ("%02x ", u->n[j]);
+			}
+			if (len == BW) {
+				io->cb_printf (".. ");
+			}
+			io->cb_printf ("\n");
+			i++;
+		}
 	}
 }
 
