@@ -164,12 +164,14 @@ static int match(RMagic *ms, struct r_magic *magic, ut32 nmagic, const ut8 *s, s
 		if (*R_MAGIC_DESC) {
 			need_separator = 1;
 			printed_something = 1;
-			if (print_sep(ms, firstline) == -1)
+			if (print_sep (ms, firstline) == -1) {
 				return -1;
+			}
 		}
 
-		if ((ms->c.li[cont_level].off = mprint(ms, m)) == -1)
+		if ((ms->c.li[cont_level].off = mprint (ms, m)) == -1) {
 			return -1;
+		}
 
 		/* and any continuations that match */
 		if (file_check_mem(ms, ++cont_level) == -1) {
@@ -191,16 +193,19 @@ static int match(RMagic *ms, struct r_magic *magic, ut32 nmagic, const ut8 *s, s
 				cont_level = m->cont_level;
 			}
 			ms->offset = m->offset;
-			if (m->flag & OFFADD)
+			if (m->flag & OFFADD) {
 				ms->offset += ms->c.li[cont_level - 1].off;
+			}
 
 			if (m->cond == COND_ELSE || m->cond == COND_ELIF) {
-				if (ms->c.li[cont_level].last_match == 1)
+				if (ms->c.li[cont_level].last_match == 1) {
 					continue;
+				}
 			}
 			flush = !mget(ms, s, m, nbytes, cont_level);
-			if (flush && m->reln != '!')
+			if (flush && m->reln != '!') {
 				continue;
+			}
 
 			switch (flush ? 1 : magiccheck(ms, m)) {
 			case -1:
@@ -210,9 +215,9 @@ static int match(RMagic *ms, struct r_magic *magic, ut32 nmagic, const ut8 *s, s
 				break;
 			default:
 				ms->c.li[cont_level].last_match = 1;
-				if (m->type != FILE_DEFAULT)
+				if (m->type != FILE_DEFAULT) {
 					ms->c.li[cont_level].got_match = 1;
-				else if (ms->c.li[cont_level].got_match) {
+				} else if (ms->c.li[cont_level].got_match) {
 					ms->c.li[cont_level].got_match = 0;
 					break;
 				}
@@ -222,8 +227,9 @@ static int match(RMagic *ms, struct r_magic *magic, ut32 nmagic, const ut8 *s, s
 				 */
 				if (*R_MAGIC_DESC) {
 					printed_something = 1;
-					if (print_sep(ms, firstline) == -1)
+					if (print_sep (ms, firstline) == -1) {
 						return -1;
+					}
 				}
 				/*
 				 * This continuation matched.  Print
@@ -235,22 +241,26 @@ static int match(RMagic *ms, struct r_magic *magic, ut32 nmagic, const ut8 *s, s
 				if (need_separator
 				    && ((m->flag & NOSPACE) == 0)
 				    && *R_MAGIC_DESC) {
-					if (file_printf (ms, " ") == -1)
+					if (file_printf (ms, " ") == -1) {
 						return -1;
+					}
 					need_separator = 0;
 				}
-				if ((ms->c.li[cont_level].off = mprint(ms, m)) == -1)
+				if ((ms->c.li[cont_level].off = mprint (ms, m)) == -1) {
 					return -1;
-				if (*R_MAGIC_DESC)
+				}
+				if (*R_MAGIC_DESC) {
 					need_separator = 1;
+				}
 
 				/*
 				 * If we see any continuations
 				 * at a higher level,
 				 * process them.
 				 */
-				if (file_check_mem(ms, ++cont_level) == -1)
+				if (file_check_mem (ms, ++cont_level) == -1) {
 					return -1;
+				}
 				break;
 			}
 		}
@@ -258,8 +268,9 @@ static int match(RMagic *ms, struct r_magic *magic, ut32 nmagic, const ut8 *s, s
 			firstline = 0;
 			returnval = 1;
 		}
-		if ((ms->flags & R_MAGIC_CONTINUE) == 0 && printed_something)
+		if ((ms->flags & R_MAGIC_CONTINUE) == 0 && printed_something) {
 			return 1; /* don't keep searching */
+		}
 	}
 	return returnval;  /* This is hit if -k is set or there is no match */
 }
@@ -268,8 +279,9 @@ static int check_fmt(RMagic *ms, struct r_magic *m) {
 	RRegex rx;
 	int rc;
 
-	if (!strchr (R_MAGIC_DESC, '%'))
+	if (!strchr (R_MAGIC_DESC, '%')) {
 		return 0;
+	}
 
 	rc = r_regex_comp (&rx, "%[-0-9\\.]*s", R_REGEX_EXTENDED|R_REGEX_NOSUB);
 	if (rc) {
@@ -288,10 +300,10 @@ char * strdupn(const char *str, size_t n) {
 	size_t len;
 	char *copy;
 
-	for (len = 0; len < n && str[len]; len++)
-		continue;
-	if (!(copy = malloc (len + 1)))
+	for (len = 0; len < n && str[len]; len++) {}
+	if (!(copy = malloc (len + 1))) {
 		return NULL;
+	}
 	(void)memcpy (copy, str, len);
 	copy[len] = '\0';
 	return copy;
@@ -323,8 +335,9 @@ static st32 mprint(RMagic *ms, struct r_magic *m) {
 			}
 			break;
 		default:
-			if (file_printf(ms, R_MAGIC_DESC, (ut8) v) == -1)
+			if (file_printf (ms, R_MAGIC_DESC, (ut8)v) == -1) {
 				return -1;
+			}
 			break;
 		}
 		t = ms->offset + sizeof(char);
@@ -348,8 +361,9 @@ static st32 mprint(RMagic *ms, struct r_magic *m) {
 			}
 			break;
 		default:
-			if (file_printf(ms, R_MAGIC_DESC, (unsigned short) v) == -1)
+			if (file_printf (ms, R_MAGIC_DESC, (unsigned short)v) == -1) {
 				return -1;
+			}
 			break;
 		}
 		t = ms->offset + sizeof(short);
@@ -374,8 +388,9 @@ static st32 mprint(RMagic *ms, struct r_magic *m) {
 			}
 			break;
 		default:
-			if (file_printf(ms, R_MAGIC_DESC, (ut32) v) == -1)
+			if (file_printf (ms, R_MAGIC_DESC, (ut32)v) == -1) {
 				return -1;
+			}
 			break;
 		}
 		t = ms->offset + sizeof(st32);
@@ -384,8 +399,9 @@ static st32 mprint(RMagic *ms, struct r_magic *m) {
   	case FILE_BEQUAD:
   	case FILE_LEQUAD:
 		v = file_signextend(ms, m, p->q);
-		if (file_printf(ms, R_MAGIC_DESC, (ut64) v) == -1)
+		if (file_printf (ms, R_MAGIC_DESC, (ut64)v) == -1) {
 			return -1;
+		}
 		t = ms->offset + sizeof(ut64);
   		break;
 
@@ -394,50 +410,56 @@ static st32 mprint(RMagic *ms, struct r_magic *m) {
   	case FILE_BESTRING16:
   	case FILE_LESTRING16:
 		if (m->reln == '=' || m->reln == '!') {
-			if (file_printf (ms, R_MAGIC_DESC, m->value.s) == -1)
+			if (file_printf (ms, R_MAGIC_DESC, m->value.s) == -1) {
 				return -1;
+			}
 			t = ms->offset + m->vallen;
 		}
 		else {
-			if (*m->value.s == '\0')
+			if (*m->value.s == '\0') {
 				p->s[strcspn (p->s, "\n")] = '\0';
-			if (file_printf (ms, R_MAGIC_DESC, p->s) == -1)
+			}
+			if (file_printf (ms, R_MAGIC_DESC, p->s) == -1) {
 				return -1;
+			}
 			t = ms->offset + strlen (p->s);
-			if (m->type == FILE_PSTRING)
+			if (m->type == FILE_PSTRING) {
 				t++;
+			}
 		}
 		break;
 	case FILE_DATE:
 	case FILE_BEDATE:
 	case FILE_LEDATE:
 	case FILE_MEDATE:
-		if (file_printf(ms, R_MAGIC_DESC, file_fmttime(p->l, 1)) == -1)
+		if (file_printf (ms, R_MAGIC_DESC, file_fmttime (p->l, 1)) == -1) {
 			return -1;
+		}
 		t = ms->offset + sizeof(time_t);
 		break;
 	case FILE_LDATE:
 	case FILE_BELDATE:
 	case FILE_LELDATE:
 	case FILE_MELDATE:
-		if (file_printf(ms, R_MAGIC_DESC, file_fmttime(p->l, 0)) == -1)
+		if (file_printf (ms, R_MAGIC_DESC, file_fmttime (p->l, 0)) == -1) {
 			return -1;
+		}
 		t = ms->offset + sizeof(time_t);
 		break;
 	case FILE_QDATE:
 	case FILE_BEQDATE:
 	case FILE_LEQDATE:
-		if (file_printf(ms, R_MAGIC_DESC, file_fmttime((ut32)p->q, 1))
-		    == -1)
+		if (file_printf (ms, R_MAGIC_DESC, file_fmttime ((ut32)p->q, 1)) == -1) {
 			return -1;
+		}
 		t = ms->offset + sizeof(ut64);
 		break;
 	case FILE_QLDATE:
 	case FILE_BEQLDATE:
 	case FILE_LEQLDATE:
-		if (file_printf(ms, R_MAGIC_DESC, file_fmttime((ut32)p->q, 0))
-		    == -1)
+		if (file_printf (ms, R_MAGIC_DESC, file_fmttime ((ut32)p->q, 0)) == -1) {
 			return -1;
+		}
 		t = ms->offset + sizeof(ut64);
 		break;
   	case FILE_FLOAT:
@@ -459,8 +481,9 @@ static st32 mprint(RMagic *ms, struct r_magic *m) {
 			}
 			break;
 		default:
-			if (file_printf(ms, R_MAGIC_DESC, vf) == -1)
+			if (file_printf (ms, R_MAGIC_DESC, vf) == -1) {
 				return -1;
+			}
 			break;
 		}
 		t = ms->offset + sizeof(float);
@@ -484,8 +507,9 @@ static st32 mprint(RMagic *ms, struct r_magic *m) {
 			}
 			break;
 		default:
-			if (file_printf(ms, R_MAGIC_DESC, vd) == -1)
+			if (file_printf (ms, R_MAGIC_DESC, vd) == -1) {
 				return -1;
+			}
 			break;
 		}
 		t = ms->offset + sizeof(double);
@@ -502,26 +526,32 @@ static st32 mprint(RMagic *ms, struct r_magic *m) {
 		rval = file_printf(ms, R_MAGIC_DESC, cp);
 		free(cp);
 
-		if (rval == -1)
+		if (rval == -1) {
 			return -1;
+		}
 
-		if ((m->str_flags & REGEX_OFFSET_START))
+		if ((m->str_flags & REGEX_OFFSET_START)) {
 			t = ms->search.offset;
-		else
+		} else {
 			t = ms->search.offset + ms->search.rm_len;
+		}
 		break;
 	}
 
 	case FILE_SEARCH:
-	  	if (file_printf(ms, R_MAGIC_DESC, m->value.s) == -1)
+		if (file_printf (ms, R_MAGIC_DESC, m->value.s) == -1) {
 			return -1;
-		if ((m->str_flags & REGEX_OFFSET_START))
+		}
+		if ((m->str_flags & REGEX_OFFSET_START)) {
 			t = ms->search.offset;
-		else t = ms->search.offset + m->vallen;
+		} else {
+			t = ms->search.offset + m->vallen;
+		}
 		break;
 	case FILE_DEFAULT:
-	  	if (file_printf(ms, R_MAGIC_DESC, m->value.s) == -1)
+		if (file_printf (ms, R_MAGIC_DESC, m->value.s) == -1) {
 			return -1;
+		}
 		t = ms->offset;
 		break;
 	default:
@@ -637,21 +667,25 @@ static int mconvert(RMagic *ms, struct r_magic *m) {
 		/* Null terminate and eat *trailing* return */
 		p->s[sizeof(p->s) - 1] = '\0';
 		len = strlen(p->s);
-		if (len-- && p->s[len] == '\n')
+		if (len-- && p->s[len] == '\n') {
 			p->s[len] = '\0';
+		}
 		return 1;
 	}
 	case FILE_PSTRING: {
 		char *ptr1 = p->s, *ptr2 = ptr1 + 1;
 		size_t len = *p->s;
-		if (len >= sizeof(p->s))
-			len = sizeof(p->s) - 1;
-		while (len--)
+		if (len >= sizeof (p->s)) {
+			len = sizeof (p->s) - 1;
+		}
+		while (len--) {
 			*ptr1++ = *ptr2++;
+		}
 		*ptr1 = '\0';
 		len = strlen(p->s);
-		if (len-- && p->s[len] == '\n')
+		if (len-- && p->s[len] == '\n') {
 			p->s[len] = '\0';
+		}
 		return 1;
 	}
 	case FILE_BESHORT:
@@ -780,11 +814,13 @@ static int mcopy(RMagic *ms, union VALUETYPE *p, int type, int indir, const ut8
 			     lines && ((b = strchr(c = b, '\n')) || (b = strchr(c, '\r')));
 			     lines--, b++) {
 				last = b;
-				if (b[0] == '\r' && b[1] == '\n')
+				if (b[0] == '\r' && b[1] == '\n') {
 					b++;
+				}
 			}
-			if (lines)
+			if (lines) {
 				last = (const char *)s + nbytes;
+			}
 
 			ms->search.s = buf;
 			ms->search.s_len = last - buf;
@@ -799,25 +835,26 @@ static int mcopy(RMagic *ms, union VALUETYPE *p, int type, int indir, const ut8
 			char *dst = p->s;
 			char *edst = &p->s[sizeof(p->s) - 1];
 
-			if (type == FILE_BESTRING16)
+			if (type == FILE_BESTRING16) {
 				src++;
+			}
 
 			/* check for pointer overflow */
 			if (src < s) {
-				file_magerror(ms, "invalid offset %zu in mcopy()",
+				file_magerror(ms, "invalid offset %u in mcopy()",
 				    offset);
 				return -1;
 			}
 			for (/*EMPTY*/; src < esrc; src += 2, dst++) {
-				if (dst < edst)
+				if (dst < edst) {
 					*dst = *src;
-				else
+				} else {
 					break;
+				}
 				if (*dst == '\0') {
-					if (type == FILE_BESTRING16 ?
-					    *(src - 1) != '\0' :
-					    *(src + 1) != '\0')
+					if (type == FILE_BESTRING16 ? *(src - 1) != '\0' : *(src + 1) != '\0') {
 						*dst = ' ';
+					}
 				}
 			}
 			*edst = '\0';
@@ -843,9 +880,10 @@ static int mcopy(RMagic *ms, union VALUETYPE *p, int type, int indir, const ut8
 	 * the usefulness of padding with zeroes eludes me, it
 	 * might even cause problems
 	 */
-	if (nbytes < sizeof(*p))
-		(void)memset(((char *)(void *)p) + nbytes, '\0',
-		    sizeof(*p) - nbytes);
+	if (nbytes < sizeof (*p)) {
+		(void)memset (((char *)(void *)p) + nbytes, '\0',
+			sizeof (*p) - nbytes);
+	}
 	return 0;
 }
 
@@ -854,8 +892,9 @@ static int mget(RMagic *ms, const ut8 *s, struct r_magic *m, size_t nbytes, unsi
 	ut32 count = m->str_range;
 	union VALUETYPE *p = &ms->ms_value;
 
-	if (mcopy(ms, p, m->type, m->flag & INDIR, s, offset, nbytes, count) == -1)
+	if (mcopy (ms, p, m->type, m->flag & INDIR, s, offset, nbytes, count) == -1) {
 		return -1;
+	}
 
 	if ((ms->flags & R_MAGIC_DEBUG) != 0) {
 		mdebug(offset, (char *)(void *)p, sizeof(union VALUETYPE));
@@ -896,8 +935,9 @@ static int mget(RMagic *ms, const ut8 *s, struct r_magic *m, size_t nbytes, unsi
 		}
 		switch (m->in_type) {
 		case FILE_BYTE:
-			if (nbytes < (offset + 1))
+			if (nbytes < (offset + 1)) {
 				return 0;
+			}
 			if (off) {
 				switch (m->in_op & FILE_OPS_MASK) {
 				case FILE_OPAND: offset = p->b & off; break;
@@ -909,14 +949,17 @@ static int mget(RMagic *ms, const ut8 *s, struct r_magic *m, size_t nbytes, unsi
 				case FILE_OPDIVIDE: offset = p->b / off; break;
 				case FILE_OPMODULO: offset = p->b % off; break;
 				}
-			} else
+			} else {
 				offset = p->b;
-			if (m->in_op & FILE_OPINVERSE)
+			}
+			if (m->in_op & FILE_OPINVERSE) {
 				offset = ~offset;
+			}
 			break;
 		case FILE_BESHORT:
-			if (nbytes < (offset + 2))
+			if (nbytes < (offset + 2)) {
 				return 0;
+			}
 			if (off) {
 				switch (m->in_op & FILE_OPS_MASK) {
 				case FILE_OPAND: offset = (short)((p->hs[0]<<8)| (p->hs[1])) & off; break;
@@ -928,15 +971,18 @@ static int mget(RMagic *ms, const ut8 *s, struct r_magic *m, size_t nbytes, unsi
 				case FILE_OPDIVIDE: offset = (short)((p->hs[0]<<8)| (p->hs[1])) / off; break;
 				case FILE_OPMODULO: offset = (short)((p->hs[0]<<8)| (p->hs[1])) % off; break;
 				}
-			} else
-				offset = (short)((p->hs[0]<<8)|
+			} else {
+				offset = (short)((p->hs[0] << 8) |
 						 (p->hs[1]));
-			if (m->in_op & FILE_OPINVERSE)
+			}
+			if (m->in_op & FILE_OPINVERSE) {
 				offset = ~offset;
+			}
 			break;
 		case FILE_LESHORT:
-			if (nbytes < (offset + 2))
+			if (nbytes < (offset + 2)) {
 				return 0;
+			}
 			if (off) {
 				switch (m->in_op & FILE_OPS_MASK) {
 				case FILE_OPAND: offset = (short)((p->hs[1]<<8)| (p->hs[0])) & off; break;
@@ -948,13 +994,17 @@ static int mget(RMagic *ms, const ut8 *s, struct r_magic *m, size_t nbytes, unsi
 				case FILE_OPDIVIDE: offset = (short)((p->hs[1]<<8)| (p->hs[0])) / off; break;
 				case FILE_OPMODULO: offset = (short)((p->hs[1]<<8)| (p->hs[0])) % off; break;
 				}
-			} else offset = (short)((p->hs[1]<<8)| (p->hs[0]));
-			if (m->in_op & FILE_OPINVERSE)
+			} else {
+				offset = (short)((p->hs[1] << 8) | (p->hs[0]));
+			}
+			if (m->in_op & FILE_OPINVERSE) {
 				offset = ~offset;
+			}
 			break;
 		case FILE_SHORT:
-			if (nbytes < (offset + 2))
+			if (nbytes < (offset + 2)) {
 				return 0;
+			}
 			if (off) {
 				switch (m->in_op & FILE_OPS_MASK) {
 				case FILE_OPAND: offset = p->h & off; break;
@@ -966,13 +1016,17 @@ static int mget(RMagic *ms, const ut8 *s, struct r_magic *m, size_t nbytes, unsi
 				case FILE_OPDIVIDE: offset = p->h / off; break;
 				case FILE_OPMODULO: offset = p->h % off; break;
 				}
-			} else offset = p->h;
-			if (m->in_op & FILE_OPINVERSE)
+			} else {
+				offset = p->h;
+			}
+			if (m->in_op & FILE_OPINVERSE) {
 				offset = ~offset;
+			}
 			break;
 		case FILE_BELONG:
-			if (nbytes < (offset + 4))
+			if (nbytes < (offset + 4)) {
 				return 0;
+			}
 			if (off) {
 				switch (m->in_op & FILE_OPS_MASK) {
 				case FILE_OPAND:
@@ -1000,13 +1054,17 @@ static int mget(RMagic *ms, const ut8 *s, struct r_magic *m, size_t nbytes, unsi
 					offset = (st32)((p->hl[0]<<24)| (p->hl[1]<<16)| (p->hl[2]<<8)| (p->hl[3])) % off;
 					break;
 				}
-			} else offset = (st32)((p->hl[0]<<24)| (p->hl[1]<<16)| (p->hl[2]<<8)| (p->hl[3]));
-			if (m->in_op & FILE_OPINVERSE)
+			} else {
+				offset = (st32) ((p->hl[0] << 24) | (p->hl[1] << 16) | (p->hl[2] << 8) | (p->hl[3]));
+			}
+			if (m->in_op & FILE_OPINVERSE) {
 				offset = ~offset;
+			}
 			break;
 		case FILE_LELONG:
-			if (nbytes < (offset + 4))
+			if (nbytes < (offset + 4)) {
 				return 0;
+			}
 			if (off) {
 				switch (m->in_op & FILE_OPS_MASK) {
 				case FILE_OPAND:
@@ -1034,13 +1092,17 @@ static int mget(RMagic *ms, const ut8 *s, struct r_magic *m, size_t nbytes, unsi
 					offset = (st32)((p->hl[3]<<24)| (p->hl[2]<<16)| (p->hl[1]<<8)| (p->hl[0])) % off;
 					break;
 				}
-			} else offset = (st32)((p->hl[3]<<24)| (p->hl[2]<<16)| (p->hl[1]<<8)| (p->hl[0]));
-			if (m->in_op & FILE_OPINVERSE)
+			} else {
+				offset = (st32) (((ut32)p->hl[3] << 24) | (p->hl[2] << 16) | (p->hl[1] << 8) | (p->hl[0]));
+			}
+			if (m->in_op & FILE_OPINVERSE) {
 				offset = ~offset;
+			}
 			break;
 		case FILE_MELONG:
-			if (nbytes < (offset + 4))
+			if (nbytes < (offset + 4)) {
 				return 0;
+			}
 			if (off) {
 				switch (m->in_op & FILE_OPS_MASK) {
 				case FILE_OPAND:
@@ -1068,13 +1130,17 @@ static int mget(RMagic *ms, const ut8 *s, struct r_magic *m, size_t nbytes, unsi
 					offset = (st32)((p->hl[1]<<24)| (p->hl[0]<<16)| (p->hl[3]<<8)| (p->hl[2])) % off;
 					break;
 				}
-			} else offset = (st32)((p->hl[1]<<24)| (p->hl[0]<<16)| (p->hl[3]<<8)| (p->hl[2]));
-			if (m->in_op & FILE_OPINVERSE)
+			} else {
+				offset = (st32) ((p->hl[1] << 24) | (p->hl[0] << 16) | (p->hl[3] << 8) | (p->hl[2]));
+			}
+			if (m->in_op & FILE_OPINVERSE) {
 				offset = ~offset;
+			}
 			break;
 		case FILE_LONG:
-			if (nbytes < (offset + 4))
+			if (nbytes < (offset + 4)) {
 				return 0;
+			}
 			if (off) {
 				switch (m->in_op & FILE_OPS_MASK) {
 				case FILE_OPAND: offset = p->l & off; break;
@@ -1086,17 +1152,21 @@ static int mget(RMagic *ms, const ut8 *s, struct r_magic *m, size_t nbytes, unsi
 				case FILE_OPDIVIDE: offset = p->l / off; break;
 				case FILE_OPMODULO: offset = p->l % off; break;
 				}
-			} else
+			} else {
 				offset = p->l;
-			if (m->in_op & FILE_OPINVERSE)
+			}
+			if (m->in_op & FILE_OPINVERSE) {
 				offset = ~offset;
+			}
 			break;
 		}
 
-		if (m->flag & INDIROFFADD)
-			offset += ms->c.li[cont_level-1].off;
-		if (mcopy (ms, p, m->type, 0, s, offset, nbytes, count) == -1)
+		if (m->flag & INDIROFFADD) {
+			offset += ms->c.li[cont_level - 1].off;
+		}
+		if (mcopy (ms, p, m->type, 0, s, offset, nbytes, count) == -1) {
 			return -1;
+		}
 		ms->offset = offset;
 
 		if ((ms->flags & R_MAGIC_DEBUG) != 0) {
@@ -1109,14 +1179,16 @@ static int mget(RMagic *ms, const ut8 *s, struct r_magic *m, size_t nbytes, unsi
 	/* Verify we have enough data to match magic type */
 	switch (m->type) {
 	case FILE_BYTE:
-		if (nbytes < (offset + 1)) /* should alway be true */
+		if (nbytes < (offset + 1)) { /* should alway be true */
 			return 0;
+		}
 		break;
 	case FILE_SHORT:
 	case FILE_BESHORT:
 	case FILE_LESHORT:
-		if (nbytes < (offset + 2))
+		if (nbytes < (offset + 2)) {
 			return 0;
+		}
 		break;
 	case FILE_LONG:
 	case FILE_BELONG:
@@ -1133,24 +1205,28 @@ static int mget(RMagic *ms, const ut8 *s, struct r_magic *m, size_t nbytes, unsi
 	case FILE_FLOAT:
 	case FILE_BEFLOAT:
 	case FILE_LEFLOAT:
-		if (nbytes < (offset + 4))
+		if (nbytes < (offset + 4)) {
 			return 0;
+		}
 		break;
 	case FILE_DOUBLE:
 	case FILE_BEDOUBLE:
 	case FILE_LEDOUBLE:
-		if (nbytes < (offset + 8))
+		if (nbytes < (offset + 8)) {
 			return 0;
+		}
 		break;
 	case FILE_STRING:
 	case FILE_PSTRING:
 	case FILE_SEARCH:
-		if (nbytes < (offset + m->vallen))
+		if (nbytes < (offset + m->vallen)) {
 			return 0;
+		}
 		break;
 	case FILE_REGEX:
-		if (nbytes < offset)
+		if (nbytes < offset) {
 			return 0;
+		}
 		break;
 	case FILE_DEFAULT:	/* nothing to check */
 	default:
@@ -1176,34 +1252,41 @@ static ut64 file_strncmp(const char *s1, const char *s2, size_t len, ut32 flags)
 	 */
 	v = 0;
 	if (0L == flags) { /* normal string: do it fast */
-		while (len-- > 0)
-			if ((v = *b++ - *a++) != '\0')
+		while (len-- > 0) {
+			if ((v = *b++ - *a++) != '\0') {
 				break;
+			}
+		}
 	} else { /* combine the others */
 		while (len-- > 0) {
 			if ((flags & STRING_IGNORE_LOWERCASE) &&
 			    islower(*a)) {
-				if ((v = tolower(*b++) - *a++) != '\0')
+				if ((v = tolower (*b++) - *a++) != '\0') {
 					break;
+				}
 			} else if ((flags & STRING_IGNORE_UPPERCASE) && isupper(*a)) {
-				if ((v = toupper(*b++) - *a++) != '\0')
+				if ((v = toupper (*b++) - *a++) != '\0') {
 					break;
+				}
 			} else if ((flags & STRING_COMPACT_BLANK) && isspace(*a)) {
 				a++;
 				if (isspace(*b++)) {
-					while (isspace(*b))
+					while (isspace (*b)) {
 						b++;
+					}
 				} else {
 					v = 1;
 					break;
 				}
 			} else if ((flags & STRING_COMPACT_OPTIONAL_BLANK) && isspace(*a)) {
 				a++;
-				while (isspace(*b))
+				while (isspace (*b)) {
 					b++;
+				}
 			} else {
-				if ((v = *b++ - *a++) != '\0')
+				if ((v = *b++ - *a++) != '\0') {
 					break;
+				}
 			}
 		}
 	}
@@ -1274,7 +1357,6 @@ static int magiccheck(RMagic *ms, struct r_magic *m) {
 		case '>': matched = fv > fl; break;
 		case '<': matched = fv < fl; break;
 		default:
-			matched = 0;
 			file_magerror(ms, "cannot happen with float: invalid relation `%c'", m->reln);
 			return -1;
 		}
@@ -1291,7 +1373,6 @@ static int magiccheck(RMagic *ms, struct r_magic *m) {
 		case '>': matched = dv > dl; break;
 		case '<': matched = dv < dl; break;
 		default:
-			matched = 0;
 			file_magerror (ms, "cannot happen with double: invalid relation `%c'", m->reln);
 			return -1;
 		}
@@ -1313,18 +1394,21 @@ static int magiccheck(RMagic *ms, struct r_magic *m) {
 	case FILE_SEARCH: { /* search ms->search.s for the string m->value.s */
 		size_t slen, idx;
 
-		if (!ms->search.s)
+		if (!ms->search.s) {
 			return 0;
+		}
 
 		slen = R_MIN (m->vallen, sizeof (m->value.s));
 		l = 0;
 		v = 0;
 
 		for (idx = 0; m->str_range == 0 || idx < m->str_range; idx++) {
-			if ((int)ms->search.offset < 0)
+			if ((int)ms->search.offset < 0) {
 				break;
-			if (slen + idx > ms->search.s_len)
+			}
+			if (slen + idx > ms->search.s_len) {
 				break;
+			}
 			v = file_strncmp (m->value.s, ms->search.s + idx, slen, m->str_flags);
 			if (v == 0) {	/* found match */
 				ms->search.offset += idx;
@@ -1338,8 +1422,9 @@ static int magiccheck(RMagic *ms, struct r_magic *m) {
 		RRegex rx;
 		char errmsg[512];
 
-		if (!ms->search.s)
+		if (!ms->search.s) {
 			return 0;
+		}
 
 		l = 0;
 		rc = r_regex_comp (&rx, m->value.s,
@@ -1383,8 +1468,9 @@ static int magiccheck(RMagic *ms, struct r_magic *m) {
 			}
 			r_regex_fini (&rx);
 		}
-		if (v == (ut64)-1)
+		if (v == (ut64)-1) {
 			return -1;
+		}
 		break;
 	}
 	default:
@@ -1395,54 +1481,62 @@ static int magiccheck(RMagic *ms, struct r_magic *m) {
 	v = file_signextend (ms, m, v);
 	switch (m->reln) {
 	case 'x':
-		if ((ms->flags & R_MAGIC_DEBUG) != 0)
-			(void) fprintf(stderr, "%"PFMT64u" == *any* = 1\n", (ut64)v);
+		if ((ms->flags & R_MAGIC_DEBUG) != 0) {
+			(void)fprintf (stderr, "%" PFMT64u " == *any* = 1\n", (ut64)v);
+		}
 		matched = 1;
 		break;
 	case '!':
 		matched = v != l;
-		if ((ms->flags & R_MAGIC_DEBUG) != 0)
-			fprintf(stderr, "%"PFMT64u" != %"PFMT64u" = %d\n", (ut64)v, (ut64)l, matched);
+		if ((ms->flags & R_MAGIC_DEBUG) != 0) {
+			fprintf (stderr, "%" PFMT64u " != %" PFMT64u " = %d\n", (ut64)v, (ut64)l, matched);
+		}
 		break;
 	case '=':
 		matched = v == l;
-		if ((ms->flags & R_MAGIC_DEBUG) != 0)
-			eprintf ("%"PFMT64u" == %"PFMT64u" = %d\n", (ut64)v, (ut64)l, matched);
+		if ((ms->flags & R_MAGIC_DEBUG) != 0) {
+			eprintf ("%" PFMT64u " == %" PFMT64u " = %d\n", (ut64)v, (ut64)l, matched);
+		}
 		break;
 	case '>':
 		if (m->flag & UNSIGNED) {
 			matched = v > l;
-			if ((ms->flags & R_MAGIC_DEBUG) != 0)
-				eprintf ("%"PFMT64u" > %"PFMT64u" = %d\n", (ut64)v, (ut64)l, matched);
+			if ((ms->flags & R_MAGIC_DEBUG) != 0) {
+				eprintf ("%" PFMT64u " > %" PFMT64u " = %d\n", (ut64)v, (ut64)l, matched);
+			}
 		} else {
 			matched = (ut64) v > (ut64) l;
-			if ((ms->flags & R_MAGIC_DEBUG) != 0)
-				eprintf ("%"PFMT64u" > %"PFMT64u" = %d\n", (st64)v, (st64)l, matched);
+			if ((ms->flags & R_MAGIC_DEBUG) != 0) {
+				eprintf ("%" PFMT64u " > %" PFMT64u " = %d\n", (st64)v, (st64)l, matched);
+			}
 		}
 		break;
 	case '<':
 		if (m->flag & UNSIGNED) {
 			matched = v < l;
-			if ((ms->flags & R_MAGIC_DEBUG) != 0)
-				eprintf ("%"PFMT64u" < %"PFMT64u" = %d\n", (ut64)v, (ut64)l, matched);
+			if ((ms->flags & R_MAGIC_DEBUG) != 0) {
+				eprintf ("%" PFMT64u " < %" PFMT64u " = %d\n", (ut64)v, (ut64)l, matched);
+			}
 		} else {
 			matched = (ut64) v < (ut64) l;
-			if ((ms->flags & R_MAGIC_DEBUG) != 0)
-				eprintf ("%"PFMT64d" < %"PFMT64d" = %d\n", (st64)v, (st64)l, matched);
+			if ((ms->flags & R_MAGIC_DEBUG) != 0) {
+				eprintf ("%" PFMT64d " < %" PFMT64d " = %d\n", (st64)v, (st64)l, matched);
+			}
 		}
 		break;
 	case '&':
 		matched = (v & l) == l;
-		if ((ms->flags & R_MAGIC_DEBUG) != 0)
-			eprintf("((%"PFMT64x" & %"PFMT64x") == %"PFMT64x") = %d\n", (ut64)v, (ut64)l, (ut64)l, matched);
+		if ((ms->flags & R_MAGIC_DEBUG) != 0) {
+			eprintf ("((%" PFMT64x " & %" PFMT64x ") == %" PFMT64x ") = %d\n", (ut64)v, (ut64)l, (ut64)l, matched);
+		}
 		break;
 	case '^':
 		matched = (v & l) != l;
-		if ((ms->flags & R_MAGIC_DEBUG) != 0)
-			eprintf ("((%"PFMT64x" & %"PFMT64x") != %"PFMT64x") = %d\n", (ut64)v, (ut64)l, (ut64)l, matched);
+		if ((ms->flags & R_MAGIC_DEBUG) != 0) {
+			eprintf ("((%" PFMT64x " & %" PFMT64x ") != %" PFMT64x ") = %d\n", (ut64)v, (ut64)l, (ut64)l, matched);
+		}
 		break;
 	default:
-		matched = 0;
 		file_magerror (ms, "cannot happen: invalid relation `%c'", m->reln);
 		return -1;
 	}
