@@ -1,9 +1,8 @@
-/* radare - LGPL - Copyright 2015-2016 - pancake */
+/* radare - LGPL - Copyright 2015-2018 - pancake */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-
 #include <r_lib.h>
 #include <r_util.h>
 #include <r_flag.h>
@@ -11,134 +10,172 @@
 #include <r_parse.h>
 
 static int replace(int argc, const char *argv[], char *newstr) {
-	int i,j,k;
+#define MAXPSEUDOOPS 10
+	int i, j, k, d;
+	char ch;
 	struct {
 		int narg;
 		char *op;
 		char *str;
+		int args[MAXPSEUDOOPS];
 	} ops[] = {
-		{ 0, "abs",  "1 = abs(1)"},
-		{ 0, "adc",  "1 = 2 + 3"},
-		{ 3, "add",  "1 = 2 + 3"},
-		{ 2, "add",  "1 += 2"},
-		{ 2, "adds",  "1 += 2"},
-		{ 3, "adds",  "1 = 2 + 3"},
-		{ 3, "addw",  "1 = 2 + 3"},
-		{ 3, "add.w",  "1 = 2 + 3"},
-		{ 0, "adf",  "1 = 2 + 3"},
-		{ 0, "adrp",  "1 = 2"},
-		{ 0, "and",  "1 = 2 & 3"},
-		{ 0, "ands",  "1 &= 2"},
-		{ 0, "asls",  "1 = 2 << 3"},
-		{ 0, "asl",  "1 = 2 << 3"},
-		{ 0, "asrs",  "1 = 2 >> 3"},
-		{ 0, "asr",  "1 = 2 >> 3"},
-		{ 0, "b",  "jmp 1"},
-		{ 0, "cbz",  "if !1 jmp 2"},
-		{ 0, "cbnz",  "if 1 jmp 2"},
-		{ 0, "b.w",  "jmp 1"},
-		{ 0, "b.gt",  "jmp ifgt 1"},
-		{ 0, "b.le",  "jmp ifle 1"},
-		{ 0, "beq lr",  "ifeq ret"},
-		{ 0, "beq",  "je 1"},
-		{ 0, "call",  "1()"},
-		{ 0, "bl",  "1()"},
-		{ 0, "blx",  "1()"},
-		{ 0, "bx lr",  "ret"},
-		{ 0, "bxeq",  "je 1"},
-		{ 0, "cmf",  "if (1 == 2)"},
-		{ 0, "cmp",  "if (1 == 2)"},
-		{ 0, "tst",  "if (1 == 2)"},
-		{ 0, "dvf",  "1 = 2 / 3"},
-		{ 0, "eor",  "1 = 2 ^ 3"},
-		{ 0, "fdv",  "1 = 2 / 3"},
-		{ 0, "fml",  "1 = 2 * 3"},
-		{ 2, "ldr",  "1 = 2"},
-		{ 2, "ldrb",  "1 = (byte) 2"},
-		{ 2, "ldrsb",  "1 = (byte) 2"},
-		{ 2, "ldr.w",  "1 = 2"},
-		{ 2, "ldrsw",  "1 = 2"},
-		{ 3, "ldr",  "1 = 2 + 3"},
-		{ 3, "ldrb",  "1 = (byte) 2 + 3"},
-		{ 3, "ldrsb",  "1 = (byte) 2 + 3"},
-		{ 3, "ldr.w",  "1 = 2 + 3"},
-		{ 3, "ldrsw",  "1 = 2 + 3"},
-		{ 0, "lsl",  "1 = 2 << 3"},
-		{ 0, "lsr",  "1 = 2 >> 3"},
-		{ 0, "mov",  "1 = 2"},
-		{ 0, "mvn",  "1 = ~2"},
-		{ 0, "movz",  "1 = 2"},
-		{ 0, "movk",  "1 = 2"},
-		{ 0, "movn",  "1 = 2"},
-		{ 0, "vmov.i32",  "1 = 2"},
-		{ 0, "muf",  "1 = 2 * 3"},
-		{ 0, "mul",  "1 = 2 * 3"},
-		{ 0, "muls",  "1 = 2 * 3"},
-		{ 0, "orr",  "1 = 2 | 3"},
-		{ 0, "rmf",  "1 = 2 % 3"},
-		{ 0, "bge",  "(>=) goto 1"},
-		{ 0, "sbc",  "1 = 2 - 3"},
-		{ 0, "sqt",  "1 = sqrt(2)"},
-		{ 0, "lsrs",  "1 = 2 >> 3"},
-		{ 0, "lsls",  "1 = 2 << 3"},
-		{ 0, "lsr",  "1 = 2 >> 3"},
-		{ 0, "lsl",  "1 = 2 << 3"},
-		{ 2, "str",  "2 = 1"},
-		{ 2, "strb",  "2 = (byte) 1"},
-		{ 2, "strh",  "2 = (half) 1"},
-		{ 2, "strh.w",  "2 = (half) 1"},
-		{ 3, "str",  "2 + 3 = 1"},
-		{ 3, "strb",  "2 + 3 = (byte) 1"},
-		{ 3, "strh",  "2 + 3 = (half) 1"},
-		{ 3, "strh.w",  "2 + 3 = (half) 1"},
-		{ 3, "sub",  "1 = 2 - 3"},
-		{ 3, "subs",  "1 = 2 - 3"},
-		{ 2, "sub",  "1 -= 2"}, // THUMB
-		{ 2, "subs",  "1 -= 2"}, // THUMB
-		{ 0, "swp",  "swap(1, 2)"},
+		{ 0, "abs", "# = abs(#)", { 1, 1 } },
+		{ 0, "adc", "# = # + #", { 1, 2, 3 } },
+		{ 3, "add", "# = # + #", { 1, 2, 3 } },
+		{ 2, "add", "# += #", { 1, 2 } },
+		{ 2, "adds", "# += #", { 1, 2 } },
+		{ 3, "adds", "# = # + #", { 1, 2, 3 } },
+		{ 3, "addw", "# = # + #", { 1, 2, 3 } },
+		{ 3, "add.w", "# = # + #", { 1, 2, 3 } },
+		{ 0, "adf", "# = # + #", { 1, 2, 3 } },
+		{ 0, "adrp", "# = #", { 1, 2 } },
+		{ 0, "adr", "# = #", { 1, 2 } },
+		{ 0, "and", "# = # & #", { 1, 2, 3 } },
+		{ 0, "ands", "# &= #", { 1, 2 } },
+		{ 0, "asls", "# = # << #", { 1, 2, 3 } },
+		{ 0, "asl", "# = # << #", { 1, 2, 3 } },
+		{ 0, "asrs", "# = # >> #", { 1, 2, 3 } },
+		{ 0, "asr", "# = # >> #", { 1, 2, 3 } },
+		{ 0, "b", "jmp #", { 1 } },
+		{ 0, "cbz", "if !# jmp #", { 1, 2 } },
+		{ 0, "cbnz", "if # jmp #", { 1, 2 } },
+		{ 0, "b.w", "jmp #", { 1 } },
+		{ 0, "b.gt", "jmp ifgt #", { 1 } },
+		{ 0, "b.le", "jmp ifle #", { 1 } },
+		{ 0, "beq lr", "ifeq ret", { 0 } },
+		{ 0, "beq", "je #", { 1 } },
+		{ 0, "call", "# ()", { 1 } },
+		{ 0, "bl", "# ()", { 1 } },
+		{ 0, "blx", "# ()", { 1 } },
+		{ 0, "bx lr", "ret", { 0 } },
+		{ 0, "bxeq", "je #", { 1 } },
+		{ 0, "cmf", "if (# == #)", { 1, 2 } },
+		{ 0, "cmn", "if (# != #)", { 1, 2 } },
+		{ 0, "cmp", "if (# == #)", { 1, 2 } },
+		{ 0, "fcmp", "if (# == #)", { 1, 2 } },
+		{ 0, "tst", "if (# == #)", { 1, 2 } },
+		{ 0, "dvf", "# = # / #", { 1, 2, 3 } },
+		{ 0, "eor", "# = # ^ #", { 1, 2, 3 } },
+		{ 1, "bkpt", "breakpoint #", { 1 } },
+		{ 1, "udf", "undefined #", { 1 } },
+		{ 2, "sxtb", "# = (char) #", { 1, 2 } },
+		{ 2, "sxth", "# = (short) #", { 1, 2 } },
+		{ 0, "fdv", "# = # / #", { 1, 2, 3 } },
+		{ 0, "fml", "# = # * #", { 1, 2, 3 } },
+		{ 2, "ldr", "# = #", { 1, 2 } },
+		{ 2, "ldrh", "# = (word) #", { 1, 2 } },
+		{ 3, "ldrh", "# = (word) # + #", { 1, 2, 3 } },
+		{ 2, "ldrb", "# = (byte) #", { 1, 2 } },
+		{ 3, "ldrb", "# = (byte) # + #", { 1, 2, 3 } },
+		{ 2, "ldrsb", "# = (byte) #", { 1, 2 } },
+		{ 2, "ldr.w", "# = #", { 1, 2 } },
+		{ 2, "ldrsw", "# = #", { 1, 2 } },
+		{ 3, "ldr", "# = # + #", { 1, 2, 3 } },
+		{ 3, "ldrb", "# = (byte) # + #", { 1, 2, 3 } },
+		{ 3, "ldrsb", "# = (byte) # + #", { 1, 2, 3 } },
+		{ 3, "ldr.w", "# = # + #", { 1, 2, 3 } },
+		{ 3, "ldrsw", "# = # + #", { 1, 2, 3 } },
+		{ 0, "lsl", "# = # << #", { 1, 2, 3 } },
+		{ 0, "lsr", "# = # >> #", { 1, 2, 3 } },
+		{ 0, "mov", "# = #", { 1, 2 } },
+		{ 0, "fmov", "# = #", { 1, 2 } },
+		{ 0, "mvn", "# = ~#", { 1, 2 } },
+		{ 0, "movz", "# = #", { 1, 2 } },
+		{ 0, "movk", "# = #", { 1, 2 } },
+		{ 0, "movn", "# = #", { 1, 2 } },
+		{ 0, "neg", "# = !#", { 1, 2 } },
+		{ 0, "sxtw", "# = #", { 1, 2 } },
+		{ 0, "stur", "# = #", { 2, 1 } },
+		{ 0, "stp", "# = (#, 2)", { 3, 1 } },
+		{ 0, "ldp", "(#, 2) = 3", { 1 } },
+		{ 0, "vmov.i32", "# = #", { 1, 2 } },
+		{ 0, "muf", "# = # * #", { 1, 2, 3 } },
+		{ 0, "mul", "# = # * #", { 1, 2, 3 } },
+		{ 0, "fmul", "# = # * #", { 1, 2, 3 } },
+		{ 0, "muls", "# = # * #", { 1, 2, 3 } },
+		{ 0, "div", "# = # / #", { 1, 2, 3 } },
+		{ 0, "fdiv", "# = # / #", { 1, 2, 3 } },
+		{ 0, "udiv", "# = (unsigned) # / #", { 1, 2, 3 } },
+		{ 0, "orr", "# = # | #", { 1, 2, 3 } },
+		{ 0, "rmf", "# = # % #", { 1, 2, 3 } },
+		{ 0, "bge", "(>=) goto #", { 1 } },
+		{ 0, "sbc", "# = # - #", { 1, 2, 3 } },
+		{ 0, "sqt", "# = sqrt(#)", { 1, 2 } },
+		{ 0, "lsrs", "# = # >> #", { 1, 2, 3 } },
+		{ 0, "lsls", "# = # << #", { 1, 2, 3 } },
+		{ 0, "lsr", "# = # >> #", { 1, 2, 3 } },
+		{ 0, "lsl", "# = # << #", { 1, 2, 3 } },
+		{ 2, "str", "# = #", { 2, 1 } },
+		{ 2, "strb", "# = (byte) #", { 2, 1 } },
+		{ 2, "strh", "# = (half) #", { 2, 1 } },
+		{ 2, "strh.w", "# = (half) #", { 2, 1 } },
+		{ 3, "str", "# + # = #", { 2, 3, 1 } },
+		{ 3, "strb", "# + # = (byte) #", { 2, 3, 1 } },
+		{ 3, "strh", "# + # = (half) #", { 2, 3, 1 } },
+		{ 3, "strh.w", "# + # = (half) #", { 2, 3, 1 } },
+		{ 3, "sub", "# = # - #", { 1, 2, 3 } },
+		{ 3, "subs", "# = # - #", { 1, 2, 3 } },
+		{ 3, "fsub", "# = # - #", { 1, 2, 3 } },
+		{ 2, "sub", "# -= #", { 1, 2 } }, // THUMB
+		{ 2, "subs", "# -= #", { 1, 2 } }, // THUMB
+		{ 0, "swp", "swap(#, 2)", { 1 } },
 		/* arm thumb */
-		{ 0, "movs",  "1 = 2"},
-		{ 0, "movw",  "1 = 2"},
-		{ 0, "movt",  "1 |= 2 << 16"},
-		{ 0, "vmov",  "1 = (float) 2 . 3"},
-		{ 0, "vdiv.f64", "1 = (float) 2 / 3" },
-		{ 0, "addw",  "1 = 2 + 3"},
-		{ 0, "sub.w",  "1 = 2 - 3"},
-		{ 0, "tst.w", "if (1 == 2)"},
-		{ 0, "lsr.w", "1 = 2 >> 3"},
-		{ 0, "lsl.w", "1 = 2 << 3"},
-		{ 0, "pop.w",  "pop 1"},
-		{ 0, "vpop",  "pop 1"},
-		{ 0, "vpush",  "push 1"},
-		{ 0, "push.w",  "push 1"},
+		{ 0, "movs", "# = #", { 1, 2 } },
+		{ 0, "movw", "# = #", { 1, 2 } },
+		{ 0, "movt", "# |= # << 16", { 1, 2 } },
+		{ 0, "vmov", "# = (float) # . #", { 1, 2, 3 } },
+		{ 0, "vdiv.f64", "# = (float) # / #", { 1, 2, 3 } },
+		{ 0, "addw", "# = # + #", { 1, 2, 3 } },
+		{ 0, "sub.w", "# = # - #", { 1, 2, 3 } },
+		{ 0, "tst.w", "if (# == #)", { 1, 2 } },
+		{ 0, "lsr.w", "# = # >> #", { 1, 2, 3 } },
+		{ 0, "lsl.w", "# = # << #", { 1, 2, 3 } },
+		{ 0, "pop.w", "pop #", { 1 } },
+		{ 0, "vpop", "pop #", { 1 } },
+		{ 0, "vpush", "push #", { 1 } },
+		{ 0, "push.w", "push #", { 1 } },
 		{ 0, NULL }
 	};
 	if (!newstr) {
 		return false;
 	}
 
-	for (i = 0; ops[i].op != NULL; i++) {
+	for (i = 0; ops[i].op; i++) {
 		if (ops[i].narg) {
-			if (argc-1 != ops[i].narg) {
+			if (argc - 1 != ops[i].narg) {
 				continue;
 			}
 		}
 		if (!strcmp (ops[i].op, argv[0])) {
-			for (j = k = 0; ops[i].str[j] != '\0'; j++, k++) {
-				if (IS_DIGIT(ops[i].str[j])) {
-					int idx = ops[i].str[j]-'0';
-					if (idx < argc) {
+			if (newstr) {
+				d = 0;
+				j = 0;
+				ch = ops[i].str[j];
+				for (j = 0, k = 0; ch != '\0'; j++, k++) {
+					ch = ops[i].str[j];
+					if (ch == '#') {
+						if (d >= MAXPSEUDOOPS) {
+							// XXX Shouldn't ever happen...
+							continue;
+						}
+						int idx = ops[i].args[d];
+						d++;
+						if (idx <= 0) {
+							// XXX Shouldn't ever happen...
+							continue;
+						}
 						const char *w = argv[idx];
 						if (w) {
 							strcpy (newstr + k, w);
 							k += strlen (w) - 1;
 						}
+					} else {
+						newstr[k] = ch;
 					}
-				} else {
-					newstr[k] = ops[i].str[j];
 				}
+				newstr[k] = '\0';
 			}
-			newstr[k] = '\0';
+
 			r_str_replace_char (newstr, '{', '(');
 			r_str_replace_char (newstr, '}', ')');
 			return true;
@@ -177,19 +214,21 @@ static int parse(RParse *p, const char *data, char *str) {
 		}
 		if (ptr) {
 			*ptr = '\0';
-			for (++ptr; *ptr==' '; ptr++);
+			for (++ptr; *ptr == ' '; ptr++) {
+				;
+			}
 			strncpy (w0, buf, sizeof (w0) - 1);
 			strncpy (w1, ptr, sizeof (w1) - 1);
 
 			optr = ptr;
-			if (*ptr == '(') { 
-				ptr = strchr (ptr+1, ')'); 
+			if (*ptr == '(') {
+				ptr = strchr (ptr+1, ')');
 			}
 			if (ptr && *ptr == '[') {
-				ptr = strchr (ptr+1, ']'); 
+				ptr = strchr (ptr+1, ']');
 			}
-			if (ptr && *ptr == '{') { 
-				ptr = strchr (ptr+1, '}'); 
+			if (ptr && *ptr == '{') {
+				ptr = strchr (ptr+1, '}');
 			}
 			if (!ptr) {
 				eprintf ("Unbalanced bracket\n");
@@ -199,14 +238,18 @@ static int parse(RParse *p, const char *data, char *str) {
 			ptr = strchr (ptr, ',');
 			if (ptr) {
 				*ptr = '\0';
-				for (++ptr; *ptr==' '; ptr++);
+				for (++ptr; *ptr == ' '; ptr++) {
+					;
+				}
 				strncpy (w1, optr, sizeof (w1) - 1);
 				strncpy (w2, ptr, sizeof (w2) - 1);
 				optr = ptr;
 				ptr = strchr (ptr, ',');
 				if (ptr) {
 					*ptr = '\0';
-					for (++ptr; *ptr==' '; ptr++);
+					for (++ptr; *ptr == ' '; ptr++) {
+						;
+					}
 					strncpy (w2, optr, sizeof (w2) - 1);
 					strncpy (w3, ptr, sizeof (w3) - 1);
 				}
@@ -235,9 +278,11 @@ static int parse(RParse *p, const char *data, char *str) {
 }
 
 static bool varsub(RParse *p, RAnalFunction *f, ut64 addr, int oplen, char *data, char *str, int len) {
-	RList *spargs, *bpargs;
+	RList *spargs = NULL;
+	RList *bpargs = NULL;
 	RAnalVar *var;
 	RListIter *iter;
+	RAnal *anal = p->analb.anal;
 	char *oldstr, *newstr;
 	char *tstr = strdup (data);
 	if (!tstr) {
@@ -275,11 +320,11 @@ static bool varsub(RParse *p, RAnalFunction *f, ut64 addr, int oplen, char *data
 		}
 	}
 
-	bpargs = p->varlist (p->anal, f, 'b');
-	spargs = p->varlist (p->anal, f, 's');
+	bpargs = p->varlist (anal, f, 'b');
+	spargs = p->varlist (anal, f, 's');
 	bool ucase = IS_UPPER (*tstr);
 	r_list_foreach (bpargs, iter, var) {
-		char *reg = p->anal->reg->name[R_REG_NAME_BP];
+		char *reg = anal->reg->name[R_REG_NAME_BP];
 		char *tmplt = NULL;
 		if (var->delta > -10 && var->delta < 10) {
 			if (p->pseudo) {
@@ -331,6 +376,9 @@ static bool varsub(RParse *p, RAnalFunction *f, ut64 addr, int oplen, char *data
 		free (oldstr);
 	}
 	r_list_foreach (spargs, iter, var) {
+		if (p->get_ptr_at) {
+			var->delta = p->get_ptr_at (p->user, var, addr);
+		}
 		if (var->delta > -10 && var->delta < 10) {
 			oldstr = r_str_newf ("[sp, %d]", var->delta);
 		} else if (var->delta > 0) {
@@ -350,20 +398,20 @@ static bool varsub(RParse *p, RAnalFunction *f, ut64 addr, int oplen, char *data
 		free (oldstr);
 		if (var->delta > -10 && var->delta < 10) {
 			oldstr = r_str_newf ("[%s, %d]",
-				p->anal->reg->name[R_REG_NAME_SP],
+				anal->reg->name[R_REG_NAME_SP],
 				var->delta);
 		} else if (var->delta > 0) {
 			oldstr = r_str_newf ("[%s, 0x%x]",
-				p->anal->reg->name[R_REG_NAME_SP],
+				anal->reg->name[R_REG_NAME_SP],
 				var->delta);
 		} else {
 			oldstr = r_str_newf ("[%s, -0x%x]",
-				p->anal->reg->name[R_REG_NAME_SP],
+				anal->reg->name[R_REG_NAME_SP],
 				-var->delta);
 		}
 		if (strstr (tstr, oldstr)) {
 			newstr = r_str_newf ("[%s %c %s]",
-				p->anal->reg->name[R_REG_NAME_BP],
+				anal->reg->name[R_REG_NAME_BP],
 				var->delta > 0 ? '+' : '-',
 				var->name);
 			tstr = r_str_replace (tstr, oldstr, newstr, 1);
@@ -373,9 +421,10 @@ static bool varsub(RParse *p, RAnalFunction *f, ut64 addr, int oplen, char *data
 		}
 		free (oldstr);
 	}
+	r_list_free (bpargs);
+	r_list_free (spargs);
 	if (len > strlen (tstr)) {
-		strncpy (str, tstr, strlen (tstr));
-		str[strlen (tstr)] = 0;
+		strcpy  (str, tstr);
 	} else {
 		// TOO BIG STRING CANNOT REPLACE HERE
 		free (tstr);
@@ -392,8 +441,8 @@ RParsePlugin r_parse_plugin_arm_pseudo = {
 	.varsub = &varsub,
 };
 
-#ifndef CORELIB
-RLibStruct radare_plugin = {
+#ifndef R2_PLUGIN_INCORE
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_PARSE,
 	.data = &r_parse_plugin_arm_pseudo,
 	.version = R2_VERSION
