@@ -121,14 +121,14 @@ static int replace(int argc, const char *argv[], char *newstr) {
 		{ "sput-char", "2[3] = (char) 1"},
 		{ "iput-int", "2[3] = (int) 1"},
 		{ "iget", "1 = 2[3]"},
-		{ "sget-byte", "1 = (byte) 2[3]"},
-		{ "iget-byte", "1 = (byte) 2[3]"},
-		{ "iget-char", "1 = (char) 2[3]"},
-		{ "iget-short", "1 = (short) 2[3]"},
-		{ "iget-wide", "1 = (wide) 2[3]"},
-		{ "iget-object", "1 = (object) 2[3]"},
-		{ "iget-boolean", "1 = (bool) 2[3]"},
-		{ "+iget-wide-volatile", "1 = (wide-volatile) 2[3]"},
+		{ "sget-byte", "1 = (byte) 2 [3]"},
+		{ "iget-byte", "1 = (byte) 2 [3]"},
+		{ "iget-char", "1 = (char) 2 [3]"},
+		{ "iget-short", "1 = (short) 2 [3]"},
+		{ "iget-wide", "1 = (wide) 2 [3]"},
+		{ "iget-object", "1 = (2) 3"},
+		{ "iget-boolean", "1 = (bool) 2 [3]"},
+		{ "+iget-wide-volatile", "1 = (wide-volatile) 2 [3]"},
 		{ "if-eq", "if (1 == 2) goto 3"},
 		{ "if-lt", "if (1 < 2) goto 3"},
 		{ "if-ne", "if (1 != 2) goto 3"},
@@ -149,6 +149,7 @@ static int replace(int argc, const char *argv[], char *newstr) {
 		{ "invoke-static", "call 2 1"},
 		{ "invoke-super", "call super 2 1"},
 		{ "invoke-super/range", "call super 2 1"},
+		{ "invoke-polymorphic", "call polymorphic 2 1" },
 		{ "invoke-virtual/range", "call 2 1"},
 		{ "invoke-virtual", "call 2 1"},
 		{ "+invoke-virtual-quick", "call 2 1"},
@@ -245,7 +246,7 @@ static int parse(RParse *p, const char *data, char *str) {
 	if (!(buf = malloc (len + 1))) {
 		return false;
 	}
-	memcpy (buf, data, len+1);
+	memcpy (buf, data, len + 1);
 
 	r_str_trim (buf);
 
@@ -363,7 +364,7 @@ RParsePlugin r_parse_plugin_dalvik_pseudo = {
 	.parse = parse,
 };
 
-#ifndef CORELIB
+#ifndef R2_PLUGIN_INCORE
 R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_PARSE,
 	.data = &r_parse_plugin_dalvik_pseudo,
