@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2014-2018 - pancake */
+/* radare - LGPL - Copyright 2014-2019 - pancake */
 
 #include <r_reg.h>
 
@@ -13,19 +13,6 @@
 #define O f->o
 #define P f->p
 
-// old new
-// XXX: word size matters
-// TODO: needs more rethinking
-/*
-R_API int r_reg_cond_set(RReg *reg, ut64 o, ut64 n, int nsz, RRegFlags *f) {
-	Z = n? 0: 1;
-	S = 0;
-	C = 0; // TODO
-	O = 0;
-	P = n&1;
-}
-*/
-
 R_API RRegItem *r_reg_cond_get(RReg *reg, const char *name) {
 	int i = R_REG_TYPE_GPR;
 	RListIter *iter;
@@ -44,6 +31,16 @@ R_API int r_reg_cond_get_value(RReg *r, const char *name) {
 	return (int)r_reg_get_value (r, r_reg_cond_get (r, name)) ? 1 : 0;
 }
 
+R_API bool r_reg_cond_set(RReg *r, const char *name, bool val) {
+	RRegItem *item = r_reg_cond_get (r, name);
+	if (item) {
+		r_reg_set_value (r, item, val);
+		return true;
+	}
+	// eprintf ("Cannot find '%s'\n", name);
+	return false;
+}
+
 R_API const char *r_reg_cond_to_string(int n) {
 	const char *cs[] = {
 		"eq", "ne", "cf", "neg", "of", "hi", "he",
@@ -95,7 +92,8 @@ R_API int r_reg_cond_from_string(const char *str) {
 	if (!strcmp (str, "le")) {
 		return R_REG_COND_LE;
 	}
-	eprintf ("| Usage: drc [condition]\n"
+	// TODO: move this into core
+	eprintf ("| Usage: drc[=] [condition](=1,0)\n"
 		 "| eq    equal\n"
 		 "| ne    not equal\n"
 		 "| cf    carry flag set\n"
@@ -123,8 +121,8 @@ R_API int r_reg_cond_bits(RReg *r, int type, RRegFlags *f) {
 	case R_REG_COND_OF:
 		return O;
 	// unsigned
-	case R_REG_COND_HI: return (!Z && C); // HIGUER
-	case R_REG_COND_HE: return Z || (!Z && C); // HIGUER OR EQUAL
+	case R_REG_COND_HI: return (!Z && C); // HIGHER
+	case R_REG_COND_HE: return Z || (!Z && C); // HIGHER OR EQUAL
 	case R_REG_COND_LO: return (Z || !C); // LOWER
 	case R_REG_COND_LOE:
 		return (Z || !C); // LOWER OR EQUAL
@@ -137,6 +135,94 @@ R_API int r_reg_cond_bits(RReg *r, int type, RRegFlags *f) {
 	return false;
 }
 
+R_API bool r_reg_cond_bits_set(RReg *r, int type, RRegFlags *f, bool v) {
+	switch (type) {
+	case R_REG_COND_EQ: Z = v; break;
+	case R_REG_COND_NE: Z = !v; break;
+	case R_REG_COND_CF: C = v; break;
+	case R_REG_COND_NEG: S = v; break;
+	case R_REG_COND_OF: O = v; break;
+	case R_REG_COND_HI:
+		if (v) {
+			Z = 0;
+			C = 1;
+		} else {
+			Z = 1;
+			C = 0;
+		}
+		break;
+	case R_REG_COND_HE:
+		if (v) {
+			Z = 1;
+		} else {
+			Z = 0;
+			C = 1;
+		}
+		break;
+	case R_REG_COND_LO:
+		if (v) {
+			Z = 1;
+			C = 0;
+		} else {
+			Z = 0;
+			C = 1;
+		}
+		break;
+	case R_REG_COND_LOE:
+		if (v) {
+			Z = 1;
+			C = 0;
+		} else {
+			Z = 0;
+			C = 1;
+		}
+		break;
+	// signed
+	case R_REG_COND_GE:
+		if (v) {
+			S = O = 1;
+		} else {
+			S = 1;
+			O = 0;
+		}
+		break;
+	case R_REG_COND_GT:
+		if (v) {
+			S = 1;
+			Z = 0;
+			O = 1;
+		} else {
+			S = 0;
+			Z = 1;
+			O = 0;
+		}
+		break;
+	case R_REG_COND_LT:
+		if (v) {
+			S = 1;
+			O = 0;
+		} else {
+			S = 1;
+			O = 1;
+		}
+		break;
+	case R_REG_COND_LE:
+		if (v) {
+			S = 0;
+			Z = 1;
+			O = 0;
+		} else {
+			S = 1;
+			Z = 0;
+			O = 1;
+		}
+		break;
+	default:
+		return false;
+	}
+	return true;
+}
+
 R_API int r_reg_cond(RReg *r, int type) {
 	RRegFlags f = { 0 };
 	r_reg_cond_retrieve (r, &f);
@@ -157,3 +243,12 @@ R_API RRegFlags *r_reg_cond_retrieve(RReg *r, RRegFlags *f) {
 	f->p = r_reg_cond_get_value (r, "parity"); // parity // intel only
 	return f;
 }
+
+R_API void r_reg_cond_apply(RReg *r, RRegFlags *f) {
+	r_return_if_fail (r && f);
+	r_reg_cond_set (r, "sign", f->s);
+	r_reg_cond_set (r, "zero", f->z);
+	r_reg_cond_set (r, "carry", f->c);
+	r_reg_cond_set (r, "overflow", f->o);
+	r_reg_cond_set (r, "parity", f->p);
+}
