@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2018 - pancake */
+/* radare - LGPL - Copyright 2009-2019 - pancake */
 
 #include <r_reg.h>
 #include <r_util.h>
@@ -13,14 +13,14 @@ static ut27 r_read_me27(const ut8 *buf, int boff) {
 }
 
 R_API ut64 r_reg_get_value_big(RReg *reg, RRegItem *item, utX *val) {
-	RRegSet *regset;
-	int off;
+	r_return_val_if_fail (reg && item, 0);
+
 	ut64 ret = 0LL;
-	if (!reg || !item) {
+	int off = BITS2BYTES (item->offset);
+	RRegSet *regset = &reg->regset[item->arena];
+	if (!regset->arena) {
 		return 0LL;
 	}
-	off = BITS2BYTES (item->offset);
-	regset = &reg->regset[item->arena];
 	switch (item->size) {
 	case 80: // word + qword
 		if (regset->arena->bytes && (off + 10 <= regset->arena->size)) {
@@ -49,8 +49,17 @@ R_API ut64 r_reg_get_value_big(RReg *reg, RRegItem *item, utX *val) {
 		}
 		ret = val->v128.Low;
 		break;
-	//case 256:// qword + qword + qword + qword
-	//	break;
+	case 256:// qword + qword + qword + qword
+		if (regset->arena->bytes && (off + 32 <= regset->arena->size)) {
+			val->v256.Low.Low = *((ut64 *)(regset->arena->bytes + off));
+			val->v256.Low.High = *((ut64 *)(regset->arena->bytes + off + 8));
+			val->v256.High.Low = *((ut64 *)(regset->arena->bytes + off + 16));
+			val->v256.High.High = *((ut64 *)(regset->arena->bytes + off + 24));
+		} else {
+			eprintf ("r_reg_get_value: null or oob arena for current regset\n");
+		}
+		ret = val->v256.Low.Low;
+		break;
 	default:
 		eprintf ("r_reg_get_value_big: Bit size %d not supported\n", item->size);
 		break;
@@ -64,10 +73,13 @@ R_API ut64 r_reg_get_value(RReg *reg, RRegItem *item) {
 	}
 	int off = BITS2BYTES (item->offset);
 	RRegSet *regset = &reg->regset[item->arena];
+	if (!regset->arena) {
+		return 0LL;
+	}
 	switch (item->size) {
 	case 1: {
 		int offset = item->offset / 8;
-		if (offset + item->size >= regset->arena->size) {
+		if (offset >= regset->arena->size) {
 			break;
 		}
 		return (regset->arena->bytes[offset] &
@@ -124,12 +136,18 @@ R_API ut64 r_reg_get_value_by_role(RReg *reg, RRegisterId role) {
 }
 
 R_API bool r_reg_set_value(RReg *reg, RRegItem *item, ut64 value) {
-	int fits_in_arena;
 	ut8 bytes[12];
 	ut8 *src = bytes;
+	r_return_val_if_fail (reg && item, false);
 
-	if (!item) {
-		eprintf ("r_reg_set_value: item is NULL\n");
+	if (r_reg_is_readonly (reg, item)) {
+		return true;
+	}
+	if (item->offset < 0) {
+		return true;
+	}
+	RRegArena *arena = reg->regset[item->arena].arena;
+	if (!arena) {
 		return false;
 	}
 	switch (item->size) {
@@ -163,13 +181,12 @@ R_API bool r_reg_set_value(RReg *reg, RRegItem *item, ut64 value) {
 		break;
 	case 1:
 		if (value) {
-			ut8 *buf = reg->regset[item->arena].arena->bytes + (item->offset / 8);
+			ut8 *buf = arena->bytes + (item->offset / 8);
 			int bit = (item->offset % 8);
 			ut8 mask = (1 << bit);
 			buf[0] = (buf[0] & (0xff ^ mask)) | mask;
 		} else {
 			int idx = item->offset / 8;
-			RRegArena *arena = reg->regset[item->arena].arena;
 			if (idx + item->size > arena->size) {
 				eprintf ("RRegSetOverflow %d vs %d\n", idx + item->size, arena->size);
 				return false;
@@ -184,11 +201,11 @@ R_API bool r_reg_set_value(RReg *reg, RRegItem *item, ut64 value) {
 		eprintf ("r_reg_set_value: Bit size %d not supported\n", item->size);
 		return false;
 	}
-	fits_in_arena = (reg->regset[item->arena].arena->size - BITS2BYTES (item->offset) - BITS2BYTES (item->size)) >= 0;
+	const bool fits_in_arena = (arena->size - BITS2BYTES (item->offset) - BITS2BYTES (item->size)) >= 0;
 	if (src && fits_in_arena) {
 		r_mem_copybits (reg->regset[item->arena].arena->bytes +
 				BITS2BYTES (item->offset),
-			src, item->size);
+				src, item->size);
 		return true;
 	}
 	eprintf ("r_reg_set_value: Cannot set %s to 0x%" PFMT64x "\n", item->name, value);
@@ -229,26 +246,31 @@ R_API R_HEAP char *r_reg_get_bvalue(RReg *reg, RRegItem *item) {
 /* packed registers */
 // packbits can be 8, 16, 32 or 64
 // result value is always casted into ut64
-// TODO: use item->packed_size
+// TODO: support packbits=128 for xmm registers
 R_API ut64 r_reg_get_pack(RReg *reg, RRegItem *item, int packidx, int packbits) {
-	int packbytes, packmod;
-	ut64 ret = 0LL;
-	RRegSet *regset;
-	int off;
-	if (!reg || !item) {
-		return 0LL;
-	}
+	r_return_val_if_fail (reg && item, 0LL);
+
 	if (packbits < 1) {
 		packbits = item->packed_size;
 	}
-	packbytes = packbits / 8;
-	packmod = packbits % 8;
+	packbits = R_MIN (64, R_MAX (0, packbits));
+
+	ut64 ret = 0LL;
+	const int packbytes = packbits / 8;
+	const int packmod = packbits % 8;
 	if (packmod) {
 		eprintf ("Invalid bit size for packet register\n");
 		return 0LL;
 	}
-	off = BITS2BYTES (item->offset);
-	regset = &reg->regset[item->arena];
+	if (packidx * packbits > item->size) {
+		eprintf ("Packed index is beyond the register size\n");
+		return 0LL;
+	}
+	RRegSet *regset = &reg->regset[item->arena];
+	if (!regset->arena) {
+		return 0LL;
+	}
+	int off = BITS2BYTES (item->offset);
 	off += (packidx * packbytes);
 	if (regset->arena->size - off - 1 >= 0) {
 		memcpy (&ret, regset->arena->bytes + off, packbytes);
@@ -256,30 +278,25 @@ R_API ut64 r_reg_get_pack(RReg *reg, RRegItem *item, int packidx, int packbits)
 	return ret;
 }
 
+// TODO: support packbits=128 for xmm registers
 R_API int r_reg_set_pack(RReg *reg, RRegItem *item, int packidx, int packbits, ut64 val) {
-	int off, packbytes, packmod;
+	r_return_val_if_fail (reg && reg->regset->arena && item, false);
 
-	if (!reg || !item) {
-		eprintf ("r_reg_set_value: item is NULL\n");
-		return false;
-	}
 	if (packbits < 1) {
 		packbits = item->packed_size;
 	}
-	off = item->offset;
-	packbytes = packbits / 8;
-	packmod = packbits % 8;
+	packbits = R_MIN (64, R_MAX (0, packbits));
+
+	int packbytes = packbits / 8;
 	if (packidx * packbits > item->size) {
 		eprintf ("Packed index is beyond the register size\n");
 		return false;
 	}
-	if (packmod) {
-		eprintf ("Invalid bit size for packet register\n");
-		return false;
-	}
+	int off = BITS2BYTES (item->offset);
+	off += (packidx * packbytes);
 	if (reg->regset[item->arena].arena->size - BITS2BYTES (off) - BITS2BYTES (packbytes) >= 0) {
-		ut8 *dst = reg->regset[item->arena].arena->bytes + BITS2BYTES (off);
-		r_mem_copybits (dst, (ut8 *)&val, packbytes);
+		ut8 *dst = reg->regset[item->arena].arena->bytes + off;
+		memcpy (dst, (ut8*)&val, packbytes);
 		return true;
 	}
 	eprintf ("r_reg_set_value: Cannot set %s to 0x%" PFMT64x "\n", item->name, val);
