@@ -17,7 +17,9 @@ typedef struct {
 
 R_API RSearch *r_search_new(int mode) {
 	RSearch *s = R_NEW0 (RSearch);
-	if (!s) return NULL;
+	if (!s) {
+		return NULL;
+	}
 	if (!r_search_set_mode (s, mode)) {
 		free (s);
 		eprintf ("Cannot init search for mode %d\n", mode);
@@ -37,7 +39,6 @@ R_API RSearch *r_search_new(int mode) {
 	s->hits = r_list_newf (free);
 	s->maxhits = 0;
 	// TODO: review those mempool sizes. ensure never gets NULL
-	s->pool = r_mem_pool_new (sizeof (RSearchHit), 1024, 10);
 	s->kws = r_list_newf (free);
 	if (!s->kws) {
 		r_search_free (s);
@@ -51,17 +52,18 @@ R_API RSearch *r_search_free(RSearch *s) {
 	if (!s) {
 		return NULL;
 	}
-	r_mem_pool_free (s->pool);
 	r_list_free (s->hits);
 	r_list_free (s->kws);
-	//r_io_free(s->iob.io); this is suposed to be a weak reference
+	//r_io_free(s->iob.io); this is supposed to be a weak reference
+	free (s->data);
 	free (s);
 	return NULL;
 }
 
 R_API int r_search_set_string_limits(RSearch *s, ut32 min, ut32 max) {
-	if (max < min)
+	if (max < min) {
 		return false;
+	}
 	s->string_min = min;
 	s->string_max = max;
 	return true;
@@ -102,7 +104,6 @@ R_API int r_search_begin(RSearch *s) {
 
 // Returns 2 if search.maxhits is reached, 0 on error, otherwise 1
 R_API int r_search_hit_new(RSearch *s, RSearchKeyword *kw, ut64 addr) {
-	RSearchHit* hit;
 	if (s->align && (addr%s->align)) {
 		eprintf ("0x%08"PFMT64x" unaligned\n", addr);
 		return 1;
@@ -111,7 +112,7 @@ R_API int r_search_hit_new(RSearch *s, RSearchKeyword *kw, ut64 addr) {
 		if (kw->last && addr == kw->last) {
 			kw->count--;
 			kw->last = s->bckwrds ? addr : addr + kw->keyword_length;
-			eprintf ("0x%08"PFMT64x" Sequencial hit ignored.\n", addr);
+			eprintf ("0x%08"PFMT64x" Sequential hit ignored.\n", addr);
 			return 1;
 		}
 	}
@@ -126,12 +127,12 @@ R_API int r_search_hit_new(RSearch *s, RSearchKeyword *kw, ut64 addr) {
 	}
 	kw->count++;
 	s->nhits++;
-	if (!(hit = r_mem_pool_alloc (s->pool))) {
-		return 0;
+	RSearchHit* hit = R_NEW0 (RSearchHit);
+	if (hit) {
+		hit->kw = kw;
+		hit->addr = addr;
+		r_list_append (s->hits, hit);
 	}
-	hit->kw = kw;
-	hit->addr = addr;
-	r_list_append (s->hits, hit);
 	return s->maxhits && s->nhits >= s->maxhits ? 2 : 1;
 }
 
@@ -336,7 +337,9 @@ static bool brute_force_match(RSearch *s, RSearchKeyword *kw, const ut8 *buf, in
 				a = tolower (a);
 				b = tolower (b);
 			}
-			if ((a & kw->bin_binmask[k]) != (b & kw->bin_binmask[k])) break;
+			if ((a & kw->bin_binmask[k]) != (b & kw->bin_binmask[k])) {
+				break;
+			}
 		}
 	} else if (kw->icase) {
 		while (j < kw->keyword_length &&
