@@ -1,69 +1,118 @@
-/* radare - LGPL - Copyright 2011-2015 - pancake */
+/* radare - LGPL - Copyright 2011-2019 - pancake */
 
 #include <r_socket.h>
 #include <r_util.h>
 
-static char *r_socket_http_answer (RSocket *s, int *code, int *rlen) {
-	const char *p;
-	int ret, olen, len = 0, bufsz = 32768, delta = 0;
-	char *dn, *res, *buf = malloc (bufsz + 32); // XXX: use r_buffer here
-	if (!buf) return NULL;
+static int __socket_slurp (RSocket *s, ut8 *buf, int bufsz) {
+	int i;
+	int chsz = 1;
+	// r_socket_block_time (s, 1, 1, 0);
+	if (r_socket_read_block (s, (ut8 *) buf, 1) != 1) {
+		return 0;
+	}
+	for (i = 1; i < bufsz; i += chsz) {
+		buf[i] =0;
+		r_socket_block_time (s, 1, 0, 1000);
+		int olen = r_socket_read_block (s, (ut8 *) buf + i , chsz);
+		if (olen != chsz) {
+			buf[i] = 0;
+			break;
+		}
+	}
+	return i;
+}
 
-	r_socket_block_time (s, 1, 5);
-	res = NULL;
-	olen = r_socket_read_block (s, (unsigned char*) buf, bufsz);
-	if (olen < 1) goto fail;
-	buf[olen] = 0;
+static char *r_socket_http_answer (RSocket *s, int *code, int *rlen) {
+	r_return_val_if_fail (s, NULL);
+	const char *p;
+	int ret, len = 0, bufsz = 32768, delta = 0;
+	char *dn, *buf = calloc (1, bufsz + 32); // XXX: use r_buffer here
+	if (!buf) {
+		return NULL;
+	}
+	char *res = NULL;
+	int olen = __socket_slurp (s, (ut8*)buf, bufsz);
 	if ((dn = (char*)r_str_casestr (buf, "\n\n"))) {
 		delta += 2;
 	} else if ((dn = (char*)r_str_casestr (buf, "\r\n\r\n"))) {
 		delta += 4;
-	} else goto fail;
+	} else {
+		goto fail;
+	}
 
 	olen -= delta;
 	*dn = 0; // chop headers
 	/* Parse Len */
 	p = r_str_casestr (buf, "Content-Length: ");
-	if (p) len = atoi (p+16);
-	else len = olen - (dn - buf);
-
-	if (len >0) {
+	if (p) {
+		len = atoi (p + 16);
+	} else {
+		len = olen - (dn - buf);
+	}
+	if (len > 0) {
 		if (len > olen) {
-			res = malloc (len+2);
-			memcpy (res, dn+delta, olen);
+			res = malloc (len + 2);
+			memcpy (res, dn + delta, olen);
 			do {
-				ret = r_socket_read_block (s,
-					(ut8*) res+olen, len-olen);
-				if (ret < 1)
+				ret = r_socket_read_block (s, (ut8*) res + olen, len - olen);
+				if (ret < 1) {
 					break;
+				}
 				olen += ret;
-			} while (olen<len);
+			} while (olen < len);
 			res[len] = 0;
 		} else {
-			res = malloc (len+1);
+			res = malloc (len + 1);
 			if (res) {
-				memcpy (res, dn+delta, len);
+				memcpy (res, dn + delta, len);
 				res[len] = 0;
 			}
 		}
-	} else res = NULL;
+	} else {
+		res = NULL;
+	}
 fail:
 	free (buf);
-// is 's' free'd? isnt this going to cause a double free?
+// is 's' free'd? isn't this going to cause a double free?
 	r_socket_close (s);
-	if (rlen) *rlen = len;
+	if (rlen) {
+		*rlen = len;
+	}
 	return res;
 }
 
 R_API char *r_socket_http_get (const char *url, int *code, int *rlen) {
+	char *curl_env = r_sys_getenv ("R2_CURL");
+	if (curl_env && *curl_env) {
+		char *encoded_url = r_str_escape (url);
+		char *res = r_sys_cmd_strf ("curl '%s'", encoded_url);
+		free (encoded_url);
+		if (res) {
+			if (code) {
+				*code = 200;
+			}
+			if (rlen) {
+				*rlen = strlen (res);
+			}
+		}
+		free (curl_env);
+		return res;
+	}
+	free (curl_env);
 	RSocket *s;
 	int ssl = !memcmp (url, "https://", 8);
 	char *response, *host, *path, *port = "80";
 	char *uri = strdup (url);
-	if (!uri) return NULL;
+	if (!uri) {
+		return NULL;
+	}
 
-	if (code) *code = 0;
-	if (rlen) *rlen = 0;
+	if (code) {
+		*code = 0;
+	}
+	if (rlen) {
+		*rlen = 0;
+	}
 	host = strstr (uri, "://");
 	if (!host) {
 		free (uri);
@@ -80,8 +129,11 @@ R_API char *r_socket_http_get (const char *url, int *code, int *rlen) {
 		path = port;
 	}
 	path = strchr (path, '/');
-	if (!path) path = "";
-	else *path++ = 0;
+	if (!path) {
+		path = "";
+	} else {
+		*path++ = 0;
+	}
 	s = r_socket_new (ssl);
 	if (!s) {
 		eprintf ("r_socket_http_get: Cannot create socket\n");
@@ -107,28 +159,31 @@ R_API char *r_socket_http_get (const char *url, int *code, int *rlen) {
 
 R_API char *r_socket_http_post (const char *url, const char *data, int *code, int *rlen) {
 	RSocket *s;
-	int ssl = !memcmp (url, "https://", 8);
-	char *response, *host, *path, *port = "80";
+	bool ssl = !memcmp (url, "https://", 8);
 	char *uri = strdup (url);
-	if (!uri) return NULL;
+	if (!uri) {
+		return NULL;
+	}
 
-	host = strstr (uri, "://");
+	char *host = strstr (uri, "://");
 	if (!host) {
 		free (uri);
 		printf ("Invalid URI");
 		return NULL;
 	}
 	host += 3;
-	port = strchr (host, ':');
-	if (!port)
-		port = (ssl)?"443":"80";
-	else
+	char *port = strchr (host, ':');
+	if (!port) {
+		port = (ssl)? "443": "80";
+	} else {
 		*port++ = 0;
-	path = strchr (host, '/');
-	if (!path)
+	}
+	char *path = strchr (host, '/');
+	if (!path) {
 		path = "";
-	else
+	} else {
 		*path++ = 0;
+	}
 	s = r_socket_new (ssl);
 	if (!s) {
 		printf ("Cannot create socket\n");
@@ -148,11 +203,10 @@ R_API char *r_socket_http_post (const char *url, const char *data, int *code, in
 			"Host: %s\r\n"
 			"Content-Length: %i\r\n"
 			"Content-Type: application/x-www-form-urlencoded\r\n"
-			"\r\n", path, host, strlen (data));
-	r_socket_write (s, (void *)data, strlen (data));
-	response = r_socket_http_answer (s, code, rlen);
+			"\r\n", path, host, (int)strlen (data));
 	free (uri);
-	return response;
+	r_socket_write (s, (void *)data, strlen (data));
+	return r_socket_http_answer (s, code, rlen);
 }
 
 #if TEST
