@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2013-2017 - pancake */
+/* radare - LGPL - Copyright 2013-2018 - pancake */
 
 // XXX: should use the same code as libr/io/cache.c
 // one malloc per write
@@ -44,6 +44,9 @@ R_API const ut8 *r_cache_get(RCache *c, ut64 addr, int *len) {
 }
 
 R_API int r_cache_set(RCache *c, ut64 addr, const ut8 *buf, int len) {
+	if (!c) {
+		return 0;
+	}
 	if (!c->buf) {
 		c->buf = malloc (len);
 		if (!c->buf) {
@@ -89,7 +92,6 @@ R_API void r_cache_flush(RCache *c) {
 	if (c) {
 		c->base = 0;
 		c->len = 0;
-		free (c->buf);
-		c->buf = NULL;
+		R_FREE (c->buf);
 	}
 }
