@@ -124,6 +124,8 @@ R_API int r_type_get_bitsize(Sdb *TDB, const char *type) {
 	const char *tmptype;
 	if (!strncmp (type, "struct ", 7)) {
 		tmptype = type + 7;
+	} else if (!strncmp (type, "union ", 6)) {
+		tmptype = type + 6;
 	} else {
 		tmptype = type;
 	}
@@ -140,11 +142,13 @@ R_API int r_type_get_bitsize(Sdb *TDB, const char *type) {
 		return 0;
 	}
 	if (!strcmp (t, "type")){
-		query = sdb_fmt ("type.%s.size", tmptype);
-		return sdb_num_get (TDB, query, 0); // returns size in bits
+		query = r_str_newf ("type.%s.size", tmptype);
+		int r = (int)sdb_num_get (TDB, query, 0); // returns size in bits
+		free (query);
+		return r;
 	}
-	if (!strcmp (t, "struct")) {
-		query = sdb_fmt ("struct.%s", tmptype);
+	if (!strcmp (t, "struct") || !strcmp (t, "union")) {
+		query = r_str_newf ("%s.%s", t, tmptype);
 		char *members = sdb_get (TDB, query, 0);
 		char *next, *ptr = members;
 		int ret = 0;
@@ -154,8 +158,10 @@ R_API int r_type_get_bitsize(Sdb *TDB, const char *type) {
 				if (!name) {
 					break;
 				}
-				query = sdb_fmt ("struct.%s.%s", tmptype, name);
+				free (query);
+				query = r_str_newf ("%s.%s.%s", t, tmptype, name);
 				char *subtype = sdb_get (TDB, query, 0);
+				R_FREE (query);
 				if (!subtype) {
 					break;
 				}
@@ -170,13 +176,19 @@ R_API int r_type_get_bitsize(Sdb *TDB, const char *type) {
 					if (elements == 0) {
 						elements = 1;
 					}
-					ret += r_type_get_bitsize (TDB, subtype) * elements;
+					if (!strcmp (t, "struct")) {
+						ret += r_type_get_bitsize (TDB, subtype) * elements;
+					} else {
+						int sz = r_type_get_bitsize (TDB, subtype) * elements;
+						ret = sz > ret ? sz : ret;
+					}
 				}
 				free (subtype);
 				ptr = next;
 			} while (next);
 			free (members);
 		}
+		free (query);
 		return ret;
 	}
 	return 0;
@@ -424,6 +436,14 @@ R_API char *r_type_format(Sdb *TDB, const char *t) {
 		free (vars);
 		return fmt;
 	}
+	if (!strcmp (kind, "typedef")) {
+		snprintf (var2, sizeof (var2), "typedef.%s", t);
+		const char *type = sdb_const_get (TDB, var2, NULL);
+		// only supports struct atm
+		if (type && r_str_startswith (type, "struct ") && strcmp (type + 7, t)) {
+			return r_type_format (TDB, type + 7);
+		}
+	}
 	return NULL;
 }
 
@@ -464,12 +484,19 @@ R_API void r_type_del(Sdb *TDB, const char *name) {
 		RListIter *iter;
 		r_list_foreach (list, iter, member) {
 			sdb_unset (TDB, sdb_fmt ("enum.%s.%s", name, member->name), 0);
-			sdb_unset (TDB, sdb_fmt ("enum.%s.0x%x", name, member->val), 0);
+			sdb_unset (TDB, sdb_fmt ("enum.%s.%s", name, member->val), 0);
 		}
 		sdb_unset (TDB, name, 0);
 		r_list_free (list);
+	} else if (!strcmp (kind, "typedef")) {
+		RStrBuf buf;
+		r_strbuf_init (&buf);
+		r_strbuf_setf (&buf, "typedef.%s", name);
+		sdb_unset (TDB, r_strbuf_get (&buf), 0);
+		r_strbuf_fini (&buf);
+		sdb_unset (TDB, name, 0);
 	} else {
-		eprintf ("Unrecognized type \"%s\"\n", kind);
+		eprintf ("Unrecognized type kind \"%s\"\n", kind);
 	}
 }
 
@@ -536,7 +563,7 @@ R_API R_OWN char *r_type_func_guess(Sdb *TDB, R_NONNULL char *func_name) {
 	int offset = 0;
 	char *str = func_name;
 	char *result = NULL;
-	char *first, *last;
+	char *first;
 	r_return_val_if_fail (TDB, false);
 	r_return_val_if_fail (func_name, false);
 
@@ -591,35 +618,9 @@ R_API R_OWN char *r_type_func_guess(Sdb *TDB, R_NONNULL char *func_name) {
 	}
 	// some names are in format module.dll_function_number, try to remove those
 	// also try module.dll_function and function_number
-	if ((first = strchr (str, '_'))) {
-		// check if the prefix is actually "dll_" otherwise don't try to
-		// interpret the name
-		const char *dll = "dll";
-		char *dll_ptr = first - strlen (dll);
-		if (dll_ptr < str || strncmp (dll_ptr, dll, strlen (dll))) {
-			goto out;
-		}
-
-		last = (char *)r_str_lchr (first, '_');
-		if (!last) {
-			goto out;
-		}
-		// middle + suffix or right half
-		if ((result = type_func_try_guess (TDB, first + 1))) {
-			goto out;
-		}
-		last[0] = 0;
-		// prefix + middle or left
-		if ((result = type_func_try_guess (TDB, str))) {
-			goto out;
-		}
-		if (last != first) {
-			// middle
-			if ((result = type_func_try_guess (TDB, first + 1))) {
-				goto out;
-			}
-		}
-		result = NULL;
+	if ((first = strstr (str, "dll_"))) {
+		result = type_func_try_guess (TDB, first + 4);
+		goto out;
 	}
 out:
 	free (str);
