@@ -1,9 +1,10 @@
-/* radare - LGPL - Copyright 2007-2018 - pancake */
+/* radare - LGPL - Copyright 2007-2019 - pancake */
 
 #include "r_anal.h"
 #include "r_cons.h"
 #include "r_util.h"
 #include "r_util/r_print.h"
+#include "r_core.h"
 
 #define DFLT_ROWS 16
 
@@ -23,6 +24,14 @@ static int libc_printf(const char *format, ...) {
 	return 0;
 }
 
+static int libc_eprintf(const char *format, ...) {
+	va_list ap;
+	va_start (ap, format);
+	vfprintf (stderr, format, ap);
+	va_end (ap);
+	return 0;
+}
+
 static RPrintIsInterruptedCallback is_interrupted_cb = NULL;
 
 R_API void r_print_portionbar(RPrint *p, const ut64 *portions, int n_portions) {
@@ -66,13 +75,27 @@ R_API void r_print_columns (RPrint *p, const ut8 *buf, int len, int height) {
 	int rows = height > 0 ? height : 10;
 	// int realrows = rows * 2;
 	bool colors = p->flags & R_PRINT_FLAGS_COLOR;
+	RConsPrintablePalette *pal = &p->cons->context->pal;
+	const char *vline = p->cons->use_utf8 ? RUNE_LINE_VERT : "|";
+	const char *block = p->cons->use_utf8 ? UTF_BLOCK : "#";
+	const char *kol[5];
+	kol[0] = pal->call;
+	kol[1] = pal->jmp;
+	kol[2] = pal->cjmp;
+	kol[3] = pal->mov;
+	kol[4] = pal->nop;
 	if (colors) {
 		for (i = 0; i < rows; i++) {
 			int threshold = i * (0xff / rows);
 			for (j = 0; j < cols; j++) {
 				int realJ = j * len / cols;
-				if (255 - buf[realJ] < threshold || (i + 1 == rows)) {
-					p->cb_printf (Color_BGRED"_" Color_RESET);
+	 			if (255 - buf[realJ] < threshold || (i + 1 == rows)) {
+					int koli = i * 5 / rows;
+					if (p->histblock) {
+						p->cb_printf ("%s%s%s", kol[koli], block, Color_RESET);
+					} else {
+						p->cb_printf ("%s%s%s", kol[koli], vline, Color_RESET);
+					}
 				} else {
 					p->cb_printf (" ");
 				}
@@ -87,7 +110,11 @@ R_API void r_print_columns (RPrint *p, const ut8 *buf, int len, int height) {
 		for (j = 0; j < cols; j++) {
 			int realJ = j * len / cols;
 			if (255 - buf[realJ] < threshold) {
-				p->cb_printf ("|");
+				if (p->histblock) {
+					p->cb_printf ("%s%s%s", Color_BGGRAY, block, Color_RESET);
+				} else {
+					p->cb_printf (vline);
+				}
 			} else if (i + 1 == rows) {
 				p->cb_printf ("_");
 			} else {
@@ -286,6 +313,7 @@ R_API RPrint* r_print_new() {
 	p->pairs = true;
 	p->resetbg = true;
 	p->cb_printf = libc_printf;
+	p->cb_eprintf = libc_eprintf;
 	p->oprintf = nullprinter;
 	p->bits = 32;
 	p->stride = 0;
@@ -319,6 +347,7 @@ R_API RPrint* r_print_new() {
 	p->esc_bslash = false;
 	p->strconv_mode = NULL;
 	memset (&p->consbind, 0, sizeof (p->consbind));
+	p->io_unalloc_ch = '.';
 	return p;
 }
 
@@ -380,6 +409,20 @@ R_API bool r_print_have_cursor(RPrint *p, int cur, int len) {
 	return false;
 }
 
+R_API bool r_print_cursor_pointer(RPrint *p, int cur, int len) {
+	r_return_val_if_fail (p, false);
+	if (!p->cur_enabled) {
+		return false;
+	}
+	int to = p->cur;
+	do {
+		if (cur + len - 1 == to) {
+			return true;
+		}
+	} while (--len);
+	return false;
+}
+
 R_API void r_print_cursor(RPrint *p, int cur, int len, int set) {
 	if (r_print_have_cursor (p, cur, len)) {
 		p->cb_printf ("%s", R_CONS_INVERT (set, 1));
@@ -391,7 +434,7 @@ R_API void r_print_addr(RPrint *p, ut64 addr) {
 		0
 	};
 	const char *white = "";
-#define PREOFF(x) (p && p->cons && p->cons->pal.x)? p->cons->pal.x
+#define PREOFF(x) (p && p->cons && p->cons->context && p->cons->context->pal.x)? p->cons->context->pal.x
 	PrintfCallback printfmt = (PrintfCallback) (p? p->cb_printf: libc_printf);
 	bool use_segoff = p? (p->flags & R_PRINT_FLAGS_SEGOFF): false;
 	bool use_color = p? (p->flags & R_PRINT_FLAGS_COLOR): false;
@@ -401,6 +444,12 @@ R_API void r_print_addr(RPrint *p, ut64 addr) {
 	if (p && p->flags & R_PRINT_FLAGS_COMPACT && p->col == 1) {
 		ch = '|';
 	}
+	if (p && p->pava) {
+		ut64 va = p->iob.p2v (p->iob.io, addr);
+		if (va != UT64_MAX) {
+			addr = va;
+		}
+	}
 	if (use_segoff) {
 		ut32 s, a;
 		a = addr & 0xffff;
@@ -482,7 +531,7 @@ R_API char* r_print_hexpair(RPrint *p, const char *str, int n) {
 	int ch, i;
 
 	if (colors) {
-#define P(x) (p->cons && p->cons->pal.x)? p->cons->pal.x
+#define P(x) (p->cons && p->cons->context->pal.x)? p->cons->context->pal.x
 		color_0x00 = P (b0x00): Color_GREEN;
 		color_0x7f = P (b0x7f): Color_YELLOW;
 		color_0xff = P (b0xff): Color_RED;
@@ -556,7 +605,7 @@ R_API char* r_print_hexpair(RPrint *p, const char *str, int n) {
 }
 
 static char colorbuffer[64];
-#define P(x) (p->cons && p->cons->pal.x)? p->cons->pal.x
+#define P(x) (p->cons && p->cons->context->pal.x)? p->cons->context->pal.x
 R_API const char *r_print_byte_color(RPrint *p, int ch) {
 	if (p->flags & R_PRINT_FLAGS_RAINBOW) {
 		// EXPERIMENTAL
@@ -683,7 +732,7 @@ static bool isAllZeros(const ut8 *buf, int len) {
 	return true;
 }
 
-#define Pal(x,y) (x->cons && x->cons->pal.y)? x->cons->pal.y
+#define Pal(x,y) (x->cons && x->cons->context->pal.y)? x->cons->context->pal.y
 R_API void r_print_hexii(RPrint *rp, ut64 addr, const ut8 *buf, int len, int step) {
 	PrintfCallback p = (PrintfCallback) rp->cb_printf;
 	bool c = rp->flags & R_PRINT_FLAGS_COLOR;
@@ -751,19 +800,32 @@ R_API void r_print_set_screenbounds(RPrint *p, ut64 addr) {
 	}
 }
 
+R_API void r_print_section(RPrint *p, ut64 at) {
+	bool use_section = p && p->flags & R_PRINT_FLAGS_SECTION;
+	if (use_section) {
+		const char *s = p->get_section_name (p->user, at);
+		if (!s) {
+			s = "";
+		}
+		char *tail = r_str_ndup (s, 19);
+		p->cb_printf ("%20s ", tail);
+		free (tail);
+	}
+}
+
 R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int base, int step, int zoomsz) {
 	PrintfCallback printfmt = (PrintfCallback) printf;
-	bool c = p->flags & R_PRINT_FLAGS_COLOR;
+	bool c = p? (p->flags & R_PRINT_FLAGS_COLOR): false;
 	const char *color_title = c? (Pal (p, offset): Color_MAGENTA): "";
-	int i, j, k, inc = 16;
+	int i, j, k, inc = p? p->cols : 16;
 	int sparse_char = 0;
 	int stride = 0;
 	int col = 0; // selected column (0=none, 1=hex, 2=ascii)
 	int use_sparse = 0;
 	bool use_header = true;
 	bool use_hdroff = true;
-	int use_pair = 1;
-	int use_offset = 1;
+	bool use_pair = true;
+	bool use_offset = true;
 	bool compact = false;
 	int use_segoff = 0;
 	int pairs = 0;
@@ -771,9 +833,12 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 	const char *pre = "";
 	int last_sparse = 0;
 	bool use_hexa = true;
+	bool use_align = false;
+	bool use_unalloc = false;
 	const char *a, *b;
 	int K = 0;
 	bool hex_style = false;
+	int rowbytes = p->cols;
 
 	len = len - (len % step);
 	if (p) {
@@ -782,9 +847,11 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 		use_header = p->flags & R_PRINT_FLAGS_HEADER;
 		use_hdroff = p->flags & R_PRINT_FLAGS_HDROFF;
 		use_segoff = p->flags & R_PRINT_FLAGS_SEGOFF;
+		use_align = p->flags & R_PRINT_FLAGS_ALIGN;
 		use_offset = p->flags & R_PRINT_FLAGS_OFFSET;
 		hex_style = p->flags & R_PRINT_FLAGS_STYLE;
 		use_hexa = !(p->flags & R_PRINT_FLAGS_NONHEX);
+		use_unalloc = p->flags & R_PRINT_FLAGS_UNALLOC;
 		compact = p->flags & R_PRINT_FLAGS_COMPACT;
 		inc = p->cols; // row width
 		col = p->col;
@@ -936,7 +1003,7 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 					printfmt (" ");
 				}
 				if (!hex_style) {
-					printfmt (" comment ");
+					printfmt (" comment");
 				}
 			}
 			printfmt ("\n");
@@ -953,10 +1020,23 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 	// is this necessary?
 	r_print_set_screenbounds (p, addr);
 	int rows = 0;
-	for (i = j = 0; i < len; i += (stride? stride: inc), j += (stride? stride: 0)) {
+	int bytes = 0;
+	bool printValue = true;
+	bool oPrintValue = true;
+	bool isPxr = (p && p->flags & R_PRINT_FLAGS_REFS);
+
+	for (i = j = 0; i < len; i += (stride? stride: inc)) {
 		if (p && p->cons && p->cons->context && p->cons->context->breaked) {
 			break;
 		}
+		rowbytes = inc;
+		if (use_align) {
+			int sz = (p && p->offsize)? p->offsize (p->user, addr + j): -1;
+			if (sz > 0) { // flags with size 0 dont work
+				rowbytes = sz;
+			}
+		}
+
 		if (use_sparse) {
 			if (checkSparse (buf + i, inc, sparse_char)) {
 				if (i + inc >= len || checkSparse (buf + i + inc, inc, sparse_char)) {
@@ -977,23 +1057,31 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 				last_sparse = 0;
 			}
 		}
-		if (use_offset) {
-			r_print_addr (p, addr + j * zoomsz);
+		ut64 at = addr + (j * zoomsz);
+		if (use_offset && !isPxr) {
+			r_print_section (p, at);
+			r_print_addr (p, at);
 		}
 		int row_have_cursor = -1;
 		ut64 row_have_addr = UT64_MAX;
 		if (use_hexa) {
-			if (!compact) {
+			if (!compact && !isPxr) {
 				printfmt ((col == 1)? "|": " ");
 			}
 			for (j = i; j < i + inc; j++) {
+				if (j!=i && use_align && rowbytes == inc) {
+					int sz = (p && p->offsize)? p->offsize (p->user, addr + j): -1;
+					if (sz >= 0) {
+						rowbytes = bytes;
+					}
+				}
 				if (row_have_cursor == -1) {
-					if (r_print_have_cursor (p, j, 1)) {
+					if (r_print_cursor_pointer (p, j, 1)) {
 						row_have_cursor = j - i;
 						row_have_addr = addr + j;
 					}
 				}
-				if (!compact && j >= len) {
+				if (!compact && ((j >= len) || bytes >= rowbytes)) {
 					if (col == 1) {
 						if (j + 1 >= inc + i) {
 							printfmt (j % 2? "  |": "| ");
@@ -1011,7 +1099,7 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 					}
 					continue;
 				}
-				const char *hl = (hex_style && p->offname (p->user, addr + j))? Color_INVERT: NULL;
+				const char *hl = (hex_style && p && p->offname (p->user, addr + j))? Color_INVERT: NULL;
 				if (hl) {
 					printfmt (hl);
 				}
@@ -1020,13 +1108,9 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 					/* TODO: check step. it should be 2/4 for base(32) and 8 for
 					 *       base(64) */
 					ut64 n = 0;
-					size_t sz_n;
-
-					if (base == 64) {
-						sz_n = sizeof (ut64);
-					} else {
-						sz_n = step == 2? sizeof (ut16): sizeof (ut32);
-					}
+					size_t sz_n = (base == 64)
+						? sizeof (ut64) : (step == 2)
+						? sizeof (ut16) : sizeof (ut32);
 					sz_n = R_MIN (left, sz_n);
 					if (j + sz_n > len) {
 						// oob
@@ -1038,7 +1122,7 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 					// stub for colors
 					if (p && p->colorfor) {
 						if (!p->iob.addr_is_mapped (p->iob.io, addr + j)) {
-							a = p->cons->pal.ai_unmap;
+							a = p->cons->context->pal.ai_unmap;
 						} else {
 							a = p->colorfor (p->user, n, true);
 						}
@@ -1050,14 +1134,38 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 					} else {
 						a = b = "";
 					}
-					if (base == 64) {
-						printfmt ("%s0x%016" PFMT64x "%s  ", a, (ut64) n, b);
-					} else if (step == 2) {
-						printfmt ("%s0x%04x%s ", a, (ut16) n, b);
+					printValue = true;
+					bool hasNull = false;
+					if (isPxr) {
+						if (n == 0) {
+							if (oPrintValue) {
+								hasNull = true;
+							}
+							printValue = false;
+						}
+					}
+					if (printValue) {
+						if (use_offset && !hasNull && isPxr) {
+							r_print_section (p, at);
+							r_print_addr (p, addr + j * zoomsz);
+						}
+						if (base == 64) {
+							printfmt ("%s0x%016" PFMT64x "%s  ", a, (ut64) n, b);
+						} else if (step == 2) {
+							printfmt ("%s0x%04x%s ", a, (ut16) n, b);
+						} else {
+							printfmt ("%s0x%08x%s ", a, (ut32) n, b);
+						}
 					} else {
-						printfmt ("%s0x%08x%s ", a, (ut32) n, b);
+						if (hasNull) {
+							const char *n = p->offname (p->user, addr + j);
+							r_print_section (p, at);
+							r_print_addr (p, addr + j * zoomsz);
+							printfmt ("..[ null bytes ]..   00000000 %s\n", n? n: "");
+						}
 					}
 					r_print_cursor (p, j, sz_n, 0);
+					oPrintValue = printValue;
 					j += step - 1;
 				} else if (base == -8) {
 					long long w = r_read_ble64 (buf + j, p && p->big_endian);
@@ -1071,28 +1179,38 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 					printfmt ("%4d ", w);
 					r_print_cursor (p, j, 1, 0);
 				} else if (base == -10) {
-					st16 w = r_read_ble16 (buf + j, p && p->big_endian);
-					r_print_cursor (p, j, 2, 1);
-					printfmt ("%7d ", w);
-					r_print_cursor (p, j, 2, 0);
+					if (j + 1 < len) {
+						st16 w = r_read_ble16 (buf + j, p && p->big_endian);
+						r_print_cursor (p, j, 2, 1);
+						printfmt ("%7d ", w);
+						r_print_cursor (p, j, 2, 0);
+					}
 					j += 1;
 				} else if (base == 10) { // "pxd"
-					int w = r_read_ble32 (buf + j, p && p->big_endian);
-					r_print_cursor (p, j, 4, 1);
-					printfmt ("%13d ", w);
-					r_print_cursor (p, j, 4, 0);
+					if (j + 3 < len) {
+						int w = r_read_ble32 (buf + j, p && p->big_endian);
+						r_print_cursor (p, j, 4, 1);
+						printfmt ("%13d ", w);
+						r_print_cursor (p, j, 4, 0);
+					}
 					j += 3;
 				} else {
 					if (j >= len) {
 						break;
 					}
-					r_print_byte (p, bytefmt, j, buf[j]);
+					if (use_unalloc && !p->iob.is_valid_offset (p->iob.io, addr + j, false)) {
+						char ch = p->io_unalloc_ch;
+						char dbl_ch_str[] = { ch, ch, 0 };
+						p->cb_printf (dbl_ch_str);
+					} else {
+						r_print_byte (p, bytefmt, j, buf[j]);
+					}
 					if (pairs && !compact && (inc & 1)) {
 						bool mustspace = (rows % 2) ? !(j&1) : (j&1);
 						if (mustspace) {
 							printfmt (" ");
 						}
-					} else if (j % 2 || !pairs) {
+					} else if (bytes % 2 || !pairs) {
 						if (col == 1) {
 							if (j + 1 < inc + i) {
 								if (!compact) {
@@ -1111,85 +1229,110 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 				if (hl) {
 					printfmt (Color_RESET);
 				}
+				bytes++;
 			}
 		}
-		if (compact) {
-			if (col == 0) {
-				printfmt (" ");
-			} else if (col == 1) {
-				//printfmt (" ");
+		if (printValue) {
+			if (compact) {
+				if (col == 0) {
+					printfmt (" ");
+				} else if (col == 1) {
+					//printfmt (" ");
+				} else {
+					printfmt ((col == 2)? "|": "");
+				}
 			} else {
-				printfmt ((col == 2)? "|": "");
+				printfmt ((col == 2)? "|": " ");
 			}
-		} else {
-			printfmt ((col == 2)? "|": " ");
-		}
-		if (!p || !(p->flags & R_PRINT_FLAGS_NONASCII)) {
-			for (j = i; j < i + inc; j++) {
-				if (j >= len) {
-					break;
-				}
-				r_print_byte (p, "%c", j, buf[j]);
-			}
-		}
-		/* ascii column */
-		if (col == 2) {
-			printfmt ("|");
-		}
-		if (p && p->flags & R_PRINT_FLAGS_REFS) {
-			ut64 *foo = (ut64 *) (buf + i);
-			ut64 off = *foo;
-			if (base == 32) {
-				off &= UT32_MAX;
-			}
-			if (p->hasrefs) {
-				char *rstr = p->hasrefs (p->user, addr + i, false);
-				if (rstr && *rstr) {
-					printfmt (" @%s", rstr);
-				}
-				free (rstr);
-				rstr = p->hasrefs (p->user, off, true);
-				if (rstr && *rstr) {
-					printfmt ("%s", rstr);
-				}
-				free (rstr);
-			}
-		}
-		if (p && p->use_comments) {
-			for (; j < i + inc; j++) {
-				printfmt (" ");
-			}
-			for (j = i; j < i + inc; j++) {
-				if (p && p->offname) {
-					a = p->offname (p->user, addr + j);
-					if (p->colorfor && a && *a) {
-						const char *color = p->colorfor (p->user, addr + j, true);
-						printfmt ("%s  ; %s"Color_RESET, color?color: "", a);
+			if (!p || !(p->flags & R_PRINT_FLAGS_NONASCII)) {
+				bytes = 0;
+				for (j = i; j < i + inc; j++) {
+					if (j!=i && use_align  && bytes >= rowbytes) {
+						int sz = (p && p->offsize)? p->offsize (p->user, addr + j): -1;
+						if (sz >= 0) {
+							printfmt (" ");
+							break;
+						}
 					}
+					if (j >= len || (use_align && bytes >= rowbytes)) {
+						break;
+					}
+					ut8 ch = (use_unalloc && p && !p->iob.is_valid_offset (p->iob.io, addr + j, false))
+						? ' ' : buf[j];
+					r_print_byte (p, "%c", j, ch);
+					bytes++;
 				}
-				char *comment = p->get_comments (p->user, addr + j);
-				if (comment) {
-					if (p && p->colorfor) {
-						a = p->colorfor (p->user, addr + j, true);
-						if (a && *a) {
-							b = Color_RESET;
+			}
+			/* ascii column */
+			if (col == 2) {
+				printfmt ("|");
+			}
+			bool eol = false;
+			if (!eol && p && p->flags & R_PRINT_FLAGS_REFS) {
+				ut64 off = 0;
+				if (i + 8 < len) {
+					ut64 *foo = (ut64 *) (buf + i);
+					off = *foo;
+				}
+				if (base == 32) {
+					off &= UT32_MAX;
+				}
+				if (p->hasrefs) {
+					char *rstr = p->hasrefs (p->user, addr + i, false);
+					if (rstr && *rstr) {
+						printfmt (" @%s", rstr);
+					}
+					free (rstr);
+					rstr = p->hasrefs (p->user, off, true);
+					if (rstr && *rstr) {
+						printfmt ("%s", rstr);
+					}
+					free (rstr);
+				}
+			}
+			if (!eol && p && p->use_comments) {
+				for (; j < i + inc; j++) {
+					printfmt (" ");
+				}
+				for (j = i; j < i + inc; j++) {
+					if (use_align && (j-i) >= rowbytes) {
+						break;
+					}
+					if (p && p->offname) {
+						a = p->offname (p->user, addr + j);
+						if (p->colorfor && a && *a) {
+							const char *color = p->colorfor (p->user, addr + j, true);
+							printfmt ("%s  ; %s%s", color ? color: "", a,
+									color ? Color_RESET : "");
+						}
+					}
+					char *comment = p->get_comments (p->user, addr + j);
+					if (comment) {
+						if (p && p->colorfor) {
+							a = p->colorfor (p->user, addr + j, true);
+							if (a && *a) {
+								b = Color_RESET;
+							} else {
+								a = b = "";
+							}
 						} else {
 							a = b = "";
 						}
-					} else {
-						a = b = "";
+						printfmt ("%s  ; %s", a, comment);
+						free (comment);
 					}
-					printfmt ("%s  ; %s", a, comment);
-					free (comment);
 				}
 			}
+			if (use_align && rowbytes < inc && bytes >= rowbytes) {
+				i -= (inc - bytes);
+			}
+			printfmt ("\n");
 		}
-		printfmt ("\n");
 		rows++;
-
+		bytes = 0;
 		if (p && p->cfmt && *p->cfmt) {
 			if (row_have_cursor != -1) {
-				int i=0;
+				int i = 0;
 				printfmt (" _________");
 				if (!compact) {
 					printfmt ("_");
@@ -1214,12 +1357,12 @@ R_API void r_print_hexdump_simple(const ut8 *buf, int len) {
 	r_print_hexdump (NULL, 0, buf, len, 16, 16, 0);
 }
 
-static const char* getbytediff(char *fmt, ut8 a, ut8 b) {
+static const char* getbytediff(RPrint *p, char *fmt, ut8 a, ut8 b) {
 	if (*fmt) {
 		if (a == b) {
-			sprintf (fmt, Color_GREEN "%02x" Color_RESET, a);
+			sprintf (fmt, "%s%02x" Color_RESET, p->cons->context->pal.graph_true, a);
 		} else {
-			sprintf (fmt, Color_RED "%02x" Color_RESET, a);
+			sprintf (fmt, "%s%02x" Color_RESET, p->cons->context->pal.graph_false, a);
 		}
 	} else {
 		sprintf (fmt, "%02x", a);
@@ -1227,13 +1370,13 @@ static const char* getbytediff(char *fmt, ut8 a, ut8 b) {
 	return fmt;
 }
 
-static const char* getchardiff(char *fmt, ut8 a, ut8 b) {
+static const char* getchardiff(RPrint *p, char *fmt, ut8 a, ut8 b) {
 	char ch = IS_PRINTABLE (a)? a: '.';
 	if (*fmt) {
 		if (a == b) {
-			sprintf (fmt, Color_GREEN "%c" Color_RESET, ch);
+			sprintf (fmt, "%s%c" Color_RESET, p->cons->context->pal.graph_true, ch);
 		} else {
-			sprintf (fmt, Color_RED "%c" Color_RESET, ch);
+			sprintf (fmt, "%s%c" Color_RESET, p->cons->context->pal.graph_false, ch);
 		}
 	} else {
 		sprintf (fmt, "%c", ch);
@@ -1242,8 +1385,8 @@ static const char* getchardiff(char *fmt, ut8 a, ut8 b) {
 	return fmt;
 }
 
-#define BD(a, b) getbytediff (fmt, (a)[i + j], (b)[i + j])
-#define CD(a, b) getchardiff (fmt, (a)[i + j], (b)[i + j])
+#define BD(a, b) getbytediff (p, fmt, (a)[i + j], (b)[i + j])
+#define CD(a, b) getchardiff (p, fmt, (a)[i + j], (b)[i + j])
 
 static ut8* M(const ut8 *b, int len) {
 	ut8 *r = malloc (len + 16);
@@ -1412,6 +1555,9 @@ R_API void r_print_progressbar(RPrint *p, int pc, int _cols) {
 	if (!p) {
 		p = &staticp;
 	}
+	const char *h_line = p->cons->use_utf8 ? RUNE_LONG_LINE_HORIZ : "-";
+	const char *block = p->cons->use_utf8 ? UTF_BLOCK : "#";
+
 	pc = R_MAX (0, R_MIN (100, pc));
 	if (p->flags & R_PRINT_FLAGS_HEADER) {
 		p->cb_printf ("%4d%% ", pc);
@@ -1419,15 +1565,17 @@ R_API void r_print_progressbar(RPrint *p, int pc, int _cols) {
 	cols -= 15;
 	p->cb_printf ("[");
 	for (i = cols * pc / 100; i; i--) {
-		p->cb_printf ("#");
+		p->cb_printf (block);
 	}
 	for (i = cols - (cols * pc / 100); i; i--) {
-		p->cb_printf ("-");
+		p->cb_printf (h_line);
 	}
 	p->cb_printf ("]");
 }
 
 R_API void r_print_rangebar(RPrint *p, ut64 startA, ut64 endA, ut64 min, ut64 max, int cols) {
+	const char *h_line = p->cons->use_utf8 ? RUNE_LONG_LINE_HORIZ : "-";
+	const char *block = p->cons->use_utf8 ? UTF_BLOCK : "#";
 	const bool show_colors = p->flags & R_PRINT_FLAGS_COLOR;
 	int j = 0;
 	p->cb_printf ("|");
@@ -1441,24 +1589,21 @@ R_API void r_print_rangebar(RPrint *p, ut64 startA, ut64 endA, ut64 min, ut64 ma
 		ut64 endB = min + ((j + 1) * mul);
 		if (startA <= endB && endA >= startB) {
 			if (show_colors & isFirst) {
-				p->cb_printf (Color_GREEN "#");
+				p->cb_printf (Color_GREEN);
 				isFirst = false;
-			} else {
-				p->cb_printf ("#");
 			}
+			p->cb_printf (block);
 		} else {
-			if (isFirst) {
-				p->cb_printf ("-");
-			} else {
-				p->cb_printf (Color_RESET "-");
-				isFirst = true;
+			if (!isFirst) {
+				p->cb_printf (Color_RESET);
 			}
+			p->cb_printf (h_line);
 		}
 	}
 	p->cb_printf ("|");
 }
 
-R_API void r_print_zoom(RPrint *p, void *user, RPrintZoomCallback cb, ut64 from, ut64 to, int len, int maxlen) {
+R_API void r_print_zoom_buf(RPrint *p, void *user, RPrintZoomCallback cb, ut64 from, ut64 to, int len, int maxlen) {
 	static int mode = -1;
 	ut8 *bufz = NULL, *bufz2 = NULL;
 	int i, j = 0;
@@ -1496,6 +1641,9 @@ R_API void r_print_zoom(RPrint *p, void *user, RPrintZoomCallback cb, ut64 from,
 
 		// TODO: memoize blocks or gtfo
 		for (i = 0; i < len; i++) {
+			if (p->cons->context->breaked) {
+				break;
+			}
 			p->iob.read_at (p->iob.io, from + j, bufz2, size);
 			bufz[i] = cb (user, p->zoom->mode, from + j, bufz2, size);
 			j += size;
@@ -1508,22 +1656,56 @@ R_API void r_print_zoom(RPrint *p, void *user, RPrintZoomCallback cb, ut64 from,
 		p->zoom->to = to;
 		p->zoom->size = len; // size;
 	}
+}
+
+R_API void r_print_zoom(RPrint *p, void *user, RPrintZoomCallback cb, ut64 from, ut64 to, int len, int maxlen) {
+	ut64 size = (to - from);
+	r_print_zoom_buf (p, user, cb, from, to, len, maxlen);
+	size = len? size / len: 0;
 	p->flags &= ~R_PRINT_FLAGS_HEADER;
-	r_print_hexdump (p, from, bufz, len, 16, 1, size);
+	r_print_hexdump (p, from, p->zoom->buf, p->zoom->size, 16, 1, size);
 	p->flags |= R_PRINT_FLAGS_HEADER;
 }
 
-R_API void r_print_fill(RPrint *p, const ut8 *arr, int size, ut64 addr, int step) {
-	if (!p || !arr) {
-		return;
+static inline void printHistBlock (RPrint *p, int k, int cols) {
+	RConsPrintablePalette *pal = &p->cons->context->pal;
+	const char *h_line = p->cons->use_utf8 ? RUNE_LONG_LINE_HORIZ : "-";
+	const char *block = p->cons->use_utf8 ? UTF_BLOCK : "#";
+	const char *kol[5];
+	kol[0] = pal->nop;
+	kol[1] = pal->mov;
+	kol[2] = pal->cjmp;
+	kol[3] = pal->jmp;
+	kol[4] = pal->call;
+	if (cols < 1) {
+		cols = 1;
 	}
 	const bool show_colors = (p && (p->flags & R_PRINT_FLAGS_COLOR));
-	char *firebow[6];
+	if (show_colors) {
+		int idx = (int) ((k * 4) / cols);
+		const char *str = kol[idx];
+		if (p->histblock) {
+			p->cb_printf ("%s%s%s", str, block, Color_RESET);
+		} else {
+			p->cb_printf ("%s%s%s", str, h_line, Color_RESET);
+		}
+	} else {
+		if (p->histblock) {
+			p->cb_printf ("%s%s%s", Color_BGGRAY, block, Color_RESET);
+		} else {
+			p->cb_printf ("%s", h_line);
+		}
+	}
+}
+
+R_API void r_print_fill(RPrint *p, const ut8 *arr, int size, ut64 addr, int step) {
+	r_return_if_fail (p && arr);
+	const bool show_colors = (p && (p->flags & R_PRINT_FLAGS_COLOR));
+	bool useUtf8 = p->cons->use_utf8;
+	const char *v_line = useUtf8 ? RUNE_LINE_VERT : "|";
 	int i = 0, j;
 
-	for (i = 0; i < 6; i++) {
-		firebow[i] = p->cb_color (i, 6, true);
-	}
+
 #define INC 5
 #if TOPLINE
 	if (arr[0] > 1) {
@@ -1533,15 +1715,21 @@ R_API void r_print_fill(RPrint *p, const ut8 *arr, int size, ut64 addr, int step
 		}
 		if (arr[0] > 1) {
 			for (i = 0; i < arr[0]; i += INC) {
-				p->cb_printf ("_");
+				p->cb_printf (h_line);
 			}
 		}
 		p->cb_printf ("\n");
 	}
 #endif
+	// get the max of columns
+	int cols = 0;
+	for (i = 0; i < size; i++) {
+		cols = arr[i] > cols ? arr[i] : cols;
+	}
+	cols /= 5;
 	for (i = 0; i < size; i++) {
 		ut8 next = (i + 1 < size)? arr[i + 1]: 0;
-		int base = 0;
+		int base = 0, k = 0;
 		if (addr != UT64_MAX && step > 0) {
 			ut64 at = addr + (i * step);
 			if (p->cur_enabled) {
@@ -1556,59 +1744,44 @@ R_API void r_print_fill(RPrint *p, const ut8 *arr, int size, ut64 addr, int step
 			} else {
 				p->cb_printf ("0x%08" PFMT64x " ", at);
 			}
-			p->cb_printf ("%02x %04x |", i, arr[i]);
+			p->cb_printf ("%03x %04x %s", i, arr[i], v_line);
 		} else {
-			p->cb_printf ("|");
-		}
-		if (show_colors) {
-			int idx = (int) (arr[i] * 5 / 255);
-			const char *k = firebow[idx];
-			p->cb_printf ("%s", k);
+			p->cb_printf (v_line);
 		}
 		if (next < INC) {
 			base = 1;
 		}
 		if (next < arr[i]) {
-			//if (arr[i]>0 && i>0) p->cb_printf ("  ");
 			if (arr[i] > INC) {
 				for (j = 0; j < next + base; j += INC) {
-					p->cb_printf (i ? " " : "'");
+					printHistBlock (p, k, cols);
+					k++;
 				}
 			}
 			for (j = next + INC; j + base < arr[i]; j += INC) {
-				p->cb_printf ("_");
+				printHistBlock (p, k, cols);
+				k++;
 			}
 		} else {
-			if (i == 0) {
-				for (j = INC; j < arr[i] + base; j += INC) {
-					p->cb_printf ("'");
-				}
-			} else {
-				for (j = INC; j < arr[i] + base; j += INC) {
-					p->cb_printf (" ");
-				}
-			}
-		}
-		//for (j=1;j<arr[i]; j+=INC) p->cb_printf (under);
-		if (show_colors) {
-			p->cb_printf ("|" Color_RESET);
-		} else {
-			p->cb_printf ("|");
+			printHistBlock (p, k, cols);
+			k++;
 		}
 		if (i + 1 == size) {
 			for (j = arr[i] + INC + base; j + base < next; j += INC) {
-				p->cb_printf ("_");
+				printHistBlock (p, k, cols);
+				k++;
 			}
 		} else if (arr[i + 1] > arr[i]) {
 			for (j = arr[i] + INC + base; j + base < next; j += INC) {
-				p->cb_printf ("_");
+				printHistBlock (p, k, cols);
+				k++;
 			}
 		}
+		if (show_colors) {
+			p->cb_printf ("%s", Color_RESET);
+		}
 		p->cb_printf ("\n");
 	}
-	for (i = 0; i < 6; i++) {
-		free (firebow[i]);
-	}
 }
 
 R_API void r_print_2bpp_row(RPrint *p, ut8 *buf) {
@@ -1674,17 +1847,18 @@ R_API void r_print_2bpp_tiles(RPrint *p, ut8 *buf, ut32 tiles) {
 	}
 }
 
-R_API const char* r_print_color_op_type(RPrint *p, ut64 anal_type) {
+R_API const char* r_print_color_op_type(RPrint *p, ut32 anal_type) {
+	RConsPrintablePalette *pal = &p->cons->context->pal;
 	switch (anal_type & R_ANAL_OP_TYPE_MASK) {
 	case R_ANAL_OP_TYPE_NOP:
-		return p->cons->pal.nop;
+		return pal->nop;
 	case R_ANAL_OP_TYPE_ADD:
 	case R_ANAL_OP_TYPE_SUB:
 	case R_ANAL_OP_TYPE_MUL:
 	case R_ANAL_OP_TYPE_DIV:
 	case R_ANAL_OP_TYPE_MOD:
 	case R_ANAL_OP_TYPE_LENGTH:
-		return p->cons->pal.math;
+		return pal->math;
 	case R_ANAL_OP_TYPE_AND:
 	case R_ANAL_OP_TYPE_OR:
 	case R_ANAL_OP_TYPE_XOR:
@@ -1696,59 +1870,62 @@ R_API const char* r_print_color_op_type(RPrint *p, ut64 anal_type) {
 	case R_ANAL_OP_TYPE_ROL:
 	case R_ANAL_OP_TYPE_ROR:
 	case R_ANAL_OP_TYPE_CPL:
-		return p->cons->pal.bin;
+		return pal->bin;
 	case R_ANAL_OP_TYPE_IO:
-		return p->cons->pal.swi;
+		return pal->swi;
 	case R_ANAL_OP_TYPE_JMP:
 	case R_ANAL_OP_TYPE_UJMP:
+		return pal->ujmp;
 	case R_ANAL_OP_TYPE_IJMP:
 	case R_ANAL_OP_TYPE_RJMP:
 	case R_ANAL_OP_TYPE_IRJMP:
 	case R_ANAL_OP_TYPE_MJMP:
-		return p->cons->pal.jmp;
+		return pal->jmp;
 	case R_ANAL_OP_TYPE_CJMP:
 	case R_ANAL_OP_TYPE_UCJMP:
 	case R_ANAL_OP_TYPE_SWITCH:
-		return p->cons->pal.cjmp;
+		return pal->cjmp;
 	case R_ANAL_OP_TYPE_CMP:
 	case R_ANAL_OP_TYPE_ACMP:
-		return p->cons->pal.cmp;
+		return pal->cmp;
 	case R_ANAL_OP_TYPE_UCALL:
+		return pal->ucall;
 	case R_ANAL_OP_TYPE_ICALL:
 	case R_ANAL_OP_TYPE_RCALL:
 	case R_ANAL_OP_TYPE_IRCALL:
 	case R_ANAL_OP_TYPE_UCCALL:
 	case R_ANAL_OP_TYPE_CALL:
 	case R_ANAL_OP_TYPE_CCALL:
-		return p->cons->pal.call;
+		return pal->call;
 	case R_ANAL_OP_TYPE_NEW:
 	case R_ANAL_OP_TYPE_SWI:
-		return p->cons->pal.swi;
+		return pal->swi;
 	case R_ANAL_OP_TYPE_ILL:
 	case R_ANAL_OP_TYPE_TRAP:
-		return p->cons->pal.trap;
+		return pal->trap;
 	case R_ANAL_OP_TYPE_CRET:
 	case R_ANAL_OP_TYPE_RET:
-		return p->cons->pal.ret;
+		return pal->ret;
 	case R_ANAL_OP_TYPE_CAST:
 	case R_ANAL_OP_TYPE_MOV:
 	case R_ANAL_OP_TYPE_LEA:
 	case R_ANAL_OP_TYPE_CMOV: // TODO: add cmov cathegory?
-		return p->cons->pal.mov;
+		return pal->mov;
 	case R_ANAL_OP_TYPE_PUSH:
 	case R_ANAL_OP_TYPE_UPUSH:
+	case R_ANAL_OP_TYPE_RPUSH:
 	case R_ANAL_OP_TYPE_LOAD:
-		return p->cons->pal.push;
+		return pal->push;
 	case R_ANAL_OP_TYPE_POP:
 	case R_ANAL_OP_TYPE_STORE:
-		return p->cons->pal.pop;
+		return pal->pop;
 	case R_ANAL_OP_TYPE_CRYPTO:
-		return p->cons->pal.crypto;
+		return pal->crypto;
 	case R_ANAL_OP_TYPE_NULL:
-		return p->cons->pal.other;
+		return pal->other;
 	case R_ANAL_OP_TYPE_UNK:
 	default:
-		return p->cons->pal.invalid;
+		return pal->invalid;
 	}
 }
 
@@ -1805,6 +1982,7 @@ R_API char* r_print_colorize_opcode(RPrint *print, char *p, const char *reg, con
 	int is_jmp = p && (*p == 'j' || ((*p == 'c') && (p[1] == 'a')))? 1: 0;
 	ut32 opcode_sz = p && *p? strlen (p) * 10 + 1: 0;
 	char previous = '\0';
+	const char *color_flag = print->cons->context->pal.flag;
 
 	if (!p || !*p) {
 		return NULL;
@@ -1812,6 +1990,7 @@ R_API char* r_print_colorize_opcode(RPrint *print, char *p, const char *reg, con
 	if (is_jmp) {
 		return strdup (p);
 	}
+	r_str_trim_head_tail (p);
 	if (opcode_sz > COLORIZE_BUFSIZE) {
 		/* return same string in case of error */
 		return strdup (p);
@@ -1822,12 +2001,18 @@ R_API char* r_print_colorize_opcode(RPrint *print, char *p, const char *reg, con
 		/* colorize numbers */
 		if ((ishexprefix (&p[i]) && previous != ':') \
 		     || (isdigit ((ut8)p[i]) && issymbol (previous))) {
-			int nlen = strlen (num);
+			const char *num2 = num;
+			ut64 n = r_num_get (NULL, p + i);
+			const char *name = print->offname (print->user, n)? color_flag: NULL;
+			if (name) {
+				num2 = name;
+			}
+			int nlen = strlen (num2);
 			if (nlen + j >= sizeof (o)) {
 				eprintf ("Colorize buffer is too small\n");
 				break;
 			}
-			memcpy (o + j, num, nlen + 1);
+			memcpy (o + j, num2, nlen + 1);
 			j += nlen;
 		}
 		previous = p[i];
@@ -1881,7 +2066,7 @@ R_API char* r_print_colorize_opcode(RPrint *print, char *p, const char *reg, con
 				j += strlen (reset);
 				o[j] = p[i];
 				if (!(p[i+1] == '$' || ((p[i+1] > '0') && (p[i+1] < '9')))) {
-					const char *color = found_var ? print->cons->pal.func_var_type : reg;
+					const char *color = found_var ? print->cons->context->pal.func_var_type : reg;
 					ut32 color_len = strlen (color);
 					if (color_len + j + 10 >= COLORIZE_BUFSIZE) {
 						eprintf ("r_print_colorize_opcode(): buffer overflow!\n");
@@ -1933,20 +2118,13 @@ R_API char* r_print_colorize_opcode(RPrint *print, char *p, const char *reg, con
 		case '0': /* address */
 			if (p[i + 1] == 'x') {
 				if (print->flags & R_PRINT_FLAGS_SECSUB) {
-					RIOSection *s = NULL;
-					SdbList *secs = print->iob.sections_vget (print->iob.io, r_num_get (NULL, p + i));
-					if (secs) {
-						s = (RIOSection *)ls_pop (secs);
-						secs->free = NULL;
-						ls_free (secs);
-						secs = NULL;
-					}
-					if (s) {
-						if (strlen (s->name) + j + 1 >= COLORIZE_BUFSIZE) {
+					RIOMap *map = print->iob.map_get (print->iob.io, r_num_get (NULL, p + i));
+					if (map && map->name) {
+						if (strlen (map->name) + j + 1 >= COLORIZE_BUFSIZE) {
 							eprintf ("stop before overflow\n");
 							break;
 						}
-						strcpy (o + j, s->name);
+						strcpy (o + j, map->name);
 						j += strlen (o + j);
 						strcpy (o + j, ".");
 						j++;
@@ -1972,7 +2150,7 @@ R_API char* r_print_colorize_opcode(RPrint *print, char *p, const char *reg, con
 // reset the status of row_offsets
 R_API void r_print_init_rowoffsets(RPrint *p) {
 	if (p->calc_row_offsets) {
-		R_FREE(p->row_offsets);
+		R_FREE (p->row_offsets);
 		p->row_offsets_sz = 0;
 	}
 }
@@ -2016,7 +2194,6 @@ R_API ut32 r_print_rowoff(RPrint *p, int i) {
 R_API int r_print_row_at_off(RPrint *p, ut32 offset) {
 	int i = 0;
 	ut32 tt;
-
 	while ((tt = r_print_rowoff (p, i)) != UT32_MAX && tt <= offset) {
 		i++;
 	}
@@ -2076,7 +2253,7 @@ R_API void r_print_hex_from_bin (RPrint *p, char *bin_str) {
 		return;
 	}
 	ut64 n, *buf = malloc (sizeof (ut64) * ((len + 63) / 64));
-	if (buf == NULL) {
+	if (!buf) {
 		eprintf ("allocation failed\n");
 		return;
 	}
@@ -2107,13 +2284,14 @@ R_API void r_print_hex_from_bin (RPrint *p, char *bin_str) {
 R_API const char* r_print_rowlog(RPrint *print, const char *str) {
 	int use_color = print->flags & R_PRINT_FLAGS_COLOR;
 	bool verbose = print->scr_prompt;
+	r_return_val_if_fail (print->cb_eprintf, NULL);
 	if (!verbose) {
 		return NULL;
 	}
 	if (use_color) {
-		eprintf ("[ ] "Color_YELLOW"%s\r["Color_RESET, str);
+		print->cb_eprintf ("[ ] "Color_YELLOW"%s\r["Color_RESET, str);
 	} else {
-		eprintf ("[ ] %s\r[", str);
+		print->cb_eprintf ("[ ] %s\r[", str);
 	}
 	return str;
 }
@@ -2121,11 +2299,12 @@ R_API const char* r_print_rowlog(RPrint *print, const char *str) {
 R_API void r_print_rowlog_done(RPrint *print, const char *str) {
 	int use_color = print->flags & R_PRINT_FLAGS_COLOR;
 	bool verbose =  print->scr_prompt;
+	r_return_if_fail (print->cb_eprintf);
 	if (verbose) {
 		if (use_color) {
-			eprintf ("\r"Color_GREEN"[x]"Color_RESET" %s\n", str);
+			print->cb_eprintf ("\r"Color_GREEN"[x]"Color_RESET" %s\n", str);
 		} else {
-			eprintf ("\r[x] %s\n", str);
+			print->cb_eprintf ("\r[x] %s\n", str);
 		}
 	}
 }
