@@ -27,9 +27,9 @@ R_API RStack *r_stack_newf(ut32 n, RStackFree f) {
 
 R_API void r_stack_free(RStack *s) {
 	if (s) {
-		if (s->free && s->top > 0) {
-			int i = 0;
-			for (i = 0; i < s->top; i++) {
+		if (s->free) {
+			int i;
+			for (i = 0; i <= s->top; i++) {
 				s->free (s->elems[i]);
 			}
 		}
@@ -43,8 +43,9 @@ R_API int r_stack_push(RStack *s, void *el) {
 		/* reallocate the stack */
 		s->n_elems *= 2;
 		s->elems = realloc (s->elems, s->n_elems * sizeof (void *));
-		if (!s->elems)
+		if (!s->elems) {
 			return false;
+		}
 	}
 
 	s->top++;
