@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2013-2018 - pancake */
+/* radare - LGPL - Copyright 2013-2019 - pancake */
 
 #include "r_types.h"
 #include "r_util.h"
@@ -13,12 +13,17 @@ R_API RStrBuf *r_strbuf_new(const char *str) {
 }
 
 R_API bool r_strbuf_equals(RStrBuf *sa, RStrBuf *sb) {
-	if (!sa || !sb || sa->len != sb->len) { // faster comparisons
+	r_return_val_if_fail (sa && sb, false);
+	if (sa->len != sb->len) {
 		return false;
 	}
 	return strcmp (r_strbuf_get (sa), r_strbuf_get (sb)) == 0;
 }
 
+R_API bool r_strbuf_is_empty(RStrBuf *sb) {
+	return sb->len == 0;
+}
+
 R_API int r_strbuf_length(RStrBuf *sb) {
 	r_return_val_if_fail (sb, 0);
 	return sb->len;
@@ -29,6 +34,43 @@ R_API void r_strbuf_init(RStrBuf *sb) {
 	memset (sb, 0, sizeof (RStrBuf));
 }
 
+R_API bool r_strbuf_copy(RStrBuf *dst, RStrBuf *src) {
+	r_return_val_if_fail (dst && src, false);
+	if (src->ptr) {
+		char *p = malloc (src->ptrlen);
+		if (!p) {
+			return false;
+		}
+		memcpy (p, src->ptr, src->ptrlen);
+		free (dst->ptr);
+		dst->ptr = p;
+		dst->ptrlen = src->ptrlen;
+	} else {
+		R_FREE (dst->ptr);
+		memcpy (dst->buf, src->buf, sizeof (dst->buf));
+	}
+	dst->len = src->len;
+	return true;
+}
+
+R_API bool r_strbuf_reserve(RStrBuf *sb, int len) {
+	r_return_val_if_fail (sb && len > 0, false);
+
+	if ((sb->ptr && len < sb->ptrlen) || (!sb->ptr && len < sizeof (sb->buf))) {
+		return true;
+	}
+	char *newptr = realloc (sb->ptr, len + 1);
+	if (!newptr) {
+		return false;
+	}
+	if (!sb->ptr) {
+		memcpy (newptr, sb->buf, sizeof (sb->buf));
+	}
+	sb->ptr = newptr;
+	sb->ptrlen = len + 1;
+	return true;
+}
+
 R_API bool r_strbuf_setbin(RStrBuf *sb, const ut8 *s, int l) {
 	r_return_val_if_fail (sb && s, false);
 	r_return_val_if_fail (l >= 0, false);
@@ -45,9 +87,9 @@ R_API bool r_strbuf_setbin(RStrBuf *sb, const ut8 *s, int l) {
 			sb->ptr = ptr;
 		}
 		memcpy (ptr, s, l);
-		*(ptr + l) = 0;
+		ptr[l] = 0;
 	} else {
-		sb->ptr = NULL;
+		R_FREE (sb->ptr);
 		memcpy (sb->buf, s, l);
 		sb->buf[l] = 0;
 	}
@@ -55,6 +97,28 @@ R_API bool r_strbuf_setbin(RStrBuf *sb, const ut8 *s, int l) {
 	return true;
 }
 
+// TODO: there's room for optimizations here
+R_API bool r_strbuf_slice(RStrBuf *sb, int from, int len) {
+	r_return_val_if_fail (sb && from >= 0 && len >= 0, false);
+	if (from < 1 && len >= sb->len) {
+		return false;
+	}
+	const char *s = r_strbuf_get (sb);
+	const char *fr = r_str_ansi_chrn (s, from + 1);
+	const char *to = r_str_ansi_chrn (s, from + len + 1);
+	char *r = r_str_newlen (fr, to - fr);
+	r_strbuf_fini (sb);
+	r_strbuf_init (sb);
+	if (from >= len) {
+		r_strbuf_set (sb, "");
+		free (r);
+		return false;
+	}
+	r_strbuf_set (sb, r);
+	free (r);
+	return true;
+}
+
 R_API bool r_strbuf_set(RStrBuf *sb, const char *s) {
 	r_return_val_if_fail (sb, false);
 
@@ -106,6 +170,27 @@ done:
 	return ret;
 }
 
+R_API bool r_strbuf_prepend(RStrBuf *sb, const char *s) {
+	r_return_val_if_fail (sb && s, false);
+	int l = strlen (s);
+	// fast path if no chars to append
+	if (l == 0) {
+		return true;
+	}
+	int newlen = l + sb->len;
+	char *ns = malloc (newlen + 1);
+	bool ret = false;
+	if (ns) {
+		memcpy (ns, s, l);
+		char *s = sb->ptr ? sb->ptr: sb->buf;
+		memcpy (ns + l, s, sb->len);
+		ns[newlen] = 0;
+		ret = r_strbuf_set (sb, ns);
+		free (ns);
+	}
+	return ret;
+}
+
 R_API bool r_strbuf_append(RStrBuf *sb, const char *s) {
 	r_return_val_if_fail (sb && s, false);
 
