@@ -2,7 +2,13 @@
 
 #include <r_th.h>
 
-#if __WINDOWS__ && !defined(__CYGWIN__)
+#if __APPLE__
+// Here to avoid polluting mach types macro redefinitions...
+#include <mach/thread_act.h>
+#include <mach/thread_policy.h>
+#endif
+
+#if __WINDOWS__
 static DWORD WINAPI _r_th_launcher(void *_th) {
 #else
 static void *_r_th_launcher(void *_th) {
@@ -50,28 +56,116 @@ R_API R_TH_TID r_th_self(void) {
 #if HAVE_PTHREAD
 	return pthread_self ();
 #elif __WINDOWS__
-	return (HANDLE)GetCurrentThreadId ();
+	return GetCurrentThread ();
 #else
-#pragma message("Not implemented on windows")
+#pragma message("Not implemented on this platform")
 	return (R_TH_TID)-1;
 #endif
 }
 
+R_API bool r_th_setname(RThread *th, const char *name) {
+#if defined(HAVE_PTHREAD_NP) && HAVE_PTHREAD_NP
+#if __linux__
+	if (pthread_setname_np (th->tid, name) != 0) {
+		eprintf ("Failed to set thread name\n");
+		return false;
+	}	
+#elif __APPLE__
+	if (pthread_setname_np (name) != 0) {
+		eprintf ("Failed to set thread name\n");
+		return false;
+	}
+#elif __FreeBSD__ || __OpenBSD__ || __DragonFly__
+	pthread_set_name_np (th->tid, name);
+#elif __NetBSD__
+	if (pthread_setname_np (th->tid, "%s", (void *)name) != 0) {
+		eprintf ("Failed to set thread name\n");
+		return false;
+	}	
+#else
+#pragma message("warning r_th_setname not implemented")
+#endif
+#endif
+	return true;
+}
+
+R_API bool r_th_getname(RThread *th, char *name, size_t len) {
+#if defined(HAVE_PTHREAD_NP) && HAVE_PTHREAD_NP
+#if __linux__ || __NetBSD__ || __APPLE__
+	if (pthread_getname_np (th->tid, name, len) != 0) {
+		eprintf ("Failed to get thread name\n");
+		return false;
+	}
+#elif (__FreeBSD__ &&  __FreeBSD_version >= 1200000) || __DragonFly__  || (__OpenBSD__ && OpenBSD >= 201905)
+	pthread_get_name_np (th->tid, name, len);
+#else
+#pragma message("warning r_th_getname not implemented")
+#endif
+#endif
+	return true;
+}
+
+R_API bool r_th_setaffinity(RThread *th, int cpuid) {
+#if __linux__
+	cpu_set_t c;
+	CPU_ZERO(&c);
+	CPU_SET(cpuid, &c);
+
+	if (sched_setaffinity (th->tid, sizeof (c), &c) != 0) {
+		eprintf ("Failed to set cpu affinity\n");
+		return false;
+	}
+#elif __FreeBSD__ || __DragonFly__
+	cpuset_t c;
+	CPU_ZERO(&c);
+	CPU_SET(cpuid, &c);
+
+	if (pthread_setaffinity_np (th->tid, sizeof (c), &c) != 0) {
+		eprintf ("Failed to set cpu affinity\n");
+		return false;
+	}
+#elif __NetBSD__
+	cpuset_t *c;
+	c = cpuset_create ();
+
+	if (pthread_setaffinity_np (th->tid, cpuset_size(c), c) != 0) {
+		cpuset_destroy (c);
+		eprintf ("Failed to set cpu affinity\n");
+		return false;
+	}
+
+	cpuset_destroy (c);
+#elif __APPLE__
+	thread_affinity_policy_data_t c = {cpuid};
+	if (thread_policy_set (pthread_mach_thread_np (th->tid),
+		THREAD_AFFINITY_POLICY, (thread_policy_t)&c, 1) != KERN_SUCCESS) {
+		eprintf ("Failed to set cpu affinity\n");
+		return false;
+	}
+#elif __WINDOWS__
+	if (SetThreadAffinityMask (th->tid, (DWORD_PTR)1 << cpuid) == 0) {
+		eprintf ("Failed to set cpu affinity\n");
+		return false;
+	}
+#else
+#pragma message("warning r_th_setaffinity not implemented")
+#endif
+	return true;
+}
+
 R_API RThread *r_th_new(R_TH_FUNCTION(fun), void *user, int delay) {
 	RThread *th = R_NEW0 (RThread);
 	if (th) {
 		th->lock = r_th_lock_new (false);
 		th->running = false;
-		th->fun = fun;	
+		th->fun = fun;
 		th->user = user;
 		th->delay = delay;
 		th->breaked = false;
 		th->ready = false;
 #if HAVE_PTHREAD
-		pthread_cond_init (&th->_cond, NULL);
-		pthread_mutex_init (&th->_mutex, NULL);
 		pthread_create (&th->tid, NULL, _r_th_launcher, th);
-#elif __WINDOWS__ && !defined(__CYGWIN__)
+#elif __WINDOWS__
 		th->tid = CreateThread (NULL, 0, _r_th_launcher, th, 0, 0);
 #endif
 	}
@@ -95,54 +189,12 @@ R_API bool r_th_kill(RThread *th, bool force) {
 #else
 	pthread_cancel (th->tid);
 #endif
-#elif __WINDOWS__ && !defined(__CYGWIN__)
+#elif __WINDOWS__
 	TerminateThread (th->tid, -1);
 #endif
 	return 0;
 }
 
-// running in parent
-R_API bool r_th_pause(RThread *th, bool enable) {
-	if (!th) {
-		return false;
-	}
-	if (enable) {
-#if HAVE_PTHREAD
-		pthread_mutex_trylock (&th->_mutex);
-#else
-#pragma message("warning r_th_pause not implemented")
-#endif
-	} else {
-#if HAVE_PTHREAD
-		// pthread_cond_signal (&th->_cond);
-		pthread_mutex_unlock (&th->_mutex);
-#else
-#pragma message("warning r_th_pause not implemented")
-#endif
-	}
-	return true;
-}
-
-// running in thread
-R_API bool r_th_try_pause(RThread *th) {
-	if (!th) {
-		return false;
-	}
-#if HAVE_PTHREAD
-	// pthread_mutex_lock (&th->_mutex);
-	// pthread_mutex_unlock (&th->_mutex);
-	if (pthread_mutex_trylock (&th->_mutex) != -1) {
-		pthread_mutex_unlock (&th->_mutex);
-	} else {
-		// oops
-	}
-	// pthread_cond_wait (&th->_cond, &th->_mutex);
-#else
-#pragma message("warning r_th_try_pause not implemented")
-#endif
-	return true;
-}
-
 R_API bool r_th_start(RThread *th, int enable) {
 	bool ret = true;
 	if (enable) {
@@ -170,7 +222,7 @@ R_API int r_th_wait(struct r_th_t *th) {
 	if (th) {
 #if HAVE_PTHREAD
 		ret = pthread_join (th->tid, &thret);
-#elif __WINDOWS__ && !defined(__CYGWIN__)
+#elif __WINDOWS__
 		ret = WaitForSingleObject (th->tid, INFINITE);
 #endif
 		th->running = false;
@@ -186,7 +238,7 @@ R_API void *r_th_free(struct r_th_t *th) {
 	if (!th) {
 		return NULL;
 	}
-#if __WINDOWS__ && !defined(__CYGWIN__)
+#if __WINDOWS__
 	CloseHandle (th->tid);
 #endif
 	r_th_lock_free (th->lock);
