@@ -1,15 +1,17 @@
-/* radare - LGPL - Copyright 2007-2018 - pancake */
+/* radare - LGPL - Copyright 2007-2019 - pancake */
 
-#if __WINDOWS__ && MINGW32 && !__CYGWIN__
+#if __WINDOWS__
 #include <stdlib.h>
 #endif
 
+#include <math.h>  /* for ceill */
 #include <r_util.h>
 #define R_NUM_USE_CALC 1
 
 static ut64 r_num_tailff(RNum *num, const char *hex);
 
-void r_srand (int seed) {
+// TODO: rename to r_num_srand()
+static void r_srand(int seed) {
 #if HAVE_ARC4RANDOM_UNIFORM
 	// no-op
 	(void)seed;
@@ -18,11 +20,11 @@ void r_srand (int seed) {
 #endif
 }
 
-int r_rand (int mod) {
+static int r_rand(int mod) {
 #if HAVE_ARC4RANDOM_UNIFORM
 	return (int)arc4random_uniform (mod);
 #else
-	return rand ()%mod;
+	return rand () % mod;
 #endif
 }
 
@@ -30,11 +32,11 @@ R_API void r_num_irand() {
 	r_srand (r_sys_now ());
 }
 
-static int rand_initialized = 0;
 R_API int r_num_rand(int max) {
+	static bool rand_initialized = false;
 	if (!rand_initialized) {
 		r_num_irand ();
-		rand_initialized = 1;
+		rand_initialized = true;
 	}
 	if (!max) {
 		max = 1;
@@ -51,7 +53,7 @@ R_API void r_num_minmax_swap(ut64 *a, ut64 *b) {
 }
 
 R_API void r_num_minmax_swap_i(int *a, int *b) {
-	if (*a>*b) {
+	if (*a > *b) {
 		ut64 tmp = *a;
 		*a = *b;
 		*b = tmp;
@@ -74,32 +76,46 @@ R_API void r_num_free(RNum *num) {
 	R_FREE (num);
 }
 
-#define KB (1024)
-#define MB (1024*KB)
-#define GB (1024*MB)
-#define TB (1024*GB)
+#define KB (1ULL << 10)
+#define MB (1ULL << 20)
+#define GB (1ULL << 30)
+#define TB (1ULL << 40)
+#define PB (1ULL << 50)
+#define EB (1ULL << 60)
 
-R_API char *r_num_units(char *buf, ut64 num) {
+/**
+ * Convert size in bytes to human-readable string
+ *
+ * Result is stored in buf (buf should be at least 8 bytes in size).
+ * If buf is NULL, memory for the new string is obtained with malloc(3),
+ * and can be freed with free(3).
+ *
+ * On success, returns a pointer to buf. It returns NULL if
+ * insufficient memory was available.
+ */
+R_API char *r_num_units(char *buf, size_t len, ut64 num) {
+	long double fnum;
 	char unit;
-	int tnum;
-	double fnum = num;
+	const char *fmt_str;
 	if (!buf) {
-		buf = malloc (32);
+		buf = malloc (len + 1);
 		if (!buf) {
 			return NULL;
 		}
 	}
-	//if (num>=TB) { unit = 'T'; fnum = num/TB; } else
-	if (num>=GB) { unit = 'G'; fnum = fnum/GB; } else
-	if (num>=MB) { unit = 'M'; fnum = fnum/MB; } else
-	if (num>=KB) { unit = 'K'; fnum = fnum/KB; } else
-		{ unit = 0; fnum = (double)num; }
-	tnum = (int)((double)(fnum - (int)fnum)*10);
-	if (tnum) {
-		snprintf (buf, 31, "%.1f%c", fnum, unit);
-	} else {
-		snprintf (buf, 31, "%.0f%c", fnum, unit);
+	fnum = num;
+	if (num >= EB) { unit = 'E'; fnum /= EB; } else
+	if (num >= PB) { unit = 'P'; fnum /= PB; } else
+	if (num >= TB) { unit = 'T'; fnum /= TB; } else
+	if (num >= GB) { unit = 'G'; fnum /= GB; } else
+	if (num >= MB) { unit = 'M'; fnum /= MB; } else
+	if (num >= KB) { unit = 'K'; fnum /= KB; } else {
+		unit = '\0';
 	}
+	fmt_str = ((double)ceill (fnum) == (double)fnum)
+		? "%.0" LDBLFMT "%c"
+		: "%.1" LDBLFMT "%c";
+	snprintf (buf, len, fmt_str, fnum, unit);
 	return buf;
 }
 
@@ -140,12 +156,23 @@ R_API ut64 r_num_get(RNum *num, const char *str) {
 	if (!str) {
 		return 0;
 	}
-	for (; *str==' '; ) {
+	for (; *str == ' '; ) {
 		str++;
 	}
 	if (!*str) {
 		return 0;
 	}
+	if (!strncmp (str, "1u", 2)) { // '1' is captured by op :(
+		if (num && num->value == UT64_MAX) {
+			num->value = 0;
+		}
+		switch (atoi (str + 2)) {
+		case 64: return (ut64)UT64_MAX;
+		case 32: return (ut64)UT32_MAX;
+		case 16: return (ut64)UT16_MAX;
+		case 8: return (ut64)UT8_MAX;
+		}
+	}
 	/* resolve string with an external callback */
 	if (num && num->callback) {
 		ok = 0;
@@ -156,7 +183,7 @@ R_API ut64 r_num_get(RNum *num, const char *str) {
 	}
 
 	if (str[0] && str[1] && str[2]) {
-		if (str[0]=='\'' && str[2]=='\'') {
+		if (str[0] == '\'' && str[2] == '\'') {
 			return (ut64)str[1];
 		}
 	}
@@ -164,11 +191,11 @@ R_API ut64 r_num_get(RNum *num, const char *str) {
 	len = strlen (str);
 	if (len > 3 && str[4] == ':') {
 		if (sscanf (str, "%04x", &s) == 1) {
-			if (sscanf (str + 5, "%04x", &a)==1) {
+			if (sscanf (str + 5, "%04x", &a) == 1) {
 				return (ut64) ((s<<4) + a);
 			}
 		}
-	} else if (len>6 && str[6] == ':') {
+	} else if (len > 6 && str[6] == ':') {
 		if (sscanf (str, "0x%04x:0x%04x", &s, &a) == 2) {
 			return (ut64) ((s << 4) + a);
 		}
@@ -190,19 +217,25 @@ R_API ut64 r_num_get(RNum *num, const char *str) {
 		ret = str[1] & 0xff;
 	// ugly as hell
 	} else if (!strncmp (str, "0xff..", 6) || !strncmp (str, "0xFF..", 6)) {
-        	ret = r_num_tailff (num, str + 5);
+		ret = r_num_tailff (num, str + 6);
 	// ugly as hell
+	} else if (!strncmp (str, "0o", 2)) {
+		if (sscanf (str + 2, "%"PFMT64o, &ret) != 1) {
+			error (num, "invalid octal number");
+		}
 	} else if (!strncmp (str, "0xf..", 5) || !strncmp (str, "0xF..", 5)) {
-        	ret = r_num_tailff (num, str + 5);
-	} else if (str[0] == '0' && str[1] == 'x') {
+		ret = r_num_tailff (num, str + 5);
+	} else if (str[0] == '0' && tolower (str[1]) == 'x') {
 		const char *lodash = strchr (str + 2, '_');
 		if (lodash) {
 			// Support 0x1000_f000_4000
 			// TODO: Only take underscores separated every 4 chars starting at the end
 			char *s = strdup (str + 2);
-			r_str_replace_char (s, '_', 0);
-			ret = strtoull (s, NULL, 16);
-			free (s);
+			if (s) {
+				r_str_replace_char (s, '_', 0);
+				ret = strtoull (s, NULL, 16);
+				free (s);
+			}
 		} else {
 			ret = strtoull (str + 2, NULL, 16);
 			// sscanf (str+2, "%"PFMT64x, &ret);
@@ -233,13 +266,19 @@ R_API ut64 r_num_get(RNum *num, const char *str) {
 		case 'b': // binary
 			ret = 0;
 			ok = true;
-			for (j = 0, i = strlen (str) - 2; i >= 0; i--, j++) {
-				if (str[i] == '1') {
-					ret|=1 << j;
-				} else if (str[i] != '0') {
-					ok = false;
-					break;
+			if (strlen (str) <= 65) { // 64 bit + the 'b' suffix
+				for (j = 0, i = strlen (str) - 2; i >= 0; i--, j++) {
+					if (str[i] == '1') {
+						ret |= (1ULL << j);
+					} else if (str[i] != '0') {
+						// eprintf ("Unexpected char in binary number string '%c'\n", str[i]);
+						ok = false;
+						break;
+					}
 				}
+			} else {
+				ok = false;
+				// eprintf ("Binary number is too large to fit in ut64\n");
 			}
 			if (!ok || !len_num) {
 				error (num, "invalid binary number");
@@ -320,9 +359,9 @@ R_API ut64 r_num_get(RNum *num, const char *str) {
 			break;
 		default:
 #if 0
-			//sscanf (str, "%"PFMT64d"%n", &ret, &chars_read);
+			// sscanf (str, "%"PFMT64d"%n", &ret, &chars_read);
 // 32bit chop
-#if __WINDOWS__ && MINGW32 && !__CYGWIN__
+#if __WINDOWS__
 			ret = _strtoui64 (str, &endptr, 10);
 #endif
 #endif
@@ -342,15 +381,15 @@ R_API ut64 r_num_get(RNum *num, const char *str) {
 #if !R_NUM_USE_CALC
 static ut64 r_num_op(RNum *num, char op, ut64 a, ut64 b) {
 	switch (op) {
-	case '+': return a+b;
-	case '-': return a-b;
-	case '*': return a*b;
+	case '+': return a + b;
+	case '-': return a - b;
+	case '*': return a * b;
 	case '/':
 		if (!b && num) num->dbz = 1;
-		return b?a/b:0;
-	case '&': return a&b;
-	case '|': return a|b;
-	case '^': return a^b;
+		return b ? a / b : 0;
+	case '&': return a & b;
+	case '|': return a | b;
+	case '^': return a ^ b;
 	}
 	return b;
 }
@@ -369,22 +408,22 @@ R_API static ut64 r_num_math_internal(RNum *num, char *s) {
 	}
 	return r_num_op (op, ret, r_num_get (num, p));
 }
-#endif
+#endif /* !R_NUM_USE_CALC */
 
 R_API ut64 r_num_math(RNum *num, const char *str) {
 #if R_NUM_USE_CALC
 	ut64 ret;
 	const char *err = NULL;
-	if (!str) return 0LL;
-	//if (!str || !*str) return 0LL;
+	if (!str || !*str) {
+		return 0LL;
+	}
+	// if (!str || !*str) return 0LL;
 	if (num) {
 		num->dbz = 0;
 	}
 	ret = r_num_calc (num, str, &err);
 	if (err) {
 		eprintf ("r_num_calc error: (%s) in (%s)\n", err, str);
-	} else if (num) {
-		num->value = ret;
 	}
 	if (num) {
 		num->value = ret;
@@ -398,12 +437,12 @@ R_API ut64 r_num_math(RNum *num, const char *str) {
 	char *group;
 	if (!str) return 0LL;
 
-	len = strlen (str)+1;
-	os = malloc (len+1);
+	len = strlen (str) + 1;
+	os = malloc (len + 1);
 
 	s = os;
 	memcpy (s, str, len);
-	for (; *s==' '; s++);
+	for (; *s == ' '; s++);
 	p = s;
 
 	do {
@@ -411,21 +450,21 @@ R_API ut64 r_num_math(RNum *num, const char *str) {
 		if (group) {
 			group[0] = '\0';
 			ret = r_num_op (op, ret, r_num_math_internal (num, p));
-			for (; p<group; p+=1) {
+			for (; p<group; p += 1) {
 				if (r_num_is_op (*p)) {
 					op = *p;
 					break;
 				}
 			}
 			group[0] = '(';
-			p = group+1;
+			p = group + 1;
 			if (r_str_delta (p, '(', ')') < 0) {
 				char *p2 = strchr (p, '(');
 				if (p2 != NULL) {
 					*p2 = '\0';
 					ret = r_num_op (op, ret, r_num_math_internal (num, p));
-					ret = r_num_op (op, ret, r_num_math (num, p2+1));
-					p = p2+1;
+					ret = r_num_op (op, ret, r_num_math (num, p2 + 1));
+					p = p2 + 1;
 					continue;
 				}
 				eprintf ("WTF!\n");
@@ -455,37 +494,43 @@ R_API double r_num_get_float(RNum *num, const char *str) {
 	return d;
 }
 
-R_API int r_num_to_bits (char *out, ut64 num) {
+R_API int r_num_to_bits(char *out, ut64 num) {
 	int size = 64, i;
 
-	if (num>>32) size = 64;
-	else if (num&0xff000000) size = 32;
-	else if (num&0xff0000) size = 24;
-	else if (num&0xff00) size = 16;
-	else if (num&0xff) size = 8;
+	if (num >> 32) {
+		size = 64;
+	} else if (num & 0xff000000) {
+		size = 32;
+	} else if (num & 0xff0000) {
+		size = 24;
+	} else if (num & 0xff00) {
+		size = 16;
+	} else if (num & 0xff) {
+		size = 8;
+	}
 	if (out) {
 		int pos = 0;
 		int realsize = 0;
 		int hasbit = 0;
-		for (i=0; i<size; i++) {
-			char bit = ((num>>(size-i-1))&1)? '1': '0';
-			if (hasbit || bit=='1') {
-				out[pos++] = bit;//size-1-i] = bit;
+		for (i = 0; i < size; i++) {
+			char bit = ((num >> (size - i - 1)) & 1) ? '1': '0';
+			if (hasbit || bit == '1') {
+				out[pos++] = bit; // size - 1 - i] = bit;
 			}
-			if (!hasbit && bit=='1') {
-				hasbit=1;
-				realsize = size-i;
+			if (!hasbit && bit == '1') {
+				hasbit = 1;
+				realsize = size - i;
 			}
 		}
-		if (realsize==0)
-		out[realsize++] = '0';
-		out[realsize] = '\0'; //Maybe not nesesary?
-
+		if (realsize == 0) {
+			out[realsize++] = '0';
+		}
+		out[realsize] = '\0'; // Maybe not nesesary?
 	}
 	return size;
 }
 
-R_API int r_num_to_trits (char *out, ut64 num) {
+R_API int r_num_to_trits(char *out, ut64 num) {
 	if (out == NULL) {
 		return false;
 	}
@@ -503,8 +548,10 @@ R_API int r_num_to_trits (char *out, ut64 num) {
 	return true;
 }
 
-R_API ut64 r_num_chs (int cylinder, int head, int sector, int sectorsize) {
-	if (sectorsize<1) sectorsize = 512;
+R_API ut64 r_num_chs(int cylinder, int head, int sector, int sectorsize) {
+	if (sectorsize < 1) {
+		sectorsize = 512;
+	}
 	return (ut64)cylinder * (ut64)head * (ut64)sector * (ut64)sectorsize;
 }
 
@@ -515,29 +562,39 @@ R_API int r_num_conditional(RNum *num, const char *str) {
 	p = s;
 	do {
 		t = strchr (p, ',');
-		if (t) *t = 0;
+		if (t) {
+			*t = 0;
+		}
 		lgt = strchr (p, '<');
 		if (lgt) {
 			*lgt = 0;
 			a = r_num_math (num, p);
-			if (lgt[1]=='=') {
-				b = r_num_math (num, lgt+2);
-				if (a>b) goto fail;
+			if (lgt[1] == '=') {
+				b = r_num_math (num, lgt + 2);
+				if (a > b) {
+					goto fail;
+				}
 			} else {
-				b = r_num_math (num, lgt+1);
-				if (a>=b) goto fail;
+				b = r_num_math (num, lgt + 1);
+				if (a >= b) {
+					goto fail;
+				}
 			}
 		} else {
 			lgt = strchr (p, '>');
 			if (lgt) {
 				*lgt = 0;
 				a = r_num_math (num, p);
-				if (lgt[1]=='=') {
-					b = r_num_math (num, lgt+2);
-					if (a<b) goto fail;
+				if (lgt[1] == '=') {
+					b = r_num_math (num, lgt + 2);
+					if (a < b) {
+						goto fail;
+					}
 				} else {
-					b = r_num_math (num, lgt+1);
-					if (a<=b) goto fail;
+					b = r_num_math (num, lgt + 1);
+					if (a <= b) {
+						goto fail;
+					}
 				}
 			} else {
 				lgt = strchr (p, '=');
@@ -547,17 +604,23 @@ R_API int r_num_conditional(RNum *num, const char *str) {
 						r_str_replace_char (p, '!', ' ');
 						r_str_replace_char (p, '=', '-');
 						n = r_num_math (num, p);
-						if (!n) goto fail;
+						if (!n) {
+							goto fail;
+						}
 					}
 				}
 				lgt = strstr (p, "==");
-				if (lgt) *lgt = ' ';
+				if (lgt) {
+					*lgt = ' ';
+				}
 				r_str_replace_char (p, '=', '-');
 				n = r_num_math (num, p);
-				if (n) goto fail;
+				if (n) {
+					goto fail;
+				}
 			}
 		}
-		p = t+1;
+		p = t + 1;
 	} while (t);
 	res = 1;
 fail:
@@ -608,36 +671,60 @@ R_API char* r_num_as_string(RNum *___, ut64 n, bool printable_only) {
 		} else if (!printable_only && (off = escape_char (str + stri, ch)) != 0) {
 			stri += off;
 		} else {
-			if (ch)
+			if (ch) {
 				return NULL;
+			}
 		}
 		ret |= (num & 0xff);
 		num >>= 8;
 	}
 	if (ret) {
 		return strdup (str);
-	} else if (!printable_only) {
+	}
+	if (!printable_only) {
 		return strdup ("\\0");
 	}
 	return NULL;
 }
 
+// SHITTY API
 R_API int r_is_valid_input_num_value(RNum *num, const char *input_value){
 	ut64 value = input_value ? r_num_math (num, input_value) : 0;
 	return !(value == 0 && input_value && *input_value == '0');
 }
 
-R_API ut64 r_get_input_num_value(RNum *num, const char *input_value){
-	ut64 value = input_value ? r_num_math (num, input_value) : 0;
-	return value;
+// SHITTY API
+R_API ut64 r_get_input_num_value(RNum *num, const char *str) {
+	return (str && *str)? r_num_math (num, str) : 0;
 }
 
-static bool isHexDigit (const char _ch) {
-	const char ch = tolower (_ch);
-	if (IS_DIGIT (ch)) {
-		return true;
+// SHITTY API
+static inline ut64 __nth_nibble(ut64 n, ut32 i) {
+	int sz = (sizeof (n) << 1) - 1;
+	int s = (sz - i) * 4;
+	return (n >> s) & 0xf;
+}
+
+R_API ut64 r_num_tail_base(RNum *num, ut64 addr, ut64 off) {
+	int i;
+	bool ready = false;
+	ut64 res = 0;
+	for (i = 0; i < 16; i++) {
+		ut64 o = __nth_nibble (off, i);
+		if (!ready) {
+			bool iseq = __nth_nibble (addr, i) == o;
+			if (i == 0 && !iseq) {
+				return UT64_MAX;
+			}
+			if (iseq) {
+				continue;
+			}
+		}
+		ready = true;
+		ut8 pos = (15 - i) * 4;
+		res |= (o << pos);
 	}
-	return (ch >= 'a' && ch <= 'f');
+	return res;
 }
 
 R_API ut64 r_num_tail(RNum *num, ut64 addr, const char *hex) {
@@ -646,7 +733,7 @@ R_API ut64 r_num_tail(RNum *num, ut64 addr, const char *hex) {
 	char *p;
 	int i;
 
-	while (*hex && (*hex == ' ' || *hex=='.')) {
+	while (*hex && (*hex == ' ' || *hex == '.')) {
 		hex++;
 	}
 	i = strlen (hex) * 4;
@@ -654,10 +741,11 @@ R_API ut64 r_num_tail(RNum *num, ut64 addr, const char *hex) {
 	if (p) {
 		strcpy (p, "0x");
 		strcpy (p + 2, hex);
-		if (isHexDigit (hex[0])) {
+		if (isxdigit ((ut8)hex[0])) {
 			n = r_num_math (num, p);
 		} else {
 			eprintf ("Invalid argument\n");
+			free (p);
 			return addr;
 		}
 		free (p);
@@ -667,29 +755,27 @@ R_API ut64 r_num_tail(RNum *num, ut64 addr, const char *hex) {
 }
 
 static ut64 r_num_tailff(RNum *num, const char *hex) {
-        ut64 n = 0;
-        char *p;
-        int i;
+	ut64 n = 0;
 
-        while (*hex && (*hex == ' ' || *hex=='.')) {
-                hex++;
-        }
-        i = strlen (hex) * 4;
-        p = malloc (strlen (hex) + 10);
-        if (p) {
-                strcpy (p, "0x");
-                strcpy (p + 2, hex);
-                if (isHexDigit (hex[0])) {
-                        n = r_num_math (num, p);
-                } else {
-                        eprintf ("Invalid argument\n");
+	while (*hex && (*hex == ' ' || *hex == '.')) {
+		hex++;
+	}
+	int i = strlen (hex) * 4;
+	char *p = malloc (strlen (hex) + 10);
+	if (p) {
+		strcpy (p, "0x");
+		strcpy (p + 2, hex);
+		if (isxdigit ((ut8)hex[0])) {
+			n = r_num_get (num, p);
+		} else {
+			eprintf ("Invalid argument\n");
 			free (p);
-                        return UT64_MAX;
-                }
-                free (p);
-        }
+			return UT64_MAX;
+		}
+		free (p);
+	}
 	ut64 left = ((UT64_MAX >>i) << i);
-        return left | n;
+	return left | n;
 }
 
 R_API int r_num_between(RNum *num, const char *input_value) {
@@ -698,6 +784,11 @@ R_API int r_num_between(RNum *num, const char *input_value) {
 	char * const str = strdup (input_value);
 	RList *nums = r_num_str_split_list (str);
 	int len = r_list_length (nums);
+	if (len < 3) {
+		free (str);
+		r_list_free (nums);
+		return -1;
+	}
 	if (len > 3) {
 		len = 3;
 	}
@@ -714,16 +805,16 @@ R_API bool r_num_is_op(const char c) {
 		c == '%' || c == '&' || c == '^' || c == '|';
 }
 
-//Assumed *str is parsed as an expression correctly
-R_API int r_num_str_len (const char *str) {
+// Assumed *str is parsed as an expression correctly
+R_API int r_num_str_len(const char *str) {
 	int i = 0, len = 0, st;
-	st = 0;//0: number, 1: op
+	st = 0; // 0: number, 1: op
 	if (str[0] == '(') {
 		i++;
 	}
 	while (str[i] != '\0') {
 		switch (st) {
-		case 0: //number
+		case 0: // number
 			while (!r_num_is_op (str[i]) && str[i] != ' '
 			  && str[i] != '\0') {
 				i++;
@@ -734,7 +825,7 @@ R_API int r_num_str_len (const char *str) {
 			len = i;
 			st = 1;
 			break;
-		case 1: //op
+		case 1: // op
 			while (str[i] != '\0' && str[i] == ' ') {
 				i++;
 			}
@@ -769,7 +860,7 @@ R_API int r_num_str_split(char *str) {
 
 R_API RList *r_num_str_split_list(char *str) {
 	int i, count = r_num_str_split (str);
-	RList *list = r_list_newf (free);
+	RList *list = r_list_new ();
 	for (i = 0; i < count; i++) {
 		r_list_append (list, str);
 		str += strlen (str) + 1;
@@ -785,3 +876,11 @@ R_API void *r_num_dup(ut64 n) {
 	*hn = n;
 	return (void*)hn;
 }
+
+R_API double r_num_cos(double a) {
+	return cos (a);
+}
+
+R_API double r_num_sin(double a) {
+	return sin (a);
+}
