@@ -486,7 +486,7 @@ R_API const char *r_utf_block_name(int idx) {
 #define r_utf_blocks_count (sizeof (r_utf_blocks) / sizeof (r_utf_blocks[0]))
 
 /* Convert an UTF-8 buf into a unicode RRune */
-R_API int r_utf8_decode (const ut8 *ptr, int ptrlen, RRune *ch) {
+R_API int r_utf8_decode(const ut8 *ptr, int ptrlen, RRune *ch) {
 	if (ptrlen < 1) {
 		return 0;
 	}
@@ -515,7 +515,7 @@ R_API int r_utf8_decode (const ut8 *ptr, int ptrlen, RRune *ch) {
 }
 
 /* Convert a unicode RRune into an UTF-8 buf */
-R_API int r_utf8_encode (ut8 *ptr, const RRune ch) {
+R_API int r_utf8_encode(ut8 *ptr, const RRune ch) {
 	if (ch < 0x80) {
 		ptr[0] = (ut8)ch;
 		return 1;
@@ -542,7 +542,7 @@ R_API int r_utf8_encode (ut8 *ptr, const RRune ch) {
 }
 
 /* Convert a unicode RRune string into an utf-8 one */
-R_API int r_utf8_encode_str (const RRune *str, ut8 *dst, const int dst_length) {
+R_API int r_utf8_encode_str(const RRune *str, ut8 *dst, const int dst_length) {
 	int i, pos = 0;
 
 	if (!str || !dst) {
@@ -558,7 +558,7 @@ R_API int r_utf8_encode_str (const RRune *str, ut8 *dst, const int dst_length) {
 }
 
 /* Returns the size in bytes of the utf-8 encoded char */
-R_API int r_utf8_size (const ut8 *ptr) {
+R_API int r_utf8_size(const ut8 *ptr) {
 	const int utf8_size[] = {
 		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
@@ -572,7 +572,7 @@ R_API int r_utf8_size (const ut8 *ptr) {
 	return (ptr[0]&0x80) ? utf8_size[ptr[0]^0x80] : 1;
 }
 
-R_API int r_utf8_strlen (const ut8 *str) {
+R_API int r_utf8_strlen(const ut8 *str) {
 	int i, len = 0;
 
 	for (i = 0; str[i]; i++) {
@@ -584,7 +584,17 @@ R_API int r_utf8_strlen (const ut8 *str) {
 	return len;
 }
 
-R_API int r_isprint (const RRune c) {
+R_API int r_isprint(const RRune c) {
+	// RRunes are most commonly single byte... We can early out with this common case.
+	if (c < 0x34F) {
+		/*
+		manually copied from top, please update if this ever changes
+		{ 0x0000, 0x001F }, { 0x007F, 0x009F }, { 0x034F, 0x034F },
+		could do a linear search, but that's a lot slower than a few compare
+		*/
+		return !( c <= 0x1F || ( c >= 0x7F && c <= 0x9F));
+	}
+
 	const int last = nonprintable_ranges_count;
 
 	int low = 0;
@@ -607,7 +617,7 @@ R_API int r_isprint (const RRune c) {
 }
 
 #if __WINDOWS__
-R_API char *r_utf16_to_utf8_l (const wchar_t *wc, int len) {
+R_API char *r_utf16_to_utf8_l(const wchar_t *wc, int len) {
 	if (!wc || !len || len < -1) {
 		return NULL;
 	}
@@ -626,7 +636,7 @@ R_API char *r_utf16_to_utf8_l (const wchar_t *wc, int len) {
 	return rutf8;
 }
 
-R_API wchar_t *r_utf8_to_utf16_l (const char *cstring, int len) {
+R_API wchar_t *r_utf8_to_utf16_l(const char *cstring, int len) {
 	if (!cstring || !len || len < -1) {
 		return NULL;
 	}
@@ -645,7 +655,36 @@ R_API wchar_t *r_utf8_to_utf16_l (const char *cstring, int len) {
 	return rutf16;
 }
 
-R_API const char *r_acp_to_utf8_l (const ut8 *str, int len) {
+R_API char *r_utf8_to_acp_l(const char *str, int len) {
+	if (!str || !len || len < -1) {
+		return NULL;
+	}
+	char *acp = NULL;
+	int wcsize, csize;
+	if ((wcsize = MultiByteToWideChar (CP_UTF8, 0, str, len, NULL, 0))) {
+		wchar_t *rutf16;
+		++wcsize;
+		if ((rutf16 = (wchar_t *)calloc (wcsize, sizeof (wchar_t)))) {
+			MultiByteToWideChar (CP_UTF8, 0, str, len, rutf16, wcsize);
+			if (len != -1) {
+				rutf16[wcsize - 1] = L'\0';
+			}
+			if ((csize = WideCharToMultiByte (CP_ACP, 0, rutf16, wcsize, NULL, 0, NULL, NULL))) {
+				++csize;
+				if ((acp = malloc (csize))) {
+					WideCharToMultiByte (CP_ACP, 0, rutf16, wcsize, acp, csize, NULL, NULL);
+					if (len != -1) {
+						acp[csize - 1] = '\0';
+					}
+				}
+			}
+			free (rutf16);
+		}
+	}
+	return acp;
+}
+
+R_API char *r_acp_to_utf8_l(const char *str, int len) {
 	if (!str || !len || len < -1) {
 		return NULL;
 	}
@@ -658,7 +697,7 @@ R_API const char *r_acp_to_utf8_l (const ut8 *str, int len) {
 			if (len != -1) {
 				rutf16[wcsize - 1] = L'\0';
 			}
-			const char *ret = r_utf16_to_utf8_l (rutf16, wcsize);
+			char *ret = r_utf16_to_utf8_l (rutf16, wcsize);
 			free (rutf16);
 			return ret;
 		}
@@ -668,7 +707,7 @@ R_API const char *r_acp_to_utf8_l (const ut8 *str, int len) {
 
 #endif // __WINDOWS__
 
-R_API int r_utf_block_idx (RRune ch) {
+R_API int r_utf_block_idx(RRune ch) {
 	const int last = r_utf_blocks_count;
 	int low, hi, mid;
 
@@ -692,18 +731,27 @@ R_API int r_utf_block_idx (RRune ch) {
 }
 
 /* str must be UTF8-encoded */
-R_API int *r_utf_block_list(const ut8 *str, int len) {
+R_API int *r_utf_block_list(const ut8 *str, int len, int **freq_list) {
 	if (!str) {
 		return NULL;
 	}
 	if (len < 0) {
 		len = strlen ((const char *)str);
 	}
-	static bool has_block[r_utf_blocks_count] = {0};
+	static int block_freq[r_utf_blocks_count] = {0};
 	int *list = R_NEWS (int, len + 1);
 	if (!list) {
 		return NULL;
 	}
+	int *freq_list_ptr = NULL;
+	if (freq_list) {
+		*freq_list = R_NEWS (int, len + 1);
+		if (!*freq_list) {
+			free (list);
+			return NULL;
+		}
+		freq_list_ptr = *freq_list;
+	}
 	int *list_ptr = list;
 	const ut8 *str_ptr = str;
 	const ut8 *str_end = str + len;
@@ -717,16 +765,48 @@ R_API int *r_utf_block_list(const ut8 *str, int len) {
 		} else {
 			block_idx = r_utf_block_idx (ch);
 		}
-		if (!has_block[block_idx]) {
-			has_block[block_idx] = true;
+		if (!block_freq[block_idx]) {
 			*list_ptr = block_idx;
 			list_ptr++;
 		}
+		block_freq[block_idx]++;
 		str_ptr += ch_bytes;
 	}
 	*list_ptr = -1;
+	if (freq_list_ptr) {
+		for (list_ptr = list; *list_ptr != -1; list_ptr++) {
+			*freq_list_ptr = block_freq[*list_ptr];
+			freq_list_ptr++;
+		}
+		*freq_list_ptr = -1;
+	}
 	for (list_ptr = list; *list_ptr != -1; list_ptr++) {
-		has_block[*list_ptr] = false;
+		block_freq[*list_ptr] = 0;
 	}
 	return list;
 }
+
+R_API RStrEnc r_utf_bom_encoding(const ut8 *ptr, int ptrlen) {
+	if (ptrlen > 3) {
+		if (ptr[0] == 0xff && ptr[1] == 0xfe && !ptr[2] && !ptr[3]) {
+			return R_STRING_ENC_UTF32LE;
+		}
+		if (!ptr[0] && !ptr[1] && ptr[2] == 0xfe && ptr[3] == 0xff) {
+			return R_STRING_ENC_UTF32BE;
+		}
+	}
+	if (ptrlen > 2) {
+		if (ptr[0] == 0xef && ptr[1] == 0xbb && ptr[2] == 0xbf) {
+			return R_STRING_ENC_UTF8;
+		}
+	}
+	if (ptrlen > 1) {
+		if (ptr[0] == 0xff && ptr[1] == 0xfe) {
+			return R_STRING_ENC_UTF16LE;
+		}
+		if (ptr[0] == 0xfe && ptr[1] == 0xff) {
+			return R_STRING_ENC_UTF16BE;
+		}
+	}
+	return R_STRING_ENC_GUESS;
+}
