@@ -1,11 +1,15 @@
 /* libgdbr - LGPL - Copyright 2017-2018 - srimanta.barua1 */
 
 #include "gdbclient/xml.h"
+#include "gdbclient/commands.h"
 #include "gdbclient/core.h"
 #include "arch.h"
 #include "gdbr_common.h"
 #include "packet.h"
 #include <r_util.h>
+#include <r_debug.h>
+
+#define MAX_PID_CHARS (5)
 
 static char *gdbr_read_feature(libgdbr_t *g, const char *file, ut64 *tot_len) {
 	ut64 retlen = 0, retmax = 0, off = 0, len = g->stub_features.pkt_sz - 2,
@@ -73,6 +77,7 @@ static char *gdbr_read_feature(libgdbr_t *g, const char *file, ut64 *tot_len) {
 				retlen -= subret_space - subret_len;
 				ret[retlen] = '\0';
 				tmp = strstr (tmp3, "<xi:include");
+				free (subret);
 				continue;
 			}
 			if (subret_len > retmax - retlen - 1) {
@@ -102,6 +107,47 @@ exit_err:
 	return NULL;
 }
 
+static char *gdbr_read_osdata(libgdbr_t *g, const char *file, ut64 *tot_len) {
+	ut64 retlen = 0, retmax = 0, off = 0, len = g->stub_features.pkt_sz - 2,
+		blksz = g->data_max;
+	char *tmp, *ret = NULL, msg[128] = { 0 }, status;
+	while (1) {
+		snprintf (msg, sizeof (msg), "qXfer:osdata:read:%s:%" PFMT64x ",%" PFMT64x, file, off, len);
+		if (send_msg (g, msg) < 0 || read_packet (g, false) < 0 || send_ack (g) < 0) {
+			goto exit_err;
+		}
+		if (g->data_len == 0) {
+			goto exit_err;
+		}
+		if (g->data_len == 1 && g->data[0] == 'l') {
+			break;
+		}
+		status = g->data[0];
+		if (retmax - retlen < g->data_len) {
+			if (!(tmp = realloc (ret, retmax + blksz))) {
+				goto exit_err;
+			}
+			retmax += blksz;
+			ret = tmp;
+		}
+		strcpy (ret + retlen, g->data + 1);
+		retlen += g->data_len - 1;
+		off = retlen;
+		if (status == 'l') {
+			break;
+		}
+	}
+	if (!ret) {
+		*tot_len = 0;
+		return NULL;
+	}
+	*tot_len = retlen;
+	return ret;
+exit_err:
+	free (ret);
+	*tot_len = 0;
+	return NULL;
+}
 
 typedef struct {
 	char type[32];
@@ -123,8 +169,9 @@ typedef struct {
 
 static void _write_flag_bits(char *buf, const gdbr_xml_flags_t *flags);
 static int _resolve_arch(libgdbr_t *g, char *xml_data);
-static RList* _extract_flags(char *flagstr);
-static RList* _extract_regs(char *regstr, RList *flags, char *pc_alias);
+static RList *_extract_flags(char *flagstr);
+static RList *_extract_regs(char *regstr, RList *flags, char *pc_alias);
+static RDebugPid *_extract_pid_info(const char *info, const char *path, int tid);
 
 static int gdbr_parse_target_xml(libgdbr_t *g, char *xml_data, ut64 len) {
 	char *regstr, *flagstr, *tmp, *profile = NULL, pc_alias[64], flag_bits[65];
@@ -132,6 +179,7 @@ static int gdbr_parse_target_xml(libgdbr_t *g, char *xml_data, ut64 len) {
 	RListIter *iter;
 	gdbr_xml_flags_t *tmpflag;
 	gdbr_xml_reg_t *tmpreg;
+	int packed_size = 0;
 	ut64 profile_len = 0, profile_max_len, regnum = 0, regoff = 0;
 	pc_alias[0] = '\0';
 	gdb_reg_t *arch_regs = NULL;
@@ -161,11 +209,6 @@ static int gdbr_parse_target_xml(libgdbr_t *g, char *xml_data, ut64 len) {
 		if (!tmpreg) {
 			continue;
 		}
-		// regsize > 64 not supported by r2 currently
-		if (tmpreg->size > 8) {
-			regoff += tmpreg->size;
-			continue;
-		}
 		memcpy (arch_regs[regnum].name, tmpreg->name, sizeof (tmpreg->name));
 		arch_regs[regnum].size = tmpreg->size;
 		arch_regs[regnum].offset = regoff;
@@ -182,9 +225,17 @@ static int gdbr_parse_target_xml(libgdbr_t *g, char *xml_data, ut64 len) {
 			tmpflag = r_list_get_n (flags, tmpreg->flagnum);
 			_write_flag_bits (flag_bits, tmpflag);
 		}
-		profile_len += snprintf (profile + profile_len, 128, "%s\t%s\t"
-					".%u\t%"PFMT64d"\t0\t%s\n", tmpreg->type,
-					tmpreg->name, tmpreg->size * 8, regoff, flag_bits);
+		packed_size = 0;
+		if (tmpreg->size >= 8 && (strstr (tmpreg->type, "fpu") ||
+			  strstr (tmpreg->type, "mmx") || strstr (tmpreg->type, "xmm") ||
+			  strstr (tmpreg->type, "ymm"))) {
+			packed_size = tmpreg->size;
+		}
+		profile_len += snprintf (profile + profile_len, 128,
+			"%s\t%s\t.%u\t%" PFMT64d "\t%d\t%s\n", tmpreg->type,
+			tmpreg->name, tmpreg->size * 8, regoff,
+			packed_size,
+			flag_bits);
 		// TODO write flag subregisters
 		if (tmpflag) {
 			int i;
@@ -209,7 +260,7 @@ static int gdbr_parse_target_xml(libgdbr_t *g, char *xml_data, ut64 len) {
 	case R_SYS_ARCH_ARM:
 		switch (g->target.bits) {
 		case 32:
-			if (!(profile = r_str_prefix (profile,
+			if (!(profile = r_str_prepend (profile,
 							"=PC	pc\n"
 							"=SP	sp\n" // XXX
 							"=A0	r0\n"
@@ -221,7 +272,7 @@ static int gdbr_parse_target_xml(libgdbr_t *g, char *xml_data, ut64 len) {
 			}
 			break;
 		case 64:
-			if (!(profile = r_str_prefix (profile,
+			if (!(profile = r_str_prepend (profile,
 							"=PC	pc\n"
 							"=SP	sp\n"
 							"=BP	x29\n"
@@ -242,7 +293,7 @@ static int gdbr_parse_target_xml(libgdbr_t *g, char *xml_data, ut64 len) {
 	case R_SYS_ARCH_X86:
 		switch (g->target.bits) {
 		case 32:
-			if (!(profile = r_str_prefix (profile,
+			if (!(profile = r_str_prepend (profile,
 						     "=PC	eip\n"
 						     "=SP	esp\n"
 						     "=BP	ebp\n"))) {
@@ -250,7 +301,7 @@ static int gdbr_parse_target_xml(libgdbr_t *g, char *xml_data, ut64 len) {
 			}
 			break;
 		case 64:
-			if (!(profile = r_str_prefix (profile,
+			if (!(profile = r_str_prepend (profile,
 						     "=PC	rip\n"
 						     "=SP	rsp\n"
 						     "=BP	rbp\n"))) {
@@ -259,7 +310,7 @@ static int gdbr_parse_target_xml(libgdbr_t *g, char *xml_data, ut64 len) {
 		}
 		break;
 	case R_SYS_ARCH_MIPS:
-		if (!(profile = r_str_prefix (profile,
+		if (!(profile = r_str_prepend (profile,
 						"=PC	pc\n"
 						"=SP	r29\n"))) {
 			goto exit_err;
@@ -268,7 +319,7 @@ static int gdbr_parse_target_xml(libgdbr_t *g, char *xml_data, ut64 len) {
 	default:
 		// TODO others
 		if (*pc_alias) {
-			if (!(profile = r_str_prefix (profile, pc_alias))) {
+			if (!(profile = r_str_prepend (profile, pc_alias))) {
 				goto exit_err;
 			}
 		}
@@ -298,6 +349,113 @@ exit_err:
 	return -1;
 }
 
+/* Reference:
+<osdata type="processes">
+<item>
+<column name="pid">1</column>
+<column name="user">root</column>
+<column name="command">/sbin/init maybe-ubiquity </column>
+<column name="cores">0</column>
+</item>
+</osdata>
+*/
+static int gdbr_parse_processes_xml(libgdbr_t *g, char *xml_data, ut64 len, int pid, RList *list) {
+	char pidstr[MAX_PID_CHARS + 1], status[1024], cmdline[1024];
+	char *itemstr, *itemstr_end, *column, *column_end, *proc_filename;
+	int ret = -1, ipid, column_data_len;
+	RDebugPid *pid_info = NULL;
+
+	// Make sure the given xml is valid
+	if (!r_str_startswith (xml_data, "<osdata type=\"processes\">")) {
+		ret = -1;
+		goto end;
+	}
+
+	column = xml_data;
+	while ((itemstr = strstr (column, "<item>"))) {
+		if (!(itemstr_end = strstr (itemstr, "</item>"))) {
+			ret = -1;
+			goto end;
+		}
+		// Get PID
+		if (!(column = strstr (itemstr, "<column name=\"pid\">"))) {
+			ret = -1;
+			goto end;
+		}
+		if (!(column_end = strstr (column, "</column>"))) {
+			ret = -1;
+			goto end;
+		}
+
+		column += sizeof ("<column name=\"pid\">") - 1;
+		column_data_len = column_end - column;
+
+		memcpy (pidstr, column, column_data_len);
+		pidstr[column_data_len] = '\0';
+
+		ipid = atoi (pidstr);
+
+		// Get cmdline
+		if (!(column = strstr (itemstr, "<column name=\"command\">"))) {
+			ret = -1;
+			goto end;
+		}
+		if (!(column_end = strstr (column, "</column>"))) {
+			ret = -1;
+			goto end;
+		}
+
+		column += sizeof ("<column name=\"command\">") - 1;
+		column_data_len = column_end - column;
+
+		memcpy (cmdline, column, column_data_len);
+		cmdline[column_data_len] = '\0';
+
+		// Attempt to read the pid's info from /proc. Non UNIX systems will have the
+		// correct pid and cmdline from the xml with everything else set to default
+		proc_filename = r_str_newf ("/proc/%d/status", ipid);
+		if (gdbr_open_file (g, proc_filename, O_RDONLY, 0) == 0) {
+			if (gdbr_read_file (g, (unsigned char *)status, sizeof (status)) != -1) {
+				pid_info = _extract_pid_info (status, cmdline, ipid);
+			} else {
+				eprintf ("Failed to read from data from procfs file of pid (%d)\n", ipid);
+			}
+			if (gdbr_close_file (g) != 0) {
+				eprintf ("Failed to close procfs file of pid (%d)\n", ipid);
+			}
+		} else {
+			eprintf ("Failed to open procfs file of pid (%d)\n", ipid);
+			if (!(pid_info = R_NEW0 (RDebugPid)) || !(pid_info->path = strdup (cmdline))) {
+				ret = -1;
+				goto end;
+			}
+			pid_info->pid = ipid;
+			pid_info->ppid = 0;
+			pid_info->uid = pid_info->gid = -1;
+			pid_info->runnable = true;
+			pid_info->status = R_DBG_PROC_STOP;
+		}
+		// Unless pid 0 is requested, only add the requested pid and it's child processes
+		if (0 == pid || ipid == pid || pid_info->ppid == pid) {
+			r_list_append (list, pid_info);
+		} else {
+			if (pid_info) {
+				free (pid_info);
+				pid_info = NULL;
+			}
+		}
+	}
+
+	ret = 0;
+end:
+	if (ret != 0) {
+		if (pid_info) {
+			free (pid_info);
+		}
+	}
+	return ret;
+}
+
 // If xml target description is supported, read it
 int gdbr_read_target_xml(libgdbr_t *g) {
 	if (!g->stub_features.qXfer_features_read) {
@@ -313,6 +471,32 @@ int gdbr_read_target_xml(libgdbr_t *g) {
 	return 0;
 }
 
+int gdbr_read_processes_xml(libgdbr_t *g, int pid, RList *list) {
+	if (!g->stub_features.qXfer_features_read) {
+		return -1;
+	}
+	ut64 len;
+	int ret = -1;
+	char *data;
+
+	if (!(data = gdbr_read_osdata (g, "processes", &len))) {
+		ret = -1;
+		goto end;
+	}
+
+	if (gdbr_parse_processes_xml (g, data, len, pid, list) != 0) {
+		ret = -1;
+		goto end;
+	}
+
+	ret = 0;
+end:
+	if (data) {
+		free (data);
+	}
+	return ret;
+}
+
 // sizeof (buf) needs to be atleast flags->num_bits + 1
 static void _write_flag_bits(char *buf, const gdbr_xml_flags_t *flags) {
 	bool fc[26] = { false };
@@ -325,7 +509,7 @@ static void _write_flag_bits(char *buf, const gdbr_xml_flags_t *flags) {
 			continue;
 		}
 		// To avoid duplicates. This skips flags if first char is same. i.e.
-		// for x86_64, it will skip VIF because VM already occured. This is
+		// for x86_64, it will skip VIF because VM already occurred. This is
 		// same as default reg-profiles in r2
 		c = tolower (flags->fields[i].name[0]) - 'a';
 		if (fc[c]) {
@@ -372,6 +556,9 @@ static int _resolve_arch(libgdbr_t *g, char *xml_data) {
 			// openocd mips?
 			g->target.arch = R_SYS_ARCH_MIPS;
 			g->target.bits = 32;
+		} else if (strstr(xml_data, "com.apple.debugserver.x86_64")) {
+			g->target.arch = R_SYS_ARCH_X86;
+			g->target.bits = 64;
 		} else {
 			eprintf ("Unknown architecture parsing XML (%s)\n", xml_data);
 		}
@@ -379,7 +566,7 @@ static int _resolve_arch(libgdbr_t *g, char *xml_data) {
 	return 0;
 }
 
-static RList* _extract_flags(char *flagstr) {
+static RList *_extract_flags(char *flagstr) {
 	char *tmp1, *tmp2, *flagsend, *field_start, *field_end;
 	ut64 num_fields, type_sz, name_sz;
 	gdbr_xml_flags_t *tmpflag = NULL;
@@ -486,8 +673,56 @@ exit_err:
 	return NULL;
 }
 
-static RList* _extract_regs(char *regstr, RList *flags, char *pc_alias) {
-	char *regstr_end, *regname, *regtype, *tmp1;
+static RDebugPid *_extract_pid_info(const char *info, const char *path, int tid) {
+	RDebugPid *pid_info = R_NEW0 (RDebugPid);
+	if (!pid_info) {
+		return NULL;
+	}
+	char *ptr = strstr (info, "State:");
+	if (ptr) {
+		switch (*(ptr + 7)) {
+		case 'R':
+			pid_info->status = R_DBG_PROC_RUN;
+			break;
+		case 'S':
+			pid_info->status = R_DBG_PROC_SLEEP;
+			break;
+		case 'T':
+		case 't':
+			pid_info->status = R_DBG_PROC_STOP;
+			break;
+		case 'Z':
+			pid_info->status = R_DBG_PROC_ZOMBIE;
+			break;
+		case 'X':
+			pid_info->status = R_DBG_PROC_DEAD;
+			break;
+		default:
+			pid_info->status = R_DBG_PROC_SLEEP;
+			break;
+		}
+	}
+	ptr = strstr (info, "PPid:");
+	if (ptr) {
+		pid_info->ppid = atoi (ptr + 5);
+	}
+	ptr = strstr (info, "Uid:");
+	if (ptr) {
+		pid_info->uid = atoi (ptr + 5);
+	}
+	ptr = strstr (info, "Gid:");
+	if (ptr) {
+		pid_info->gid = atoi (ptr + 5);
+	}
+	pid_info->pid = tid;
+	pid_info->path = path ? strdup (path) : NULL;
+	pid_info->runnable = true;
+	pid_info->pc = 0;
+	return pid_info;
+}
+
+static RList *_extract_regs(char *regstr, RList *flags, char *pc_alias) {
+	char *regstr_end, *regname, *regtype, *tmp1, *tmpregstr, *feature_end, *typegroup;
 	ut32 flagnum, regname_len, regsize, regnum;
 	RList *regs;
 	RListIter *iter;
@@ -496,10 +731,53 @@ static RList* _extract_regs(char *regstr, RList *flags, char *pc_alias) {
 	if (!(regs = r_list_new ())) {
 		return NULL;
 	}
-	while ((regstr = strstr (regstr, "<reg"))) {
-		if (!(regstr_end = strchr (regstr, '/'))) {
+	// Set gpr as the default register type for all of the following registers until `feature` is found
+	typegroup = "gpr";
+	while ((tmpregstr = strstr (regstr, "<reg"))) {
+		if (!(regstr_end = strchr (tmpregstr, '/'))) {
 			goto exit_err;
 		}
+		// Most regs don't have group/type params, attempt to get the type from `feature`.
+		// Multiple registers can be wrapped with a certain feature so this typegroup
+		// applies on all of the following registers until </feature>
+		if (r_str_startswith (regstr, "<feature")) {
+			// Verify that we found the feature in the current node
+			feature_end = strchr (regstr, '>');
+			// To parse features of other architectures refer to:
+			// https://sourceware.org/gdb/onlinedocs/gdb/Standard-Target-Features.html#Standard-Target-Features
+            // - x86
+			if ((tmp1 = strstr (regstr, "core")) != NULL && tmp1 < feature_end) {
+				typegroup = "gpr";
+			} else if ((tmp1 = strstr (regstr, "segments")) != NULL && tmp1 < feature_end) {
+				typegroup = "seg";
+			} else if ((tmp1 = strstr (regstr, "linux")) != NULL && tmp1 < feature_end) {
+				typegroup = "gpr";
+			// Includes avx.512
+			} else if ((tmp1 = strstr (regstr, "avx")) != NULL && tmp1 < feature_end) {
+				typegroup = "ymm";
+			} else if ((tmp1 = strstr (regstr, "mpx")) != NULL && tmp1 < feature_end) {
+				typegroup = "seg";
+			// - arm
+			} else if ((tmp1 = strstr (regstr, "m-profile")) != NULL && tmp1 < feature_end) {
+				typegroup = "gpr";
+			} else if ((tmp1 = strstr (regstr, "pfe")) != NULL && tmp1 < feature_end) {
+				typegroup = "fpu";
+			} else if ((tmp1 = strstr (regstr, "vfp")) != NULL && tmp1 < feature_end) {
+				typegroup = "fpu";
+			} else if ((tmp1 = strstr (regstr, "iwmmxt")) != NULL && tmp1 < feature_end) {
+				typegroup = "xmm";
+			// -- Aarch64
+			} else if ((tmp1 = strstr (regstr, "sve")) != NULL && tmp1 < feature_end) {
+				typegroup = "ymm";
+			} else {
+				typegroup = "gpr";
+			}
+		}
+		// Reset to typegroup in case the previous register had a group/type parameter
+		// that indicated it's specific type which doesn't correspond to type defined by
+		// the parent feature tag
+		regtype = typegroup;
+		regstr = tmpregstr;
 		*regstr_end = '\0';
 		// name
 		if (!(regname = strstr (regstr, "name="))) {
@@ -528,13 +806,19 @@ static RList* _extract_regs(char *regstr, RList *flags, char *pc_alias) {
 			}
 			regnum = strtoul (tmp1, NULL, 10);
 		}
-		// type
-		regtype = "gpr";
 		flagnum = r_list_length (flags);
 		if ((tmp1 = strstr (regstr, "group="))) {
 			tmp1 += 7;
 			if (r_str_startswith (tmp1, "float")) {
 				regtype = "fpu";
+			} else if (r_str_startswith (tmp1, "mmx")) {
+				regtype = "mmx";
+			} else if (r_str_startswith (tmp1, "sse")) {
+				regtype = "xmm";
+			} else if (r_str_startswith (tmp1, "vector")) {
+				regtype = "ymm";
+			} else if (r_str_startswith (tmp1, "system")) {
+				regtype = "seg";
 			}
 			// We need type information in r2 register profiles
 		} else if ((tmp1 = strstr (regstr, "type="))) {
@@ -563,6 +847,16 @@ static RList* _extract_regs(char *regstr, RList *flags, char *pc_alias) {
 			}
 			// We need type information in r2 register profiles
 		}
+		// Move unidentified vector/large registers from gpr to xmm since r2 set/get
+		// registers doesn't support >64bit registers atm(but it's still possible to
+		// read them using gdbr's implementation through dr/drt)
+		if (regsize > 64 && !strcmp (regtype, "gpr")) {
+			regtype = "xmm";
+		}
+		// Move appropriately sized unidentified xmm registers from fpu to xmm
+		if (regsize == 128 && !strcmp (regtype, "fpu")) {
+			regtype = "xmm";
+		}
 		if (!(tmpreg = calloc (1, sizeof (gdbr_xml_reg_t)))) {
 			goto exit_err;
 		}
@@ -592,7 +886,12 @@ static RList* _extract_regs(char *regstr, RList *flags, char *pc_alias) {
 			r_list_set_n (regs, regnum, tmpreg);
 		}
 		*regstr_end = '/';
-		regstr = regstr_end + 2;
+		regstr = regstr_end + 3;
+		if (r_str_startswith (regstr, "</feature>")) {
+			regstr += sizeof ("</feature>");
+			// Revert to default
+			typegroup = "gpr";
+		}
 	}
 	regs->free = free;
 	return regs;
