@@ -40,8 +40,10 @@
 #include <grub/fshelp.h>
 #include <r_types.h>
 
+#ifndef MIN
 #define MIN(a, b) ((a) < (b) ? (a) : (b))
 #define MAX(a, b) ((a) > (b) ? (a) : (b))
+#endif
 
 #define REISERFS_SUPER_BLOCK_OFFSET 0x10000
 #define REISERFS_MAGIC_LEN 12
@@ -750,7 +752,7 @@ grub_reiserfs_iterate_dir (grub_fshelp_node_t item,
   do
     {
       struct grub_reiserfs_directory_header *directory_headers;
-      struct grub_fshelp_node directory_item;
+      struct grub_fshelp_node directory_item = {0};
       grub_uint16_t entry_count, entry_number;
       struct grub_reiserfs_item_header *item_headers;
 
