@@ -14,8 +14,9 @@ static int allwrite(BufferOp op, int fd, const char *buf, ut32 len) {
 	ut32 w;
 	while (len > 0) {
 		w = op (fd, buf, len);
-		if (w != len)
+		if (w != len) {
 			return 0;
+		}
 		buf += w;
 		len -= w;
 	}
@@ -24,19 +25,24 @@ static int allwrite(BufferOp op, int fd, const char *buf, ut32 len) {
 
 int buffer_flush(buffer *s) {
 	int p = s->p;
-	if (!p) return 1;
+	if (!p) {
+		return 1;
+	}
 	s->p = 0;
 	return allwrite (s->op, s->fd, s->x, p);
 }
 
 int buffer_putalign(buffer *s, const char *buf, ut32 len) {
 	ut32 n;
-	if (!s || !s->x || !buf)
+	if (!s || !s->x || !buf) {
 		return 0;
+	}
 	while (len > (n = s->n - s->p)) {
 		memcpy (s->x + s->p, buf, n);
 		s->p += n; buf += n; len -= n;
-		if (!buffer_flush (s)) return 0;
+		if (!buffer_flush (s)) {
+			return 0;
+		}
 	}
 	/* now len <= s->n - s->p */
 	memcpy (s->x + s->p, buf, len);
@@ -45,6 +51,8 @@ int buffer_putalign(buffer *s, const char *buf, ut32 len) {
 }
 
 int buffer_putflush(buffer *s, const char *buf, ut32 len) {
-	if (!buffer_flush (s)) return 0;
+	if (!buffer_flush (s)) {
+		return 0;
+	}
 	return allwrite (s->op, s->fd, buf, len);
 }
