@@ -5,7 +5,7 @@
 #include <stdarg.h>
 #include <stdlib.h>
 #include <fcntl.h>
-#include <unistd.h>
+#include <ctype.h>
 #include "sdb.h"
 
 typedef struct {
@@ -229,6 +229,7 @@ repeat:
 	if (next) *next = ';';
 	eq = strchr (p, '=');
 	if (eq) {
+		d = 1;
 		*eq++ = 0;
 		if (*eq == '$') {
 			next = strchr (eq + 1, ';');
@@ -245,6 +246,7 @@ repeat:
 		}
 	} else {
 		val = NULL;
+		d = 0;
 	}
 	if (!is_ref) {
 		next = strchr (val? val: cmd, ';');
@@ -305,13 +307,15 @@ next_quote:
 			SdbListIter *it;
 			SdbNs *ns;
 			ls_foreach (s->ns, it, ns) {
-				int len = strlen (ns->name);
-				if (len<(long)sizeof (root)) {
-					memcpy (root, ns->name, len+1);
+				int name_len = strlen (ns->name);
+				if (name_len < (long)sizeof (root)) {
+					memcpy (root, ns->name, name_len + 1);
 					walk_namespace (out, root,
-						sizeof (root)-len,
-						root+len, ns, encode);
-				} else eprintf ("TODO: Namespace too long\n");
+						sizeof (root) - name_len,
+						root + name_len, ns, encode);
+				} else {
+					eprintf ("TODO: Namespace too long\n");
+				}
 			}
 			goto fail;
 		} else
@@ -376,6 +380,7 @@ next_quote:
 			fflush (stdout);
 			ls_free (l);
 		} else {
+			d = 1;
 			sdb_unset_like (s, cmd + 1);
 		}
 	} else if (*cmd == '+' || *cmd == '-') {
@@ -755,6 +760,14 @@ next_quote:
 				*json++ = 0;
 				ok = sdb_json_set (s, cmd, json, val, 0);
 			} else {
+				while (*val && isspace (*val)) {
+					val++;
+				}
+				int i = strlen (cmd) - 1;
+				while (i >= 0 && isspace (cmd[i])) {
+					cmd[i] = '\0';
+					i--;
+				}
 				ok = sdb_set (s, cmd, val, 0);
 			}
 			if (encode) {
@@ -810,6 +823,7 @@ runNext:
 			bufset = 0;
 		}
 		cmd = next + 1;
+		encode = 0;
 		goto repeat;
 	}
 	if (eq) {
