@@ -1,6 +1,6 @@
 #!/bin/sh
 
-git clone https://github.com/radare/radare2-extras
+git clone https://github.com/radareorg/radare2-extras
 cd radare2-extras
 ./configure --prefix=/usr
 make
