@@ -10,11 +10,8 @@ gmake --help >/dev/null 2>&1
 [ $? = 0 ] && MAKE=gmake
 
 # find root
-cd `dirname $PWD/$0` ; cd ..
-if [ -d .hg ]; then
-	echo hg up -C -r "${REV}"
-	hg up -C -r "${REV}"
-elif [ -d .git ]; then
+cd "$(dirname "$PWD/$0")" ; cd ..
+if [ -d .git ]; then
 	echo git checkout "${REV}"
 	git checkout "${REV}"
 fi
