@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/bin/bash
 
 # You can modify these variables
 PREFIX="/usr"
@@ -8,7 +8,7 @@ USE_SIMULATOR=0
 SIMULATOR_ARCHS="x86_64"
 PACKAGE_RADARE=0
 EMBED_BITCODE=1
-CFLAGS="-O2"
+CFLAGS="-O2 -miphoneos-version-min=10.0"
 DOSH=0
 ARCHS="" # Will be set by -archs argument. If you want to set it -> e.g. ARCHS="armv7+arm64".
 MERGE_LIBS=1 # Will merge libs if you build for arm and simulator 
@@ -17,11 +17,13 @@ MERGE_LIBS=1 # Will merge libs if you build for arm and simulator
 export PATH=/Applications/Xcode.app/Contents/Developer/Platforms/iPhoneOS.platform/Developer/usr/bin:$PATH
 export PATH=`pwd`/sys:${PATH}
 export CC=`pwd`/sys/ios-sdk-gcc
-export LD="xcrun --sdk iphoneos ld"
 export IOSVER=9.0
 export IOSINC=`pwd`/sys/ios-include
 export USE_IOS_STATIC=0
 
+echo "If xcrun --sdk iphoneos cant find the profile use this line:"
+echo " sudo xcode-select -switch /Applications/Xcode.app"
+
 PLUGINS_CFG=plugins.ios-store.cfg
 #PLUGINS_CFG=plugins.ios.cfg
 
@@ -41,11 +43,12 @@ iosConfigure() {
 
 iosClean() {
 	make clean
-	rm -rf libr/.libr libr/libr.a libr/libr.dylib
+	rm -rf libr/.libr libr/.libr2 libr/libr.a libr/libr.dylib shlr/libr_shlr.a
+	rm -rf shlr/capstone
 }
 
 iosBuild() {
-	time make -j4 || exit 1
+	time make -j4 AR="xcrun --sdk ${SDK} ar" || exit 1
 	# Build and sign
 	( cd binr/radare2 ; make ios_sdk_sign )
 	make install DESTDIR="$INSTALL_DST"
@@ -200,6 +203,7 @@ for a in `IFS=+ echo ${CPU}` ; do
         CPUS="-arch $a ${CPUS}"
 done
 export CPUS="${CPUS}"
+export LD="xcrun --sdk ${SDK} ld"
 export ALFLAGS="${CPUS}"
 export LDFLAGS="${LDFLAGS} ${CPUS}"
 	export PS1="[ios-sdk-$CPU]> "
@@ -237,11 +241,12 @@ if [ "${USE_SIMULATOR}" = 1 ]; then
 fi
 
 # check if arm archs were selected and if so build radare2 for them
+# XXX this is a bashism
 if [ "${#ARCHS}" -gt 0 ]; then
 	iosClean
 	iosConfigure
 	if [ "$?" = 0 ]; then
-		export CPU=$ARCHS
+		export CPU="$ARCHS"
 		export SDK=iphoneos
 		echo "Building for $CPU"
 		sleep 1
