@@ -2,7 +2,7 @@
 mad list >/dev/null 2>&1
 if [ $? = 0 ]; then
 	make clean
-	echo './configure --without-ssl --prefix=/usr --with-little-endian' | mad sh
+	echo './configure --prefix=/usr' | mad sh
 	echo make | mad sh
 	cd maemo
 	make
