@@ -11,7 +11,7 @@ import subprocess
 import sys
 
 BUILDDIR = 'build'
-BACKENDS = ['ninja', 'vs2015', 'vs2017']
+BACKENDS = ['ninja', 'vs2015', 'vs2017', 'vs2019']
 
 PATH_FMT = {}
 R2_PATH = {
@@ -164,20 +164,28 @@ def win_dist(args):
     PATH_FMT['BUILDDIR'] = builddir
 
     makedirs(r'{DIST}')
-    copy(r'{BUILDDIR}\binr\*\*.exe', r'{DIST}')
-    copy(r'{BUILDDIR}\libr\*\*.dll', r'{DIST}')
+    makedirs(r'{DIST}\bin')
+    copy(r'{BUILDDIR}\binr\*\*.exe', r'{DIST}\bin')
+
+    r2_bat_fname = args.install + r'\bin\r2.bat'
+    log.debug('create "%s"', r2_bat_fname)
+    with open(r2_bat_fname, 'w') as r2_bat:
+        r2_bat.write('@"%~dp0\\radare2" %*\n')
+
+    copy(r'{BUILDDIR}\libr\*\*.dll', r'{DIST}\bin')
     makedirs(r'{DIST}\{R2_LIBDIR}')
     if args.shared:
         copy(r'{BUILDDIR}\libr\*\*.lib', r'{DIST}\{R2_LIBDIR}')
     else:
         copy(r'{BUILDDIR}\libr\*\*.a', r'{DIST}\{R2_LIBDIR}')
-    win_dist_libr2()
+    win_dist_libr2(install_webui=args.webui)
 
-def win_dist_libr2(**path_fmt):
+def win_dist_libr2(install_webui=False, **path_fmt):
     """[R_API] Add libr2 data/www/include/doc to dist directory"""
     PATH_FMT.update(path_fmt)
 
-    copytree(r'{ROOT}\shlr\www', r'{DIST}\{R2_WWWROOT}')
+    if install_webui:
+        copytree(r'{ROOT}\shlr\www', r'{DIST}\{R2_WWWROOT}')
     copytree(r'{ROOT}\libr\magic\d\default', r'{DIST}\{R2_SDB}\magic')
     makedirs(r'{DIST}\{R2_SDB}\syscall')
     copy(r'{BUILDDIR}\libr\syscall\d\*.sdb', r'{DIST}\{R2_SDB}\syscall')
@@ -194,7 +202,6 @@ def win_dist_libr2(**path_fmt):
     copy(r'{ROOT}\libr\include\sdb\*.h', r'{DIST}\{R2_INCDIR}\sdb')
     copy(r'{ROOT}\libr\include\r_util\*.h', r'{DIST}\{R2_INCDIR}\r_util')
     copy(r'{ROOT}\libr\include\r_crypto\*.h', r'{DIST}\{R2_INCDIR}\r_crypto')
-    copytree(r'{ROOT}\libr\include\msvc', r'{DIST}\{R2_INCDIR}\msvc')
     makedirs(r'{DIST}\{R2_FORTUNES}')
     copy(r'{ROOT}\doc\fortunes.*', r'{DIST}\{R2_FORTUNES}')
     copytree(r'{ROOT}\libr\bin\d', r'{DIST}\{R2_SDB}\format',
@@ -213,15 +220,23 @@ def build(args):
     log.info('Building radare2')
     r2_builddir = os.path.join(ROOT, args.dir)
     options = ['-D%s' % x for x in args.options]
+    if args.webui:
+        options.append('-Duse_webui=true')
+    if args.local:
+        options.append('-Dlocal=true')
     if not os.path.exists(r2_builddir):
         meson(ROOT, r2_builddir, prefix=args.prefix, backend=args.backend,
               release=args.release, shared=args.shared, options=options)
     if args.backend != 'ninja':
-        if args.xp:
+        # XP support was dropped in Visual Studio 2019 v142 platform
+        if args.backend == 'vs2017' and args.xp:
             xp_compat(r2_builddir)
         if not args.project:
             project = os.path.join(r2_builddir, 'radare2.sln')
-            msbuild(project, '/m')
+            params = ['/m']
+            if args.backend == 'vs2017' and args.xp:
+                params.append('/p:XPDeprecationWarning=false')
+            msbuild(project, *params)
     else:
         ninja(r2_builddir)
 
@@ -242,8 +257,10 @@ def main():
 
     # Create parser
     parser = argparse.ArgumentParser(description='Mesonbuild scripts for radare2')
-    parser.add_argument('--asan', action='store_true',
-            help='Build radare2 with ASAN support.')
+    # --sanitize=address,signed-integer-overflow for faster build
+    parser.add_argument('--sanitize', nargs='?',
+            const='address,undefined,signed-integer-overflow', metavar='sanitizers',
+            help='Build radare2 with sanitizer support (default: %(const)s)')
     parser.add_argument('--project', action='store_true',
             help='Create a visual studio project and do not build.')
     parser.add_argument('--release', action='store_true',
@@ -252,6 +269,8 @@ def main():
             help='Choose build backend (default: %(default)s)')
     parser.add_argument('--shared', action='store_true',
             help='Link dynamically (shared library) rather than statically')
+    parser.add_argument('--local', action='store_true',
+            help='Adds support for local/side-by-side installation (sets rpath if needed)')
     parser.add_argument('--prefix', default=None,
             help='Set project installation prefix')
     parser.add_argument('--dir', default=BUILDDIR, required=False,
@@ -268,6 +287,8 @@ def main():
             help='Uninstall')
     parser.add_argument('--symstall', action='store_true',
             help='Install using symlinks')
+    parser.add_argument('--webui', action='store_true',
+            help='Install WebUIs')
     if os.name == 'nt':
         parser.add_argument('--install', help='Installation directory')
     else:
@@ -278,19 +299,19 @@ def main():
     if args.alias:
         print("alias m=\"" + os.path.abspath(__file__) + "\"")
         sys.exit(0);
-    if args.asan:
+    if args.sanitize:
         if os.uname().sysname == 'OpenBSD':
-            log.error("Asan insupported under OpenBSD")
+            log.error("Sanitizers unsupported under OpenBSD")
             sys.exit(1)
         cflags = os.environ.get('CFLAGS')
         if not cflags:
             cflags = ''
-        os.environ['CFLAGS'] = cflags + ' -fsanitize=address'
+        os.environ['CFLAGS'] = cflags + ' -fsanitize=' + args.sanitize
         if os.uname().sysname != 'Darwin':
           ldflags = os.environ.get('LDFLAGS')
           if not ldflags:
               ldflags = ''
-          os.environ['LDFLAGS'] = ldflags + ' -lasan'
+          os.environ['LDFLAGS'] = ldflags + ' -fsanitize=' + args.sanitize
 
     # Check arguments
     if args.pull:
@@ -298,9 +319,12 @@ def main():
     if args.project and args.backend == 'ninja':
         log.error('--project is not compatible with --backend ninja')
         sys.exit(1)
-    if args.xp and args.backend == 'ninja':
+    if args.xp and args.backend in 'ninja':
         log.error('--xp is not compatible with --backend ninja')
         sys.exit(1)
+    if args.xp and args.backend in 'vs2019':
+        log.error('--xp is not compatible with --backend vs2019')
+        sys.exit(1)
     if os.name == 'nt' and args.install and os.path.exists(args.install):
         log.error('%s already exists', args.install)
         sys.exit(1)
