@@ -43,11 +43,15 @@ RebuildIOSDebug() {
 RebuildJava() {
 	Rebuild shlr/java
 	Rebuild libr/asm
+	Rebuild libr/anal
 	Rebuild libr/bin
 	Rebuild libr/core
 }
 
 RebuildCapstone() {
+	if [ ! -d shlr/capstone ]; then
+		make -C shlr capstone
+	fi
 	Rebuild shlr/capstone
 	Rebuild libr/asm
 	Rebuild libr/anal
@@ -74,17 +78,30 @@ RebuildGdb() {
 	Rebuild libr/debug
 }
 
+RebuildWindbg() {
+	Rebuild shlr/windbg
+	Rebuild libr/io
+	Rebuild libr/debug
+}
+
 RebuildZip() {
 	Rebuild shlr/zip
 	Rebuild libr/io
 }
 
+RebuildTcc() {
+	Rebuild shlr/tcc
+	Rebuild libr/parse
+}
+
 case "$1" in
 grub|fs)RebuildFs; ;;
 bin)    RebuildBin ; ;;
 gdb)    RebuildGdb ; ;;
+windbg) RebuildWindbg ; ;;
 sdb)    RebuildSdb ; ;;
 spp)    RebuildSpp ; ;;
+tcc)    RebuildTcc ; ;;
 bin)    RebuildBin ; ;;
 zip)    RebuildZip ; ;;
 java)   RebuildJava ; ;;
